# -*- coding: utf-8 -*-
"""Implements the functionality for cutting up multiple segments
   in parallel using multiple processes.
"""

import multiprocessing as mp
import logging
import traceback

class _Config(object):
  pass
config = None

def _worker_init(video_cutter):
  global config
  config = _Config()
  config.video_cutter = video_cutter

def _process_video((start, end, destination, delete, min_length)):
  global config
  error = None
  try:
    config.video_cutter.cut(start, end, destination, delete, min_length=min_length)
  except Exception, e:
    error = str(e) + "\n" + traceback.format_exc()
    
  return (destination, error is None, error)

def batch_process_video(video_cutter, videolist, num_processes=mp.cpu_count(),
                        overwrite=False):
  """Batch-processes a number of videos.
     video_cutter is an instance of VideoCutter.
     videolist contains the videos to process, in the tuple format
       (start in ms, end in ms, destination filename)
     num_processes is the number of processes to use (default: cpu count on system)
     overwrite, if true, causes the destination to be overwritten if it already exists
     Returns true if all cuts succeeded
  """
  log = logging.getLogger("video")
  pool = mp.Pool(num_processes, _worker_init, (video_cutter,))
  results = pool.imap(_process_video, ((v[0], v[1], v[2], overwrite, v[3]) for v in videolist))
  pool.close()
  all_success = True
  for (name, success, error) in results:
    if success:
      log.info((u"%s: success" % name).encode("utf-8"))
    else:
      log.error((u"%s: %s" % (name, error.decode("utf-8"))).encode("utf-8"))
      all_success = False
  pool.join()
  return all_success
