#!/bin/bash

# Read dai.cs.rutgers.edu username and password
echo "Username for dai.cs.rutgers.edu:"
read USER
echo "Password for dai.cs.rutgers.edu:"
read -s PW

DIR=$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )
cd "${DIR}"

SIGNSTREAM_DIR="$DIR/.."

SCRIPTS_DIR="scripts"
RELEASE_DIR="release"
PATCH_DIR="patches"
PATCHES_DIR="$DIR/$PATCH_DIR"

FILES_DIR="$SIGNSTREAM_DIR/$SCRIPTS_DIR"
RUNCOMMAND_SCRIPT="RunCommand.scpt"
DISABLE_COMMAND="DisableQuarantine.command"
UPDATE_SCRIPT="Update.command"
GETSIGNBANK_SCRIPT="GetSignBank.command"

# could programmatically get this in the future instead of updating it here every release...
# NOTE: the spaces at the begin/end of the below lists are important - for the 'contains' check below
ALL_VERSIONS=" 3.0.3 3.0.4 3.0.5 3.0.6 3.0.7 3.0.8 3.0.9 3.0.10 3.0.11 3.0.12 3.0.13 3.0.14 3.0.15 3.0.16 3.0.17 3.1.0 3.1.1 3.1.2 3.1.3 3.1.4 3.2.0 3.2.1 3.2.2 3.2.3 3.2.4 3.2.5 3.2.6 3.2.7 3.2.8 3.2.9 3.2.10 3.2.11 3.2.12 3.3.0 3.3.1 3.3.2 3.3.3 3.3.4 3.3.5 3.3.6 "

PRE_336_VERSIONS=" 3.0.3 3.0.4 3.0.5 3.0.6 3.0.7 3.0.8 3.0.9 3.0.10 3.0.11 3.0.12 3.0.13 3.0.14 3.0.15 3.0.16 3.0.17 3.1.0 3.1.1 3.1.2 3.1.3 3.1.4 3.2.0 3.2.1 3.2.2 3.2.3 3.2.4 3.2.5 3.2.6 3.2.7 3.2.8 3.2.9 3.2.10 3.2.11 3.2.12 3.3.0 3.3.1 3.3.2 3.3.3 3.3.4 3.3.5 "

mkdir -p $PATCH_DIR

# Loop through each VERSION 
for VERSION in $ALL_VERSIONS; do

cd $PATCHES_DIR
VERSION_DIR="patch-$VERSION"
mkdir -p $VERSION_DIR
cd $VERSION_DIR
cp $FILES_DIR/$RUNCOMMAND_SCRIPT $FILES_DIR/$DISABLE_COMMAND $FILES_DIR/$UPDATE_SCRIPT $FILES_DIR/$GETSIGNBANK_SCRIPT .

PATCH_VERSION="0"

# Loop until we find a patch PATCH_VERSION which does not yet exist for this SignStream VERSION
while true; do
  PATCH_VERSION=$(( $PATCH_VERSION + 1 ))

  # Change signstream url based on version tag
  SIGNSTREAM_PATCH_FN='SignStream3-'"$VERSION"'-patch-'"$PATCH_VERSION"'.zip'
  SIGNSTREAM_PATCH_URL='http://dai.cs.rutgers.edu/SignStream/download/release/patches/'"$SIGNSTREAM_PATCH_FN"

  SIGNSTREAM_PATCH_REQUEST=$( curl -L --silent --write-out "HTTPSTATUS:%{http_code}" --output "$SIGNSTREAM_PATCH_FN" "$SIGNSTREAM_PATCH_URL" )

  if [ "x"$( echo "$SIGNSTREAM_PATCH_REQUEST" | grep '^HTTPSTATUS:'  | sed -e 's/HTTPSTATUS://' ) != "x200" ]; then
    break
  else
    echo "Patch for SignStream $VERSION patch version $PATCH_VERSION exists"

    # Delete the zipped version
    rm -rf "$SIGNSTREAM_PATCH_FN"
  fi

done # while true; do

# PATCH_VERSION is now at what we should upload it as

PATCH_FILE=".patch-$VERSION-$PATCH_VERSION"
touch $PATCH_FILE

# Edit the max version variable in Update.command to this version
sed -E -e "s/UPDATE_COMMAND_MAX_VERSION=.*/UPDATE_COMMAND_MAX_VERSION=\"$VERSION\"/" -i "" $UPDATE_SCRIPT

# NOTE: the space before/after VERSION is important
if [[ $PRE_336_VERSIONS =~ " $VERSION " ]]; then
  sed -E -e "s/GET_CURRENT_VERSION_VERSION=.*/GET_CURRENT_VERSION_VERSION=\"1\"/" -i "" $UPDATE_SCRIPT

  sed -E -e "s/GET_CURRENT_VERSION_VERSION=.*/GET_CURRENT_VERSION_VERSION=\"1\"/" -i "" $GETSIGNBANK_SCRIPT
fi

ZIPFILE="SignStream3-$VERSION-patch-$PATCH_VERSION.zip"
rm -f $ZIPFILE; zip -q $ZIPFILE $RUNCOMMAND_SCRIPT $DISABLE_COMMAND $UPDATE_SCRIPT $GETSIGNBANK_SCRIPT $PATCH_FILE
echo "Zipped $ZIPFILE"

# Take the newly created zip archive and scp it to the DAI server
expect $SIGNSTREAM_DIR/$SCRIPTS_DIR/scp.exp $USER $PW $ZIPFILE dai.cs.rutgers.edu /cbim/Prod/SignStream/download/$RELEASE_DIR/$PATCH_DIR
echo "Sent zipfile via scp to server"

cd $PATCHES_DIR
rm -rf $VERSION_DIR
echo "Deleted directory $VERSION_DIR"

done # for VERSION in $ALL_VERSIONS; do


# end of script
