package edu.bu.signstream.common.db.parser;

import edu.bu.signstream.common.db.UtilityFilesXMLWriter;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.Reference;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.NonManualFieldOrdering;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import org.w3c.dom.CharacterData;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.awt.*;
import java.util.*;
import java.util.logging.Level;

/**
 * The SS3ZipXmlFilesParser class parses SignStream files of format  filename.ss3 or fileName.ss3.zip
 * fileName.ss3 ot filename.ss3.zip is a zipped directory that contains one or more macroUnit.xml files
 * and optionally utility files.
 */

public class SS3ZipXmlFilesParser {

	/**
     * Parses database xml file.
     * @param  fileName a unzipped file name of an xml file.
     * @return  populated SS3Database object
     */
    public SS3Database parseSS3Database(String fileName) {
			try {
				DocumentBuilder builder = DocumentBuilderFactory.newInstance()
						.newDocumentBuilder();
				Document doc = null;

				try {
					doc = builder.parse(fileName);
				} catch (java.net.MalformedURLException ex) {
					String space = "%20";
					StringBuffer sb = new StringBuffer();
					StringTokenizer tokenizer = new StringTokenizer(fileName);
					sb.append(tokenizer.nextElement());

					while (tokenizer.hasMoreTokens()) {
						sb.append(space);
						sb.append(tokenizer.nextElement());
					}
					doc = builder.parse(sb.toString());
				}

				NodeList nodes1 = doc.getElementsByTagName("SIGNSTREAM-DATABASE");
				Element excEl = (Element) nodes1.item(0);

				/* TODO: compare database and app versions
				String ssVersion = excEl.getAttribute("SIGNSTREAM-VERSION");
				String currentVersion = SS3Singleton.getSignStreamApplication().getVersion();
				if (currentVersion > ssVersion) {
					fix problems with old collection
				} else if (currentVersion < ssVersion) {
					you opened something from the future how
				}
				*/

				String sStr = excEl.getAttribute("S");
				String eStr = excEl.getAttribute("E");
				String partName = excEl.getAttribute("PARTICIPANT");

				TimeInfo startTimeInfo = new TimeInfo();
				TimeInfo endTimeInfo = new TimeInfo();
				startTimeInfo.setTimeScale(600);
				startTimeInfo.setMovieTime(Integer.parseInt(sStr));
				endTimeInfo.setTimeScale(600);
				endTimeInfo.setMovieTime(Integer.parseInt(eStr));

				SS3Database db = new SS3Database(startTimeInfo, endTimeInfo);
				db.setFileName(fileName);
				String excerpt = excEl.getAttribute("EXCERPT");
				String excerptID = excEl.getAttribute("EXCERPT_NUMBER");
				db.setExcerpt(excerpt);
				db.setNumber(excerptID);

				db.setExcerptObj(new Excerpt(excerptID, excerpt));

				db.setStartTimeInfo(startTimeInfo);
				db.setEndTimeInfo(endTimeInfo);
				db.setParticipantName(partName);

				for (int i = 0; i < nodes1.getLength(); i++) {
					Element el1 = (Element) nodes1.item(i);
					NodeList utilsList = el1.getElementsByTagName("UTILS");

					for (int ii = 0; ii < utilsList.getLength(); ii++) {
						Element utilsEl = (Element) utilsList.item(ii);
						String pixelInTimeFrameStr = utilsEl
								.getAttribute("TP_PIXEL_IN_TIME_FRAME");
						String onScreenLocationStr = utilsEl
								.getAttribute("TP_ONSCREEN_LOCATION");
						String movieTimeStr = utilsEl.getAttribute("TP_MOVIE_TIME");

						double pixelInTimeFrame = 0;
						int onScreenLocation = 0;
						int movieTime = 0;

						if(pixelInTimeFrameStr == null || pixelInTimeFrameStr.trim().length()>0) {
                            try {
							    pixelInTimeFrame = Double.valueOf(pixelInTimeFrameStr);
						    } catch (Exception ex) {
							    // do nothing, this exception will be thrown if pixelInTimeFrameStr is or not a number
                                ;
						    }
                        }

                        if(onScreenLocationStr == null || onScreenLocationStr.trim().length()>0) {
						    try {
							    onScreenLocation = Integer.parseInt(onScreenLocationStr);
						    } catch (Exception ex) {
                                // do nothing, this exception will be thrown if onScreenLocationStr is not a number
                                ;
						    }
                        }

						if(movieTimeStr == null || movieTimeStr.trim().length()>0) {
                            try {
							    movieTime = Integer.parseInt(movieTimeStr);
						    } catch (Exception ex) {
                                // do nothing, this exception will be thrown if movieTimeStr is not a number
							    ;
						    }
                        }

						db.setTemporalPartitionPixelInTimeFrame(pixelInTimeFrame);
						db.setTemporalPartitionOnScreenLocation(onScreenLocation);
						db.setTemporalPartitionMovieTime(movieTime);
					}

					NodeList refList = el1.getElementsByTagName("REFERENCES");

					Element el2 = (Element) refList.item(0);
					NodeList cSchRefList = el2
							.getElementsByTagName("CODING-SCHEMES");
					NodeList prtRefList = el2.getElementsByTagName("PARTICIPANTS");

					NodeList antRefList = el2.getElementsByTagName("ANNOTATORS");

					ArrayList<SS3Participant> selectedParticipants = new ArrayList<SS3Participant>();
					Element prtElement0 = (Element) prtRefList.item(0);

					if (prtElement0 != null) {
						NodeList prtcpNode = prtElement0
								.getElementsByTagName("REF");
						ParticipantsLibrary participantsLibrary = SS3Singleton
								.getManager().getLoadedParticipantsLibrary();

						for (int ii = 0; ii < prtcpNode.getLength(); ii++) {
							Element prtElement = (Element) prtcpNode.item(ii);
							SS3Participant participant = null;
							String participantID = prtElement.getAttribute("ID");
							String participantLabel = prtElement.getAttribute("LABEL");
							if (prtElement.getAttributes().getLength() > 2) {
								participant = new SS3Participant();

								participant.setId(participantID);
								participant.setLabel(participantLabel);
								participant.setBackground(prtElement.getAttribute("BACKGROUND"));
								participant.setComments(prtElement.getAttribute("COMMENTS"));
								participant.setDominantHand(prtElement.getAttribute("DOMINANT_HAND"));
								participant.setFullName(prtElement.getAttribute("FULL_NAME"));
								participant.setGender(prtElement.getAttribute("GENDER"));
								participant.setNativeLanguage(prtElement.getAttribute("NATIVE_LANGUAGE"));
								participant.setParentsInfo(prtElement.getAttribute("PARENT_INFO"));
								participant.setYearOfBirth(prtElement.getAttribute("YEAR_OF_BIRTH"));

								participantsLibrary.addLocalParticipant(participant);
								new UtilityFilesXMLWriter().saveParticipantsLibrary(participantsLibrary);
							} else if (participantID != null && participantID.length() > 0) {
								participant = participantsLibrary.getParticipant(participantID);
							}
							if (participant != null) {
								String graphFileID = prtElement.getAttribute("GRAPH_FILE_ID");
								if (!graphFileID.isEmpty()) {
									participant.setGraphFileID(graphFileID);
								}
								selectedParticipants.add(participant);
							}
						}
					}

					ArrayList<SS3Participant> selectedAnnotators = new ArrayList<SS3Participant>();
					Element antElement0 = (Element) antRefList.item(0);

					if (antElement0 != null) {
						NodeList antNode = antElement0.getElementsByTagName("REF");

						// check, had not been loaded
						ParticipantsLibrary annotatorsLibrary = SS3Singleton
								.getManager().getLoadedAnnotatorsLibrary();

						for (int jj = 0; jj < antNode.getLength(); jj++) {
							Element antElement = (Element) antNode.item(jj);
							String annotatorID = antElement.getAttribute("ID");
							String annotatorLabel = antElement.getAttribute("LABEL");

							if (annotatorID != null && annotatorID.length() > 0) {
								SS3Participant annotator = annotatorsLibrary
										.getParticipant(annotatorID);
								if (annotator != null && annotator.getLabel().equals(annotatorLabel)) {
									if (!selectedAnnotators.contains(annotator)) {
										selectedAnnotators.add(annotator);
									}									
								}
							}
						}
					}

					ArrayList<Reference> cShRef = parseReferenses(cSchRefList);
					db.setCodingSchemeRef(cShRef);
					db.setSelectedParticipants(selectedParticipants);
					db.setSelectedAnnotators(selectedAnnotators);

					NodeList mediaList = el1.getElementsByTagName("MEDIA-FILES");
					db.setMediaFiles(parseMFiles(mediaList));

					Element element = (Element) mediaList.item(0);
					String mds = element.getAttribute("MOVIE_DURATION");
					String pds = element.getAttribute("PACKET_DURATION");
					String tss = element.getAttribute("MOVIE_TIMESCALE");
					// Note: getAttribute returns empty string if not attr is not there
					
					int movieDuration = 0;
					int packetDuration = 0;
					int movieTimescale = 0;

					try {
						movieDuration = Integer.parseInt(mds);
					} catch (NumberFormatException ex) {
						;
					}

					try {
						packetDuration = Integer.parseInt(pds);
					} catch (NumberFormatException ex) {
						;
					}

					try {
						movieTimescale = Integer.parseInt(tss);
					} catch (NumberFormatException ex) {
						;
					}

                    System.out.println("PARSED TIMESCALE FROM FILE:");
                    System.out.println("movieDuration = " + movieDuration);
                    System.out.println("packetDuration = " + packetDuration);
                    System.out.println("movieTimescale = " + movieTimescale);

                    db.setMovieDuration(movieDuration);
					db.setPacketDuration(packetDuration);
					db.setMovieTimescale(movieTimescale);

					NodeList mediaOnScreenLocationList = el1
							.getElementsByTagName("MEDIA_FILES_ON_SCREEN_LOCATION");
					db.setVideoOnScreenLocations(parseMFilesOnScreenLocation(mediaOnScreenLocationList));

					NodeList graphList = el1.getElementsByTagName("GRAPH-FILES");
					db.setGraphFiles(parseGFiles(graphList));

					NodeList nmFieldOrderList = el1.getElementsByTagName("NON-MANUAL-GRAPH-FIELD-ORDER");
					db.setCustomNmFieldOrder(parseNmFieldOrdering(nmFieldOrderList));

					SS3CodingScheme cScheme = new SS3CodingScheme();
					NodeList lCodingSchemeList = el1
							.getElementsByTagName("LOCAL-CODING-SCHEME");
					cScheme.setFields(parseFields(lCodingSchemeList));
					db.setLocalSS3CodingScheme(cScheme);

					NodeList dbList = el1.getElementsByTagName("DATABASE");

					for (int l = 0; l < dbList.getLength(); l++) {
						Element el10 = (Element) dbList.item(i);

						NodeList tracksList = el10.getElementsByTagName("TRACKS");
						
						ArrayList<SS3Track> tracks = parseTracks(tracksList, db);
						for (SS3Track track : tracks) {
							db.addTrack(track);	
						}

						NodeList glossWindowList = el10
								.getElementsByTagName("GLOSS-WINDOW");
						SS3GlossWindow glossWinow = parseSS3GlossWindow(glossWindowList, db);

						db.setSS3GlossWindow(glossWinow);

						NodeList ndsList = el10.getElementsByTagName("NOTES");

						ArrayList<SS3Note> notes = parseNotes(ndsList);
						for (int k = 0; k < notes.size(); k++) {
							SS3Note note = notes.get(k);
							db.addNote(note);
						}
					}
				}
				return db;
			} catch (Exception e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());
			}
			return null;
		}

		/*
		 * This method parses gloss-window node of a database (xml) file. Bellow is an example of a node.
		 *
		 * <GLOSS-WINDOW>
		 *     <DISPLAY-TIME-PERIOD/>
		 *     <TEMPORAL-PARTITIONS>
		 *         <TEMPORAL-PARTITION ID="499147" LABEL="Temporal Partition 111">
		 *             <TEMPORAL-PARTITION-SEGMENT ID="499148" LABEL="Segment 1">
		 *                 <GLOSS-WINDOW-ITEM E="-1" ID="6046711" LABEL="U1" S="-1">
		 *                     <STATEMENT-FIELD ENTITY-IDS="129010308179213, 129010309531615" FIELD-ID="10000"/>
		 *                     <STATEMENT-FIELD ENTITY-IDS="129010314238016, 129010316766137" FIELD-ID="27"/>
		 *                 </GLOSS-WINDOW-ITEM>
		 *             </TEMPORAL-PARTITION-SEGMENT>
		 *         </TEMPORAL-PARTITION>
		 *     </TEMPORAL-PARTITIONS>
		 * </GLOSS-WINDOW>
		 */
		private SS3GlossWindow parseSS3GlossWindow(NodeList glsWindowNode, SS3Database database) {
			SS3Database db = database;
			SS3GlossWindow glsWindow = new SS3GlossWindow();
			Element glsWindowElement = (Element) glsWindowNode.item(0);

			NodeList displayTimePeriodNode = glsWindowElement
					.getElementsByTagName("DISPLAY-TIME-PERIOD");
			Element displayTimePeriodElement = (Element) displayTimePeriodNode
					.item(0);
			SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
			displayTimePeriod.setLabel("Display Time Period");
			glsWindow.setDisplayTimePeriod(displayTimePeriod);

			NodeList displayTimePeriodGlsWindowItemNode = displayTimePeriodElement
					.getElementsByTagName("GLOSS-WINDOW-ITEM");
			for (int i = 0; i < displayTimePeriodGlsWindowItemNode.getLength(); i++) {
				Element displayTimePeriodGlsWindowItemEl = (Element) displayTimePeriodGlsWindowItemNode
						.item(i);
				SS3GlossWindowItem displayTimePeriodWindowItem = new SS3GlossWindowItem(
						"", "Display Time Period");

				displayTimePeriodWindowItem.setId(displayTimePeriodGlsWindowItemEl
						.getAttribute("ID"));
				displayTimePeriodWindowItem
						.setLabel(displayTimePeriodGlsWindowItemEl
								.getAttribute("LABEL"));

				TimeInfo startTimeInfo = new TimeInfo();
				startTimeInfo.setMovieTime(Integer
						.parseInt(displayTimePeriodGlsWindowItemEl
								.getAttribute("S")));
				displayTimePeriodWindowItem.setStartTimeInfo(startTimeInfo);

				TimeInfo endTimeInfo = new TimeInfo();
				endTimeInfo.setMovieTime(Integer
						.parseInt(displayTimePeriodGlsWindowItemEl
								.getAttribute("E")));
				displayTimePeriodWindowItem.setEndTimeInfo(endTimeInfo);

				displayTimePeriod
						.addSS3GlossWindowItem(displayTimePeriodWindowItem);
			}

			NodeList temporalPartitionsNode = glsWindowElement
					.getElementsByTagName("TEMPORAL-PARTITIONS");
			Element temporalPartitionsElement = (Element) temporalPartitionsNode
					.item(0);
			NodeList temporalPartitionNode = temporalPartitionsElement
					.getElementsByTagName("TEMPORAL-PARTITION");

			for (int j = 0; j < temporalPartitionNode.getLength(); j++) {
				Element temporalPartitionEl = (Element) temporalPartitionNode
						.item(j);

				SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
				String temporalPartitionLabel = temporalPartitionEl
						.getAttribute("LABEL");

				temporalPartition.setID(temporalPartitionEl.getAttribute("ID"));
				temporalPartition.setLabel(temporalPartitionLabel);
				glsWindow.addTemporalPartition(temporalPartition);

				HashMap<String, SS3GlossWindowItem> segmentItems = new HashMap<String, SS3GlossWindowItem>();

				NodeList temporalPartitionSegmentNode = temporalPartitionEl
						.getElementsByTagName("TEMPORAL-PARTITION-SEGMENT");

				for (int l = 0; l < temporalPartitionSegmentNode.getLength(); l++) {
					Element temporalPartitionSegmentEl = (Element) temporalPartitionSegmentNode
							.item(l);

					SS3GlossWindowSegment temporalPartitionSegment = new SS3GlossWindowSegment();
					String temporalPartitionSegmentID = temporalPartitionSegmentEl
							.getAttribute("ID");
					
					SS3Participant part = db.getSelectedParticipant(
							temporalPartitionSegmentEl.getAttribute("PARTICIPANT_ID"));
					String isDef = temporalPartitionSegmentEl.getAttribute("ISDEFAULT");
					
					String temporalPartitionSegmentLabel = temporalPartitionSegmentEl
							.getAttribute("LABEL");
					
					String note = temporalPartitionSegmentEl.getAttribute("NOTE");
					if(note != null && !note.isEmpty()) {
						SS3GlossWindowItemNote noteItem = new SS3GlossWindowItemNote();
						noteItem.setNote(note);
						temporalPartitionSegment.setNote(noteItem);	
					}
					
					boolean isDefault = isDef.equals("true") ? true : false;	// will not throw exception

					temporalPartitionSegment.setID(temporalPartitionSegmentID);
					temporalPartitionSegment.setParticipant(part);
					int indexOfSeparator = temporalPartitionSegmentLabel.indexOf(Util.SEGMENT_TIER_SEPARATOR);
					temporalPartitionSegment.setLabel(part != null ? temporalPartitionSegmentLabel : temporalPartitionSegmentLabel.substring(indexOfSeparator + 1));
					
					temporalPartition.addTemporalPartitionSegment(temporalPartitionSegment);
										
					// always try to set the 0th segment tier to the default, in case none of them are set to true
					if (isDefault || l == 0) {
						temporalPartition.setDefaultSegment(temporalPartitionSegment);
					}

					NodeList temporalPartitionSegmentGlossWindowItemNode = temporalPartitionSegmentEl
							.getElementsByTagName("GLOSS-WINDOW-ITEM");

					for (int k = 0; k < temporalPartitionSegmentGlossWindowItemNode
							.getLength(); k++) {
						Element temporalPartitionSegmentGlossWindowItemEl = (Element) temporalPartitionSegmentGlossWindowItemNode
								.item(k);

						SS3GlossWindowItem temporalPartitionSegmentWindowItem = new SS3GlossWindowItem(
								temporalPartition.getLabel(),
								temporalPartitionSegment.getLabel());

						temporalPartitionSegmentWindowItem
								.setId(temporalPartitionSegmentGlossWindowItemEl
										.getAttribute("ID"));
						temporalPartitionSegmentWindowItem
								.setLabel(temporalPartitionSegmentGlossWindowItemEl
										.getAttribute("LABEL"));

						String startTime = temporalPartitionSegmentGlossWindowItemEl
								.getAttribute("S");
						SS3GlossWindowItem enteredItem = segmentItems
								.get(startTime);

						if (enteredItem == null) {
							TimeInfo windowItemStartTimeInfo = new TimeInfo();
							windowItemStartTimeInfo.setMovieTime(Integer
									.parseInt(startTime));
							temporalPartitionSegmentWindowItem
									.setStartTimeInfo(windowItemStartTimeInfo);

							TimeInfo windowItemEndTimeInfo = new TimeInfo();
							windowItemEndTimeInfo
									.setMovieTime(Integer
											.parseInt(temporalPartitionSegmentGlossWindowItemEl
													.getAttribute("E")));
							temporalPartitionSegmentWindowItem
									.setEndTimeInfo(windowItemEndTimeInfo);

							segmentItems.put(startTime,
									temporalPartitionSegmentWindowItem);

						} else {
							temporalPartitionSegmentWindowItem
									.setStartTimeInfo(enteredItem
											.getStartTimeInfo());
							temporalPartitionSegmentWindowItem
									.setEndTimeInfo(enteredItem.getEndTimeInfo());
						}

						temporalPartitionSegment
								.addSS3GlossWindowItem(temporalPartitionSegmentWindowItem);

						NodeList statementFieldNode = temporalPartitionSegmentGlossWindowItemEl
								.getElementsByTagName("STATEMENT-FIELD");

						for (int s = 0; s < statementFieldNode.getLength(); s++) {
							Element statementFieldEl = (Element) statementFieldNode
									.item(s);

							SS3StatementField statementField = new SS3StatementField();
							statementField.setId(statementFieldEl
									.getAttribute("FIELD-ID"));
							String entityIDs = statementFieldEl
									.getAttribute("ENTITY-IDS");
							
							String color = statementFieldEl.getAttribute("COLOR");
							if(color != null && !color.isEmpty()) {
								int rgbColor = Integer.parseInt(color);
								statementField.setColor(new Color(rgbColor));
							}

							StringTokenizer st = new StringTokenizer(entityIDs,
									", ");
							while (st.hasMoreTokens()) {
								// get the (already existing) SS3Entity object from the corresponding db track
								String ss3EntityID = st.nextToken();
								SS3Entity ss3Entity = db.getTrack(statementField.getId()).getSS3Entity(ss3EntityID);
								if (ss3Entity != null) {
									statementField.addSS3Entity(ss3Entity);
								}
							}
							
							//Fix for disappearing translation fields. Only happens when no translation field is found for an utterance.
							if (statementField.getId().equals(Util.FREE_TEXT_FIELD_ID) && statementField.getSS3Entities().size() == 0) {
								outer:for (SS3Entity e : db.getTrack(Util.FREE_TEXT_FIELD_ID).getEntities()) {
									
									//Check to make sure Entity doesn't already have a parent.
									for (SS3GlossWindowItem i : segmentItems.values()) {
										if (i.getSS3StatementField(Util.FREE_TEXT_FIELD_ID) != null && i.getSS3StatementField(Util.FREE_TEXT_FIELD_ID).getSS3Entities().size() > 0 && i.getSS3StatementField(Util.FREE_TEXT_FIELD_ID).getSS3Entities().get(0).getID().equals(e.getID())) {
											continue outer;
										}
									}
									
									//1. Check if utterance ID matches and translation's parent entity ID.
									if (e.getParentEntityID().equals(temporalPartitionSegmentWindowItem.getId())) {
										statementField.addSS3Entity(e);
										e.getItems().get(0).setStartTimeInfo(temporalPartitionSegmentWindowItem.getStartTimeInfo());
										e.getItems().get(0).setEndTimeInfo(temporalPartitionSegmentWindowItem.getEndTimeInfo());
										break;
									//2. Check if End Movie time matches any translation's end movie time.
									} else if (e.getItems().get(0).getEndTimeInfo().getMovieTime() == temporalPartitionSegmentWindowItem.getEndTimeInfo().getMovieTime()) {
										statementField.addSS3Entity(e);
										e.getItems().get(0).setStartTimeInfo(temporalPartitionSegmentWindowItem.getStartTimeInfo());
										e.getItems().get(0).setEndTimeInfo(temporalPartitionSegmentWindowItem.getEndTimeInfo());
										break;
									//3. Check if End Movie time of any translation falls within utterance's range (between start/end time).
									} else if (e.getItems().get(0).getEndTimeInfo().getMovieTime() + 2000 ==  temporalPartitionSegmentWindowItem.getEndTimeInfo().getMovieTime()
											&& e.getItems().get(0).getStartTimeInfo().getMovieTime() + 2000 == temporalPartitionSegmentWindowItem.getStartTimeInfo().getMovieTime()) {
										statementField.addSS3Entity(e);
										e.getItems().get(0).setStartTimeInfo(temporalPartitionSegmentWindowItem.getStartTimeInfo());
										e.getItems().get(0).setEndTimeInfo(temporalPartitionSegmentWindowItem.getEndTimeInfo());
										break;
									}
								}
							} 

							temporalPartitionSegmentWindowItem
									.addField(statementField);
						}
					}

					SS3GlossWindowSegment listOfSegmentItems = new SS3GlossWindowSegment();

					Collection<SS3GlossWindowItem> collection = segmentItems
							.values();
					Iterator<SS3GlossWindowItem> iterator = collection.iterator();
					while (iterator.hasNext()) {
						SS3GlossWindowItem item = iterator.next();
						SS3GlossWindowItem tpItem = new SS3GlossWindowItem(
								temporalPartitionLabel,
								temporalPartitionSegmentLabel);
						tpItem.setLabel(item.getLabel());
						tpItem.setStartTimeInfo(item.getStartTimeInfo());
						tpItem.setEndTimeInfo(item.getEndTimeInfo());
						listOfSegmentItems.addSS3GlossWindowItem(tpItem);
					}

					temporalPartition.setGlossWindowSegment(listOfSegmentItems);
				}
			}

			return glsWindow;
		}

		private ArrayList<SS3Note> parseNotes(NodeList nodes) {
			ArrayList<SS3Note> list = new ArrayList<SS3Note>();
			Element element = (Element) nodes.item(0);
			NodeList noteNode = element.getElementsByTagName("NOTE");

			for (int i = 0; i < noteNode.getLength(); i++) {
				SS3Note note = new SS3Note();
				Element line = (Element) noteNode.item(i);
				note.setID(line.getAttribute("ID"));
				note.setCoderID(line.getAttribute("CODER-ID"));
				note.setTimeCreated(line.getAttribute("TIME_CREATED"));
				note.setTimeUpdated(line.getAttribute("TIME_UPDATED"));

				NodeList sNode = line.getElementsByTagName("SUBJECT");
				for (int j = 0; j < sNode.getLength(); j++) {
					Element sEl = (Element) sNode.item(j);
					note.setSubject(getCharacterDataFromElement(sEl));
				}

				NodeList bNode = line.getElementsByTagName("BODY");
				for (int l = 0; l < bNode.getLength(); l++) {
					Element bEl = (Element) bNode.item(l);
					note.setBody(getCharacterDataFromElement(bEl));
				}

				list.add(note);
			}
			return list;
		}

    private HashMap<String, SS3Field> parseFields(NodeList nodes) {
        HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();

        if (nodes.getLength() == 0)
            return fields;

        Element element = (Element) nodes.item(0);
        NodeList fNode = element.getElementsByTagName("FIELD");

        for (int i = 0; i < fNode.getLength(); i++) {
            SS3Field fld = new SS3Field();
            Element fEl = (Element) fNode.item(i);
            fld.setId(fEl.getAttribute("ID"));
            fld.setLabel(fEl.getAttribute("LABEL"));
            fld.setName(fEl.getAttribute("NAME"));
            fld.setPrefix(fEl.getAttribute("PREFIX"));
            String alignmentStr = fEl.getAttribute("ALIGNMENT");
            try {
                fld.setAlignment(SS3Field.Alignment.valueOf(alignmentStr));
            } catch (IllegalArgumentException e) {
                fld.setAlignment(SS3Field.Alignment.CENTER);
            }
            fld.setType(fEl.getAttribute("TYPE"));
            fld.setGroup(fEl.getAttribute("GROUP"));
            
			String color = fEl.getAttribute("COLOR");
			if(color != null && !color.isEmpty()) {
				int rgbColor = Integer.parseInt(color);
				fld.setColor(new Color(rgbColor));
			}

            NodeList valNode = fEl.getElementsByTagName("VALUE");
            for (int j = 0; j < valNode.getLength(); j++) {
                SS3FieldValue val = new SS3FieldValue();
                Element valEl = (Element) valNode.item(j);
                val.setId(valEl.getAttribute("ID"));
                 val.setIid(valEl.getAttribute("IID"));
                val.setLabel(valEl.getAttribute("LABEL"));
                val.setName(valEl.getAttribute("NAME"));
                val.setType(valEl.getAttribute("TYPE"));
                fld.addValue(val);
            }
            fields.put(fld.getId(), fld);

            SS3Singleton.getSS3CodingSchemeCollection()
                    .getDefaultCodingScheme().addField(fld);
        }
        return fields;
    }

		private ArrayList<Reference> parseReferenses(NodeList nodes) {
			ArrayList<Reference> list = new ArrayList<Reference>();
			Element element = (Element) nodes.item(0);
			NodeList prtcpNode = element.getElementsByTagName("REF");

			for (int i = 0; i < prtcpNode.getLength(); i++) {
				Reference ref = new Reference();
				Element line = (Element) prtcpNode.item(i);
				ref.setId(line.getAttribute("ID"));
				ref.setFileName(line.getAttribute("FILE_NAME"));
				ref.setSignature(line.getAttribute("SIGNATURE"));
				list.add(ref);
			}
			return list;
		}

		private HashMap<String, java.awt.Point> parseMFilesOnScreenLocation(
				NodeList nodes) {
			HashMap<String, java.awt.Point> locations = new HashMap<String, java.awt.Point>();

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Starting parsing media on screen locations");

			Object obj = nodes.item(0);
			if (obj != null) {
				Element element = (Element) obj;
				NodeList mfNode = element.getElementsByTagName("MEDIA-FILE");

				for (int i = 0; i < mfNode.getLength(); i++) {
					Element line = (Element) mfNode.item(i);
					String fileName = line.getAttribute("FILE_NAME");
					String x = line.getAttribute("LOCATION_X");
					String y = line.getAttribute("LOCATION_Y");

					try {
						int X = Integer.parseInt(x);
						int Y = Integer.parseInt(y);
						locations.put(fileName, new java.awt.Point(X, Y));
					} catch (NullPointerException ex) {
						SS3SignStreamApplication.logger.log(Level.INFO,
								"Parsing media on screen location: fileName = "
										+ fileName + " could not parse properly");
					}

					SS3SignStreamApplication.logger.log(Level.INFO,
							"Parsed media on screen location: fileName = "
									+ fileName + ", x = " + x + ", y = " + y);
				}
			}

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Media on screen locations parsed");
			return locations;
		}

		private HashMap<String, SS3MediaFile> parseMFiles(NodeList nodes) {
			HashMap<String, SS3MediaFile> mFiles = new HashMap<String, SS3MediaFile>();

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Starting parsing video references");

			Element element = (Element) nodes.item(0);
			NodeList mfNode = element.getElementsByTagName("MEDIA-FILE");

			for (int i = 0; i < mfNode.getLength(); i++) {
				SS3MediaFile mFile = new SS3MediaFile();
				Element line = (Element) mfNode.item(i);
				mFile.setId(line.getAttribute("ID"));
				mFile.setFile(line.getAttribute("PATH"));
				mFiles.put(mFile.getId(), mFile);

				SS3SignStreamApplication.logger.log(Level.INFO,
						"Parsed video reference: " + mFile);
			}
			SS3SignStreamApplication.logger.log(Level.INFO,
					"Video references parsed");
			return mFiles;
		}

		private HashMap<String, SS3MediaFile> parseGFiles(NodeList nodes) {
			HashMap<String, SS3MediaFile> gFiles = new HashMap<String, SS3MediaFile>();
			
			if (nodes.getLength() == 0) {
				return gFiles;
			}

			SS3SignStreamApplication.logger.log(Level.INFO,
					"Starting parsing graph file references");

			Element element = (Element) nodes.item(0);
			NodeList gNode = element.getElementsByTagName("GRAPH-FILE");

			for (int i = 0; i < gNode.getLength(); i++) {
				SS3MediaFile gFile = new SS3MediaFile();
				Element line = (Element) gNode.item(i);
				gFile.setId(line.getAttribute("ID"));
				gFile.setFile(line.getAttribute("PATH"));
				gFile.setName(line.getAttribute("NAME"));
				gFiles.put(gFile.getId(), gFile);

				SS3SignStreamApplication.logger.log(Level.INFO,
						"Parsed graph file reference: " + gFile);
			}
			SS3SignStreamApplication.logger.log(Level.INFO,
					"Graph file references parsed");
			return gFiles;
		}

		private NonManualFieldOrdering parseNmFieldOrdering(NodeList nodes) {
			if (nodes.getLength() == 0) {
				return null;
			}

			ArrayList<String> nmFieldOrdering = new ArrayList<String>();
			Element el = (Element) nodes.item(0);
			NodeList nmFieldOrderNodes = el.getElementsByTagName("F");

			for (int i = 0; i < nmFieldOrderNodes.getLength(); i++) {
				Element fieldIDElem = (Element) nmFieldOrderNodes.item(i);
				nmFieldOrdering.add(fieldIDElem.getAttribute("ID"));
			}

			return new NonManualFieldOrdering(nmFieldOrdering);
		}

		// point nodes to "TRACKS"
		private ArrayList<SS3Track> parseTracks(NodeList nodes, SS3Database db) {
			ArrayList<SS3Track> tracks = new ArrayList<SS3Track>();
			Element element = (Element) nodes.item(0);
			NodeList tNode = element.getElementsByTagName("TRACK");
			
			// right handshape
			SS3Track track1000 = null;
			
			// left handshape
			SS3Track track2000 = null;
			
			// right hand
			SS3Track track10501 = null;
			
			// left hand
			SS3Track track10502 = null;
			
			// gloss1
			SS3Track track10000 = null;
			
			// gloss2
			SS3Track track10001 = null;
			
			boolean setGlossDependencies = false;

			for (int i = 0; i < tNode.getLength(); i++) {
				SS3Track track = new SS3Track();
				Element tEl = (Element) tNode.item(i);
				
				String fieldID = tEl.getAttribute("FID");
				
				track.setId(fieldID);

				NodeList ncNode = tEl.getElementsByTagName("NON_COMPLETE_EVENTS");
				for (int nc = 0; nc < ncNode.getLength(); nc++) {
					Element entEl = (Element) ncNode.item(nc);
					NodeList ncItmNode = entEl.getElementsByTagName("ITEM");

					for (int nci = 0; nci < ncItmNode.getLength(); nci++) {
						SS3Item ncItem = new SS3Item();

						Element ncItmEl = (Element) ncItmNode.item(nci);
						TimeInfo ncStartTimeInfo = new TimeInfo();
						ncStartTimeInfo.setMovieTime(Integer.parseInt(ncItmEl
								.getAttribute("S")));
						ncItem.setStartTimeInfo(ncStartTimeInfo);
						TimeInfo ncEndTimeInfo = new TimeInfo();
						ncEndTimeInfo.setMovieTime(Integer.parseInt(ncItmEl
								.getAttribute("E")));
						ncItem.setEndTimeInfo(ncEndTimeInfo);
						ncItem.setText(getCharacterDataFromElement(ncItmEl));
						String sf = ncItmEl.getAttribute("SF");
						String ef = ncItmEl.getAttribute("EF");
						ncItem.setStartTimeFinal(Util.isTrue(sf));
						ncItem.setEndTimeFinal(Util.isTrue(ef));
						String startEntID = ncItmEl
								.getAttribute("COMP_START_ENTITY_ID");
						String endEntID = ncItmEl
								.getAttribute("COMP_END_ENTITY_ID");
						ncItem.setStartCompaundEntityID(startEntID);
						ncItem.setEndCompaundEntityID(endEntID);
						track.addNonCompleteEvent(ncItem);
					}
				}

				NodeList etNode = tEl.getElementsByTagName("ENTERED_TEXT");
				String fid = tEl.getAttribute("FID");
				for (int et = 0; et < etNode.getLength(); et++) {
					Element etEl = (Element) etNode.item(et);
					NodeList etItmNode = etEl.getElementsByTagName("ITEM");

					for (int eti = 0; eti < etItmNode.getLength(); eti++) {
						Element etItmEl = (Element) etItmNode.item(eti);

						TimeInfo etStartTimeInfo = new TimeInfo();
						etStartTimeInfo.setMovieTime(Integer.parseInt(etItmEl
								.getAttribute("S")));

						String enteredTxt = getCharacterDataFromElement(etItmEl);

						if (enteredTxt.trim().length() != 0) {
							track.addEnteredTextEvent(new SS3Item(enteredTxt,
									etStartTimeInfo));
						}
					}
				}

				NodeList entNode = tEl.getElementsByTagName("ENTITY");
				for (int j = 0; j < entNode.getLength(); j++) {
					SS3Entity entity = new SS3Entity();
					Element entEl = (Element) entNode.item(j);
					entity.setParticipantID(entEl.getAttribute("PARTICIPANT-ID"));
					entity.setID(entEl.getAttribute("ID"));
					entity.setCoderID(entEl.getAttribute("CODER-ID"));
					entity.setTimeCreated(entEl.getAttribute("TIME_CREATED"));
					entity.setTimeUpdated(entEl.getAttribute("TIME_UPDATED"));
					entity.setRefEntityId(entEl.getAttribute("REF_ENTITY_ID"));
					entity.setParentEntityID(entEl.getAttribute("PARENT_ENTITY_ID"));

					String compStartEntityID = entEl
							.getAttribute("COMP_START_ENTITY_ID");
					if (compStartEntityID != null) {
						entity.setStartCompaundEntityID(compStartEntityID);
					}

					String compEndEntityID = entEl
							.getAttribute("COMP_END_ENTITY_ID");
					if (compEndEntityID != null) {
						entity.setEndCompaundEntityID(compEndEntityID);
					}

					entity.setReference(entEl.getAttribute("REF"));
					
					if (fieldID.equalsIgnoreCase(Util.DOM_HANDSHAPE_FIELD_ID)
							|| fieldID.equalsIgnoreCase(Util.NDOM_HANDSHAPE_FIELD_ID)
							|| fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID)
							|| fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
					
						String lockedGlossPhoneAlignment = entEl.getAttribute("LOCKED_GLOSS_PHON_ALIGNMENT");
						boolean isLockedGlossPhoneAlignment = "T".equalsIgnoreCase(lockedGlossPhoneAlignment);
						entity.setLockedGlossPhoneAlignments(isLockedGlossPhoneAlignment);
						
						if(fieldID.equalsIgnoreCase(Util.DOM_HANDSHAPE_FIELD_ID)) {
							track1000 = track;
						}
						
						if(fieldID.equalsIgnoreCase(Util.NDOM_HANDSHAPE_FIELD_ID)) {
							track2000 = track;
						}
						
						if(fieldID.equalsIgnoreCase(Util.DOM_HAND_FIELD_ID)) {
							track10501 = track;
						}
						
						if(fieldID.equalsIgnoreCase(Util.NDOM_HAND_FIELD_ID)) {
							track10502 = track;
						}
					}

					String depGlsID = null, depHandID, depHandShapeID, depLocationID, depSignTypeID;

					if ((fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID) || fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID))) {
						
						if(fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
							track10000 = track;
						}

						if(fieldID.equalsIgnoreCase(Util.NDOM_GLOSS_FIELD_ID)) {
							track10001 = track;
						}
						
						depGlsID = entEl.getAttribute("DEPENDENT_GLOSS_ID");
						depHandID = entEl.getAttribute("DEPENDENT_HAND_ID");
						depHandShapeID = entEl.getAttribute("DEPENDENT_HAND_SHAPE_ID");
						depLocationID = entEl.getAttribute("DEPENDENT_LOCATION_ID");
						depSignTypeID = entEl.getAttribute("DEPENDENT_SIGN_TYPE_ID");
						
						if(depGlsID == null || depGlsID.length() == 0) {
							depGlsID = entEl.getAttribute("CORRESPONDING_GLOSS_ENTITY_ID");
						}
						
						entity.setDependentGlossEntityID(depGlsID);
						entity.setDependentHandEntityID(depHandID);
						entity.setDependentHandShapeEntityID(depHandShapeID);
						entity.setDependentLocationEntityID(depLocationID);
						entity.setDependentSignTypeEntityID(depSignTypeID);

						GlossEntityProperties properties = entity
								.getGlossEntityProperties();

						if (properties == null) {
							properties = new GlossEntityProperties();
							entity.setGlossEntityProperties(properties);
						}

						String dominance = entEl.getAttribute("DOMINANCE");
						String markedNumberOfHands = entEl
								.getAttribute("MARKED_NUMBER_OF_HANDS");

						properties.setDominamtHand(dominance);
						properties
								.setMarkedNumberOfHandsString(markedNumberOfHands);
						
						NodeList sbNode = entEl.getElementsByTagName("SIGNBANKDATA");
						if (sbNode != null) {
							for (int jj = 0; jj < sbNode.getLength(); jj++) {
								Element sbEl = (Element) sbNode.item(jj);
								
								String verified = sbEl.getAttribute("VERIFIED");
								String differingFromOccurrence = sbEl.getAttribute("DIFF_FROM_OCCURRENCE");
								String resource = sbEl.getAttribute("RESOURCE");
								String mainGlossID = sbEl.getAttribute("MAIN_GLOSS_ID");
								String variantID = sbEl.getAttribute("VARIANT_ID");
								String occurrenceID = sbEl.getAttribute("OCCURRENCE_ID");
								
								if (verified != null && !verified.isEmpty() && verified.equals("T"))
									properties.setVerifiedFromSignBank(true);
								
								if (differingFromOccurrence != null && !differingFromOccurrence.isEmpty() && differingFromOccurrence.equals("T"))
									properties.setDifferingFromOccurrence(true);
								
								if (resource != null && !resource.isEmpty())
									properties.setSourceResource(resource);
								
								if (mainGlossID != null && !mainGlossID.isEmpty())
									properties.setMainGlossID(mainGlossID);
								
								if (variantID != null && !variantID.isEmpty())
									properties.setVariantID(variantID);
								
								if (occurrenceID != null && !occurrenceID.isEmpty())
									properties.setOccurrenceID(occurrenceID);
							}
						}
						
						
						/**
						 * <ENTITY> <PROPERTIES SIGN=
						 * "FINGERSPELLED/LOAN/LEXICAL/CLASSIFIER/GESTURE/NUMBER"
						 * NAME_SIGN="T/F" PASSIVE_BASE_ARM="T/F" NO_OF_HANDS="1/2"
						 * 
						 * TWOHANDED_HADSHAPES="SAME/ALTERNATING_SAME/DIFFERENT"
						 * START_END_HANDSHAPES="SAME/DIFFERENT"
						 * LEFT_HAND_START_END_HANDSHAPE="SAME/DIFFERENT"
						 * RIGHT_HAND_START_END_HANDSHAPE="SAME/DIFFERENT" />
						 * <DOMINANCE="L/R"/> <MARKED_NUMBER_OF_HANDS="T/F"/>
						 * </ENTITY>
						 */
						NodeList propNode = entEl
								.getElementsByTagName("PROPERTIES");
						for (int jj = 0; jj < propNode.getLength(); jj++) {
							// SS3Entity entity = new SS3Entity();
							Element propEl = (Element) propNode.item(jj);
							String sign = propEl.getAttribute("SIGN");
							String nameSign = propEl.getAttribute("NAME_SIGN");
							String passiveBaseArm = propEl
									.getAttribute("PASSIVE_BASE_ARM");
							String numberOfHands = propEl
									.getAttribute("NO_OF_HANDS");

							String twoHanded = propEl
									.getAttribute("TWOHANDED_HADSHAPES");

							String startEndHandshape = propEl
									.getAttribute("START_END_HANDSHAPES");
							String leftHandStartEndHandshape = propEl
									.getAttribute("LEFT_HAND_START_END_HANDSHAPE");
							String rightHandStartEndHandshape = propEl
									.getAttribute("RIGHT_HAND_START_END_HANDSHAPE");

							properties.setClassifierAsString(sign);
							properties.setNameSign(nameSign.trim()
									.equalsIgnoreCase("T"));
							properties.setPassiveBaseArm(passiveBaseArm.trim()
									.equalsIgnoreCase("T"));
							
							properties.setTwoHanded(!entity.getDependentGlossEntityID().trim().isEmpty() ||
									numberOfHands.trim().equalsIgnoreCase("2"));
							
							properties
									.setTwoHandedHandshapesClassification(twoHanded);
							properties.setStartEndHandshape(startEndHandshape);

							properties
									.setStartEndHandshapeLeftHand(leftHandStartEndHandshape);
							properties
									.setStartEndHandshapeRightHand(rightHandStartEndHandshape);

							// if(twoHanded.trim().equalsIgnoreCase("SAME")) {
							// properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
							// } else
							// if(twoHanded.trim().equalsIgnoreCase("ALTERNATING_SAME"))
							// {
							// properties.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
							// } else
							// if(twoHanded.trim().equalsIgnoreCase("DIFFERENT")) {
							// properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);
							// } else {
							// properties.setTwoHandedHandshapesClassification("");
							// }
							/*
							 * <ENTITY CODER-ID="" COMP_END_ENTITY_ID=""
							 * COMP_START_ENTITY_ID="" DOMINANCE="R"
							 * MARKED_NUMBER_OF_HANDS="T" > <PROPERTIES
							 * NAME_SIGN="T" NO_OF_HANDS="2" PASSIVE_BASE_ARM="T"
							 * SIGN="" START_END_HANDSHAPES=""
							 * TWOHANDED_HADSHAPES="DIFFERENT HANDSHAPES LR"
							 * LEFT_HAND_START_END_HANDSHAPE
							 * ="DIFFERENT START/END HANDSHAPE"
							 * RIGHT_HAND_START_END_HANDSHAPE
							 * ="SAME START/END HANDSHAPE" />
							 */
							// if(startEndHandshape.trim().equalsIgnoreCase("SAME"))
							// {
							// properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							// } else
							// if(startEndHandshape.trim().equalsIgnoreCase("DIFFERENT"))
							// {
							// properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
							// } else {
							// properties.setStartEndHandshape("");
							// }
							//
							// if(leftHandStartEndHandshape.trim().equalsIgnoreCase("SAME"))
							// {
							// properties.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							// } else
							// if(leftHandStartEndHandshape.trim().equalsIgnoreCase("DIFFERENT"))
							// {
							// properties.setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
							// } else {
							// properties.setStartEndHandshapeLeftHand("");
							// }
							//
							// if(rightHandStartEndHandshape.trim().equalsIgnoreCase("SAME"))
							// {
							// properties.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
							// } else
							// if(rightHandStartEndHandshape.trim().equalsIgnoreCase("DIFFERENT"))
							// {
							// properties.setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
							// } else {
							// properties.setStartEndHandshapeLeftHand("");
							// }
							//
							// System.out.println("Entity ID = " + entity.getID() +
							// ", properties = " + properties);
							//
							entity.setGlossEntityProperties(properties);
						}
					}

					NodeList itmNode = entEl.getElementsByTagName("ITEM");

					for (int l = 0; l < itmNode.getLength(); l++) {
						SS3Item item = new SS3Item();
						Element itmEl = (Element) itmNode.item(l);
						item.setId(itmEl.getAttribute("VID"));

						String textOnly = itmEl.getAttribute("TEXT_ONLY");
						if (textOnly != null
								& textOnly.trim().equalsIgnoreCase("T")) {
							item.setTextOnly(true);
						}

						item.setLocationRegionID(itmEl.getAttribute("LOCATION_REGION_ID"));
						String locationTypeIDStr = itmEl.getAttribute("LOCATION_TYPE_ID");
						if (locationTypeIDStr != null && !locationTypeIDStr.isEmpty()) {
							int locationTypeID = Integer.parseInt(locationTypeIDStr);
							if (SS3Singleton.getLocationConfig().getLocationType(locationTypeID) == null) {
								continue;
							}
							item.setLocationTypeID(locationTypeID);
						}


						item.setNoteID(itmEl.getAttribute("NOTE_ID"));

						TimeInfo startTimeInfo = new TimeInfo();
						startTimeInfo.setMovieTime(Integer.parseInt(itmEl
								.getAttribute("S")));
						TimeInfo endTimeInfo = new TimeInfo();
						endTimeInfo.setMovieTime(Integer.parseInt(itmEl
								.getAttribute("E")));
						
						if (fieldID.equals(Util.FREE_TEXT_FIELD_ID)) {
							String validTime = entEl.getAttribute("VALID_TIME");
							boolean isValidTime = (validTime != null && validTime.equals("T")) ? true : false;
							if (!isValidTime) {	// if time is wrong in db, then set it to the correct time
								int packetDuration = db.getPacketDuration();
								int buffer = packetDuration*2;
								
								int newStartMoveTime = startTimeInfo.getMovieTime() - buffer;
								int newEndMovieTime = endTimeInfo.getMovieTime() - buffer;
								
								startTimeInfo.setMovieTime(newStartMoveTime);
								endTimeInfo.setMovieTime(newEndMovieTime);
							}
							entity.setValidTime(true);	// if we went through the block, or not, set this to valid time
						}

						item.setStartTimeInfo(startTimeInfo);
						item.setEndTimeInfo(endTimeInfo);
						String txt = getCharacterDataFromElement(itmEl);
						item.setText(txt);
						entity.addItem(item);
					}
					
					String lockedLeftRightAlignment = entEl.getAttribute("LOCKED_LEF_RIGHT_ALIGNMENT");
					String lockedGlossPhoneAlignment = entEl.getAttribute("LOCKED_GLOSS_PHON_ALIGNMENT");
					
					if(lockedGlossPhoneAlignment.trim().length() == 0) {
						setGlossDependencies = true;
					}
					
					boolean isLockedLeftRightAlignment = "T".equalsIgnoreCase(lockedLeftRightAlignment);
					boolean isLockedGlossPhoneAlignment = "T".equalsIgnoreCase(lockedGlossPhoneAlignment);
					
					// need to move this logic down here - for locking information
					// check if lockedLeftRight is correct when compared to the dependent gloss (it has been known to be wrong in the db)
					if (depGlsID != null) {
						SS3Track depTrack = null;
						if (track.equals(track10000)) {
							depTrack = track10001;
						} else if (track.equals(track10001)) {
							depTrack = track10000;
						}
						
						if (depTrack != null) {
							SS3Entity depGls = depTrack.getSS3Entity(depGlsID);
							if (depGls != null) {
								int entityStartTime = entity.getStartTime();
								int entityEndTime = entity.getEndTime();
								int depGlsStartTime = depGls.getStartTime();
								int depGlsEndTime = depGls.getEndTime();
								
								if (entityStartTime != depGlsStartTime || entityEndTime != depGlsEndTime) {
									isLockedLeftRightAlignment = false;
									
									depGls.setLockedLeftRightAlignments(isLockedLeftRightAlignment);
								}
							}
						}
					}
					
					entity.setLockedGlossPhoneAlignments(isLockedGlossPhoneAlignment);
					entity.setLockedLeftRightAlignments(isLockedLeftRightAlignment);

					// Check to make sure entity start time is <= end time.
					// if not then don't add to track.
					if (!entity.getItems().isEmpty()) {
						int entityStartTime = entity.getItems().get(0).getStartTimeInfo().getMovieTime();
						int entityEndTime = entity.getItems().get(entity.getItems().size() - 1).getEndTimeInfo().getMovieTime();
						if (entityStartTime <= entityEndTime) {
							track.addEntity(entity);
						}
					}
				}
				tracks.add(track);
			}
			
			if(setGlossDependencies) {
//				SS3Track track1000 = null;
//				SS3Track track2000 = null;
//				SS3Track track10501 = null;
//				SS3Track track10502 = null;
//				// gloss1
//				SS3Track track10000 = null;
//				// gloss2
//				SS3Track track10001 = null
				
				if(track1000 != null) {
					ArrayList<SS3Entity> entities = track1000.getEntities();
					for(SS3Entity entity : entities) {
						//entity.getStartTime();
						entity.getItems();
					}
				}
			}
			
			// fix signs saved previously into annotation file
			syncTracksToSignBank(track10000, track10001, track1000, track2000);
			syncTracksToSignBank(track10001, track10000, track1000, track2000);
						
			return tracks;
		}

		private String getCharacterDataFromElement(Element e) {
			Node child = e.getFirstChild();
			if (child instanceof CharacterData) {
				CharacterData cd = (CharacterData) child;
				return cd.getData();
			}
			return "";
		}
		
		/**
		 * <pre>
		 * Helper method: for all the entities within entitySourceHandTrack (just parsed)
		 * determine whether their sign bank properties (if they exist for a given entity) 
		 * are up to date with the current sign bank collection (also should be parsed at this point).
		 * 
		 * If an entity was added/edited via the SignBank interface, it was given a SIGNBANKDATA element in the annotation XML file.
		 * We determine here whether, since the last time this annotation file was opened, the entity now differs
		 * from the SignBank variant (read: occurrence) that it was sourced from. This would mean differing in terms of 
		 * handshapes and any other data that is coded via the Morph-Phon window.
		 * 
		 * This method is meant to be called twice, once with gloss1 as entitySource and once with gloss2 as entitySource.
		 * NOTE: Thus there will be inevitably some duplication of work since the gloss data being checked 
		 * is common between the gloss1 and gloss2 fields. This is unavoidable because there is a GlossEntityProperties object
		 * for each GlossChainedEventsEntity object (thus, a two handed sign is composed of two GCEE objects, each with
		 * their own GEP object). Because it remains possible to add a one-handed sign exclusively in the nondominant (gloss2) field,
		 * we cannot only loop through gloss1 and assume to hit all the signs in the annotation file.
		 * </pre>
		 * 
		 * @param entitySourceHandTrack The SS3Track to loop through and sync each entity found therein
		 * @param oppositeHandTrack The SS3Track on the opposite hand to entitySourceHandTrack (e.g. if entitySourceHandTrack 
		 * is gloss1, then oppositeHandTrack must be gloss2 -- this will be checked first, and if incorrect input will return false).
		 * @param domHSTrack The SS3Track for the dominant handshape field (right handshape field)
		 * @param nondomHSTrack The SS3Track for the nondominant handshape field (left handshape field)
		 * @return false if for any reason any of the entities encounters an error; true otherwise.
		 */
		private boolean syncTracksToSignBank(SS3Track entitySourceHandTrack, SS3Track oppositeHandTrack, 
				SS3Track domHSTrack, SS3Track nondomHSTrack) {
			if (entitySourceHandTrack == null || oppositeHandTrack == null 
					|| domHSTrack == null || nondomHSTrack == null)
				return false;
			
			// first sanity check the field id's on the tracks are all as expected given the inputs 
			if ( !domHSTrack.getId().equals(Util.DOM_HANDSHAPE_FIELD_ID))
				return false;
			
			if ( !nondomHSTrack.getId().equals(Util.NDOM_HANDSHAPE_FIELD_ID))
				return false;
			
			if (entitySourceHandTrack.getId().equals(Util.DOM_GLOSS_FIELD_ID)
					&& !oppositeHandTrack.getId().equals(Util.NDOM_GLOSS_FIELD_ID))
				return false;

			if (entitySourceHandTrack.getId().equals(Util.NDOM_GLOSS_FIELD_ID)
					&& !oppositeHandTrack.getId().equals(Util.DOM_GLOSS_FIELD_ID))
				return false;
			
			// then proceed to loop through the entitySourceHandTrack and sync all entities
			for (SS3Entity entity: entitySourceHandTrack.getEntities()) {
				if (entity.getGlossEntityProperties().isVerifiedFromSignBank()) {
					for (SS3Item item: entity.getItems()) {
						String itemText = item.getText();
						if (itemText != null && !itemText.isEmpty()) {
							if (!itemText.equals("Final Hold") && !itemText.equals("Initial Hold")
									&& !itemText.equals("Onset") && !itemText.equals("Offset")) {	// ignore these events
								if (!syncEntityToSignBank(entity, itemText, 
										entitySourceHandTrack, oppositeHandTrack, domHSTrack, nondomHSTrack)) {
									continue;
								}
								
								break;	// should only need to sync for one SS3Item per SS3Entity
							}
						}		
					}						
				}
			}
			
			return true;
		}
		
		/**
		 * <pre>
		 * Helper method for syncTracksToSignBank()
		 * Take in a single SS3Entity with its label, and compare the sign data found in its GlossEntityProperies
		 * against the sign data from the SignBank variant (read: occurrence) which it came from.
		 * If any discrepancies are found, then the entity is marked as differing.
		 * </pre>
		 * 
		 * @param entity The SS3Entity to compare
		 * @param entityLabel The label of the SS3Entity
		 * @param entitySourceHandTrack The track (either gloss1 or gloss2) which this SS3Entity came from
		 * @param oppositeHandTrack  The opposite track (gloss1 or gloss2) of what this SS3Entity came from
		 * @param domHSTrack The SS3Track for the dominant handshape field (right handshape field)
		 * @param nondomHSTrack The SS3Track for the nondominant handshape field (left handshape field)
		 * @return false if for any reason the sync encounters an error; true otherwise.
		 */
		private boolean syncEntityToSignBank(SS3Entity entity, String entityLabel,
				SS3Track entitySourceHandTrack, SS3Track oppositeHandTrack, 
				SS3Track domHSTrack, SS3Track nondomHSTrack) {
			SignBankCollection signBankCollection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
			SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
			
			GlossEntityProperties properties = entity.getGlossEntityProperties();

			String mainGlossID = properties.getMainGlossID();
			String variantID = properties.getVariantID();
			String occurrenceID = properties.getOccurrenceID();
			
			SignBankOccurrence entityOccurrence = signBankCollection.getOccurrenceWithID(mainGlossID, variantID, occurrenceID);
			
			if (entityOccurrence == null)
				return false;
			
			// NEXT get occurrence data
			String occSourceResource = entityOccurrence.getSIgnBankResourceFileName();

			String occDomStartHS = entityOccurrence.getDomStartHS();
			String occNondomStartHS = entityOccurrence.getNondomStartHS();
			String occDomEndHS = entityOccurrence.getDomEndHS();
			String occNondomEndHS = entityOccurrence.getNondomEndHS();
			
			String occLeftRightHS = entityOccurrence.getLeftRightHS();
			String occStartEndHS = entityOccurrence.getStartEndHS();
			String occStartEndLeftHS = entityOccurrence.getStartEndLeftHS();
			String occStartEndRightHS = entityOccurrence.getStartEndRightHS();

			boolean occTwoHanded = entityOccurrence.isTwoHanded();
			boolean occMarkedHands = entityOccurrence.isMarkedHands();
			boolean occPassiveBaseArm = entityOccurrence.isPassiveBaseArm();
			
			String occSignType = entityOccurrence.getSignType();
			
			// NEXT get entity data			
			String leftRightHS = properties.getTwoHandedHandshapesClassification();
			String startEndHS = properties.getStartEndHandshape();
			String startEndLeftHS = properties.getStartEndHandshapeLeftHand();
			String startEndRightHS = properties.getStartEndHandshapeRightHand();
			
			boolean twoHanded = properties.isTwoHanded();
			boolean markedHands = properties.isMarkedNumberOfHands();
			boolean passiveBaseArm = properties.isPassiveBaseArm();
			
			String signType = null;
			if (properties.isLexical())
				signType = "LEXICAL";
			else if (properties.isFingerspelled()) 
				signType = "FINGERSPELLED";
			else if (properties.isLoan())
				signType = "LOAN";
			else if (properties.isNumber())
				signType = "NUMBER";
			else if (properties.isClassifier())
				signType = "CLASSIFER";
			else if (properties.isGesture()) 
				signType = "GESTURE";

			// GLOSS1 is DOM
			// GLOSS2 is NONDOM
			SS3Track domGlossTrack = null, nondomGlossTrack = null;
			String domGlossEntityID = null, nondomGlossEntityID = null, domHSEntityID = null, nondomHSEntityID = null;
			String domStartHS = null, nondomStartHS = null, domEndHS = null, nondomEndHS = null;
			
			if (entitySourceHandTrack.getId().equals(Util.DOM_GLOSS_FIELD_ID)) {	// if entity came from gloss1/dom field
				domGlossTrack = entitySourceHandTrack;
				nondomGlossTrack = oppositeHandTrack;
				
				domGlossEntityID = entity.getID();
				domHSEntityID = entity.getDependentHandShapeEntityID();
				
				nondomGlossEntityID = entity.getDependentGlossEntityID();	
			} else if (entitySourceHandTrack.getId().equals(Util.NDOM_GLOSS_FIELD_ID)) {	// if entity came from gloss2/nondom field
				 nondomGlossTrack = entitySourceHandTrack;
				 domGlossTrack = oppositeHandTrack;
				
				nondomGlossEntityID = entity.getID();
				nondomHSEntityID = entity.getDependentHandShapeEntityID();
				
				domGlossEntityID = entity.getDependentGlossEntityID();
			}
			
			// get dom labels	from the coding scheme
			if (domGlossEntityID != null && !domGlossEntityID.isEmpty()) {
				if (domHSEntityID == null || domHSEntityID.isEmpty()) {
					for (SS3Entity dGlossEntity: domGlossTrack.getEntities()) {
						if (dGlossEntity.getID().equals(domGlossEntityID)) {
							domHSEntityID = dGlossEntity.getDependentHandShapeEntityID();							
							break;
						}
					}
				}
				
				if (domHSEntityID != null && !domHSEntityID.isEmpty()) {	// check again in case hs entity was not found in track
					for (SS3Entity dHSEntity: domHSTrack.getEntities()) {
						if (dHSEntity.getID().equals(domHSEntityID)) {
							ArrayList<SS3Item> domHSItems = dHSEntity.getItems();
							if (!domHSItems.isEmpty()) {
								int domHSItemsSize = domHSItems.size();
								SS3Item domStartHSItem = domHSItems.get(0);
								SS3Item domEndHSItem = domHSItems.get(domHSItemsSize - 1);
								
								String domStartHSItemID = domStartHSItem.getId();
								String domEndHSItemID = domEndHSItem.getId();
								
								SS3FieldValue domStartHSValue = 
										cs.getFieldsMapSS3FieldValue(Util.DOM_HANDSHAPE_FIELD_ID, domStartHSItemID);
								domStartHS = domStartHSValue.getLabel();
								
								SS3FieldValue domEndHSValue = 
										cs.getFieldsMapSS3FieldValue(Util.DOM_HANDSHAPE_FIELD_ID, domEndHSItemID);
								domEndHS = domEndHSValue.getLabel();
								
								break;
							}
						}
					}
				}
			}			
			
			// get nondom labels from the coding scheme
			if (nondomGlossEntityID != null && !nondomGlossEntityID.isEmpty()) {
				if (nondomHSEntityID == null || nondomHSEntityID.isEmpty()) {
					for (SS3Entity ndGlossEntity: nondomGlossTrack.getEntities()) {
						if (ndGlossEntity.getID().equals(nondomGlossEntityID)) {
							nondomHSEntityID = ndGlossEntity.getDependentHandShapeEntityID();								
							break;
						}
					}
				}
				
				if (nondomHSEntityID != null && !nondomHSEntityID.isEmpty()) {	// check again in case hs entity was not found in track
					for (SS3Entity ndHSEntity: nondomHSTrack.getEntities()) {
						if (ndHSEntity.getID().equals(nondomHSEntityID)) {
							ArrayList<SS3Item> nondomHSItems = ndHSEntity.getItems();
							if (!nondomHSItems.isEmpty()) {
								int nondomHSItemsSize = nondomHSItems.size();
								SS3Item nondomStartHSItem = nondomHSItems.get(0);
								SS3Item nondomEndHSItem = nondomHSItems.get(nondomHSItemsSize - 1);
								
								String nondomStartHSItemID = nondomStartHSItem.getId();
								String nondomEndHSItemID = nondomEndHSItem.getId();
								
								SS3FieldValue nondomStartHSValue = 
										cs.getFieldsMapSS3FieldValue(Util.NDOM_HANDSHAPE_FIELD_ID, nondomStartHSItemID);
								nondomStartHS = nondomStartHSValue.getLabel();
								
								SS3FieldValue nondomEndHSValue =
										cs.getFieldsMapSS3FieldValue(Util.NDOM_HANDSHAPE_FIELD_ID, nondomEndHSItemID);
								nondomEndHS = nondomEndHSValue.getLabel();
								
								break;
							}
						}
					}
				}
			}			
			
			// NEXT begin to compare the sign-bank-defined occurrence data to the saved entity data
			boolean differingFromOccurrence = false;
			
			// first with the hs labels
			if (occDomStartHS != null && domStartHS != null && !occDomStartHS.equals(domStartHS))
				differingFromOccurrence = true;
			if (occNondomStartHS != null && nondomStartHS != null && !occNondomStartHS.equals(nondomStartHS))
				differingFromOccurrence = true;
			if (occDomEndHS != null && domEndHS != null && !occDomEndHS.equals(domEndHS))
				differingFromOccurrence = true;
			if (occNondomEndHS != null && nondomEndHS != null && !occNondomEndHS.equals(nondomEndHS))
				differingFromOccurrence = true;

			// then with the hs same/diff classifications
			if (occLeftRightHS != null) {
				if (twoHanded && occLeftRightHS.equals("SAME") 
						&& !leftRightHS.equals(GlossEntityProperties.SAME_HANDSHAPES_LR)) {
					differingFromOccurrence = true;
					
					if (occStartEndHS != null) {
						if (occStartEndHS.equals("SAME") && 
								!startEndHS.equals(GlossEntityProperties.SAME_START_END_HANDSHAPE))
							differingFromOccurrence = true;
						else if (occStartEndHS.equals("DIFFERENT") && 
								!startEndHS.equals(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE))
							differingFromOccurrence = true;
					}
				} else if (twoHanded && occLeftRightHS.equals("DIFFERENT") && 
						!leftRightHS.equals(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR)) {
					differingFromOccurrence = true;
					
					if (occStartEndLeftHS != null) {
						if (occStartEndLeftHS.equals("SAME") && 
								!startEndLeftHS.equals(GlossEntityProperties.SAME_START_END_HANDSHAPE))
							differingFromOccurrence = true;
						else if (occStartEndLeftHS.equals("DIFFERENT") && 
								!startEndLeftHS.equals(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE))
							differingFromOccurrence = true;
					}
					
					if (occStartEndRightHS != null) {
						if (occStartEndRightHS.equals("SAME") && 
								!startEndRightHS.equals(GlossEntityProperties.SAME_START_END_HANDSHAPE))
							differingFromOccurrence = true;
						else if (occStartEndRightHS.equals("DIFFERENT") && 
								!startEndRightHS.equals(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE))
							differingFromOccurrence = true;
					}
				}
			}
			
			if (occTwoHanded != twoHanded)
				differingFromOccurrence = true;
			
			if (occMarkedHands != markedHands)
				differingFromOccurrence = true;
			
			if (occPassiveBaseArm != passiveBaseArm)
				differingFromOccurrence = true;
			
			if ( !occSignType.equals(signType))
				differingFromOccurrence = true;
			
			properties.setDifferingFromOccurrence(differingFromOccurrence);
			
			// regardless of differing, always set the source resource on the entity properties to the now current source resource
			properties.setSourceResource(occSourceResource);			
			properties.setVerifiedFromSignBank(true);	// should already be set anyways
			
			return true;
		}

		// public static void main(String[] args) {
		// String file = "SignStreamDatabases" + java.io.File.separator
		// + "Test" + java.io.File.separator + "macro_unit_1.xml";
		// SS3ZipFilesParser parser = new SS3ZipFilesParser();
		// SS3Database db = parser.parseSS3Database(file);
		// }
	}
