package edu.bu.signstream.common.util;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Hashtable;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class ErrorMessages {

    /**
     * This class contains warning messages that are used in SignStream.
     *
     * To add another message, please add
     * public static String NEW_MESSAGE_TITLE = "x";
     * where x is any unique String, and NEW_MESSAGE_TITLE is this message identifier,
     * then add warning message wording in a String array:
     * String[] newMessageWording = new String[4];
     * newMessageWording[0] = "Warning: warning line 1 ";
     * newMessageWording[1] = "warning line 2 ";
     * newMessageWording[2] = "warning line 3 ";
     * newMessageWording[3] = "warning line 4.";
     * messages.put(ErrorMessages.NEW_MESSAGE_TITLE, newMessageWording);
     *
     * To show warning message in an application, please call
     * SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.NEW_MESSAGE_TITLE);
     */

			
	public static String EVENT_START_TIME_PRECEDE_END_TIME = "0";
	public static String EVENT_END_TIME_FOLLOW_START_TIME = "1";
	public static String CHAINS_CANNOT_OVERLAP = "2";	
	public static String EVENT_EXTEND_BEYOND_UTTERANCE = "3";
	public static String GLOSS_EVENT_EXTEND_BEYOND_UTTERANCE = "4";
	public static String GLOSS_CHAINS_CANNOT_OVERLAP = "5";	
	public static String UTTERANCE_START_BEFORE_MOVIE_START = "6";
	public static String UTTERANCE_END_AFTER_MOVIE_END = "7";
	public static String UTTERANCE_MAXIMAL_START_TIME = "8";
	public static String UTTERANCE_MINIMAL_END_TIME = "9";

	public static String SET_START_INITIAL_HOLD_CANT_DELETE_ONSET = "11";
	public static String SET_START_VALUE__CANT_DELETE_INITIAL_HOLD = "12";
	public static String SET_START_VALUE_CANT_DELETE_ONSET = "13";
	public static String SET_START_FINAL_HOLD_CANT_DELETE_VALUE = "14";
	public static String SET_START_OFFSET_CANT_DELETE_VALUE = "15";
	public static String SET_START_OFFSET_CANT_DELETE_FINAL_HOLD = "16";

	public static String SET_END_ONSET_CANT_DELETE_INITIAL_HOLD = "17";
	public static String SET_END_INITIAL_HOLD_CANT_DELETE_VALUE = "18";
	public static String SET_END_ONSET_CANT_DELETE_VALUE = "19";
	public static String SET_END_VALUE_CANT_DELETE_FINAL_HOLD = "20";
	public static String SET_END_VALUE_CANT_DELETE_OFFSET = "21";
	public static String SET_END_FINAL_HOLD_CANT_DELETE_OFFSET = "22";
	
	public static String ADD_FINAL_HOLD_CANT_ADJUST_OFFSET = "23";
	public static String ADD_INITIAL_HOLD_CANT_ADJUST_ONSET = "24";
	
	public static String CANT_CREATE_GLOSS_WRONG_ORDER = "25";
	public static String SELECT_VALID_VIDEO_FILE = "26";
	
	public static String CANT_LOCK_GLOSSES = "27";
	
	public static String UNLOCK_GLOSS_PHON_ALIGNMENTS = "28";
	public static String UNLOCK_RIGHT_LEFT_HAND_ALIGNMENTS = "29";
	
	public static String FUNCTIONALITY_NOT_YET_IMPLEMENTED = "30";

    public static String EVENT_NOT_SELECTED = "31";
    
    public static String CANT_EDIT_NON_SEGMENT_TIER = "32";
    public static String CANT_SELECT_DISPLAY_TIME_PERIOD = "33";
    
    public static String SET_ONE_FRAME_GLOSS_DIFF_HANDSHAPES = "34";
    public static String SELECT_HANDSHAPES_MULTIPLE = "35";
    public static String SELECT_HANDSHAPES_EXACTLY_ONE = "36";
    public static String CANT_SET_DIF_START_END_ONE_FRAME = "37";
    public static String CANT_EDIT_NON_TEMPORAL_PARTITION = "38";
    
    public static String TEXT_CONTAINS_SEGMENT_TIER_SEPARATOR = "50";

    public static String ADD_DUPLICATE_ANNOTATOR = "51";

	private JLabel msgLabel1 = new JLabel();
	private JLabel msgLabel2 = new JLabel();
	private JLabel msgLabel3 = new JLabel();
	private JLabel msgLabel4 = new JLabel();
	
	private JDialog dialog = null;
	
	Hashtable<String, String[]> messages = new Hashtable<String, String[]>();

	public ErrorMessages() {

		String[] glossChainExtendBeyondUtterance = new String[4];
		glossChainExtendBeyondUtterance[0] = "Warning: The gloss chain cannot extend beyond";
		glossChainExtendBeyondUtterance[1] = "the start or end time of the utterance.";
		glossChainExtendBeyondUtterance[2] = "You must adjust the boundary of the utterance";
		glossChainExtendBeyondUtterance[3] = "and then try again.";
		messages.put(ErrorMessages.GLOSS_EVENT_EXTEND_BEYOND_UTTERANCE, glossChainExtendBeyondUtterance);

		String[] glossChainsCannotOverlap = new String[4];
		glossChainsCannotOverlap[0] = "Warning: The gloss chain cannot overlap";
		glossChainsCannotOverlap[1] = "with another gloss chain. You must";
		glossChainsCannotOverlap[2] = "adjust the boundaries of the adjacent";
		glossChainsCannotOverlap[3] = "gloss chain and then try again.";
		messages.put(ErrorMessages.GLOSS_CHAINS_CANNOT_OVERLAP, glossChainsCannotOverlap);

		String[] eventExtendBeyondUtterance = new String[4];
		eventExtendBeyondUtterance[0] = "Warning: The event cannot extend beyond ";
		eventExtendBeyondUtterance[1] = "the start or end time of the utterance.";
		eventExtendBeyondUtterance[2] = "You must adjust the boundary of the ";
		eventExtendBeyondUtterance[3] = "utterance and then try again.";
		messages.put(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE, eventExtendBeyondUtterance);
		
		String[] chainsCannotOverlap = new String[4];
		chainsCannotOverlap[0] = "Warning: The events chain cannot overlap";
		chainsCannotOverlap[1] = "with another events chain. You must";
		chainsCannotOverlap[2] = "adjust the boundaries of the adjacent";
		chainsCannotOverlap[3] = "events chain and then try again.";
		messages.put(ErrorMessages.CHAINS_CANNOT_OVERLAP, chainsCannotOverlap);
		
		String[] eventStartTimePrecedeEndTime = new String[4];
		eventStartTimePrecedeEndTime[0] = "Warning: The event's start time ";
		eventStartTimePrecedeEndTime[1] = "must precede the event's end time. ";
		eventStartTimePrecedeEndTime[2] = "You must manually adjust that ";
		eventStartTimePrecedeEndTime[3] = "end time and then try again. ";
		messages.put(ErrorMessages.EVENT_START_TIME_PRECEDE_END_TIME, eventStartTimePrecedeEndTime);
		
		String[] eventEndTimeFollowsStartTime = new String[4];
		eventEndTimeFollowsStartTime[0] = "Warning: The event's end time ";
		eventEndTimeFollowsStartTime[1] = "cannot be earlier than its start time.";
		eventEndTimeFollowsStartTime[2] = "You must manually adjust that ";
		eventEndTimeFollowsStartTime[3] = "start time and then try again. ";
		messages.put(ErrorMessages.EVENT_END_TIME_FOLLOW_START_TIME, eventEndTimeFollowsStartTime);
		
		String[] utteranceStartBeforeMovieStart = new String[4];
		utteranceStartBeforeMovieStart[0] = "Warning: The utterance cannot extend beyond ";
		utteranceStartBeforeMovieStart[1] = "the start or end time of the movie.";
		utteranceStartBeforeMovieStart[2] = "You must adjust the boundary of the ";
		utteranceStartBeforeMovieStart[3] = "utterance and then try again.";
		messages.put(ErrorMessages.UTTERANCE_START_BEFORE_MOVIE_START, utteranceStartBeforeMovieStart);
		
		String[] utteranceEndAfterMovieEnd = new String[4];
		utteranceEndAfterMovieEnd[0] = "Warning: The utterance cannot extend beyond ";
		utteranceEndAfterMovieEnd[1] = "the start or end time of the movie.";
		utteranceEndAfterMovieEnd[2] = "You must adjust the boundary of the ";
		utteranceEndAfterMovieEnd[3] = "utterance and then try again.";
		messages.put(ErrorMessages.UTTERANCE_END_AFTER_MOVIE_END, utteranceEndAfterMovieEnd);

		String[] utteranceMaximalStartTime = new String[4];
		utteranceMaximalStartTime[0] = "Warning: The utterance start time cannot come after ";
		utteranceMaximalStartTime[1] = "any start time of any manual/nonmanual/translation event ";
		utteranceMaximalStartTime[2] = "contained in any of its dependent segment tiers. ";
		utteranceMaximalStartTime[3] = "Delete and/or shorten any events necessary and try again. ";
		messages.put(ErrorMessages.UTTERANCE_MAXIMAL_START_TIME, utteranceMaximalStartTime);
		
		String[] utteranceMinimalEndTime = new String[4];
		utteranceMinimalEndTime[0] = "Warning: The utterance end time cannot come before ";
		utteranceMinimalEndTime[1] = "any end time of any manual/nonmanual/translation event ";
		utteranceMinimalEndTime[2] = "contained in any of its dependent segment tiers. ";
		utteranceMinimalEndTime[3] = "Delete and/or shorten any events necessary and try again. ";
		messages.put(ErrorMessages.UTTERANCE_MINIMAL_END_TIME, utteranceMinimalEndTime);
		
		// The final hold must end before the offset ends.  Please modify the offset and try again.		
		
		String[] setStartInitialHoldCantDeleteOnset = new String[4];
		setStartInitialHoldCantDeleteOnset[0] = "Warning: The onset must start";
		setStartInitialHoldCantDeleteOnset[1] = "before the initial hold starts.";
		setStartInitialHoldCantDeleteOnset[2] = "Please modify the onset and try again.";
		setStartInitialHoldCantDeleteOnset[3] = "";
		messages.put(ErrorMessages.SET_START_INITIAL_HOLD_CANT_DELETE_ONSET, setStartInitialHoldCantDeleteOnset);
		
		String[] setStartValueCantDeleteInitialHold = new String[4];
		setStartValueCantDeleteInitialHold[0] = "Warning: The initial hold must start";
		setStartValueCantDeleteInitialHold[1] = "before the value starts. Please modify";
		setStartValueCantDeleteInitialHold[2] = "the initial hold and try again.";
		setStartValueCantDeleteInitialHold[3] = "";
		messages.put(ErrorMessages.SET_START_VALUE__CANT_DELETE_INITIAL_HOLD, setStartValueCantDeleteInitialHold);

		String[] setStartValueCantDeleteOnset = new String[4];
		setStartValueCantDeleteOnset[0] = "Warning: The onset must start before";
		setStartValueCantDeleteOnset[1] = "the value starts.";
		setStartValueCantDeleteOnset[2] = "Please modify the onset and try again.";
		setStartValueCantDeleteOnset[3] = "";
		messages.put(ErrorMessages.SET_START_VALUE_CANT_DELETE_ONSET, setStartValueCantDeleteOnset);
		
		String[] setStartFinalHoldCantDeleteValue = new String[4];
		setStartFinalHoldCantDeleteValue[0] = "Warning: The value must start";
		setStartFinalHoldCantDeleteValue[1] = "before the final hold starts.";
		setStartFinalHoldCantDeleteValue[2] = "Please modify the value and ";
		setStartFinalHoldCantDeleteValue[3] = "try again.";
		messages.put(ErrorMessages.SET_START_FINAL_HOLD_CANT_DELETE_VALUE, setStartFinalHoldCantDeleteValue);

		String[] setStartOffsetCantDeleteValue = new String[4];
		setStartOffsetCantDeleteValue[0] = "Warning: The value must start";
		setStartOffsetCantDeleteValue[1] = "before the offset starts.";
		setStartOffsetCantDeleteValue[2] = "Please modify the value and ";
		setStartOffsetCantDeleteValue[3] = "try again.";
		messages.put(ErrorMessages.SET_START_OFFSET_CANT_DELETE_VALUE, setStartOffsetCantDeleteValue);
		
		String[] setStartOffsetCantDeleteFinalHold = new String[4];
		setStartOffsetCantDeleteFinalHold[0] = "Warning: The final hold must start";
		setStartOffsetCantDeleteFinalHold[1] = "before the offset starts.";
		setStartOffsetCantDeleteFinalHold[2] = "Please modify the final hold and";
		setStartOffsetCantDeleteFinalHold[3] = "try again.";
		messages.put(ErrorMessages.SET_START_OFFSET_CANT_DELETE_FINAL_HOLD, setStartOffsetCantDeleteFinalHold);
		
		String[] setEndOnsetCantDeleteInitialHold = new String[4];
		setEndOnsetCantDeleteInitialHold[0] = "Warning: The onset must end before";
		setEndOnsetCantDeleteInitialHold[1] = "the initial hold ends.";
		setEndOnsetCantDeleteInitialHold[2] = "Please modify the initial hold and";
		setEndOnsetCantDeleteInitialHold[3] = "try again.";
		messages.put(ErrorMessages.SET_END_ONSET_CANT_DELETE_INITIAL_HOLD, setEndOnsetCantDeleteInitialHold);

		String[] setEndInitialHoldCantDeleteValue = new String[4];
		setEndInitialHoldCantDeleteValue[0] = "Warning: The initial hold must";
		setEndInitialHoldCantDeleteValue[1] = "end before the value ends.";
		setEndInitialHoldCantDeleteValue[2] = "Please modify the initial hold";
		setEndInitialHoldCantDeleteValue[3] = "and try again.";
		messages.put(ErrorMessages.SET_END_INITIAL_HOLD_CANT_DELETE_VALUE, setEndInitialHoldCantDeleteValue);

		String[] setEndOnsetCantDeleteValue = new String[4];
		setEndOnsetCantDeleteValue[0] = "Warning: The onset must end before";
		setEndOnsetCantDeleteValue[1] = "the value ends.";
		setEndOnsetCantDeleteValue[2] = "Please modify the value and try again.";
		setEndOnsetCantDeleteValue[3] = "";
		messages.put(ErrorMessages.SET_END_ONSET_CANT_DELETE_VALUE, setEndOnsetCantDeleteValue);
		
		String[] setEndValueCantDeleteFinalHold = new String[4];
		setEndValueCantDeleteFinalHold[0] = "Warning: The value must end before";
		setEndValueCantDeleteFinalHold[1] = "the final hold ends.";
		setEndValueCantDeleteFinalHold[2] = "Please modify the final hold and";
		setEndValueCantDeleteFinalHold[3] = "try again.";
		messages.put(ErrorMessages.SET_END_VALUE_CANT_DELETE_FINAL_HOLD, setEndValueCantDeleteFinalHold);

		String[] setEndValueCantDeleteOffset = new String[4];
		setEndValueCantDeleteOffset[0] = "Warning: The value must end ";
		setEndValueCantDeleteOffset[1] = "before the offset ends.";
		setEndValueCantDeleteOffset[2] = "Please modify the offset and";
		setEndValueCantDeleteOffset[3] = "try again.";
		messages.put(ErrorMessages.SET_END_VALUE_CANT_DELETE_OFFSET, setEndValueCantDeleteOffset);
		
		String[] setEndFinalHoldDeleteOffset = new String[4];
		setEndFinalHoldDeleteOffset[0] = "Warning: The final hold must end";
		setEndFinalHoldDeleteOffset[1] = "before the offset ends.";
		setEndFinalHoldDeleteOffset[2] = "Please modify the offset and";
		setEndFinalHoldDeleteOffset[3] = "try again.";
		messages.put(ErrorMessages.SET_END_FINAL_HOLD_CANT_DELETE_OFFSET, setEndFinalHoldDeleteOffset);
		
		String[] addFinalHoldCantAdjustOffset = new String[4];
		addFinalHoldCantAdjustOffset[0] = "Warning: The chained event's final hold";
		addFinalHoldCantAdjustOffset[1] = "object cannot be added to accommodate ";
		addFinalHoldCantAdjustOffset[2] = "offset's end time. You must manually adjust";
		addFinalHoldCantAdjustOffset[3] = "offset's end time and then try again.";
		messages.put(ErrorMessages.ADD_FINAL_HOLD_CANT_ADJUST_OFFSET, addFinalHoldCantAdjustOffset);

		String[] addInitialHoldCantAdjustOnset = new String[4];
		addInitialHoldCantAdjustOnset[0] = "Warning: The entity's initial hold object";
		addInitialHoldCantAdjustOnset[1] = "cannot be added to accommodate onset's ";
		addInitialHoldCantAdjustOnset[2] = "start time. You must manually adjust onset's";
		addInitialHoldCantAdjustOnset[3] = "start time and then try again.";
		messages.put(ErrorMessages.ADD_INITIAL_HOLD_CANT_ADJUST_ONSET, addInitialHoldCantAdjustOnset);
		
		// CANT_CREATE_GLOSS_WRONG_ORDER, verify with Carol
		String[] cantCreateGlossWrongOrder = new String[4];
		cantCreateGlossWrongOrder[0] = "Warning: Can't create gloss in that order.";
		cantCreateGlossWrongOrder[1] = "";
		cantCreateGlossWrongOrder[2] = "s";
		cantCreateGlossWrongOrder[3] = "";
		messages.put(ErrorMessages.CANT_CREATE_GLOSS_WRONG_ORDER, cantCreateGlossWrongOrder);
		
		// "Warning: Please select at least one valid and available video file and try again."
		// SELECT_VALID_VIDEO_FILE
		String[] selectValidVideoFile = new String[4];
		selectValidVideoFile[0] = "Warning: Please select at least";
		selectValidVideoFile[1] = "one valid and available video";
		selectValidVideoFile[2] = "file and try again.";
		selectValidVideoFile[3] = "";
		messages.put(ErrorMessages.SELECT_VALID_VIDEO_FILE, selectValidVideoFile);
		
		// CANT_LOCK_GLOSSES
		String[] cantLockGlosses = new String[4];
		cantLockGlosses[0] = "Warning: This operation cannot be completed";
		cantLockGlosses[1] = "because it would create overlap with";
		cantLockGlosses[2] = "surrounding gloss items. Please readjust";
		cantLockGlosses[3] = "the adjacent glosses and try again.";
		messages.put(ErrorMessages.CANT_LOCK_GLOSSES, cantLockGlosses);

		String[] unlockGlossPhoneAlignments = new String[4];
		unlockGlossPhoneAlignments[0] = "Warning: This operation cannot";
		unlockGlossPhoneAlignments[1] = "be completed. You may wish to";
		unlockGlossPhoneAlignments[2] = "unlock Gloss-Phon alignments";
		unlockGlossPhoneAlignments[3] = "for the gloss and try again.";
		messages.put(ErrorMessages.UNLOCK_GLOSS_PHON_ALIGNMENTS, unlockGlossPhoneAlignments);

		
		String[] unlockLeftRightAlignments = new String[4];
		unlockGlossPhoneAlignments[0] = "Warning: This operation cannot be";
		unlockGlossPhoneAlignments[1] = "completed. You may wish to unlock";
		unlockGlossPhoneAlignments[2] = "the alignments between the glosses";
		unlockGlossPhoneAlignments[3] = "for the two hands and try again.";
		messages.put(ErrorMessages.UNLOCK_RIGHT_LEFT_HAND_ALIGNMENTS, unlockLeftRightAlignments);
		
		//FUNCTIONALITY_NOT_YET_IMPLEMENTED
		String[] functionalityNotImplemented = new String[4];
		functionalityNotImplemented[0] = "Warning: This functionality";
		functionalityNotImplemented[1] = "is not yet implemented.";
		functionalityNotImplemented[2] = "";
		functionalityNotImplemented[3] = "";
		messages.put(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED, functionalityNotImplemented);

        // EVENT_NOT_SELECTED
        String[] eventNotSelected = new String[4];
        eventNotSelected[0] = "Warning: This operation cannot be";
        eventNotSelected[1] = "completed. You may wish to select";
        eventNotSelected[2] = "an event or a text and try again.";
        eventNotSelected[3] = "";
        messages.put(ErrorMessages.EVENT_NOT_SELECTED, eventNotSelected);
        
        String[] cantEditNonSegmentTier = new String[4];
        cantEditNonSegmentTier[0] = "You cannot rename/delete a";
        	cantEditNonSegmentTier[1] = "Display Time Period layer";
        	cantEditNonSegmentTier[2] = "or a Temporal Partition";
    		cantEditNonSegmentTier[3] = "";
    		messages.put(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER, cantEditNonSegmentTier);
    		
    		String[] cantSelectDisplayTimePeriod = new String[4];
    		cantSelectDisplayTimePeriod[0] = "Please first select either";
    		cantSelectDisplayTimePeriod[1] = "a Temporal Partition to add under";
    		cantSelectDisplayTimePeriod[2] = "or a Segment to add after";
    		cantSelectDisplayTimePeriod[3] = "";
    		messages.put(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD, cantSelectDisplayTimePeriod);
    		
    		String[] setOneFrameGlossDiffHandshapes = new String[4];
    		setOneFrameGlossDiffHandshapes[0] = "Warning: If you want to";
    		setOneFrameGlossDiffHandshapes[1] = "create a one-frame gloss";
    		setOneFrameGlossDiffHandshapes[2] = "then both hands must have";
    		setOneFrameGlossDiffHandshapes[3] = "the same start/end handshapes.";
    		messages.put(ErrorMessages.SET_ONE_FRAME_GLOSS_DIFF_HANDSHAPES, setOneFrameGlossDiffHandshapes);
    		
    		String[] selectHandshapesMultiple = new String[4];
    		selectHandshapesMultiple[0] = "You must select more than one";
    		selectHandshapesMultiple[1] = "handshape since this gloss";
    		selectHandshapesMultiple[2] = "is currently set to";
    		selectHandshapesMultiple[3] = "dif start/end handshapes.";
    		messages.put(ErrorMessages.SELECT_HANDSHAPES_MULTIPLE, selectHandshapesMultiple);
    		
    		String[] selectHandshapesOnlyOne = new String[4];
    		selectHandshapesOnlyOne[0] = "You must select exactly one";
    		selectHandshapesOnlyOne[1] = "handshape since this gloss is";
    		selectHandshapesOnlyOne[2] = "exactly one frame long.";
    		selectHandshapesOnlyOne[3] = "";
    		messages.put(ErrorMessages.SELECT_HANDSHAPES_EXACTLY_ONE, selectHandshapesOnlyOne);
    		
    		String[] cantSetDifStartEndOneFrame = new String[4];
    		cantSetDifStartEndOneFrame[0] = "This is currently a one-frame gloss.";
    		cantSetDifStartEndOneFrame[1] = "If you want to switch to";
    		cantSetDifStartEndOneFrame[2] = "diff start/end handshapes,";
    		cantSetDifStartEndOneFrame[3] = "you must first extend the gloss.";
    		messages.put(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME, cantSetDifStartEndOneFrame);
    		
    		String[] cantEditNonTemporalPartition = new String[4];
    		cantEditNonTemporalPartition[0] = "You cannot rename/delete a";
    		cantEditNonTemporalPartition[1] = "Display Time Period layer";
    		cantEditNonTemporalPartition[2] = "";
    		cantEditNonTemporalPartition[3] = "";
        	messages.put(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION, cantEditNonTemporalPartition);
        		
    		String [] textContainsSegmentTierSeparator = new String[4];
    		textContainsSegmentTierSeparator[0] = "Warning: The text you have entered";
    		textContainsSegmentTierSeparator[1] = "cannot contain a colon (\":\") character";
    		textContainsSegmentTierSeparator[2] = "";
    		textContainsSegmentTierSeparator[3] = "";
    		messages.put(ErrorMessages.TEXT_CONTAINS_SEGMENT_TIER_SEPARATOR, textContainsSegmentTierSeparator);

    		String[] addDuplicateAnnotator = new String[4];
    		addDuplicateAnnotator[0] = "Error: This annotator/participant has already";
			addDuplicateAnnotator[1] = "been added to the collection. Please select a";
			addDuplicateAnnotator[2] = "different annotator/participant to add.";
			addDuplicateAnnotator[3] = "";
			messages.put(ErrorMessages.ADD_DUPLICATE_ANNOTATOR, addDuplicateAnnotator);
	}
	
	public void showErrorMessage(String messageKey) {
		String[] msg = messages.get(messageKey);
		
		if(msg == null) {
			return;
		}

		if (dialog == null) {

			msgLabel1.setText("Warning: The gloss chain cannot extend beyond");
			msgLabel2.setText("the start or end time of the utterance.");
			msgLabel3.setText("You must adjust the boundary of the utterance");
			msgLabel4.setText("and then try again.");

			UserInterfaceUtil.setLabelLNF(msgLabel1);
			UserInterfaceUtil.setLabelLNF(msgLabel2);
			UserInterfaceUtil.setLabelLNF(msgLabel3);
			UserInterfaceUtil.setLabelLNF(msgLabel4);

			dialog = new JDialog(SS3Singleton.getSignStreamApplication(), true);
			dialog.setTitle("Warning message");

			final JButton okBtn = new JButton("OK");

			JPanel msgPanel = new JPanel();
			GridBagConstraints gbc = new GridBagConstraints();

			msgPanel.setLayout(new GridBagLayout());
			gbc.anchor = GridBagConstraints.WEST;

			gbc.gridwidth = 3;
			gbc.gridx = 0;
			gbc.gridy = 0;
			msgPanel.add(msgLabel1, gbc);

			gbc.gridy = 1;
			gbc.gridx = 0;
			msgPanel.add(msgLabel2, gbc);

			gbc.gridy = 2;
			gbc.gridx = 0;
			msgPanel.add(msgLabel3, gbc);

			gbc.gridy = 3;
			gbc.gridx = 0;
			msgPanel.add(msgLabel4, gbc);

			gbc.gridy = 4;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);

			gbc.gridwidth = 1;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.gridy = 5;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);
			gbc.gridx = 2;
			msgPanel.add(okBtn, gbc);

			okBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					dialog.setVisible(false);
				}
			});

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					okBtn.requestFocus();
				}
			});

			Dimension d = new Dimension();
			int width = msgPanel.getPreferredSize().width + 60;
			int height = msgPanel.getPreferredSize().height + 70;

			d.setSize(width, height);
			
			dialog.setMinimumSize(d);
			dialog.setPreferredSize(d);
			dialog.getContentPane().add(msgPanel);
		}
		
		msgLabel1.setText(msg[0]);
		msgLabel2.setText(msg[1]);
		msgLabel3.setText(msg[2]);
		msgLabel4.setText(msg[3]);

		UserInterfaceUtil.setLabelLNF(msgLabel1);
		UserInterfaceUtil.setLabelLNF(msgLabel2);
		UserInterfaceUtil.setLabelLNF(msgLabel3);
		UserInterfaceUtil.setLabelLNF(msgLabel4);
		
		JPanel panel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView();
		
		if(SS3Singleton.getMediaToolBar(null).getSelectedField() instanceof PresentationField) {
			panel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getDatabaseView();
		}
		
		dialog.setLocation(250, 250);
		dialog.setLocationRelativeTo(panel);
		dialog.setVisible(true);
	}

	private JPopupMenu saveCollectionPopup = null;
	private JLabel cMsgLabel1 = new JLabel();
	private JLabel cMsgLabel2 = new JLabel();
	private JLabel cMsgLabel3 = new JLabel();
	private JLabel cMsgLabel4 = new JLabel();
	
	public void showSaveCollectionErrorMessage(String[] msg) {	
		if(msg == null) {
			return;
		}

		if (saveCollectionPopup == null) {
			
			String str1 = "Use any character for a collection name, except for the following:";
			String str2 = "- reserved characters: < > : \" / \\| ? *";
			String str3 = "- integer value zero";
			String str4 = "- Do not use the following reserved device names:";
			String str5 = "CON, PRN, AUX, NUL, COM1, COM2, COM3, COM4, COM5,";
			String str6 = "COM6, COM7, COM8, COM9, LPT1, LPT2, LPT3, LPT4,";
			String str7 = "LPT5, LPT6, LPT7, LPT8, and LPT9";
			
			cMsgLabel1.setText("Warning: The gloss chain cannot extend beyond");
			cMsgLabel2.setText("the start or end time of the utterance.");
			cMsgLabel3.setText("You must adjust the boundary of the utterance");
			cMsgLabel4.setText("and then try again.");

			UserInterfaceUtil.setLabelLNF(cMsgLabel1);
			UserInterfaceUtil.setLabelLNF(cMsgLabel2);
			UserInterfaceUtil.setLabelLNF(cMsgLabel3);
			UserInterfaceUtil.setLabelLNF(cMsgLabel4);
			
			JLabel l1 = new JLabel(str1);
			JLabel l2 = new JLabel(str2);
			JLabel l3 = new JLabel(str3);
			JLabel l4 = new JLabel(str4);
			JLabel l5 = new JLabel(str5);
			JLabel l6 = new JLabel(str6);
			JLabel l7 = new JLabel(str7);
			
			UserInterfaceUtil.setLabelLNF(l1);
			UserInterfaceUtil.setLabelLNF(l2);
			UserInterfaceUtil.setLabelLNF(l3);
			UserInterfaceUtil.setLabelLNF(l4);
			UserInterfaceUtil.setLabelLNF(l5);
			UserInterfaceUtil.setLabelLNF(l6);
			UserInterfaceUtil.setLabelLNF(l7);

			saveCollectionPopup = new JPopupMenu();

			final JButton okBtn = new JButton("OK");

			JPanel msgPanel = new JPanel();
			GridBagConstraints gbc = new GridBagConstraints();

			msgPanel.setLayout(new GridBagLayout());
			gbc.anchor = GridBagConstraints.WEST;

			gbc.gridwidth = 3;
			gbc.gridx = 0;
			gbc.gridy = 0;
			msgPanel.add(cMsgLabel1, gbc);

			gbc.gridy = 1;
			gbc.gridx = 0;
			msgPanel.add(cMsgLabel2, gbc);

			gbc.gridy = 2;
			gbc.gridx = 0;
			msgPanel.add(cMsgLabel3, gbc);

			gbc.gridy = 3;
			gbc.gridx = 0;
			msgPanel.add(cMsgLabel4, gbc);

			gbc.gridy = 4;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);
			
			gbc.gridy = 5;
			gbc.gridx = 0;
			msgPanel.add(l1, gbc);

			gbc.gridy = 6;
			gbc.gridx = 0;
			msgPanel.add(l2, gbc);

			gbc.gridy = 7;
			gbc.gridx = 0;
			msgPanel.add(l3, gbc);

			gbc.gridy = 8;
			gbc.gridx = 0;
			msgPanel.add(l4, gbc);

			gbc.gridy = 9;
			gbc.gridx = 0;
			msgPanel.add(l5, gbc);

			gbc.gridy = 10;
			gbc.gridx = 0;
			msgPanel.add(l6, gbc);

			gbc.gridy = 11;
			gbc.gridx = 0;
			msgPanel.add(l7, gbc);

			gbc.gridwidth = 1;
			gbc.anchor = GridBagConstraints.EAST;
			gbc.gridy = 12;
			gbc.gridx = 0;
			msgPanel.add(new JLabel(" "), gbc);
			gbc.gridx = 2;
			msgPanel.add(okBtn, gbc);

			okBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					saveCollectionPopup.setVisible(false);
				}
			});

			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					okBtn.requestFocus();
				}
			});

			Dimension d = new Dimension();
			int width = msgPanel.getPreferredSize().width + 30;
			int height = msgPanel.getPreferredSize().height + 20;

			d.setSize(width, height);

			saveCollectionPopup.add(msgPanel);
			saveCollectionPopup.setPopupSize(d);
		}
		
		cMsgLabel1.setText(msg[0]);
		cMsgLabel2.setText(msg[1]);
		cMsgLabel3.setText(msg[2]);
		cMsgLabel4.setText(msg[3]);

		UserInterfaceUtil.setLabelLNF(msgLabel1);
		UserInterfaceUtil.setLabelLNF(msgLabel2);
		UserInterfaceUtil.setLabelLNF(msgLabel3);
		UserInterfaceUtil.setLabelLNF(msgLabel4);
		
		JPanel panel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getUtteranceView();
		// popup.show(field.getSignStreamSegmentPanel().getUtteranceView(), 250, 250);
		if(panel != null) {
			panel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getDatabaseView();
		}
		saveCollectionPopup.show(panel, 250, 250);
		saveCollectionPopup.setVisible(true);
	}
}
