package edu.bu.signstream.common.util.vo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.awt.Color;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;

public class CodingSchemaField {
	private int fieldId;
	private CodingSchemaFieldType fieldType;
	private int legacyOnsetID = -10, legacyOffsetID = -10;
	private String name, label, constraint, timeAlignment, prefix;
	private Color color = Color.BLACK;
	private ArrayList csValues = new ArrayList();
	private ArrayList defCsValues = new ArrayList();

	public CodingSchemaField(
		String fieldId,
		String name,
		String label,
		String constraint,
		String timeAlignment,
		String prefix,
		String color,
		ArrayList csValues) {
		this.fieldId = Integer.parseInt(fieldId);
		this.name = name;
		this.label = label;
		this.constraint = constraint;
		this.timeAlignment = timeAlignment;
		this.prefix = prefix;
		this.color = Util.hexString2Color(color);
		this.csValues = csValues;
		setDefaultCodingSchemeValues();
	}

	public void setCodingSchemaFieldType(CodingSchemaFieldType fieldType) {
		this.fieldType = fieldType;
	}

	public CodingSchemaFieldType getCodingSchemaFieldType() {
		return fieldType;
	}

	public HashMap getCodingSchemeValuesAsHashMap() {
		HashMap csValuesHm = new HashMap();

		Iterator dfIt = defCsValues.iterator();
		while (dfIt.hasNext()) {
			CodingSchemaValue csValue = (CodingSchemaValue) dfIt.next();
			csValuesHm.put("" + csValue.getValueId(), csValue);
		}

		Iterator csIt = csValues.iterator();
		while (csIt.hasNext()) {
			CodingSchemaValue csValue = (CodingSchemaValue) csIt.next();
			csValuesHm.put("" + csValue.getValueId(), csValue);
		}

		return csValuesHm;
	}

	public CodingSchemaField() {
		setDefaultCodingSchemeValues();
	}

	private void setDefaultCodingSchemeValues() {
		CodingSchemaValue holdVal = new CodingSchemaValue();
		holdVal.setValueId(Constants.HOLD_CODE);
		holdVal.setName(Constants.HOLD_VAL);
		defCsValues.add(holdVal);

		CodingSchemaValue onsetVal = new CodingSchemaValue();
		onsetVal.setValueId(Constants.ONSET_CODE);
		onsetVal.setName(Constants.ONSET_VAL);
		defCsValues.add(onsetVal);

		CodingSchemaValue offsetVal = new CodingSchemaValue();
		offsetVal.setValueId(Constants.OFFSET_CODE);
		offsetVal.setName(Constants.OFFSET_VAL);
		defCsValues.add(offsetVal);
	}

	public void setLegacyOffsetID(int legacyOffsetID) {
		this.legacyOffsetID = legacyOffsetID;
	}

	public int getLegacyOffsetID() {
		return legacyOffsetID;
	}

	public void setLegacyOnsetID(int legacyOnsetID) {
		this.legacyOnsetID = legacyOnsetID;
	}

	public int getLegacyOnsetID() {
		return legacyOnsetID;
	}

	public ArrayList getDefaultCodingSchemeValues() {
		return defCsValues;
	}

	@Override
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("CodingSchemeValue:");
		sb.append("\n fieldId = ");
		sb.append(fieldId);
		sb.append("\n name = ");
		sb.append(name);
		sb.append("\n label = ");
		sb.append(label);
		sb.append("\n color = ");
		sb.append(color);

		Iterator it = csValues.iterator();
		while (it.hasNext()) {
			CodingSchemaValue value = (CodingSchemaValue) it.next();
			sb.append(value.toString());
		}

		return sb.toString();
	}

	public String getPrefix() {
		return prefix;
	}

	public void setPrefix(String prefix) {
		this.prefix = prefix;
	}

	public Color getColor() {
		return color;
	}

	public String getHexColor() {
		if (!(color.equals(java.awt.Color.black))) {
			return Util.color2HexString(color);
		}
		return null;
	}

	public int getFieldId() {
		return fieldId;
	}

	public String getLabel() {
		return label;
	}

	public String getName() {
		return name;
	}

	public ArrayList getCodingSchemeValues() {
		return csValues;
	}

	public ArrayList getCodingSchemeExcludingDefaultValues() {
		ArrayList defCodes = Constants.getReservedCodingSchemaCodes();
		ArrayList list = new ArrayList();

		for (int i = 0; i < csValues.size(); i++) {
			CodingSchemaValue value = (CodingSchemaValue) csValues.get(i);

			if (!defCodes.contains("" + value.getValueId()))
				list.add(value);
		}
		return list;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setFieldId(int i) {
		fieldId = i;
	}

	public void setLabel(String string) {
		label = string;
	}

	public void setName(String string) {
		name = string;
	}

	public void setCodingSchemeValues(ArrayList list) {
		csValues = list;
	}

	public void setTimeAlignment(boolean timeAlgnm) {
		timeAlignment = timeAlgnm ? "true" : "false";
	}

	public void addCodingSchemeValues(CodingSchemaValue value) {
		csValues.add(value);
	}

	public void addCodingSchemaValue(int valueID, String value, String label) {
		CodingSchemaValue csValue = new CodingSchemaValue();
		csValue.setValueId(valueID);
		csValue.setName(value);
		csValue.setLabel(label);
		addCodingSchemeValues(csValue);
	}

	public int getNextAvailableValueID() {
		int valueId = 0;

		while (true) {
			if (!isValueIDExist(valueId))
				return valueId;
			valueId++;
		}
	}

	private boolean isValueIDExist(int valueId) {
		for (int i = 0; i < csValues.size(); i++) {
			CodingSchemaValue value = (CodingSchemaValue) csValues.get(i);
			if (valueId == value.getValueId())
				return true;
		}
		return false;
	}

	public String getConstraint() {
		return constraint;
	}

	public String getTimeAlignment() {
		return timeAlignment;
	}

	public void setConstraint(String string) {
		constraint = string;
	}

	public void setTimeAlignment(String string) {
		timeAlignment = string;
	}
}
