package edu.bu.signstream.common.util.vo.ss3.codingScheme;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplates;

import java.awt.*;
import java.util.*;

public class SS3CodingScheme {
	private String ID = null;
	private String superCodingSchemeID = null;

	private String ssVersion;
	private String dbVersion;

	private ArrayList cSchemeRef = new ArrayList();
	private ArrayList prtRef = new ArrayList();
	private ArrayList authors = new ArrayList();
	private HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
	private CSHandShapeTemplates handShapeTemplates = null;

	private ArrayList defaultSegment = new ArrayList();

	private ValueCollections vCollections;
	private Color color = Color.GRAY;

	public static char DOMINANT = 'D';
	public static char NON_DOMINANT = 'N';
	public static char FINGERSPELLING = 'L';

	private HashMap fTypes = new HashMap();
	private HashMap imgs = new HashMap();

	private CSUtilityInformation utilityInfo = new CSUtilityInformation();

	private ArrayList<FieldsGroup> fieldsGroup = new ArrayList<FieldsGroup>();

	public FieldsGroup getDependentNonmanualFieldsGroup(String fieldID) {
		for (int index = 0; index < fieldsGroup.size(); index++) {
			FieldsGroup group = fieldsGroup.get(index);
			if (fieldID.equalsIgnoreCase(group.getID()))
				return group;
		}
		return null;
	}

	public ArrayList<FieldsGroup> loadFieldsGroups() {
		fieldsGroup.clear();

		FieldsGroup headGroup = new FieldsGroup("Head", 0);
		FieldsGroup faceGroup = new FieldsGroup("Face", 1);
		FieldsGroup bodyGroup = new FieldsGroup("Body", 2);
		FieldsGroup grmGroup = new FieldsGroup("Grammatical", 3);

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			SS3Field field = fields.get(key);
			String group = field.getGroup();

			if (group != null && group.trim().length() >= 1) {
				if (group.equalsIgnoreCase("FACE")) {
					faceGroup.addFieldID(field.getId());
				} else if (group.equalsIgnoreCase("HEAD")) {
					headGroup.addFieldID(field.getId());
				} else if (group.equalsIgnoreCase("BODY")) {
					bodyGroup.addFieldID(field.getId());
				} else if (group.equalsIgnoreCase("GRAMMATICAL")) {
					grmGroup.addFieldID(field.getId());
				}
			}
		}

		fieldsGroup.add(headGroup);
		fieldsGroup.add(faceGroup);
		fieldsGroup.add(bodyGroup);
		fieldsGroup.add(grmGroup);
		return fieldsGroup;
	}

	public FieldsGroup loadManualFieldGroup() {

		ArrayList<String> fieldIDs = new ArrayList<String>();
		for (SS3Field field : fields.values()) {
			if (field.getGroup().equalsIgnoreCase("MANUAL")) {
				fieldIDs.add(field.getId());
			}
		}
		Collections.sort(fieldIDs, new Comparator<String>() {
			@Override
			public int compare(String o1, String o2) {
				return Integer.parseInt(o1) - Integer.parseInt(o2);
			}
		});

		FieldsGroup manualGroup = new FieldsGroup("Manual", 0);
		for (String fieldID : fieldIDs) {
			manualGroup.addFieldID(fieldID);
		}

		return manualGroup;
	}
	
	public FieldsGroup loadGphFieldGroup() {
		FieldsGroup graphGroup = new FieldsGroup("Graph", 0);

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			SS3Field field = fields.get(key);
			String group = field.getGroup();

			if (group != null && group.trim().length() >= 1) {
				if (group.equalsIgnoreCase(Constants.TYPE_GRAPH)) {
					graphGroup.addFieldID(field.getId());
				}
			}
		}

		return graphGroup;
	}

	// clones only field objects. Use for importing legacy coding scheme only!
	public SS3CodingScheme shallowCloneCodingScheme() {
		SS3CodingScheme cs = new SS3CodingScheme();
		cs.setID("" + Util.getUniqueNumber());
		cs.setSuperCodingSchemeID(getID());

		Set set = fields.keySet();
		Iterator it = set.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object object = fields.get(key);
			SS3Field field = (SS3Field) object;
			SS3Field nField = field.cloneField();
			cs.addField(nField);
		}

		return cs;
	}

	/***
	 * @items - list of CSHandShapeItem objects
	 * @returns list of SS3FieldValue objects
	 */
	public ArrayList<SS3FieldValue> getCorrespondingFieldValues(
			ArrayList<CSHandShapeItem> items, String fieldType) {

		String fieldID = getFieldID(fieldType);
		SS3Field field = getField(fieldID);

		HashMap<String, SS3FieldValue> values = field.getValues();
		ArrayList<SS3FieldValue> selectedValues = new ArrayList<SS3FieldValue>();

		for (CSHandShapeItem item : items) {
			String itemID = item.getId();

			Set<String> set = values.keySet();
			Iterator<String> it = set.iterator();

			while (it.hasNext()) {
				Object key = it.next();
				SS3FieldValue value = values.get(key);

				if (itemID.equalsIgnoreCase(value.getIid())) {
					value.setCSHandShapeItem(item);
					selectedValues.add(value);
				}
			}
		}

		return selectedValues;
	}

	// items - an collection of HandShapeImgItem objects
	public ArrayList getCorrespondingImages(ArrayList items) {
		ArrayList images = new ArrayList();

		for (int i = 0; i < items.size(); i++) {
			HandShapeImgItem item = (HandShapeImgItem) items.get(i);
			SS3FieldTypeImg image = getImage(item.getId());
			images.add(image);
		}

		return images;
	}

	public String getSignStreamVersion() {
		return ssVersion;
	}

	public void setSignStreamVersion(String ssVersion) {
		this.ssVersion = ssVersion;
	}

	public String getDbVersion() {
		return dbVersion;
	}

	public void setDbVersion(String dbVersion) {
		this.dbVersion = dbVersion;
	}

	public void setValueCollections(ValueCollections vCollections) {
		this.vCollections = vCollections;
	}

	public ValueCollections getValueCollections() {
		return vCollections;
	}

	public ValueCollection getValueCollection(String cID) {
		return vCollections.getValueCollection(cID);
	}

	public SS3FieldValue getValueCollectionSS3FieldValue(String cID, String vID) {
		ValueCollection vCol = vCollections.getValueCollection(cID);
		return vCol.getValue(vID);
	}

	public HashMap<String, SS3Field> getDependentRootFields() {
		HashMap<String, SS3Field> ht = new HashMap<String, SS3Field>();
		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			SS3FieldType fType = getFieldType(type);
			String indType = "D";

			if (fType != null && indType.equalsIgnoreCase(fType.getDepType()))
				ht.put(field.getId(), field);
		}
		return ht;
	}

	public HashMap getIndependentFields() {
		HashMap ht = new HashMap();
		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			SS3FieldType fType = getFieldType(type);
			String indType = "I";

			if (fType != null && indType.equalsIgnoreCase(fType.getDepType()))
				ht.put(field.getId(), field);
		}
		return ht;
	}

	public ArrayList<SS3Field> getFields(String dType) {
		ArrayList<SS3Field> list = new ArrayList<SS3Field>();

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			if (type.equalsIgnoreCase(dType))
				list.add(field);
		}
		return list;
	}

	public ArrayList<SS3Field> getFields(String... dTypes) {
		ArrayList<SS3Field> results = new ArrayList<SS3Field>();
		for (String dType : dTypes) {
			results.addAll(getFields(dType));
		}
		return results;
	}

	public String getLeftHandFieldID() {
		return getFieldID(Constants.TYPE_NDOM_HAND);
	}

	public String getRightHandFieldID() {
		return getFieldID(Constants.TYPE_DOM_HAND);
	}

	public String getLeftHandShapeFieldID() {
		return getFieldID(Constants.TYPE_NDOM_HAND_SHAPE);
	}

	public String getRightHandShapeFieldID() {
		return getFieldID(Constants.TYPE_DOM_HAND_SHAPE);
	}

	public String getFieldID(String cType) {
		Set keys = fields.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext()) {
			String key = (String) it.next();
			Object obj = fields.get(key);

			SS3Field field = (SS3Field) obj;
			String type = field.getType();

			if (type.equalsIgnoreCase(cType)) {
				return field.getId();
			}
		}
		return null;
	}

	public ArrayList<SS3Field> getDependentFields(String fieldID) {
		ArrayList<SS3Field> list = new ArrayList<SS3Field>();

		SS3Field field = getField(fieldID);
		String type = field.getType();

		SS3FieldType fType = (SS3FieldType) fTypes.get(type);
		ArrayList<SS3FieldType> types = fType.containsFields();

		Set<String> keys = fields.keySet();
		Iterator<String> it = keys.iterator();
		while (it.hasNext()) {
			String key = it.next();
			Object obj = fields.get(key);

			SS3Field fld = (SS3Field) obj;
			String t = fld.getType();

			for (int j = 0; j < types.size(); j++) {
				SS3FieldType ft = types.get(j);
				String tp = ft.getType();

				if (tp.equalsIgnoreCase(t))
					list.add(fld);
			}
		}
		return list;
	}

	public void addImage(SS3FieldTypeImg img) {
		imgs.put(img.getId(), img);
	}

	public SS3FieldTypeImg getImage(String id) {
		return (SS3FieldTypeImg) imgs.get(id);
	}

	public SS3Field getField(String fieldID) {
		return fields.get(fieldID);
	}
	
	public SS3FieldValue getFieldsMapSS3FieldValue(String fieldID, String valueID) {
		SS3Field field = getField(fieldID);
		return field.getFieldValue(valueID); 
	}

	public SS3FieldType getFieldType(String id) {
		return (SS3FieldType) fTypes.get(id);
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(String clr) {
		this.color = Util.hexString2Color(clr);
	}

	public ArrayList getFieldImg(String fieldID) {
		SS3Field field = getField(fieldID);
		return field.getValuesAsList();
	}

	public void addFieldType(SS3FieldType type) {
		fTypes.put(type.getType(), type);
	}

	public void addField(SS3Field field) {
		fields.put(field.getId(), field);
	}

	public void removeField(SS3Field field) {
		fields.remove(field.getId());
	}

	public ArrayList getAuthors() {
		return authors;
	}

	public void setAuthors(ArrayList list) {
		authors = list;
	}

	public HashMap<String, SS3Field> getFields() {
		return fields;
	}

	public void setFields(HashMap map) {
		fields = map;
	}

	public ArrayList getParticipantRef() {
		return prtRef;
	}

	public void setParticipantRef(ArrayList list) {
		prtRef = list;
	}

	public ArrayList getCodingSchemeRef() {
		return cSchemeRef;
	}

	public void setCodingSchemeRef(ArrayList list) {
		cSchemeRef = list;
	}

	@Override
	public String toString() {
		String name = utilityInfo.getCodingSchemeName();
		if (name == null)
			name = "Coding Schema";

		return name;
	}

	public ArrayList getDefaultSegment() {
		return defaultSegment;
	}

	public void setDefaultSegment(ArrayList list) {
		defaultSegment = list;
	}

	public void addFieldIDToDfltSegment(String fieldID) {
		defaultSegment.add(fieldID);
	}

	public CSUtilityInformation getCSUtilityInformation() {
		return utilityInfo;
	}

	public String getID() {
		return ID;
	}

	public void setID(String id) {
		ID = id;
	}

	public String getSuperCodingSchemeID() {
		return superCodingSchemeID;
	}

	public void setSuperCodingSchemeID(String superCodingSchemeID) {
		this.superCodingSchemeID = superCodingSchemeID;
	}

	/**
	 * @return
	 */
	public CSHandShapeTemplates getHandShapeTemplates() {
		return handShapeTemplates;
	}

	/**
	 * @param templates
	 */
	public void setHandShapeTemplates(CSHandShapeTemplates templates) {
		handShapeTemplates = templates;
	}
}
