package edu.bu.signstream.common.util.vo.ss3.codingScheme;

import java.util.*;
import java.awt.Color;
import java.io.Serializable;
import edu.bu.signstream.common.util.Util;

public class SS3Field implements Serializable, Comparable<Object> {

	private static final long serialVersionUID = 1L;
	
	private String id;
	private String name;
	private String label;
	private String prefix;
	private Color color;
	private String type;
	private Alignment alignment = Alignment.CENTER;


	public enum Alignment {
		RIGHT("R"),
		LEFT("L"),
		CENTER("C"),
		RIGHT_LEFT("RL");

		private String label;

		Alignment(String label) {
			this.label = label;
		}

		public String getLabel() {
			return label;
		}
	}
	
	private String group = "";

	private String comment = "";

	private String legacyID;

	private HashMap<String, SS3FieldValue> values = new HashMap<String, SS3FieldValue>();
	
	private int groupID = -1;
	
	public SS3Field() {}

	public int getGroupID() {
		return groupID;
	}

	public void setGroupID(int groupID) {
		this.groupID = groupID;
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public SS3Field cloneField() {
		SS3Field field = new SS3Field();
		field.setId(getId());
		field.setName(getName());
		field.setLabel(getLabel());
		field.setPrefix(getPrefix());
		field.setColor(getColor());
		field.setType(getType());
		field.setAlignment(getAlignment());

		ArrayList<SS3FieldValue> lValues = getValuesAsList();

		for (int i = 0; i < lValues.size(); i++) {
			field.addValue(lValues.get(i).cloneFieldValue());
		}

		return field;
	}	

	public void setLegacyID(String legacyID) {
		this.legacyID = legacyID;
	}

	public String getLegacyID() {
		return legacyID;
	}

	public void addValue(SS3FieldValue value) {
		values.put(value.getId(), value);
	}

	public void remove(SS3FieldValue value) {
		values.remove(value.getId());
	}

	public void setAlignment(Alignment alignment) {
		this.alignment = alignment;
	}

	public Alignment getAlignment() {
		return alignment;
	}

	public Color getColor() {
		return color;
	}

	public void setColor(String color) {
		this.color = Util.hexString2Color(color);
	}

	public String getId() {
		return id;
	}

	public String getLabel() {
		return label;
	}

	// returns label if available, othervise - name
	public String getLabelOrName() {
		if (label != null && label.length() > 0)
			return label;
		return name;
	}

	public String getName() {
		return name;
	}

	public String getPrefix() {
		return prefix;
	}

	public String getType() {
		return type;
	}

	public SS3FieldValue getFieldValue(String valueID) {
		return values.get(valueID);
	}

	public HashMap<String, SS3FieldValue> getValues() {
		return values;
	}

	@Override
	public int compareTo(Object obj) {
		SS3Field tmp = (SS3Field) obj;
		// data is the value you want to compare
		return this.getLabel().toLowerCase().compareTo(
				tmp.getLabel().toLowerCase());
	}

	public ArrayList<SS3FieldValue> getValuesAsList() {
		Set<String> set = values.keySet();
		Iterator<String> it = set.iterator();
		ArrayList<SS3FieldValue> list = new ArrayList<SS3FieldValue>();

		while (it.hasNext()) {
			SS3FieldValue value = values.get(it.next());
			list.add(value);
		}
		return list;
	}

	public void setColor(Color color) {
		this.color = color;
	}

	public void setId(String string) {
		id = string;
	}

	public void setLabel(String string) {
		label = string;
	}

	public void setName(String string) {
		name = string;
	}

	public void setPrefix(String string) {
		prefix = string;
	}

	public void setType(String string) {
		type = string;
	}

	public void setValues(ArrayList<SS3FieldValue> list) {
		for (int i = 0; i < list.size(); i++) {
			addValue(list.get(i));
		}
	}
	
	public void removeAllValues() {
		values.clear();
	}

	@Override
	public String toString() {
		return name + " <" + label + ">" + comment;
	}
}
