package edu.bu.signstream.common.util.vo.ss3.db;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.EventsEntity;

public class SS3GlossWindowSegment {

	private String ID = "";
	private String label = "";
	private boolean isDefault = false;
	private SS3Participant participant = null;

	private ArrayList<SS3GlossWindowItem> glossWindowItems = new ArrayList<SS3GlossWindowItem>();

	private SS3GlossWindowItemNote note = new SS3GlossWindowItemNote();

	public String getID() {
		return ID;
	}

	public void setID(String id) {
		ID = id;
	}

	public String getLabel() {
		return label;
	}

	public void setLabel(String label) {
		this.label = label;
		
		for (SS3GlossWindowItem item: glossWindowItems) {
			item.setSegmentLabel(label);
		}
	}
	
	public boolean isDefault() {
		return isDefault;
	}
	
	/**
	 * Sets this segment tier to be default. It is using access level protected
	 * because it is intended only to be called by SS3TemporalPartition, 
	 * as part of a sequence in which all segment tiers are set simultaneously
	 * to ensure that one and only one are set to default.
	 * @param isDefault setting to either true for default or false for not default
	 */
	protected void setDefault(boolean isDefault) {
		this.isDefault = isDefault;
	}
	
	public SS3Participant getParticipant() {
		return participant;
	}
	
	public void setParticipant(SS3Participant participant) {
		this.participant = participant;
	}

	public boolean isDominantHandRight() {
		if (participant == null) return true;
		return participant.isDominantHandRight();
	}
	
	public String getParticipantLabel() {		
		return participant != null ? participant.getLabel() : "";
	}
	
	public void delete(EventsEntity entity) {
		int startTime = entity.getStartTimeInfo().getMovieTime();
		int endTime = entity.getEndTimeInfo().getMovieTime();
		
		ArrayList<SS3GlossWindowItem> itemsToRemove = new ArrayList<SS3GlossWindowItem>();
		
		for(SS3GlossWindowItem item : glossWindowItems) {
			int sMovieTime = item.getStartTimeInfo().getMovieTime();
			int eMovieTime = item.getEndTimeInfo().getMovieTime();
			
			if(sMovieTime == startTime && eMovieTime == endTime) {
				itemsToRemove.add(item);				
			}
		}
				
		for(SS3GlossWindowItem itemToRemove : itemsToRemove) {
			glossWindowItems.remove(itemToRemove);
		}
	}
	
	//public void addOrUpdateGlossWindowItem(SS3GlossWindowItem item) {
	//	item.getId();
	//}

	public void addSS3GlossWindowItem(SS3GlossWindowItem item) {
		// if item is added after this segment was initially created, the label may have changed
		item.setSegmentLabel(label);
		glossWindowItems.add(item);
		Collections.sort(glossWindowItems, new Comparator<SS3GlossWindowItem>() {
			@Override
			public int compare(SS3GlossWindowItem o1, SS3GlossWindowItem o2) {
				int startTime1 = o1.getStartTimeInfo().getMovieTime();
				int startTime2 = o2.getStartTimeInfo().getMovieTime();
				
				return startTime1 - startTime2;
			}
			
		});
	}

	public ArrayList<SS3GlossWindowItem> getSS3GlossWindowItems() {
		return glossWindowItems;
	}

	public void setItems(ArrayList<SS3GlossWindowItem> items) {
		this.glossWindowItems = items;
		Collections.sort(glossWindowItems, new Comparator<SS3GlossWindowItem>() {
			@Override
			public int compare(SS3GlossWindowItem o1, SS3GlossWindowItem o2) {
				int startTime1 = o1.getStartTimeInfo().getMovieTime();
				int startTime2 = o2.getStartTimeInfo().getMovieTime();
				
				return startTime1 - startTime2;
			}
			
		});
	}

	public SS3GlossWindowItem getSS3GlossWindowItem(String id) {
		for (SS3GlossWindowItem glossWindowItem : glossWindowItems) {
			if (glossWindowItem.getId().equalsIgnoreCase(id))
				return glossWindowItem;
		}
		return null;
	}
	
	@Override
	public String toString() {
		String ret = "SS3GlossWindowSegment: participantLabel =  " + getParticipantLabel() + ", label = " + getLabel();
		return ret;
	}
	
	public SS3GlossWindowItemNote getNote() {
		return note;
	}

	public void setNote(SS3GlossWindowItemNote note) {
		this.note = note;
	}

	@Override
	public boolean equals(Object obj) {
		return (obj instanceof SS3GlossWindowSegment) && ((SS3GlossWindowSegment) obj).ID.equals(ID);
	}
}
