package edu.bu.signstream.fileConvertor;

import java.io.File;
import java.awt.*;
import java.awt.event.*;
import java.util.logging.Level;
import javax.swing.*;
import edu.bu.signstream.common.util.vo.SignStreamDocument;
import edu.bu.signstream.common.db.parser.SS2Parser;
import edu.bu.signstream.common.db.XMLAdaptor;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SS2ToSS3Convertor extends JFrame implements ActionListener {
   private JButton openBtn = new JButton("Open");
   private JButton saveBtn = new JButton("Save");
   private JButton convertBtn = new JButton("Convert");
   private JLabel openLbl = new JLabel("File or directory to convert");
   private JLabel saveLbl = new JLabel("Directory to save converted files");

   private final int OPEN = 1;
   private final int SAVE = 2;
   private final int CONVERT = 3;

   private File fileToConvert = null;
   private File saveDir = null;

   public SS2ToSS3Convertor() {
      try {
         UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
      } catch (Exception ex) {
         ex.printStackTrace();
         SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
      }

      initUI();
   }

   private void initUI() {
      JPanel centerPane = createCenterPane();

      openBtn.setToolTipText(
         "Check this to select file or directory to convert");
      openBtn.setActionCommand("" + OPEN);

      saveBtn.setToolTipText(
         "Check this to select directory save converted files");
      saveBtn.setActionCommand("" + SAVE);

      convertBtn.setToolTipText("Check this to convert selected files");
      convertBtn.setActionCommand("" + CONVERT);

      openBtn.addActionListener(this);
      saveBtn.addActionListener(this);
      convertBtn.addActionListener(this);

      Container contentPane = getContentPane();
      contentPane.setLayout(new BorderLayout(10, 10));

      contentPane.add(centerPane, BorderLayout.CENTER);
   }

   private JPanel createCenterPane() {
      JPanel pane = new JPanel();

      GridBagLayout gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      pane.setLayout(gbl);

      JLabel title = new JLabel("SS2 files converter");
      title.setFont(new Font("Times-Roman", Font.BOLD + Font.ITALIC, 16));

      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      pane.add(title, gbc);

      gbc.anchor = GridBagConstraints.NORTH;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.insets = new Insets(0, 0, 10, 0);
      pane.add(new JSeparator(), gbc);

      gbc.anchor = GridBagConstraints.WEST;
      gbc.gridwidth = 1;
      gbc.insets = new Insets(10, 10, 10, 10);
      pane.add(openBtn, gbc);

      pane.add(Box.createHorizontalStrut(10));
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      pane.add(openLbl, gbc);

      gbc.gridwidth = 1;
      pane.add(saveBtn, gbc);

      pane.add(Box.createHorizontalStrut(10));
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      pane.add(saveLbl, gbc);

      ButtonPanel bPane = new ButtonPanel();
      bPane.add(convertBtn);

      gbc.gridwidth = 1;

      gbc.fill = GridBagConstraints.NONE;
      gbc.anchor = GridBagConstraints.SOUTH;
      gbc.insets = new Insets(15, 0, 0, 0);
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.gridwidth = 7;
      pane.add(bPane);

      return pane;
   }

   public static void main(String[] args) {
      SS2ToSS3Convertor convertor = new SS2ToSS3Convertor();
      convertor.setSize(500, 300);
      convertor.show();
      convertor.toFront();
   }

   @Override
public void actionPerformed(ActionEvent ae) {
      String str = ae.getActionCommand();

      switch (Integer.parseInt(str)) {
         case OPEN :
            JFileChooser chooser = new JFileChooser();
            chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
            //chooser.setMultiSelectionEnabled(true);
            int state = chooser.showOpenDialog(null); //.showOpenDialog(null);
            File file = chooser.getSelectedFile();

            if (file != null && state == JFileChooser.APPROVE_OPTION) {
               fileToConvert = file;
               openLbl.setText(file.getName());
            }
            break;
         case SAVE :
            JFileChooser sChooser = new JFileChooser();
            sChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

            int sstate = sChooser.showOpenDialog(null);
            File sfile = sChooser.getSelectedFile();
            if (sfile != null && sstate == JFileChooser.APPROVE_OPTION) {
               saveDir = sfile;
               saveLbl.setText(sfile.getName());
            }
            break;
         case CONVERT :
            String errorMsg = null;
            if (fileToConvert == null)
               errorMsg = "Please select file to convert";
            if (saveDir == null)
               errorMsg = "Please select directory to save converted files";
            if (errorMsg != null) {
            	SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", errorMsg);
            } else {
               XMLAdaptor adaptor = new XMLAdaptor();
               SS2Parser p = new SS2Parser();
               String msg = convertDirectory(fileToConvert, p, adaptor);   
               
              // JOptionPane.showConfirmDialog(null, msg, "Warning", JOptionPane.CLOSED_OPTION);
               
               // SS3Singleton.getManager().getSignStreamApplication().confirmationDialog("Warning", msg);
               break;
            }
      }
   }

   private String convertDirectory(
      File dFile,
      SS2Parser p,
      XMLAdaptor adaptor) {
      StringBuffer sb = new StringBuffer();

      if (dFile.isDirectory()) {
         File[] files = dFile.listFiles();
         for (int i = 0; i < files.length; i++) {
            sb.append(convertDirectory(files[i], p, adaptor));
         }
      } else if (dFile.isFile()) {
         sb.append(convertFile(dFile, p, adaptor));
      }
      return sb.toString();
   }

   private String convertFile(File file, SS2Parser p, XMLAdaptor adaptor) {
      try {
         SignStreamDocument doc = p.parseDBFileData(file);

         String dv = System.getProperty("file.separator", "/");
         adaptor.saveDocument(
            doc,
            saveDir.getPath() + dv + file.getName() + ".xml");
      } catch (Exception ex) {
    	 ex.printStackTrace();
         SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
         return "\n " + file.getName() + ": " + ex.getMessage();
      }

      return "\n " + file.getName() + ": converted and saved";
   }
}

class ButtonPanel extends JPanel {
   JPanel pane = new JPanel();

   public ButtonPanel() {
      pane.setLayout(new FlowLayout(FlowLayout.CENTER));
      setLayout(new BorderLayout(0, 5));
      //add(new JSeparator(), BorderLayout.NORTH);
      add(pane, BorderLayout.CENTER);
   }

   public void add(JButton button) {
      pane.add(button);
   }
}
