package edu.bu.signstream.grepresentation.fields;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public abstract class Field extends JPanel {
	
	private static final long serialVersionUID = 1L;
	protected double Y;
	protected Track track; // String fieldID;
	protected SS3Track ss3Track;
	protected Color color = null;
	protected SignStreamSegmentPanel segmentPanel;
	protected PresentationField parentField = null;
	
	protected String segmentID = "";
	
	public abstract ArrayList getEntities();
	
	public ArrayList<EventsEntity> getEntitiesBetween(int startMovieTime, int endMovieTime, boolean exclusive) {
		ArrayList entities = getEntities();	// however this Field-inherited class returns all entities
		ArrayList<EventsEntity> entitiesBetween = new ArrayList<EventsEntity>();

		for (Object obj: entities) {
			EventsEntity entity = (EventsEntity) obj;
			
			if (entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null)
				continue;
			
			// if either startMovieTime or endMovieTime fall within the bounds of this entity, add it 
			if ( !exclusive) {
				int startMovieCoord = segmentPanel.getZoomer()
						.getCoordinateTimeConvertor().convertTimeToCoordinate(startMovieTime);
				int endMovieCoord = segmentPanel
						.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(endMovieTime);
				if (entity.contains(startMovieCoord) || entity.contains(endMovieCoord)) {
					entitiesBetween.add(entity);
					continue;
				}
			}
			
			int entStartTime = entity.getStartTimeInfo().getMovieTime();
			int entEndTime = entity.getEndTimeInfo().getMovieTime();
			
			if (startMovieTime <= entStartTime && entEndTime <= endMovieTime) {
				entitiesBetween.add(entity);
			}
		}
		
		return entitiesBetween;
	}

	public abstract EventsEntity getEntityWithRefID(String refEntityID);

	public abstract EventsEntity getEntity(String entityID);

	public abstract void highlight(boolean highlight, String refID);

	public abstract Event getEventAt(int x);

	public abstract EventsEntity getEntityAt(int x);

	public abstract void deleteEntity(EventsEntity entity);

	public abstract int getFieldSize();

	public abstract void deleteEmptyEntities();

	public abstract boolean isEmptyInterval(int startTime, int endTime);

	public abstract void addEventsEntity(EventsEntity entity);

	public abstract int getStartTimeCoordinate();

	public abstract int getEndTimeCoordinate();

	public abstract void unselect();

	public abstract Event getSelectedEvent();

	public abstract EventsEntity getSelectedEntity();
	
	public Field(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.parentField = parentField;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}
	
	public void setSegmentID(String segmentID) {
		this.segmentID = segmentID;
	}

	public String getSegmentID() {
		return segmentID;
	}

	public String getFieldID() {
		if (track != null)
			return track.getFieldId(); // fieldID;
		if (ss3Track != null)
			return ss3Track.getId();
		return null;
	}

	// update creation && update time here
	public SS3Track getSS3Track() {
		if (ss3Track != null) 
			ss3Track.removeEntities();
		
		ArrayList<EventsEntity> entities = getEntities();

		for (EventsEntity entity : entities) {
			ss3Track.addEntity(entity.getSS3Entity());
		}

		return ss3Track;
	}

	@Override
	public int getY() {
		return (int) Y;
	}

	public Event selectEvent(int x) {
		Event event = getEventAt(x);
		if (event != null)
			event.select();
		return event;
	}

	public boolean contains(int y) {
		int y1 = (int) (Y - SegmentGraphReprUtil.axesTopHeight - 1);
		int y2 = (int) (Y + SegmentGraphReprUtil.valueHeight + 2);
		return y1 <= y && y <= y2;
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		this.Y = Y;
	}

	public int getStartTime() {
		return ss3Track.getStartTime();
	}

	public int getEndTime() {
		return ss3Track.getEndTime();
	}

	public Color getColor() {
		return color;
	}

	public void setColor(Color color) {
		this.color = color;
	}
	
	public PresentationField getParentField() {
		return parentField;
	}
	
	public void setParentField(PresentationField parentField) {
		this.parentField = parentField;
	}
}
