package edu.bu.signstream.grepresentation.fields;

import java.util.ArrayList;
import java.util.Iterator;

public class FieldWrappersCollection {
   private ArrayList<FieldWrapper> wrappers = new ArrayList<FieldWrapper>();

   public ArrayList<FieldWrapper> getWrappers() {
      return wrappers;
   }
   
   public void unselectAll() {
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         wrapper.unselectAll();
      }
   }
   
   public void removeWrapper(FieldWrapper wrapper) {
	   wrappers.remove(wrapper);
   }
   
   public void removeFieldWrapperWithFieldID(String fieldID) {
      FieldWrapper wrapper = getFieldWrapperWithFieldID(fieldID);
      wrappers.remove(wrapper);
   }
   
   public void moveField(int fromIndex, int toIndex) {
	   FieldWrapper o = wrappers.remove(fromIndex);
      wrappers.add(toIndex, o);
   }

   public boolean isFieldVisible(String fieldID) {
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         if (fieldID != null
            && wrapper.getField() != null
            && fieldID.equalsIgnoreCase(wrapper.getField().getFieldID()))
            return true;
      }
      return false;
   }

   public FieldWrapper getUpdatableFieldWrapper() {
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         if (wrapper.isUpdatable())
            return wrapper;
      }
      if (!wrappers.isEmpty())
         return wrappers.get(0);
      return null;
   }

   /**
    * Removes all of the wrappers from this collection.
    * */
   public void removeAll() {
      wrappers.clear();
   }

   public void repaint() {
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         wrapper.repaint();
      }
   }

   public FieldWrapper getFieldWrapper(String dType) {
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         String type = wrapper.getType();

         if (dType.equalsIgnoreCase(type))
            return wrapper;
      }
      return null;
   }

   public FieldWrapper getFieldWrapperWithFieldID(String fieldID) {
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         if (fieldID.equalsIgnoreCase(wrapper.getField().getFieldID()))
            return wrapper;
      }
      return null;
   }

   public ArrayList<FieldWrapper> getVisibleWrappers() {
      ArrayList<FieldWrapper> list = new ArrayList<FieldWrapper>();
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         if (wrapper.isVisible())
            list.add(wrapper);
      }
      return list;
   }

   public FieldWrapper getFieldWrapperAt(int Y) {
         
      for(FieldWrapper wrapper : wrappers) {   	     	  
    	  if (wrapper.isVisible() && wrapper.contains(Y))
            return wrapper;
      }
      return null;
   }

   public int getSize() {
      return wrappers.size();
   }

   public void addFieldWrapper(FieldWrapper wrapper) {
      wrappers.add(wrapper);
   }
   
   public void addFieldWrapperAfter(FieldWrapper wrapperToAdd, FieldWrapper wrapperToAddAfter) {
	   if(wrapperToAdd == null || wrapperToAddAfter == null)
		   return;
		   
	   for(int i = 0; i < wrappers.size(); i++) {
		   if(wrapperToAddAfter.equals(wrappers.get(i))) {
			   wrappers.add(i+1, wrapperToAdd);
		   }
	   }
   }

   public int getStartTime() {
      int startTime = -1;

      if (wrappers.size() > 0) {
         FieldWrapper wrpr = wrappers.get(0);
         startTime = wrpr.getStartTime();

         for (int i = 1; i < wrappers.size(); i++) {
            FieldWrapper wrapper = wrappers.get(i);
            if (wrapper != null) {
               int time = wrapper.getStartTime();
               startTime = startTime < time ? startTime : time;
            }
         }
      }
      return startTime;
   }

   public int getEndTime() {
      int endTime = -1;

      if (wrappers.size() > 0) {
         FieldWrapper wrpr = wrappers.get(0);
         endTime = wrpr.getEndTime();

         for (int i = 0; i < wrappers.size(); i++) {
            FieldWrapper wrapper = wrappers.get(i);
            if (wrapper != null) {
               int time = wrapper.getEndTime();
               endTime = endTime > time ? endTime : time;
            }
         }
      }
      return endTime;
   }

   public int getEndTimeCoordinate() {
      int endCoord = -1;

      if (wrappers.size() > 0) {
         FieldWrapper wrpr = wrappers.get(0);
         endCoord = wrpr.getEndTimeCoordinate();

         for (int i = 0; i < wrappers.size(); i++) {
            FieldWrapper wrapper = wrappers.get(i);
            int coord = wrapper.getEndTimeCoordinate();
            endCoord = endCoord > coord ? endCoord : coord;
         }
      }
      return endCoord;
   }

   public Event getSelectedEvent() {
		  Iterator<FieldWrapper> iter = wrappers.iterator();
		  while(iter.hasNext()) {
			  FieldWrapper wrapper = iter.next();
			  Event event = wrapper.getSelectedEvent();
			  if(event!=null) {
				  return event; 
			   }
		  }
		  return null;
   }

   public FieldWrapper getSelectedFieldWrapper() {
	  Iterator<FieldWrapper> iter = wrappers.iterator();
	  while(iter.hasNext()) {
		  FieldWrapper wrapper = iter.next();
		   if(wrapper.getSelectedEvent()!=null) {
			  return wrapper; 
		   }
	  }
      return null;
   }

   public EventsEntity getSelectedEntity() {
	   Iterator<FieldWrapper> iter = wrappers.iterator();
	   while(iter.hasNext()) {
		   EventsEntity entity = iter.next().getSelectedEntity();
         if (entity != null)
            return entity;
      }
      return null;
   }

   public void selectItemIfEntitySelected() {
	   Iterator<FieldWrapper> iter = wrappers.iterator();
	   while(iter.hasNext()) {
		   iter.next().selectItemIfEntitySelected();
       }
   }

   public void selectEntityIfItemSelected() {
	   Iterator<FieldWrapper> iter = wrappers.iterator();
	   while(iter.hasNext()) {
		   iter.next().selectEntityIfItemSelected();   
	   }
   }

//   public void setDependencies() {
//	   Iterator<FieldWrapper> iter = wrappers.iterator();
//	   while(iter.hasNext()) {
//		   iter.next().setDependencies();
//      }
//   }
}
