package edu.bu.signstream.grepresentation.fields;

import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class NonManualFieldOrdering {

    private static NonManualFieldOrdering defaultFieldOrder;

    private ArrayList<String> rawOrderList;

    public NonManualFieldOrdering(ArrayList<String> rawOrderList) {
        this.rawOrderList = new ArrayList<String>(rawOrderList);
    }

    public NonManualFieldOrdering() {
        this(new ArrayList<String>());
    }

    public static void setDefaultFieldOrder(NonManualFieldOrdering order) {
        defaultFieldOrder = order;
    }

    public static NonManualFieldOrdering getDefaultFieldOrder() {
        return defaultFieldOrder;
    }

    public static NonManualFieldOrdering getCorrectFieldOrder(SS3Database database) {
        if (database.getCustomNmFieldOrder() != null) {
            return database.getCustomNmFieldOrder();
        }
        return defaultFieldOrder;
    }

    public static void sortLabelObjectList(List<LabelObject> orderList, final NonManualFieldOrdering order) {
        Collections.sort(orderList, new Comparator<LabelObject>() {
            @Override
            public int compare(LabelObject o1, LabelObject o2) {
                int idx1 = order.getRawOrderList().indexOf(o1.getFieldID());
                if (idx1 == -1) {
                    idx1 = Integer.MAX_VALUE;
                }
                int idx2 = order.getRawOrderList().indexOf(o2.getFieldID());
                if (idx2 == -1) {
                    idx2 = Integer.MAX_VALUE;
                }
                return idx1 - idx2; // when sorting, compare two LabelObjects by their positions in the default ordering
            }
        });
    }

    public static void sortFieldWrapperList(List<FieldWrapper> orderList, final NonManualFieldOrdering order) {
        Collections.sort(orderList, new Comparator<FieldWrapper>() {
            @Override
            public int compare(FieldWrapper o1, FieldWrapper o2) {
                int idx1 = order.getRawOrderList().indexOf(o1.getField().getFieldID());
                if (idx1 == -1) {
                    idx1 = Integer.MAX_VALUE;
                }
                int idx2 = order.getRawOrderList().indexOf(o2.getField().getFieldID());
                if (idx2 == -1) {
                    idx2 = Integer.MAX_VALUE;
                }
                return idx1 - idx2; // when sorting, compare two LabelObjects by their positions in the default ordering
            }
        });
    }

    public ArrayList<String> getRawOrderList() {
        return new ArrayList<String>(rawOrderList);
    }

    public ArrayList<String> getOrderList() {
        if (this == defaultFieldOrder) {
            return getRawOrderList();
        }

        ArrayList<String> newOrderList = new ArrayList<String>(rawOrderList);
        for (String fieldID : getDefaultFieldOrder().getOrderList()) {
            if (!newOrderList.contains(fieldID)) {
                newOrderList.add(fieldID);
            }
        }
        return newOrderList;
    }

    public void add(String fieldID) {
        rawOrderList.add(fieldID);
    }

    public void add(int index, String fieldID) {
        rawOrderList.add(index, fieldID);
    }

    public void remove(String fieldID) {
        rawOrderList.remove(fieldID);
    }

    public boolean hasField(String fieldID) {
        return rawOrderList.contains(fieldID);
    }

    @Override
    public String toString() {
        ArrayList<String> labels = new ArrayList<String>();
        for (String fieldID : rawOrderList) {
            labels.add(SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme().getField(fieldID).getLabel());
        }
        return labels.toString();
    }
}
