package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class EnteredText extends JPanel {

	private static final long serialVersionUID = 1L;
	private SignStreamSegmentPanel segmentPanel;
	private GlossField glsField;

	private ArrayList<Text> enteredText = new ArrayList<Text>();

	private int startSelectionCoord = -1;
	private int endSelectionCoord = -1;
	private boolean selectedSegment = false;

	private FontMetrics fontMetrics;

	private String text = "";

	public EnteredText(SignStreamSegmentPanel segmentPanel, GlossField glsField) {
		this.segmentPanel = segmentPanel;
		this.glsField = glsField;

		fontMetrics = getFontMetrics(SegmentGraphReprUtil.getGraphReprFont());
	}

	public ArrayList<Text> getEnteredText() {
		return enteredText;
	}
	
	public ArrayList<Text> getEnteredTextBetween(int startTime, int endTime) {
		ArrayList<Text> enteredText = getEnteredText();
		ArrayList<Text> enteredTextBetween = new ArrayList<Text>();

		for (Text text: enteredText) {
			if (text.getStartTime() == null || text.getEndTime() == null)
				continue;
			
			int entStartTime = text.getStartTime().getMovieTime();
			int entEndTime = text.getEndTime().getMovieTime();
			
			if (startTime <= entStartTime && entEndTime <= endTime) {
				enteredTextBetween.add(text);
			}
		}
		
		return enteredTextBetween;
	}

	public void deleteText(Text textToDelete) {
		enteredText.remove(textToDelete);
	}

	public void addText(Text text) {
		enteredText.add(text);
	}

	public void setText(String text) {
		this.text = text;
	}
	
	public void removeText(ArrayList<Text> textToRemove) {
		for(Text txt : textToRemove) {
			enteredText.remove(txt);
		}
	}

	public Text getPrecedingText(int coord) {
		Text precedingText = null;

		for (Text entText : enteredText) {
			if (entText.getEndCoord() <= coord
					&& (precedingText == null || entText.getEndCoord() < precedingText
							.getEndCoord())) {
				precedingText = entText;
			}
		}
		return precedingText;
	}

	public Text getFollowingText(int coord) {
		Text followingText = null;

		for (Text entText : enteredText) {
			if (entText.getStartCoord() <= coord
					&& (followingText == null || entText.getEndCoord() < followingText
							.getStartCoord())) {
				followingText = entText;
			}
		}
		return followingText;
	}

	public ArrayList<Text> getSortedEnteredTextInInteval2(int startCoord, int endCoord) {
			ArrayList<Text> text = new ArrayList<Text>();
			for (Text entText : enteredText) {
				
				int txtStartCoord = entText.getStartCoord();
				int txtWidth = fontMetrics.stringWidth(entText.getText());
				int txtEndCoord = txtStartCoord + txtWidth;
				
				if (startCoord <= txtEndCoord && txtStartCoord < endCoord) {				
					text.add(entText);
				}
			}
			return text;
		}

	public ArrayList<Text> getSortedEnteredTextInInteval(int startCoord,
			int endCoord) {
		ArrayList<Text> text = new ArrayList<Text>();
		for (Text entText : enteredText) {
			if (startCoord <= entText.getStartCoord()) {
				if (endCoord == -1 || entText.getStartCoord() < endCoord) {
					text.add(entText);
				}
			}
		}
		return text;
	}

	public ArrayList<Text> getSelectedText() {

		ArrayList<Text> selectedText = new ArrayList<Text>();

		for (Text text : enteredText) {
			if (text.isSelected())
				selectedText.add(text);
			// return text;
		}
		return selectedText;
	}

	public String getSentence(ArrayList<Text> text) {
		StringBuffer sb = new StringBuffer();
		Iterator<Text> iterator = text.iterator();
		while (iterator.hasNext()) {
			sb.append(" ");
			Text txt = iterator.next();
			sb.append(txt.getText());
		}
		return sb.toString().trim();
	}

	public void synchronize(ArrayList<Text> enteredTextInInterval,
			int startCoord, int endCoord) {
		// remove text objects
		for (Text t : enteredTextInInterval) {
			enteredText.remove(t);
		}

		// create new objects
		StringTokenizer tk = new StringTokenizer(text);
		int tokenStartCoord = startCoord;
		boolean visible = true;

		boolean tokenStarted = false;
		String singleToken = null;

		while (tk.hasMoreTokens()) {
			String token = tk.nextToken();

			int charAt = token.lastIndexOf('"');

			if (charAt >= 0) {
				if (!tokenStarted) {
					singleToken = token;
					tokenStarted = true;
				} else {
					singleToken = singleToken + " " + token;
					tokenStarted = false;

					int width = fontMetrics.stringWidth(singleToken);
					int tokenEndCoord = tokenStartCoord + width;

					if (tokenEndCoord > endCoord) {
						visible = false;
					}

					Text text1 = new Text(singleToken, tokenStartCoord,
							tokenEndCoord);
					text1.setVisible(visible);

					enteredText.add(text1);

					tokenStartCoord = tokenEndCoord + 10;

					singleToken = null;
				}
			} else if (tokenStarted) {
				singleToken = singleToken + " " + token;
			} else {

				int width = fontMetrics.stringWidth(token);
				int tokenEndCoord = tokenStartCoord + width;

				if (tokenEndCoord > endCoord) {
					visible = false;
				}

				Text text1 = new Text(token, tokenStartCoord, tokenEndCoord);
				text1.setVisible(visible);

				enteredText.add(text1);

				tokenStartCoord = tokenEndCoord + 10;
			}
		}
		
		if (tokenStarted) {
			int width = fontMetrics.stringWidth(singleToken);
			int tokenEndCoord = tokenStartCoord + width;

			if (tokenEndCoord > endCoord) {
				visible = false;
			}

			Text text1 = new Text(singleToken, tokenStartCoord, tokenEndCoord);
			text1.setVisible(visible);
			
			enteredText.add(text1);

			tokenStartCoord = tokenEndCoord + 10;
		}

		text = "";
	}

	public void unselectAll() {
		for (Text text : enteredText) {
			text.setSelected(false);
		}
	}

	public GlossField getGlossField() {
		return glsField;
	}

	public Text getTextAt(int coord) {
		for (Text text : enteredText) {

			int startCoord = text.getStartCoord();
			int endCoord = text.getEndCoord();

			if (endCoord == startCoord && fontMetrics != null) {
				endCoord = startCoord + fontMetrics.stringWidth(text.getText());
			}
			
			if(startCoord == endCoord) {
				endCoord = startCoord + 10;
			}

			if (startCoord <= coord && endCoord >= coord) {
				return text;
			}
		}
		return null;
	}

	public void selectTextBetweenSelectedEvents() {

		int firstSelected = -1;
		int lastSelected = -1;

		for (int i = 0; i < enteredText.size(); i++) {
			Text text = enteredText.get(i);

			if (text.isSelected()) {
				if (firstSelected == -1) {
					firstSelected = i;
				} else {
					lastSelected = i;
				}
			}
		}

		if (lastSelected > -1) {
			for (int i = firstSelected; i < lastSelected + 1; i++) {
				Text text = enteredText.get(i);
				text.setSelected(true);
			}
		}
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		fontMetrics = getFontMetrics(font);
		segmentPanel.getPainter().drawEnteredGlossText(enteredText, g2d, Y,
				fontMetrics, glsField);
	}

	public int getEndSelectionCoord() {
		return endSelectionCoord;
	}

	public SignStreamSegmentPanel getSegmentPanel() {
		return segmentPanel;
	}

	public int getStartSelectionCoord() {
		return startSelectionCoord;
	}

	public void setEndSelectionCoord(int i) {
		endSelectionCoord = i;
	}

	public void setSegmentPanel(SignStreamSegmentPanel panel) {
		segmentPanel = panel;
	}

	public ArrayList<Event> createGlossChainedEvent(int startTime, int endTime) {
		ArrayList<Event> list = new ArrayList<Event>();

		ArrayList<Text> selectedText = getSelectedText();
		if (!selectedText.isEmpty()) {
			String text = "";

			for (Text t : selectedText) {
				text = text + t.getText() + " ";
			}

			text = text.trim();	// disallow entering whitespace at front or end upon newly adding event from entered text

			SS3Item item = new SS3Item();
			item.setId(Util.getLongUniqueNumber());
			TimeInfo startTimeInfo = new TimeInfo();
			startTimeInfo.setMovieTime(startTime);
			TimeInfo endTimeInfo = new TimeInfo();
			endTimeInfo.setMovieTime(endTime);
			item.setStartTimeInfo(startTimeInfo);
			item.setEndTimeInfo(endTimeInfo);
			item.setText(text); // selectedText.getText());

			GlossChainedEvent glsEvent = new GlossChainedEvent(item,
					segmentPanel);
			glsEvent.select();
			ChainedEventsCollection collection = new ChainedEventsCollection();
			collection.setTextValueEvent(glsEvent);

			GlossChainedEventsEntity entity = new GlossChainedEventsEntity(
					collection, segmentPanel, glsField);
			entity.determineParentEntity();
			glsField.addGlossChainedEventsEntity(entity);

			for (Text t1 : selectedText) {
				enteredText.remove(t1);
			}

			glsEvent.select();

			list.add(glsEvent);
			list.add(entity);
		}

		return list;
	}

	public void moveTextObjectsForward(int legacyStartCoord, int newStartCoord,
			int prevEventEndCoord) {

		// select Text objects that are in between legacyStartCoord and
		// maxEndCoord
		ArrayList<Text> textToMove = new ArrayList<Text>();
		for (Text text : enteredText) {
			if (prevEventEndCoord <= text.getEndCoord()
					|| text.getStartCoord() <= newStartCoord) {
				textToMove.add(text);
			}
		}

		// sort textToMove Text objects
		int[] startCoordsToSort = new int[textToMove.size()];
		for (int counter = 0; counter < textToMove.size(); counter++) {
			Text txt = textToMove.get(counter);
			startCoordsToSort[counter] = txt.getEndCoord();
		}
		Sorter.sort(startCoordsToSort);

		// create sorted list of Text objects to move
		ArrayList<Text> sortedText = new ArrayList<Text>();
		for (int i = 0; i < startCoordsToSort.length; i++) {
			int startCoord = startCoordsToSort[i];
			for (Text txt : textToMove) {
				if (txt.getStartCoord() == startCoord) {
					sortedText.add(txt);
					textToMove.remove(txt);
				}
			}
		}

		// move objects
		int txtEndCoord = newStartCoord - 10;
		boolean visible = true;
		for (int j = sortedText.size() - 1; j >= 0; j--) {
			Text t = sortedText.get(j);

			int width = fontMetrics.stringWidth(t.getText());
			int txtStartCoord = txtEndCoord - width;

			if (prevEventEndCoord >= txtStartCoord) {
				visible = false;
			}

			t.setStartCoord(txtStartCoord);
			t.setEndCoord(txtEndCoord);
			t.setVisible(visible);

			txtStartCoord = txtStartCoord - 10;
		}
	}

	public void setStartSelectionCoord(int coord) {
		startSelectionCoord = coord;
	}

	public boolean isSelectedSegment() {
		return selectedSegment;
	}

	public void setSelectedSegment(boolean b) {
		selectedSegment = b;
	}
}
