package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollectionManager;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;

public class GlossChainedEventsEntity extends ChainedEventsEntity implements Compoundable {

	private static final long serialVersionUID = 1L;
	
	private boolean isLockedLeftRightAlignments = false;
	
	private ChainedEventsEntity dependentHandEntity = null;
	private HandShapeEntity dependentHandShapeEntity = null;
	private GlossChainedEventsEntity dependentGlossEntity = null;
	private LocationEntity dependentLocationEntity = null;
	private SignTypeEntity dependentSignTypeEntity = null;

    private String dependentGlossEntityID;
    private String dependentHandEntityID;
    private String dependentHandShapeEntityID;
    private String dependentLocationEntityID;
    private String dependentSignTypeEntityID;

    private GlossChainedEventsEntity previousEntity = null;
    private GlossChainedEventsEntity nextEntity = null;

    private ChainedEventsCollection chainedEventsCollection = null;

    protected ChainedEventsCollectionManager manager;

    private String correspondingGlossEntityID = "";

    private Compoundable startCompound = null;
    private Compoundable endCompound = null;

    private GlossEntityProperties glossProperties = new GlossEntityProperties();

    public String getDependentGlossEntityID() {
		return dependentGlossEntityID;
	}

	public void setDependentGlossEntityID(String dependentGlossEntityID) {
		this.dependentGlossEntityID = dependentGlossEntityID;
	}

	public String getDependentHandEntityID() {
		return dependentHandEntityID;
	}

	public void setDependentHandEntityID(String dependentHandEntityID) {
		this.dependentHandEntityID = dependentHandEntityID;
	}

	public String getDependentHandShapeEntityID() {
		return dependentHandShapeEntityID;
	}

	public void setDependentHandShapeEntityID(String dependentHandShapeEntityID) {
		this.dependentHandShapeEntityID = dependentHandShapeEntityID;
	}

	public String getDependentLocationEntityID() {
		return dependentLocationEntityID;
	}

	public void setDependentLocationEntityID(String dependentLocationEntityID) {
		this.dependentLocationEntityID = dependentLocationEntityID;
	}

	public String getDependentSignTypeEntityID() {
		return dependentSignTypeEntityID;
	}

	public void setDependentSignTypeEntityID(String dependentSignTypeEntityID) {
		this.dependentSignTypeEntityID = dependentSignTypeEntityID;
	}

	public GlossChainedEventsEntity(ChainedEventsCollection collection, SignStreamSegmentPanel segmentPanel, Field field) {
		super(collection, segmentPanel, field);
		setID(Util.getLongUniqueNumber());
		this.manager = new ChainedEventsCollectionManager(collection,
				segmentPanel);

		this.chainedEventsCollection = collection;

        this.getSS3Entity();
	}
	
	public boolean isLockedLeftRightAlignments() {
		return isLockedLeftRightAlignments;
	}
	
	public void setLockedLeftRightAlignments(boolean isLockedLeftRightAlignments) {
		this.isLockedLeftRightAlignments = isLockedLeftRightAlignments;
	}

	public GlossEntityProperties getGlossEntityProperties() {
		if(glossProperties == null) {
			glossProperties = new GlossEntityProperties();
		}
		return glossProperties;
	}
	
	@Override
	public GlossChainedEventsEntity clone(String fieldID) {
		ChainedEventsCollection cll = manager.getChainedEventsCollection().cloneChainedEventsCollection();
		GlossChainedEventsEntity ret = new GlossChainedEventsEntity(cll, segmentPanel, getField());
		ret.determineParentEntity();
		return ret;
	}

	@Override
	public String addOffsetEvent() {
		if (nextEntity != null) {
			int nextEntityStartMovieTime = nextEntity.getStartTimeInfo().getMovieTime();
			int packetDuration = (int) segmentPanel.getMultipleMovieController().getPacketDuration();

			int selectedEventEndTime = getEndTimeInfo().getMovieTime() + packetDuration;

			if (selectedEventEndTime > nextEntityStartMovieTime) {
				return ErrorMessages.CHAINS_CANNOT_OVERLAP;
			}
		}

		manager.addOffsetEvent();

		return null;
	}
	
	@Override
	public void deleteOnset() {
		super.deleteOnset();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		int time = getStartTimeInfo().getMovieTime();
		
		if (isLockedLeftRightAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getOnset() != null) {
				depGlsEntity.deleteOnset();
				depGlsEntity.getStartTimeInfo().setMovieTime(time);
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteOffset() {
		super.deleteOffset();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		int time = getEndTimeInfo().getMovieTime();
		
		if (isLockedLeftRightAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getOffset() != null) {
				depGlsEntity.deleteOffset();
				depGlsEntity.getEndTimeInfo().setMovieTime(time);
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}

		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteInitialHold() {
		super.deleteInitialHold();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		ChainedEventsEntity handEntity = getDependentHandEntity();
		HandShapeEntity handShapeEntity = getDependentHandShapeEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		int time = getStartTimeInfo().getMovieTime();
		
		if (isLockedGlossPhonAlignments()) {
			if (handEntity != null) {
				handEntity.getStartTimeInfo().setMovieTime(time);
				if (handShapeEntity != null) {
					MediaToolbarCommons.setHandshapeStartTime(
							handShapeEntity, time);
				}
			}
		}
		
		if (isLockedLeftRightAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getInitialHold() != null) {
				depGlsEntity.deleteInitialHold();
				depGlsEntity.getStartTimeInfo().setMovieTime(time);
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}
	
	@Override
	public void deleteFinalHold() {
		super.deleteFinalHold();

		GlossChainedEventsEntity depGlsEntity = getDependentGlossEntity();
		ChainedEventsEntity handEntity = getDependentHandEntity();
		HandShapeEntity handShapeEntity = getDependentHandShapeEntity();
		LocationEntity locationEntity = getDependentLocationEntity();
		
		int time = getEndTimeInfo().getMovieTime();
		
		if (isLockedGlossPhonAlignments()) {
			if (handEntity != null) {
				handEntity.getEndTimeInfo().setMovieTime(time);
				if (handShapeEntity != null) {
					MediaToolbarCommons.setHandshapeEndTime(
							handShapeEntity, time);
				}
			}
		}
		
		if (isLockedLeftRightAlignments()) {
			if (depGlsEntity != null && depGlsEntity.getFinalHold() != null) {
				depGlsEntity.deleteFinalHold();
				depGlsEntity.getEndTimeInfo().setMovieTime(time);
			}
		}

		if (locationEntity != null) {
			locationEntity.updateTimes();
		}
		
		SignTypeEntity signTypeEntity = getDependentSignTypeEntity();
		if (signTypeEntity != null) {
			signTypeEntity.updateTimes();
		}
	}

	public ArrayList<EventsEntity> getDependentEntities() {
		ArrayList<EventsEntity> dEntities = new ArrayList<EventsEntity>();
		
		if(dependentHandEntity != null) {
			dEntities.add(dependentHandEntity);
		}
		
		if(dependentHandShapeEntity != null) {
			dEntities.add(dependentHandShapeEntity);
		}
		
		if(dependentGlossEntity != null) {
			dEntities.add(dependentGlossEntity);
			
			EventsEntity depHandEntity = dependentGlossEntity.getDependentHandEntity();
			if(depHandEntity != null) {
				dEntities.add(depHandEntity);
			}
			
			EventsEntity depHandShapeEntity = dependentGlossEntity.getDependentHandShapeEntity(); 
			if(depHandShapeEntity != null) {
				dEntities.add(depHandShapeEntity);
			}
		}

		if (dependentLocationEntity != null) {
			dEntities.add(dependentLocationEntity);
		}
		
		if (dependentSignTypeEntity != null) {
			dEntities.add(dependentSignTypeEntity);
		}
		return dEntities;
	}
	
	public void setGlossEntityProperties(GlossEntityProperties glossProperties) {
		this.glossProperties = glossProperties;
	}
	
	@Override
	public ChainedEventsCollection getChainedEventsCollection() {
		return chainedEventsCollection;
	}

	public ChainedEventsEntity getDependentHandEntity() {
		return dependentHandEntity;
	}

	public void setCorrespondingGlossEntityID(String correspondingGlossEntityID) {
		this.correspondingGlossEntityID = correspondingGlossEntityID;
	}
	
	public String getCorrespondingGlossEntityID() {
		return correspondingGlossEntityID;
	}
	
	public void setDependentHandEntity(ChainedEventsEntity dependentHandEntity) {
		if(dependentHandEntity != null) {
	        setDependentHandEntityID(dependentHandEntity.getID());
		}
		this.dependentHandEntity = dependentHandEntity;
	}

	public HandShapeEntity getDependentHandShapeEntity() {
		return dependentHandShapeEntity;
	}

	public void setDependentHandShapeEntity(
			HandShapeEntity dependentHandShapeEntity) {
        this.dependentHandShapeEntity = dependentHandShapeEntity;
        if(dependentHandEntity != null) {
            this.setDependentHandShapeEntityID(dependentHandShapeEntity.getID());
		}
	}

	public GlossChainedEventsEntity getDependentGlossEntity() {
		return dependentGlossEntity;
	}

	public void setDependentGlossEntity(
			GlossChainedEventsEntity dependentGlossEntity) {
		this.dependentGlossEntity = dependentGlossEntity;

        if(dependentGlossEntity != null) {
		    setCorrespondingGlossEntityID(dependentGlossEntity.getID());
        } else {
            setCorrespondingGlossEntityID(null);
        }
	}

	public LocationEntity getDependentLocationEntity() {
		return dependentLocationEntity;
	}

	public void setDependentLocationEntity(LocationEntity dependentLocationEntity) {
    	if (dependentLocationEntity != null) {
    		setDependentLocationEntityID(dependentLocationEntity.getID());

			ArrayList<LocationEvent> events = dependentLocationEntity.getEvents();
			HashMap<LocationType, Region> locationRegionsMap = new HashMap<LocationType, Region>();
			for (LocationEvent event : events) {
				locationRegionsMap.put(event.getLocationType(), event.getLocation());
			}
			glossProperties.setLocationInfo(new LocationInfo(locationRegionsMap));
		}
		this.dependentLocationEntity = dependentLocationEntity;
	}
	
	public SignTypeEntity getDependentSignTypeEntity() {
		return dependentSignTypeEntity;
	}

	public void setDependentSignTypeEntity(SignTypeEntity dependentSignTypeEntity) {
		this.dependentSignTypeEntity = dependentSignTypeEntity;
    	if (dependentSignTypeEntity != null) {
    		setDependentSignTypeEntityID(dependentSignTypeEntity.getID());
		}
	}

	@Override
	public GlossChainedEventsEntity getPreviousEntity() {
		return previousEntity;
	}

	public void setPreviousEntity(GlossChainedEventsEntity previousEntity) {
		this.previousEntity = previousEntity;
	}

	@Override
	public GlossChainedEventsEntity getNextEntity() {
		return nextEntity;
	}

	public void setNextEntity(GlossChainedEventsEntity nextEntity) {
		this.nextEntity = nextEntity;
	}

	@Override
	public String getText() {
        if(getTextValue() == null) {
            return "";
        } else {
		    return getTextValue().getText();
        }
	}

	@Override
	public String getText(boolean start) {
		return getText();
	}

	@Override
	public boolean isEndCompound() {
		return endCompound != null;
	}

	@Override
	public boolean isStartCompound() {
		return startCompound != null;
	}

	@Override
	public Compoundable getEndCompound() {
		return endCompound;
	}

	@Override
	public Compoundable getStartCompound() {
		return startCompound;
	}

	@Override
	public void setEndCompound(Compoundable endCompound) {
		this.endCompound = endCompound;
	}

	@Override
	public void setStartCompound(Compoundable startCompound) {
		this.startCompound = startCompound;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		g2d.setFont(font);
		FontMetrics fontMetrics = getFontMetrics(font);
		
		segmentPanel.getPainter().drawGlossEntity(this, g2d, Y, fontMetrics, getField().getFieldID(), previousEntity, nextEntity);
	}
}
