package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.logging.Level;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

/**
 * Non-manual fields fall into two categories: anatomical and functional.
 * 
 * Anatomical fields refer to particular physical muscle movement.
 * Functional/interpretive fields provide a functional interpretation for a
 * particular movement or set of movements.
 * */

public class GlossField extends NonManualField { // implements ActionListener {

	private static final long serialVersionUID = 1L;

	int selectedCompoundStartCoord = -1;
	int selectedCompoundEndCoord = -1;
	
	private NewGlossChainedEvent newGlossEvent;
	private GlossChainedEventsEntityCollection collection = null;

	public GlossField(SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
	}

	public GlossField(Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.track = track;
		createTrackEntity(track);
	}

	public GlossField(SS3Track ss3track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = ss3track;
		createTrackEntity(ss3track);
	}

	public GlossChainedEventsEntityCollection getGlossChainedEventsEntityCollection() {
		return collection;
	}
	
	public int getPrevEntityOrTextStartCoord(int coord) {
		return collection.getPrevEntityOrTextStartCoord(coord);
	}
	
	public int getNextEntityOrTextStartCoord(int coord) {
		return collection.getNextEntityOrTextStartCoord(coord);
	}
	
	public ArrayList<GlossChainedEventsEntity> getEntitiesInInterval(int coord1, int coord2) {
		return collection.getEntitiesInInterval(coord1, coord2);
	}
	
    
	@Override
	public void deleteEntity(EventsEntity entity) {
		GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity)entity;
		collection.deleteEntity(glsEntity);
	}
	
	public void setGlossChainedEventsEntityCollection(GlossChainedEventsEntityCollection collection) {
		this.collection = collection;
	}
	
	public void setSelectCompoundStartCoord(int selectedCompoundStartCoord) {
		this.selectedCompoundStartCoord = selectedCompoundStartCoord;
	
		EnteredText enteredText = null;

		try {
			enteredText = getGlossChainedEventsEntityCollection().getEnteredText();
			enteredText.unselectAll();
		} catch (NullPointerException e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
		}
	}

	public void setSelectCompoundEndCoord(int selectedCompoundEndCoord) {
		this.selectedCompoundEndCoord = selectedCompoundEndCoord;
	}

	public int getSelectedCompoundStartCoord() {
		return selectedCompoundStartCoord;
	}

	public int getSelectedCompoundEndCoord() {
		return selectedCompoundEndCoord;
	}

	public NewGlossChainedEvent getNewGlossChainedEvent() {
		return newGlossEvent;
	}

	public void setNewGlossChainedEvent(NewGlossChainedEvent newGlossEvent) {
		this.newGlossEvent = newGlossEvent;
	}

	public ArrayList<Compoundable> getEventsToCreateCompound() {
		
		if (selectedCompoundStartCoord == -1 || selectedCompoundEndCoord == -1)
			return null;

		ArrayList<NotCompleteGlossEvent> nonComplEventsInInterval = collection
				.getNotCompleteGlossEventsInInterval(
						selectedCompoundStartCoord, selectedCompoundEndCoord);
		ArrayList<GlossChainedEventsEntity> entitiesInInterval = collection.getEntitiesInInterval(
				selectedCompoundStartCoord, selectedCompoundEndCoord);
		
		ArrayList<Compoundable> compounds = new ArrayList<Compoundable>();

		collection.sort(entitiesInInterval);
		
		for(GlossChainedEventsEntity entity : entitiesInInterval) {
			compounds.add(entity);
		}

		if (nonComplEventsInInterval.size() == 0) {
			return compounds;
		} else if (nonComplEventsInInterval.size() > 2) {
			return null;

		} else {
			NotCompleteGlossEvent presNotComplEvent = null;
			NotCompleteGlossEvent posNotComplEvent = null;

			NotCompleteGlossEvent nCmlGlsEvent1 = nonComplEventsInInterval.get(0);
			
			if (nCmlGlsEvent1.isEndMovieTimeFinal()) {
				presNotComplEvent = nCmlGlsEvent1;
			} else {
				posNotComplEvent = nCmlGlsEvent1;
			}

			if (nonComplEventsInInterval.size() == 2) {
				NotCompleteGlossEvent nCmlGlsEvent2 = nonComplEventsInInterval
						.get(1);
				if (presNotComplEvent != null
						&& nCmlGlsEvent2.isStartMovieTimeFinal()
						&& presNotComplEvent.getEndTimeInfo().getMovieTime() <= nCmlGlsEvent2
								.getStartTimeInfo().getMovieTime()) {
					posNotComplEvent = nCmlGlsEvent2;
				} else if (posNotComplEvent != null
						&& nCmlGlsEvent2.isEndMovieTimeFinal()
						&& nCmlGlsEvent2.getEndTimeInfo().getMovieTime() <= posNotComplEvent
								.getStartTimeInfo().getMovieTime()) {
					presNotComplEvent = nCmlGlsEvent2;
				}
			}

			if (compounds.size() == 0) {
				if (presNotComplEvent != null)
					compounds.add(presNotComplEvent);
				if (posNotComplEvent != null)
					compounds.add(posNotComplEvent);
			} else {
				Compoundable strEntity = compounds.get(0);
				Compoundable endEntity = compounds.get(compounds.size() - 1);
				if (presNotComplEvent != null
						&& strEntity.getStartTimeInfo().getMovieTime() >= presNotComplEvent
								.getEndTimeInfo().getMovieTime()) {
					compounds.add(0, presNotComplEvent);
				}
				if (posNotComplEvent != null
						&& endEntity.getEndTimeInfo().getMovieTime() <= posNotComplEvent
								.getStartTimeInfo().getMovieTime()) {
					compounds.add(compounds.size(), posNotComplEvent);
				}
			}
			return compounds;
		}
	}

	@Override
	public void deleteEmptyEntities() {
		ArrayList<GlossChainedEventsEntity> entities = collection.getEntities();
		ArrayList<GlossChainedEventsEntity> entitiesToDelete = new ArrayList<GlossChainedEventsEntity>();
		for(GlossChainedEventsEntity entity : entities) {
			if(entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null || entity.getValueOrTextValueEvent() == null) {
				entitiesToDelete.add(entity);
			}
		}
		
		for(GlossChainedEventsEntity toDelete : entitiesToDelete) {
			collection.deleteEntity(toDelete);
		}
	//	collection.deleteEmptyEntities();
	//	collection.updateGlossEntities();
	}

	@Override
	public ArrayList getEntities() {
		return collection.getEntities();
	}
	
	
	public void addGlossChainedEventsEntity(GlossChainedEventsEntity entity) {
		collection.addEntity(entity);	
	}

	@Override
	public Event getEventAt(int x) {
		return collection.getEventAt(x);
	}
	
	@Override
	public EventsEntity getEntityAt(int x) {
		return collection.getGlossChainedEventsEntityAt(x);
	}

	@Override
	public Track getTrack() {
		return track;
	}

	private void createTrackEntity(SS3Track track) {
		ArrayList<GlossChainedEventsEntity> chainedEvents = new ArrayList<GlossChainedEventsEntity >();
		ArrayList<SS3Entity> entities = track.getEntities();
		ArrayList<SS3Item> holds = new ArrayList<SS3Item>();

		ArrayList<SS3Item> txtOnly = new ArrayList<SS3Item>();
	 
		for (SS3Entity entity : entities) {
			ChainedEventsCollection collection = new ChainedEventsCollection();
			ArrayList<SS3Item> items = entity.getItemsAsList();
			for (SS3Item item : items) {
				String id = item.getId();

				if ((track.getId().equals(Util.DOM_GLOSS_FIELD_ID) ||
						track.getId().equals(Util.NDOM_GLOSS_FIELD_ID)) && item.isTextOnly()) {
					txtOnly.add(item);
				} else {
					if (id.equalsIgnoreCase("" + Constants.ONSET_CODE)) {
						collection.setOnsetEvent(new ChainedEvent(item,
								segmentPanel));
					} else if (id.equalsIgnoreCase("" + Constants.OFFSET_CODE)) {
						collection.setOffsetEvent(new ChainedEvent(item,
								segmentPanel));
					} else if (!id.equalsIgnoreCase("" + Constants.HOLD_CODE)) {
						collection.setTextValueEvent(new GlossChainedEvent(
								item, segmentPanel));
					} else {
						holds.add(item);
					}
				}
			}
			
			// means its a chained entity and not a text-only
			if (collection.getTextValueEvent() != null) {
				for (SS3Item item : holds) {
					int valueStartTime = collection.getTextValueEvent()
							.getStartTimeInfo().getMovieTime();
					int valueEndTime = collection.getTextValueEvent()
							.getEndTimeInfo().getMovieTime();
					int holdStartTime = item.getStartTimeInfo().getMovieTime();
					int holdEndTime = item.getEndTimeInfo().getMovieTime();

					if (holdStartTime < valueStartTime) {
						collection.setInitialHoldEvent(new ChainedEvent(item,
								segmentPanel));
					} else if (holdEndTime > valueEndTime) {
						collection.setFinalHoldEvent(new ChainedEvent(item,
								segmentPanel));
					}
				}
				
				GlossChainedEventsEntity chainedEventEntity = new GlossChainedEventsEntity(collection, segmentPanel, this);

				chainedEventEntity.setGlossEntityProperties(entity.getGlossEntityProperties());
				chainedEventEntity.setCorrespondingGlossEntityID(entity.getDependentGlossEntityID());
				
				String dependentGlossEntityID = entity.getDependentGlossEntityID();
				String dependentHandEntityID = entity.getDependentHandEntityID();
				String dependentHandShapeEntityID = entity.getDependentHandShapeEntityID();
				String dependentLocationEntityID = entity.getDependentLocationEntityID();
				String dependentSignTypeEntityID = entity.getDependentSignTypeEntityID();
				boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
				boolean isLockedLeftRightAlignments = entity.isLockedLeftRightAlignments();
							
				chainedEventEntity.setDependentGlossEntityID(dependentGlossEntityID);
				chainedEventEntity.setDependentHandEntityID(dependentHandEntityID);
				chainedEventEntity.setDependentHandShapeEntityID(dependentHandShapeEntityID);
				chainedEventEntity.setDependentLocationEntityID(dependentLocationEntityID);
				chainedEventEntity.setDependentSignTypeEntityID(dependentSignTypeEntityID);
				
				chainedEventEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);
				chainedEventEntity.setLockedLeftRightAlignments(isLockedLeftRightAlignments);

				chainedEventEntity.setID(entity.getID());
				chainedEventEntity.setRefEntityId(entity.getRefEntityId());
				//chainedEventEntity.setReference(entity.getReference());
	            chainedEventEntity.setParentEntity(getParentField().getEntity(entity.getParentEntityID()));

//				if (entity.isValueStartTimeLocked() && entity.isValueEndTimeLocked()) {
//					chainedEventEntity.setLockedLeftRightAlignments(true);
//				}
				
//				|| entity.isInitialHoldStartTimeLocked())
//					chainedEventEntity.lockStart();
	//
//				if (entity.isValueEndTimeLocked() || entity.isFinalHoldEndTimeLocked())
//					chainedEventEntity.lockEnd();

				chainedEvents.add(chainedEventEntity);
			}
		}
// 
		// GlossChainedEventsEntityCollection 
		collection = new GlossChainedEventsEntityCollection(chainedEvents, segmentPanel);
//		ArrayList<GlossChainedEventsEntity> entities, segmentPanel);		
// 		ChainedEventsEntityCollection(chainedEvents, segmentPanel);
// 		collection.updateGlossEntities();

		// set compound mappings
		for (int c = 0; c < entities.size(); c++) {
			SS3Entity ent = entities.get(c);
			String sCmpEntID = ent.getStartCompaundEntityID();
			String eCmpEntID = ent.getEndCompaundEntityID();

			if (Util.isNotEmpty(sCmpEntID) || Util.isNotEmpty(eCmpEntID)) {
				GlossChainedEventsEntity mEnt = collection.getEntity(ent.getID());
				if (mEnt != null) {
					if (Util.isNotEmpty(sCmpEntID)) {
						GlossChainedEventsEntity sEnt = collection.getEntity(sCmpEntID);
						mEnt.setStartCompound(sEnt);
					}
					if (Util.isNotEmpty(eCmpEntID)) {
						GlossChainedEventsEntity eEnt = collection.getEntity(eCmpEntID);
						mEnt.setEndCompound(eEnt);
					}
				}
			}
		}

		EnteredText entText = new EnteredText(segmentPanel, this);
		for(SS3Item txt: txtOnly) {
			Text text = new Text(txt.getText());
			text.setStartTime(txt.getStartTimeInfo().getMovieTime());
			text.setEndTime(txt.getEndTimeInfo().getMovieTime());
			entText.addText(text);
		}
		
		collection.setEnteredTextObject(entText);
	}

	private void createTrackEntity(Track track) {
		ArrayList<GlossChainedEventsEntity> chainedEvents = new ArrayList<GlossChainedEventsEntity>();
		ArrayList trackValues = track.getTrackValues();

		ArrayList<TrackValue> values = new ArrayList<TrackValue>();
		HashMap<String, TrackValue> onset = new HashMap<String, TrackValue>();
		HashMap<String, TrackValue> onsetHold = new HashMap<String, TrackValue>();
		HashMap<String, TrackValue> offset = new HashMap<String, TrackValue>();
		HashMap<String, TrackValue> offsetHold = new HashMap<String, TrackValue>();

		for (int i = 0; i < trackValues.size(); i++) {
			TrackValue trackValue = (TrackValue) trackValues.get(i);
			int valueId = trackValue.getValueId();

			if (valueId == Constants.HOLD_CODE) {
				int startFrame = trackValue.getStartTimeInfo().getMovieTime();
				int endFrame = trackValue.getEndTimeInfo().getMovieTime();
				onsetHold.put("" + endFrame, trackValue);
				offsetHold.put("" + startFrame, trackValue);
			} else if (valueId == Constants.ONSET_CODE) {
				int endFrame = trackValue.getEndTimeInfo().getMovieTime();
				onset.put("" + endFrame, trackValue);
			} else if (valueId == Constants.OFFSET_CODE) {
				int startFrame = trackValue.getStartTimeInfo().getMovieTime();
				offset.put("" + startFrame, trackValue);
			} else {
				values.add(trackValue);
			}
		}

		for (TrackValue trackValue : values) { 
			int startingFrame = trackValue.getStartTimeInfo().getMovieTime();
			int endingFrame = trackValue.getEndTimeInfo().getMovieTime();

			TrackValue onsetHoldValue = onsetHold.get(""
					+ startingFrame);
			TrackValue offsetHoldValue = offsetHold.get(""
					+ endingFrame);
			TrackValue onsetValue = null, offsetValue = null;

			if (onsetHoldValue == null) {
				onsetValue = onset.get("" + startingFrame);
			} else {
				onsetValue = onset.get(""
						+ onsetHoldValue.getStartTimeInfo().getMovieTime());
			}

			if (offsetHoldValue == null) {
				offsetValue = onset.get("" + endingFrame);
			} else {
				offsetValue = onset.get(""
						+ offsetHoldValue.getEndTimeInfo().getMovieTime());
			}

			ChainedEventsCollection collection = new ChainedEventsCollection();
			collection.setOnsetEvent(new ChainedEvent(onsetValue, segmentPanel));
			collection.setInitialHoldEvent(new ChainedEvent(onsetHoldValue, segmentPanel));
			collection.setTextValueEvent(new GlossChainedEvent(trackValue, segmentPanel));
			collection.setFinalHoldEvent(new ChainedEvent(offsetHoldValue, segmentPanel));
			collection.setOffsetEvent(new ChainedEvent(offsetValue, segmentPanel));

			GlossChainedEventsEntity chainedEvent = new GlossChainedEventsEntity(collection, segmentPanel, this);
			chainedEvent.determineParentEntity();

			chainedEvents.add(chainedEvent);
		}

		collection = new GlossChainedEventsEntityCollection(chainedEvents, segmentPanel); 
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		this.Y = Y;
		
		ArrayList<GlossChainedEventsEntity> glossEntities = collection.getEntities();
		
		for(GlossChainedEventsEntity entity : glossEntities) {
			entity.paint(g2d, Y, font);
		}

		if (selectedCompoundStartCoord > -1 && selectedCompoundEndCoord > -1) {
			g2d.setColor(Palette.SELECTED_BACKGROUND_COLOR);
			g2d.fillRect(selectedCompoundStartCoord, (int) (Y - 13),
					selectedCompoundEndCoord - selectedCompoundStartCoord, 19);
		}

		try {
			getGlossChainedEventsEntityCollection().getEnteredText().paint(g2d, Y, font);
		} catch (NullPointerException ex) {
			;
		}
	}
	
	@Override
	public boolean isEmptyInterval(int startTime, int endTime) {
		
		int startCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(startTime);
		int endCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(endTime);
		
		return collection.getEntitiesInInterval(startCoord, endCoord).isEmpty();
	}
}
