package edu.bu.signstream.grepresentation.fields.glossField;

import java.awt.Color;
import java.awt.FontMetrics;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.TimeInfo;

//create Compoundable interface, NotCompleteGlossEvent and NonManulsEntities should implement

public class NotCompleteGlossEvent implements Compoundable {

   private String id = "";
   private String text = "";

   private int startMovieTime = -1;
   private int endMovieTime = -1;

   private int startMovieCoord = -1;
   private int endMovieCoord = -1;

   private boolean startMovieTimeFinal = false;
   private boolean endMovieTimeFinal = false;
   
   private Compoundable startCompound = null;
   private Compoundable endCompound = null;
   
   private FontMetrics fontMetrics;
   private Color color = Color.BLUE;
   private CoordinateTimeConverter convertor = null;
   
   private boolean selected = false;

   public NotCompleteGlossEvent(
      String text,
      int startMovieTime,
      boolean startMovieTimeFinal,
      int endMovieTime,
      boolean endMovieTimeFinal,
      CoordinateTimeConverter convertor) {

      this.convertor = convertor;
      this.text = text;
      setStartMovieTime(startMovieTime, startMovieTimeFinal);
      setEndMovieTime(endMovieTime, endMovieTimeFinal);
   }
   
   @Override
public String getID() {
      return id;
   }
   
   @Override
public void setID(String id) {
      this.id = id;
   }

   @Override
public boolean isEndCompound() {
      return endCompound != null;
   }
   
   @Override
public boolean isStartCompound() {
      return startCompound != null;
   }
   
   @Override
public Compoundable getEndCompound() {
      return endCompound;
   }
   
   @Override
public Compoundable getStartCompound() {
      return startCompound;
   }

   @Override
public void setEndCompound(Compoundable endCompound) {
      this.endCompound = endCompound;
   }
   
   @Override
public void setStartCompound(Compoundable startCompound) {
      this.startCompound = startCompound;
   }

   public void setFontMetrics(FontMetrics fontMetrics) {
      this.fontMetrics = fontMetrics;
   }

   public FontMetrics getFontMetrics() {
      return fontMetrics;
   }
   
   @Override
public TimeInfo getStartTimeInfo() {
	   TimeInfo timeInfo = new TimeInfo();
	   timeInfo.setMovieTime(startMovieTime);
	   return timeInfo;
   }
   
   @Override
public TimeInfo getEndTimeInfo() {
	   TimeInfo timeInfo = new TimeInfo();
	   timeInfo.setMovieTime(endMovieTime);
	   return timeInfo;
   }

   @Override
public int getEndTimeCoordinate() {
      return endMovieCoord;
   }

   public boolean isEndMovieTimeFinal() {
      return endMovieTimeFinal;
   }

   @Override
public int getStartTimeCoordinate() {
      return startMovieCoord;
   }

   public boolean isStartMovieTimeFinal() {
      return startMovieTimeFinal;
   }

   public String getText() {
      return text;
   }
   
   @Override
public String getText(boolean start) {
	   return getText();
   }

   public void setEndMovieCoordinate(int endMovieCoord, boolean endMovieTimeFinal) {
      this.endMovieCoord = endMovieCoord;
      this.endMovieTime = convertor.convertCoordinateToTime(endMovieCoord);
      this.endMovieTimeFinal = endMovieTimeFinal;
   }

   public void setEndMovieTime(int endMovieTime, boolean endMovieTimeFinal) {
      this.endMovieTime = endMovieTime;
      this.endMovieCoord = convertor.convertTimeToCoordinate(endMovieTime);
      this.endMovieTimeFinal = endMovieTimeFinal;
   }

   public void setStartMovieCoordinate(
      int startMovieCoord,
      boolean startMovieTimeFinal) {
      this.startMovieCoord = startMovieCoord;
      this.startMovieTime = convertor.convertCoordinateToTime(startMovieCoord);
      this.startMovieTimeFinal = startMovieTimeFinal;
   }

   public void setStartMovieTime(
      int startMovieTime,
      boolean startMovieTimeFinal) {
      this.startMovieTime = startMovieTime;
      this.startMovieCoord = convertor.convertTimeToCoordinate(startMovieTime);
      this.startMovieTimeFinal = startMovieTimeFinal;
   }

   public void setText(String string) {
      text = string;
   }

   public boolean isSelected() {
      return selected;
   }

   public void setSelected(boolean b) {
      selected = b;
   }

   public Color getColor() {
      return color;
   }

   public void setColor(Color color) {
      this.color = color;
   }
   
   @Override
public String toString() {
      StringBuffer sb = new StringBuffer();
      sb.append("NotCompleteGlossEvent: ");
      sb.append(text);
      sb.append(", startMovieTime = ");
      sb.append(startMovieTime);
      sb.append(", endMovieTime = ");
      sb.append(endMovieTime);
      sb.append(", startMovieTimeFinal = ");
      sb.append(startMovieTimeFinal);
      sb.append(", endMovieTimeFinal = ");
      sb.append(startMovieTimeFinal);
      return sb.toString();
   } 
}
