
package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEvent;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossClassifications;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.LeftRightHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeSubEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.locationField.*;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.AccessSignBankActionListener;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.signTypeField.*;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GlossNRelatedView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolbarCommons;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.grepresentation.view.signbank.SignBankMainGloss;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.signbank.SignBankDialog;

public class AssociatedFieldsPanel extends JPanel implements ItemListener, ActionListener {

	private static final long serialVersionUID = 1L;

	private final JDialog parent;

	private SignBankDialog signBankDialog = null;

	private JTabbedPane tabbedPane;

	private GlossFieldWrapper wrapper = null;
	private HandShapePanel hshPanel = null;

	private LocationPanel locationPanel;

	private final JLabel primaryEntryLb = new JLabel("Primary Entry");
	private final JTextField primaryEntryField = new JTextField();

	private final JLabel variantLb = new JLabel("Entry/Variant");
	private final JTextField variantField = new JTextField();
	private final JButton searchSignBank = new JButton("Search Sign Bank");

	private final JButton addNewSign = new JButton("Add To Sign Bank");

	private final JLabel domHandLbl = new JLabel("Dominant Hand");
	private final JRadioButton domHandR = new JRadioButton("Right");
	private final JRadioButton domHandL = new JRadioButton("Left");

	private final JButton enter = new JButton("Enter");
	private final JButton cancel = new JButton("Cancel");
	private final JButton bypass = new JButton("Bypass Sign Bank Check");

	private SignType currentSignType;

	private final int ENTER_BYPASS = -1;
	private final int ENTER = 0;
	private final int CANCEL = 1;

	private final int SIGN_BANK = 8;
	private final int ADD_NEW = 9;

	public static final int TAB_HANDSHAPES = 0;
	public static final int TAB_LOCATION = 1;

	private GlossNRelatedView view = null;

	private HandShapePreprocessor processor = null;
	private HandShapeFormBean hshFormBean = null;

	private final LoadedProperties loadedProperties = new LoadedProperties();

	private AccessSignBankActionListener accessSignBankListener = null;

	private GlossChainedEventsEntity currentGlossEntity;
	private boolean currentPartipantRightHanded;

	private String dominantHand;

	public AssociatedFieldsPanel(GlossFieldWrapper wrapper, GlossNRelatedView view, JDialog parent) {
		this.wrapper = wrapper;
		this.view = view;
		this.parent = parent;
		this.processor = new HandShapePreprocessor(view.getSegment());

		enter.setActionCommand("" + ENTER);
		cancel.setActionCommand("" + CANCEL);
		bypass.setActionCommand("" + ENTER_BYPASS);
		addGlossFieldDocumentListener();

		enter.addActionListener(new WeakActionListener(this, enter));
		cancel.addActionListener(new WeakActionListener(this, cancel));
		bypass.addActionListener(new WeakActionListener(this, bypass));

		searchSignBank.addActionListener(this);
		searchSignBank.setActionCommand("" + SIGN_BANK);

		addNewSign.addActionListener(this);
		addNewSign.setActionCommand("" + ADD_NEW);

		ButtonGroup domHandGrp = new ButtonGroup();
		domHandGrp.add(domHandR);
		domHandGrp.add(domHandL);

		domHandR.setEnabled(false);
		domHandL.setEnabled(false);

		update(wrapper, view);


		initUI();
	}

	private void addGlossFieldDocumentListener() {
		variantField.getDocument().addDocumentListener(new DocumentListener() {
			@Override
			public void insertUpdate(DocumentEvent e) {
				changed();
			}
			@Override
			public void removeUpdate(DocumentEvent e) {
				changed();
			}
			@Override
			public void changedUpdate(DocumentEvent e) {
			}

			public void changed() {
				if (variantField.getText().trim().isEmpty())
					enter.setEnabled(false);
				else
					enter.setEnabled(true);
			}
		});
	}

	public void showTab(int tabNum) {
		if (tabbedPane != null) {
			tabbedPane.setSelectedIndex(tabNum);
		}
	}

	private void populateHandshapeFormBean(GlossChainedEventsEntity entity) {
		// retrieve selected handshapes for right and left hands
		HandShapeEntity hshEntity1 = entity.getDependentHandShapeEntity();
		// 1000, 2000
		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		ArrayList<HandShapeImage> selectedHandImagesRight = new ArrayList<HandShapeImage>();
		ArrayList<HandShapeImage> selectedHandImagesLeft = new ArrayList<HandShapeImage>();

		if (hshEntity1 != null) {
			String fID = hshEntity1.getField().getFieldID();
			SS3Field field = cs.getField(fID);
			ArrayList<HandShapeEvent> hShapes = hshEntity1.getHandShapeEvents();

			for (HandShapeEvent event : hShapes) {
				SS3FieldValue value = field.getFieldValue(event.getID());
				HandShapeImage image = new HandShapeImage(value, cs);
				if (currentPartipantRightHanded) {
					selectedHandImagesRight.add(image);
				} else {
					selectedHandImagesLeft.add(image);
				}
			}
		}

		GlossChainedEventsEntity ndEntity = entity.getDependentGlossEntity();

		if (ndEntity != null && ndEntity.getDependentHandShapeEntity() != null) {
			HandShapeEntity hshEntity2 = ndEntity.getDependentHandShapeEntity();
			String ndfID = hshEntity2.getField().getFieldID();

			SS3Field ndfield = cs.getField(ndfID);
			ArrayList<HandShapeEvent> ndHShapes = hshEntity2.getHandShapeEvents();

			for (HandShapeEvent event : ndHShapes) {
				SS3FieldValue value = ndfield.getFieldValue(event.getID());
				HandShapeImage image = new HandShapeImage(value, cs);
				if (currentPartipantRightHanded) {
					selectedHandImagesLeft.add(image);
				} else {
					selectedHandImagesRight.add(image);
				}
			}
		}

		if (wrapper.getGlossField().getFieldID().equals(Util.DOM_GLOSS_FIELD_ID)) {
			if (isDominantHandRight()) {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesRight);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesLeft);
			} else {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesLeft);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesRight);
			}
		} else {
			if (isDominantHandRight()) {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesLeft);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesRight);
			} else {
				hshFormBean.setSelectedDomHandShapeImages(selectedHandImagesRight);
				hshFormBean.setSelectedNdomHandShapeImages(selectedHandImagesLeft);
			}
		}
	}

	protected void update(GlossFieldWrapper wrapper, GlossNRelatedView glsView) {

		this.wrapper = wrapper;
		this.view = glsView;

		loadedProperties.reset();

		EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

		if (ent == null)
			return;

		currentGlossEntity = (GlossChainedEventsEntity) ent;
		SS3GlossWindowItem windowItem = currentGlossEntity.getParentEntity().getSS3GlossWindowItem();
		SS3TemporalPartition partition = currentGlossEntity.getSignStreamSegmentPanel().getSS3Database()
				.getSS3GlossWindow().getTemporalPartitionByLabel(windowItem.getPartitionLabel());
		SS3GlossWindowSegment segment = partition.getTemporalPartitionSegmentByLabel(windowItem.getSegmentLabel());
		currentPartipantRightHanded = segment.getParticipant() == null || segment.getParticipant().isDominantHandRight();

		dominantHand = currentPartipantRightHanded ? "R" : "L";
		if (currentPartipantRightHanded) {
			domHandR.setSelected(true);
		} else {
			domHandL.setSelected(true);
		}

		hshFormBean = new HandShapeFormBean();
		processor.setRightHandDominant(currentPartipantRightHanded);
		processor.populateFormBean(hshFormBean);
		hshFormBean.setGlsFieldWrapper(wrapper);

		populateHandshapeFormBean(currentGlossEntity);

		GlossChainedEvent event = currentGlossEntity.getGlossChainedEvent();

		variantField.setText(event.getText().trim());

		primaryEntryField.setText(null);

		String fieldID = currentGlossEntity.getField().getFieldID();

		GlossEntityProperties properties = currentGlossEntity.getGlossEntityProperties();

		ArrayList<HandShapeImage> rhList = new ArrayList<HandShapeImage>(hshFormBean.getSelectedDomHandShapeImages());
		ArrayList<HandShapeImage> lhList = new ArrayList<HandShapeImage>(hshFormBean.getSelectedNdomHandShapeImages());

		if((rhList.size() > 0 && lhList.size() == 0) || (rhList.size() == 0 && lhList.size() > 0)) {
			properties.setTwoHanded(false);
		}

		if (currentPartipantRightHanded) {
			properties.setDominamtHand("R");
		} else {
			properties.setDominamtHand("L");
		}

		if(hshPanel == null) {
			this.hshPanel = new HandShapePanel(this);
		}

		if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
			// if dominant hand right (right handed signer), check right hand;
			// otherwise check left
			if (currentPartipantRightHanded) {
				properties.setOnehandedSignHand("R");
			} else {
				properties.setOnehandedSignHand("L");
			}
		} else {
			if (currentPartipantRightHanded) {
				properties.setOnehandedSignHand("L");
			} else {
				properties.setOnehandedSignHand("R");
			}
		}

		int entityStartTime = currentGlossEntity.getStartTimeInfo().getMovieTime();
		int entityEndTime = currentGlossEntity.getEndTimeInfo().getMovieTime();
		if (entityStartTime == entityEndTime) {
			properties.setOneFrameGloss(true);
		} else {
			properties.setOneFrameGloss(false);
		}

		FieldWrappersCollection collection = view.getSegment()
				.getFieldWrappersCollection();
		GlossFieldWrapper ndWrp;

		if (properties.getClassifier() == null) {

			boolean rhEmpty = rhList.isEmpty();
			boolean lhEmpty = lhList.isEmpty();

			if (rhEmpty && !lhEmpty) {
				properties.setTwoHanded(false);
				boolean rIdentical = isIdenticalStartEndHandshapes(lhList);

				if (rIdentical) {
					properties
							.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties
							.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}

			} else {
				if (!rhEmpty && lhEmpty) {
					properties.setTwoHanded(false);
					boolean lIdentical = isIdenticalStartEndHandshapes(rhList);

					if (lIdentical) {
						properties
								.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					} else {
						properties
								.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					}
				} else if (!rhEmpty && !lhEmpty) {
					properties.setTwoHanded(true);

					boolean isIdenticalLeftRight = isIdenticalHandshapes(rhList, lhList);

					if (isIdenticalLeftRight) {
						properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
						boolean lIdentical = isIdenticalStartEndHandshapes(rhList);

						if (lIdentical) {
							properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						} else {
							properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
						}
					} else {
						properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);

						if (isIdenticalStartEndHandshapes(lhList)) {
							properties
									.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						} else {
							properties
									.setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
						}

						if (isIdenticalStartEndHandshapes(rhList)) {
							properties
									.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
						} else {
							properties
									.setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
						}
					}

					properties.setTwoHandedEnabled(true);

					String gls = currentGlossEntity.getText();

					String handednessPattern = "(!)?(neu:|([a-z_1-3][p:]))?((.)([12][h])(.))(alt.)?(.*)?";
					Pattern.compile(handednessPattern);
					Pattern handednessR = Pattern.compile(handednessPattern);
					Matcher handednessMatcher = handednessR.matcher(gls);

					// handedness
					if (handednessMatcher.find()) {

						String oneHandedPattern = "(!)?(neu:)?((1h)){1}(.*)?";
						Pattern oneHandedR = Pattern.compile(oneHandedPattern);
						Matcher oneHandedMatcher = oneHandedR.matcher(gls);

						if (oneHandedMatcher.find()) {
							properties.setTwoHanded(false);

						} else {
							properties.setTwoHanded(true);

							String twoHandedAltPattern = "(!)?(neu:)?((2h).alt)(.*)?";
							Pattern twoHandedAltR = Pattern
									.compile(twoHandedAltPattern);
							Matcher twoHandedAltMatcher = twoHandedAltR
									.matcher(gls);

							if (twoHandedAltMatcher.find()) {
								properties
										.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
							}
						}
					}
				}
			}

			String gls = currentGlossEntity.getText();
			// name sign
			String nameSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-){1}(.*)?";
			Pattern nameSignR = Pattern.compile(nameSignPattern);
			Matcher nameSignMatcher = nameSignR.matcher(gls);

			if (nameSignMatcher.find()) {
				hshPanel.setNameSign(true);
			} else {
				hshPanel.setNameSign(false);
			}

			String fingerspelledPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(fs-){1}(.*)?";
			Pattern fingerspelledR = Pattern.compile(fingerspelledPattern);
			Matcher fingerspelledMatcher = fingerspelledR.matcher(gls);

			if (fingerspelledMatcher.find()) {
				hshPanel.setFingerspelled(true);
				currentSignType = SignType.FINGERSPELLED;
			} else {
				// loan sign
				String loanSignPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(#){1}(.*)?";
				Pattern loanSignR = Pattern.compile(loanSignPattern);
				Matcher loanSignMatcher = loanSignR.matcher(gls);

				if (loanSignMatcher.find()) {
					hshPanel.setLoan(true);
					currentSignType = SignType.LOAN;
				} else {
					String classifierPattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-)?((CL)|(DCL)|(LCL)|(SCL)|(BCL)|(BPCL)|(PCL)|(ICL)){1}([-_:].*)?$";
					Pattern classifierR = Pattern.compile(classifierPattern);
					Matcher classifierMatcher = classifierR.matcher(gls);

					if (classifierMatcher.find()) {
						hshPanel.setClassifier(true);
						currentSignType = SignType.CLASSIFIER;
					} else {

						String gesturePattern = "(!)?(neu:|([a-z_1-3][p:]))?([(1h)][(2h)][(2h)alt.])?(ns-)?((\"){1}(.){1,}(\"){1})(.*)?";
						Pattern gestureR = Pattern.compile(gesturePattern);
						Matcher gestureMatcher = gestureR.matcher(gls);

						if (gestureMatcher.find()) {
							hshPanel.setGesture(true);
							currentSignType = SignType.GESTURE;
						} else {
							hshPanel.setLexical(true);
							currentSignType = SignType.LEXICAL;

							if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
								ndWrp = (GlossFieldWrapper) collection
										.getFieldWrapperWithFieldID(Util.NDOM_GLOSS_FIELD_ID);
							} else {
								ndWrp = (GlossFieldWrapper) collection
										.getFieldWrapperWithFieldID(Util.DOM_GLOSS_FIELD_ID);
							}
						}
					}
				}
			}
		} else {
			hshPanel.setFingerspelled(properties.isFingerspelled());
			hshPanel.setLoan(properties.isLoan());
			hshPanel.setNumber(properties.isNumber());
			hshPanel.setLexical(properties.isLexical());
			hshPanel.setClassifier(properties.isClassifier());
			hshPanel.setGesture(properties.isGesture());
			hshPanel.setNameSign(properties.isNameSign());

			if (properties.isFingerspelled()) {
				currentSignType = SignType.FINGERSPELLED;
			} else if (properties.isLoan()) {
				currentSignType = SignType.LOAN;
			} else if (properties.isLexical()) {
				currentSignType = SignType.LEXICAL;
			} else if (properties.isClassifier()) {
				currentSignType = SignType.CLASSIFIER;
			} else if (properties.isGesture()) {
				currentSignType = SignType.GESTURE;
			} else if (properties.isNumber()) {
				currentSignType = SignType.NUMBER;
			}

			if (fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(Util.NDOM_GLOSS_FIELD_ID);
			} else {
				ndWrp = (GlossFieldWrapper) collection
						.getFieldWrapperWithFieldID(Util.DOM_GLOSS_FIELD_ID);
			}

			NonManualField ndField = (NonManualField) ndWrp.getField();
			boolean isPossibleToAddEvent = ndField.isEmptyInterval(currentGlossEntity
																		   .getStartTimeInfo().getMovieTime(), currentGlossEntity.getEndTimeInfo()
																		   .getMovieTime());

			if (!isPossibleToAddEvent) {
				String correspondingID = currentGlossEntity.getCorrespondingGlossEntityID();
				EventsEntity correspondingEntity = ndField
						.getEntity(correspondingID);
				if (correspondingEntity != null) {
					isPossibleToAddEvent = true;
				}
			}

			properties.setTwoHandedEnabled(isPossibleToAddEvent);

		}

		hshFormBean.setGlossEntityProperties(properties);

		if (this.hshPanel != null) {
			this.hshPanel.update();
		}

		checkSignTypeAndUpdateSignBankFunctionality();

		// populate LoadedProperties object 
		loadedProperties.setGloss(variantField.getText().trim());
		loadedProperties.setFingerspelled(hshPanel.isFingerspelled());
		loadedProperties.setLoan(hshPanel.isLoan());
		loadedProperties.setNumber(hshPanel.isNumber());
		loadedProperties.setLexical(hshPanel.isLexical());
		loadedProperties.setClassifier(hshPanel.isClassifier());
		loadedProperties.setGesture(hshPanel.isGesture());
		loadedProperties.setNameSign(hshPanel.isNameSign());

		loadedProperties.setTwoHanded(properties.isTwoHanded());
		loadedProperties.setMarkedNumberOfHands(properties.isMarkedNumberOfHands());
		loadedProperties.setPassiveBaseArm(properties.isPassiveBaseArm());
		loadedProperties.setOneFrameGloss(properties.isOneFrameGloss());

		if(hshPanel != null){
			loadedProperties.setSameDifStartEndHandshapes(hshPanel.getSelectedSameDifStartEndHandshapes());
			loadedProperties.setSameDIfferentLeftRightHandshapes(hshPanel.getSelectedSameDifLeftRightHandshapes());
			loadedProperties.setSameDifStartEndHandshapesL(hshPanel.getSelectedSameDifStartEndHandshapesL());
			loadedProperties.setSameDifStartEndHandshapesR(hshPanel.getSelectedSameDifStartEndHandshapesR());
		}

		loadedProperties.setRightHandSelectedHandshapes(hshFormBean.getSelectedDomHandShapeImages());
		loadedProperties.setLeftHandSelectedHandshapes(hshFormBean.getSelectedNdomHandShapeImages());

		if (locationPanel == null) {
			locationPanel = new LocationPanel(this);
		}

		locationPanel.resetDialogs();
		locationPanel.setCurrentLocationInfo(properties.getLocationInfo().cloneInfo());

		SignBankCollection coll = SS3Singleton.getSignStreamApplication().getSignBankCollection();
		enter.setEnabled(coll.contains(loadedProperties.getGloss()) != null);

		if (tabbedPane != null) {
			showTab(TAB_HANDSHAPES);
		}

		revalidate();
		parent.pack();
	}

	public HandShapeFormBean getHandShapeFormBean() {
		return hshFormBean;
	}

	private boolean isIdenticalHandshapes(ArrayList<HandShapeImage> rhList,
										  ArrayList<HandShapeImage> lhList) {
		if (rhList.size() == lhList.size()) {
			for (int i = 0; i < rhList.size(); i++) {
				HandShapeImage rImg = rhList.get(i);
				HandShapeImage lImg = lhList.get(i);
				String rId = rImg.getSS3FieldValue().getId();
				String lId = lImg.getSS3FieldValue().getId();

				if (!rId.equalsIgnoreCase(lId)) {
					return false;
				}
			}
			return true;
		} else {
			return false;
		}
	}

	private boolean isIdenticalStartEndHandshapes(ArrayList<HandShapeImage> list) {
		if (list.isEmpty()) {
			return true;
		}

		HandShapeImage fImg = list.get(0);
		HandShapeImage lImg = list.get(list.size() - 1);

		String rId = fImg.getSS3FieldValue().getId();
		String lId = lImg.getSS3FieldValue().getId();

		return rId.equalsIgnoreCase(lId);

	}

	private JPanel createGlossPanel() {
		JPanel panel = new JPanel(new GridBagLayout());

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		panel.add(primaryEntryLb, c);

		c.gridx++;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 2;
		panel.add(primaryEntryField, c);

		c.gridx = 0;
		c.gridy++;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = 1;
		panel.add(variantLb, c);

		c.gridx++;
		c.weightx = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 2;
		panel.add(variantField, c);

		c.gridx += 2;
		c.weightx = 0;
		c.fill = GridBagConstraints.NONE;
		c.gridwidth = 1;
		panel.add(searchSignBank, c);


		c.gridx = 0;
		c.gridy++;
		panel.add(domHandLbl, c);

		c.gridx++;
		c.insets = new Insets(0, 15, 0, 0);
		panel.add(domHandR, c);

		c.gridx++;
		c.insets = new Insets(0, 0, 0, 30);
		panel.add(domHandL, c);

		c.gridx++;
		c.insets = new Insets(0, 0, 0, 0);
		panel.add(addNewSign, c);

		panel.setBorder(new EmptyBorder(5, 5, 5, 5));
		return panel;
	}

	private JPanel createPosPanel() {
		JPanel panel = new JPanel();

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		panel.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(0, 0, 10, 0);

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(cancel);
		buttonPanel.add(enter);
		buttonPanel.add(bypass);

		gbc.anchor = GridBagConstraints.SOUTH;
		gbc.insets = new Insets(15, 0, 0, 0);
		gbc.fill = GridBagConstraints.HORIZONTAL;
		panel.add(buttonPanel, gbc);

		return panel;
	}

	private JPanel createCenterPanel() {
		JPanel panel = new JPanel();

		OrientationPanel oPanel = new OrientationPanel(new OrientationBean());
		MovementPanel mPanel = new MovementPanel(new MovementFormBean());

		tabbedPane = new JTabbedPane();
		tabbedPane.addTab("Sign Type & Handshapes", hshPanel);
		tabbedPane.addTab("Location", locationPanel);
		tabbedPane.addTab("<html><p style='color: gray;'>Orientation</p></html>", oPanel);
		tabbedPane.addTab("<html><p style='color: gray;'>Movement</p></html>", mPanel);

		tabbedPane.setEnabledAt(2, false);
		tabbedPane.setEnabledAt(3, false);

		tabbedPane.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				int newTabIndex = ((JTabbedPane)e.getSource()).getSelectedIndex();
				boolean enable = newTabIndex == 0;
				addNewSign.setEnabled(enable);
				enter.setEnabled(enable);

				locationPanel.resetDialogs();
			}
		});

		panel.setLayout(new BorderLayout());

		panel.add(tabbedPane, BorderLayout.CENTER);

		return panel;
	}

	private void initUI() {
		JPanel glsPanel = createGlossPanel();
		JPanel posPanel = createPosPanel();
		JPanel cPanel = createCenterPanel();

		setLayout(new BorderLayout(0, 10));
		add(glsPanel, BorderLayout.NORTH);

		Dimension size = new Dimension(600, 500);
		cPanel.setMinimumSize(size);
		cPanel.setPreferredSize(size);
		add(cPanel, BorderLayout.CENTER);

		add(posPanel, BorderLayout.SOUTH);

//		GridBagConstraints gbc = new GridBagConstraints();
//		setLayout(new GridBagLayout());
//
//		gbc.anchor = GridBagConstraints.NORTHWEST;
//		gbc.gridwidth = GridBagConstraints.REMAINDER;
//		gbc.fill = GridBagConstraints.HORIZONTAL;
//		gbc.insets = new Insets(0, 0, 10, 0);
//		add(glsPanel, gbc);
//
//		gbc.anchor = GridBagConstraints.NORTH;
//		gbc.fill = GridBagConstraints.HORIZONTAL;
//		gbc.insets = new Insets(0, 0, 10, 0);
//		add(new JSeparator(), gbc);
//
//		gbc.anchor = GridBagConstraints.NORTHWEST;
//		gbc.gridwidth = GridBagConstraints.REMAINDER;
//		add(cPanel, gbc);
//
//		gbc.anchor = GridBagConstraints.NORTH;
//		gbc.fill = GridBagConstraints.HORIZONTAL;
//		gbc.insets = new Insets(0, 0, 10, 0);
//		add(posPanel, gbc);
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
		removeAll();
		initUI();
		revalidate();
		parent.pack();
	}

	public boolean isChanged() {

		String glsText = variantField.getText().trim();
		if (!glsText.equalsIgnoreCase(loadedProperties.getGloss())) {
			return true;
		}

		if(hshPanel.isFingerspelled() != loadedProperties.isFingerspelled()) {
			return true;
		}

		if(hshPanel.isLoan() != loadedProperties.isLoan()) {
			return true;
		}

		if(hshPanel.isNumber() != loadedProperties.isNumber()) {
			return true;
		}

		if(hshPanel.isLexical() != loadedProperties.isLexical()) {
			return true;
		}

		if(hshPanel.isClassifier() != loadedProperties.isClassifier()) {
			return true;
		}

		if(hshPanel.isGesture() !=  loadedProperties.isGesture()) {
			return true;
		}

		if(hshPanel.isNameSign() != loadedProperties.isNameSign()) {
			return true;
		}

		if(hshPanel.isPassiveBaseArm() != loadedProperties.isPassiveBaseArm()) {
			return true;
		}

		if(hshPanel.isMarkedNumberOfHands() != loadedProperties.isMarkedNumberOfHands()) {
			return true;
		}

		if(hshPanel.isTwoHandedSelected() != loadedProperties.isTwoHanded()) {
			return true;
		}

		LeftRightHandshapesType sameDifHandshapes = hshPanel.getSelectedSameDifLeftRightHandshapes();
		StartEndHandshapesType sameDifStartEndHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
		StartEndHandshapesType sameDifStartEndHandshapesL = hshPanel.getSelectedSameDifStartEndHandshapesL();
		StartEndHandshapesType sameDifStartEndHandshapesR = hshPanel.getSelectedSameDifStartEndHandshapesR();

		LeftRightHandshapesType loadedSameDifHandshapes = loadedProperties.getSameDIfferentLeftRightHandshapes();
		StartEndHandshapesType loadedSameDifStartEndHandshapes = loadedProperties.getSameDifStartEndHandshapes();
		StartEndHandshapesType loadedSameDifStartEndHandshapesR = loadedProperties.getSameDifStartEndHandshapesR();
		StartEndHandshapesType loadedSameDifStartEndHandshapesL = loadedProperties.getSameDifStartEndHandshapesL();

		
		if(sameDifHandshapes != loadedSameDifHandshapes) {
			return true;
		}

		if(sameDifStartEndHandshapes != loadedSameDifStartEndHandshapes) {
			return true;
		}

		if(sameDifStartEndHandshapesL != loadedSameDifStartEndHandshapesL) {
			return true;
		}

		if(sameDifStartEndHandshapesR != loadedSameDifStartEndHandshapesR) {
			return true;
		}

		ArrayList<HandShapeImage> selLeftHandShapes = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		ArrayList<HandShapeImage> selRightHandShapes = hshPanel.getSelectedRightHandshapesPane().getHandshapes();

		ArrayList<HandShapeImage> selLeftHandShapes1 = loadedProperties.getLeftHandSelectedHandshapes();
		ArrayList<HandShapeImage> selRightHandShapes1 = loadedProperties.getRightHandSelectedHandshapes();

		if(selLeftHandShapes.isEmpty() != selLeftHandShapes1.isEmpty()) {
			return true;
		}

		if(selRightHandShapes.isEmpty() != selRightHandShapes1.isEmpty()) {
			return true;
		}

		int rSize1 = selRightHandShapes1.size();
		int rSize = selRightHandShapes.size();

		if(rSize1 != rSize) {
			if(rSize1 == 2 && rSize == 1) {
				HandShapeImage rHandShape11 = selRightHandShapes1.get(0);
				HandShapeImage rHandShape12 = selRightHandShapes1.get(1);
				HandShapeImage rHandShape = selRightHandShapes.get(0);
				String rHandShape11ID = rHandShape11.getSS3FieldValue().getId();
				String rHandShape12ID = rHandShape12.getSS3FieldValue().getId();
				if(!rHandShape11ID.equalsIgnoreCase(rHandShape12ID)) {
					return true;
				}
				String rHandShapeID = rHandShape.getSS3FieldValue().getId();
				if(!rHandShape11ID.equalsIgnoreCase(rHandShapeID)) {
					return true;
				}
			} else {
				return true;
			}
		}

		int i = 0 ;
		for(HandShapeImage rHandShape : selRightHandShapes) {
			HandShapeImage rHandShape1 = selRightHandShapes1.get(i);
			String rhShapeID = rHandShape.getSS3FieldValue().getId();
			String rhShapeID1 = rHandShape1.getSS3FieldValue().getId();
			if(!rhShapeID.equalsIgnoreCase(rhShapeID1)) {
				return true;
			}
			i++;
		}

		int lSize1 = selLeftHandShapes1.size();
		int lSize = selLeftHandShapes.size();

		if(lSize1 != lSize) {
			if(lSize1 == 2 && lSize == 1) {
				HandShapeImage lHandShape11 = selLeftHandShapes1.get(0);
				HandShapeImage lHandShape12 = selLeftHandShapes1.get(1);
				HandShapeImage lHandShape = selLeftHandShapes.get(0);
				String lHandShape11ID = lHandShape11.getSS3FieldValue().getId();
				String lHandShape12ID = lHandShape12.getSS3FieldValue().getId();
				if(!lHandShape11ID.equalsIgnoreCase(lHandShape12ID)) {
					return true;
				}
				String lHandShapeID = lHandShape.getSS3FieldValue().getId();
				if(!lHandShape11ID.equalsIgnoreCase(lHandShapeID)) {
					return true;
				}
			} else {
				return true;
			}
		}

		int j = 0 ;
		for(HandShapeImage lHandShape : selLeftHandShapes) {
			HandShapeImage lHandShape1 = selLeftHandShapes1.get(j);
			String lhShapeID = lHandShape.getSS3FieldValue().getId();
			String lhShapeID1 = lHandShape1.getSS3FieldValue().getId();
			if(!lhShapeID.equalsIgnoreCase(lhShapeID1)) {
				return true;
			}
			j++;
		}

		return false;
	}

	public void saveChanges() {
		updateChanges();
		SS3Singleton.getApplicationStateController()
				.setLoadedCollectionUpdated(true);
	}

	public void saveChanges(SignBankVariant variant) {
		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		GlossChainedEventsEntity sEntity = (GlossChainedEventsEntity) mediaToolBar.getSelectedEventsEntity();

		if (sEntity == null) {
			sEntity = wrapper.getGlossField()
					.getGlossChainedEventsEntityCollection()
					.getSelectedEntity();
		}

		GlossEntityProperties properties = sEntity.getGlossEntityProperties();

		properties.setVerifiedFromSignBank(true);

		// compare against the defaultOccurrence - from which we get the source resource
		SignBankOccurrence defaultOccurrence = variant.getDefaultOccurrence();
		String sourceResource = defaultOccurrence.getSIgnBankResourceFileName();
		properties.setSourceResource(sourceResource);

		String mainGlossID = variant.getParentMainGloss().getMainGlossID();
		String variantID = variant.getVariantID();
		String occurrenceID = defaultOccurrence.getOccurrenceID();
		properties.setMainGlossID(mainGlossID);
		properties.setVariantID(variantID);
		properties.setOccurrenceID(occurrenceID);

		ArrayList<HandShapeImage> domImages, nondomImages;
		int domImagesSize, nondomImagesSize;

		// TODO: FIX
		if (isDominantHandRight()) {
			domImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		} else {
			domImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		}
		domImagesSize = domImages.size();
		nondomImagesSize = nondomImages.size();

		String domStartHS = null;
		String nondomStartHS = null;
		String domEndHS = null;
		String nondomEndHS = null;

		if (domImagesSize > 0) {
			domStartHS = domImages.get(0).getSS3FieldValue().getLabel();
			domEndHS = domImages.get(domImagesSize - 1).getSS3FieldValue().getLabel();
		}

		if (nondomImagesSize > 0) {
			nondomStartHS = nondomImages.get(0).getSS3FieldValue().getLabel();
			nondomEndHS = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue().getLabel();
		}

		String occDomStartHS = defaultOccurrence.getDomStartHS();
		String occNondomStartHS = defaultOccurrence.getNondomStartHS();
		String occDomEndHS = defaultOccurrence.getDomEndHS();
		String occNondomEndHS = defaultOccurrence.getNondomEndHS();

		String occLeftRightHS = defaultOccurrence.getLeftRightHS();
		String occStartEndHS = defaultOccurrence.getStartEndHS();
		String occStartEndLeftHS = defaultOccurrence.getStartEndLeftHS();
		String occStartEndRightHS = defaultOccurrence.getStartEndRightHS();

		boolean occTwoHanded = defaultOccurrence.isTwoHanded();
		boolean occMarkedHands = defaultOccurrence.isMarkedHands();
		boolean occPassiveBaseArm = defaultOccurrence.isPassiveBaseArm();

		String occSignType = defaultOccurrence.getSignType();

		boolean differingFromOccurrence = false;

		if (occDomStartHS != null && domStartHS != null && !occDomStartHS.equals(domStartHS))
			differingFromOccurrence = true;
		if (occNondomStartHS != null && nondomStartHS != null && !occNondomStartHS.equals(nondomStartHS))
			differingFromOccurrence = true;
		if (occDomEndHS != null && domEndHS != null && !occDomEndHS.equals(domEndHS))
			differingFromOccurrence = true;
		if (occNondomEndHS != null && nondomEndHS != null && !occNondomEndHS.equals(nondomEndHS))
			differingFromOccurrence = true;
		
		LeftRightHandshapesType leftRightHS = hshPanel.getSelectedSameDifLeftRightHandshapes();
		StartEndHandshapesType startEndHS = hshPanel.getSelectedSameDifStartEndHandshapes();
		StartEndHandshapesType startEndLeftHS = hshPanel.getSelectedSameDifStartEndHandshapesL();
		StartEndHandshapesType startEndRightHS = hshPanel.getSelectedSameDifStartEndHandshapesR();
		
		boolean twoHanded = hshPanel.isTwoHandedSelected();
		boolean markedHands = hshPanel.isMarkedNumberOfHands();
		boolean passiveBaseArm = hshPanel.isPassiveBaseArm();

		String signType = null;
		if (hshPanel.isLexical())
			signType = "LEXICAL";
		if (hshPanel.isFingerspelled())
			signType = "FINGERSPELLED";
		if (hshPanel.isLoan())
			signType = "LOAN";
		if (hshPanel.isNumber())
			signType = "NUMBER";
		if (hshPanel.isClassifier())
			signType = "CLASSIFER";
		if (hshPanel.isGesture())
			signType = "GESTURE";

		if (occLeftRightHS != null) {
			if (occLeftRightHS.equals("SAME") && leftRightHS != LeftRightHandshapesType.SAME) {
				differingFromOccurrence = true;

				if (occStartEndHS != null) {
					if (occStartEndHS.equals("SAME") && startEndHS != StartEndHandshapesType.SAME)
						differingFromOccurrence = true;
					else if (occStartEndHS.equals("DIFFERENT") && startEndHS != StartEndHandshapesType.DIFFERENT)
						differingFromOccurrence = true;
				}
			} else if (occLeftRightHS.equals("DIFFERENT") && leftRightHS != LeftRightHandshapesType.DIFFERENT) {
				differingFromOccurrence = true;

				if (occStartEndLeftHS != null) {
					if (occStartEndLeftHS.equals("SAME") && startEndLeftHS != StartEndHandshapesType.SAME)
						differingFromOccurrence = true;
					else if (occStartEndLeftHS.equals("DIFFERENT") && startEndLeftHS != StartEndHandshapesType.DIFFERENT)
						differingFromOccurrence = true;
				}

				if (occStartEndRightHS != null) {
					if (occStartEndRightHS.equals("SAME") && startEndRightHS != StartEndHandshapesType.SAME)
						differingFromOccurrence = true;
					else if (occStartEndRightHS.equals("DIFFERENT") && startEndRightHS != StartEndHandshapesType.DIFFERENT)
						differingFromOccurrence = true;
				}
			}
		}

		if (occTwoHanded != twoHanded)
			differingFromOccurrence = true;

		if (occMarkedHands != markedHands)
			differingFromOccurrence = true;

		if (occPassiveBaseArm != passiveBaseArm)
			differingFromOccurrence = true;

		if ( !occSignType.equals(signType))
			differingFromOccurrence = true;

		properties.setDifferingFromOccurrence(differingFromOccurrence);

		// we need to set the same sign bank properties on the dependent (other hand) gloss entity
		// because on "file open" both hands (both SS3Track objects) will be need to be parsed separately
		// (there could be one handed signs on the nondominant hand - an unlikely but possible use case)
		GlossChainedEventsEntity dependentEntity = sEntity.getDependentGlossEntity();
		if (dependentEntity != null) {
			GlossEntityProperties dependentProperties = dependentEntity.getGlossEntityProperties();
			dependentProperties.setVerifiedFromSignBank(true);
			dependentProperties.setSourceResource(sourceResource);
			dependentProperties.setDifferingFromOccurrence(differingFromOccurrence);
			dependentProperties.setMainGlossID(mainGlossID);
			dependentProperties.setVariantID(variantID);
			dependentProperties.setOccurrenceID(occurrenceID);
		}

		updateChanges();
		SS3Singleton.getApplicationStateController()
				.setLoadedCollectionUpdated(true);
	}

	protected void updateChanges() {

		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);

		if (currentGlossEntity == null) {
			currentGlossEntity = wrapper.getGlossField()
					.getGlossChainedEventsEntityCollection()
					.getSelectedEntity();
			SS3GlossWindowItem windowItem = currentGlossEntity.getParentEntity().getSS3GlossWindowItem();
			SS3TemporalPartition partition = currentGlossEntity.getSignStreamSegmentPanel().getSS3Database()
					.getSS3GlossWindow().getTemporalPartitionByLabel(windowItem.getPartitionLabel());
			SS3GlossWindowSegment segment = partition.getTemporalPartitionSegmentByLabel(windowItem.getSegmentLabel());
			currentPartipantRightHanded = segment.getParticipant() == null || segment.getParticipant().isDominantHandRight();
		}

		GlossChainedEventsEntity depGlsEntity = currentGlossEntity.getDependentGlossEntity();
		GlossChainedEvent selectedTextEvent = (GlossChainedEvent) currentGlossEntity.getTextValue();
		GlossClassifications glsClassification = selectedTextEvent.getGlossClassifications();

		GlossEntityProperties properties = currentGlossEntity.getGlossEntityProperties();

		if(hshPanel.isFingerspelled()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_FINGERSPELLED);
		} else if(hshPanel.isLoan()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_LOAN);
		} else if(hshPanel.isNumber()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_NUMBERS);
		} else if(hshPanel.isLexical()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_LEXICAL);
		} else if(hshPanel.isClassifier()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_CLASSIFIER);
		} else if(hshPanel.isGesture()) {
			glsClassification.setSignClassifications(GlossClassifications.SIGN_GESTURE);
		}

		glsClassification.setNameSign(hshPanel.isNameSign());
		glsClassification.setOneFrameGloss(hshPanel.isOneFrameGloss());
		glsClassification.setPassiveBaseArm(hshPanel.isPassiveBaseArm());
		glsClassification.setMarkedNamberOfHands(hshPanel.isMarkedNumberOfHands());

		properties.setOneFrameGloss(hshPanel.isOneFrameGloss());

		if(hshPanel.isTwoHandedSelected()) {
			glsClassification.setNumberOfHands(2);
			properties.setTwoHanded(true);
		} else {
			glsClassification.setNumberOfHands(1);
			properties.setTwoHanded(false);
		}
		
		LeftRightHandshapesType handshapeClassification = hshPanel.getSelectedSameDifLeftRightHandshapes();
		boolean sameHandshapes = handshapeClassification == LeftRightHandshapesType.SAME;

		glsClassification.setSameHandshapes(sameHandshapes);

		if(sameHandshapes) {
			StartEndHandshapesType sameDiffStartEndHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
			glsClassification.setDomHandShapesSameDIfferent(sameDiffStartEndHandshapes);
			glsClassification.setNondomHandShapesSameDIfferent(sameDiffStartEndHandshapes);
		} else {
			StartEndHandshapesType sameDifStartEndHandshapesL = hshPanel.getSelectedSameDifStartEndHandshapesL();
			StartEndHandshapesType sameDifStartEndHandshapesR = hshPanel.getSelectedSameDifStartEndHandshapesR();
			
			if(isDominantHandRight()) {
				glsClassification.setDomHandShapesSameDIfferent(sameDifStartEndHandshapesR);
				glsClassification.setNondomHandShapesSameDIfferent(sameDifStartEndHandshapesL);
			} else {
				glsClassification.setDomHandShapesSameDIfferent(sameDifStartEndHandshapesL);
				glsClassification.setNondomHandShapesSameDIfferent(sameDifStartEndHandshapesR);
			}
		}

		String fieldID = null;

		Field field = SS3Singleton.getMediaToolBar(null).getSelectedField();
		if(field != null) {
			fieldID = field.getFieldID();
		}

		if(fieldID == null) {
			EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

			if (ent == null)
				return;

			GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;
			fieldID = entity.getField().getFieldID();
		}

		FieldWrappersCollection collection = view.getSegment().getFieldWrappersCollection();

		String newID = fieldID.equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID) ? Util.NDOM_GLOSS_FIELD_ID : Util.DOM_GLOSS_FIELD_ID;

		GlossFieldWrapper depGlsWrapper = (GlossFieldWrapper) collection.getFieldWrapperWithFieldID(newID);

		if (hshPanel.isTwoHandedSelected()) {
			if(depGlsEntity != null) {
				String text = currentGlossEntity.getText();
				depGlsEntity.getTextValue().setText(text);
				depGlsEntity.getGlossEntityProperties().setTwoHanded(true);
			} else {
				int startCoord = currentGlossEntity.getStartTimeCoordinate();
				int endCoord = currentGlossEntity.getEndTimeCoordinate();

				GlossField glsField = (GlossField) depGlsWrapper.getField();

				ArrayList<GlossChainedEventsEntity> entities = glsField
						.getEntitiesInInterval(startCoord, endCoord);

				boolean overlap = false;
				for (GlossChainedEventsEntity entity : entities) {
					int entityStart = entity.getStartTimeCoordinate();
					int entityEnd = entity.getEndTimeCoordinate();
					if (startCoord < entityEnd && endCoord > entityStart) {
						overlap = true;
						break;
					}
				}

				if (entities.isEmpty() || !overlap) {
					ChainedEventsCollection chainedEventsCollection = new ChainedEventsCollection();

					int startMovieTime = selectedTextEvent.getStartTimeInfo()
							.getMovieTime();
					TimeInfo ndStartTimeInfo = new TimeInfo();
					ndStartTimeInfo.setMovieTime(startMovieTime);

					int endMovieTime = selectedTextEvent.getEndTimeInfo()
							.getMovieTime();
					TimeInfo ndEndTimeInfo = new TimeInfo();
					ndEndTimeInfo.setMovieTime(endMovieTime);

					GlossChainedEvent ndTextEvent = new GlossChainedEvent(
							currentGlossEntity.getSignStreamSegmentPanel());
					ndTextEvent.setStartTimeInfo(ndStartTimeInfo);
					ndTextEvent.setEndTimeInfo(ndEndTimeInfo);
					ndTextEvent.setText(selectedTextEvent.getText());

					chainedEventsCollection.setTextValueEvent(ndTextEvent);
					depGlsEntity = new GlossChainedEventsEntity(
							chainedEventsCollection,
							currentGlossEntity.getSignStreamSegmentPanel(), glsField);
					depGlsEntity.determineParentEntity();
					depGlsEntity.setCorrespondingGlossEntityID(currentGlossEntity.getID());
					currentGlossEntity.setCorrespondingGlossEntityID(depGlsEntity.getID());
					currentGlossEntity.setDependentGlossEntity(depGlsEntity);
					depGlsEntity.setDependentGlossEntity(currentGlossEntity);
					depGlsEntity.getGlossEntityProperties().setTwoHanded(true);

					currentGlossEntity.setLockedLeftRightAlignments(true);
					depGlsEntity.setLockedLeftRightAlignments(true);
					depGlsWrapper.getGlossField().addGlossChainedEventsEntity(depGlsEntity);

					LocationEntity depLocEntity = currentGlossEntity.getDependentLocationEntity();
					depGlsEntity.setDependentLocationEntity(depLocEntity);

					if (depLocEntity != null) {
						LocationFieldWrapper domLocWrapper = (LocationFieldWrapper) collection.getFieldWrapperWithFieldID(Util.DOM_LOCATION_FIELD_ID);
						LocationFieldWrapper ndomLocWrapper = (LocationFieldWrapper) collection.getFieldWrapperWithFieldID(Util.NDOM_LOCATION_FIELD_ID);
						if (wrapper.getGlossField().getFieldID().equals(Util.NDOM_GLOSS_FIELD_ID)) {
							
							// GD: copying the below sign type block, but for locations - in case it is ever necessary
							if (domLocWrapper == null || ndomLocWrapper == null) {								
								ArrayList<LabelObject> arr = view.getLableView().getLabels();
								LabelObject domLocObj = null, ndomLocObj = null;
								for (LabelObject obj: arr) {
									if (obj.getFieldID().equals(Util.DOM_LOCATION_FIELD_ID))
										domLocObj = obj;
									else if (obj.getFieldID().equals(Util.NDOM_LOCATION_FIELD_ID))
										ndomLocObj = obj;
								}
								
								if (domLocObj != null)
									domLocWrapper = (LocationFieldWrapper) domLocObj.getWrapper();
								if (ndomLocObj != null)
									ndomLocWrapper = (LocationFieldWrapper) ndomLocObj.getWrapper();
							}
							
							ndomLocWrapper.getField().deleteEntity(depLocEntity);
							domLocWrapper.getField().addEventsEntity(depLocEntity);
						}
						depLocEntity.setRefGlossEntity(depGlsEntity);
					}

					SignTypeEntity depSignTypeEnt = currentGlossEntity.getDependentSignTypeEntity();
					depGlsEntity.setDependentSignTypeEntity(depSignTypeEnt);

					if (depSignTypeEnt != null) {
						SignTypeFieldWrapper domSignTypeWrapper = (SignTypeFieldWrapper) collection.getFieldWrapperWithFieldID(Util.DOM_SIGN_TYPE_FIELD_ID);
						SignTypeFieldWrapper ndomSignTypeWrapper = (SignTypeFieldWrapper) collection.getFieldWrapperWithFieldID(Util.NDOM_SIGN_TYPE_FIELD_ID);
						if (wrapper.getGlossField().getFieldID().equals(Util.NDOM_GLOSS_FIELD_ID)) {
							
							// GD: need to do this block because the ndomSignTypeWrapper is often null, and domSignTypeWrapper is just for sanity
							// we retrieve the null wrapper from its label object 
							if (domSignTypeWrapper == null || ndomSignTypeWrapper == null) {								
								ArrayList<LabelObject> arr = view.getLableView().getLabels();
								LabelObject domSignTypeObj = null, ndomSignTypeObj = null;
								for (LabelObject obj: arr) {
									if (obj.getFieldID().equals(Util.DOM_SIGN_TYPE_FIELD_ID))
										domSignTypeObj = obj;
									else if (obj.getFieldID().equals(Util.NDOM_SIGN_TYPE_FIELD_ID))
										ndomSignTypeObj = obj;
								}
								
								if (domSignTypeObj != null)
									domSignTypeWrapper = (SignTypeFieldWrapper) domSignTypeObj.getWrapper();
								if (ndomSignTypeObj != null)
									ndomSignTypeWrapper = (SignTypeFieldWrapper) ndomSignTypeObj.getWrapper();
							}
							
							ndomSignTypeWrapper.getField().deleteEntity(depSignTypeEnt);
							domSignTypeWrapper.getField().addEventsEntity(depSignTypeEnt);
						}
						depSignTypeEnt.setRefGlossEntity(depGlsEntity);
					}
				} else {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return;
				}
			}
		} else {
			// delete second gloss and related
			if (depGlsEntity != null) {
				depGlsWrapper.getGlossField().deleteEntity(depGlsEntity);

				ChainedEventsEntity depHandEntity = depGlsEntity.getDependentHandEntity();
				HandShapeEntity depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();

				RightLeftHandFieldWrapper depHandWrapper;

				if(depGlsWrapper.getGlossField().getFieldID().equalsIgnoreCase(Util.DOM_GLOSS_FIELD_ID)) {
					depHandWrapper = (RightLeftHandFieldWrapper)collection.getFieldWrapperWithFieldID(Util.DOM_HAND_FIELD_ID);
				} else {
					depHandWrapper = (RightLeftHandFieldWrapper)collection.getFieldWrapperWithFieldID(Util.NDOM_HAND_FIELD_ID);
				}

				if(depHandEntity != null) {
					NonManualField rl = (NonManualField)depHandWrapper.getField();
					rl.deleteEntity(depHandEntity);
				}

				if(depHandShapeEntity != null) {
					depHandShapeEntity.getField().deleteEntity(depHandShapeEntity);
				}

				LocationEntity depLocEntity = depGlsEntity.getDependentLocationEntity();
				if (depLocEntity != null) {
					LocationFieldWrapper domLocWrapper = (LocationFieldWrapper) collection.getFieldWrapperWithFieldID(Util.DOM_LOCATION_FIELD_ID);
					LocationFieldWrapper ndomLocWrapper = (LocationFieldWrapper) collection.getFieldWrapperWithFieldID(Util.NDOM_LOCATION_FIELD_ID);
					if (wrapper.getGlossField().getFieldID().equals(Util.NDOM_GLOSS_FIELD_ID)) {

						// GD: copying the above sign type block, but for locations - in case it is ever necessary
						if (domLocWrapper == null || ndomLocWrapper == null) {								
							ArrayList<LabelObject> arr = view.getLableView().getLabels();
							LabelObject domLocObj = null, ndomLocObj = null;
							for (LabelObject obj: arr) {
								if (obj.getFieldID().equals(Util.DOM_LOCATION_FIELD_ID))
									domLocObj = obj;
								else if (obj.getFieldID().equals(Util.NDOM_LOCATION_FIELD_ID))
									ndomLocObj = obj;
							}
							
							if (domLocObj != null)
								domLocWrapper = (LocationFieldWrapper) domLocObj.getWrapper();
							if (ndomLocObj != null)
								ndomLocWrapper = (LocationFieldWrapper) ndomLocObj.getWrapper();
						}
						
						domLocWrapper.getField().deleteEntity(depLocEntity);
						ndomLocWrapper.getField().addEventsEntity(depLocEntity);
					}
					depLocEntity.setRefGlossEntity(currentGlossEntity);
				}

				SignTypeEntity depSignTypeEntity = depGlsEntity.getDependentSignTypeEntity();
				if (depSignTypeEntity != null) {
					SignTypeFieldWrapper domSignTypeWrapper = (SignTypeFieldWrapper) collection.getFieldWrapperWithFieldID(Util.DOM_SIGN_TYPE_FIELD_ID);
					SignTypeFieldWrapper ndomSignTypeWrapper = (SignTypeFieldWrapper) collection.getFieldWrapperWithFieldID(Util.NDOM_SIGN_TYPE_FIELD_ID);
					if (wrapper.getGlossField().getFieldID().equals(Util.NDOM_GLOSS_FIELD_ID)) {

						// GD: copying the above sign type block
						if (domSignTypeWrapper == null || ndomSignTypeWrapper == null) {						
							ArrayList<LabelObject> arr = view.getLableView().getLabels();
							LabelObject domSignTypeObj = null, ndomSignTypeObj = null;
							for (LabelObject obj: arr) {
								if (obj.getFieldID().equals(Util.DOM_SIGN_TYPE_FIELD_ID))
									domSignTypeObj = obj;
								else if (obj.getFieldID().equals(Util.NDOM_SIGN_TYPE_FIELD_ID))
									ndomSignTypeObj = obj;
							}
							
							if (domSignTypeObj != null)
								domSignTypeWrapper = (SignTypeFieldWrapper) domSignTypeObj.getWrapper();
							if (ndomSignTypeObj != null)
								ndomSignTypeWrapper = (SignTypeFieldWrapper) ndomSignTypeObj.getWrapper();
						}
						
						domSignTypeWrapper.getField().deleteEntity(depSignTypeEntity);
						ndomSignTypeWrapper.getField().addEventsEntity(depSignTypeEntity);
					}
					depSignTypeEntity.setRefGlossEntity(currentGlossEntity);
				}

				depGlsEntity = null;
			}
		}

		GlossEntityProperties properties2 = null;

		if (depGlsEntity != null)
			properties2 = depGlsEntity.getGlossEntityProperties();

		if (hshPanel.isFingerspelled()) {
			properties.setFingerspelled();
			if (properties2 != null)
				properties2.setFingerspelled();
		} else if (hshPanel.isLoan()) {
			properties.setLoan();
			if (properties2 != null)
				properties2.setLoan();
		} else if (hshPanel.isNumber()) {
			properties.setNumber();
			if (properties2 != null)
				properties2.setNumber();
		} else if (hshPanel.isLexical()) {
			properties.setLexical();
			if (properties2 != null)
				properties2.setLexical();
		} else if (hshPanel.isClassifier()) {
			properties.setClassifier();
			if (properties2 != null)
				properties2.setClassifier();
		} else if (hshPanel.isGesture()) {
			properties.setGesture();
			if (properties2 != null)
				properties2.setGesture();
		}

		properties.setNameSign(hshPanel.isNameSign());
		properties.setDominamtHand(dominantHand);
		properties.setMarkedNumberOfHands(hshPanel.isMarkedNumberOfHands());
		properties.setPassiveBaseArm(hshPanel.isPassiveBaseArm());

		if (properties2 != null) {
			properties2.setNameSign(hshPanel.isNameSign());
			properties2.setDominamtHand(dominantHand);
			properties2
					.setMarkedNumberOfHands(hshPanel.isMarkedNumberOfHands());
			properties2.setPassiveBaseArm(hshPanel.isPassiveBaseArm());
		}

		String text = variantField.getText().trim();

		currentGlossEntity.getGlossChainedEvent().setText(text);

		if (depGlsEntity != null) {
			depGlsEntity.getGlossChainedEvent().setText(text);
		}

		HandShapeBean hspBean = hshPanel.getHandShapeBean();

		addOrUpdateHandShapes(hspBean, currentGlossEntity, collection);

		if (depGlsEntity != null) {
			Event event = currentGlossEntity.getValueOrTextValueEvent();
			ChainedEvent onset = currentGlossEntity.getOnset();
			ChainedEvent initialHold = currentGlossEntity.getInitialHold();
			ChainedEvent offset = currentGlossEntity.getOffset();
			ChainedEvent finalHold = currentGlossEntity.getFinalHold();

			Event ndEvent = depGlsEntity.getValueOrTextValueEvent();
			ChainedEvent ndOnset = depGlsEntity.getOnset();
			ChainedEvent ndInitialHold = depGlsEntity.getInitialHold();
			ChainedEvent ndOffset = depGlsEntity.getOffset();
			ChainedEvent ndFinalHold = depGlsEntity.getFinalHold();

			if (currentGlossEntity.isLockedLeftRightAlignments() && depGlsEntity.isLockedLeftRightAlignments()) {
				ndEvent.getStartTimeInfo().setMovieTime(
						event.getStartTimeInfo().getMovieTime());
				ndEvent.getEndTimeInfo().setMovieTime(
						event.getEndTimeInfo().getMovieTime());

				SignStreamSegmentPanel panel = wrapper.getField()
						.getSignStreamSegmentPanel();

				if (initialHold != null) {
					if (ndInitialHold != null) {
						ndInitialHold.getStartTimeInfo().setMovieTime(
								initialHold.getStartTimeInfo().getMovieTime());
						ndInitialHold.getEndTimeInfo().setMovieTime(
								initialHold.getEndTimeInfo().getMovieTime());
					} else {
						ndInitialHold = new ChainedEvent(panel);
						ndInitialHold.setStartTimeInfo(initialHold
															   .getStartTimeInfo().cloneTimeInfo());
						ndInitialHold.setEndTimeInfo(initialHold.getEndTimeInfo()
															 .cloneTimeInfo());
						depGlsEntity.setInitialHoldEvent(ndInitialHold);
					}
				} else {
					depGlsEntity.setInitialHoldEvent(null);
				}

				if (onset != null) {
					if (ndOnset != null) {
						ndOnset.getStartTimeInfo().setMovieTime(
								onset.getStartTimeInfo().getMovieTime());
						ndOnset.getEndTimeInfo().setMovieTime(
								onset.getEndTimeInfo().getMovieTime());
					} else {
						ndOnset = new ChainedEvent(panel);
						ndOnset.setStartTimeInfo(onset.getStartTimeInfo()
														 .cloneTimeInfo());
						ndOnset.setEndTimeInfo(onset.getEndTimeInfo()
													   .cloneTimeInfo());
						depGlsEntity.setOnsetEvent(ndOnset);
					}
				} else {
					depGlsEntity.setOnsetEvent(null);
				}

				if (finalHold != null) {
					if (ndFinalHold != null) {
						ndFinalHold.getStartTimeInfo().setMovieTime(
								finalHold.getStartTimeInfo().getMovieTime());
						ndFinalHold.getEndTimeInfo().setMovieTime(
								finalHold.getEndTimeInfo().getMovieTime());
					} else {
						ndFinalHold = new ChainedEvent(panel);
						ndFinalHold.setStartTimeInfo(finalHold.getStartTimeInfo()
															 .cloneTimeInfo());
						ndFinalHold.setEndTimeInfo(finalHold.getEndTimeInfo()
														   .cloneTimeInfo());
						depGlsEntity.setFinalHoldEvent(ndFinalHold);
					}
				} else {
					depGlsEntity.setFinalHoldEvent(null);
				}

				if (offset != null) {
					if (ndOffset != null) {
						ndOffset.getStartTimeInfo().setMovieTime(
								offset.getStartTimeInfo().getMovieTime());
						ndOffset.getEndTimeInfo().setMovieTime(
								offset.getEndTimeInfo().getMovieTime());
					} else {
						ndOffset = new ChainedEvent(panel);
						ndOffset.setStartTimeInfo(offset.getStartTimeInfo()
														  .cloneTimeInfo());
						ndOffset.setEndTimeInfo(offset.getEndTimeInfo()
														.cloneTimeInfo());
						depGlsEntity.setOffsetEvent(ndOffset);
					}
				} else {
					depGlsEntity.setOffsetEvent(null);
				}

			}
		}

		properties.clearHandshapeInformation();

		if(hshPanel.isTwoHandedSelected()) {
			properties2.clearHandshapeInformation();
			if(handshapeClassification == LeftRightHandshapesType.SAME) {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.SAME_HANDSHAPES_LR);
				StartEndHandshapesType sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
				if(sameDiffHandshapes == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} else if(handshapeClassification == LeftRightHandshapesType.ALTERNATING) {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR);
				StartEndHandshapesType sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
				if(sameDiffHandshapes == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			} else {
				properties.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);
				properties2.setTwoHandedHandshapesClassification(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR);
				
				StartEndHandshapesType selectedStartEndHandshapeR = hshPanel.getSelectedSameDifStartEndHandshapesR();
				StartEndHandshapesType selectedStartEndHandshapeL = hshPanel.getSelectedSameDifStartEndHandshapesL();
				
				if(selectedStartEndHandshapeR == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeRightHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeRightHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
				
				if(selectedStartEndHandshapeL == StartEndHandshapesType.SAME) {
					properties.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeLeftHand(GlossEntityProperties.SAME_START_END_HANDSHAPE);
				} else {
					properties.setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
					properties2.setStartEndHandshapeLeftHand(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
				}
			}
		} else {
			StartEndHandshapesType sameDiffHandshapes = hshPanel.getSelectedSameDifStartEndHandshapes();
			if(sameDiffHandshapes == StartEndHandshapesType.SAME) {
				properties.setStartEndHandshape(GlossEntityProperties.SAME_START_END_HANDSHAPE);
			} else {
				properties.setStartEndHandshape(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE);
			}
		}

		if(!properties.isTwoHanded()) {
			currentGlossEntity.setDependentGlossEntity(null);
			currentGlossEntity.setCorrespondingGlossEntityID(null);
		}

		if (depGlsEntity != null && currentGlossEntity.isLockedGlossPhonAlignments()) { //check if locked to hand shape
			Event initialHold = depGlsEntity.getInitialHold();
			Event finalHold = depGlsEntity.getFinalHold();
//            
			if(initialHold != null) {
				int initialStartMovieTime = initialHold.getStartTimeInfo().getMovieTime();
				depGlsEntity.getDependentHandEntity().getStartTimeInfo().setMovieTime(initialStartMovieTime);
				depGlsEntity.getDependentHandShapeEntity().getFirstHandShapeEvent().getStartTimeInfo().setMovieTime(initialStartMovieTime);
				depGlsEntity.getDependentHandShapeEntity().getFirstHandShapeEvent().getEndTimeInfo().setMovieTime(initialStartMovieTime);
			}
			if(finalHold != null) {
				int finalEndMovieTime = finalHold.getEndTimeInfo().getMovieTime();
				depGlsEntity.getDependentHandEntity().getEndTimeInfo().setMovieTime(finalEndMovieTime);
				depGlsEntity.getDependentHandShapeEntity().getLastHandShapeEvent().getStartTimeInfo().setMovieTime(finalEndMovieTime);
				depGlsEntity.getDependentHandShapeEntity().getLastHandShapeEvent().getEndTimeInfo().setMovieTime(finalEndMovieTime);
			}
		}

		hshPanel.getSelectedRightHandshapesPane().removeAll();
		hshPanel.getSelectedLeftHandshapesPane().removeAll();

		properties.setLocationInfo(locationPanel.getCurrentLocationInfo().cloneInfo());
		addOrUpdateLocations(currentGlossEntity, collection);
		addOrUpdateSignTypes(currentGlossEntity, collection);

		wrapper.getSignStreamSegmentPanel().repaint();
		wrapper.getSignStreamSegmentPanel().updateLoadedDatabase();
		wrapper.getSignStreamSegmentPanel().getUtteranceView().updateUtteranceLists();

		// set start/end times to themselves to make sure handshapes won't get misaligned for badly saved glosses
		int start = mediaToolBar.getSelectedEvent().getStartTimeInfo().getMovieTime();
		int end = mediaToolBar.getSelectedEvent().getEndTimeInfo().getMovieTime();
		MediaToolbarCommons.setSelectedEventStartTime(mediaToolBar, start);
		MediaToolbarCommons.setSelectedEventEndTime(mediaToolBar, end);
 	}

 	private void addOrUpdateSignTypes(GlossChainedEventsEntity glossEntity, FieldWrappersCollection collection) {
		SignTypeEntity dependentSignTypeEntity = glossEntity.getDependentSignTypeEntity();
		TimeInfo startTimeInfo = glossEntity.getStartTimeInfo().cloneTimeInfo();
		TimeInfo endTimeInfo = glossEntity.getEndTimeInfo().cloneTimeInfo();

		GlossChainedEventsEntity depGlsEntity = glossEntity.getDependentGlossEntity();

		SignTypeFieldWrapper signTypeFieldWrapper;
		String signTypeFieldID;
		if (!isCurrentGlossDominant() && depGlsEntity == null) {
			signTypeFieldID = Util.NDOM_SIGN_TYPE_FIELD_ID;
		} else {
			signTypeFieldID = Util.DOM_SIGN_TYPE_FIELD_ID;
		}
        signTypeFieldWrapper = (SignTypeFieldWrapper) collection.getFieldWrapperWithFieldID(signTypeFieldID);
        if (signTypeFieldWrapper == null) {
            signTypeFieldWrapper = (SignTypeFieldWrapper) view.getSegment().getRootLabelObject()
                    .getGlossNRelatedLabelObject().getChildWithFieldID(signTypeFieldID).getWrapper();
        }

		if (dependentSignTypeEntity != null) {
			dependentSignTypeEntity.getEvent().setSignType(currentSignType);
			dependentSignTypeEntity.getEvent().setNameSign(glossEntity.getGlossEntityProperties().isNameSign());
			dependentSignTypeEntity.setStartTimeInfo(startTimeInfo);
			dependentSignTypeEntity.setEndTimeInfo(endTimeInfo);
			if (depGlsEntity != null) {
				depGlsEntity.setDependentSignTypeEntity(dependentSignTypeEntity);
			}
		} else {
			SignTypeEvent event = new SignTypeEvent(
					signTypeFieldWrapper.getSignStreamSegmentPanel(),
					startTimeInfo,
					endTimeInfo
			);
			event.setSignType(currentSignType);
			event.setNameSign(glossEntity.getGlossEntityProperties().isNameSign());
			dependentSignTypeEntity = new SignTypeEntity(event, signTypeFieldWrapper.getSignStreamSegmentPanel(), signTypeFieldWrapper.getField());
			dependentSignTypeEntity.setID(Util.getLongUniqueNumber());
			signTypeFieldWrapper.addEventsEntity(dependentSignTypeEntity);

			glossEntity.setDependentSignTypeEntity(dependentSignTypeEntity);
			if (depGlsEntity != null) {
				depGlsEntity.setDependentSignTypeEntity(dependentSignTypeEntity);
			}

			if (isCurrentGlossDominant() || (!isCurrentGlossDominant() && depGlsEntity == null)) {
				dependentSignTypeEntity.setRefGlossEntity(glossEntity);
			} else {
				dependentSignTypeEntity.setRefGlossEntity(depGlsEntity);
			}
		}
	}

	private void addOrUpdateLocations(GlossChainedEventsEntity glossEntity, FieldWrappersCollection collection) {
		SignStreamSegmentPanel segmentPanel = glossEntity.getSignStreamSegmentPanel();
		LocationEntity depLocEntity = glossEntity.getDependentLocationEntity();
		LocationInfo newInfo = locationPanel.getCurrentLocationInfo();

		GlossChainedEventsEntity depGlsEntity = glossEntity.getDependentGlossEntity();

		LocationFieldWrapper lfWrapper;
		String locFieldID;
		if (!isCurrentGlossDominant() && depGlsEntity == null) { // one-handed non-dominant gloss
			locFieldID = Util.NDOM_LOCATION_FIELD_ID;
		} else {
			locFieldID = Util.DOM_LOCATION_FIELD_ID;
		}
		lfWrapper = (LocationFieldWrapper) collection.getFieldWrapperWithFieldID(locFieldID);
		if (lfWrapper == null) {
		    lfWrapper = (LocationFieldWrapper) view.getSegment().getRootLabelObject()
                    .getGlossNRelatedLabelObject().getChildWithFieldID(locFieldID).getWrapper();
        }


		if (newInfo.isEmpty()) {
		    if (depLocEntity != null) {
                lfWrapper.getField().deleteEntity(depLocEntity);
                glossEntity.setDependentLocationEntityID(null);
                glossEntity.setDependentLocationEntity(null);
		    }
		    return;
		}

		int startTime = glossEntity.getStartTimeInfo().getMovieTime();
        int endTime = glossEntity.getEndTimeInfo().getMovieTime();

        ArrayList<LocationEvent> eventsList = new ArrayList<LocationEvent>();

        // populates the list of LocationEvents that will get passed to the LocationEntity.
        // sets the start/end times of each LocationEvent relative to what order they appear in on-screen
        ArrayList<LocationType> locationTypes = SS3Singleton.getLocationConfig().getLocationTypes();
        for (int i = 0; i < locationTypes.size(); i++) {
            LocationType type = locationTypes.get(i);
            int time;
            if (i == 0) {
                time = startTime;
            } else if (i == locationTypes.size() - 1) {
                time = endTime;
            } else {
                time = startTime + (i * (endTime - startTime) / (newInfo.size() - 1));
            }
            TimeInfo startTimeInfo = new TimeInfo();
            startTimeInfo.setMovieTime(time);
            LocationEvent event = new LocationEvent(segmentPanel, type, newInfo.getLocation(type));
            event.setStartTimeInfo(startTimeInfo);

            eventsList.add(event);
        }

        if (depLocEntity != null) {
            depLocEntity.reset();
            depLocEntity.setEvents(eventsList);
            if (depGlsEntity != null) {
                depGlsEntity.setDependentLocationEntity(depLocEntity);
            }
        } else {
            LocationEventsCollection newLocEventsColl = new LocationEventsCollection();
            for (LocationEvent locationEvent : eventsList) {
                newLocEventsColl.addEvent(locationEvent);
            }

            LocationEntity newLocEnt = new LocationEntity(Util.getLongUniqueNumber(), newLocEventsColl, segmentPanel, lfWrapper.getField());
            newLocEnt.setStartTimeInfo(newLocEventsColl.getFirstEvent().getStartTimeInfo().cloneTimeInfo());
            newLocEnt.setEndTimeInfo(newLocEventsColl.getLastEvent().getEndTimeInfo().cloneTimeInfo());
            lfWrapper.addEventsEntity(newLocEnt);

            glossEntity.setDependentLocationEntity(newLocEnt);
            if (depGlsEntity != null) {
                depGlsEntity.setDependentLocationEntity(newLocEnt);
            }

            if (!isCurrentGlossDominant() && depGlsEntity == null) {
                newLocEnt.setRefGlossEntity(glossEntity);
            } else {
                newLocEnt.setRefGlossEntity(depGlsEntity);
            }
        }
	}

	private void addOrUpdateHandShapes(HandShapeBean bean, GlossChainedEventsEntity entity, FieldWrappersCollection collection) {

		GlossChainedEventsEntity depGlsEntity = entity.getDependentGlossEntity();
		String fieldID = entity.getField().getFieldID();

		// 10000 - right handshapes, 10001 - left handshapes

		ArrayList<HandShapeImage> domHndShapeImgs = new ArrayList<HandShapeImage>(bean.getSelectedDomHandShapeImages());
		ArrayList<HandShapeImage> ndomHndShapeImgs = new ArrayList<HandShapeImage>(bean.getSelectedNdomHandShapeImages());

		ChainedEventsEntity dhEntity, ndhEntity;
		HandShapeEntity dHshEntity, ndHshEntity;

		SignStreamSegmentPanel segmentPanel = entity.getSignStreamSegmentPanel();

		ChainedEventsEntity handEntity = entity.getDependentHandEntity();
		HandShapeEntity handShapeEntity = entity.getDependentHandShapeEntity();
		GlossChainedEventsEntity ndGlsEntity = entity.getDependentGlossEntity();
		ChainedEventsEntity ndHandEntity = null;
		HandShapeEntity ndHandShapeEntity = null;

		if(ndGlsEntity != null) {
			ndHandEntity = ndGlsEntity.getDependentHandEntity();
			ndHandShapeEntity = ndGlsEntity.getDependentHandShapeEntity();
		}

		RightLeftHandFieldWrapper domHandWrapper = (RightLeftHandFieldWrapper) collection.getFieldWrapper(Constants.TYPE_DOM_HAND);
		HandShapeFieldWrapper domHandShapeWrapper = (HandShapeFieldWrapper) collection.getFieldWrapper(Constants.TYPE_DOM_HAND_SHAPE);

		RightLeftHandFieldWrapper ndomHandWrapper = (RightLeftHandFieldWrapper) collection.getFieldWrapper(Constants.TYPE_NDOM_HAND);
		HandShapeFieldWrapper ndomHandShapeWrapper = (HandShapeFieldWrapper) collection.getFieldWrapper(Constants.TYPE_NDOM_HAND_SHAPE);


		if(!domHndShapeImgs.isEmpty()) {
			// add/update handshapes on right hand
			if(fieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					handEntity.getTextValue().setText(entity.getText());
				}
				if(handShapeEntity != null) {
					ChainedEvent value = entity.getTextValue();
					ChainedEvent initHold = entity.getInitialHold();
					ChainedEvent finalHold = entity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}

					SS3Entity ent = createSS3Entity(domHndShapeImgs, startTimeInfo, endTimeInfo);
					handShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					dhEntity = addHandEntity(domHandWrapper, entity, segmentPanel);

					dHshEntity = createHandShapeEntity(bean, domHndShapeImgs, dhEntity,
													   domHandShapeWrapper.getHandShapeField());

					domHandShapeWrapper.getHandShapeField().addHandShapeEntity(dHshEntity);
					entity.setDependentHandEntity(dhEntity);
					entity.setDependentHandShapeEntity(dHshEntity);

					entity.setLockedGlossPhonAlignments(true);
					dhEntity.setLockedGlossPhonAlignments(true);
					dHshEntity.setLockedGlossPhonAlignments(true);
				}
			} else {
				if(ndHandEntity != null) {
					depGlsEntity.getTextValue().setText(entity.getText());
				}
				if(ndHandShapeEntity != null) {
					ChainedEvent value = depGlsEntity.getTextValue();
					ChainedEvent initHold = depGlsEntity.getInitialHold();
					ChainedEvent finalHold = depGlsEntity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}

					SS3Entity ent = createSS3Entity(domHndShapeImgs, startTimeInfo, endTimeInfo);
					ndHandShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					dhEntity = addHandEntity(domHandWrapper, depGlsEntity, segmentPanel);

					dHshEntity = createHandShapeEntity(bean, domHndShapeImgs, dhEntity,
													   domHandShapeWrapper.getHandShapeField());

					domHandShapeWrapper.getHandShapeField().addHandShapeEntity(dHshEntity);
					depGlsEntity.setDependentHandEntity(dhEntity);
					depGlsEntity.setDependentHandShapeEntity(dHshEntity);

					depGlsEntity.setLockedGlossPhonAlignments(true);
					dhEntity.setLockedGlossPhonAlignments(true);
					dHshEntity.setLockedGlossPhonAlignments(true);
				}
			}
		} else {
			// delete handshapes
			if(fieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					domHandWrapper.getField().deleteEntity(handEntity);
				}

				if(handShapeEntity != null) {
					domHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);
				}

				entity.setDependentHandEntity(null);
				entity.setDependentHandShapeEntity(null);
			} else {
				if(ndHandEntity != null) {
					domHandWrapper.getField().deleteEntity(handEntity);
				}
				if(ndHandShapeEntity != null) {
					domHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);
				}

				if(depGlsEntity != null) {
					depGlsEntity.setDependentHandEntity(null);
					depGlsEntity.setDependentHandShapeEntity(null);
				}
			}
		}

		if(!ndomHndShapeImgs.isEmpty()) {
			// add/update handshapes on right hand
			if(fieldID.equals(Util.NDOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					handEntity.getTextValue().setText(entity.getText());
				}
				if(handShapeEntity != null) {
					ChainedEvent value = entity.getTextValue();
					ChainedEvent initHold = entity.getInitialHold();
					ChainedEvent finalHold = entity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}

					SS3Entity ent = createSS3Entity(ndomHndShapeImgs, startTimeInfo, endTimeInfo);
					handShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					ndhEntity = addHandEntity(ndomHandWrapper, entity, segmentPanel);

					ndHshEntity = createHandShapeEntity(bean, ndomHndShapeImgs, ndhEntity,
													   ndomHandShapeWrapper.getHandShapeField());

					ndomHandShapeWrapper.getHandShapeField().addHandShapeEntity(ndHshEntity);
					entity.setDependentHandEntity(ndhEntity);
					entity.setDependentHandShapeEntity(ndHshEntity);

					entity.setLockedGlossPhonAlignments(true);
					ndhEntity.setLockedGlossPhonAlignments(true);
					ndHshEntity.setLockedGlossPhonAlignments(true);
				}
			} else {
				// assert ndEntity != null;
				if(ndHandEntity != null) {
					ndHandEntity.getTextValue().setText(entity.getText());
				}
				if(ndHandShapeEntity != null) {
					ChainedEvent value = depGlsEntity.getTextValue();
					ChainedEvent initHold = depGlsEntity.getInitialHold();
					ChainedEvent finalHold = depGlsEntity.getFinalHold();

					TimeInfo startTimeInfo; // = null;
					TimeInfo endTimeInfo; // = null;

					if (initHold != null) {
						startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
					} else {
						startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
					}

					if (finalHold != null) {
						endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
					} else {
						endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
					}

					SS3Entity ent = createSS3Entity(ndomHndShapeImgs, startTimeInfo, endTimeInfo);
					ndHandShapeEntity.resetEntity(ent);
				} else {
					// add entity to Right Hand field
					ndhEntity = addHandEntity(ndomHandWrapper, depGlsEntity, segmentPanel);

					ndHshEntity = createHandShapeEntity(bean, ndomHndShapeImgs, ndhEntity,
													   ndomHandShapeWrapper.getHandShapeField());

					ndomHandShapeWrapper.getHandShapeField().addHandShapeEntity(ndHshEntity);

					if(depGlsEntity != null) {
						depGlsEntity.setDependentHandEntity(ndhEntity);
						depGlsEntity.setDependentHandShapeEntity(ndHshEntity);
						depGlsEntity.setLockedGlossPhonAlignments(true);
					}

					ndhEntity.setLockedGlossPhonAlignments(true);
					ndHshEntity.setLockedGlossPhonAlignments(true);
				}
			}
		} else {
			// delete handshapes
			if(fieldID.equals(Util.NDOM_GLOSS_FIELD_ID)) {
				if(handEntity != null) {
					ndomHandWrapper.getField().deleteEntity(handEntity);
				}

				if(handShapeEntity != null) {
					ndomHandShapeWrapper.getHandShapeField().deleteEntity(handShapeEntity);
				}

				entity.setDependentHandEntity(null);
				entity.setDependentHandShapeEntity(null);
			} else {
				if(ndHandEntity != null) {
					ndomHandWrapper.getField().deleteEntity(ndHandEntity);
				}
				if(ndHandShapeEntity != null) {
					ndomHandShapeWrapper.getHandShapeField().deleteEntity(ndHandShapeEntity);
				}

				if(depGlsEntity != null) {
					depGlsEntity.setDependentHandEntity(null);
					depGlsEntity.setDependentHandShapeEntity(null);
				}
			}
		}
	}

	private SS3Entity createSS3Entity(ArrayList<HandShapeImage> handShapeImgs,
									  TimeInfo startTimeInfo, TimeInfo endTimeInfo) {
		SS3Entity ent = new SS3Entity();

		if (handShapeImgs.size() == 1) {
			HandShapeImage img = handShapeImgs.get(0);
			SS3FieldValue value = img.getSS3FieldValue();

			SS3Item item1 = new SS3Item(value.getId(), value.getIid(),
										startTimeInfo, endTimeInfo);
			ent.addItem(item1);
		} else {

			int startTime = startTimeInfo.getMovieTime();
			HandShapeImage img1 = handShapeImgs.get(0);
			SS3FieldValue value1 = img1.getSS3FieldValue();

			SS3Item item1 = new SS3Item(value1.getId(), value1.getIid(),
										startTimeInfo, startTimeInfo.cloneTimeInfo());
			ent.addItem(item1);

			int step = (endTimeInfo.getMovieTime() - startTimeInfo
					.getMovieTime()) / (handShapeImgs.size() - 1);

			for (int i = 1; i < handShapeImgs.size() - 1; i++) {
				HandShapeImage img = handShapeImgs.get(i);
				SS3FieldValue value = img.getSS3FieldValue();
				TimeInfo startTimeInfo1 = startTimeInfo.cloneTimeInfo();
				startTime = startTime + step;
				startTimeInfo1.setMovieTime(startTime);
				SS3Item item3 = new SS3Item(value.getId(), value.getIid(),
											startTimeInfo1, startTimeInfo1.cloneTimeInfo());
				ent.addItem(item3);
			}

			HandShapeImage img2 = handShapeImgs.get(handShapeImgs.size() - 1);
			SS3FieldValue value2 = img2.getSS3FieldValue();
			TimeInfo endTimeInfo2 = endTimeInfo.cloneTimeInfo();
			SS3Item item4 = new SS3Item(value2.getId(), value2.getIid(),
										endTimeInfo2, endTimeInfo2.cloneTimeInfo());
			ent.addItem(item4);
		}
		return ent;
	}

	private HandShapeEntity createHandShapeEntity(HandShapeBean bean,
												  ArrayList<HandShapeImage> handShapeImages,
												  ChainedEventsEntity superEntity, Field field) {

		SignStreamSegmentPanel segmentPanel = superEntity
				.getSignStreamSegmentPanel();

		HandShapeEntity hsEntity = new HandShapeEntity(segmentPanel, field);
		hsEntity.setID(Util.getUniqueNumber());

		hsEntity.setField(field);
		hsEntity.setHandShapeType(bean.getType());
		hsEntity.setRefEntityId(superEntity.getID());
		HandShapeSubEntity subEntity = new HandShapeSubEntity(segmentPanel, field);
		subEntity.setID(hsEntity.getID());
		hsEntity.addHandShapeSubEntity(subEntity);

		ChainedEvent event = superEntity.getTextValue();

		int startTime = event.getStartTimeInfo().getMovieTime();

		if (handShapeImages.size() == 1) {
			HandShapeImage img = handShapeImages.get(0);
			SS3FieldValue value = img.getSS3FieldValue();
			String vid = value.getId();

			SS3Item item1 = new SS3Item();
			item1.setId(vid);
			item1.setImageID(value.getIid());

			TimeInfo startTimeInfo1 = event.getStartTimeInfo().cloneTimeInfo();
			TimeInfo startTimeInfo2 = event.getEndTimeInfo().cloneTimeInfo();
			item1.setStartTimeInfo(startTimeInfo1);
			item1.setEndTimeInfo(startTimeInfo2);
			subEntity.addHandShape(item1);
		} else if (handShapeImages.size() > 1) {
			HandShapeImage img1 = handShapeImages.get(0);
			SS3FieldValue value1 = img1.getSS3FieldValue();
			String vid1 = value1.getId();

			SS3Item item1 = new SS3Item();
			item1.setId(vid1);
			item1.setImageID(value1.getIid());

			TimeInfo startTimeInfo1 = event.getStartTimeInfo().cloneTimeInfo();
			item1.setStartTimeInfo(startTimeInfo1);
			item1.setEndTimeInfo(startTimeInfo1.cloneTimeInfo());
			subEntity.addHandShape(item1);

			int step = (event.getEndTimeInfo().getMovieTime() - event
					.getStartTimeInfo().getMovieTime())
					/ (handShapeImages.size() - 1);

			for (int i = 1; i < handShapeImages.size() - 1; i++) {
				HandShapeImage img = handShapeImages.get(i);
				SS3FieldValue value = img.getSS3FieldValue();
				String vid = value.getId();
				SS3Item item = new SS3Item();
				item.setId(vid);
				item.setImageID(value.getIid());

				TimeInfo startTimeInfo = event.getStartTimeInfo()
						.cloneTimeInfo();
				startTimeInfo.setMovieTime(startTime + i * step);
				item.setStartTimeInfo(startTimeInfo);
				item.setEndTimeInfo(startTimeInfo.cloneTimeInfo());

				subEntity.addHandShape(item);
			}

			HandShapeImage img2 = handShapeImages
					.get(handShapeImages.size() - 1);
			SS3FieldValue value2 = img2.getSS3FieldValue();
			String vid2 = value2.getId();

			SS3Item item2 = new SS3Item();
			item2.setId(vid2);
			item2.setImageID(value2.getIid());

			TimeInfo startTimeInfo2 = event.getEndTimeInfo().cloneTimeInfo();
			item2.setStartTimeInfo(startTimeInfo2);
			item2.setEndTimeInfo(startTimeInfo2.cloneTimeInfo());
			subEntity.addHandShape(item2);
		}

		hsEntity.setLockedGlossPhonAlignments(true);
		hsEntity.highlight();

		subEntity.determineParentEntity();
		hsEntity.determineParentEntity();

		return hsEntity;
	}

	private ChainedEventsEntity addHandEntity(RightLeftHandFieldWrapper wrp,
											  GlossChainedEventsEntity glsEntity, SignStreamSegmentPanel panel) {

		ChainedEvent value = glsEntity.getTextValue();
		ChainedEvent initHold = glsEntity.getInitialHold();
		ChainedEvent finalHold = glsEntity.getFinalHold();

		TimeInfo startTimeInfo; // = null;
		TimeInfo endTimeInfo; // = null;

		glsEntity.setLockedGlossPhonAlignments(true);

		if (initHold != null) {
			startTimeInfo = initHold.getStartTimeInfo().cloneTimeInfo();
		} else {
			startTimeInfo = value.getStartTimeInfo().cloneTimeInfo();
		}

		if (finalHold != null) {
			endTimeInfo = finalHold.getEndTimeInfo().cloneTimeInfo();
		} else {
			endTimeInfo = value.getEndTimeInfo().cloneTimeInfo();
		}

		ChainedEventsCollection collection = new ChainedEventsCollection();

		GlossChainedEvent gEvent = new GlossChainedEvent(panel);
		gEvent.setStartTimeInfo(startTimeInfo);
		gEvent.setEndTimeInfo(endTimeInfo);
		gEvent.setText(value.getText());
		collection.setTextValueEvent(gEvent);

		ChainedEventsEntity handEntity = new ChainedEventsEntity(collection,
																 panel, wrp.getField());

		handEntity.setID(Util.getUniqueNumber());
		handEntity.setField(wrp.getField());
		handEntity.determineParentEntity();

		wrp.addEventsEntity(handEntity);
		handEntity.setRefEntityId(glsEntity.getID());
		handEntity.highlight();

		handEntity.setLockedGlossPhonAlignments(true);
		return handEntity;
	}

	public void addToSignBank(String mainGlossStr, String variantStr) {
		SignBankCollection collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();


		// check first if this sign exists in collection already - we will disallow if so
		SignBankVariant matchingVariant = collection.contains(mainGlossStr, variantStr);
		if (matchingVariant != null) {
			String warningDialogText =
					"Sign already found in bank.\nPlease enter a different set of values";
			String warningTitleText = "Duplicate Sign";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String options[] = {"OK"};

			try {
				JOptionPane.showOptionDialog(this,
											 warningDialogText,
											 warningTitleText,
											 optionType,
											 messageType,
											 null,
											 options,
											 options[0]);
			} catch (HeadlessException he) {
				System.out.println("");
			}

			return;
		}

		// next check that there is at least one handshape on the dominant (right) hand before allowing the addition
		ArrayList<HandShapeImage> domHandShapeImages;
		if (isDominantHandRight()) {
			domHandShapeImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		} else {
			domHandShapeImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		}
		if (domHandShapeImages.size() < 1) {
			String warningDialogText =
					"You must enter at least one handshape for the dominant hand before adding to the Sign Bank.";
			String warningTitleText = "Select Handshapes";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.ERROR_MESSAGE;
			String options[] = {"OK"};

			try {
				JOptionPane.showOptionDialog(this,
											 warningDialogText,
											 warningTitleText,
											 optionType,
											 messageType,
											 null,
											 options,
											 options[0]);
			} catch (HeadlessException he) {
				System.out.println("");
			}

			return;
		}

		// and when two-handed is selected, it means you must be intending to add both hands, so check nondominant (left) handshapes as well
		if (hshPanel.isTwoHandedSelected()) {
			ArrayList<HandShapeImage> nondomHandShapeImages;
			if (isDominantHandRight()) {
				nondomHandShapeImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
			} else {
				nondomHandShapeImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
			}
			if (nondomHandShapeImages.size() < 1) {
				String warningDialogText =
						"You must enter at least one handshape for the nondominant hand before adding to the Sign Bank.";
				String warningTitleText = "Select Handshapes";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.ERROR_MESSAGE;
				String options[] = {"OK"};

				try {
					JOptionPane.showOptionDialog(this,
												 warningDialogText,
												 warningTitleText,
												 optionType,
												 messageType,
												 null,
												 options,
												 options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}

				return;
			}
		}

		String resourceFilename = Util.SIGN_BANK_DEFAULT_LOCAL_FILE;

//		SignBankResource resource;
//		String resourceFilename = (String) resourceSelector.getSelectedItem();
//		resource = resourceCollection.getResourceByFileName(resourceFilename);

		String mgID = Util.getUniqueNumber();
		String varID = Util.getUniqueNumber();
		String occID = Util.getUniqueNumber();

		// check whether ID already taken in SB collection - and if so get a new one
		while (collection.getOccurrenceWithID(mgID, varID, occID) != null) {
			occID = Util.getUniqueNumber();
		}

		// if there is not an existing main gloss with the label given, then create a new main gloss object to save
		SignBankMainGloss mainGloss = collection.getMainGlossesList().get(mainGlossStr);
		if (mainGloss == null) {
			mainGloss = new SignBankMainGloss(mgID, mainGlossStr.toUpperCase(), varID);
		}
		SignBankVariant newVariant = new SignBankVariant(mainGloss, varID, variantStr, occID, "1");
		SignBankOccurrence newOccurrence = new SignBankOccurrence(newVariant, occID);

		// get data points to set in the occurrence object

		ArrayList<HandShapeImage> domImages, nondomImages;
		int domImagesSize, nondomImagesSize;
		if (isDominantHandRight()) {
			domImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
		} else {
			domImages = hshPanel.getSelectedLeftHandshapesPane().getHandshapes();
			nondomImages = hshPanel.getSelectedRightHandshapesPane().getHandshapes();
		}
		domImagesSize = domImages.size();
		nondomImagesSize = nondomImages.size();

		String domStartHS = null;
		String nondomStartHS = null;
		String domEndHS = null;
		String nondomEndHS = null;

		if (domImagesSize > 0) {
			domStartHS = domImages.get(0).getSS3FieldValue().getLabel();
			domEndHS = domImages.get(domImagesSize - 1).getSS3FieldValue().getLabel();
		}

		if (nondomImagesSize > 0) {
			nondomStartHS = nondomImages.get(0).getSS3FieldValue().getLabel();
			nondomEndHS = nondomImages.get(nondomImagesSize - 1).getSS3FieldValue().getLabel();
		}

		String participant = null;

		String signType = null;
		if (hshPanel.isLexical())
			signType = "LEXICAL";
		if (hshPanel.isFingerspelled())
			signType = "FINGERSPELLED";
		if (hshPanel.isLoan())
			signType = "LOAN";
		if (hshPanel.isNumber())
			signType = "NUMBER";
		if (hshPanel.isClassifier())
			signType = "CLASSIFER";
		if (hshPanel.isGesture())
			signType = "GESTURE";

		String twoHanded = hshPanel.isTwoHandedSelected() ? "T" : "F";

		String passiveBaseArm = hshPanel.isPassiveBaseArm() ? "T" : "F";

		String markedHands = hshPanel.isMarkedNumberOfHands() ? "T" : "F";
		
		LeftRightHandshapesType leftRightHSIndex = hshPanel.getSelectedSameDifLeftRightHandshapes();
		StartEndHandshapesType startEndHSIndex = hshPanel.getSelectedSameDifStartEndHandshapes();
		StartEndHandshapesType startEndLeftHSIndex = hshPanel.getSelectedSameDifStartEndHandshapesL();
		StartEndHandshapesType startEndRightHSIndex = hshPanel.getSelectedSameDifStartEndHandshapesR();
		
		String leftRightHS = leftRightHSIndex == LeftRightHandshapesType.SAME ? "SAME" : "DIFFERENT";
		String startEndHS = "", startEndLeftHS = "", startEndRightHS = "";
		
		if (leftRightHSIndex == LeftRightHandshapesType.SAME) {	// if left right is the same
			startEndHS = startEndHSIndex == StartEndHandshapesType.SAME ? "SAME" : "DIFFERENT";
		} else {	// if left right is different
			startEndLeftHS = startEndLeftHSIndex == StartEndHandshapesType.SAME ? "SAME" : "DIFFERENT";
			startEndRightHS = startEndRightHSIndex == StartEndHandshapesType.SAME ? "SAME" : "DIFFERENT";
		}


		newOccurrence.setSignBankResourceFileName(resourceFilename);
		newOccurrence.setDomStartHS(domStartHS);
		newOccurrence.setNondomStartHS(nondomStartHS);
		newOccurrence.setDomEndHS(domEndHS);
		newOccurrence.setNondomEndHS(nondomEndHS);
		newOccurrence.setParticipant(participant);
		newOccurrence.setSignType(signType);
		newOccurrence.setTwoHanded(twoHanded);
		newOccurrence.setPassiveBaseArm(passiveBaseArm);
		newOccurrence.setMarkedHands(markedHands);
		newOccurrence.setLeftRightHS(leftRightHS);
		newOccurrence.setStartEndHS(startEndHS);
		newOccurrence.setStartEndLeftHS(startEndLeftHS);
		newOccurrence.setStartEndRightHS(startEndRightHS);

		newVariant.getOccurrencesList().put(newOccurrence.getOccurrenceID(), newOccurrence);
		mainGloss.getVariantsList().put(newVariant.getVariantLabel(), newVariant);

		// no harm in overwriting the same main gloss with an additional variant
		collection.getMainGlossesList().put(mainGloss.getMainGlossLabel(), mainGloss);

		// write file now
		SS3Singleton.getSignBankWriter().writeResourceToSignBank(collection, resourceFilename);

		// re-parse the entire sign bank collection fresh, application class reference will be set
		SS3Singleton.getSignStreamApplication().parseSignBank();

		// notify the user of a successful addition to the sign bank
		String warningDialogText =
				"Successfully added to sign bank.";
		String warningTitleText = "Added Sign";
		int optionType = JOptionPane.YES_NO_OPTION;
		int messageType = JOptionPane.ERROR_MESSAGE;
		String options[] = {"OK"};

		try {
			JOptionPane.showOptionDialog(this,
										 warningDialogText,
										 warningTitleText,
										 optionType,
										 messageType,
										 null,
										 options,
										 options[0]);
		} catch (HeadlessException he) {
			System.out.println("");
		}
	}

	public void addToSignBank() {
		String mainGlossStr = primaryEntryField.getText().trim();
		String variantStr = variantField.getText().trim();

		addToSignBank(mainGlossStr, variantStr);
	}

	/**
	 * To be called by the sign bank panel exclusively.
	 * Take the occurrence object passed in and set the values on this panel accordingly.
	 * @param signBankOccurrence The sign bank occurrence object from the sign bank results list
	 */
	public void populateFromSignBankOccurrence(SignBankOccurrence signBankOccurrence) {
		SignBankVariant parentVariant = signBankOccurrence.getParentVariant();
		SignBankMainGloss parentMainGloss = parentVariant.getParentMainGloss();

		String variantLabel = parentVariant.getVariantLabel().trim();
		String mainGlossLabel = parentMainGloss.getMainGlossLabel().trim();

		String domStartHS = signBankOccurrence.getDomStartHS();
		String nondomStartHS = signBankOccurrence.getNondomStartHS();
		String domEndHS = signBankOccurrence.getDomEndHS();
		String nondomEndHS = signBankOccurrence.getNondomEndHS();
		String signType = signBankOccurrence.getSignType();
		boolean twoHanded = signBankOccurrence.isTwoHanded();
		boolean passiveBaseArm = signBankOccurrence.isPassiveBaseArm();
		boolean markedHands = signBankOccurrence.isMarkedHands();
		String leftRightHS = signBankOccurrence.getLeftRightHS();
		String startEndHS = signBankOccurrence.getStartEndHS();
		String startEndLeftHS = signBankOccurrence.getStartEndLeftHS();
		String startEndRightHS = signBankOccurrence.getStartEndRightHS();


		//Clear Handshapes to remove any popups about handedness.
		ArrayList<HandShapeImage> selectedHandImagesDom = new ArrayList<HandShapeImage>();
		ArrayList<HandShapeImage> selectedHandImagesNdom = new ArrayList<HandShapeImage>();
		hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(selectedHandImagesDom);
		hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(selectedHandImagesNdom);
		hshPanel.updateSelectedHandshapesPanels(selectedHandImagesDom, selectedHandImagesNdom);
		hshPanel.setNameSign(false);

		variantField.setText(variantLabel);
		
		primaryEntryField.setText(mainGlossLabel);
		
		if (variantLabel.startsWith("ns-")) {
			hshPanel.setNameSign(true);
		} else {
			hshPanel.setNameSign(false);
		}

		if (signType != null) {
			if (signType.equals("LEXICAL")) {
				hshPanel.setLexical(true);
				hshPanel.onLexicalClassifierSelected();
				setCurrentSignType(SignType.LEXICAL);				
			}
			if (signType.equals("FINGERSPELLED")) {
				hshPanel.setFingerspelled(true);
				hshPanel.onFingerspelledLoanNumberSelected();
				setCurrentSignType(SignType.NUMBER);				
			}
			if (signType.equals("LOAN")) {
				hshPanel.setLoan(true);
				hshPanel.onFingerspelledLoanNumberSelected();
				setCurrentSignType(SignType.LOAN);				
			}
			if (signType.equals("NUMBER")) {
				hshPanel.setNumber(true);
				hshPanel.onFingerspelledLoanNumberSelected();
				setCurrentSignType(SignType.NUMBER);				
			}
			if (signType.equals("CLASSIFIER")) {
				hshPanel.setClassifier(true);
				hshPanel.onLexicalClassifierSelected();
				setCurrentSignType(SignType.CLASSIFIER);				
			}
			if (signType.equals("GESTURE")) {
				hshPanel.setGesture(true);
				hshPanel.onGestureSelected();
				setCurrentSignType(SignType.GESTURE);				
			}
		}

		if (twoHanded) {
			hshPanel.selectTwoHanded();
			LeftRightHandshapesType leftRightType = LeftRightHandshapesType.valueOf(leftRightHS);
			hshPanel.setSelectedSameDifLeftRightHandshapes(leftRightType);
			if (leftRightType == LeftRightHandshapesType.SAME) {
				if (startEndHS != null && !startEndHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndHS));
				} else {
					if (startEndRightHS != null && !startEndRightHS.isEmpty()) {
						hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndRightHS));
					} else if (startEndLeftHS != null && !startEndLeftHS.isEmpty()) {
						hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndLeftHS));
					}
				}
			} else {
				if (startEndRightHS != null && !startEndRightHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.valueOf(startEndRightHS));
				}
				if (startEndLeftHS != null && !startEndLeftHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.valueOf(startEndLeftHS));
				}
			}
		} else {
			hshPanel.selectOneHanded();
			if (startEndHS != null && !startEndHS.isEmpty()) {
				hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndHS));
			} else {
				if (startEndRightHS != null && !startEndRightHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndRightHS));
				} else if (startEndLeftHS != null && !startEndLeftHS.isEmpty()) {
					hshPanel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.valueOf(startEndLeftHS));
				}
			}
		}

		hshPanel.setPassiveBaseArmSelected(passiveBaseArm);

		hshPanel.setMarkedNumberOfHands(markedHands);

		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		ArrayList<SS3FieldValue> domFieldValues = cs.getFieldImg(Util.DOM_HANDSHAPE_FIELD_ID);
		ArrayList<SS3FieldValue> ndomFieldValues = cs.getFieldImg(Util.NDOM_HANDSHAPE_FIELD_ID);

		SS3FieldValue domHandStartValue = null, domHandEndValue = null, ndomHandStartValue = null, ndomHandEndValue = null;

		// dom start hs
		domHandStartValue = Util.matchSignBank(domFieldValues, domStartHS);

		if (domHandStartValue != null) {
			HandShapeImage image = new HandShapeImage(domHandStartValue, cs);
			selectedHandImagesDom.add(image);
		}

		// right end hs
		domHandEndValue = Util.matchSignBank(domFieldValues, domEndHS);


		if (domHandEndValue != null) {
			HandShapeImage image = new HandShapeImage(domHandEndValue, cs);
			selectedHandImagesDom.add(image);
		}

		// left start hs
		ndomHandStartValue = Util.matchSignBank(ndomFieldValues, nondomStartHS) ;

		if (ndomHandStartValue != null) {
			HandShapeImage image = new HandShapeImage(ndomHandStartValue, cs);
			selectedHandImagesNdom.add(image);
		}

		// left end hs
		ndomHandEndValue = Util.matchSignBank(ndomFieldValues, nondomEndHS);

		if (ndomHandEndValue != null) {
			HandShapeImage image = new HandShapeImage(ndomHandEndValue, cs);
			selectedHandImagesNdom.add(image);
		}

		// set the arraylists of handshapes
		hshPanel.getHandShapeBean().setUpdated(true);
		if(hshPanel.isNondomHandSelected()){
			hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(selectedHandImagesNdom);
			hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(selectedHandImagesDom);
			hshPanel.updateSelectedHandshapesPanels(selectedHandImagesNdom, selectedHandImagesDom);
		}else{
			hshPanel.getHandShapeBean().setSelectedDomHandShapeImages(selectedHandImagesDom);
			hshPanel.getHandShapeBean().setSelectedNdomHandShapeImages(selectedHandImagesNdom);
			hshPanel.updateSelectedHandshapesPanels(selectedHandImagesDom, selectedHandImagesNdom);
		}
		hshPanel.repaint();
		
		// Disable/Enable SignBank search buttons if necessary when inserting from SignBank
		checkSignTypeAndUpdateSignBankFunctionality();

		revalidate();
		repaint();
	}

	/**
	 * To be called by the sign bank panel exclusively.
	 * Set the main gloss text and variant text to the labels passed in.
	 * @param mainGlossLabel The main gloss label from the sign bank panel results list
	 * @param variantLabel The variant label from the sign bank panel results list
	 */
	public void populateMainGlossAndVariantLabels(String mainGlossLabel, String variantLabel) {
		if (mainGlossLabel == null || variantLabel == null)
			return;

		primaryEntryField.setText(mainGlossLabel.trim());
		variantField.setText(variantLabel.trim());
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
			case ENTER_BYPASS:
				saveChanges();
				view.update();
				view.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getCSFieldDialog()
                        .getCSFieldPanel().showAllFields();
				SS3Singleton.getSignStreamApplication().repaint();
				parent.setVisible(false);
				System.gc();
				break;
			case ENTER:

				SignBankCollection collection = null;
				String mainGlossStr = "", trimmedVariantStr = "", originalVariantStr = "";

				// first, before closing out, check that the user didn't edit the labels of main gloss & variant - to ensure that those labels are in the SB
				if (hshPanel.isLexical() || hshPanel.isNumber()) {
					collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();

					mainGlossStr = primaryEntryField.getText().trim();
					originalVariantStr = variantField.getText().trim();
					trimmedVariantStr = Util.trimAsterisksAndPlusSigns(originalVariantStr);
					// check first if this sign DOES NOT exist in collection already - we will disallow if so
					SignBankVariant matchingVariant = collection.contains(trimmedVariantStr);
					SignBankVariant matchingOriginalVariant = collection.contains(originalVariantStr);
					if (matchingVariant == null && matchingOriginalVariant == null) {
						String warningDialogText =
								"Sign was not found in the sign bank.\nPlease either search for an existing sign, or add a new one.";
						String warningTitleText = "Sign Not Found";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"OK"};

						try {
							JOptionPane.showOptionDialog(this,
														 warningDialogText,
														 warningTitleText,
														 optionType,
														 messageType,
														 null,
														 options,
														 options[0]);
						} catch (HeadlessException he) {
							System.out.println("");
						}

						return;
					} else if (matchingOriginalVariant == null) {
						//addToSignBank(mainGlossStr, untrimmedVariantStr);

						// reset this variant in the case we just added to sign bank
						// (inside addToSignBank() the sign bank itself got re-parsed into `collection`)
						//matchingUntrimmedVariant = collection.contains(mainGlossStr, untrimmedVariantStr);
						saveChanges();
					} else {
						saveChanges(matchingOriginalVariant);
					}


				} else {
					saveChanges();
				}

				view.update();
				view.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getCSFieldDialog()
                        .getCSFieldPanel().showAllFields();
				SS3Singleton.getSignStreamApplication().repaint();
				parent.setVisible(false);
				System.gc();
				break;
			case CANCEL:
				parent.setVisible(false);
				break;
			case SIGN_BANK:
				collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
				if ( !collection.canAccessSignBank()) {
					// an error occurred - please fix the error
					String warningDialogText =
							"An error occurred in accessing the sign bank.\n" +
									"Either a sign bank file is missing, or a local file was not found.\n" +
									"Make sure files are named exactly globalSignBank.xml and localSignBank.xml";
					String warningTitleText = "Cannot Access Sign Bank";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
													 warningDialogText,
													 warningTitleText,
													 optionType,
													 messageType,
													 null,
													 options,
													 options[0]);
					} catch (HeadlessException he) {
						System.out.println("");
					}

					return;
				}

				String variantLabel = variantField.getText().trim();

				signBankDialog = new SignBankDialog(this, variantLabel);
				signBankDialog.setLocation(new Point(this.getWidth(), 0));
				signBankDialog.setVisible(true);

				break;
			case ADD_NEW:

				// disallow non-sign-bank signs from being added to the sign bank
				if (hshPanel.isFingerspelled() || hshPanel.isGesture() || hshPanel.isClassifier()) {
					return;
				}

				collection = SS3Singleton.getSignStreamApplication().getSignBankCollection();
				if ( !collection.canAccessSignBank()) {
					// an error occurred - please fix the error
					String warningDialogText =
							"An error occurred in accessing the sign bank.\n" +
									"Either a sign bank file is missing, or a local file was not found.\n" +
									"Make sure files are named exactly globalSignBank.xml and localSignBank.xml";
					String warningTitleText = "Cannot Access Sign Bank";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
													 warningDialogText,
													 warningTitleText,
													 optionType,
													 messageType,
													 null,
													 options,
													 options[0]);
					} catch (HeadlessException he) {
						System.out.println("");
					}

					return;
				}

				mainGlossStr = primaryEntryField.getText().toUpperCase().trim();
				trimmedVariantStr = variantField.getText().toUpperCase().trim();

				// first check empty labels
				if (mainGlossStr == null || mainGlossStr.isEmpty() || trimmedVariantStr == null || trimmedVariantStr.isEmpty()) {

					String warningDialogText =
							"Enter values for both variant and main gloss";
					String warningTitleText = "Enter Values";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
													 warningDialogText,
													 warningTitleText,
													 optionType,
													 messageType,
													 null,
													 options,
													 options[0]);
					} catch (HeadlessException he) {
						System.out.println("");
					}

					primaryEntryField.requestFocus();
					return;
				}

				// then check to sanitize input text
				// disallow these characters:  "  '  ,  !  <  >
				String regex = ".*[\"',!<>].*";

				if (mainGlossStr.matches(regex) || trimmedVariantStr.matches(regex)) {
					String warningDialogText =
							"Cannot enter these characters for main gloss or variant label:\n" +
									" +  \"  '  ,  !  <  >";
					String warningTitleText = "Invalid Label";
					int optionType = JOptionPane.YES_NO_OPTION;
					int messageType = JOptionPane.ERROR_MESSAGE;
					String options[] = {"OK"};

					try {
						JOptionPane.showOptionDialog(this,
													 warningDialogText,
													 warningTitleText,
													 optionType,
													 messageType,
													 null,
													 options,
													 options[0]);
					} catch (HeadlessException he) {
						System.out.println("");
					}

					primaryEntryField.requestFocus();
					return;
				}
				addToSignBank();
				break;
		}

		checkSignTypeAndUpdateSignBankFunctionality();
	}

	public void checkSignTypeAndUpdateSignBankFunctionality() {
		if (hshPanel.isLexical() || hshPanel.isNumber() || hshPanel.isLoan())
			enableSignBankFunctionality(true);
		else if (hshPanel.isFingerspelled() || hshPanel.isGesture() || hshPanel.isClassifier())
			enableSignBankFunctionality(false);
	}

	public void enableSignBankFunctionality(boolean enabled) {
		primaryEntryField.setEnabled(enabled);
		addNewSign.setEnabled(enabled);
	}

	public boolean isOneHandedSelected() {
		return hshPanel.isOneHandedSelected();
	}

	public boolean isTwoHandedSelected() {
		return hshPanel.isTwoHandedSelected();
	}

	public boolean isDominantHandRight() {
		return Util.RIGHT_HAND.equals(dominantHand);
	}

	public boolean isCurrentGlossDominant() {
		return wrapper.getGlossField().getFieldID().equals(Util.DOM_GLOSS_FIELD_ID);
	}

	public String getDominantHand() {
		return dominantHand;
	}

	public JDialog getParentDialog() {
		return parent;
	}

	public JButton getEnterButton() {
		return enter;
	}

	public GlossChainedEventsEntity getCurrentGlossEntity() {
		return currentGlossEntity;
	}

	public SignType getCurrentSignType() {
		return currentSignType;
	}

	public void setCurrentSignType(SignType currentSignType) {
		this.currentSignType = currentSignType;
	}
}

class LoadedProperties {

	private String gloss = "";

	private boolean fingerspelled = false;
	private boolean loan = false;
	private boolean number = false;
	private boolean lexical = false;
	private boolean classifier = false;
	private boolean gesture = false;
	private boolean nameSign = false;
	private boolean twoHanded = false;
	private boolean oneFrameGloss = false;
	private boolean markedNumberOfHands = false;
	private boolean passiveBaseArm = false;

//	private int sameDifHandshapes = 0;
	private LeftRightHandshapesType sameDIfferentLeftRightHandshapes = LeftRightHandshapesType.SAME;
	private StartEndHandshapesType sameDifStartEndHandshapesL = StartEndHandshapesType.SAME;
	private StartEndHandshapesType sameDifStartEndHandshapesR = StartEndHandshapesType.SAME;
	private StartEndHandshapesType sameDifStartEndHandshapes = StartEndHandshapesType.SAME;
	
	private ArrayList<HandShapeImage> rightHandSelectedHandshapes = new ArrayList<HandShapeImage>();
	private ArrayList<HandShapeImage> leftHandSelectedHandshapes = new ArrayList<HandShapeImage>();
	
//	public int getSameDifHandshapes() {
//		return sameDifHandshapes;
//	}
//	public void setSameDifHandshapes(int sameDifHandshapes) {
//		this.sameDifHandshapes = sameDifHandshapes;
//	}
	public LeftRightHandshapesType getSameDIfferentLeftRightHandshapes() {
		return sameDIfferentLeftRightHandshapes;
	}
	public void setSameDIfferentLeftRightHandshapes(
			LeftRightHandshapesType sameDIfferentLeftRightHandshapes) {
		this.sameDIfferentLeftRightHandshapes = sameDIfferentLeftRightHandshapes;
	}

	public void setSameDifStartEndHandshapes(
			StartEndHandshapesType sameDifStartEndHandshapes) {
		this.sameDifStartEndHandshapes = sameDifStartEndHandshapes;
	}
	
	public StartEndHandshapesType getSameDifStartEndHandshapes() {
		return sameDifStartEndHandshapes;
	}
	
	public StartEndHandshapesType getSameDifStartEndHandshapesL() {
		return sameDifStartEndHandshapesL;
	}
	public void setSameDifStartEndHandshapesL(StartEndHandshapesType sameDifStartEndHandshapesL) {
		this.sameDifStartEndHandshapesL = sameDifStartEndHandshapesL;
	}
	public StartEndHandshapesType getSameDifStartEndHandshapesR() {
		return sameDifStartEndHandshapesR;
	}
	public void setSameDifStartEndHandshapesR(StartEndHandshapesType sameDifStartEndHandshapesR) {
		this.sameDifStartEndHandshapesR = sameDifStartEndHandshapesR;
	}
	public boolean isFingerspelled() {
		return fingerspelled;
	}
	public void setFingerspelled(boolean fingerspelled) {
		this.fingerspelled = fingerspelled;
	}
	public boolean isLoan() {
		return loan;
	}
	public void setLoan(boolean loan) {
		this.loan = loan;
	}
	public boolean isNumber() {
		return number;
	}
	public void setNumber(boolean number) {
		this.number = number;
	}
	public boolean isLexical() {
		return lexical;
	}
	public void setLexical(boolean lexical) {
		this.lexical = lexical;
	}
	public boolean isClassifier() {
		return classifier;
	}
	public void setClassifier(boolean classifier) {
		this.classifier = classifier;
	}
	public boolean isGesture() {
		return gesture;
	}
	public void setGesture(boolean gesture) {
		this.gesture = gesture;
	}
	public boolean isNameSign() {
		return nameSign;
	}
	public void setNameSign(boolean nameSign) {
		this.nameSign = nameSign;
	}
	public boolean isTwoHanded() {
		return twoHanded;
	}
	public void setTwoHanded(boolean twoHanded) {
		this.twoHanded = twoHanded;
	}
	public boolean isOneFrameGloss() {
		return oneFrameGloss;
	}
	public void setOneFrameGloss(boolean oneFrameGloss) {
		this.oneFrameGloss = oneFrameGloss;
	}
	public boolean isMarkedNumberOfHands() {
		return markedNumberOfHands;
	}
	public void setMarkedNumberOfHands(boolean markedNumberOfHands) {
		this.markedNumberOfHands = markedNumberOfHands;
	}
	public boolean isPassiveBaseArm() {
		return passiveBaseArm;
	}
	public void setPassiveBaseArm(boolean passiveBaseArm) {
		this.passiveBaseArm = passiveBaseArm;
	}
	public ArrayList<HandShapeImage> getRightHandSelectedHandshapes() {
		return rightHandSelectedHandshapes;
	}
	public void setRightHandSelectedHandshapes(
			ArrayList<HandShapeImage> rightHandSelectedHandshapes) {
		this.rightHandSelectedHandshapes.clear();
		this.rightHandSelectedHandshapes.addAll(rightHandSelectedHandshapes);
	}
	public ArrayList<HandShapeImage> getLeftHandSelectedHandshapes() {
		return leftHandSelectedHandshapes;
	}
	public void setLeftHandSelectedHandshapes(
			ArrayList<HandShapeImage> leftHandSelectedHandshapes) {
		this.leftHandSelectedHandshapes.clear();
		this.leftHandSelectedHandshapes.addAll(leftHandSelectedHandshapes);
	}
	public String getGloss() {
		return gloss;
	}
	public void setGloss(String gloss) {
		this.gloss = gloss;
	}
	public void reset() {
		this.gloss = "";
		this.fingerspelled = false;
		this.loan = false;
		this.number = false;
		this.lexical = false;
		this.classifier = false;
		this.gesture = false;
		this.nameSign = false;
		this.twoHanded = false;
		this.oneFrameGloss = false;
		this.markedNumberOfHands = false;
		this.passiveBaseArm = false;
//		this.sameDifHandshapes = 0;
		this.sameDIfferentLeftRightHandshapes = LeftRightHandshapesType.SAME;
		this.sameDifStartEndHandshapesL = StartEndHandshapesType.SAME;
		this.sameDifStartEndHandshapesR = StartEndHandshapesType.SAME;

		rightHandSelectedHandshapes.clear();
		leftHandSelectedHandshapes.clear();
	}
}
