package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.ListCellRenderer;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.*;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeItem;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplate;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.handShapes.CSHandShapeTemplates;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.LeftRightHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.handshapeTypes.StartEndHandshapesType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.DblHandshapeDialog;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandShapesPaletteFormBean;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.handShapes.HandshapeDialog;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.fields.nonManualField.GlossEntityProperties;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignType;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.listener.WeakItemListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.ui.ButtonPanel;

public class HandShapePanel extends JPanel implements ItemListener, ActionListener {

	private static final long serialVersionUID = 1L;

	public static final String RIGHT_HAND_TITLE = "Right Hand";
	public static final String LEFT_HAND_TITLE = "Left Hand";
	public static final String RIGHT_N_LEFT_HANDS_TITLE = "Right and Left Hands";

	private JDialog parentDialog = null;

	private final JRadioButton fingerspelled = new JRadioButton("Fingerspelled");
	private final JRadioButton loan = new JRadioButton("Loan sign");
	private final JRadioButton number = new JRadioButton("Number");
	private final JRadioButton lexical = new JRadioButton("Lexical sign");
	private final JRadioButton classifier = new JRadioButton("Classifier");
	private final JRadioButton gesture = new JRadioButton("Gesture");
	private final JCheckBox nameSign = new JCheckBox("Name sign");

	private JRadioButton oneHanded = new JRadioButton("One-handed");
	private JRadioButton twoHanded = new JRadioButton("Two-handed");

	private JRadioButton dominantHand = new JRadioButton("Dominant Hand");
	private JRadioButton nonDominantHand = new JRadioButton("Non-Dominant Hand");

	private JButton selectHandshapes = new JButton("Select handshapes");
	private JButton clearHandshapes = new JButton("Clear");

	private AssociatedFieldsPanel associatedFieldsPanel = null;
	
	private JCheckBox oneFrameGloss = new JCheckBox("One-Frame Gloss");

	private JCheckBox markedForm = new JCheckBox("Marked # of Hands");
	private JCheckBox passiveBaseArm = new JCheckBox("Passive base arm");

	private JComboBox<LeftRightHandshapesType> sameDifLeftRightHandshapes = new JComboBox<LeftRightHandshapesType>(LeftRightHandshapesType.values());
	private JComboBox<StartEndHandshapesType> sameDifStartEndHandshapes = new JComboBox<StartEndHandshapesType>(StartEndHandshapesType.values());
	private JComboBox<StartEndHandshapesType> sameDifStartEndHandshapesL = new JComboBox<StartEndHandshapesType>(StartEndHandshapesType.values());
	private JComboBox<StartEndHandshapesType> sameDifStartEndHandshapesR = new JComboBox<StartEndHandshapesType>(StartEndHandshapesType.values());

	private final int ONE_HANDED = 1;
	private final int TWO_HANDED = 2;
	private final int ONE_HND_R = 3;
	private final int ONE_HND_L = 4;
	private final int FINGERSPELLED = 5;
	private final int LOAN = 6;
	private final int LEXICAL = 7;
	private final int CLASSIFIER = 8;
	private final int GESTURE = 9;
	private final int NUMBER = 10;

	private final int SELECT_HANDSHAPES = 0;
	private final int CLEAR_HANDSHAPES = 1;

	private ButtonGroup grp1 = new ButtonGroup();
	private ButtonGroup oneHndGrp = new ButtonGroup();

	// contains entered information. Loaded from preprocessor
	private HandShapeFormBean hshFormBean;
	// contains updated information. Will be used by postprocessor
	private HandShapeBean bean;

	private GlossChainedEventsEntity selectedEntity = null;
	private GlossFieldWrapper wrapper = null;
	// private Segment segment = null;

	private JPanel southPanel = null;

    private SelectedHandshapesPanel ndomHPanel = null;
	private SelectedHandshapesPanel domHPanel = null;

	private int selectedIndex = -1;

	private SameHandshapesLeftRightComboBoxListener sameDifLeftRightHandshapesComboBoxListener = null;
	private SameHandshapesStartEndComboBoxListener sameDifStartEndHandshapesComboBoxListener = null;
	private SameHandshapesStartEndRightComboBoxListener sameDifStartEndHandshapesRightComboBoxListener = null;
	private SameHandshapesStartEndLeftComboBoxListener sameDifStartEndHandshapesLeftComboBoxListener = null;

	public HandShapePanel(AssociatedFieldsPanel associatedFieldsPanel) {
		this.associatedFieldsPanel = associatedFieldsPanel;
		this.parentDialog = associatedFieldsPanel.getParentDialog();
		
		this.hshFormBean = associatedFieldsPanel.getHandShapeFormBean();
		this.bean = new HandShapeBean();

		this.bean.setSelectedDomHandShapeImages(hshFormBean.getSelectedDomHandShapeImages());
		this.bean.setSelectedNdomHandShapeImages(hshFormBean.getSelectedNdomHandShapeImages());
		this.bean.setType(hshFormBean.getType());
		this.bean.setRightHandSelected(hshFormBean.isRightHandSelected());

		this.wrapper = hshFormBean.getGlsFieldWrapper();
		this.selectedEntity = (GlossChainedEventsEntity) SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();
		//associatedFieldsPanel.getEnterButton().setEnabled(false);

		oneHanded.addItemListener(new WeakItemListener(this, oneHanded));
		twoHanded.addItemListener(new WeakItemListener(this, twoHanded));
		dominantHand.addItemListener(new WeakItemListener(this, dominantHand));
		nonDominantHand.addItemListener(new WeakItemListener(this, nonDominantHand));

		dominantHand.setActionCommand("" + ONE_HND_R);
		nonDominantHand.setActionCommand("" + ONE_HND_L);
		oneHanded.setActionCommand("" + ONE_HANDED);
		twoHanded.setActionCommand("" + TWO_HANDED);

		fingerspelled.setActionCommand("" + FINGERSPELLED);
		loan.setActionCommand("" + LOAN);
		lexical.setActionCommand("" + LEXICAL);
		classifier.setActionCommand("" + CLASSIFIER);
		gesture.setActionCommand("" + GESTURE);
		number.setActionCommand("" + NUMBER);

		fingerspelled.addActionListener(new WeakActionListener(this, fingerspelled));
		loan.addActionListener(new WeakActionListener(this, loan));
		number.addActionListener(new WeakActionListener(this, number));
		lexical.addActionListener(new WeakActionListener(this, lexical));
		classifier.addActionListener(new WeakActionListener(this, classifier));
		gesture.addActionListener(new WeakActionListener(this, gesture));

		selectHandshapes.setActionCommand("" + SELECT_HANDSHAPES);
		clearHandshapes.setActionCommand("" + CLEAR_HANDSHAPES);
		selectHandshapes.addActionListener(new WeakActionListener(this, selectHandshapes));
		clearHandshapes.addActionListener(new WeakActionListener(this, clearHandshapes));

		sameDifLeftRightHandshapes.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapes.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapesL.setRenderer(new CenteredTextCellRenderer());
		sameDifStartEndHandshapesR.setRenderer(new CenteredTextCellRenderer());

		java.awt.Font cFont = sameDifLeftRightHandshapes.getFont();
		java.awt.Font font = new java.awt.Font(cFont.getName(), cFont.getStyle(), 10);

		sameDifLeftRightHandshapes.setFont(font);
		sameDifStartEndHandshapes.setFont(font);
		sameDifStartEndHandshapesL.setFont(font);
		sameDifStartEndHandshapesR.setFont(font);

		Dimension d1 = sameDifLeftRightHandshapes.getPreferredSize();

		int w0 = sameDifStartEndHandshapesR.getPreferredSize().width;
		int w1 = sameDifStartEndHandshapesL.getPreferredSize().width;

		int w = w0 > w1 ? w0 : w1;

		Dimension d = new Dimension(2 * w + 10, d1.height);
		Dimension dd = new Dimension(w + 5, d1.height);

		sameDifLeftRightHandshapes.setPreferredSize(d);
		sameDifLeftRightHandshapes.setMaximumSize(d);
		sameDifLeftRightHandshapesComboBoxListener = new SameHandshapesLeftRightComboBoxListener(this);
		sameDifLeftRightHandshapes.addActionListener(new WeakActionListener(sameDifLeftRightHandshapesComboBoxListener, sameDifLeftRightHandshapes));
		
		sameDifStartEndHandshapes.setPreferredSize(d);
		sameDifStartEndHandshapes.setMinimumSize(d);		
		sameDifStartEndHandshapesComboBoxListener = new SameHandshapesStartEndComboBoxListener(this);
		sameDifStartEndHandshapes.addActionListener(new WeakActionListener(sameDifStartEndHandshapesComboBoxListener, sameDifStartEndHandshapes));

		sameDifStartEndHandshapesR.setRenderer(new DefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				JLabel comp = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				comp.setText(value.toString() + " (R)");
				return comp;
			}
		});

		sameDifStartEndHandshapesR.setMinimumSize(dd);
		sameDifStartEndHandshapesR.setSize(dd);
		sameDifStartEndHandshapesR.setPreferredSize(dd);
		sameDifStartEndHandshapesRightComboBoxListener = new SameHandshapesStartEndRightComboBoxListener(this);
		sameDifStartEndHandshapesR.addActionListener(new WeakActionListener(sameDifStartEndHandshapesRightComboBoxListener, sameDifStartEndHandshapesR));

		sameDifStartEndHandshapesL.setRenderer(new DefaultListCellRenderer() {
			@Override
			public Component getListCellRendererComponent(JList<?> list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
				JLabel comp = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
				comp.setText(value.toString() + " (L)");
				return comp;
			}
		});
		sameDifStartEndHandshapesL.setMinimumSize(dd);
		sameDifStartEndHandshapesL.setSize(dd);
		sameDifStartEndHandshapesL.setPreferredSize(dd);
		sameDifStartEndHandshapesLeftComboBoxListener = new SameHandshapesStartEndLeftComboBoxListener(this);
		sameDifStartEndHandshapesL.addActionListener(new WeakActionListener(sameDifStartEndHandshapesLeftComboBoxListener, sameDifStartEndHandshapesL));

		dominantHand.setMinimumSize(dd);
		dominantHand.setSize(dd);
		dominantHand.setPreferredSize(dd);
		dominantHand.setHorizontalAlignment(SwingConstants.CENTER);

		nonDominantHand.setMinimumSize(dd);
		nonDominantHand.setSize(dd);
		nonDominantHand.setPreferredSize(dd);
		nonDominantHand.setHorizontalAlignment(SwingConstants.CENTER);

		grp1.add(oneHanded);
		grp1.add(twoHanded);

		oneHndGrp.add(nonDominantHand);
		oneHndGrp.add(dominantHand);

		ButtonGroup classifierGroup = new ButtonGroup();
		classifierGroup.add(fingerspelled);
		classifierGroup.add(loan);
		classifierGroup.add(number);
		classifierGroup.add(lexical);
		classifierGroup.add(classifier);
		classifierGroup.add(gesture);


		setPreferredSize(new Dimension(500, 340));

		populate();
		init();
	}

	public void update() {
		this.hshFormBean = associatedFieldsPanel.getHandShapeFormBean();
		this.bean.setSelectedDomHandShapeImages(hshFormBean.getSelectedDomHandShapeImages());
		this.bean.setSelectedNdomHandShapeImages(hshFormBean.getSelectedNdomHandShapeImages());
		this.bean.setType(hshFormBean.getType());
		this.bean.setRightHandSelected(hshFormBean.isRightHandSelected());
		this.wrapper = hshFormBean.getGlsFieldWrapper();
		this.selectedEntity = (GlossChainedEventsEntity) SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

        ArrayList<HandShapeImage> rhImages =  new ArrayList<HandShapeImage>(hshFormBean.getSelectedDomHandShapeImages());
        ArrayList<HandShapeImage> lhImages =  new ArrayList<HandShapeImage>(hshFormBean.getSelectedNdomHandShapeImages());

        ndomHPanel.setHandshapes(lhImages);
		ndomHPanel.repaint();
		ndomHPanel.revalidate();

        domHPanel.setHandshapes(rhImages);
		domHPanel.repaint();
		domHPanel.revalidate();

		populate();
		removeAll();
		init();

		repaint();
		revalidate();
	}

    public void updateSelectedHandshapesPanels(ArrayList<HandShapeImage> domHandshapes, ArrayList<HandShapeImage> ndomHandshapes) {
		domHPanel.setHandshapes(domHandshapes);
		domHPanel.repaint();
		domHPanel.revalidate();
		
		ndomHPanel.setHandshapes(ndomHandshapes);
		ndomHPanel.repaint();
		ndomHPanel.revalidate();
		
		hshFormBean.setSelectedDomHandShapeImages(domHandshapes);
		hshFormBean.setSelectedNdomHandShapeImages(ndomHandshapes);
		selectedHandshapesPanelUpdated();
    }

	public void clean() {
		domHPanel.clean();
		ndomHPanel.clean();

		oneHanded.removeItemListener(this);
		twoHanded.removeItemListener(this);
		dominantHand.removeItemListener(this);
		nonDominantHand.removeItemListener(this);

		sameDifLeftRightHandshapes.removeActionListener(sameDifLeftRightHandshapesComboBoxListener);
	}
	
	public String getSelectedSameDifLeftRightHandshapesStr() {
		return (String)sameDifLeftRightHandshapes.getSelectedItem(); //.getSelectedIndex();
	} 

	public LeftRightHandshapesType getSelectedSameDifLeftRightHandshapes() {
		return (LeftRightHandshapesType) sameDifLeftRightHandshapes.getSelectedItem();
	}
	
	public void setSelectedSameDifLeftRightHandshapes(LeftRightHandshapesType type) {
		sameDifLeftRightHandshapes.setSelectedItem(type);
	}

	public void setSelectedSameDifLeftRightHandshapesVisible(boolean visible) {
		sameDifLeftRightHandshapes.setVisible(visible);
	}

	public StartEndHandshapesType getSelectedSameDifStartEndHandshapes() {
		return (StartEndHandshapesType) sameDifStartEndHandshapes.getSelectedItem();
	}
	
	public void setSelectedSameDifStartEndHandShapes(StartEndHandshapesType type) {
		sameDifStartEndHandshapes.setSelectedItem(type);
	}
	
	public void setSelectedSameDifStartEndHandshapesVisible(boolean visible) {
		sameDifStartEndHandshapes.setVisible(visible);
	}

	public StartEndHandshapesType getSelectedSameDifStartEndHandshapesL() {
		return (StartEndHandshapesType) sameDifStartEndHandshapesL.getSelectedItem();
	}
	
	public void setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType type) {
		sameDifStartEndHandshapesL.setSelectedItem(type);
	}
	
	public void setSelectedSameDifStartEndHandshapesLeftVisible(boolean visible) {
		sameDifStartEndHandshapesL.setVisible(visible);
	}

	public StartEndHandshapesType getSelectedSameDifStartEndHandshapesR() {
		return (StartEndHandshapesType) sameDifStartEndHandshapesR.getSelectedItem();
	}
	
	public void setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType type) {
		sameDifStartEndHandshapesR.setSelectedItem(type);
	}
	
	public void setSelectedSameDifStartEndHandshapesRightVisible(boolean visible) {
		sameDifStartEndHandshapesR.setVisible(visible);
	}
	
	public void setOneFrameGloss(boolean selected) {
		oneFrameGloss.setSelected(selected);
	}
	
	public boolean isOneFrameGloss() {
		return oneFrameGloss.isSelected();
	}
	
	public void setMarkedNumberOfHands(boolean selected) {
		markedForm.setSelected(selected);
	}

	public boolean isMarkedNumberOfHands() {
		return markedForm.isSelected();
	}

	public void setPassiveBaseArmEnabled(boolean enable) {
		passiveBaseArm.setEnabled(enable);
	}
	
	public void setPassiveBaseArmSelected(boolean selected) {
		passiveBaseArm.setSelected(selected);
	}
	
	public boolean isPassiveBaseArm() {
		return passiveBaseArm.isSelected();
	}

	public boolean isTwoHanded() {
		return twoHanded.isSelected();
	}

	public void setSelectedIndex(int selectedIndex) {
		this.selectedIndex = selectedIndex;
	}

	public void selectOneHanded() {
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			oneHanded.setSelected(true);
		}		
		
		//oneHanded.setSelected(true);
	}

	public void selectTwoHanded() {
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			twoHanded.setSelected(true);
		}
		//twoHanded.setSelected(true);
		
		// sameDifHandshapes.setSelectedIndex(0);
		// sameDifStartEndHandshapes.setSelectedIndex(0);
	}

	public HandShapeFormBean getHandShapeFormBean() {
		return hshFormBean;
	}

	public SelectedHandshapesPanel getSelectedLeftHandshapesPane() {
		return ndomHPanel;
	}

	public SelectedHandshapesPanel getSelectedRightHandshapesPane() {
		return domHPanel;
	}

	private JPanel selectedImagesPanel() {
		SS3CodingScheme scheme = hshFormBean.getSS3CodingScheme();

		JLabel domLbl = new JLabel("Dominant Hand");
		JLabel ndomLbl = new JLabel("Non-Dominant Hand");

		ArrayList<HandShapeImage> domhList = bean.getSelectedDomHandShapeImages();
		domHPanel = new SelectedHandshapesPanel(domhList, scheme, this);
		domHPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		ArrayList<HandShapeImage> ndomhList = bean.getSelectedNdomHandShapeImages();
		ndomHPanel = new SelectedHandshapesPanel(ndomhList, scheme, this);
		ndomHPanel.setBorder(BorderFactory.createLineBorder(Color.black));

		JPanel panel = new JPanel();
		GridBagLayout gridBag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		panel.setLayout(gridBag);

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.ipady = 0;
		c.weightx = 0.5;
		gridBag.setConstraints(domLbl, c);
		panel.add(domLbl, c);
		c.gridx = 1;

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(clearHandshapes);

		c.fill = GridBagConstraints.NORTHEAST;
		panel.add(buttonPanel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 28; // make this component tall
		c.ipadx = 300;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(domHPanel, c);
		panel.add(domHPanel);

		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 0; // reset to default
		c.weighty = 1.0; // request any extra vertical space
		c.ipady = 10;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(ndomLbl, c);
		panel.add(ndomLbl);

		c.gridx = 0;
		c.gridy = GridBagConstraints.RELATIVE;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.ipady = 28; // make this component tall
		c.ipadx = 300;
		c.weightx = 0.0;
		c.gridwidth = 3;
		gridBag.setConstraints(ndomHPanel, c);
		panel.add(ndomHPanel);

		panel.setBorder(BorderFactory.createTitledBorder("Selected Handshapes"));
		return panel;
	}

	/**
     * Updates and returns HandShapeBean object.
     * @return HandShapeBean object that has selected HandShapeImage objects for both hands
     */
    public HandShapeBean getHandShapeBean() {
		ArrayList<HandShapeImage> rhList = domHPanel.getHandshapes();
		ArrayList<HandShapeImage> lhList = ndomHPanel.getHandshapes();
		
        bean.setSelectedDomHandShapeImages(rhList);
        bean.setSelectedNdomHandShapeImages(lhList);

		// bean.setMarkedForm(markedForm.isSelected());
		return bean;
	}

	private void populate() {
		// this is done so that setSelectedIndex does not trigger a removal of nondominant handshape images
		// that should only occur with a mouse click on the sameDifHandshapes combobox
		// this is undone at the end of this function
		sameDifLeftRightHandshapesComboBoxListener.setIgnoreFlag(true);

		String fieldID = wrapper.getField().getFieldID();

		EventsEntity ent = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();

		GlossChainedEventsEntity entity = (GlossChainedEventsEntity) ent;
		
		GlossEntityProperties properties = entity.getGlossEntityProperties();
		//String prClassification = properties.getTwoHandedHandshapesClassification();
		
		String startEndHandshape = properties.getStartEndHandshape();
		String startEndHandshapeLeft = properties.getStartEndHandshapeLeftHand();
		String startEndHandshapeRight = properties.getStartEndHandshapeRightHand();

		twoHanded.setEnabled(true);

		if (fieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
			dominantHand.setSelected(true);
		} else {
			nonDominantHand.setSelected(true);
		}

		dominantHand.setEnabled(false);
		nonDominantHand.setEnabled(false);
		
		oneFrameGloss.setSelected(properties.isOneFrameGloss());
		oneFrameGloss.setEnabled(false);

		String twoHandedHandshapesClassification = properties
				.getTwoHandedHandshapesClassification();

		if (properties.isTwoHanded()) {
			twoHanded.setSelected(true);
			if (twoHandedHandshapesClassification == null
					|| twoHandedHandshapesClassification.trim().length() == 0) {
				sameDifLeftRightHandshapes.setSelectedIndex(0);
				sameDifStartEndHandshapes.setSelectedIndex(0);
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GlossEntityProperties.SAME_HANDSHAPES_LR)) {
				sameDifLeftRightHandshapes.setSelectedIndex(0);
				if (startEndHandshape
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapes.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapes.setSelectedIndex(0);
				}
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR)) {
				sameDifLeftRightHandshapes.setSelectedIndex(1);
				if (startEndHandshape
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapes.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapes.setSelectedIndex(0);
				}
			} else if (twoHandedHandshapesClassification
					.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_HANDSHAPES_LR)) {
				sameDifLeftRightHandshapes.setSelectedIndex(2);
				if (startEndHandshapeRight
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapesR.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapesR.setSelectedIndex(0);
				}

				if (startEndHandshapeLeft
						.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
					sameDifStartEndHandshapesL.setSelectedIndex(1);
				} else {
					sameDifStartEndHandshapesL.setSelectedIndex(0);
				}
			}
		} else {
			oneHanded.setSelected(true);
			if (startEndHandshape
					.equalsIgnoreCase(GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE)) {
				sameDifStartEndHandshapes.setSelectedIndex(1);
			} else {
				// startEndHandshape
				sameDifStartEndHandshapes.setSelectedIndex(0);
			}
		}

		markedForm.setSelected(properties.isMarkedNumberOfHands());
		passiveBaseArm.setSelected(properties.isPassiveBaseArm());
		
		// undo the ignore on the sameDifHandshapes listener
		sameDifLeftRightHandshapesComboBoxListener.setIgnoreFlag(false);
	}

	public void init() {
		GridBagLayout gbl = new GridBagLayout();
		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(gbl);

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = gbc.weighty = 0.5;

		gbc.gridwidth = 4;

		JPanel signTypePanel = new JPanel(new GridLayout(2, 4));
		signTypePanel.setBorder(new TitledBorder("Sign Type"));
		signTypePanel.add(fingerspelled);
		signTypePanel.add(loan);
		signTypePanel.add(number);
		signTypePanel.add(new JLabel());
		signTypePanel.add(lexical);
		signTypePanel.add(classifier);
		signTypePanel.add(gesture);
		signTypePanel.add(nameSign);

		centerPanel.add(signTypePanel, gbc);

		gbc.gridx = 0;
		gbc.gridy++;
		gbc.gridwidth = 1;
		centerPanel.add(oneHanded, gbc);

		gbc.gridx++;
		centerPanel.add(dominantHand, gbc);

		gbc.gridx++;
		centerPanel.add(nonDominantHand, gbc);


		gbc.gridx = 0;
		gbc.gridy++;
		gbc.gridwidth = 1;
		centerPanel.add(twoHanded, gbc);

		gbc.gridx++;
		gbc.gridwidth = 2;

		if (oneHanded.isSelected()) {
			centerPanel.add(sameDifStartEndHandshapes, gbc);
			sameDifStartEndHandshapes.setVisible(true);
		} else if (twoHanded.isSelected()) {
			centerPanel.add(sameDifLeftRightHandshapes, gbc);
			sameDifLeftRightHandshapes.setVisible(true);

			gbc.gridy++;
			if (sameDifLeftRightHandshapes.getSelectedIndex() == 0 || sameDifLeftRightHandshapes.getSelectedIndex() == 1) {
				centerPanel.add(sameDifStartEndHandshapes, gbc);
				sameDifStartEndHandshapes.setVisible(true);
			} else {
				gbc.gridwidth = 1;
				centerPanel.add(sameDifStartEndHandshapesL, gbc);
				sameDifStartEndHandshapesL.setVisible(true);
				gbc.gridx++;
				centerPanel.add(sameDifStartEndHandshapesR, gbc);
				sameDifStartEndHandshapesR.setVisible(true);
			}
		}

		gbc.gridx = 0;
		gbc.gridy++;
		gbc.gridwidth = 1;
		centerPanel.add(markedForm, gbc);

		gbc.gridx++;
		centerPanel.add(oneFrameGloss, gbc);


		gbc.gridx = 0;
		gbc.gridy++;
		centerPanel.add(passiveBaseArm, gbc);

		gbc.gridx += 2;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		centerPanel.add(selectHandshapes, gbc);

		setLayout(new BorderLayout());

		centerPanel.setBorder(new EmptyBorder(2, 2, 2, 2));
		add(centerPanel, BorderLayout.CENTER);

		southPanel = selectedImagesPanel();
		add(southPanel, BorderLayout.SOUTH);

		invalidate();
		revalidate();
		repaint();
		parentDialog.pack();
	}

	public void updateSelectedIconsPanel() {
		remove(southPanel);
		southPanel = selectedImagesPanel();
		add(southPanel, BorderLayout.SOUTH);
		invalidate();
		revalidate();
		repaint();
	}

	private JDialog proceedRemoveDialog = null;
	private JButton proceedRemoveBtn = null;
	private JButton cancelBtn = null;

	private void removeHandshapes() {

		if (proceedRemoveDialog == null) {
			proceedRemoveDialog = new JDialog(parentDialog, "Handshapes ", true);

			cancelBtn = new JButton("Cancel");
			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					twoHanded.setSelected(true);
					
					passiveBaseArm.setEnabled(true);
					
					proceedRemoveDialog.dispose();
				}
			});

			proceedRemoveBtn = new JButton("Proceed");
			proceedRemoveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					if (oneHanded.isSelected()) {
						selectOnehandedSign();
					}
					
					// ensure switch parameters on proceed
					if(isFingerspelled() || isNumber() || isLoan()) {
						sameDifStartEndHandshapes.setVisible(false);
						setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
						sameDifStartEndHandshapesL.setVisible(false);
						setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
						sameDifStartEndHandshapesR.setVisible(false);
						setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
						markedForm.setSelected(false);
					}

					// default for all types for proceeding to switch to one-handed is same handshapes on left/right
					sameDifLeftRightHandshapes.setSelectedIndex(0);
					
					passiveBaseArm.setSelected(false);
					passiveBaseArm.setEnabled(false);

					if (domHPanel != null && ndomHPanel != null) {
//						SelectedHandshapesPanel domSHPanel, ndomSHPanel;
//						if (associatedFieldsPanel.isDominantHandRight()) {
//							domSHPanel = domHPanel;
//							ndomSHPanel = ndomHPanel;
//						} else {
//							domSHPanel = ndomHPanel;
//							ndomSHPanel = domHPanel;
//						}

                        String fieldID = associatedFieldsPanel.getCurrentGlossEntity().getField().getFieldID();

						if (fieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
							ndomHPanel.setHandshapes(new ArrayList<HandShapeImage>());
							ndomHPanel.repaint();
							ndomHPanel.revalidate();
							hshFormBean.setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
						} else {
							domHPanel.setHandshapes(new ArrayList<HandShapeImage>());
							domHPanel.repaint();
							domHPanel.revalidate();
							hshFormBean.setSelectedDomHandShapeImages(new ArrayList<HandShapeImage>());
						}
					}

					proceedRemoveDialog.dispose();
				}
			});

			proceedRemoveDialog.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent we) {
					twoHanded.setSelected(true);
				}
			});

			JLabel l1 = new JLabel(" If you complete this operation,");
			JLabel l2 = new JLabel(
					"previously selected handshapes will be discarded.");
			JLabel l3 = new JLabel("");
			JLabel l4 = new JLabel("(This operation cannot be undone.)");
			JLabel l5 = new JLabel("");

			JPanel warningPanel = new JPanel();

			GridBagConstraints c = new GridBagConstraints();
			warningPanel.setLayout(new GridBagLayout());

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(1, 1, 1, 1);

			c.gridy = 0;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l1, c);

			c.gridy = 1;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l2, c);

			c.gridy = 2;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l3, c);

			c.gridy = 3;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l4, c);

			c.gridy = 4;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(l5, c);

			ButtonPanel buttonPanel = new ButtonPanel();
			buttonPanel.add(cancelBtn);
			buttonPanel.add(proceedRemoveBtn);

			c.gridy = 5;
			c.gridx = 2;
			c.gridwidth = 1;
			c.anchor = GridBagConstraints.EAST;
			warningPanel.add(buttonPanel, c);

			c.anchor = GridBagConstraints.WEST;
			c.gridy = 6;
			c.gridx = 0;
			c.gridwidth = 3;
			warningPanel.add(new JLabel(""), c);

			warningPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10,
					10));

			JPanel panel = new JPanel(new BorderLayout(20, 20));
			panel.add(warningPanel, BorderLayout.CENTER);

			Container contentPane = proceedRemoveDialog.getContentPane();
			contentPane.setLayout(new BorderLayout());
			contentPane.add(panel, BorderLayout.CENTER);
			proceedRemoveDialog.pack();
		}

		Point p = parentDialog.getLocation();
		int dx = (parentDialog.getWidth() - proceedRemoveDialog.getWidth()) / 2;
		int dy = (parentDialog.getHeight() - proceedRemoveDialog.getHeight()) / 2;

		int x = (p.x + dx);
		int y = (p.y + dy);

		proceedRemoveDialog.setLocation(x, y);
		proceedRemoveDialog.setVisible(true);
	}

	@Override
	public void itemStateChanged(ItemEvent event) {

		JRadioButton obj = (JRadioButton) event.getSource();
		String str = obj.getActionCommand();

		switch (Integer.parseInt(str)) {

		case ONE_HANDED:
			selectHandshapes.requestFocus();
			selectHandshapes.requestFocus(true);
			selectHandshapes.requestFocusInWindow();

			if (domHPanel != null && ndomHPanel != null) {

				ArrayList<HandShapeImage> rhList1 = domHPanel.getHandshapes();
				ArrayList<HandShapeImage> lhList1 = ndomHPanel.getHandshapes();

				if (oneHanded.isSelected() && !rhList1.isEmpty()
						&& !lhList1.isEmpty()) {
					removeHandshapes();
				} else if (oneHanded.isSelected()) {
					selectOnehandedSign();
				}
			} else if (oneHanded.isSelected()) {
				selectOnehandedSign();
				
				if(isFingerspelled() || isNumber() 
						|| isLoan()) {
					sameDifLeftRightHandshapes.setVisible(false);
					setSelectedSameDifLeftRightHandshapes(LeftRightHandshapesType.SAME);
					sameDifStartEndHandshapes.setVisible(false);
					setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesL.setVisible(false);
					setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesR.setVisible(false);
					setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
				}
				
				//Setting marked # hands to false as this is the default option in all cases.
				markedForm.setSelected(false);
			}
			
			break;

		case TWO_HANDED:

			if (twoHanded.isSelected()) {
				// dominantHand.setEnabled(false);
				// nonDominantHand.setEnabled(false);
				selectHandshapes.setEnabled(true);

				if(isFingerspelled() || isNumber() || isLoan()) {
					sameDifLeftRightHandshapes.setVisible(true);
					setSelectedSameDifLeftRightHandshapes(LeftRightHandshapesType.SAME);
					sameDifStartEndHandshapes.setVisible(false);
					setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesL.setVisible(false);
					setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesR.setVisible(false);
					setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
				} else {
					setSelectedSameDifLeftRightHandshapes(LeftRightHandshapesType.SAME);
					sameDifLeftRightHandshapes.setVisible(true);
					setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapes.setVisible(true);
					setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesL.setVisible(true);
					setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesR.setVisible(true);
				}
				

                //
                // boolean changed = isChanged();
                //
                associatedFieldsPanel.getEnterButton().setEnabled(true);

                //

				// if same handshapes L&R are selected and L or R handshape is
				// populated, double the selection
				removeAll();
				init();

				// if(sameDifHandshapes.getSelectedIndex() == 0) {
				ArrayList<HandShapeImage> rhList = domHPanel.getHandshapes();
				ArrayList<HandShapeImage> lhList = ndomHPanel.getHandshapes();

				if (rhList.isEmpty() && !lhList.isEmpty()) {
					domHPanel.setHandshapes(lhList);
					domHPanel.repaint();
					domHPanel.revalidate();
                    hshFormBean.setSelectedDomHandShapeImages(lhList);
				}

				if (lhList.isEmpty() && !rhList.isEmpty()) {
					ndomHPanel.setHandshapes(rhList);
					ndomHPanel.repaint();
					ndomHPanel.revalidate();
                    hshFormBean.setSelectedNdomHandShapeImages(rhList);
				}
				
				if(isFingerspelled() || isNumber() 
						|| isLoan()) {
					markedForm.setSelected(true);
					sameDifStartEndHandshapes.setVisible(false);
					setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesL.setVisible(false);
					setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
					sameDifStartEndHandshapesR.setVisible(false);
					setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
				}
				else {
					//For all non finger, number and loan signs, set marked # of hands to false.  This is to preserve the default behavior
					markedForm.setSelected(false);

					sameDifStartEndHandshapes.setVisible(true);
					sameDifStartEndHandshapesL.setVisible(true);
					sameDifStartEndHandshapesR.setVisible(true);
				}
				
				passiveBaseArm.setEnabled(true);
				passiveBaseArm.setSelected(false);
			}
			break;
		}
	}

	private void selectOnehandedSign() {
		// dominantHand.setEnabled(true);
		// nonDominantHand.setEnabled(true);
		selectHandshapes.setEnabled(true);

		removeAll();
		init();
	}
	
	// private JDialog d = null;

    private HandshapeDialog sameHandDialog = null;
    private DblHandshapeDialog twoHandedDialog = null;

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {

		case SELECT_HANDSHAPES:
			HandShapesPaletteFormBean palletFormBean = new HandShapesPaletteFormBean();

			SS3CodingScheme scheme = hshFormBean.getSS3CodingScheme();
			palletFormBean.setCodingScheme(scheme);

			CSHandShapeTemplates templates = scheme.getHandShapeTemplates();
			CSHandShapeTemplate handShapeTemplate = templates.getSelectedHandShapeTemplate();

			if (twoHanded.isSelected()) {
				// 2-handed
				int sameDifHandshapesSelectedIndex = sameDifLeftRightHandshapes.getSelectedIndex();

				if (sameDifHandshapesSelectedIndex == 0 || sameDifHandshapesSelectedIndex == 1) {
					// one palette for both hands
					String hndType;

					if (dominantHand.isSelected()) {
						palletFormBean.setSelectedHshImages(bean.getSelectedDomHandShapeImages());
						hndType = Constants.TYPE_DOM_HAND_SHAPE;
						if (associatedFieldsPanel.isDominantHandRight()) {
							palletFormBean.setRightHandSelected(true);
						} else {
							palletFormBean.setRightHandSelected(false);
						}
					} else {
						palletFormBean.setSelectedHshImages(bean.getSelectedNdomHandShapeImages());
						hndType = Constants.TYPE_NDOM_HAND_SHAPE;
						if (associatedFieldsPanel.isDominantHandRight()) {
							palletFormBean.setRightHandSelected(false);
						} else {
							palletFormBean.setRightHandSelected(true);
						}
					}

					if (isFingerspelled() || isLoan()) {
						selectedIndex = 1;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.FINGERSPELLED);
					} else if (isNumber()) {
						selectedIndex = 2;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.NUMBERS);
					} else {
						selectedIndex = 0;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.ALL);
					}

					ArrayList<CSHandShapeItem> items = handShapeTemplate
							.getHandShapePalette(handShapeTemplate.getSelectedPaletteID()).getHandShapeItems();
					ArrayList<SS3FieldValue> list = scheme.getCorrespondingFieldValues(items, hndType);

					palletFormBean.setValues(list);

					palletFormBean.setTitle(RIGHT_N_LEFT_HANDS_TITLE);
					palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT_N_LEFT);

 					if (selectedIndex < 0)
                        selectedIndex = 0;

                    loadOrUpdateSameHandshapeDialog(palletFormBean, bean, this,
                            hshFormBean.getSS3CodingScheme(), parentDialog,
                            selectedIndex);
				} else {
					// two palettes
					HandShapesPaletteFormBean palletFormBean2 = new HandShapesPaletteFormBean();

					boolean isDominantHandRight = associatedFieldsPanel.isDominantHandRight();

					String rid, lid = "";

					if (isFingerspelled() || isLoan()) {
						if (isDominantHandRight) {
							rid = CSHandShapeTemplate.FINGERSPELLED;
							lid = CSHandShapeTemplate.FINGERSPELLED_UNMARKED;
							palletFormBean.setPaletteID(1);
							palletFormBean2.setPaletteID(3);
						} else {
							rid = CSHandShapeTemplate.FINGERSPELLED_UNMARKED;
							lid = CSHandShapeTemplate.FINGERSPELLED;
							palletFormBean.setPaletteID(3);
							palletFormBean2.setPaletteID(1);
						}
					} else if (isNumber()) {
						// handShapeTemplate.selectPalette(CSHandShapeTemplate.NUMBERS);

						if (isDominantHandRight) {
							rid = CSHandShapeTemplate.NUMBERS;
							lid = CSHandShapeTemplate.NUMBERS_UNMARKED;
							palletFormBean.setPaletteID(2);
							palletFormBean2.setPaletteID(3);
						} else {
							rid = CSHandShapeTemplate.NUMBERS_UNMARKED;
							lid = CSHandShapeTemplate.NUMBERS;
							palletFormBean.setPaletteID(3);
							palletFormBean2.setPaletteID(2);
						}
					} else {
						// handShapeTemplate.selectPalette(CSHandShapeTemplate.ALL);
						if (isDominantHandRight) {
							rid = CSHandShapeTemplate.ALL;
							lid = CSHandShapeTemplate.ALL_UNMARKED;
							palletFormBean.setPaletteID(0);
							palletFormBean2.setPaletteID(3);
						} else {
							rid = CSHandShapeTemplate.ALL_UNMARKED;
							lid = CSHandShapeTemplate.ALL;
							palletFormBean.setPaletteID(3);
							palletFormBean2.setPaletteID(0);
						}
					}

					ArrayList<HandShapeImage> selectedDomHandShapeImages = bean.getSelectedDomHandShapeImages();
					ArrayList<HandShapeImage> selectedNdomHandShapeImages = bean.getSelectedNdomHandShapeImages();

					ArrayList<HandShapeImage> rightHandImages, leftHandImages;
					if (associatedFieldsPanel.isDominantHandRight()) {
						rightHandImages = selectedDomHandShapeImages;
						leftHandImages = selectedNdomHandShapeImages;
					} else {
						leftHandImages = selectedDomHandShapeImages;
						rightHandImages = selectedNdomHandShapeImages;
					}

					ArrayList<CSHandShapeItem> rhUnmarkedItems = handShapeTemplate.getHandShapePalette(rid).getHandShapeItems();
					ArrayList<SS3FieldValue> rhList = scheme.getCorrespondingFieldValues(rhUnmarkedItems, Constants.TYPE_DOM_HAND_SHAPE);
					handShapeTemplate.selectPalette(rid);
					handShapeTemplate.selectPalette2(lid);

					palletFormBean.setSelectedHshImages(rightHandImages);
					palletFormBean.setValues(rhList);
					palletFormBean.setRightHandSelected(isDominantHandRight);
					palletFormBean.setTitle(RIGHT_HAND_TITLE);
					palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT);

					ArrayList<CSHandShapeItem> lhUnmarkedItems = handShapeTemplate.getHandShapePalette(lid).getHandShapeItems();

					ArrayList<SS3FieldValue> lhList = scheme.getCorrespondingFieldValues(lhUnmarkedItems, Constants.TYPE_NDOM_HAND_SHAPE);

					palletFormBean2.setSelectedHshImages(leftHandImages);
					palletFormBean2.setValues(lhList);
					palletFormBean2.setRightHandSelected(!palletFormBean.isRightHandSelected());
					palletFormBean2.setTitle(LEFT_HAND_TITLE);
					palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.LEFT);

                    twoHandedDialog = new DblHandshapeDialog(palletFormBean, palletFormBean2, bean, this, hshFormBean.getSS3CodingScheme());

                    twoHandedDialog.setLocationRelativeTo(null); //parentDialog);
                    twoHandedDialog.setLocation(parentDialog.getWidth() + 15, 10);
                    twoHandedDialog.setVisible(true);
                    twoHandedDialog.toFront();
				}
			} else {
					if (isFingerspelled() || isLoan()) {
						selectedIndex = 1;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.FINGERSPELLED);
					} else if (isNumber()) {
						selectedIndex = 2;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.NUMBERS);
					} else {
						selectedIndex = 0;
						handShapeTemplate
								.selectPalette(CSHandShapeTemplate.ALL);
					}

				ArrayList<CSHandShapeItem> items = handShapeTemplate.getHandShapePalette(handShapeTemplate.getSelectedPaletteID()).getHandShapeItems();

				if (dominantHand.isSelected()) {
					palletFormBean.setSelectedHshImages(bean.getSelectedDomHandShapeImages());
					palletFormBean.setValues(scheme.getCorrespondingFieldValues(items, Constants.TYPE_DOM_HAND_SHAPE));
					if (associatedFieldsPanel.isDominantHandRight()) {
						palletFormBean.setRightHandSelected(true);
						palletFormBean.setTitle(RIGHT_HAND_TITLE);
						palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT);
					} else {
						palletFormBean.setRightHandSelected(false);
						palletFormBean.setTitle(LEFT_HAND_TITLE);
						palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.LEFT);
					}
				} else if (nonDominantHand.isSelected()) {
					palletFormBean.setSelectedHshImages(bean.getSelectedNdomHandShapeImages());
					palletFormBean.setValues(scheme.getCorrespondingFieldValues(items, Constants.TYPE_NDOM_HAND_SHAPE));
					if (associatedFieldsPanel.isDominantHandRight()) {
						palletFormBean.setRightHandSelected(false);
						palletFormBean.setTitle(LEFT_HAND_TITLE);
						palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.LEFT);
					} else {
						palletFormBean.setRightHandSelected(true);
						palletFormBean.setTitle(RIGHT_HAND_TITLE);
						palletFormBean.setPaletteType(HandShapesPaletteFormBean.PaletteType.RIGHT);
					}
				}

                loadOrUpdateSameHandshapeDialog(palletFormBean, bean,
                        this, hshFormBean.getSS3CodingScheme(), parentDialog,
                        selectedIndex);
			}
			break;

		case CLEAR_HANDSHAPES:
			bean.clearDomHandShapeImages();
			bean.clearNdomHandShapeImages();
			hshFormBean.setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
			hshFormBean.setSelectedDomHandShapeImages(new ArrayList<HandShapeImage>());
			if (sameHandDialog != null && isFingerspelled()) {
				sameHandDialog.getPalette().getPalette().actionPerformed(null);	// this is to clear the text inside the palette window
			}
			updateSelectedIconsPanel();
			break;

		case FINGERSPELLED:
			onFingerspelledLoanNumberSelected();
			associatedFieldsPanel.setCurrentSignType(SignType.FINGERSPELLED);
			break;
		case LOAN:
			onFingerspelledLoanNumberSelected();
			associatedFieldsPanel.setCurrentSignType(SignType.LOAN);
			break;
		case NUMBER:
			onFingerspelledLoanNumberSelected();
			associatedFieldsPanel.setCurrentSignType(SignType.NUMBER);
			break;
		case GESTURE:
			onGestureSelected();
			associatedFieldsPanel.setCurrentSignType(SignType.GESTURE);
			break;
		case LEXICAL:
			onLexicalClassifierSelected();
			associatedFieldsPanel.setCurrentSignType(SignType.LEXICAL);
			break;
		case CLASSIFIER:
			onLexicalClassifierSelected();
			associatedFieldsPanel.setCurrentSignType(SignType.CLASSIFIER);
			break;
		}
		
		// Enable/Disable Sign Bank buttons based on this sign's type
		associatedFieldsPanel.checkSignTypeAndUpdateSignBankFunctionality();
	}

    private void loadOrUpdateSameHandshapeDialog(HandShapesPaletteFormBean formBean,
                                                 HandShapeBean bean,
                                                 HandShapePanel panel,
                                                 SS3CodingScheme scheme,
                                                 JDialog dialog, int selectedIndex) {
        if(twoHandedDialog != null)
            twoHandedDialog.setVisible(false);

        if(sameHandDialog == null) {
             sameHandDialog = new HandshapeDialog(formBean, bean, panel, scheme,dialog, selectedIndex);
             sameHandDialog.setLocationRelativeTo(null); //parentDialog);
             sameHandDialog.setLocation(parentDialog.getWidth() + 15, 10);
        } else {
             sameHandDialog.update(formBean, bean, selectedIndex);
        }
        sameHandDialog.setVisible(true);
        sameHandDialog.toFront();
    }

	private boolean justLoaded = true;

	public void selectedHandshapesPanelUpdated() {
		ArrayList<HandShapeImage> ndomImages =  new ArrayList<HandShapeImage>(bean.getSelectedNdomHandShapeImages());
		ArrayList<HandShapeImage> domImages =  new ArrayList<HandShapeImage>(bean.getSelectedDomHandShapeImages());

		if (justLoaded) {
			justLoaded = false;
		} else {

			JButton enterHandshapes = associatedFieldsPanel.getEnterButton();

			if (twoHanded.isSelected()) {
				if (!ndomImages.isEmpty() && !domImages.isEmpty()) {
					enterHandshapes.setEnabled(true);
				} else {
					enterHandshapes.setEnabled(false);
				}
			} else if (oneHanded.isSelected()) {
				if (ndomImages.isEmpty() && domImages.isEmpty()) {
					enterHandshapes.setEnabled(false);
				} else {
					enterHandshapes.setEnabled(true);
				}
			}
		}
	}

	public boolean isOneHandedSelected() {
		return oneHanded.isSelected();
	}

	public boolean isTwoHandedSelected() {
		return twoHanded.isSelected();
	}

	public boolean isDomHandSelected() {
		return dominantHand.isSelected();
	}

	public boolean isNondomHandSelected() {
		return nonDominantHand.isSelected();
	}

	public String getSelectedSameDifferentLeftRightHandshapes() {
		int selectedIndex = sameDifLeftRightHandshapes.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_HANDSHAPES_LR;
		} else if (selectedIndex == 1) {
			return GlossEntityProperties.ALTERNATING_SAME_HANDSHAPES_LR;
		} else {
			return GlossEntityProperties.DIFFERENT_HANDSHAPES_LR;
		}
	}

	public String getSelectedSameDifferentStartEndHandshapes() {
		int selectedIndex = sameDifStartEndHandshapes.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	public String getSelectedSameDifferentLeftHandshape() {
		int selectedIndex = sameDifStartEndHandshapesL.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}

	public String getSelectedSameDifferentRightHandshape() {
		int selectedIndex = sameDifStartEndHandshapesR.getSelectedIndex();
		if (selectedIndex == 0) {
			return GlossEntityProperties.SAME_START_END_HANDSHAPE;
		} else {
			return GlossEntityProperties.DIFFERENT_START_END_HANDSHAPE;
		}
	}
	
	//NOTES:
	//Current approach is just to check if there are any handshapes currently
	//if not, then "defaults should apply" 
	//not sure if/how toggling visibility settings of the SameDif stuff should work
	//changing sign type from lexical, to, say, fingerspelled, no longer gets rid of 
	//the other info but the display in utterance view seems to disappear.
	
	void onFingerspelledLoanNumberSelected() {
		//look at whether or not these visibility settings should be toggled by default or not
		setSelectedSameDifStartEndHandshapesVisible(false);
		setSelectedSameDifStartEndHandshapesLeftVisible(false);
		setSelectedSameDifStartEndHandshapesRightVisible(false);
		//if there were no previously selected signs, set defaults
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			selectOneHanded();
			setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
			setPassiveBaseArmEnabled(false);
		}
	}

	void onLexicalClassifierSelected() {
		setSelectedSameDifStartEndHandshapesVisible(true);
		setSelectedSameDifLeftRightHandshapesVisible(true);
		//if there were no previously selected signs, set defaults
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {
			selectTwoHanded();
			setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
			setSelectedSameDifLeftRightHandshapes(LeftRightHandshapesType.SAME);
			setMarkedNumberOfHands(false);
			setPassiveBaseArmSelected(false);
		}
	}
	
	void onGestureSelected() {
		setSelectedSameDifStartEndHandshapesVisible(true);
		setSelectedSameDifStartEndHandshapesLeftVisible(true);
		setSelectedSameDifStartEndHandshapesRightVisible(true);
		if (domHPanel.getHandshapes().size() == 0 &&
				ndomHPanel.getHandshapes().size() == 0) {		
			selectOneHanded();
			setPassiveBaseArmEnabled(false);
			setMarkedNumberOfHands(false);
			setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
			setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
		}
	}

	public boolean isFingerspelled() {
		return fingerspelled.isSelected();
	}

	public void setFingerspelled(boolean selected) {
		fingerspelled.setSelected(selected);
	}

	public boolean isLoan() {
		return loan.isSelected();
	}

	public void setLoan(boolean selected) {
		loan.setSelected(selected);
	}

	public boolean isNumber() {
		return number.isSelected();
	}

	public void setNumber(boolean selected) {
		number.setSelected(selected);
	}

	public boolean isLexical() {
		return lexical.isSelected();
	}

	public void setLexical(boolean selected) {
		lexical.setSelected(selected);
	}

	public boolean isClassifier() {
		return classifier.isSelected();
	}

	public void setClassifier(boolean selected) {
		classifier.setSelected(selected);
	}

	public boolean isGesture() {
		return gesture.isSelected();
	}

	public void setGesture(boolean selected) {
		gesture.setSelected(selected);
	}

	public boolean isNameSign() {
		return nameSign.isSelected();
	}

	public void setNameSign(boolean selected) {
		nameSign.setSelected(selected);
	}

	public AssociatedFieldsPanel getAssociatedFieldsPanel() {
		return associatedFieldsPanel;
	}
}

class CenteredTextCellRenderer extends JLabel implements ListCellRenderer<Object> {
	private static final long serialVersionUID = 1L;

	public CenteredTextCellRenderer() {
	}

	@Override
	public Component getListCellRendererComponent(JList list, Object value,
			int index, boolean isSelected, boolean cellHasFocus) {

		setHorizontalAlignment(SwingConstants.CENTER);
		setText(value.toString());
		return this;
	}
}

class SameHandshapesLeftRightComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesLeftRightComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {
			panel.removeAll();
			panel.init();
			
			LeftRightHandshapesType selectedSameDifLeftRightHandshapesIndex = panel.getSelectedSameDifLeftRightHandshapes();
			HandShapeBean bean = panel.getHandShapeBean();

			if (selectedSameDifLeftRightHandshapesIndex == LeftRightHandshapesType.SAME || selectedSameDifLeftRightHandshapesIndex == LeftRightHandshapesType.ALTERNATING) {
				if (panel.isOneHandedSelected()) {
					if (panel.isDomHandSelected()) {
						bean.setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
						panel.getHandShapeFormBean().setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
					} else {
						bean.setSelectedDomHandShapeImages(new ArrayList<HandShapeImage>());
						panel.getHandShapeFormBean().setSelectedDomHandShapeImages(new ArrayList<HandShapeImage>());
					}
				} else {
					if (bean.getSelectedNdomHandShapeImages().isEmpty()) {
						bean.setSelectedNdomHandShapeImages(bean.getSelectedDomHandShapeImages());
						panel.getHandShapeFormBean().setSelectedNdomHandShapeImages(bean.getSelectedDomHandShapeImages());
					} else {
						bean.setSelectedDomHandShapeImages(bean.getSelectedNdomHandShapeImages());
						panel.getHandShapeFormBean().setSelectedDomHandShapeImages(bean.getSelectedNdomHandShapeImages());
					}
				}
			} else if (selectedSameDifLeftRightHandshapesIndex == LeftRightHandshapesType.DIFFERENT) {
				bean.clearNdomHandShapeImages();
				panel.getHandShapeFormBean().setSelectedNdomHandShapeImages(new ArrayList<HandShapeImage>());
			}
			panel.updateSelectedIconsPanel();
		}
	}
}

class SameHandshapesStartEndComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {			
			StartEndHandshapesType selectedSameDifStartEndHandshapesIndex = panel.getSelectedSameDifStartEndHandshapes();
			
			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesIndex == StartEndHandshapesType.DIFFERENT && panel.isOneFrameGloss()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
					panel.setSelectedSameDifStartEndHandShapes(StartEndHandshapesType.SAME);
			}
		}
	}
}

class SameHandshapesStartEndRightComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndRightComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {			
			StartEndHandshapesType selectedSameDifStartEndHandshapesRightIndex = panel.getSelectedSameDifStartEndHandshapesR();
			
			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesRightIndex == StartEndHandshapesType.DIFFERENT && panel.isOneFrameGloss()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
					panel.setSelectedSameDifStartEndHandshapesRight(StartEndHandshapesType.SAME);
			}
		}
	}
}

class SameHandshapesStartEndLeftComboBoxListener implements ActionListener {
	private HandShapePanel panel = null;
	private boolean ignore = false;

	public SameHandshapesStartEndLeftComboBoxListener(HandShapePanel panel) {
		this.panel = panel;
	}

	public void setIgnoreFlag(boolean ignore) {
		this.ignore = ignore;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if ( ! ignore) {			
			StartEndHandshapesType selectedSameDifStartEndHandshapesLeftIndex = panel.getSelectedSameDifStartEndHandshapesL();
			
			// disallow 1-frame and different start/end handshapes
			if (selectedSameDifStartEndHandshapesLeftIndex == StartEndHandshapesType.DIFFERENT && panel.isOneFrameGloss()) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SET_DIF_START_END_ONE_FRAME);
					panel.setSelectedSameDifStartEndHandshapesLeft(StartEndHandshapesType.SAME);
			}
		}
	}
}
