package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationDisplayPanel;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.util.*;

public class LocationPanel extends JPanel {

    private ArrayList<LocationDisplayPanel> locDispPanels = new ArrayList<LocationDisplayPanel>();

    public LocationPanel(AssociatedFieldsPanel afp) {

        // for each location type in the coding schema, a new LocationDisplayPanel is added to the list
        // in the order that is specified in the coding schema. To add a new location type, edit the
        // coding schema itself.
        final ArrayList<LocationType> locationTypes = SS3Singleton.getLocationConfig().getLocationTypes();
        for (LocationType locationType : locationTypes) {
            locDispPanels.add(new LocationDisplayPanel(locationType, afp));
        }

        // all LocationDisplayPanels are siblings of each other. Go through the list of panels
        // and set each panel's list of siblings to the list of all panels that doesn't include itself.
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            ArrayList<LocationDisplayPanel> siblings = new ArrayList<LocationDisplayPanel>();
            for (LocationDisplayPanel locationDisplayPanel : locDispPanels) {
                if (!locationDisplayPanel.equals(locDispPanel)) {
                    siblings.add(locationDisplayPanel);
                }
            }
            locDispPanel.setSiblingPanels(siblings);
            locDispPanel.init();
        }

        initUI();
    }

    private void initUI() {
        setLayout(new FlowLayout());
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            add(locDispPanel);
        }
    }

    public void resetDialogs() {
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            locDispPanel.resetDialog();
        }
    }

    public LocationInfo getCurrentLocationInfo() {
        HashMap<LocationType, Region> regions = new HashMap<LocationType, Region>();
        for (LocationDisplayPanel locDispPanel : locDispPanels) {
            regions.put(locDispPanel.getLocationType(), locDispPanel.getRegion());
        }
        return new LocationInfo(regions);
    }

    public void setCurrentLocationInfo(LocationInfo currentInfo) {
        ArrayList<LocationType> locationTypes = SS3Singleton.getLocationConfig().getLocationTypes();
        for (int i = 0; i < locDispPanels.size(); i++) {
            locDispPanels.get(i).setRegion(currentInfo.getLocation(locationTypes.get(i)));
        }
    }
}
