package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.text.DocumentFilter;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.listener.WeakMouseListener;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;

public class PaletteUtil {

	private HashMap<SS3FieldValue, JLabel> handshapes = new HashMap<SS3FieldValue, JLabel>();

	private HashMap<String, SS3FieldValue> letters = new HashMap<String, SS3FieldValue>();

	private SS3CodingScheme defaultCodingScheme = null;
	private HandShapePalletClient hsClient = null;

	private DocumentFilter filterOne = null;
	private LabelMouseListener labelMouseListener = null;
	private LetterTypedListener letterTypedListener = null;

	private boolean isLetter = false;

	private JTextField lettersTxtfld = new JTextField();

	public PaletteUtil(SS3CodingScheme defaultCodingScheme,
			HandShapePalletClient hsClient) {
		this.defaultCodingScheme = defaultCodingScheme;
		this.hsClient = hsClient;
	}

	public boolean isLetter() {
		return isLetter;
	}

	public HashMap<SS3FieldValue, JLabel> getHandshapes() {
		return handshapes;
	}

	public HandShapePalletClient getHandShapePalletClient() {
		return hsClient;
	}

	public JTextField getLettersTxtfld() {
		return lettersTxtfld;
	}

	public boolean containsLetter(String letter) {
		return letters.containsKey(letter);
	}

	public String getLetter(SS3FieldValue value) {
		Set<String> key = letters.keySet();
		Iterator<String> iterator = key.iterator();
		while (iterator.hasNext()) {
			String letter = iterator.next();
			SS3FieldValue value1 = letters.get(letter);
			if (value.getIid().equals(value1.getIid())) {
				return letter;
			}
		}
		return "";
	}

	public void setLetter(boolean isLetter) {
		if (isLetter) {
			if (letterTypedListener == null) {
				letterTypedListener = new LetterTypedListener(this);
				lettersTxtfld.addKeyListener(letterTypedListener);
			}
			if (filterOne == null) {
				filterOne = new NonNumericFilter();
				javax.swing.text.Document textDocOne = lettersTxtfld
						.getDocument();
				((javax.swing.text.AbstractDocument) textDocOne)
						.setDocumentFilter(filterOne);
			}
		}
		this.isLetter = isLetter();
		if (labelMouseListener != null) {
			labelMouseListener.setLetter(isLetter);
		}
	}

	public SS3FieldValue getFieldValueForLetter(String letter) {
		return letters.get(letter);
	}

	public void addLetter(String letter, SS3FieldValue value) {
		SS3FieldValue v = letters.get(letter);

		if (v == null) {
			letters.put(letter, value);
		}
	}

	public JLabel getHandhsapeVisualRepresentation(final SS3FieldValue value,
			Color backgroundColor) {

		if (labelMouseListener == null) {
			labelMouseListener = new LabelMouseListener(this);
			labelMouseListener.setLetter(isLetter);
		}

		String iid = value.getIid();
		String letter = value.getLetter();

		Set<SS3FieldValue> key = handshapes.keySet();
		Iterator<SS3FieldValue> iterator = key.iterator();
		while (iterator.hasNext()) {
			SS3FieldValue v = iterator.next();
			if (v.getIid().equalsIgnoreCase(iid)
					&& v.getLetter().equalsIgnoreCase(letter)) {
				return handshapes.get(v);
			}
		}

		HandShapeImage handshapeImage = new HandShapeImage(value,
				defaultCodingScheme);

		Image image = handshapeImage.getImage();

		JLabel label = new JLabel();
		Font lblFont = new Font(label.getFont().getName(), Font.PLAIN, 10);
		label.setFont(lblFont);
		label.setBorder(BorderFactory.createLineBorder(UIPalette
				.getBackground()));
		label.setBackground(UIPalette.getBackground());
		label.setOpaque(true);
		SameWidhtIcon icon = new SameWidhtIcon(image);
		label.setIcon(icon);
		label.setText(value.getLabelOrName());
		
		handshapes.put(value, label);

		label.setBackground(backgroundColor);
		int height = label.getPreferredSize().height;
		label.setPreferredSize(new Dimension(120, height));
		label.setMinimumSize(new Dimension(120, height));
		label.setMaximumSize(new Dimension(120, height));
 
		label.addMouseListener(new WeakMouseListener(labelMouseListener, label));

		return label;
	}
}
