package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.listener.WeakActionListener;
import edu.bu.signstream.grepresentation.listener.WeakMouseListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class SortedHandshapesPalette extends PalettePanel implements
		ActionListener {

	private static final long serialVersionUID = 1L;
	
	private JTextField tf = new JTextField();
	private JButton searchButton = new JButton("Search");
	private JButton enterButton = new JButton("Enter");
	private JPanel searchEnterPanel = new JPanel();

	private JButton expandAllButton = new JButton("Expand all triangles");
	private JButton collapseAllButton = new JButton("Collapse all triangles");

	private final int EXPAND_ALL = 1;
	private final int COLLAPSE = 2;
	private final int SEARCH = 3;
	private final int ENTER = 4;
	
	private ExpandButtonMouseListener mouseListener = null;

	private JPanel handshapesPanel = new JPanel();
	private GridBagConstraints c = new GridBagConstraints();
	private JScrollPane scrollPane = new JScrollPane(handshapesPanel);
	
	private PaletteUtil paletteUtil = null;

	private HashMap<String, SS3FieldValue> hm = new HashMap<String, SS3FieldValue>();

	private HashMap<FieldValueWrapper, TriangleLabel> loadedExpandableWrappers = new HashMap<FieldValueWrapper, TriangleLabel>();
	private ArrayList<ArrayList<FieldValueWrapper>> sortedHandshapesCollectionLeft = new ArrayList<ArrayList<FieldValueWrapper>>();
	private ArrayList<ArrayList<FieldValueWrapper>> sortedHandshapesCollectionCenter = new ArrayList<ArrayList<FieldValueWrapper>>();
	private ArrayList<ArrayList<FieldValueWrapper>> sortedHandshapesCollectionRight = new ArrayList<ArrayList<FieldValueWrapper>>();
	
	public SortedHandshapesPalette(ArrayList<SS3FieldValue> handshapes, PaletteUtil paletteUtil) {
		this.paletteUtil = paletteUtil;
		paletteUtil.setLetter(false);

		expandAllButton.setActionCommand("" + EXPAND_ALL);
		collapseAllButton.setActionCommand("" + COLLAPSE);
		searchButton.setActionCommand("" + SEARCH);

		for (int i = 0; i < handshapes.size(); i++) {
			Object obj = handshapes.get(i);
			SS3FieldValue value = (SS3FieldValue) obj;
			hm.put(value.getId(), value);
		}

		searchEnterPanel.setLayout(new GridLayout(1, 2, 1, 1));
		searchEnterPanel.add(searchButton);
		searchEnterPanel.add(enterButton);

		searchButton.setEnabled(false);
		enterButton.setEnabled(false);
		tf.setEnabled(false);

		int width = searchButton.getPreferredSize().width;
		int height = searchButton.getPreferredSize().height;
		enterButton.setPreferredSize(new Dimension(width, height));

		tf.setPreferredSize(new Dimension(2 * width, height));
		collapseAllButton.setPreferredSize(new Dimension(2 * width, height));
		expandAllButton.setPreferredSize(new Dimension(2 * width, height));

		expandAllButton.addActionListener(new WeakActionListener(this, this));
		collapseAllButton.addActionListener(new WeakActionListener(this, this));
		
		this.mouseListener = new ExpandButtonMouseListener(handshapesPanel, this);

		populate();
		initUI();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		case ENTER:

			break;
		case EXPAND_ALL:
			for (TriangleLabel t : loadedExpandableWrappers.values()) {
				if (!t.isExpended()) {
					t.action();
				}
			}
			handshapesPanel.removeAll();
			populateHahdshapesPanel();
			handshapesPanel.repaint();
			handshapesPanel.revalidate();
			break;
		case COLLAPSE:
			for (TriangleLabel t : loadedExpandableWrappers.values()) {
				if (t.isExpended()) {
					t.action();
				}
			}
			handshapesPanel.removeAll();
			populateHahdshapesPanel();
			handshapesPanel.repaint();
			handshapesPanel.revalidate();
			break;
		case SEARCH:
			break;
		}
	}

	private void initUI() {
		setLayout(new GridBagLayout());

		handshapesPanel.setLayout(new GridBagLayout());

		JPanel controlsPanel = new JPanel();
		GridBagLayout gridbag1 = new GridBagLayout();
		GridBagConstraints c1 = new GridBagConstraints();
		controlsPanel.setLayout(gridbag1);

		c1.insets = new Insets(1, 1, 1, 1);
		c1.gridwidth = 1;

		c1.gridy = 1;
		c1.gridx = 1;
		controlsPanel.add(tf, c1);
		c1.gridx = 2;
		controlsPanel.add(expandAllButton, c1);

		c1.gridy = 2;
		c1.gridx = 1;
		controlsPanel.add(searchEnterPanel, c1);
		c1.gridx = 2;
		controlsPanel.add(collapseAllButton, c1);

		populateHahdshapesPanel();

		Dimension dd = controlsPanel.getPreferredSize();

		scrollPane.setPreferredSize(new Dimension(450, 385 - (int) dd.getHeight())); 
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		c.gridwidth = 1;
		c.gridy = 1;
		c.gridx = 1;
		add(controlsPanel, c);

		c.gridwidth = 1;
		c.gridy = 2;
		c.gridx = 1;
		add(scrollPane, c);
	}
	
	public void populateHahdshapesPanel() {		
		handshapesPanel.removeAll();
		handshapesPanel.setBackground(UIPalette.getBackground());

		c.insets = new Insets(0, 0, 0, 0);
		
		ArrayList<FieldValueWrapper> visibleWrappersLeft = getVisibleWrappers(sortedHandshapesCollectionLeft);
		ArrayList<FieldValueWrapper> visibleWrappersCenter = getVisibleWrappers(sortedHandshapesCollectionCenter);
		ArrayList<FieldValueWrapper> visibleWrappersRight = getVisibleWrappers(sortedHandshapesCollectionRight);

		
		int longestHandshapeeList = visibleWrappersLeft.size();
		if (longestHandshapeeList < visibleWrappersCenter.size()) {
			longestHandshapeeList = visibleWrappersCenter.size();
		}

		if (longestHandshapeeList < visibleWrappersRight.size()) {
			longestHandshapeeList = visibleWrappersRight.size();
		}

		c.gridy = GridBagConstraints.RELATIVE;
		c.anchor = GridBagConstraints.PAGE_START;
		c.fill = GridBagConstraints.BOTH;

		int y = 1;
		int height = 10;
		for (int i = 0; i < longestHandshapeeList; i++) {
			c.gridwidth = 1;
			c.gridy = GridBagConstraints.RELATIVE;
			c.gridy = y;
			c.gridx = 1;

			if (visibleWrappersLeft.size() > i) {
				FieldValueWrapper wrapperLeft = visibleWrappersLeft.get(i);
				SS3FieldValue valueLeft = wrapperLeft.getSS3FieldValue();

				JLabel l1 = paletteUtil.getHandhsapeVisualRepresentation(valueLeft,
						wrapperLeft.getBackground());

				if (wrapperLeft.isFolder()) {
					TriangleLabel btn = loadedExpandableWrappers.get(wrapperLeft);
					if(btn == null) {
						btn = new TriangleLabel(wrapperLeft);
						btn.addMouseListener(new WeakMouseListener(mouseListener, btn));
						loadedExpandableWrappers.put(wrapperLeft, btn);
					}
					handshapesPanel.add(btn, c);
				} else {
					handshapesPanel.add(new JLabel(" "), c);
				}

				c.gridx = 2;
				c.weightx = 1;
				c.anchor = GridBagConstraints.PAGE_START;
				handshapesPanel.add(l1, c);
			}

			c.gridx = 3;

			if (visibleWrappersCenter.size() > i) {
				final FieldValueWrapper wrapperCenter = visibleWrappersCenter
						.get(i);
				final SS3FieldValue valueCenter = wrapperCenter
						.getSS3FieldValue();

				c.anchor = GridBagConstraints.NORTHEAST;

				JLabel l2 = paletteUtil.getHandhsapeVisualRepresentation(valueCenter,
						wrapperCenter.getBackground());

				if (wrapperCenter.isFolder()) {
					TriangleLabel btn = loadedExpandableWrappers.get(wrapperCenter);
					if(btn == null) {
						btn = new TriangleLabel(wrapperCenter);
						btn.addMouseListener(new WeakMouseListener(mouseListener, btn));
						loadedExpandableWrappers.put(wrapperCenter, btn);
					}

					handshapesPanel.add(btn, c);
				} else {
					handshapesPanel.add(new JLabel(" "), c);
				}

				c.gridx = 4;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l2, c);
			}

			c.gridx = 5;

			if (visibleWrappersRight.size() > i) {
				final FieldValueWrapper wrapperRight = visibleWrappersRight
						.get(i);
				final SS3FieldValue valueRight = wrapperRight
						.getSS3FieldValue();

				c.anchor = GridBagConstraints.NORTHEAST;

				JLabel l3 = paletteUtil.getHandhsapeVisualRepresentation(valueRight,
						wrapperRight.getBackground());

				if (wrapperRight.isFolder()) {

					TriangleLabel btn = loadedExpandableWrappers.get(wrapperRight);
					if(btn == null) {
						btn = new TriangleLabel(wrapperRight);
						btn.addMouseListener(new WeakMouseListener(mouseListener, btn));
						loadedExpandableWrappers.put(wrapperRight, btn);
					}

					handshapesPanel.add(btn, c);
				} else {
					JLabel l = new JLabel(" ");
					handshapesPanel.add(l, c);
				}

				c.gridx = 6;
				c.weightx = 1;
				c.anchor = GridBagConstraints.NORTHWEST;
				handshapesPanel.add(l3, c);
			}
			y++;
		}

		int j = 21 - y;
		for (int k = 0; k < j; k++) {
			c.gridwidth = 1;
			c.gridy = GridBagConstraints.RELATIVE;
			c.gridy = y;
			handshapesPanel.add(new JLabel(), c);
			c.gridx = 2;
			handshapesPanel.add(createPlainLabel(height), c);
			y++;
		}
	}

	private void populate() {
		Color color2 = new Color(255, 255, 255);
		Color color1 = new Color(123, 166, 172);
		Color color = color2;

		ArrayList<FieldValueWrapper> leftValues1 = new ArrayList<FieldValueWrapper>();
		leftValues1.add(createFieldValueWrapper("14", "HSH_014", "A", false,
				true, color));
		leftValues1.add(createFieldValueWrapper("12", "HSH_012", "10", false,
				false, color));
		leftValues1.add(createFieldValueWrapper("82", "HSH_082", "S", false,
				false, color));
		leftValues1.add(createFieldValueWrapper("34", "HSH_034", "cocked-S",
				false, false, color));
		leftValues1.add(createFieldValueWrapper("72", "HSH_072", "T", false,
				false, color));
		sortedHandshapesCollectionLeft.add(leftValues1);

		color = color1;
		ArrayList<FieldValueWrapper> leftValues2 = new ArrayList<FieldValueWrapper>();
		leftValues2.add(createFieldValueWrapper("1", "HSH_001", "1", false,
				true, color));
		leftValues2.add(createFieldValueWrapper("42", "HSH_042", "D", false,
				false, color));
		leftValues2.add(createFieldValueWrapper("79", "HSH_079", "X", false,
				false, color));
		leftValues2.add(createFieldValueWrapper("80", "HSH_080",
				"X-over-thumb", false, false, color));
		leftValues2.add(createFieldValueWrapper("22", "HSH_022", "bent-1",
				false, false, color));
		leftValues2.add(createFieldValueWrapper("53", "HSH_053", "G/Q", false,
				false, color));
		leftValues2.add(createFieldValueWrapper("49", "HSH_049",
				"flat-F/flat-G", false, false, color));
		leftValues2.add(createFieldValueWrapper("26", "HSH_026",
				"alt-G/bent-L", false, false, color));
		sortedHandshapesCollectionLeft.add(leftValues2);

		color = color2;
		ArrayList<FieldValueWrapper> leftValues3 = new ArrayList<FieldValueWrapper>();
		leftValues3.add(createFieldValueWrapper("76", "HSH_076", "U/H", false,
				true, color));
		leftValues3.add(createFieldValueWrapper("40", "HSH_040", "crvd-U",
				false, false, color));
		leftValues3.add(createFieldValueWrapper("29", "HSH_029", "bent-U",
				false, false, color));
		leftValues3.add(createFieldValueWrapper("83", "HSH_083", "cocked-U",
				false, false, color));
		leftValues3.add(createFieldValueWrapper("75", "HSH_075", "U-L", false,
				false, color));
		leftValues3.add(createFieldValueWrapper("86", "HSH_086", "bent-U-L",
				false, false, color));
		sortedHandshapesCollectionLeft.add(leftValues3);

		color = color1;
		ArrayList<FieldValueWrapper> leftValues4 = new ArrayList<FieldValueWrapper>();
		leftValues4.add(createFieldValueWrapper("77", "HSH_077", "V/2", false,
				true, color));
		leftValues4.add(createFieldValueWrapper("41", "HSH_041", "crvd-V",
				false, false, color));
		leftValues4.add(createFieldValueWrapper("88", "HSH_088", "bent-V",
				false, false, color));
		sortedHandshapesCollectionLeft.add(leftValues4);

		color = color2;
		ArrayList<FieldValueWrapper> leftValues5 = new ArrayList<FieldValueWrapper>();
		leftValues5.add(createFieldValueWrapper("2", "HSH_002", "3", false,
				true, color));
		leftValues5.add(createFieldValueWrapper("84", "HSH_084", "crvd-3",
				false, false, color));
		sortedHandshapesCollectionLeft.add(leftValues5);

		color = color1;
		ArrayList<FieldValueWrapper> leftValues6 = new ArrayList<FieldValueWrapper>();
		leftValues6.add(createFieldValueWrapper("8", "HSH_008", "6", false,
				true, color));
		leftValues6.add(createFieldValueWrapper("78", "HSH_078", "W", false,
				false, color));
		leftValues6.add(createFieldValueWrapper("85", "HSH_085", "crvd-W",
				false, false, color));
		sortedHandshapesCollectionLeft.add(leftValues6);

		color = color2;
		ArrayList<FieldValueWrapper> leftValues7 = new ArrayList<FieldValueWrapper>();
		leftValues7.add(createFieldValueWrapper("9", "HSH_009", "7", false,
				true, color));
		leftValues7.add(createFieldValueWrapper("90", "HSH_090", "cocked-7",
				false, false, color));
		leftValues7.add(createFieldValueWrapper("89", "HSH_089", "open-7",
				false, false, color));
		sortedHandshapesCollectionLeft.add(leftValues7);

		color = color2;
		ArrayList<FieldValueWrapper> centerValues1 = new ArrayList<FieldValueWrapper>();
		centerValues1.add(createFieldValueWrapper("10", "HSH_010", "8", false,
				true, color));
		centerValues1.add(createFieldValueWrapper("32", "HSH_032", "cocked-8",
				false, false, color));
		centerValues1.add(createFieldValueWrapper("66", "HSH_066", "open-8",
				false, false, color));
		centerValues1.add(createFieldValueWrapper("13", "HSH_013", "25", false,
				false, color));
		sortedHandshapesCollectionCenter.add(centerValues1);

		color = color1;
		ArrayList<FieldValueWrapper> centerValues2 = new ArrayList<FieldValueWrapper>();
		centerValues2.add(createFieldValueWrapper("44", "HSH_044", "F/9",
				false, true, color));
		centerValues2.add(createFieldValueWrapper("33", "HSH_033", "cocked-F",
				false, false, color));
		centerValues2.add(createFieldValueWrapper("67", "HSH_067", "open-F",
				false, false, color));
		sortedHandshapesCollectionCenter.add(centerValues2);

		color = color2;
		ArrayList<FieldValueWrapper> centerValues3 = new ArrayList<FieldValueWrapper>();
		centerValues3.add(createFieldValueWrapper("3", "HSH_003", "4", false,
				false, color));
		sortedHandshapesCollectionCenter.add(centerValues3);

		color = color1;
		ArrayList<FieldValueWrapper> centerValues4 = new ArrayList<FieldValueWrapper>();
		centerValues4.add(createFieldValueWrapper("4", "HSH_004", "5", false,
				true, color));
		centerValues4.add(createFieldValueWrapper("35", "HSH_035", "crvd-5",
				false, false, color));
		centerValues4.add(createFieldValueWrapper("5", "HSH_005", "5-C", false,
				false, color));
		centerValues4.add(createFieldValueWrapper("6", "HSH_006", "5-C-L",
				false, false, color));
		centerValues4.add(createFieldValueWrapper("7", "HSH_007", "5-C-tt",
				false, false, color));
		sortedHandshapesCollectionCenter.add(centerValues4);

		color = color2;
		ArrayList<FieldValueWrapper> centerValues5 = new ArrayList<FieldValueWrapper>();
		centerValues5.add(createFieldValueWrapper("20", "HSH_020", "B", false,
				true, color));
		centerValues5.add(createFieldValueWrapper("19", "HSH_019", "B-xd",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("46", "HSH_046", "flat-B",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("18", "HSH_018", "B-L",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("92", "HSH_092", "Vulcan",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("36", "HSH_036", "crvd-B",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("37", "HSH_037",
				"crvd-flat-B", false, false, color));
		centerValues5.add(createFieldValueWrapper("39", "HSH_039",
				"crvd-sprd-B", false, false, color));
		centerValues5.add(createFieldValueWrapper("23", "HSH_023", "bent-B",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("91", "HSH_091", "bent-B-xd",
				false, false, color));
		centerValues5.add(createFieldValueWrapper("24", "HSH_024", "bent-B-L",
				false, false, color));
		sortedHandshapesCollectionCenter.add(centerValues5);

		color = color1;
		ArrayList<FieldValueWrapper> centerValues6 = new ArrayList<FieldValueWrapper>();
		centerValues6.add(createFieldValueWrapper("30", "HSH_030", "C", false,
				true, color));
		centerValues6.add(createFieldValueWrapper("71", "HSH_071", "sml-C/3",
				false, false, color));
		centerValues6.add(createFieldValueWrapper("74", "HSH_074", "tight-C",
				false, false, color));
		centerValues6.add(createFieldValueWrapper("73", "HSH_073", "tight-C/2",
				false, false, color));
		sortedHandshapesCollectionCenter.add(centerValues6);

		color = Color.GRAY;
		ArrayList<FieldValueWrapper> centerValues7 = new ArrayList<FieldValueWrapper>();
		centerValues7.add(createFieldValueWrapper("94", "HSH_094", "Other",
				false, false, color));
		sortedHandshapesCollectionCenter.add(centerValues7);

		color = color2;
		ArrayList<FieldValueWrapper> rightValues1 = new ArrayList<FieldValueWrapper>();
		rightValues1.add(createFieldValueWrapper("65", "HSH_065", "O", false,
				true, color));
		rightValues1.add(createFieldValueWrapper("21", "HSH_021", "baby-O",
				false, false, color));
		rightValues1.add(createFieldValueWrapper("51", "HSH_051", "flat-O",
				false, false, color));
		rightValues1.add(createFieldValueWrapper("50", "HSH_050", "flat-O/2",
				false, false, color));
		rightValues1.add(createFieldValueWrapper("45", "HSH_045",
				"fanned-flat-O", false, false, color));
		sortedHandshapesCollectionRight.add(rightValues1);

		color = color1;
		ArrayList<FieldValueWrapper> rightValues2 = new ArrayList<FieldValueWrapper>();
		rightValues2.add(createFieldValueWrapper("43", "HSH_043", "E", false,
				true, color));
		rightValues2.add(createFieldValueWrapper("61", "HSH_061", "loose-E",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues2);

		color = color2;
		ArrayList<FieldValueWrapper> rightValues3 = new ArrayList<FieldValueWrapper>();
		rightValues3.add(createFieldValueWrapper("62", "HSH_062", "M", false,
				true, color));
		rightValues3.add(createFieldValueWrapper("15", "HSH_015", "alt-M",
				false, false, color));
		rightValues3.add(createFieldValueWrapper("27", "HSH_027", "bent-M",
				false, false, color));
		rightValues3.add(createFieldValueWrapper("52", "HSH_052", "full-M",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues3);

		color = color1;
		ArrayList<FieldValueWrapper> rightValues41 = new ArrayList<FieldValueWrapper>();
		rightValues41.add(createFieldValueWrapper("63", "HSH_063", "N", false,
				true, color));
		rightValues41.add(createFieldValueWrapper("16", "HSH_016", "alt-N",
				false, false, color));
		rightValues41.add(createFieldValueWrapper("28", "HSH_028", "bent-N",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues41);

		color = color2;
		ArrayList<FieldValueWrapper> rightValues4 = new ArrayList<FieldValueWrapper>();
		rightValues4.add(createFieldValueWrapper("58", "HSH_058", "P/K", false,
				true, color));
		rightValues4.add(createFieldValueWrapper("17", "HSH_017", "alt-P",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues4);

		color = color1;
		ArrayList<FieldValueWrapper> rightValues5 = new ArrayList<FieldValueWrapper>();
		rightValues5.add(createFieldValueWrapper("60", "HSH_060", "L", false,
				true, color));
		rightValues5.add(createFieldValueWrapper("59", "HSH_059", "L-X", false,
				false, color));
		rightValues5.add(createFieldValueWrapper("38", "HSH_038", "crvd-L",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues5);

		color = color2;
		ArrayList<FieldValueWrapper> rightValues6 = new ArrayList<FieldValueWrapper>();
		rightValues6.add(createFieldValueWrapper("70", "HSH_070", "R", false,
				true, color));
		rightValues6.add(createFieldValueWrapper("87", "HSH_087", "R-L", false,
				false, color));
		sortedHandshapesCollectionRight.add(rightValues6);

		color = color1;
		ArrayList<FieldValueWrapper> rightValues7 = new ArrayList<FieldValueWrapper>();
		rightValues7.add(createFieldValueWrapper("57", "HSH_057", "I", false,
				true, color));
		rightValues7.add(createFieldValueWrapper("81", "HSH_081", "Y", false,
				false, color));
		rightValues7.add(createFieldValueWrapper("56", "HSH_056", "I-L-Y",
				false, false, color));
		rightValues7.add(createFieldValueWrapper("95", "HSH_095", "bent-I-L-Y",
				false, false, color));
		rightValues7.add(createFieldValueWrapper("55", "HSH_055", "Horns",
				false, false, color));
		rightValues7.add(createFieldValueWrapper("25", "HSH_025", "bent-Horns",
				false, false, color));
		rightValues7.add(createFieldValueWrapper("64", "HSH_064", "O/2-Horns",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues7);

		color = Color.GRAY;
		ArrayList<FieldValueWrapper> rightValues8 = new ArrayList<FieldValueWrapper>();
		rightValues8.add(createFieldValueWrapper("93", "HSH_093", "Rlxd",
				false, false, color));
		sortedHandshapesCollectionRight.add(rightValues8);
	}

	private ArrayList<FieldValueWrapper> getVisibleWrappers(
			ArrayList<ArrayList<FieldValueWrapper>> sortedHandshapesCollection) {
		ArrayList<FieldValueWrapper> visibleWrappers = new ArrayList<FieldValueWrapper>();

		for (ArrayList<FieldValueWrapper> wrappers : sortedHandshapesCollection) {
			FieldValueWrapper folderWrapper = wrappers.get(0);
			visibleWrappers.add(folderWrapper);

			if (folderWrapper.isExpended()) {
				for (int i = 1; i < wrappers.size(); i++) {
					visibleWrappers.add(wrappers.get(i));
				}
			}
		}
		return visibleWrappers;
	}
}

class LabelMouseListener extends MouseAdapter {
	private PaletteUtil paletteUtil = null;
	private boolean isLetter = false;
	
	public LabelMouseListener(PaletteUtil paletteUtil) { 
		this.paletteUtil = paletteUtil;
	}
	
	public void setLetter(boolean isLetter) {
		this.isLetter = isLetter;
	} 
	
	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getSource() instanceof JLabel) {
			JLabel label = (JLabel)e.getSource();
			
			HashMap<SS3FieldValue, JLabel> icons =  paletteUtil.getHandshapes();

			Set<SS3FieldValue> keySet = icons.keySet();
			Iterator<SS3FieldValue> iterator = keySet.iterator();
		    while (iterator.hasNext()) {		    	
		    	SS3FieldValue value = iterator.next();
		    	JLabel label1 = icons.get(value);
		    	if(label.equals(label1)) {
		    		paletteUtil.getHandShapePalletClient().setItem(value);
		    		
		    		if(isLetter) {
		    			String str = paletteUtil.getLettersTxtfld().getText();
						String letter = value.getLetter(); 
						String newStr = str+letter;				
						paletteUtil.getLettersTxtfld().setText(newStr);
		    		}
		    		
		    		SS3Singleton.getApplicationStateController()
					.setLoadedCollectionUpdated(true);
		    		return;
		    	}
		    }
		}
	}
}

class ExpandButtonMouseListener extends MouseAdapter {
	private JPanel handshapesPanel = null;
	private SortedHandshapesPalette pallet = null;

	public ExpandButtonMouseListener(JPanel handshapesPanel,
			SortedHandshapesPalette pallet) {
		this.handshapesPanel = handshapesPanel;
		this.pallet = pallet;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getSource() instanceof TriangleLabel) {
			TriangleLabel tLabel = (TriangleLabel) e.getSource();
			tLabel.action();
			handshapesPanel.removeAll();
			pallet.populateHahdshapesPanel();
			handshapesPanel.repaint();
			handshapesPanel.invalidate();
			handshapesPanel.revalidate();
		}
	}
}

class TriangleLabel extends JLabel {
	private static final long serialVersionUID = 1L;
	private boolean expanded = false;
	private FieldValueWrapper wrapper = null;

	public TriangleLabel(FieldValueWrapper wrapper) {
		this.wrapper = wrapper;

		setBackground(UIPalette.getBackground());
		setPreferredSize(new Dimension(15, 30));
		setExpended(wrapper.isExpended());
		repaint();

		setBorder(BorderFactory.createLineBorder(Color.red));
	}

	public void click() {
		this.expanded = !expanded;
	}

	public void setExpended(boolean expanded) {
		this.expanded = expanded;
	}

	public boolean isExpended() {
		return expanded;
	}

	public void action() {
		wrapper.setExpended(!wrapper.isExpended());
		setExpended(wrapper.isExpended());
		repaint();
	}

	@Override
	public void paint(Graphics g) {

		if (!expanded) {
			g.drawLine(3, 3, 3, 12);
			g.drawLine(3, 3, 8, 8);
			g.drawLine(8, 8, 3, 13);
		} else {
			g.drawLine(3, 3, 13, 3);
			g.drawLine(3, 3, 8, 8);
			g.drawLine(8, 8, 13, 3);
		}
	}
}
