package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

/**
 * This class displays a {@link Region}'s image
 * and its child regions outlined with boxes that highlight
 * when the user mouses over them. Clicking on a sub-region in
 * the image will select the region in the Combo Box in the corresponding
 * {@link RegionSelectPanel}
 */
public class ImagePanel extends JPanel {

    private static final Color[] rainbow = new Color[]{
            Color.RED,
            Color.YELLOW,
            Color.GREEN,
            Color.BLUE,
            Color.CYAN,
            Color.MAGENTA,
    };

    private RegionSelectPanel ssPanel;
    private LocationImage image;
    private Region parentRegion;
    private Region clickedRegion;
    private Region hoveredRegion;

    private boolean flip;

    private boolean selectionStarted = false;
    private boolean initDefault = false;

    ImagePanel(final Region rcs, final RegionSelectPanel ssPanel) {
        this.ssPanel = ssPanel;
        this.parentRegion = rcs;

        image = SS3Singleton.getLocationConfig().getImage(rcs.getImageID());

        AssociatedFieldsPanel associatedFieldsPanel = SS3Singleton.getNonManuelFieldDialogs()
                .getMorphologicalPhonologicalInfoDialog().getAssociatedFieldsPanel();
        this.flip = !associatedFieldsPanel.getDominantHand().equals(image.getDominance());

        addMouseMotionListener(new MouseAdapter() {

            @Override
            public void mouseExited(MouseEvent e) {
                super.mouseExited(e);
                hoveredRegion = null;
                updateHoveredRegion();
            }

            @Override
            public void mouseMoved(MouseEvent e) {
                super.mouseMoved(e);

                int mx = e.getX(), my = e.getY();
                hoveredRegion = null;
                for (Region subregion : parentRegion.getSubregions()) {
                    ArrayList<Polygon> polygons = parentRegion.getPolygons(subregion);
                    for (Polygon poly : polygons) {
                        Polygon toCheck;
                        if (flip) {
                            toCheck = PolygonHelper.flipHorizontal(image.getBufferedImage().getWidth(), poly);
                        } else {
                            toCheck = poly;
                        }
                        if (toCheck.contains(mx,my)) {
                            hoveredRegion = subregion;
                            // todo break here
                        }
                    }
                }
                updateHoveredRegion();
                repaint();
            }
        });

        addMouseListener(new MouseAdapter() {
            @Override
            public void mouseClicked(MouseEvent e) {
                if (hoveredRegion != null) {
                    clickedRegion = hoveredRegion;
                    updateHoveredRegion();
                    updateDefaultRegion();
                    if (e.getClickCount() == 2) {
                        ssPanel.forward();
                    }
                }
            }
        });
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension(image.getBufferedImage().getWidth(), image.getBufferedImage().getHeight());
    }

    @Override
    public void paintComponent(Graphics _g) {
        super.paintComponent(_g);
        Graphics2D g = (Graphics2D) _g;

        g.clearRect(0, 0, image.getBufferedImage().getWidth(), image.getBufferedImage().getHeight());

        AffineTransform currentTransform = g.getTransform();
        if (flip) {
            Util.flipHorizontal(image.getBufferedImage().getWidth(), g);
        }

        g.drawImage(image.getBufferedImage(), 0, 0, image.getBufferedImage().getWidth(), image.getBufferedImage().getHeight(), this);

        int i = 0;
        for (Region subregion : parentRegion.getSubregions()) {
            if (subregion.equals(clickedRegion)) {
                g.setColor(Color.green);
                g.setStroke(new BasicStroke(3));
            } else {
                g.setColor(Color.white);
                g.setStroke(new BasicStroke(1.3f));
            }
            for (Polygon poly : parentRegion.getPolygons(subregion)) {
                g.drawPolygon(poly);
            }
            Color currentColor = rainbow[i++ % rainbow.length];
            if (hoveredRegion != null && hoveredRegion.equals(subregion)) {
                g.setColor(new Color(currentColor.getRed(), currentColor.getGreen(), currentColor.getBlue(), 128));
                for (Polygon poly : parentRegion.getPolygons(subregion)) {
                    g.fillPolygon(poly);
                }
            }
        }

        g.setTransform(currentTransform);
    }

    public void selectRegion(Region subregion) {
        hoveredRegion = subregion;
        repaint();
    }

    public void setClickedRegion(Region subregion) {
        clickedRegion = subregion;
    }

    private void updateHoveredRegion() {
        if (!selectionStarted && hoveredRegion != null) {
            selectionStarted = true;
        }

        if (selectionStarted && !initDefault) {
            if (ssPanel.getDefaultRegion() == null) {
                ssPanel.setDefaultRegion(parentRegion.getSubregions().get(0));
            }
            initDefault = true;
        }
        ssPanel.selectRegion(hoveredRegion);
    }

    private void updateDefaultRegion() {
        ssPanel.setDefaultRegion(clickedRegion);
    }

}