package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;

/**
 * This class stores the IDs for parent regions if ever a reference to them is needed,
 * as well as a list of regions and location types parsed from the coding schema. It
 * also contains utility methods to find regions or location types by their IDs.
 */
public class LocationConfig {
    public static final String REGION_FULL_BODY = "0";
    public static final String REGION_HEAD = "1";
    public static final String REGION_BODY = "2";
    public static final String REGION_ARM = "3";
    public static final String REGION_HAND = "4";

    public static String IMAGE_BASE_PATH;

    private final HashMap<String, LocationImage> images = new HashMap<String, LocationImage>();

    private final HashMap<String, Region> regions = new HashMap<String, Region>();

    private final ArrayList<LocationType> locationTypes = new ArrayList<LocationType>();

    public LocationImage getImage(String id) {
        return images.get(id);
    }

    public void addImage(LocationImage image) {
        images.put(image.getId(), image);
    }

    public Region getRegion(String id) {
        if (id.equals("-1")) {
            return Region.NONE;
        }
        return regions.get(id);
    }

    public HashMap<String, Region> getRegions() {
        return regions;
    }

    public void addRegion(Region rc) {
        regions.put(rc.getId(), rc);
    }

    public void addLocationType(LocationType locType) {
        if (!locationTypes.contains(locType)) {
            locationTypes.add(locType);
        }
        Collections.sort(locationTypes, new Comparator<LocationType>() {
            @Override
            public int compare(LocationType o1, LocationType o2) {
                return o1.getOrder() - o2.getOrder();
            }
        });
    }

    public ArrayList<LocationType> getLocationTypes() {
        return locationTypes;
    }

    public LocationType getLocationType(int id) {
        for (LocationType locationType : locationTypes) {
            if (locationType.getId() == id) {
                return locationType;
            }
        }
        return null;
    }
}
