package edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.Stack;

/**
 * This dialog allows the user to select a body region with the mouse. A panel displayed on this
 * dialog can potentially lead to opening another panel as a "child" of the current one, so a Stack
 * is used to keep track of the top-most panel, also allowing for us to go back to the last displayed panel.
 */
public class RegionSelectDialog extends JDialog {

    private LocationDisplayPanel locPanel;
    private JLabel topLevelNameLabel = new JLabel();
    private Stack<RegionSelectPanel> rspStack = new Stack<RegionSelectPanel>();

    public RegionSelectDialog(LocationDisplayPanel locPanel, String locationName) {
        super(SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog(), true);
        this.locPanel = locPanel;
        setTitle(locationName + " location");

        setDefaultCloseOperation(WindowConstants.HIDE_ON_CLOSE);

        Region currentRegion = locPanel.getRegion();
        if (!currentRegion.equals(Region.NONE)) {
            ArrayList<Region> hierarchy = new ArrayList<Region>();
            while (currentRegion != null) {
                hierarchy.add(currentRegion);
                currentRegion = currentRegion.getParent();
            }
            System.out.println(hierarchy);

            for (int i = hierarchy.size() - 1; i >= 1; i--) {
                RegionSelectPanel rsp = new RegionSelectPanel(hierarchy.get(i), this, i != hierarchy.size() - 1);
                Region child = hierarchy.get(i - 1);
                rsp.setDefaultRegion(child);
                rsp.selectRegion(child);
                rspStack.push(rsp);
            }
        } else {
            rspStack.push(new RegionSelectPanel(
                    SS3Singleton.getLocationConfig().getRegion(LocationConfig.REGION_FULL_BODY),
                    this,
                    false
            ));
        }
        initUI();
    }

    private void initUI() {
        Font size18 = new Font(null, Font.PLAIN,18);

        getContentPane().removeAll();

        setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));

        topLevelNameLabel.setText("Inside " + rspStack.peek().getRegion().getName());
        topLevelNameLabel.setFont(size18);
        topLevelNameLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

        add(topLevelNameLabel);
        add(rspStack.peek());
        pack();

        validate();
        getContentPane().repaint();
    }

    public void goBack() {
        rspStack.pop();
        initUI();
    }

    void selectedRegion(Region result, boolean cont) {
        if (result.isParent() && cont) {
            showNext(result);
        } else {
            locPanel.selectedRegion(result);
            setVisible(false);
        }
    }

    private void showNext(Region rc) {
        rspStack.push(new RegionSelectPanel(rc, this, true));
        initUI();
    }

    public void reset() {
        rspStack.clear();
        rspStack.push(new RegionSelectPanel(SS3Singleton.getLocationConfig().getRegion(LocationConfig.REGION_FULL_BODY), this, false));
        initUI();
    }

}
