package edu.bu.signstream.grepresentation.fields.graphField;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.Line2D;
import java.awt.geom.QuadCurve2D;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.GraphFileRow;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class GraphField extends NonManualField {

	private static final long serialVersionUID = 1L;

	private ArrayList<ArrayList<PointObject>> xcoordsArray;

	private List<GraphFileRow> graphRowList;
	
	private double ssMovieDuration;
	private double frameRate;
	private int numTotalFrames;
	private double realMovieDuration;
	private double factor;
	private double durationPerFrame;
	
	private int count = 0;
	
	private CoordinateTimeConverter converter = null;
	
	private List<PresentationEventsEntity> utterances = null;

	protected boolean maximized = false;

	public GraphField(SS3Track track, SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		super(segmentPanel, parentField);
		this.ss3Track = track;

		this.converter = segmentPanel.getZoomer().getCoordinateTimeConvertor();
		
		this.ssMovieDuration = segmentPanel.getMultipleMovieController().getMovieDuration();
		this.numTotalFrames = (int) segmentPanel.getMultipleMovieController().getTotalNoOfFrames();
		this.frameRate = segmentPanel.getMultipleMovieController().getFrameRate();
		// this is milliseconds apparently
//		this.realMovieDuration = segmentPanel.getMultipleMovieController().getFirstMovie().getMovieRealTime();
		this.realMovieDuration = segmentPanel.getMultipleMovieController().getMovieDurationMs();
		this.factor = ssMovieDuration / realMovieDuration;
		this.durationPerFrame = ssMovieDuration / numTotalFrames;

		this.utterances = this.getParentField().getEntities();
	}

	// override getFieldID and return tmp ID (to get SignSteamSegmentPanel to work
	// where indicated) until SS3Track is ready
	@Override
	public String getFieldID() {
		return ss3Track.getId();
	}

	public void setRawGraphList(List<GraphFileRow> rowList) {
		this.graphRowList = rowList;
	}

	@Override
	public void paint(Graphics2D g2d, double Y, Font font) {
		// copy the graphics passed in so we can set different rending hints specifically for this field
		Graphics2D g2 = (Graphics2D) g2d.create();		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_NORMALIZE);

		// NOTE: Y value input is towards the bottom of the field/wrapper onscreen, so use this as a starting point to plot
		int height = SegmentGraphReprUtil.getCorrectCellHeight(this);
		int wrapperHalfHeight = height / 2;
		int offset = maximized ? 16 : 2;
		
		// arbitary Y offset to make it work
		Y = Y - wrapperHalfHeight / 2 - offset;
		double newMin = Y - wrapperHalfHeight;
		double newMax = Y + wrapperHalfHeight;
		
		// limit range a bit because it looked like going outside the bounds otherwise
		newMin += offset;
		newMax -= offset;
		
		double xAxis = (newMin + newMax) / 2;

		// parse file

		xcoordsArray = convertFramesToMovieTimes();

		if (xcoordsArray == null || xcoordsArray.isEmpty()) {
			return;
		}
		// min as max, and max as min, because Y is really negative with respect to the
		// top left of the screen (where painting starts)
		double rawMax = getYMin(xcoordsArray);
		double rawMin = getYMax(xcoordsArray);
		
		// input data is normalized between [-1, 1]
		rawMax = -1;
		rawMin = 1;

		// styles for the graphics objects
		float px1 = 1f;
		float px2 = 2f;
		float[] dashingPattern = { px2, px2 };
		Color medianColor = Color.GRAY;
		Color graphColor = Color.RED;

		for (ArrayList<PointObject> pointsArray : xcoordsArray) {
			int currentArraySize = pointsArray.size();

			// draw median as a dotted line
			g2.setColor(medianColor);
			g2.setStroke(
					new BasicStroke(px1, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, px1, dashingPattern, px2));
			g2.draw(new Line2D.Double(getXMin(pointsArray), xAxis, getXMax(pointsArray), xAxis));

			PointObject prevP, currP;
			double[] x = new double[currentArraySize], y = new double[currentArraySize];
			double beginX, endX, beginY, beginRelY, endY, endRelY;
			
			// get all the xcoords and all the ycoords to supply to the spline interpolator
			for (int k = 1; k < currentArraySize; k++) {
				int i = k - 1;
				
				prevP = pointsArray.get(i);
				beginX = prevP.getXCoord();
				beginY = prevP.getYCoord();
				beginRelY = calculate(beginY, rawMin, rawMax, newMin, newMax);
				
				x[i] = beginX;
				y[i] = beginRelY;
				
				currP = pointsArray.get(k);
				endX = currP.getXCoord();
				endY = currP.getYCoord();
				endRelY = calculate(endY, rawMin, rawMax, newMin, newMax);
				
				x[k] = endX;
				y[k] = endRelY;
			}
			
			SplineInterpolator si = new SplineInterpolator();
			PolynomialSplineFunction psf = si.interpolate(x, y);
			
			g2.setStroke(new BasicStroke(px2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
			g2.setColor(graphColor);

			// draw the graph
			double lastDiff = 0;
			for (int c = 1; c < currentArraySize; c++) {
				int a = c - 1;
				
				PointObject pointA = pointsArray.get(a);
				double xa = pointA.getXCoord();
				double ya = calculate(pointA.getYCoord(), rawMin, rawMax, newMin, newMax);
				
				PointObject pointC = pointsArray.get(c);
				double xc = pointC.getXCoord();
				double yc = calculate(pointC.getYCoord(), rawMin, rawMax, newMin, newMax);

				// check if coords, when converted to signstream-time, are within the visible bounds - if not, then don't paint
				int xtime1 = converter.convertCoordinateToTime((int)xa);
				int xtime4 = converter.convertCoordinateToTime((int)xc);
				
				if (xtime4 < SegmentGraphReprUtil.utteranceViewVisibleMovieOnset
						|| xtime1 > SegmentGraphReprUtil.utteranceViewVisibleMovieOffset) {
					continue;
				}

				// if there is a large gap between the last x coordinate
				// and the current one, don't connect those points
				double currDiff = Math.abs(xc - xa);
				if (currDiff / lastDiff < 2) { // (more than a 200% increase -> don't draw)
					int precision = 1;
					double tension = 0.5;
					precisionPaint(precision, tension, g2, psf, xa, ya, xc, yc);
				}
				lastDiff = currDiff;
			}
		}
	}
	
	/**
	 * Precision corresponds to how many points to interpolate between the real (frame) points
	 * Following the expression: numPoints = Math.pow(2, p) - 1, where p (precision) >= 1 (can we go to zero? for no precision?))
	 * Recursive function which decreases 
	 * @param precision
	 */
	private void precisionPaint(int precision, double tension, Graphics2D g2, PolynomialSplineFunction psf, 
			double xa, double ya, double xc, double yc) {
		
		// base case - done interpolating extra points, and stop painting
		if (precision <= 0)
			return;

		// interpolated value b (between [a, c])
		double xb = (xa + xc) / 2;		
		double yb = psf.value(xb);

	    double distAB = Math.sqrt(Math.pow( (xb - xa), 2) + Math.pow( (yb - ya), 2) );
		double distBC = Math.sqrt(Math.pow( (xc - xb), 2) + Math.pow( (yc - yb) , 2) );
		
	    double fa = tension * distAB / (distAB+distBC);   // scaling factor for triangle Ta
	    double fb = tension * distBC / (distAB+distBC);   // ditto for Tb, simplifies to fb=t-fa
	    
	    double xACwidth = (xc - xa);
	    double yACheight = (yc - ya);
	    
	    double cp1x = xb - fa * xACwidth;
	    double cp1y = yb - fa * yACheight;
	    double cp2x = xb + fb * xACwidth;
	    double cp2y = yb + fb * yACheight;
	    
////	// A. method: curve using midpoints as control points (known wrong)
////	
////	shape.setCurve(beginX, beginRelY, 
////							cp1x, cp1y, 
////							cp2x, cp2y, 
////							endX, endRelY);
////	g2.draw(shape);				
//	
//	
//	    // B. method: 3 straight lines through point1, control1, control2, point2	
//	    g2.drawLine((int)xa, (int)ya, 
//	    		(int)xb, (int)yb);
//	    g2.drawLine((int)xb, (int)yb, 
//	    		(int)xc, (int)yc);

	    // C. method: calculate control points from the polynomial function - with equal derivatives on both sides, then graph curves
		// http://scaledinnovation.com/analytics/splines/aboutSplines.html
	    QuadCurve2D  quadShape = new QuadCurve2D.Double();
		quadShape.setCurve(xa, ya, 
								cp1x, cp1y, 
								xb, yb);
		g2.draw(quadShape);		
		quadShape.setCurve(xb, yb, 
								cp2x, cp2y, 
								xc, yc);
		g2.draw(quadShape);	
	    

		precision--;
		precisionPaint(precision, tension, g2, psf, xa, ya, xb, yb);
		precisionPaint(precision, tension, g2, psf, xb, yb, xc, yc);
	}

	private ArrayList<ArrayList<PointObject>> convertFramesToMovieTimes() {

		if (graphRowList == null || graphRowList.isEmpty()) {
			return null;
		}

		ArrayList<ArrayList<PointObject>> retArray = new ArrayList<ArrayList<PointObject>>();

		List<List<GraphFileRow>> gfFieldLists = getGraphFieldsByUtterances();

		PointObject pointObj = null;

		for (List<GraphFileRow> entryList : gfFieldLists) {

			if (entryList.size() != 0) {

				ArrayList<PointObject> utterancePoints = new ArrayList<PointObject>();

				for (GraphFileRow entry : entryList) {
					
					double timeInSeconds = entry.getTime() / 1000; // entry time (from file) is in milliseconds;
					double currentFrame = (frameRate * timeInSeconds);		// frameRate is framesPerSecond

					int currentSSTime = (int) Math.round(currentFrame * durationPerFrame);	// need to round, as coordinate converter expects an int
					int convertedCoord = converter.convertTimeToCoordinate(currentSSTime);

					pointObj = new PointObject(convertedCoord, entry.getFieldValue());
					
					// don't add the same point twice - TODO: necessary?
					boolean add = true;
					for (PointObject other: utterancePoints) {
						if (other.getXCoord().equals(pointObj.getXCoord())) {
							add = false;
							break;
						}
					}
					if (add) {
						utterancePoints.add(pointObj);
					}
				}
				if (utterancePoints != null && utterancePoints.size() > 0) {
					retArray.add(utterancePoints);
				}
			}
		}

		return retArray;
	}

	private List<List<GraphFileRow>> getGraphFieldsByUtterances() {
		List<List<GraphFileRow>> gfFieldLists = new ArrayList<List<GraphFileRow>>();
		List<GraphFileRow> gff = null;

		long packetDuration = segmentPanel.getMultipleMovieController().getPacketDuration();

		for (PresentationEventsEntity utterance : utterances) {
			gff = new ArrayList<GraphFileRow>();
			
			int utteranceStartTime = utterance.getStartTime();
			int utteranceEndTime = utterance.getEndTime();
			
			int lastIndex = 0;
			
			// only grab the utterances which are visible on screen currently - to prevent memory bloat on painting
			if (SegmentGraphReprUtil.utteranceViewVisibleMovieOnset <= utteranceEndTime) {
				if ( SegmentGraphReprUtil.utteranceViewVisibleMovieOffset <= utteranceStartTime) {
					continue;		// case: visible portion completely before utterance start, so ignore it
				} else { /* okay */ }		// case: visible portion shows this utterance either fully or partly, so include it
			}
			else if (SegmentGraphReprUtil.utteranceViewVisibleMovieOffset >= utteranceStartTime) {
				if (SegmentGraphReprUtil.utteranceViewVisibleMovieOnset >= utteranceEndTime) {
					continue;		// case: visible portion completey after utterance end, so ignore it
				} else { /* okay */ } 	// case: visible portion shows this utterance either fully or partly, so include it
			}

			// TODO: refactor this - start at previous loop's end to avoid repeating all the lines up to that point again
			for (int i = lastIndex; i < graphRowList.size(); i++, lastIndex++) {
				GraphFileRow row = graphRowList.get(i);
				
				double val = row.getTime() * factor;

				if (val >= utteranceStartTime - packetDuration && val <= utteranceEndTime + packetDuration) {
					gff.add(row);
				}
			}

			gfFieldLists.add(gff);
		}

		return gfFieldLists;
	}

	/*
	 * Put in: rawInput, return: newOutput
	 */
	private double calculate(double rawInput, double rawMin, double rawMax, double newMin, double newMax) {

		double rawRange = rawMax - rawMin;
		double newRange = newMax - newMin;
		double translatedInput = rawInput - rawMin;

		double numerator = newRange * translatedInput;
		double denominator = rawRange;

		double result = numerator / denominator + newMin;

		return result;
	}

	private double getYMin(ArrayList<ArrayList<PointObject>> xcoordsArray) {
		double min = Double.MAX_VALUE;

		for (int i = 0; i < xcoordsArray.size(); i++) {
			ArrayList<PointObject> pointsArray = xcoordsArray.get(i);
			for (int j = 0; j < pointsArray.size(); j++) {
				PointObject pointObj = pointsArray.get(j);
				double val = pointObj.getYCoord();
				if (val < min) {
					min = val;
				}
			}
		}

		return min;
	}

	private double getYMax(ArrayList<ArrayList<PointObject>> xcoordsArray) {
		double max = Double.MIN_VALUE;

		for (int i = 0; i < xcoordsArray.size(); i++) {
			ArrayList<PointObject> pointsArray = xcoordsArray.get(i);
			for (int j = 0; j < pointsArray.size(); j++) {
				PointObject pointObj = pointsArray.get(j);
				double val = pointObj.getYCoord();
				if (val > max) {
					max = val;
				}
			}
		}

		return max;
	}

	private double getXMin(ArrayList<PointObject> pointsArray) {
		double min = Double.MAX_VALUE;

		for (int i = 0; i < pointsArray.size(); i++) {
			PointObject pointObj = pointsArray.get(i);
			double val = pointObj.getXCoord();
			if (val < min) {
				min = val;
			}
		}

		return min;
	}

	private double getXMax(ArrayList<PointObject> pointsArray) {
		double max = Double.MIN_VALUE;

		for (int i = 0; i < pointsArray.size(); i++) {
			PointObject pointObj = pointsArray.get(i);
			double val = pointObj.getXCoord();
			if (val > max) {
				max = val;
			}
		}

		return max;
	}

	public boolean isMaximized() {
		return maximized;
	}

	public void toggleMaximized() {
		maximized = !maximized;
	}
}

class PointObject {

	Integer xcoord;
	Double ycoord;

	public PointObject() {
	}

	public PointObject(Integer xcoord, Double ycoord) {
		super();
		setXCoord(xcoord);
		setYCoord(ycoord);
	}

	public void setXCoord(Integer xcoord) {
		this.xcoord = xcoord;
	}

	public void setYCoord(Double ycoord) {
		this.ycoord = ycoord;
	}

	public Integer getXCoord() {
		return this.xcoord;
	}

	public Double getYCoord() {
		return this.ycoord;
	}

	@Override
	public String toString() {
		return "XCOORD: " + xcoord + ", YCOORD: " + ycoord + "\n";
	}
}