package edu.bu.signstream.grepresentation.fields.handShapeField;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.EventsIterator;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.GeneralHandShapePanelClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapeImage;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.HandShapePalletClient;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.NewHandShape;
import edu.bu.signstream.grepresentation.fields.handShapeField.dialogs.SelectedHandshapesPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class HandShapeFieldWrapper
   extends FieldWrapper
   implements HandShapePalletClient, GeneralHandShapePanelClient {

   private HandShapeField field;

   private JDialog d = null;

   private int mousePressedAltAt;

   public HandShapeFieldWrapper(
      HandShapeField field,
      SignStreamSegmentPanel segmentPanel) {
      super(field, segmentPanel);
      this.field = field;
   }
   
   public HandShapeField getHandShapeField() {
	   return field;
   }
   
	@Override
	public void removeAll() {}


   public EventsEntity getEntityWithRefID(String refEntityId) {
      return field.getEntityWithRefID(refEntityId);
   }


   @Override
public void highlight(boolean highlight, String refID) {
      field.highlight(highlight, refID);
      highlightDependentWrappers(highlight, refID);
   }

   private void highlightDependentWrappers(boolean highlight, String refID) {
       for (FieldWrapper wrapper : dependentWrappers) {
           wrapper.highlight(highlight, refID);
       }
   }

   @Override
public FieldWrapper getDependentFieldWrapper(String fieldID) {
      for (FieldWrapper wrp : dependentWrappers) {
         String fID = wrp.getField().getFieldID();
         if (fieldID.equalsIgnoreCase(fID))
            return wrp;
      }
      return null;
   }

   @Override
public void mouseClickedAlt(Point point) {
	   mousePressedAltAt = point.x;
   }

   @Override
public Event getSelectedEvent() {
      return field.getSelectedEvent();
   }

   @Override
public void unselectAll() {
      field.unselect();
       for (FieldWrapper wrapper : dependentWrappers) {
           wrapper.unselectAll();
       }
   }

   @Override
public boolean contains(int Y) {
      return field.contains(Y);
   }

   // MouseListener
   @Override
public void mouseClicked(Point point, JPanel panel) {}

   @Override
public void mouseRightClicked(Point point) {
   }

   @Override
public void mouseLeftClicked(Point point) {
   }

   @Override
public void mouseDblLeftClicked(Point point, JPanel panel) {
		  // 2016-11-07 removing all the below to disallow re-selection of the hs images from the main window - must be done via Morph-Phon window.
		  // With only one mechanism to change it, we can better control only "sanctioned" entries 
   }

   @Override
public void setItem(Object obj, boolean bothHands) {
      addItem(obj, field);

      if (bothHands) {
         String type = getType();
         FieldWrapper wrp;

         if (type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SHAPE)) {
            wrp =
               segmentPanel.getSegment(getField().getSegmentID()).getFieldWrapper(
                  Constants.TYPE_DOM_HAND_SHAPE);
         } else {
            wrp =
               segmentPanel.getSegment(getField().getSegmentID()).getFieldWrapper(
                  Constants.TYPE_NDOM_HAND_SHAPE);
         }

         HandShapeField hsFld = (HandShapeField) wrp.getField();
         addItem(obj, hsFld);
      }

      d.dispose();
      SS3Singleton.getSignStreamApplication().repaint();
   }

   @Override
public void setItem(Object obj) {
      addItem(obj, field);
      d.dispose();
      SS3Singleton.getSignStreamApplication().repaint();
   }

   private void addItem(Object obj, HandShapeField hsField) {
      SS3FieldValue value = (SS3FieldValue) obj;
      String valueID = value.getId();

      SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
      HandShapesImageManager hsManager =
         SS3Singleton.getHandShapesImageManager();
      SS3FieldTypeImg ftImg = cs.getImage(value.getIid());

      Image lImage = hsManager.getImage(ftImg.getPath1());
      Image sImage = hsManager.getImage(ftImg.getPath2());

      HandShapeEvent event;
      Event ev = hsField.getSelectedEvent();
      
      SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

      if (ev != null) {
         event = (HandShapeEvent) ev;
      } else {
         SS3Item item = new SS3Item();
         event = new HandShapeEvent(item, segmentPanel);
         event.setStartTimeCoordinate(mousePressedAltAt);

         EventsEntity entity = hsField.getEntityAt(mousePressedAltAt);

         HandShapeEntity hsEntity = (HandShapeEntity) entity;
         hsEntity.addHandShapeEvent(event);
      }

      event.setID(valueID);
      event.setLargeImage(lImage);
      event.setSmallImage(sImage);
      event.setSS3FieldTypeImg(ftImg);
   }

   @Override
public void mouseDblRightClicked(Point point, JPanel panel) {	   
   }

   // MouseMotionListener
   @Override
public void mouseMoved(Point point, JPanel panel) {
      if (!isEntitySelected()) {
         EventsEntity entity = field.getEntityAt(point.x);

         if (entity != null) {
            HandShapeEntity hsEntity = (HandShapeEntity) entity;
            HandShapeEvent hsEvent = (HandShapeEvent) field.getEventAt(point.x);

            if (hsEvent == null) {
               EventsIterator iterator = hsEntity.getEventsIterator();

               HandShapeEvent pHsEvent =
                  (HandShapeEvent) iterator.previous(point.x);
               HandShapeEvent nHsEvent =
                  (HandShapeEvent) iterator.next(point.x);

               if (pHsEvent == null) {
                  hsEvent = nHsEvent;
               } else if (nHsEvent == null) {
                  hsEvent = pHsEvent;
               } else {
                  int x1 = point.x - pHsEvent.getStartTimeCoordinate();
                  int x2 = nHsEvent.getStartTimeCoordinate() - point.x;
                  hsEvent = x1 <= x2 ? pHsEvent : nHsEvent;
               }

               hsEvent.setMagnifiedImage(true);
               panel.repaint();
               panel.setToolTipText(hsEvent.getToolTipText());
            }

            hsEvent.setMagnifiedImage(true);
            panel.repaint();
            panel.setToolTipText(hsEvent.getToolTipText());
         } else {
            panel.setToolTipText("");
         }

         panel.setCursor(getCursor(point));
      }
   }

   @Override
public void mousePressedAlt(Point point, JPanel panel) {
   }

    public void mouseReleasedAlt(Point point) {

    }

   @Override
public void mouseReleased(Point point, JPanel panel) {}

   @Override
public void mouseDragged(Point point, JPanel panel) {
   }

   @Override
public void addEventsEntity(EventsEntity entity) {
      field.addHandShapeEntity(entity);
   }

   @Override
public void delete(int eventAt) {
      EventsEntity entity = field.getEntityAt(eventAt);

      if (entity != null)
         field.deleteEntity(entity);
   }

    private Cursor getCursor(Point point) {

        Event event = field.getEventAt(point.x);
        if (event != null && event.isSelected())
            return new Cursor(Cursor.E_RESIZE_CURSOR); // .HAND_CURSOR;
        return new Cursor(Cursor.DEFAULT_CURSOR);
    }

   @Override
public void addHandShape(NewHandShape hs) {
      HandShapeEntity hsEntity = field.getEntityWithNewSubEntity();
      if (hsEntity != null) {
         NewHandShapeEntity newEntity = hsEntity.getNewHandShapeEntity();

         HandShapeSubEntity subEntity =
            new HandShapeSubEntity(segmentPanel, field);
         subEntity.determineParentEntity();

         SelectedHandshapesPanel panel = hs.getPanel();
         ArrayList<HandShapeImage> hShapes = panel.getHandshapes();

         if (hShapes.size() == 1) {
             HandShapeImage hsImg = hShapes.get(0);
             HandShapeImage clonedImg = hsImg.cloneHandShapeImage();
             hShapes.add(clonedImg);
         }

         int startTimeCoord = newEntity.getStartTimeCoord();
         int duration = newEntity.getEndTimeCoord() - startTimeCoord;

         int step = duration / (hShapes.size() - 1);

         for (int i = 0; i < hShapes.size(); i++) {
            HandShapeImage image = hShapes.get(i);
            String id = image.getSS3FieldValue().getId();
            SS3Item item = new SS3Item();
            item.setId(id);

            HandShapeEvent hsEvent = new HandShapeEvent(item, segmentPanel);
            int coordinate = startTimeCoord + i * step;

            hsEvent.setStartTimeCoordinate(coordinate);
            hsEvent.setEndTimeCoordinate(coordinate);
            subEntity.addHandShape(hsEvent);
         }

         hsEntity.setNewHandShapeEntity(null);
         hsEntity.addHandShapeSubEntity(subEntity);

         SS3Singleton.getSignStreamApplication().repaint();
      }
   }

   @Override
public void cancelNewHandShape() {
      HandShapeEntity hsEntity = field.getEntityWithNewSubEntity();
      hsEntity.setNewHandShapeEntity(null);
      SS3Singleton.getSignStreamApplication().repaint();
   }

   @Override
   public void mouseHoldReleased(Point point, JPanel panel) {
   }
}

class DeleteHandshapeEntityListener implements ActionListener {
	private EventsEntity entity = null; 
	private HandShapeField field = null;
	private JPanel panel = null;

	public DeleteHandshapeEntityListener(EventsEntity entity, HandShapeField field, JPanel panel) {
		this.entity = entity;
		this.field = field;
		this.panel = panel;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		final JPopupMenu popup = new JPopupMenu();

		JLabel msgLabel = new JLabel("Are you sure you want to delete the object?");
		UserInterfaceUtil.setLabelLNF(msgLabel);

		final JButton okBtn = new JButton("Proceed");
		final JButton cancelBtn = new JButton("Cancel");

		JPanel msgPanel = new JPanel();
		GridBagConstraints gbc = new GridBagConstraints();

		msgPanel.setLayout(new GridBagLayout());
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridwidth = 3;
		gbc.gridx = 0;
		gbc.gridy = 0;
		msgPanel.add(msgLabel, gbc);

		gbc.gridy = 1;
		gbc.gridx = 1;
		msgPanel.add(new JLabel(" "), gbc);

		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridy = 3;
		gbc.gridx = 0;
		msgPanel.add(new JLabel(" "), gbc);
		gbc.gridx = 1;
		msgPanel.add(okBtn, gbc);
		gbc.gridx = 2;
		msgPanel.add(cancelBtn, gbc);

		cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				popup.setVisible(false);
			}
		});

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {				
				field.deleteEntity(entity);
				
				field.deleteEmptyEntities();
				
				SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				
				panel.repaint();
				popup.setVisible(false);
			}
		});

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				okBtn.requestFocus();
			}
		});

		int width = msgPanel.getPreferredSize().width + 30;
		int height = msgPanel.getPreferredSize().height + 20;

		popup.add(msgPanel);
		popup.setPopupSize(new Dimension(width, height));
		popup.setLocation(entity.getStartTimeCoordinate(), 200);
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}
}

