package edu.bu.signstream.grepresentation.fields.handShapeField;

// import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.CollectionManager;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class HandShapesManager extends CollectionManager {
	// collection of HandShapeEvent objects
	private ArrayList<HandShapeEvent> collection = new ArrayList<HandShapeEvent>();

	private SignStreamSegmentPanel segmentPanel = null;

	public HandShapesManager(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	public void deleteEntity() {
		collection.clear();
	}

	public boolean hasEvents() {
		return !collection.isEmpty();
	}

	public void addHandShape(String vID, String toolTipText, int startTime) {
		TrackValue value = new TrackValue();
		value.setValueId(vID);
      TimeInfo timeInfo = new TimeInfo();
      timeInfo.setMovieTime(startTime);
		value.setStartTimeInfo(timeInfo);
		value.setTextValue(toolTipText);

		HandShapeEvent event = new HandShapeEvent(value, segmentPanel);
		collection.add(event);
		sortEvents();
	}

	public void addHandShape(TrackValue value) {
		HandShapeEvent event = new HandShapeEvent(value, segmentPanel);
		addHandShape(event);
	}

	public void addHandShape(SS3Item item) {
		HandShapeEvent event = new HandShapeEvent(item, segmentPanel);
      event.setImageID(item.getImageID());
		addHandShape(event);
	}

	public void addHandShape(HandShapeEvent event) {
		collection.add(event);
		sortEvents();
	}

	public ArrayList<HandShapeEvent> getHandShapesCollection() {
		return collection;
	}

	public HandShapeEvent getSelectedHandShapeEvent() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			if (event.isSelected())
				return event;
		}
		return null;
	}

	public void deleteHandShape(String vID, int startTime) {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			String vlID = "" + event.getID();
			if (vlID.equalsIgnoreCase(vID))
				collection.remove(event);
		}
		sortEvents();
	}

	public void sortEvents() {
		int keys[] = new int[collection.size()];
		HashMap map = new HashMap();

		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			map.put("" + event.getStartTimeInfo().getMovieTime(), event);
			keys[i] = event.getStartTimeInfo().getMovieTime();
		}

		Arrays.sort(keys);

		collection.removeAll(collection);
		for (int j = 0; j < keys.length; j++) {
			collection.add((HandShapeEvent)map.get("" + keys[j]));
		}
	}

	@Override
	public void unselectEntity() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			event.unselect();
		}
	}

	@Override
	public Event getSelectedEvent() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			if (event.isSelected())
				return event;
		}
		return null;
	}

	@Override
	public Event getEventAt(int x) {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);

			if (event.contains(x))
				return event;
		}
		return null;
	}

	@Override
	public Event getFirstEvent() {
		if (collection.size() == 0) {
			return null;
		}

		HandShapeEvent event = collection.get(0);
		for (int i = 1; i < collection.size(); i++) {
			HandShapeEvent hsEvent = collection.get(i);					
			if (hsEvent.getStartTimeInfo().getMovieTime() < event.getStartTimeInfo().getMovieTime())
				event = hsEvent;
		}
		return event;
	}

	@Override
	public Event getLastEvent() {
		if (collection.size() <= 0)
			return null;

		HandShapeEvent event = collection.get(0);
		for (int i = 1; i < collection.size(); i++) {
			HandShapeEvent hsEvent = collection.get(i);		
			if (hsEvent.getStartTimeInfo().getMovieTime() > event.getStartTimeInfo().getMovieTime())
				event = hsEvent;
		}
		return event;
	}

	@Override
	public void resetAll() {
		for (int i = 0; i < collection.size(); i++) {
			HandShapeEvent event = collection.get(i);
			event.reset();
		}
	}

	@Override
	public boolean adjustEntity(int startTime, int endTime) {
		setEntityStartTime(startTime);
		setEntityEndTime(endTime);
		return true;
	}

	public boolean setEntityStartTime(int time) {
		Event event = getFirstEvent();
		event.getStartTimeInfo().setMovieTime(time);
		return true;
	}

	public boolean setEntityEndTime(int time) {
		Event event = getLastEvent();
		event.getEndTimeInfo().setMovieTime(time);
		return true;
	}

	public boolean setSelectedEventStartTime(int time) {
		Event event = getSelectedEvent();
		event.getStartTimeInfo().setMovieTime(time);
		return true;
	}

	public boolean setSelectedEventEndTime(int time) {
		Event event = getSelectedEvent();
		event.getEndTimeInfo().setMovieTime(time);
		return true;
	}
}
