package edu.bu.signstream.grepresentation.fields.handShapes.joints;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class JointsEventsEntity extends ChainedEventsEntity {

   // list of ChainedEvents
   ArrayList events = new ArrayList();

   public JointsEventsEntity(
      String fieldID,
      SignStreamSegmentPanel segmentPanel, Field field) {

     super(new ChainedEventsCollection(), segmentPanel, field);
   }
   
   public void addEvent(JointsEvent event) {
      events.add(event);
   }


   public ArrayList getEvents() {
      return events;
   }

   @Override
public void setStartTime(int startTime) {
      Event event = getFirstEvent();
      if (event != null)
         event.getStartTimeInfo().setMovieTime(startTime);
   }

   @Override
public void setEndTime(int endTime) {
      Event event = getLastEvent();
      if (event != null)
         event.getEndTimeInfo().setMovieTime(endTime);
   }

   public int getStartTime() {
      Event event = getFirstEvent();
      if (event == null)
         return 0;
      return event.getStartTimeInfo().getMovieTime();
   }

   public int getEndTime() {
      Event event = getLastEvent();
      if (event == null)
         return 0;
      return event.getEndTimeInfo().getMovieTime();
   }

   @Override
public Event getFirstEvent() {
      if (events.isEmpty())
         return null;

      Event event = (Event) events.get(0);

      for (int i = 1; i < events.size(); i++) {
         Event evnt = (Event) events.get(i);
         if (evnt.getStartTimeInfo().getMovieTime() < event.getStartTimeInfo().getMovieTime())
            event = evnt;
      }
      return event;
   }

   @Override
public Event getLastEvent() {
      if (events.isEmpty())
         return null;

      Event event = (Event) events.get(0);

      for (int i = 1; i < events.size(); i++) {
         Event evnt = (Event) events.get(i);
         if (evnt.getStartTimeInfo().getMovieTime() > event.getStartTimeInfo().getMovieTime())
            event = evnt;
      }
      return event;
   }

   @Override
public Event getSelectedEvent() {
      for (int i = 1; i < events.size(); i++) {
         Event event = (Event) events.get(i);
         if (event.isSelected())
            return event;
      }
      return null;
   }

   @Override
public boolean selectEntityIfEventSelected() {
      Event event = getSelectedEvent();
      if (getSelectedEvent() != null) {
         event.unselect();
         return select();
      }
      return false;
   }

   @Override
public boolean selectEventIfEntitySelected() {
      return false;
   }

   @Override
public boolean select() {
      this.selected = true;
      return true;
   }

   @Override
public boolean unselect() {
      this.selected = false;
      return true;
   }

   @Override
public boolean adjustEntity(int x1, int x2) {
      return true;
   }

   @Override
public void reset() {
      for (int i = 1; i < events.size(); i++) {
         Event event = (Event) events.get(i);
         event.unselect();
      }
      unselect();
   }

   @Override
public boolean delete() {
      events.clear();
      return true;
   }

   @Override
public Event getEventAt(int x) {
      for (int i = 1; i < events.size(); i++) {
         ChainedEvent event = (ChainedEvent) events.get(i);
         if (event.contains(x))
            return event;
      }
      return null;
   }

   @Override
public ArrayList getEventsToAdd() {
      return new ArrayList();
   }

   @Override
public String getToolTipText(int x) {
      Event event = getEventAt(x);

      if (event == null)
         return "";

      return event.getToolTipText();
   }

   @Override
public boolean selectEvent(int x) {
      Event event = getEventAt(x);

      if (event == null)
         return false;

      return event.select();
   }
   
   @Override
public TimeInfo getStartTimeInfo() {
      Event event = getFirstEvent();
      if (event == null)
         return null;
      return event.getStartTimeInfo();
   }
      
   @Override
public TimeInfo getEndTimeInfo() {
      Event event = getLastEvent();
      if (event == null)
         return null;
      return event.getEndTimeInfo();
   }

   @Override
public void paint(Graphics2D g2d, double Y, Font font) {
   }
}
