package edu.bu.signstream.grepresentation.fields.handShapes.joints;

import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class JointsFieldWrapper extends NonManualFieldWrapper {

   protected JointsField jField;

   public JointsFieldWrapper(
      JointsField jField,
      SignStreamSegmentPanel segmentPanel) {
      super(jField, segmentPanel);
      this.jField = jField;
   }
   
   @Override
public void unselectAll() {
     jField.unselect();
   }
   
   @Override
public void mouseClicked(Point point, JPanel panel) {
      Event event = jField.getEventAt(point);
      if (event != null)
         event.select();
   }

   // not functional
   @Override
public void mouseRightClicked(Point point) {}

   // not functional
   @Override
public void mouseLeftClicked(Point point) {}

   @Override
public void mouseClickedAlt(Point point) {}

   // add new chained event
   public void mousePressedAlt(Point point) {}

   // change field ID
   @Override
public void mouseDblLeftClicked(final Point point, final JPanel panel) {

      final String fieldID = jField.getFieldID();
      final SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

      SS3Field field = cs.getField(fieldID);
      final ArrayList values = field.getValuesAsList();
      final JPopupMenu popup = new JPopupMenu();

      final ChainedEvent event = jField.getEventAt(point);

      if (event == null)
         return;

      for (int i = 0; i < values.size(); i++) {
         SS3FieldValue value = (SS3FieldValue) values.get(i);
         popup.add(new JLabel(value.getName()));
      }

      popup.setPreferredSize(popup.getPreferredSize());
      popup.show(panel, (int) point.getX(), (int) point.getY());
      popup.setVisible(true);
		
	  SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);

      popup.addMouseListener(new MouseAdapter() {
         @Override
		public void mousePressed(MouseEvent ev) {

            try {
               int popupHeight = popup.getPreferredSize().height;
               int choosenHeight = ev.getPoint().y;
               int elementHeight = popupHeight / (values.size() + 1);
               int choosenEl = choosenHeight / elementHeight;
               if (choosenHeight % elementHeight == 0)
                  choosenEl--;

               SS3FieldValue csValue = (SS3FieldValue) values.get(choosenEl);
               String id = csValue.getId();

               event.setID(id);
               panel.repaint();

               popup.setVisible(false);
            } catch (Exception e) {
               // ignore. IndexOutOfBoundException will occur when
               // user will click on the border of the tool bar.
            }
         }
      });
   }

   @Override
public void mouseDblRightClicked(Point point, JPanel panel) {}

   @Override
public void mouseMoved(Point point, JPanel panel) {}

   @Override
public void mouseDragged(Point point, JPanel panel) {}

   @Override
public void mouseReleased(Point point, final JPanel panel) {}
   //
}
