package edu.bu.signstream.grepresentation.fields.locationField;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;
import java.util.ArrayList;

public class LocationEntity extends EventsEntity {
    private LocationEventsCollection events;

    private GlossChainedEventsEntity refGlossEntity;

    public LocationEntity(String id, LocationEventsCollection events, SignStreamSegmentPanel panel, Field field) {
        this(events, panel, field);
        setID(id);
    }

    public LocationEntity(LocationEventsCollection events, SignStreamSegmentPanel segmentPanel, Field field) {
        super(segmentPanel, field);
        this.events = events;
    }

    public void setEvents(ArrayList<LocationEvent> events) {
        this.events.deleteAll();
        for (LocationEvent event : events) {
            this.events.addEvent(event);
        }
    }

    public boolean isEmptyLocationEntity() {
        for (LocationEvent event : events.getEvents()) {
            if (!event.isNoneLocation()) {
                return false;
            }
        }
        return true;
    }

    public LocationEventsCollection getLocationEventsCollection() {
        return events;
    }

    @Override
    public SS3Entity getSS3Entity() {
        SS3Entity ss3Entity = new SS3Entity();
        for (LocationEvent event : events.getEvents()) {
            ss3Entity.addItem(new SS3Item(event));
        }
        ss3Entity.setID(getID());
        return ss3Entity;
    }

    public void setStartTime(int startTime) {
        TimeInfo t = new TimeInfo();
        t.setMovieTime(startTime);
        getFirstEvent().setStartTimeInfo(t);
    }

    public void setEndTime(int endTime) {
        TimeInfo t = new TimeInfo();
        t.setMovieTime(endTime);
        getLastEvent().setEndTimeInfo(t);
    }

    @Override
    public int getStartTimeCoordinate() {
        return getFirstEvent().getStartTimeCoordinate();
    }

    // should return Start Time Coordinate!!!
    @Override
    public int getEndTimeCoordinate() {
        return getLastEvent().getStartTimeCoordinate();
    }

    @Override
    public Event getFirstEvent() {
        return events.getFirstEvent();
    }

    @Override
    public Event getLastEvent() {
        return events.getLastEvent();
    }

    public int getStartTime() {
        LocationEvent event = events.getFirstEvent();
        if (event == null)
            return 0;
        return event.getStartTimeInfo().getMovieTime();
    }

    public int getEndTime() {
        LocationEvent event = events.getLastEvent();
        if (event == null)
            return 0;
        return event.getEndTimeInfo().getMovieTime();
    }

    @Override
    public boolean selectEntityIfEventSelected() {
        return true;
    }

    public ArrayList<LocationEvent> getEvents() {
        return events.getEvents();
    }

    @Override
    public boolean selectEventIfEntitySelected() {
        return true;
    }

    @Override
    public boolean select() {
        return true;
    }

    @Override
    public boolean unselect() {
        return true;
    }

    @Override
    public boolean adjustEntity(int x1, int x2) {
        CoordinateTimeConverter convertor =
                segmentPanel.getZoomer().getCoordinateTimeConvertor();
        double startTime = convertor.convertCoordinateToTime(x1);
        double endTime = convertor.convertCoordinateToTime(x2);
        return events.adjustEntity((int) startTime, (int) endTime);
    }

    @Override
    public Event getSelectedEvent() {
        return null;
    }

    @Override
    public void reset() {
        selected = false;
        events.resetAll();
    }

    @Override
    public boolean delete() {
        return events.deleteAll();
    }

    @Override
    public Event getEventAt(int x) {
        return null;
    }

    @Override
    public ArrayList<String> getEventsToAdd() {
        return null;
    }

    @Override
    public String getToolTipText(int x) {
        StringBuilder str = new StringBuilder("[");
        for (int i = 0; i < events.size() - 1; i++) {
            str.append(events.getEvent(i).getToolTipText()).append(" -> ");
        }
        str.append(events.getEvent(events.size() - 1).getToolTipText()).append("]");
        return str.toString();
    }

    @Override
    public boolean selectEvent(int x) {
        return true;
    }

    @Override
    public void paint(Graphics2D g2d, double Y, Font font) {
        if (isEmptyLocationEntity()) {
            return;
        }

        g2d.setFont(font);

        int X1 = getStartTimeCoordinate();
        int X2 = getEndTimeCoordinate();

        int y = (int) Y;

        ArrayList<LocationEvent> eventsList = getLocationEventsCollection().getEvents();

        g2d.setColor(Color.BLACK);

        for (int i = 0; i < eventsList.size(); i++) {
            LocationEvent locationEvent = eventsList.get(i);
            if (!locationEvent.isNoneLocation()) {
                String currentText = locationEvent.getText();
                int x;
                if (i == 0) {
                    x = X1;
                } else if (i == eventsList.size() - 1) {
                    x = X2 - g2d.getFontMetrics().stringWidth(currentText);
                } else {
                    x = locationEvent.getStartTimeCoordinate() - g2d.getFontMetrics().stringWidth(currentText) / 2;
                }
                g2d.drawString(currentText, x, y);
            }
        }

        g2d.drawLine(X1, y + 1, X2, y + 1);
        g2d.drawLine(X1, y + 2, X2, y + 2);
    }

    @Override
    public String toString() {
        return "LocationEntity: " + super.toString();
    }

    @Override
    public TimeInfo getStartTimeInfo() {
        LocationEvent event = events.getFirstEvent();
        if (event == null)
            return null;
        return event.getStartTimeInfo();
    }

    @Override
    public TimeInfo getEndTimeInfo() {
        LocationEvent event = events.getLastEvent();
        if (event == null)
            return null;
        return event.getEndTimeInfo();
    }

    @Override
    public void setStartTimeInfo(TimeInfo startTimeInfo) {
        LocationEvent event = events.getFirstEvent();
        if (event != null)
            event.setStartTimeInfo(startTimeInfo);
    }

    @Override
    public void setEndTimeInfo(TimeInfo endTimeInfo) {
        LocationEvent event = events.getLastEvent();
        if (event != null)
            event.setEndTimeInfo(endTimeInfo);
    }

    public void updateTimes() {
        if (refGlossEntity != null) {
            setStartTimeInfo(refGlossEntity.getStartTimeInfo().cloneTimeInfo());
            setEndTimeInfo(refGlossEntity.getEndTimeInfo().cloneTimeInfo());
        }

        for (int i = 0; i < events.size(); i++) {
            LocationEvent event = events.getEvent(i);
            int time;
            if (i == 0) {
                time = getStartTime();
            } else if (i == events.size() - 1) {
                time = getEndTime();
            } else {
                time = getStartTime() + (i * (getEndTime() - getStartTime()) / (events.size() - 1));
            }
            TimeInfo startTimeInfo = new TimeInfo();
            startTimeInfo.setMovieTime(time);
            event.setStartTimeInfo(startTimeInfo);
        }
    }

    @Override
    public boolean contains(int x) {
        return super.contains(x);
    }

    public void setRefGlossEntity(GlossChainedEventsEntity refDomGlossEntity) {
        this.refGlossEntity = refDomGlossEntity;
    }
}
