package edu.bu.signstream.grepresentation.fields.locationField;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

import java.awt.*;

public class LocationEvent implements Event {

	private String id;

	private SignStreamSegmentPanel segmentPanel;

	private LocationType locationType;
	private Region location;

	private TimeInfo startTimeInfo = new TimeInfo();
	private TimeInfo endTimeInfo = new TimeInfo();

	public LocationEvent(SignStreamSegmentPanel segmentPanel) {
		this(segmentPanel, null, Region.NONE);
	}

	public LocationEvent(SignStreamSegmentPanel segmentPanel, LocationType locationType, Region location) {
		this.segmentPanel = segmentPanel;
		this.locationType = locationType;
		this.location = location;
	}

	public boolean isNoneLocation() {
		return location.equals(Region.NONE);
	}

	@Override
	public TimeInfo getStartTimeInfo() {
		return startTimeInfo;
	}

	@Override
	public TimeInfo getEndTimeInfo() {
		return endTimeInfo;
	}

	@Override
	public void setStartTimeInfo(TimeInfo startTimeInfo) {
		this.startTimeInfo = startTimeInfo;
		this.endTimeInfo = startTimeInfo.cloneTimeInfo();
	}

	@Override
	public void setEndTimeInfo(TimeInfo endTimeInfo) {
		this.endTimeInfo = endTimeInfo;
		this.startTimeInfo = endTimeInfo.cloneTimeInfo();
	}

	@Override
	public int getStartTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(startTimeInfo.getMovieTime());
	}

	@Override
	public int getEndTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(endTimeInfo.getMovieTime());
	}

	@Override
	public void setStartTimeCoordinate(int coord) {
		int newMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);
		startTimeInfo.setMovieTime(newMovieTime);
	}

	@Override
	public void setEndTimeCoordinate(int coord) {
		int newMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(coord);
		endTimeInfo.setMovieTime(newMovieTime);
	}

	public String getText() {
		return location.getAbbreviation();
	}

	@Override
	public String getToolTipText() {
		return location.getName();
	}

	@Override
	public boolean select() {
		return false;
	}

	@Override
	public boolean unselect() {
		return false;
	}

	@Override
	public void highlight() {}

	@Override
	public void unhighlight() {}

	@Override
	public boolean isSelected() {
		return false;
	}

	@Override
	public boolean isHighlighted() {
		return false;
	}

	@Override
	public void setColor(Color color) {}

	@Override
	public Color getColor() { return null; }

	@Override
	public String getID() { return id; }

	@Override
	public void setID(String id) {
		this.id = id;
	}

	public void setSS3Item(SS3Item item) {
		setStartTimeInfo(item.getStartTimeInfo().cloneTimeInfo());
		setEndTimeInfo(item.getEndTimeInfo().cloneTimeInfo());
		locationType = SS3Singleton.getLocationConfig().getLocationType(item.getLocationTypeID());
		location = SS3Singleton.getLocationConfig().getRegion(item.getLocationRegionID());
	}

	public LocationType getLocationType() {
		return locationType;
	}

	public Region getLocation() {
		return location;
	}
}
