package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Color;

import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedEvent implements Event {
	protected String id;
	protected String text;
	protected String ttText;

	protected TimeInfo startTimeInfo = new TimeInfo();
	protected TimeInfo endTimeInfo = new TimeInfo();
	
	public boolean isLockedStartTime() {
		return lockedStartTime;
	}

	public void setLockedStartTime(boolean lockedStartTime) {
		this.lockedStartTime = lockedStartTime;
	}

	public boolean isLockedEndTime() {
		return lockedEndTime;
	}

	public void setLockedEndTime(boolean lockedEndTime) {
		this.lockedEndTime = lockedEndTime;
	}

	private boolean lockedStartTime = true;
	private boolean lockedEndTime = true;

	protected int startTime = -1, endTime;

	private Color color = null;

	protected SignStreamSegmentPanel segmentPanel;

	// Set color schema edu.bu.signstream.common.fields.vo.Palette
	protected boolean selected = false;

	private boolean highlighted = false;

	// protected MediaToolBar1 mToolBar = SSSingleton.getMediaToolBar();

	public ChainedEvent(TrackValue value, SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		setTrackValue(value);
	}

	public ChainedEvent(SS3Item item, SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		setSS3Item(item);
	}

	public ChainedEvent(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

    public SignStreamSegmentPanel getSignStreamSegmentPanel() {
        return segmentPanel;
    }

	@Override
	public void setColor(Color color) {
		this.color = color;
	}

	@Override
	public Color getColor() {
		return color;
	}

	public void setID(String id) {
		this.id = id;
	}

	public String getID() {
		return id;
	}

	public boolean isExist() {
		if (startTimeInfo.getMovieTime() == -1)
			return false;
		return true;
	}

	// Event interface
	@Override
	public TimeInfo getStartTimeInfo() {
		return startTimeInfo; // value.getStartTime();
	}

	// Event interface
	@Override
	public TimeInfo getEndTimeInfo() {
		return endTimeInfo; // value.getEndTime();
	}

	// Event interface
	@Override
	public void setStartTimeInfo(TimeInfo startTimeInfo) {
		this.startTimeInfo = startTimeInfo; // value.setStartTime(startTime);
	}

	// Event interface
	@Override
	public void setEndTimeInfo(TimeInfo endTimeInfo) {
		this.endTimeInfo = endTimeInfo; // value.setEndTime(endTime);
	}

	// // Event interface
	@Override
	public int getStartTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(startTimeInfo.getMovieTime());
	}

	// Event interface
	@Override
	public int getEndTimeCoordinate() {
		return segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(endTimeInfo.getMovieTime());
	}

	// Event interface
	@Override
	public void setStartTimeCoordinate(int coord) {

		int endTimeCoord = getEndTimeCoordinate();
		if (coord > endTimeCoord) {
			this.endTimeInfo.setMovieTime(segmentPanel.getZoomer()
					.getCoordinateTimeConvertor()
					.convertCoordinateToTime(coord));
			this.startTimeInfo.setMovieTime(segmentPanel.getZoomer()
					.getCoordinateTimeConvertor().convertCoordinateToTime(
							endTimeCoord));
		} else {
			this.startTimeInfo.setMovieTime(segmentPanel.getZoomer()
					.getCoordinateTimeConvertor()
					.convertCoordinateToTime(coord));
		}
	}

	// Event interface
	@Override
	public void setEndTimeCoordinate(int coord) {
		int startTimeCoord = getStartTimeCoordinate();

		if (startTimeCoord > coord) {
			this.endTimeInfo.setMovieTime(segmentPanel.getZoomer()
					.getCoordinateTimeConvertor().convertCoordinateToTime(
							startTimeCoord));
			this.startTimeInfo.setMovieTime(segmentPanel.getZoomer()
					.getCoordinateTimeConvertor()
					.convertCoordinateToTime(coord));
		} else {
			this.endTimeInfo.setMovieTime(segmentPanel.getZoomer()
					.getCoordinateTimeConvertor()
					.convertCoordinateToTime(coord));
		}
	}

	public boolean contains(int x) {
		//
		// System.out.println("getStartTimeCoordinate = " +
		// getStartTimeCoordinate() + ", getEndTimeCoordinate() = " +
		// getEndTimeCoordinate());
		//
		return (getStartTimeCoordinate() <= x && x <= getEndTimeCoordinate());
	}

	public void reset() {
		unselect();
		selected = false;
	}

	@Override
	public boolean select() {
		highlight();
		selected = true;
		return selected;
	}

	@Override
	public boolean unselect() {
		this.selected = false;
		unhighlight();
		// mToolBar.setEnable(true);
		return true;
	}

	@Override
	public void highlight() {
		this.highlighted = true;
	}

	@Override
	public void unhighlight() {
		this.highlighted = false;
	}

	@Override
	public boolean isHighlighted() {
		return this.highlighted;
	}

	@Override
	public boolean isSelected() {
		return selected;
	}

	@Override
	public String getToolTipText() {
		double duration = getEndTimeInfo().getMovieTime()
				- getStartTimeInfo().getMovieTime();
		String t = ttText != null && ttText.length() > 0 ? ttText : text;
		return t + ", " + duration / 1000 + " sec";
	}

	public String getText() {
		return text == null ? "" : text;
	}

	public void setText(String text) {
		this.text = text;
	}

	public void setToolTipText(String ttText) {
		this.ttText = ttText;
	}

//	private String getValueType() {
//		if (id == Constants.TYPE_HOLD) {
//			return "HOLD";
//		} else if (id == Constants.TYPE_ONSET) {
//			return "ONSET";
//		} else if (id == Constants.TYPE_OFFSET) {
//			return "OFFSET";
//		}
//		return "";
//	}

	public void setTrackValue(TrackValue value) {
		if (value != null) {
			this.id = "" + value.getValueId();
			this.text = value.getTextValue();
			this.startTimeInfo = value.getStartTimeInfo();
			this.endTimeInfo = value.getEndTimeInfo();
		}
	}

	public void setSS3Item(SS3Item item) {
		this.id = item.getId();
		this.text = item.getText();
		this.startTimeInfo = item.getStartTimeInfo();
		this.endTimeInfo = item.getEndTimeInfo();
	}

	public SS3Item getSS3Item() {
		SS3Item item = new SS3Item();
		item.setId(id);
		item.setStartTimeInfo(startTimeInfo);
		item.setEndTimeInfo(endTimeInfo);
		item.setText(text);
		return item;
	}

	public ChainedEvent cloneChainedEvent() {
		return new ChainedEvent(getSS3Item(), segmentPanel);
	}
}
