package edu.bu.signstream.grepresentation.fields.nonManualField;

import java.awt.Font;
import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.HashMap;

import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.TrackValue;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Painter;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class ChainedEventsEntityCollection {
	private ArrayList<ChainedEventsEntity> entities = new ArrayList<ChainedEventsEntity>();
	private SignStreamSegmentPanel segmentPanel;

	private NewChainedEvent newEvent;
	private EnteredText enteredText;

	public ChainedEventsEntityCollection(
			ArrayList<ChainedEventsEntity> entities,
			SignStreamSegmentPanel segmentPanel) {
		this.entities = entities;
		this.segmentPanel = segmentPanel;

        sortEntities();
	}

	public EnteredText getEnteredText() {
		return enteredText;
	}

	public void setEnteredText(EnteredText enteredText) {
		this.enteredText = enteredText;
	}

	public ChainedEventsEntity getNextEntity(int coord) {
		ChainedEventsEntity nextEntity = null;
		for (ChainedEventsEntity entity : entities) {
			if(entity.getStartTimeCoordinate() > coord) {
				if(nextEntity == null) {
					nextEntity = entity;
				} else if(entity.getStartTimeCoordinate() < nextEntity.getStartTimeCoordinate()) {
					nextEntity = entity;
				}
			}
		}
		return nextEntity;
	}
	
	public ChainedEventsEntity getPreviousEntity(int coord) {
		ChainedEventsEntity prevEntity = null;
		for (ChainedEventsEntity entity : entities) {
			if(entity.getStartTimeCoordinate() < coord) {
				if(prevEntity == null) {
					prevEntity = entity;
				} else if(entity.getStartTimeCoordinate() > prevEntity.getStartTimeCoordinate()) {
					prevEntity = entity;
				}
			}
		}
		return prevEntity;
	}

	public ArrayList<ChainedEventsEntity> getEntitiesInInterval(int x1, int x2) {
		ArrayList<ChainedEventsEntity> list = new ArrayList<ChainedEventsEntity>();
		
		segmentPanel.getZoomer().getCoordinateTimeConvertor();

		int X1 = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(x1);
		int X2 = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(x2);
		
		for (ChainedEventsEntity entity : entities) {
			if ((entity.isEntityInsideElement(X1, X2))
					|| (entity.contains(X1) || entity.contains(X2))) {
				list.add(entity);
			}
		}

		return list;
	}

	public void removeEmptyEntities() {
		ArrayList<ChainedEventsEntity> entitiesToDelete = new ArrayList<ChainedEventsEntity>();

		for (ChainedEventsEntity entity : entities) {
			if (entity.getStartTimeInfo() == null || entity.getEndTimeInfo() == null) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}
	}
	
	/**
	 * Cleanup method: if there are hanging references to a deleted field value, which is residing in this 
	 * ChainedEventsEntityCollection with an invalid field value ID, then it has since been deleted from the
	 * coding schema, but not yet form this collection.
	 * @param fieldID The Field ID of the field to get from the coding scheme
	 * @return true if succesful, false otherwise
	 */
	boolean deleteInvalidEntities(String fieldID) {
		ArrayList<ChainedEventsEntity> entitiesToDelete = new ArrayList<ChainedEventsEntity>();
		
		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		SS3Field ss3Field = cs.getField(fieldID);
		ArrayList<SS3FieldValue> ss3FieldValues = ss3Field.getValuesAsList();
		ArrayList<String> ss3FieldValueIDs = new ArrayList<String>();
		
		for (SS3FieldValue ss3FieldValue: ss3FieldValues) {
			String valueID = ss3FieldValue.getId();
			if (valueID == null || valueID.isEmpty())
				continue;
			
			ss3FieldValueIDs.add(valueID);
		}
		
		if (ss3FieldValueIDs.isEmpty()) {
			return false;
		}
		
		for (ChainedEventsEntity entity: entities) {
			String valueID = entity.getValueOrTextValueEvent().getID();
			if ( !ss3FieldValueIDs.contains(valueID)) {
				entitiesToDelete.add(entity);
			}
		}

		for (ChainedEventsEntity entityToDelete : entitiesToDelete) {
			deleteEntity(entityToDelete);
		}		
		
		return true;
	}

	public ChainedEventsEntity getEntityWithRefID(String refEntityId) {
		for (ChainedEventsEntity entity : entities) {
			if (refEntityId != null
					&& refEntityId.equalsIgnoreCase(entity.getRefEntityId()))
				return entity;
		}
		return null;
	}

	public ChainedEventsEntity getEntity(String entityID) {
		for (ChainedEventsEntity entity : entities) {
			if (entity.getID().equalsIgnoreCase(entityID))
				return entity;
		}
		return null;
	}

	public ArrayList<ChainedEventsEntity> getEntities() {
		return entities;
	}

	public int size() {
		return entities.size();
	}

	public void deleteEntity(ChainedEventsEntity entity) {
		ChainedEventsEntity previousEntity = entity.getPreviousEntity();
		ChainedEventsEntity nextEntity = entity.getNextEntity();

		if (previousEntity != null) {
			previousEntity.setNextEntity(nextEntity);
		}

		if (nextEntity != null) {
			nextEntity.setPreviousEntity(previousEntity);
		}

		boolean removed = entities.remove(entity);

        if(!removed) {
            int startMovieTime = entity.getStartTimeInfo().getMovieTime();
            int endMovieTime = entity.getEndTimeInfo().getMovieTime();

            ChainedEventsEntity entityToRemove = null;

            for(ChainedEventsEntity entity1 : entities) {
                int startMovieTime1 = entity1.getStartTimeInfo().getMovieTime();
                int endMovieTime1 = entity1.getEndTimeInfo().getMovieTime();

                if(startMovieTime == startMovieTime1 && endMovieTime == endMovieTime1) {
                    entityToRemove = entity1;
                }
            }

            entities.remove(entityToRemove);
        }
	}

	public int getStartTime() {
		removeEmptyEntities();
		int startTime = 0;
		for (ChainedEventsEntity entity : entities) {
			startTime = startTime < entity.getStartTimeInfo().getMovieTime() ? startTime
					: entity.getStartTimeInfo().getMovieTime();
		}
		return startTime;
	}

	public int getEndTime() {
		removeEmptyEntities();
		int endTime = 0;
		for (ChainedEventsEntity entity : entities) {
			endTime = endTime > entity.getEndTimeInfo().getMovieTime() ? endTime
					: entity.getEndTimeInfo().getMovieTime();
		}
		return endTime;
	}

	public int getStartTimeCoordinate() {
		int time = getStartTime();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeToCoordinate(time);
	}

	public int getEndTimeCoordinate() {
		int time = getEndTime();

		return SS3Singleton.getVideoControlManager().getZoomer()
				.getCoordinateTimeConvertor().convertTimeToCoordinate(time);
	}

	public void unselect() {
		for (ChainedEventsEntity entity : entities) {
			entity.unselect();
		}
	}

	public void reset() {
		for (ChainedEventsEntity entity : entities) {
			entity.reset();
		}
	}

	public void addEntity(ChainedEventsEntity entity) {
		
		int startMovieTime = entity.getStartTimeInfo().getMovieTime();
		
		ChainedEventsEntity previousEntity = null;
		ChainedEventsEntity nextEntity = null;

		for (ChainedEventsEntity ent : entities) {
			int entMovieStartTime = ent.getStartTimeInfo().getMovieTime();
			
			// exit function in case that this event is already in the list
			if (entMovieStartTime == startMovieTime)
				return;

			if (entMovieStartTime < startMovieTime) {
				if (previousEntity == null) {
					previousEntity = ent;
				} else if (entMovieStartTime > previousEntity
						.getStartTimeInfo().getMovieTime()) {
					previousEntity = ent;
				}
			} else {
				if (nextEntity == null) {
					nextEntity = ent;
				} else if (entMovieStartTime < nextEntity.getStartTimeInfo()
						.getMovieTime()) {
					nextEntity = ent;
				}
			}
		}
		
		if (previousEntity != null) {
			ChainedEventsEntity next = previousEntity.getNextEntity();
			if (next != null) {
				entity.setNextEntity(next);
				next.setPreviousEntity(entity);
			}
			previousEntity.setNextEntity(entity);
			entity.setPreviousEntity(previousEntity);

		} else if (nextEntity != null) {
			entity.setNextEntity(nextEntity);
			nextEntity.setPreviousEntity(entity);
		}

		this.entities.add(entity);
	}

	public void selectEntity(int x) {
		EventsEntity entity = getChainedEventsEntityAt(x);
		if (entity != null)
			entity.select();
	}

	public ChainedEventsEntity getChainedEventsEntityAt(int coordX) {
		for (ChainedEventsEntity entity : entities) {
			if (entity.contains(coordX))
				return entity;
		}
		return null;
	}

	public Event getSelectedEvent() {
		for (ChainedEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) { 
				return event;
			}
		}
		return null;
	}

	public ChainedEventsEntity getSelectedEntity() {
		for (ChainedEventsEntity entity : entities) {
			Event event = entity.getSelectedEvent();
			if(event != null) {
				return entity;
			}
		}
		return null;
	}

	public ChainedEvent getEventAt(int x) {
		ChainedEventsEntity entity = getChainedEventsEntityAt(x);
		if (entity == null)
			return null;

		return entity.getChainedEventsCollection().getEventAt(x);
	}

	public void setSelectedEvementValueID(String valueID) {
		Event event = getSelectedEvent();

		if (event instanceof ChainedEvent) {
			ChainedEvent chainedEvent = (ChainedEvent) event;
			chainedEvent.setID(valueID);
		}
	}

	public NewChainedEvent getNewChainedEvent() {
		return newEvent;
	}

	public void setNewChainedEvent(NewChainedEvent newEvent) {
		this.newEvent = newEvent;
	}

	public ChainedEventsCollection createNewChainedEventCollection(
			String valueID) {
		if (valueID != null && newEvent != null) {
			TrackValue trackValue = new TrackValue();
			trackValue.setValueId(valueID);

			trackValue.setStartTimeInfo(newEvent.getStartTimeInfo());
			trackValue.setEndTimeInfo(newEvent.getEndTimeInfo());

			ChainedEvent event = new ChainedEvent(trackValue, segmentPanel);

			ChainedEventsCollection collection = new ChainedEventsCollection();
			collection.setValueEvent(event);
			newEvent = null;

			return collection;
		}
		return null;
	}

    public void sortEntities() {
        int entitiesSize = entities.size();

        if(entitiesSize == 1)
            return;

        int[] startTimes = new int[entitiesSize];
        HashMap<String, ChainedEventsEntity> map = new HashMap<String, ChainedEventsEntity>();

        int i = 0;
        for(ChainedEventsEntity entity : entities) {
            int movieTime = entity.getStartTimeInfo().getMovieTime();
            map.put(""+movieTime, entity);
            startTimes[i] = movieTime;
            i++;
        }

        Sorter.sort(startTimes);

        i = 0;
        while(i < entitiesSize-1) {
            ChainedEventsEntity glsEntity = map.get(""+startTimes[i]);
            i++;
            ChainedEventsEntity nextGlsEntity = map.get(""+startTimes[i]);
            glsEntity.setNextEntity(nextGlsEntity);
            nextGlsEntity.setPreviousEntity(glsEntity);
        }
    }

	public void sort(ArrayList<ChainedEventsEntity> entitiesToSort) {
		removeEmptyEntities();

		HashMap<String, ChainedEventsEntity> map = new HashMap<String, ChainedEventsEntity>();

		int[] startTimes = new int[entitiesToSort.size()];

		ArrayList<ChainedEventsEntity> toRemove = new ArrayList<ChainedEventsEntity>();

		for (int i = 0; i < entitiesToSort.size(); i++) {
			ChainedEventsEntity entity = entitiesToSort.get(i);
			if (entity != null && entity.getStartTimeInfo() != null) {
				int startTime = entity.getStartTimeInfo().getMovieTime();
				map.put("" + startTime, entity);
				startTimes[i] = startTime;
			} else {
				toRemove.add(entity);
			}
		}

		for (ChainedEventsEntity cEnt : toRemove) {
			entitiesToSort.remove(cEnt);
		}

		Sorter.sort(startTimes);
		entitiesToSort.clear();

		for (int j = 0; j < startTimes.length; j++) {
			ChainedEventsEntity obj = map.get("" + startTimes[j]);
			entitiesToSort.add(obj);
		}
	}

	public void paint(Graphics2D g2d, double Y, Font font) {
		for (ChainedEventsEntity entity : entities) {
			entity.paint(g2d, Y, font);
		}

		if (newEvent != null) {
			Painter painter = segmentPanel.getPainter();
			painter.drawNewEvent(newEvent, g2d, Y);
		}
	}
}