package edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;

public class AddFinalHoldActionListener implements ActionListener {

   private ChainedEventsEntity selectedEntity = null;
   
   public AddFinalHoldActionListener() {}

	public void setSelectedEntity(ChainedEventsEntity entity) {
		this.selectedEntity = entity;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		Event utterance = selectedEntity.getParentEntity().getEvent();

		SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();

		int maxOffsetStartTime = segmentPanel.getMultipleMovieController()
				.getFollowingFrameTime(
						selectedEntity.getEndTimeInfo().getMovieTime());
		
		int frameDuration =  (int)segmentPanel.getMultipleMovieController().getPacketDuration();

		if (selectedEntity.getOffset() == null && maxOffsetStartTime > utterance.getEndTimeInfo().getMovieTime()) {	
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
			return;

		}
		if(selectedEntity.getOffset() == null) {
			ChainedEventsEntity nextEntity = selectedEntity.getNextEntity();
			if(nextEntity != null) {
				int nextEntiyStartTime = nextEntity.getStartTimeInfo().getMovieTime();
				int maxOffsetTime = nextEntiyStartTime - frameDuration;
				if(selectedEntity.getEndTimeInfo().getMovieTime() > maxOffsetTime) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return;
				}
			}
		}
		
		String warning = selectedEntity.addFinalHoldEvent();
		if (warning != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(warning);
			return;
		}
		Event selectedEvent = selectedEntity.getFinalHold();
		selectedEntity.getValueOrTextValueEvent().unselect();
		selectedEvent.select();
		MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(segmentPanel);
		mToolBar.setSelectedEvent(selectedEvent);
		
		int currentTime = selectedEvent.getEndTimeInfo().getMovieTime();
		segmentPanel.getSlider().setSliderPositionTime(currentTime);
		segmentPanel.getMultipleMovieController().setCurrentTime(currentTime);
		
		if (selectedEntity instanceof GlossChainedEventsEntity) {
			GlossChainedEventsEntity glsSelectedEntity = (GlossChainedEventsEntity) selectedEntity;
			boolean isLockedGlossAlignments = glsSelectedEntity.isLockedGlossPhonAlignments();
			boolean isLockedLeftRightAlignments = glsSelectedEntity.isLockedLeftRightAlignments();
		
			int time = selectedEvent.getEndTimeInfo().getMovieTime();
		
			if(isLockedGlossAlignments) {
				ChainedEventsEntity handEntity  = glsSelectedEntity.getDependentHandEntity();
				HandShapeEntity handShapeEntity = glsSelectedEntity.getDependentHandShapeEntity();
			
				if(handEntity != null) {
					ChainedEventsEntity nextHndEntity = handEntity.getNextEntity();

					if (nextHndEntity == null) {
						handEntity.getTextValue().getEndTimeInfo().setMovieTime(time);
					} else if (nextHndEntity.getEndTimeInfo().getMovieTime() < time) {
						SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					} else {
						handEntity.getTextValue().getEndTimeInfo().setMovieTime(time);
					}
				
					if(handShapeEntity != null) {
						if (handShapeEntity.getEvents().size() != 1) {
							handShapeEntity.getEndTimeInfo().setMovieTime(time);
							handShapeEntity.getLastHandShapeEvent().getStartTimeInfo().setMovieTime(time);
						} else {
							handShapeEntity.getFirstHandShapeEvent().getStartTimeInfo().setMovieTime(selectedEntity.getStartTimeInfo().getMovieTime());
							handShapeEntity.getFirstHandShapeEvent().getEndTimeInfo().setMovieTime(time);
						}
					}
				}
			}
		
			if(isLockedLeftRightAlignments) {
				GlossChainedEventsEntity depGlsEntity = glsSelectedEntity.getDependentGlossEntity();
				if(depGlsEntity != null && depGlsEntity.getFinalHold() == null) {
					ChainedEventsEntity nextDepGlsEntity = depGlsEntity.getNextEntity();

					if (depGlsEntity.getOffset() == null && nextDepGlsEntity != null) {
						int minHoldMovieTime = nextDepGlsEntity.getStartTimeInfo().getMovieTime() - frameDuration;
						int selectedEntityStartMovieTime = depGlsEntity.getEndTimeInfo().getMovieTime();

						if (selectedEntityStartMovieTime > minHoldMovieTime) {
							SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return;
						}
					}
				
					String depWarning = depGlsEntity.addFinalHoldEvent();
					if (depWarning != null) {
						SS3Singleton.getErrorMessages().showErrorMessage(depWarning);
						return;
					}
					if(depGlsEntity.isLockedGlossPhonAlignments()) {
						ChainedEventsEntity depHandEntity = depGlsEntity.getDependentHandEntity();
						HandShapeEntity depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();
					
						if(depHandEntity != null) {
							ChainedEventsEntity nextHndEntity = depHandEntity.getNextEntity();

							if (nextHndEntity == null) {
								depHandEntity.getTextValue().getEndTimeInfo().setMovieTime(time);
							} else if (nextHndEntity.getStartTimeInfo().getMovieTime() < time) {
								SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return;
							} else {
								depHandEntity.getTextValue().getEndTimeInfo().setMovieTime(time);
							}
						
							if(depHandShapeEntity != null) {
								if (depHandShapeEntity.getEvents().size() != 1) {
									depHandShapeEntity.getEndTimeInfo().setMovieTime(time);
									depHandShapeEntity.getLastHandShapeEvent().getStartTimeInfo().setMovieTime(time);
								} else {
									depHandShapeEntity.getFirstHandShapeEvent().getStartTimeInfo().setMovieTime(selectedEntity.getStartTimeInfo().getMovieTime());
									depHandShapeEntity.getFirstHandShapeEvent().getEndTimeInfo().setMovieTime(time);
								}
							}
						}
					}
				}
			}

			LocationEntity depLocEntity = glsSelectedEntity.getDependentLocationEntity();
			if (depLocEntity != null) {
				depLocEntity.updateTimes();
			}
			
			SignTypeEntity depSignTypeEnt = glsSelectedEntity.getDependentSignTypeEntity();
			if (depSignTypeEnt != null) {
				depSignTypeEnt.updateTimes();
			}
		}
		segmentPanel.repaint();
	}
}