package edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;

public class AddInitialHoldActionListener implements ActionListener {
	private ChainedEventsEntity selectedEntity = null;

	public AddInitialHoldActionListener() {
	}

	public void setSelectedEntity(ChainedEventsEntity entity) {
		this.selectedEntity = entity;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		Event utterance = selectedEntity.getParentEntity().getEvent();

		SignStreamSegmentPanel segmentPanel = SS3Singleton
				.getSignStreamApplication().getSignStreamSegmentPanel();

		int frameDuration = (int) segmentPanel.getMultipleMovieController()
				.getPacketDuration();

		int minEntityStartTime = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTime(
						selectedEntity.getStartTimeInfo().getMovieTime());

		if (selectedEntity.getOnset() == null
				&& minEntityStartTime < utterance.getStartTimeInfo()
						.getMovieTime()) {
			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
			return;
		}

		ChainedEventsEntity prevEntity = selectedEntity.getPreviousEntity();
		
		if (selectedEntity.getOnset() == null && prevEntity != null) {
			int minHoldMovieTime = prevEntity.getEndTimeInfo().getMovieTime()
					+ frameDuration;
			int selectedEntityStartMovieTime = selectedEntity
					.getStartTimeInfo().getMovieTime();

			if (selectedEntityStartMovieTime < minHoldMovieTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(
						ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		String warning = selectedEntity.addInitialHoldEvent();
		if (warning != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(warning);
			return;
		} 
		Event selectedEvent = selectedEntity.getInitialHold();

		selectedEntity.getValueOrTextValueEvent().unselect();
		selectedEvent.select();
		MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(segmentPanel);
		mToolBar.setSelectedEvent(selectedEvent);
		
		int currentTime = selectedEvent.getStartTimeInfo().getMovieTime();
		segmentPanel.getSlider().setSliderPositionTime(currentTime);
		segmentPanel.getMultipleMovieController().setCurrentTime(currentTime);

		if (selectedEntity instanceof GlossChainedEventsEntity) {
			GlossChainedEventsEntity glsSelectedEntity = (GlossChainedEventsEntity) selectedEntity;
			boolean isLockedGlossAlignments = glsSelectedEntity.isLockedGlossPhonAlignments();
			boolean isLockedLeftRightAlignments = glsSelectedEntity.isLockedLeftRightAlignments();
			
			int time = selectedEvent.getStartTimeInfo().getMovieTime();
			
			if(isLockedGlossAlignments) {
				ChainedEventsEntity handEntity  = glsSelectedEntity.getDependentHandEntity();
				HandShapeEntity handShapeEntity = glsSelectedEntity.getDependentHandShapeEntity();
				
				if(handEntity != null) {
					ChainedEventsEntity prvHndEntity = handEntity.getPreviousEntity();

					if (prvHndEntity == null) {
						Event txt = handEntity.getTextValue();
						Event value = handEntity.getValue();
						Event txtOrValue = handEntity.getValueOrTextValueEvent();
						
						handEntity.getValueOrTextValueEvent().getStartTimeInfo().setMovieTime(time);
					} else if (prvHndEntity.getEndTimeInfo().getMovieTime() > time) {
						SS3Singleton.getErrorMessages().showErrorMessage(
								ErrorMessages.CHAINS_CANNOT_OVERLAP);
						return;
					} else {
						handEntity.getTextValue().getStartTimeInfo().setMovieTime(time);
					}
					
					if(handShapeEntity != null) {
						handShapeEntity.getStartTimeInfo().setMovieTime(time);
					}
				}
			}
			
			if(isLockedLeftRightAlignments) {
				GlossChainedEventsEntity depGlsEntity = glsSelectedEntity.getDependentGlossEntity();
				if(depGlsEntity != null && depGlsEntity.getInitialHold() == null) {
					ChainedEventsEntity prevDepGlsEntity = depGlsEntity.getPreviousEntity();

					if (depGlsEntity.getOnset() == null && prevDepGlsEntity != null) {
						int minHoldMovieTime = prevDepGlsEntity.getEndTimeInfo().getMovieTime() + frameDuration;
						int selectedEntityStartMovieTime = depGlsEntity.getStartTimeInfo().getMovieTime();

						if (selectedEntityStartMovieTime < minHoldMovieTime) {
							SS3Singleton.getErrorMessages().showErrorMessage(
									ErrorMessages.CHAINS_CANNOT_OVERLAP);
							return;
						}
					}
					
					String depWarning = depGlsEntity.addInitialHoldEvent();
					if (depWarning != null) {
						SS3Singleton.getErrorMessages().showErrorMessage(depWarning);
						return;
					}
					if(depGlsEntity.isLockedGlossPhonAlignments()) {
						ChainedEventsEntity depHandEntity = depGlsEntity.getDependentHandEntity();
						HandShapeEntity depHandShapeEntity = depGlsEntity.getDependentHandShapeEntity();
						
						if(depHandEntity != null) {
							ChainedEventsEntity prvHndEntity = depHandEntity.getPreviousEntity();

							if (prvHndEntity == null) {
								depHandEntity.getTextValue().getStartTimeInfo().setMovieTime(time);
							} else if (prvHndEntity.getEndTimeInfo().getMovieTime() > time) {
								SS3Singleton.getErrorMessages().showErrorMessage(
										ErrorMessages.CHAINS_CANNOT_OVERLAP);
								return;
							} else {
								depHandEntity.getTextValue().getStartTimeInfo().setMovieTime(time);
							}
							
							if(depHandShapeEntity != null) {
								depHandShapeEntity.getStartTimeInfo().setMovieTime(time);
							}
						}
					}
				}
			}

			LocationEntity depLocEntity = glsSelectedEntity.getDependentLocationEntity();
			if (depLocEntity != null) {
				depLocEntity.updateTimes();
			}
			
			SignTypeEntity depSignTypeEnt = glsSelectedEntity.getDependentSignTypeEntity();
			if (depSignTypeEnt != null) {
				depSignTypeEnt.updateTimes();
			}
		}
		segmentPanel.repaint();
	}
}
