package edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;

public class AddOffsetActionListener implements ActionListener {

   private ChainedEventsEntity selectedEntity = null;

   public AddOffsetActionListener(){}

   public void setSelectedEntity(ChainedEventsEntity entity) {
	   this.selectedEntity = entity;
   }
   
   public void actionPerformed(ActionEvent e) {
		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		Event utterance = selectedEntity.getParentEntity().getEvent();

		SignStreamSegmentPanel segmentPanel = SS3Singleton
				.getSignStreamApplication().getSignStreamSegmentPanel();

		int minOffsetStartTime = segmentPanel.getMultipleMovieController()
				.getFollowingFrameTime(
						selectedEntity.getEndTimeInfo().getMovieTime());

		if (minOffsetStartTime > utterance.getEndTimeInfo().getMovieTime()) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
			return;
		} 
			
		ChainedEventsEntity nextEntity = selectedEntity.getNextEntity();
		
		if(nextEntity != null) {
			int nextEntiyStartTime = nextEntity.getStartTimeInfo().getMovieTime();
			int frameDuration =  (int)segmentPanel.getMultipleMovieController().getPacketDuration();
			int maxOffsetTime = nextEntiyStartTime - frameDuration;
			if(selectedEntity.getEndTimeInfo().getMovieTime() > maxOffsetTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}
		
		String seWarning = selectedEntity.addOffsetEvent();
		if (seWarning != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
			return;
		}
		Event offset = selectedEntity.getOffset(); 
		selectedEntity.getValueOrTextValueEvent().unselect();
		offset.select();
		MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(segmentPanel);
		mToolBar.setSelectedEvent(offset);
		
		int currentTime = offset.getEndTimeInfo().getMovieTime();
		segmentPanel.getSlider().setSliderPositionTime(currentTime);
		segmentPanel.getMultipleMovieController().setCurrentTime(currentTime);


		if(selectedEntity instanceof GlossChainedEventsEntity) {
			GlossChainedEventsEntity glsSelectedEntity = (GlossChainedEventsEntity)selectedEntity;
			
			if(glsSelectedEntity.isLockedLeftRightAlignments()) {
				GlossChainedEventsEntity ndGlsEntity = glsSelectedEntity.getDependentGlossEntity();
				if(ndGlsEntity != null && ndGlsEntity.getOffset() == null) {
					String ndGlsWarning = ndGlsEntity.addOffsetEvent();
					if(ndGlsWarning != null) {
						SS3Singleton.getErrorMessages().showErrorMessage(ndGlsWarning);	
					}
				}
			}

			LocationEntity depLocEntity = glsSelectedEntity.getDependentLocationEntity();
			if (depLocEntity != null) {
				depLocEntity.updateTimes();
			}
			
			SignTypeEntity depSignTypeEnt = glsSelectedEntity.getDependentSignTypeEntity();
			if (depSignTypeEnt != null) {
				depSignTypeEnt.updateTimes();
			}
		}
		segmentPanel.repaint();
   }
}
