package edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeEntity;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;

public class AddOnsetActionListener implements ActionListener {
	private ChainedEventsEntity selectedEntity = null;

	public AddOnsetActionListener() {
	}

	public void setSelectedEntity(ChainedEventsEntity entity) {
		this.selectedEntity = entity;
	}

	public void actionPerformed(ActionEvent e) {

		MediaToolBar mediaToolBar = SS3Singleton.getMediaToolBar(null);
		Event utterance = selectedEntity.getParentEntity().getEvent();

		SignStreamSegmentPanel segmentPanel = SS3Singleton
				.getSignStreamApplication().getSignStreamSegmentPanel();

		int minOnsetStartTime = segmentPanel.getMultipleMovieController()
				.getPrecedingFrameTime(
						selectedEntity.getStartTimeInfo().getMovieTime());

		if (minOnsetStartTime < utterance.getStartTimeInfo().getMovieTime()) {
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
			return;
		}
			
		ChainedEventsEntity prevEntity = selectedEntity.getPreviousEntity();
		if(prevEntity != null) {
			int frameDuration =  (int)segmentPanel.getMultipleMovieController().getPacketDuration();
			int minOnsetMovieTime = prevEntity.getEndTimeInfo().getMovieTime() + frameDuration;
			if(selectedEntity.getStartTimeInfo().getMovieTime() < minOnsetMovieTime) {
				SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CHAINS_CANNOT_OVERLAP);
				return;
			}
		}

		String seWarning = selectedEntity.addOnsetEvent();
		if (seWarning != null) {
			SS3Singleton.getErrorMessages().showErrorMessage(seWarning);
			return;
		}
		Event selectedEvent = selectedEntity.getOnset();
		selectedEntity.getValueOrTextValueEvent().unselect();
		selectedEvent.select();
		MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(segmentPanel);
		mToolBar.setSelectedEvent(selectedEvent);
		
		int currentTime = selectedEvent.getStartTimeInfo().getMovieTime();
		segmentPanel.getSlider().setSliderPositionTime(currentTime);
		segmentPanel.getMultipleMovieController().setCurrentTime(currentTime);
	
		
		if(selectedEntity instanceof GlossChainedEventsEntity) {
			GlossChainedEventsEntity glsSelectedEntity = (GlossChainedEventsEntity)selectedEntity;
			
			if(glsSelectedEntity.isLockedLeftRightAlignments()) {
				GlossChainedEventsEntity ndGlsEntity = glsSelectedEntity.getDependentGlossEntity();
				if(ndGlsEntity != null && ndGlsEntity.getOnset() == null) {
					String ndGlswarning = ndGlsEntity.addOnsetEvent();
					if(ndGlswarning != null) {
						SS3Singleton.getErrorMessages().showErrorMessage(ndGlswarning);	
					}
				}
			}

			LocationEntity depLocEntity = glsSelectedEntity.getDependentLocationEntity();
			if (depLocEntity != null) {
				depLocEntity.updateTimes();
			}
			
			SignTypeEntity depSignTypeEnt = glsSelectedEntity.getDependentSignTypeEntity();
			if (depSignTypeEnt != null) {
				depSignTypeEnt.updateTimes();
			}
		}
		segmentPanel.repaint();
	}
}