package edu.bu.signstream.grepresentation.fields.presentation;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.NewChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.mouseDelegateListeners.DeletePresentationEntityActionListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class PresentationFieldWrapper extends FieldWrapper {
	private int counter = 1;
	protected PresentationField prField = null;

	private ArrayList<SegmentFieldWrapper> segmentsFieldWrapper = new ArrayList<SegmentFieldWrapper>();

	public PresentationFieldWrapper(PresentationField field, SignStreamSegmentPanel segmentPanel) {
		super(field, segmentPanel);
		this.prField = field;
	}

	public ArrayList<SegmentFieldWrapper> getSegmentsFieldWrappers() {
		return segmentsFieldWrapper;
	}
	
	public void removeAllEntities() {
		prField.removeAllEntities();
	}


	public void addSegmentFieldWrapper(SegmentFieldWrapper wrapper) {
		segmentsFieldWrapper.add(wrapper);
	}
	
	public SignStreamSegmentPanel getSegmentPanel(){
		return segmentPanel;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		prField.addEventsEntity(entity);
	}

	// MouseListener
	@Override
	public void mouseClicked(Point point, JPanel panel) {
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
		int movieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertCoordinateToTime(point.x);

		segmentPanel.getMultipleMovieController().setCurrentTime(movieTime);

		Event sEvent = field.getEventAt(point.x);

		if (!isEntitySelected() && sEvent == null && movieTime < segmentPanel.getMultipleMovieController().getMovieDuration()) {
			NewEvent event = new NewEvent(point.x, segmentPanel);
			if (prField.getNextEntity(event.getStartTime()) != null) {
				// If this entity has an event in front of it, set a reference to it
				event.setNextEntity(prField.getNextEntity(event.getStartTime()));
			} else {
				// If this entity has no event in front of it, create a temporary "boundary" event at
				// the end of the movie duration to prevent this new event from going out of bounds.
				PresentationEvent endBound = new PresentationEvent(segmentPanel);
				endBound.getStartTimeInfo()
					.setMovieTime(segmentPanel.getMultipleMovieController().getMovieDuration());
				event.setNextEntity(new PresentationEventsEntity(endBound, segmentPanel, field));
			}
			
			if (prField.getPreviousEntity(event.getStartTime()) != null) {
				// If this entity has an event behind it, set a reference to it
				event.setPreviousEntity(prField.getPreviousEntity(event.getStartTime()));
			} else {
				// If this entity has no event behind it, create a temporary "boundary" event at the
				// beginning of the movie to prevent this new event from going out of bounds.
				PresentationEvent startBound = new PresentationEvent(segmentPanel);
				startBound.getEndTimeInfo().setMovieTime(0);
				event.setPreviousEntity(new PresentationEventsEntity(startBound, segmentPanel, field));
			}
			prField.setNewEvent(event);
		}
	}

	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}
	
	public void mouseCtrlAltClicked(Point point) {

		final SegmentGraphicRepresentation repPanel = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation();

		EventsEntity entity = prField.getSelectedEntity();

		final JPopupMenu popup = new JPopupMenu();
		JMenuItem delete = new JMenuItem("Delete");
		UserInterfaceUtil.setMenuItemLNF(delete);
		delete.addActionListener(new DeletePresentationEntityActionListener(
				entity, prField, this, repPanel));
		popup.add(delete);

		JMenuItem cancel = new JMenuItem("Cancel");
		UserInterfaceUtil.setMenuItemLNF(cancel);
		popup.add(cancel);

		popup.show(repPanel, point.x, point.y);
		popup.setPreferredSize(popup.getPreferredSize());
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}

	/*
	 * Show selected utterance/segment, etc
	 */
	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {

		Event event = field.selectEvent(point.x);

		SS3SignStreamApplication.logger.log(Level.INFO, "Selected event " + event);
		
		if (event == null)
			return;
		
		PresentationField pField = (PresentationField) field;
		
		String defaultSegmentTierLabel = null;
		if (pField.getTemporalPartition() != null) {
			defaultSegmentTierLabel = pField.getTemporalPartition().getDefaultSegment().getLabel();
		}

		if (defaultSegmentTierLabel != null) {
			for (SegmentFieldWrapper segmentFieldWrapper: segmentsFieldWrapper) {
				String segmentTierLabel = segmentFieldWrapper.getField().getName();
				if (defaultSegmentTierLabel.equals(segmentTierLabel)) {
					// re-route to load just the default segment tier of the clicked temporal partition
					segmentFieldWrapper.mouseDblLeftClicked(point, panel);
				}
			}
		}
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {
	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {
		if (panel == null)
			return;
		panel.setCursor(new Cursor(getCursorType(point)));

		Event event = prField.getEventAt(point.x);
		if (event != null) {
			panel.setToolTipText(event.getToolTipText());
		} else {
			panel.setToolTipText("");
		}
	}

	/**
	 * Deals with event logic when a user ctrl+clicks or alt+clicks and drags the mouse; or when 
	 * they regular click and drag the mouse.
	 * @param point
	 * @param panel
	 */
	@Override
	public void mouseDragged(Point point, JPanel panel) {
		int movieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(point.x);

		// Returns null if user drags normally, and not null if user is creating a new event
		NewChainedEvent newEvent = prField.getNewEvent();
		if (newEvent != null) {
			// Returns a temporary entity representing the end of the movie if there is no event
			// after this one.
			PresentationEventsEntity nextPresEntity = 
					(PresentationEventsEntity) newEvent.getNextEntity();
			// Returns a temporary entity representing the start of the movie if there is no event
			// before this one.
			PresentationEventsEntity prevPresEntity = 
					(PresentationEventsEntity) newEvent.getPreviousEntity();
			
			// If there is an event after this one, limit this event's end time to that event's 
			// start time. Otherwise, limit it to the end of the movie.
			int maxNewEventTime = nextPresEntity.getStartTime();
			// If there is an event before this one, limit this event's start time to that event's
			// end time. Otherwise, limit it to zero to prevent a negative event start time
			int minNewEventTime = prevPresEntity.getEndTime();
					
			if(movieTime > maxNewEventTime) {
				newEvent.getEndTimeInfo().setMovieTime(maxNewEventTime);
				segmentPanel.getMultipleMovieController().goToPacketTimeStamp(maxNewEventTime);
			} else if(movieTime < minNewEventTime) {
				newEvent.getEndTimeInfo().setMovieTime(minNewEventTime);
				segmentPanel.getMultipleMovieController().goToPacketTimeStamp(minNewEventTime);
			} else {
				newEvent.getEndTimeInfo().setMovieTime(movieTime);
				segmentPanel.getMultipleMovieController().goToPacketTimeStamp(movieTime);
			}
			
			Slider slider = segmentPanel.getSlider();
			slider.setSliderPositionCoordNoPanel(point.x);
			SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
			segmentPanel.getDatabaseView().keepUtteranceVisible(point.x);
			
		} else {
			if (panel.getCursor().getType() != Cursor.E_RESIZE_CURSOR)
				return;

			Event sEvent = prField.getSelectedEvent();

			if (MOVE_DIRECTION == MOVE_START_POINT) {
				
		    	  SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				int seStartCoord = sEvent.getStartTimeCoordinate();

				if (seStartCoord < point.x
						&& point.x >= sEvent.getEndTimeCoordinate()) {

					long maxPossibleStartTime = segmentPanel
							.getMultipleMovieController().frameBackward(
									sEvent.getEndTimeInfo().getMovieTime());
					prField.getSelectedEvent().getStartTimeInfo().setMovieTime(
							(int) maxPossibleStartTime);

					String[] msg = new String[4];
					msg[0] = "Warning: The event's start time ";
					msg[1] = "cannot exceed  event's end time. ";
					msg[2] = "You must manually adjust that ";
					msg[3] = "end time and then try again. ";
					cantAutomaticallyAdjustObjectsTime(msg);
					return;
				} else if (seStartCoord > point.x) {
					ArrayList<?> entities = prField.getEntities();
					for (int i = 0; i < entities.size(); i++) {
						PresentationEventsEntity ent = (PresentationEventsEntity) entities
								.get(i);
						if (!ent.isSelected() && !ent.getEvent().isSelected()) {
							if (point.x <= ent.getEndTimeCoordinate()
									&& ent.getEndTimeCoordinate() <= seStartCoord) {
								String[] msg = new String[4];
								msg[0] = "Warning: The field's preceding object's end time";
								msg[1] = "setting cannot be automatically adjusted to accommodate";
								msg[2] = "the requested start time. You must manually adjust that";
								msg[3] = "end time and then try again.";
								cantAutomaticallyAdjustObjectsTime(msg);
								return;
							}
						}
					}
				}

				sEvent.setStartTimeCoordinate(point.x);
				segmentPanel.getMultipleMovieController().goToPacketTimeStamp(
						movieTime);
			}

			if (MOVE_DIRECTION == MOVE_END_POINT) {
				
		    	SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				
				int seEndCoord = sEvent.getEndTimeCoordinate();

				if (seEndCoord > point.x
						&& point.x <= sEvent.getStartTimeCoordinate()) {
					long minPossibleEndTime = segmentPanel
							.getMultipleMovieController().frameForward(
									sEvent.getStartTimeInfo().getMovieTime());
					prField.getSelectedEvent().getEndTimeInfo().setMovieTime(
							(int) minPossibleEndTime);

					String[] msg = new String[4];
					msg[0] = "Warning: The event's start time ";
					msg[1] = "cannot exceed  event's end time. ";
					msg[2] = "You must manually adjust that ";
					msg[3] = "end time and then try again. ";
					cantAutomaticallyAdjustObjectsTime(msg);
					return;
				} else if (seEndCoord < point.x) {
					ArrayList<?> entities = prField.getEntities();
					for (int i = 0; i < entities.size(); i++) {
						PresentationEventsEntity ent = (PresentationEventsEntity) entities.get(i);
						if (!ent.isSelected() && !ent.getEvent().isSelected()) {
							if (point.x >= ent.getStartTimeCoordinate()
									&& ent.getStartTimeCoordinate() >= seEndCoord) {

								prField.getSelectedEvent()
										.setEndTimeCoordinate(
												ent.getStartTimeCoordinate());
								segmentPanel.getMultipleMovieController()
										.goToPacketTimeStamp(
												ent.getStartTimeInfo()
														.getMovieTime());

								String[] msg = new String[4];
								msg[0] = "Warning: The field's following object's start time";
								msg[1] = "setting cannot be automatically adjusted to accommodate";
								msg[2] = "the requested end time. You must manually adjust that";
								msg[3] = "end time and then try again.";
								cantAutomaticallyAdjustObjectsTime(msg);
								return;
							}
						}
					}
				}

				prField.getSelectedEvent().setEndTimeCoordinate(point.x);
				segmentPanel.getMultipleMovieController().goToPacketTimeStamp(
						movieTime);
			}

            int updatedStartMovieTime = sEvent.getStartTimeInfo().getMovieTime();
            int updatedEndMovieTime = sEvent.getEndTimeInfo().getMovieTime();

            PresentationEventsEntity selectedEntity = (PresentationEventsEntity) field.getSelectedEntity();
            ArrayList<PresentationEventsEntity> dependentEntities = selectedEntity.getDependentEntities();
            for(PresentationEventsEntity segmentEntity : dependentEntities) {
                segmentEntity.getStartTimeInfo().setMovieTime(updatedStartMovieTime);
                segmentEntity.getEndTimeInfo().setMovieTime(updatedEndMovieTime);
            }
            
            Slider slider = segmentPanel.getSlider();
    		slider.setSliderPositionCoord(point.x);
		}
	}

	@Override
	public void mouseReleased(Point point, JPanel panel) {
		
		NewEvent newEvent = prField.getNewEvent();
		if (newEvent != null) {
			
	  	    SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

			ArrayList<?> entities = prField.getEntities();
			
			int newEventStartCoord = newEvent.getStartTimeCoordinate();
			int newEventEndCoord = newEvent.getEndTimeCoordinate();
			
			// If event was created by dragging backwards, switch the start and end coordinates
			if(newEventEndCoord < newEventStartCoord) {
				newEvent.setStartTimeCoordinate(newEventEndCoord);
				newEvent.setEndTimeCoordinate(newEventStartCoord);
				newEventStartCoord = newEvent.getStartTimeCoordinate();
				newEventEndCoord = newEvent.getEndTimeCoordinate();
			}

			for (int i = 0; i < entities.size(); i++) {
				PresentationEventsEntity ent = (PresentationEventsEntity) entities
						.get(i);
				ent.unhighlight();
				ent.unselect();
				ent.getEvent().unhighlight();
				ent.getEvent().unselect();
			}

			SS3GlossWindowItem item = new SS3GlossWindowItem("", "");
			item.setId(Util.getUniqueNumber());

			PresentationEvent event = new PresentationEvent(item, segmentPanel);
			
			int movieStartTime = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertCoordinateToTime(newEvent.getStartTimeCoordinate());
			
			int movieEndTime = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertCoordinateToTime(newEvent.getEndTimeCoordinate());
			
			TimeInfo startMovieTimeInfo = new TimeInfo();
			startMovieTimeInfo.setMovieTime(movieStartTime);
			
			TimeInfo endMovieTimeInfo = new TimeInfo();
			endMovieTimeInfo.setMovieTime(movieEndTime);
			
			event.setStartTimeInfo(startMovieTimeInfo);
			event.setEndTimeInfo(endMovieTimeInfo);
			
			event.setStartTimeCoordinate(newEvent.getStartTimeCoordinate());
			event.setEndTimeCoordinate(newEvent.getEndTimeCoordinate());
			item.setEvent(event);

			event.select();
			
			// Deals with utterances overlapping or being too small
			PresentationEventsEntity nextPresEntity = 
					(PresentationEventsEntity) newEvent.getNextEntity();
			PresentationEventsEntity prevPresEntity = 
					(PresentationEventsEntity) newEvent.getPreviousEntity();
			
			if(event.getStartTimeInfo().getMovieTime() < prevPresEntity.getEndTime()){
				event.getStartTimeInfo().setMovieTime(prevPresEntity.getEndTime());
			}
			
			int frameDuration = segmentPanel.getZoomer()
					.getCoordinateTimeConvertor().getMovieFrameDuration();
			int startMovieTime = event.getStartTimeInfo().getMovieTime();
			int mEndMovieTime = startMovieTime + frameDuration * 5;
			if (event.getEndTimeInfo().getMovieTime() < mEndMovieTime) {
				event.getEndTimeInfo().setMovieTime(mEndMovieTime);
			}
			
			if(event.getEndTimeInfo().getMovieTime() > nextPresEntity.getStartTime()){
				event.getEndTimeInfo().setMovieTime(nextPresEntity.getStartTime());
			}

			boolean isUnique = false;
			counter = prField.getFieldSize() + 1;

			String label = "";

			while (!isUnique) {
				label = "U" + counter;
				isUnique = prField.isEventLabelUnique(label);
				counter++;
			}

			event.setText(label);

			PresentationEventsEntity entity = new PresentationEventsEntity(
					event, segmentPanel, prField);

			prField.addEventsEntity(entity);

			item.setStartTimeInfo(event.getStartTimeInfo());
			item.setEndTimeInfo(event.getEndTimeInfo());
			item.setLabel(event.getText());
			item.setId(event.getID());

			prField.getSS3GlossWindowSegment().addSS3GlossWindowItem(item);

			for (int j = 0; j < segmentsFieldWrapper.size(); j++) {
				SegmentFieldWrapper wrapper = segmentsFieldWrapper.get(j);

				PresentationEventsEntity clonedEntity = entity.clone();
				entity.getStartTimeInfo().addDependentMovieTime(
						clonedEntity.getStartTimeInfo());
				entity.getEndTimeInfo().addDependentMovieTime(
						clonedEntity.getEndTimeInfo());
				entity.addDependentEntity(clonedEntity);
				wrapper.addEventsEntity(clonedEntity);

				SS3GlossWindowItem pItem = new SS3GlossWindowItem(wrapper.getPartitionLabel(), wrapper.getSegmentLabel());
				pItem.setId(Util.getUniqueNumber());
				pItem.setStartTimeInfo(item.getStartTimeInfo());
				pItem.setEndTimeInfo(item.getEndTimeInfo());
				pItem.setLabel(event.getText());
				//
				PresentationEvent pEvent = clonedEntity.getEvent();
				pEvent.setSS3GlossWindowItem(pItem);
				pItem.setEvent(pEvent);
				item.addSS3GlossWindowItem(pItem);

				PresentationField dependentPresentationField = (PresentationField) wrapper
						.getField();
				dependentPresentationField.getSS3GlossWindowSegment()
						.addSS3GlossWindowItem(pItem);
			}

			entity.select();

			MediaToolBar tb = segmentPanel.getToolBarPanels().getMediaToolBar();
			tb.selectEvent(event, entity, prField);

			prField.setNewEvent(null);
		} else {
			MediaToolBar tb = segmentPanel.getToolBarPanels().getMediaToolBar();
			tb.updateEndTimeText();
		}

 		segmentPanel.repaint();
	}

	private int getCursorType(Point point) {
		int cursorType = Cursor.DEFAULT_CURSOR;
		Event event = prField.getSelectedEvent();

		if (event != null) {
			int startCoord = event.getStartTimeCoordinate();
			int endCoord = event.getEndTimeCoordinate();

			if (startCoord - 3 < point.x && point.x < startCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_START_POINT;
			}

			if (endCoord - 3 < point.x && point.x < endCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_END_POINT;
			}
		}
		return cursorType;
	}

	@Override
	public void unselectAll() {
		field.unselect();
	}

	@Override
	public int getStartTime() {
		return field.getStartTime();
	}

	@Override
	public int getEndTime() {
		return field.getEndTime();
	}

	@Override
	public int getViewLineNumber() {
		if (viewLineNumber >= 0)
			return viewLineNumber;
		return Integer.parseInt(field.getFieldID());
	}

	@Override
	public Event getSelectedEvent() {
		return field.getSelectedEvent();
	}

	private void cantAutomaticallyAdjustObjectsTime(String[] msg) {
		final JPopupMenu popup = new JPopupMenu();

		JLabel msgLabel1 = new JLabel(msg[0]);
		JLabel msgLabel2 = new JLabel(msg[1]);
		JLabel msgLabel3 = new JLabel(msg[2]);
		JLabel msgLabel4 = new JLabel(msg[3]);

		UserInterfaceUtil.setLabelLNF(msgLabel1);
		UserInterfaceUtil.setLabelLNF(msgLabel2);
		UserInterfaceUtil.setLabelLNF(msgLabel3);
		UserInterfaceUtil.setLabelLNF(msgLabel4);

		final JButton okBtn = new JButton("OK");

		JPanel msgPanel = new JPanel();
		GridBagConstraints gbc = new GridBagConstraints();

		msgPanel.setLayout(new GridBagLayout());
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridwidth = 3;
		gbc.gridx = 0;
		gbc.gridy = 0;
		msgPanel.add(msgLabel1, gbc);

		gbc.gridy = 1;
		gbc.gridx = 0;
		msgPanel.add(msgLabel2, gbc);

		gbc.gridy = 2;
		gbc.gridx = 0;
		msgPanel.add(msgLabel3, gbc);

		gbc.gridy = 3;
		gbc.gridx = 0;
		msgPanel.add(msgLabel4, gbc);

		gbc.gridy = 4;
		gbc.gridx = 0;
		msgPanel.add(new JLabel(" "), gbc);

		gbc.gridwidth = 1;
		gbc.anchor = GridBagConstraints.EAST;
		gbc.gridy = 5;
		gbc.gridx = 0;
		msgPanel.add(new JLabel(" "), gbc);
		gbc.gridx = 2;
		msgPanel.add(okBtn, gbc);

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				popup.setVisible(false);
			}
		});

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				okBtn.requestFocus();
			}
		});

		Dimension d = new Dimension();
		int width = msgPanel.getPreferredSize().width + 30;
		int height = msgPanel.getPreferredSize().height + 20;
		d.setSize(width, height);

		CoordinateTimeConverter convertor = segmentPanel.getZoomer()
				.getCoordinateTimeConvertor();
		int currentTime = segmentPanel.getMultipleMovieController()
				.getCurrentTime();
		int x = convertor.convertTimeToCoordinate(currentTime);

		int y = field.getY() + height / 2 + 50;

		popup.add(msgPanel);
		popup.setPopupSize(d);
		popup.show(field.getSignStreamSegmentPanel().getDatabaseView(), x, y);
		popup.setVisible(true);
		
		SS3Singleton.getSignStreamApplication().forcePopupInvisibleOnWindowLostFocus(popup);
	}

	@Override
	public void mouseHoldReleased(Point point, JPanel panel) {

		Event event = field.getSelectedEvent();

		if (event == null)
			return;

		final PresentationEvent pEvent = (PresentationEvent) event;

		final SegmentGraphicRepresentation repPanel = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation();

		final JPopupMenu popup = new JPopupMenu();
		int startCoord = pEvent.getStartTimeCoordinate();
		int width = event.getEndTimeCoordinate() - startCoord + 2;
		final JTextField labelTextField = new JTextField();
		labelTextField.setText(pEvent.getText());
		labelTextField.setPreferredSize(new Dimension(width, 17));
		labelTextField.setBounds(new Rectangle(0, 0, 0, 0));
		labelTextField.setBackground(UIPalette.getBackground());
		labelTextField.setForeground(Color.black);
		labelTextField.setSize(5, 10);
		popup.setBounds(0, 0, 0, 0);
		popup.setBorder(BorderFactory.createEmptyBorder());
		popup.setBackground(UIPalette.getBackground());
		popup.add(labelTextField);

		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				labelTextField.requestFocus();
			}
		});

		labelTextField.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				JTextField textField = (JTextField) e.getSource();
				String text = textField.getText();
				pEvent.setText(text);

				PresentationEventsEntity selectedEntity = (PresentationEventsEntity) field
						.getSelectedEntity();
				ArrayList<PresentationEventsEntity> entities = selectedEntity
						.getDependentEntities();

				SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);

				for (PresentationEventsEntity entity : entities) {
					entity.getEvent().setText(text);
					SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				}

				if (e.getKeyCode() == 10) {
					popup.setVisible(false);
					repPanel.repaint();
				}
			}

			@Override
			public void keyTyped(KeyEvent e) {
				;
			}

			@Override
			public void keyPressed(KeyEvent e) {
				;
			}
		});

		popup.setPreferredSize(popup.getPreferredSize());
		int y = field.getY() - 11;

		popup.show(repPanel, startCoord, y);
		popup.setVisible(true);
		
		labelTextField.addFocusListener(new FocusListener() {
			@Override
			public void focusGained(FocusEvent e) {
			}
			@Override
			public void focusLost(FocusEvent e) {
				popup.setVisible(false);
			}		
		});
	}
}
