package edu.bu.signstream.grepresentation.fields.presentation;

import java.awt.Cursor;
import java.awt.HeadlessException;
import java.awt.Point;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.JOptionPane;
import javax.swing.JPanel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.Manager;
import edu.bu.signstream.grepresentation.reguestsHandler.RequestHandler;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.main.Slider;
import edu.bu.signstream.grepresentation.view.main.Zoomer;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SegmentFieldWrapper extends FieldWrapper {
	private PresentationField prField = null;
	private String partitionLabel = null;
	private String segmentLabel = null;

	public SegmentFieldWrapper(PresentationField field,
			SignStreamSegmentPanel segmentPanel, String partitionLabel,
			String segmentLabel) {
		super(field, segmentPanel);
		this.prField = field;
		this.partitionLabel = partitionLabel;
		this.segmentLabel = segmentLabel;
	}

	public void removeAllEntities() {
		prField.removeAllEntities();
	}

	public void setPartitionLabel(String partitionLabel) {
		this.partitionLabel = partitionLabel;
	}

	public String getPartitionLabel() {
		return partitionLabel;
	}

	public void setSegmentLabel(String segmentLabel) {
		this.segmentLabel = segmentLabel;
	}

	public String getSegmentLabel() {
		return segmentLabel;
	}

	@Override
	public void highlight(boolean highlight, String refID) {
		field.highlight(highlight, refID);
	}

	@Override
	public boolean contains(int Y) {
		return field.contains(Y);
	}

	public boolean canMoveEvent(Event event, int x) {
		return false;
	}

	@Override
	public void addEventsEntity(EventsEntity entity) {
		prField.addEventsEntity(entity);
	}

	// MouseListener
	@Override
	public void mouseClicked(Point point, JPanel panel) {
		Event event = field.selectEvent(point.x);
		if (event != null) {
			EventsEntity entity = field.getEntityAt(point.x);
			SS3Singleton.getMediaToolBar(null)
					.selectEvent(event, entity, field);
			event.select();
		}
	}

	@Override
	public void mouseClickedAlt(Point point) {
	}

	@Override
	public void mousePressedAlt(Point point, JPanel panel) {
	}

	@Override
	public void mouseRightClicked(Point point) {
	}

	@Override
	public void mouseLeftClicked(Point point) {
	}

	public void mouseDblClicked(Point point) {
	}
	
	public void mouseCntrAltDblLeftClicked(Point point) {
	}

	/*
	 * Show selected utterance/segment, etc
	 */
	@Override
	public void mouseDblLeftClicked(Point point, JPanel panel) {
		SS3SignStreamApplication.logger.log(Level.INFO, "Updating loaded database");
		
		segmentPanel.updateLoadedDatabase();

		int segmentMovieTime = segmentPanel.getMultipleMovieController().getCurrentTime();
		int segmentOnScreenLocation = segmentPanel.getDatabaseView().getVisiblePoint().x;
		
		SS3Database ss3 = segmentPanel.getSS3Database();
//		ss3.setTemporalPartitionPixelInTimeFrame(pixelInTimeFrame);
		ss3.setTemporalPartitionMovieTime(segmentMovieTime);
		ss3.setTemporalPartitionOnScreenLocation(segmentOnScreenLocation);
		
		SS3SignStreamApplication.logger.log(Level.INFO, "Updated Loaded database");

		Manager mngr = SS3Singleton.getManager();
		RequestHandler handler = mngr.getRequestHandler();
		handler.init(segmentPanel);

		SegmentGraphicRepresentation rep = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation(); 

		rep.getFieldWrappersCollection().unselectAll();

		ArrayList<PresentationEventsEntity> eventsEntitiesToLoad = field.getEntities();
		
		Event event = field.selectEvent(point.x);

		SS3SignStreamApplication.logger.log(Level.INFO, "Selected event " + event);
		
		if (event == null)
			return;

		PresentationEvent pEvent;

		if (event instanceof PresentationEventsEntity) {
			PresentationEventsEntity pEntity = (PresentationEventsEntity) event;
			pEvent = pEntity.getEvent();
		} else if (event instanceof PresentationEvent) {
			pEvent = (PresentationEvent) event;
		} else {
			return;
		}
		
		SS3SignStreamApplication.logger.log(Level.INFO, "Casted event " + pEvent);
		
		PresentationEventsEntity presentationEntity = null;
		
		EventsEntity entity = field.getSelectedEntity();
		if (entity != null) {
			presentationEntity = (PresentationEventsEntity) entity;
			
			MediaToolBar bar = SS3Singleton.getMediaToolBar(segmentPanel);
			bar.setSelectedEntity(presentationEntity);
			bar.setCurrentParentField(field);
		}

		ArrayList<SS3GlossWindowItem> surroundingItems = new ArrayList<SS3GlossWindowItem>();
		
		int size = eventsEntitiesToLoad.size();
		
		int startIndex = 0;
		int endIndex = size;	// top bound exclusive
		
		for (PresentationEventsEntity eventsEntity: eventsEntitiesToLoad.subList(startIndex, endIndex)) {
			surroundingItems.add(eventsEntity.getSS3GlossWindowItem());
		}
		
		SS3SignStreamApplication.logger.log(Level.INFO, "Selected segment tier represents " + surroundingItems.size() + " of WindowItem objects");
		SS3SignStreamApplication.logger.log(Level.INFO, "Loading window items");

		int movieStartTime = pEvent.getStartTimeInfo().getMovieTime();
		int movieEndTime = pEvent.getEndTimeInfo().getMovieTime();
		
		SegmentGraphReprUtil.utteranceMovieTimeOnset = movieStartTime; 
		SegmentGraphReprUtil.utteranceMovieTimeOffset = movieEndTime; 	
		
		SegmentGraphReprUtil.utteranceViewVisibleMovieOnset = movieStartTime;
		SegmentGraphReprUtil.utteranceViewVisibleMovieOffset = movieEndTime;

        double w = segmentPanel.getDatabaseView().getVisibleWidth2();
		int clipDuration = pEvent.getEndTimeInfo().getMovieTime() - pEvent.getStartTimeInfo().getMovieTime();
		long packetDuration = SS3Singleton.getVideoControlManager().getMultipleMovieController().getPacketDuration();

		segmentPanel.getSegments().clear();
		handler.convertMultipleItemsAndAddToSegmentPanel(surroundingItems, (PresentationField) field);
		handler.loadPredefinedPanelSelection();
		boolean displayWarningDialog = handler.displayWarningDialog();
		
		segmentPanel.getSlider().setSliderPositionCoord(entity.getStartTimeCoordinate());

		segmentPanel.getUtteranceView().setVisible(entity.getStartTimeCoordinate(), entity.getEndTimeCoordinate(), true);
		
		Zoomer zoomer = segmentPanel.getZoomer();
		segmentPanel.getZoomer().fillEntireUtterancePage(clipDuration, 0, w);
        segmentPanel.getUtteranceView().setZoomInOutValue(zoomer.zoom());

		SS3Singleton.getMediaToolBar(null).setPlayButtonLabel(MediaToolBar.UTTERANCE);	
		SS3Singleton.getMediaToolBar(null).selectEvent(null, null, null);

		segmentPanel.getUtteranceView().getSegmentTemplate().selectTemplate();
		segmentPanel.getUtteranceView().getSegmentTemplate().getSegmentControlPanel().setSelectedUtterance(entity);
		segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut().requestFocus();
		
		if (displayWarningDialog) {
			String warningDialogText = "Warning: some glosses in this collection were detected " 
					+ "\nto have lost their handshapes prior to the previous save."
					+ "\nThey have been recovered and their gloss name has been "
					+ "\nprepended with \"(recovered)\" for ease in searching.";
			String warningTitleText = "Recovered Glosses Warning";
			int optionType = JOptionPane.YES_NO_OPTION;
			int messageType = JOptionPane.WARNING_MESSAGE;
			String options[] = {"OK"};
			try {
				JOptionPane.showOptionDialog(SS3Singleton.getSignStreamApplication(),
						warningDialogText,
						warningTitleText,
						optionType,
						messageType,
						null,
						options,
						options[0]);
			} catch (HeadlessException he) {
				System.out.println("");
			}
		}
	}

	@Override
	public void mouseDblRightClicked(Point point, JPanel panel) {

	}

	// MouseMotionListener
	@Override
	public void mouseMoved(Point point, JPanel panel) {

		if (panel != null) {
			panel.setCursor(new Cursor(getCursorType(point)));

			Event event = prField.getEventAt(point.x);
			if (event != null) {
				panel.setToolTipText(event.getToolTipText());
			} else {
				panel.setToolTipText("");
			}
		}
	}

	@Override
	public void mouseDragged(Point point, JPanel panel) {

		if (panel.getCursor().getType() != Cursor.E_RESIZE_CURSOR)
			return;

		if (MOVE_DIRECTION == MOVE_START_POINT) {
			prField.getSelectedEvent().setStartTimeCoordinate(point.x);
			segmentPanel.getMultipleMovieController().setCurrentTime(
					prField.getSelectedEvent().getStartTimeInfo()
							.getMovieTime());
		}
		if (MOVE_DIRECTION == MOVE_END_POINT) {
			prField.getSelectedEvent().setEndTimeCoordinate(point.x);
			segmentPanel.getMultipleMovieController().setCurrentTime(
					prField.getSelectedEvent().getEndTimeInfo().getMovieTime());
		}

		Slider slider = segmentPanel.getSlider();
		slider.setSliderPositionCoord(point.x);
	}

	private int counter = 1;

	@Override
	public void mouseReleased(Point point, JPanel panel) {
		NewEvent newEvent = prField.getNewEvent();

		if (newEvent != null) {

			PresentationEvent event = new PresentationEvent(segmentPanel);
			event.setStartTimeCoordinate(newEvent.getStartTimeCoordinate());
			event.setEndTimeCoordinate(newEvent.getEndTimeCoordinate());

			int frameDuration = segmentPanel.getZoomer()
					.getCoordinateTimeConvertor().getMovieFrameDuration();
			int starMovieTime = event.getStartTimeInfo().getMovieTime();
			int mEndMovieTime = starMovieTime + frameDuration * 5;
			if (event.getEndTimeInfo().getMovieTime() < mEndMovieTime) {
				event.getEndTimeInfo().setMovieTime(mEndMovieTime);
			}

			boolean isUnique = false;
			counter = prField.getFieldSize() + 1;

			String label = "";

			while (!isUnique) {
				label = "U" + counter;
				isUnique = prField.isEventLabelUnique(label);
				counter++;
			}

			event.setText(label);

			PresentationEventsEntity entity = new PresentationEventsEntity(
					event, segmentPanel, prField);

			prField.addEventsEntity(entity);

			entity.select();

			MediaToolBar tb = segmentPanel.getToolBarPanels().getMediaToolBar();
			tb.selectEvent(event, entity, prField);

			prField.setNewEvent(null);
		} else {
			Event event = field.selectEvent(point.x);
			if (event != null) {
				EventsEntity entity = field.getEntityAt(point.x);
				SS3Singleton.getMediaToolBar(null).selectEvent(event, entity,
						prField);
			}
		}
	}

	private int getCursorType(Point point) {
		int cursorType = Cursor.DEFAULT_CURSOR;
		Event event = prField.getSelectedEvent();

		if (event != null) {
			int startCoord = event.getStartTimeCoordinate();
			int endCoord = event.getEndTimeCoordinate();

			if (startCoord - 3 < point.x && point.x < startCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_START_POINT;
			}

			if (endCoord - 3 < point.x && point.x < endCoord + 3) {
				cursorType = Cursor.E_RESIZE_CURSOR;
				MOVE_DIRECTION = MOVE_END_POINT;
			}
		}
		return cursorType;
	}

	@Override
	public void unselectAll() {
		field.unselect();
	}

	@Override
	public int getStartTime() {
		return field.getStartTime();
	}

	@Override
	public int getEndTime() {
		return field.getEndTime();
	}

	@Override
	public int getViewLineNumber() {
		if (viewLineNumber >= 0)
			return viewLineNumber;
		return Integer.parseInt(field.getFieldID());
	}

	@Override
	public Event getSelectedEvent() {
		return field.getSelectedEvent();
	}

	@Override
	public void mouseHoldReleased(Point point, JPanel panel) {
	}
}
