package edu.bu.signstream.grepresentation.listener;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Level;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GlossFieldsLabelView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.NonManualFieldsLabelView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;

public class GlossNRelatedFieldsListener extends SignStreamMouseListener {

	public GlossNRelatedFieldsListener(SignStreamSegmentPanel segmentPanel,
			SegmentGraphicRepresentation nonManualRep,
			SegmentGraphicRepresentation glossRep,
			NonManualFieldsLabelView nmLabelView,
			GlossFieldsLabelView glsLabelView) {

		super(segmentPanel, nonManualRep, glossRep, nmLabelView, glsLabelView);
	}

	private boolean mouseDragged = false;
	private boolean mouseDraggedAlt = false;

	/**
	 * set re sizable cursor (chained events, chained event entity, and slider)
	 * and show chained event duration
	 */
	@Override
	public void mouseMoved(MouseEvent e) {

		Point point = e.getPoint();

		if (slider != null
				&& point.x - 3 < slider.getSliderPosition()
				&& point.x + 3 > slider.getSliderPosition()) {
			glossRep.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		} else {
			glossRep.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		// change magnified events to normal
		FieldWrappersCollection wrappers = glossRep
				.getFieldWrappersCollection();

		FieldWrapper rhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_DOM_HAND_SHAPE);
		FieldWrapper lhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_NDOM_HAND_SHAPE);

		boolean minified = false;

		if (minifyAllHandShapeIcons(rhWrapper))
			minified = true;

		if (minifyAllHandShapeIcons(lhWrapper))
			minified = true;

		if (minified) {
			glossRep.repaint();
		}

		FieldWrapper wrapper = glossRep.getFieldWrappersCollection()
				.getFieldWrapperAt(point.y);

		if (wrapper != null) {
			wrapper.mouseMoved(point, glossRep);
			repaintAll();
		}
	}

	@Override
	public void mouseDragged(MouseEvent e) {

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Dragged");
		
		MultipleMovieController controller = segmentPanel.getMultipleMovieController(); 
		controller.pause();

		mouseDragged = true;
		Point point = e.getPoint();

		if (slider.getSliderPosition() == point.x)
			return;

		boolean isCommandPressed = isCommandPressed(e);
		boolean isShiftPressed = isShiftPressed(e);
		boolean isAltPressed = isAltPressed(e);

		if (!isCommandPressed && !isShiftPressed && !isAltPressed) {
			int startTime = SS3Singleton.getVideoControlManager().getZoomer()
					.getCoordinateTimeConvertor()
					.convertCoordinateToTime(point.x);

			SS3Singleton.getVideoControlManager().getMultipleMovieController()
					.setCurrentTime(startTime);
			
			SS3Singleton.getMediaToolBar(null).adjustSlider(startTime);

			repaintAll();
			return;
		} else if (isShiftPressed || isAltPressed) {
			FieldWrapper wrapper = glossRep.getFieldWrappersCollection()
					.getFieldWrapperAt(point.y);

			if (wrapper instanceof NonManualFieldWrapper) {
				NonManualFieldWrapper nmWrapper = (NonManualFieldWrapper) wrapper;
				glossRep.setCursor(new Cursor(nmWrapper.getCursorType(point)));
			}

			if (wrapper != null)
				wrapper.mouseDragged(point, glossRep);
			repaintAll();
		}
	}

	
	public FieldWrapper getFieldWrapperWithSelectedEvent() {
		return glossRep.getFieldWrappersCollection().getSelectedFieldWrapper();
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	private int ex = -10;

	@Override
	public void mousePressed(MouseEvent e) {
        segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut().requestFocus();

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Pressed");

		if (ex == e.getPoint().x)
			return;
		
		ex = e.getPoint().x;

		boolean isShiftPressed = isShiftPressed(e);
		boolean isAltPressed = isAltPressed(e);
		boolean isCtrlPressed = isCtrlPressed(e);

		if (!isShiftPressed && !isAltPressed && !isCtrlPressed)
			return;

		if (isAltPressed && isCtrlPressed)
			return;

		FieldWrapper wrapper = glossRep.getFieldWrappersCollection()
				.getFieldWrapperAt(e.getPoint().y);

		if (wrapper == null)
			return;

		if (wrapper instanceof GlossFieldWrapper) {
			GlossFieldWrapper glsWrapper = (GlossFieldWrapper) wrapper;
			GlossField glsField = (GlossField) glsWrapper.getField();

			if (isShiftPressed) {
				glsField.setSelectCompoundStartCoord(e.getPoint().x - 3);
				repaintAll();
			} else if (isAltPressed || isCtrlPressed) {
				glsWrapper.mousePressedAlt(e.getPoint(), glossRep);

				Event selectedEvent = wrapper.getSelectedEvent();
				if (selectedEvent != null) {
					segmentPanel.setSelectedEvent(wrapper.getSelectedEvent());
					segmentPanel.setSelectedFieldWrapper(wrapper);
				}

				repaintAll();
			}
		} else {
			if (isAltPressed || isCtrlPressed) {
				wrapper.mousePressedAlt(e.getPoint(), glossRep);
				repaintAll();
				if (e.getPoint().y > SegmentGraphReprUtil.axesBottomHeight) {
					unselectAll();
				}
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Released");

		Point point = e.getPoint();
		FieldWrapper wrapper = glossRep.getFieldWrappersCollection().getFieldWrapperAt(e.getPoint().y);

		if (mouseDragged) {
			mouseDragged = false;
			
			try {
				int time = segmentPanel.getMultipleMovieController().getCurrentTime();
				long packetDuration = segmentPanel.getMultipleMovieController().getPacketDuration();
				int frames = (int)(time/packetDuration);
				int cTime = (int)(frames * packetDuration);
				slider.setSliderPositionTime(cTime);
			} catch(NullPointerException ex) {
				;
			}
		}

		if (wrapper != null) {
			Event selectedEvent = wrapper.getSelectedEvent();
			if (selectedEvent != null) {
				segmentPanel.setSelectedFieldWrapper(wrapper);
				segmentPanel.setSelectedEvent(selectedEvent);
			}

			wrapper.mouseReleased(point, glossRep);
			SS3Singleton.getMediaToolBar(null).updateEndTimeText();

			repaintAll();
		}

		// unused block
		if (wrapper instanceof HandShapeFieldWrapper && mouseDraggedAlt) {
			HandShapeFieldWrapper hsWrapper = (HandShapeFieldWrapper) wrapper;
			hsWrapper.mouseReleasedAlt(point);
			mouseDraggedAlt = false;
			repaintAll();
		}
	}

	// to prevent multiple call of dblClick method
	private Point lastMouseClickedPoint = null;

	// select chained event or chained events entity
	@Override
	public void mouseClicked(final MouseEvent e) {

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Clicked");

		if (mouseDragged) {
			// do not select
			mouseDragged = false;
			return;
		}
		
		unselectAll();
		segmentPanel.repaint();

		Point point = e.getPoint();

		FieldWrapper wrapper = glossRep.getFieldWrappersCollection()
				.getFieldWrapperAt(point.y);
		
		EventsEntity entityAt = null;
		if (wrapper != null) {
			if (wrapper instanceof GlossFieldWrapper && e.getClickCount() == 2 && !isAltPressed(e) && !isCtrlPressed(e)) {
				GlossFieldWrapper glsFldWrapper = (GlossFieldWrapper) wrapper;
				glsFldWrapper.mouseCtrlAltClicked(point);
				return;
			}

			wrapper.mouseClicked(point, glossRep);

			if (wrapper.getSelectedEvent() != null) {
				entityAt = wrapper.getEntityAt(point.x);

				if (entityAt != null) {
					unselectAll();
					wrapper.mouseClicked(point, glossRep);
				}
			}

			// right btn
			if (e.getButton() == MouseEvent.BUTTON3 || isCtrlPressed(e) || isAltPressed(e)) {
				wrapper.mouseDblRightClicked(point, glossRep);
				highlightLabel(wrapper);
				return;
			}

			if (e.getClickCount() == 2 && wrapper != null) {

				if (lastMouseClickedPoint != null
						&& lastMouseClickedPoint.x == e.getX()
						&& lastMouseClickedPoint.y == e.getY()) {
					lastMouseClickedPoint = e.getPoint();
					return;
				}

				lastMouseClickedPoint = e.getPoint();

				// left btn
				if (e.getButton() == MouseEvent.BUTTON1) {
					wrapper.mouseDblLeftClicked(point, glossRep);
					highlightLabel(wrapper);
					return;
				}
			}
		}

		if (wrapper != null) {
			Event event = wrapper.getSelectedEvent();
			if (event != null) {
				segmentPanel.setSelectedEvent(wrapper.getSelectedEvent());
				segmentPanel.setSelectedFieldWrapper(wrapper);
				
				
				
				SS3Singleton.getMediaToolBar(null).selectEvent(event, entityAt,
						wrapper.getField());
				event.select();
			}
		}
		repaintAll();
	}

	@Override
	protected void highlightLabel(FieldWrapper wrapper) {
		String fieldID = wrapper.getField().getFieldID();

		LabelObject parentObject = glsLabelView.getParentLabelObject();

		ArrayList<LabelObject> glsLabels = parentObject.getAllChildren();
		// highlight only selected field label
		for (LabelObject object : glsLabels) {
			String id = object.getFieldID();
			if (fieldID.equalsIgnoreCase(id)) {
				object.setUpdatable(true);
			} else {
				object.setUpdatable(false);
			}
		}

		ArrayList<LabelObject> labels = nmLabelView.getLabels();
		// unhighlight all labels
		for (LabelObject object : labels) {
			object.setUpdatable(false);
		}

		nmLabelView.repaint();
		glsLabelView.repaint();
	}
}
