package edu.bu.signstream.grepresentation.listener;

import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.logging.Level;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.GlossFieldsLabelView;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.NonManualFieldsLabelView;
import edu.bu.signstream.grepresentation.view.SegmentGraphicRepresentation;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;

public class NonManualFieldsListener extends SignStreamMouseListener {

	public NonManualFieldsListener(SignStreamSegmentPanel segmentPanel,
			SegmentGraphicRepresentation nonManualRep,
			SegmentGraphicRepresentation glossRep,
			NonManualFieldsLabelView nmLabelView,
			GlossFieldsLabelView glsLabelView) {

		super(segmentPanel, nonManualRep, glossRep, nmLabelView, glsLabelView);
	}

	private boolean mouseDragged = false;
	private boolean mouseDraggedAlt = false;
	
//	// to prevent multiple call of dblClick method

	@Override
	public void mouseMoved(MouseEvent e) {
		
		Point point = e.getPoint();

		glossRep.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

		if (slider != null && point.x - 3 < slider.getSliderPosition()
				&& point.x + 3 > slider.getSliderPosition()) {
			nonManualRep.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		} else {
			nonManualRep.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		// change magnified events to normal
		FieldWrappersCollection wrappers = glossRep
				.getFieldWrappersCollection();

		FieldWrapper rhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_DOM_HAND_SHAPE);
		FieldWrapper lhWrapper = wrappers
				.getFieldWrapper(Constants.TYPE_NDOM_HAND_SHAPE);

		boolean minified = false;

		if (minifyAllHandShapeIcons(rhWrapper))
			minified = true;

		if (minifyAllHandShapeIcons(lhWrapper))
			minified = true;

		if (minified)
			glossRep.repaint();

		FieldWrapper wrapper = nonManualRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);
		if (wrapper != null)
			wrapper.mouseMoved(point, nonManualRep);
	}

	@Override
	public void mouseDragged(MouseEvent e) {
		
//		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Dragged");
//		SS3SignStreamApplication.logger.log(Level.INFO, "Current Movie Time: " + segmentPanel.getMultipleMovieController().getCurrentTime());
		
		MultipleMovieController controller = segmentPanel.getMultipleMovieController(); 
		controller.pause();

		mouseDragged = true;
		Point point = e.getPoint();

		if (slider.getSliderPosition() == point.x)
			return;

		boolean isCommandPressed = isCommandPressed(e);
		boolean isShiftPressed = isShiftPressed(e);
		boolean isAltPressed = isAltPressed(e);
		boolean isCtrlPressed = isCtrlPressed(e);

		if (!isCommandPressed && !isShiftPressed && !isAltPressed && !isCtrlPressed) {
			int movieTime = SS3Singleton.getVideoControlManager().getZoomer()
					.getCoordinateTimeConvertor().convertCoordinateToTime(point.x);

			controller.setCurrentTime(movieTime);

			SS3Singleton.getMediaToolBar(null).adjustSlider(movieTime);

			repaintAll();
			return;
		} else if (isShiftPressed || isAltPressed || isCtrlPressed) {
			FieldWrapper wrapper = nonManualRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

			if (wrapper instanceof NonManualFieldWrapper) {
				NonManualFieldWrapper nmWrapper = (NonManualFieldWrapper) wrapper;
				nonManualRep.setCursor(new Cursor(nmWrapper
						.getCursorType(point)));
			}

			if (wrapper != null)
				wrapper.mouseDragged(point, nonManualRep);
			repaintAll();
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
	}

	@Override
	public void mouseExited(MouseEvent e) {
	}

	private int ex = -10;

	@Override
	public void mousePressed(MouseEvent e) {
		segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut().requestFocus();

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Pressed");

        boolean isAltPressed = isAltPressed(e);
        boolean isCtrlPressed = isCtrlPressed(e);

		if (ex == e.getPoint().x)
			return;

		ex = e.getPoint().x;

		FieldWrapper wrapper = nonManualRep.getFieldWrappersCollection().getFieldWrapperAt(e.getPoint().y);

		if (wrapper == null)
			return;
		
		Event event = wrapper.getField().getEventAt(e.getPoint().x);		
		if (event == null) {	// only check to begin dragging if nothing is occupying this space already
			if (isAltPressed || isCtrlPressed) {
				wrapper.mousePressedAlt(e.getPoint(), nonManualRep);	// begin drawing line for new nonmanual event
				repaintAll();
				if (e.getPoint().y > SegmentGraphReprUtil.axesBottomHeight) {
					unselectAll();
					repaintAll();
				} 
			}
		}
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		
		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Released");
		
		Point point = e.getPoint();
		
		if (mouseDragged) {
			mouseDragged = false;
			
			try {
				int time = segmentPanel.getMultipleMovieController().getCurrentTime();
				long packetDuration = segmentPanel.getMultipleMovieController().getPacketDuration();
				int frames = (int)(time/packetDuration);
				int cTime = (int)(frames * packetDuration);
				slider.setSliderPositionTime(cTime);
			} catch(NullPointerException ex) {
				ex.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
			}
		}	
		
		FieldWrapper wrapper = nonManualRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

		if (wrapper != null) {
			wrapper.mouseReleased(point, nonManualRep);
			segmentPanel.setSelectedFieldWrapper(wrapper);
			Event selectedEvent = wrapper.getSelectedEvent();
			segmentPanel.setSelectedEvent(selectedEvent);
			SS3Singleton.getMediaToolBar(null).updateEndTimeText();

			repaintAll();
		}

		if (wrapper instanceof HandShapeFieldWrapper && mouseDraggedAlt) {		// this is dead code also - b/c hs wrapper
			HandShapeFieldWrapper hsWrapper = (HandShapeFieldWrapper) wrapper;
			hsWrapper.mouseReleasedAlt(point);
			mouseDraggedAlt = false;
			repaintAll();
		}
	}

	// select chained event or chained events entity
	@Override
	public void mouseClicked(final MouseEvent e) {

		SS3SignStreamApplication.logger.log(Level.INFO, "Mouse Clicked");
		SS3SignStreamApplication.logger.log(Level.INFO, "Current Movie Time: " + segmentPanel.getMultipleMovieController().getCurrentTime());
		
		if (mouseDragged) {
			// do not select
			mouseDragged = false;
			return;
		}

		Point point = e.getPoint();
		
		unselectAll();
		
		FieldWrapper wrapper = nonManualRep.getFieldWrappersCollection().getFieldWrapperAt(point.y);

		if (wrapper != null) {
            int clickCount = e.getClickCount();
            //
            System.out.println("clickCount = " + clickCount);
            //
            
			// right btn
			if (e.getButton() == MouseEvent.BUTTON3 || isCtrlPressed(e) || isAltPressed(e)) {
				if (wrapper instanceof NonManualFieldWrapper) {
					NonManualFieldWrapper nmFieldWrapper = (NonManualFieldWrapper) wrapper;
					nmFieldWrapper.mouseCtrlAltClicked(point, nonManualRep);
					highlightLabel(wrapper);
				}
			} else if (e.getClickCount() == 2 && e.getButton() == MouseEvent.BUTTON1) {	// left double click
				wrapper.mouseDblLeftClicked(point, nonManualRep);
				highlightLabel(wrapper);
				return;
			}

			wrapper.mouseClicked(point, nonManualRep);

		}
		repaintAll();
	}

	@Override
	protected void highlightLabel(FieldWrapper wrapper) {
		String fieldID = wrapper.getField().getFieldID();

		try {
			ArrayList labels = nmLabelView.getLabels();
			// highlight only selected field label
			for (int i = 0; i < labels.size(); i++) {
				LabelObject object = (LabelObject) labels.get(i);
				String id = object.getFieldID();
				if (fieldID.equalsIgnoreCase(id)) {
					object.setUpdatable(true);
				} else {
					object.setUpdatable(false);
				}
			}
			nmLabelView.repaint();

			ArrayList glsLabels = glsLabelView.getLabels();
			// unhighlight all labels
			for (int j = 0; j < glsLabels.size(); j++) {
				LabelObject object = (LabelObject) glsLabels.get(j);
				object.setUpdatable(false);
			}
			glsLabelView.repaint();
		} catch (NullPointerException ex) {
			return;
		}
	}
}
