package edu.bu.signstream.grepresentation.reguestsHandler;

import java.util.ArrayList;

import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Track;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldType;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextField;
import edu.bu.signstream.grepresentation.fields.freeTextField.FreeTextFieldWrapper;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.GlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.graphField.GraphField;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeField;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsField;
import edu.bu.signstream.grepresentation.fields.handShapes.joints.JointsFieldWrapper;
import edu.bu.signstream.grepresentation.fields.lTranslationField.LiteralTranslationWrapper;
import edu.bu.signstream.grepresentation.fields.lTranslationField.LiteraltranslationField;
import edu.bu.signstream.grepresentation.fields.legacyGlossField.LegacyGlossFieldWrapper;
import edu.bu.signstream.grepresentation.fields.locationField.LocationField;
import edu.bu.signstream.grepresentation.fields.locationField.LocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.movementField.MovementField;
import edu.bu.signstream.grepresentation.fields.movementField.MovementFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.*;
import edu.bu.signstream.grepresentation.fields.orientationField.OrientationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.partOfSpeechField.PartOfSpeechField;
import edu.bu.signstream.grepresentation.fields.partOfSpeechField.PartOfSpeechFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeField;
import edu.bu.signstream.grepresentation.fields.signTypeField.SignTypeFieldWrapper;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class FieldsFactory {

	public FieldWrapper createFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {

		SS3CodingScheme cs = SS3Singleton.getSS3CodingSchemeCollection()
				.getDefaultCodingScheme();

		String id = track.getId();
		SS3Field field = cs.getField(id);

		SS3FieldType ft = cs.getFieldType(field.getType());

		String type = ft != null ? ft.getType() : field.getType();

		FieldWrapper wrapper = null;
		
		if (type.equalsIgnoreCase(Constants.TYPE_GRAPH)) {

			wrapper = createGraphWrapper(track, segmentPanel, parentField);

		}else if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL)) {

			wrapper = createNonManualFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_NON_MANUAL_FREE_TEXT)) {
			wrapper = createNonManualFreeTextFieldWrapper(track, segmentPanel, parentField);
		} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_GLOSS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_GLOSS)) {

			wrapper = createGlossFieldWrapper(track, segmentPanel, parentField);

			if (type.equalsIgnoreCase(Constants.TYPE_NDOM_GLOSS)) {

				ArrayList entities = wrapper.getField().getEntities();
                for (Object entity1 : entities) {
                    ChainedEventsEntity entity = (ChainedEventsEntity) entity1;
                    ChainedEvent event = entity.getTextValue();
                    SS3Entity ent = track.getSS3Entity(entity.getID());

                    if (ent.getColor() != null)
                        event.setColor(ent.getColor());
                }
			}

		} else if (type.equalsIgnoreCase(Constants.TYPE_GLOSS_LEGACY)) {

			wrapper = createLegacyGlossFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_FREE_TEXT)) {

			wrapper = createFreeTextFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_SHAPE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SHAPE)) {

			wrapper = createHandShapeFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_POS_TAG)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_POS_TAG)) {

			wrapper = createPartOfSpeechFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_TEXT)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_TEXT)) {

			wrapper = createLiteralTranslationFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_LOCATION)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_LOCATION)) {

			wrapper = createHandShapeLocationFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_MOVEMENT)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_MOVEMENT)) {

			wrapper = createHandShapeMovementFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_ORIENTATION)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_ORIENTATION)) {

			wrapper = createHandShapeOrientationFieldWrapper(track,
					segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND)) {

			wrapper = createRightLeftFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_NON_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_NON_SEL_FINGERS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_NON_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_NON_BASE_JOINTS)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_SPREAD)
				|| type.equalsIgnoreCase(Constants.TYPE_DOM_HAND_THUMB_CONTACT)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_SPREAD)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_HAND_THUMB_CONTACT)) {

			wrapper = createJointsFieldWrapper(track, segmentPanel, parentField);

		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_LOCATION) || type.equalsIgnoreCase(Constants.TYPE_NDOM_LOCATION)) {
			wrapper = createLocationFieldWrapper(track, segmentPanel, parentField);
		} else if (type.equalsIgnoreCase(Constants.TYPE_DOM_SIGN_TYPE)
				|| type.equalsIgnoreCase(Constants.TYPE_NDOM_SIGN_TYPE)) {
			wrapper = createSignTypeFieldWrapper(track, segmentPanel, parentField);
		}

		if (wrapper != null)
			wrapper.setType(type);

		return wrapper;
	}
	
	public FieldWrapper createGraphWrapper(SS3Track track, SignStreamSegmentPanel segmentPanel,
			PresentationField parentField) {
		
		GraphField field = new GraphField(track, segmentPanel, parentField);
		GraphFieldWrapper wrapper = new GraphFieldWrapper(field, segmentPanel);
		if (wrapper != null)
			wrapper.setType(Constants.TYPE_GRAPH);
		wrapper.setVisible(true);
		return wrapper;
	}

	private FieldWrapper createJointsFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {

		JointsField field = new JointsField(track, segmentPanel, parentField);
		JointsFieldWrapper wrapper = new JointsFieldWrapper(field, segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createNonManualFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		NonManualField field = new NonManualField(track, segmentPanel, parentField);
		return new NonManualFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createNonManualFreeTextFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		NonManualField field = new NonManualField(track, segmentPanel, parentField);
		return new NonManualFreeTextFieldWrapper(field, segmentPanel);
	}

	private GlossFieldWrapper createGlossFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new GlossFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createLegacyGlossFieldWrapper(Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new LegacyGlossFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createLegacyGlossFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new LegacyGlossFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createGlossFieldWrapper(Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new GlossFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createRightLeftFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		 NonManualField field = new NonManualField(track, segmentPanel, parentField);
		return new RightLeftHandFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createRightLeftFieldWrapper(Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		GlossField field = new GlossField(track, segmentPanel, parentField);
		return new RightLeftHandFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createFreeTextFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		FreeTextField field = new FreeTextField(track, segmentPanel, parentField);
		return new FreeTextFieldWrapper(field, segmentPanel);
	}

	private FieldWrapper createHandShapeFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {

		HandShapeField field = new HandShapeField(track, segmentPanel, parentField);
		HandShapeFieldWrapper wrapper = new HandShapeFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeFieldWrapper(Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		HandShapeField field = new HandShapeField(track, segmentPanel, parentField);
		HandShapeFieldWrapper wrapper = new HandShapeFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeLocationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		LocationField field = new LocationField(track, segmentPanel, parentField);
		LocationFieldWrapper wrapper = new LocationFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeOrientationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		HandShapeField field = new HandShapeField(track, segmentPanel, parentField);
		OrientationFieldWrapper wrapper = new OrientationFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createHandShapeMovementFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		MovementField field = new MovementField(track, segmentPanel, parentField);
		MovementFieldWrapper wrapper = new MovementFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createPartOfSpeechFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		PartOfSpeechField field = new PartOfSpeechField(track, segmentPanel, parentField);
		PartOfSpeechFieldWrapper wrapper = new PartOfSpeechFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createPartOfSpeechFieldWrapper(Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		PartOfSpeechField field = new PartOfSpeechField(track, segmentPanel, parentField);
		PartOfSpeechFieldWrapper wrapper = new PartOfSpeechFieldWrapper(field,
				segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createLiteralTranslationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		LiteraltranslationField field = new LiteraltranslationField(track,
				segmentPanel, parentField);
		LiteralTranslationWrapper wrapper = new LiteralTranslationWrapper(
				field, segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}

	private FieldWrapper createLocationFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		LocationField field = new LocationField(track, segmentPanel, parentField);
		LocationFieldWrapper wrapper = new LocationFieldWrapper(field, segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}
	
	private FieldWrapper createSignTypeFieldWrapper(SS3Track track,
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		SignTypeField field = new SignTypeField(track, segmentPanel, parentField);
		SignTypeFieldWrapper wrapper = new SignTypeFieldWrapper(field, segmentPanel);
		wrapper.setVisible(false);
		return wrapper;
	}
}
