package edu.bu.signstream.grepresentation.reguestsHandler;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.db.*;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.*;
import edu.bu.signstream.grepresentation.fields.glossField.*;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationInfo;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.LocationType;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.locations.Region;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEntity;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeEvent;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeFieldWrapper;
import edu.bu.signstream.grepresentation.fields.handShapeField.HandShapeSubEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEntity;
import edu.bu.signstream.grepresentation.fields.locationField.LocationEvent;
import edu.bu.signstream.grepresentation.fields.locationField.LocationFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEvent;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsCollection;
import edu.bu.signstream.grepresentation.fields.nonManualField.ChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.fields.rightLeftField.RightLeftHandFieldWrapper;
import edu.bu.signstream.grepresentation.fields.signTypeField.*;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.RootLabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

import java.awt.geom.Line2D;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;

public class RequestHandler {

	private SS3Database database;
	private SignStreamSegmentPanel segmentPanel;
	private FieldsFactory factory;
	private boolean displayWarningDialog;

	public void init(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
		this.factory = new FieldsFactory();
		this.database = segmentPanel.getSS3Database();
		this.displayWarningDialog = false;

	}
	
	public boolean displayWarningDialog() {
		return this.displayWarningDialog;
	}

	// take a panel object with the Segment objects already loaded and load
	// utterance
	public void loadPredefinedPanelSelection() {
		segmentPanel.loadUtterance(segmentPanel.getSegments());
	}

	public Segment convertMultipleItemsAndAddToSegmentPanel(ArrayList<SS3GlossWindowItem> items,
															PresentationField parentField) {

		Segment segment = convertItemsToSegment(items, parentField);
		segmentPanel.getSegments().add(segment);

		return segment;
	}

	public Segment convertMultipleItemsAndAddToSegmentPanelAtIndex(ArrayList<SS3GlossWindowItem> items,
																   PresentationField parentField, int indexToInsert) {

		Segment segment = convertItemsToSegment(items, parentField);

		if (indexToInsert > -1 && indexToInsert <= segmentPanel.getSegments().size())
			segmentPanel.getSegments().add(indexToInsert, segment);

		return segment;
	}

	public Segment convertItemsToSegment(ArrayList<SS3GlossWindowItem> items, PresentationField parentField) {

		// check if we are adding the same segment tier (parentField) as an existing one
		// because, if so, just make a duplicate reference to it, so any changes made in
		// one reflect in the other
		if (!segmentPanel.getSegments().isEmpty()) {
			for (Segment seg : segmentPanel.getSegments()) {
				if (seg.getParentField().equals(parentField)) {
					return seg;
				}
			}
		}

		RootLabelObject root = getCodingSchemaAsTreeNode(items, parentField);
		Segment segment = new Segment(segmentPanel, parentField);
		segment.setSS3GlossWindowItems(items);
		segment.setRootLabelObject(root);

		return segment;
	}

	private RootLabelObject getCodingSchemaAsTreeNode(ArrayList<SS3GlossWindowItem> items, PresentationField parentField) {

		RootLabelObject root = new RootLabelObject();
		LabelObject nonManObject = new LabelObject("", "");
		LabelObject glossNRelatedObject = new LabelObject("", "");
		LabelObject graphObject = new LabelObject("", "");
		root.setNonManuelLabelObject(nonManObject);
		root.setGlossNRelatedLabelObject(glossNRelatedObject);
		root.setGraphLabelObject(graphObject);
		SS3CodingScheme schema = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		List<SS3Field> gphFields = schema.getFields(Constants.TYPE_GRAPH);
		// the collection's graph file(s)
		ArrayList<SS3MediaFile> graphFilesList = database.getGraphFilesCollection();

		for (int i = 0; i < graphFilesList.size(); i++) {
			String graphFilePath = graphFilesList.get(i).getFile().getAbsolutePath();
			Map<String, List<GraphFileRow>> rawGraphDataMap = parseGraphFileToRawFormat(graphFilePath);

			// for each hashmap in the ret - call addgraph field for it with the data
			for (Map.Entry<String, List<GraphFileRow>> entry : rawGraphDataMap.entrySet()) {

				String graphID = entry.getKey();

				// TODO: temporary hardcoding - delete the below later
				if (graphID.equals("10")) {
					graphID = "40001";
				} else if (graphID.equals("12")) {
					graphID = "40002";
				}

				List<GraphFileRow> rowList = entry.getValue();
				if (rowList == null) {
					continue;
				}

				for (SS3Field gf : gphFields) {
					if (gf.getId().trim().equalsIgnoreCase(graphID.trim())) {
						LabelObject lobj = addField(gf, graphObject, false, items, parentField);
						GraphFieldWrapper gphWrapper = ((GraphFieldWrapper) lobj.getWrapper());
						gphWrapper.setRawGraphList(rowList);
						gphWrapper.setGraphFileID(graphFilesList.get(i).getId());
						break;
					}
				}

			}
		}
        
        // add non-manual fields
        ArrayList<SS3Field> nmFields = schema.getFields(Constants.TYPE_NON_MANUAL, Constants.TYPE_NON_MANUAL_FREE_TEXT);

        for (SS3Field nmField : nmFields) {
            addField(nmField, nonManObject, false, items, parentField);
        }

		// add manual fields in the right order
		// dom gloss, ndom gloss, dom hand/dom hshape, ndom hand/ndom hshape, translation
		GlossFieldWrapper domGlsWrapper, ndomGlsWrapper;
		RightLeftHandFieldWrapper domHWrapper, ndomHWrapper;
		HandShapeFieldWrapper domHsWrapper, ndomHsWrapper;
		LocationFieldWrapper domLocationWrp, ndomLocationWrp;
		SignTypeFieldWrapper domSignTypeWrp, ndomSignTypeWrp;

		SS3Field domGlsField = schema.getField(schema.getFieldID(Constants.TYPE_DOM_GLOSS));
		SS3Field ndomGlsField = schema.getField(schema.getFieldID(Constants.TYPE_NDOM_GLOSS));
		LabelObject domGlsObj, ndomGlsObj;

        domGlsObj = addField(domGlsField, glossNRelatedObject, false, items, parentField);
        ndomGlsObj = addField(ndomGlsField, glossNRelatedObject, false, items, parentField);

		domGlsWrapper = (GlossFieldWrapper) domGlsObj.getWrapper();
		ndomGlsWrapper = (GlossFieldWrapper) ndomGlsObj.getWrapper();

		SS3Field ndLtField = schema.getField(schema.getFieldID(Constants.TYPE_NDOM_TEXT));

		SS3Track track = database.getTrack(ndLtField.getId(), items);

		ArrayList<SS3Entity> entities = track.getEntities();

		if (entities.size() == 1) {
			SS3Entity ent = entities.get(0);
			ArrayList<SS3Item> entItems = ent.getItems();
			if (entItems.size() == 1) {
				SS3Item itm = entItems.get(0);
				String text = itm.getText();
				if (text.indexOf(" ") > 0) {
					track.clear();
				}
			}
		}

		// set dependency, gloss entities
		if (domGlsWrapper != null && ndomGlsWrapper != null) {
			GlossField glsField = (GlossField) domGlsWrapper.getField();

			GlossChainedEventsEntityCollection collection = glsField
					.getGlossChainedEventsEntityCollection();
			ArrayList<GlossChainedEventsEntity> glsEntities = collection
					.getEntities();

			for (GlossChainedEventsEntity cEntity : glsEntities) {
				String entID = cEntity.getDependentGlossEntityID();

				GlossField glsFld = ndomGlsWrapper.getGlossField();

				GlossChainedEventsEntityCollection glsCollection = glsFld
						.getGlossChainedEventsEntityCollection();
				GlossChainedEventsEntity ndEntity = glsCollection.getEntity(entID);

				if (ndEntity != null) {
					cEntity.setDependentGlossEntity(ndEntity);
					ndEntity.setDependentGlossEntity(cEntity);
				}
			}
		}

		// add right hand
		SS3Field domHField = schema.getField(schema.getFieldID(Constants.TYPE_DOM_HAND));
		SS3Field domHShapeField = schema.getField(schema.getFieldID(Constants.TYPE_DOM_HAND_SHAPE));
		SS3Field domLocField = schema.getField(schema.getFieldID(Constants.TYPE_DOM_LOCATION));
		SS3Field domSignTypeField = schema.getField(schema.getFieldID(Constants.TYPE_DOM_SIGN_TYPE));
		SS3Field ndomHField = schema.getField(schema.getFieldID(Constants.TYPE_NDOM_HAND));
		SS3Field ndomHShapeField = schema.getField(schema.getFieldID(Constants.TYPE_NDOM_HAND_SHAPE));
		SS3Field ndomLocField = schema.getField(schema.getFieldID(Constants.TYPE_NDOM_LOCATION));
		SS3Field ndomSignTypeField = schema.getField(schema.getFieldID(Constants.TYPE_NDOM_SIGN_TYPE));

		LabelObject domHObj, domHShapeObj, domLocLabelObj, domSignTypeLabelObj, ndomHObj, ndomHShapeObj, ndomLocLabelObj, ndomSignTypeLabelObj;

        domHObj = addField(domHField, glossNRelatedObject, false, items, parentField);
        domHShapeObj = addField(domHShapeField, domHObj, true, items, parentField);
        ndomHObj = addField(ndomHField, glossNRelatedObject, false, items, parentField);
        ndomHShapeObj = addField(ndomHShapeField, ndomHObj, true, items, parentField);

        domLocLabelObj = addField(domLocField, glossNRelatedObject, false, items, parentField);
        ndomLocLabelObj = addField(ndomLocField, glossNRelatedObject, false, items, parentField);
        domSignTypeLabelObj = addField(domSignTypeField, glossNRelatedObject, false, items, parentField);
        ndomSignTypeLabelObj = addField(ndomSignTypeField, glossNRelatedObject, false, items, parentField);

        domHObj.setLabel("dom hand");
        domHShapeObj.setLabel("dom hshape");
        ndomHObj.setLabel("ndom hand");
        ndomHShapeObj.setLabel("ndom hshape");
        domLocLabelObj.setLabel("dom loc");
        ndomLocLabelObj.setLabel("ndom loc");
        domSignTypeLabelObj.setLabel("dom sign type");
        ndomSignTypeLabelObj.setLabel("ndom sign type");

		domHWrapper = (RightLeftHandFieldWrapper) domHObj.getWrapper();
		domHsWrapper = (HandShapeFieldWrapper) domHShapeObj.getWrapper();
		ndomHWrapper = (RightLeftHandFieldWrapper) ndomHObj.getWrapper();
		ndomHsWrapper = (HandShapeFieldWrapper) ndomHShapeObj.getWrapper();

		domLocationWrp = (LocationFieldWrapper) domLocLabelObj.getWrapper();
		ndomLocationWrp = (LocationFieldWrapper) ndomLocLabelObj.getWrapper();
		domSignTypeWrp = (SignTypeFieldWrapper) domSignTypeLabelObj.getWrapper();
		ndomSignTypeWrp = (SignTypeFieldWrapper) ndomSignTypeLabelObj.getWrapper();

		setGlossAndRelated(domGlsWrapper, ndomGlsWrapper, domHWrapper, domHsWrapper, domLocationWrp, domSignTypeWrp);
		setGlossAndRelated(ndomGlsWrapper, domGlsWrapper, ndomHWrapper, ndomHsWrapper, ndomLocationWrp, ndomSignTypeWrp);

		SS3Field etField = schema.getField(Util.FREE_TEXT_FIELD_ID);
		addField(etField, glossNRelatedObject, false, items, parentField);

		setDependency(domGlsWrapper, ndomGlsWrapper, domHWrapper, ndomHWrapper, domHsWrapper, ndomHsWrapper);

        addSignTypeDependencies(domGlsWrapper, ndomGlsWrapper, domSignTypeWrp, ndomSignTypeWrp);

		return root;
	}

	private void setGlossAndRelated(GlossFieldWrapper glsWrp1, GlossFieldWrapper glsWrp2,
									RightLeftHandFieldWrapper handWrp, HandShapeFieldWrapper handShapeWrp, LocationFieldWrapper locationWrp,
									SignTypeFieldWrapper signTypeWrp) {

		GlossChainedEventsEntityCollection glsCollection = glsWrp1.getGlossField()
				.getGlossChainedEventsEntityCollection();

		if (glsCollection != null) {
			ArrayList<GlossChainedEventsEntity> handGlosses = glsCollection.getEntities();

			for (GlossChainedEventsEntity gloss : handGlosses) {

				setDependentGlossEntity(gloss, glsWrp2);

				setDependencies(gloss, glsWrp1.getGlossField().getFieldID(), handWrp, handShapeWrp, locationWrp, signTypeWrp);
			}
		}
	}

	private void setDependencies(GlossChainedEventsEntity gloss, String glossFieldID, RightLeftHandFieldWrapper handWrp,
								 HandShapeFieldWrapper handShapeWrp, LocationFieldWrapper locationWrp, SignTypeFieldWrapper signTypeWrp) {
		String depHandID = gloss.getDependentHandEntityID();
		String depHandShapeID = gloss.getDependentHandShapeEntityID();
		String depLocationID = gloss.getDependentLocationEntityID();
		String depSignTypeID = gloss.getDependentSignTypeEntityID();

		// block concerning dependencies for hand ID (depHandID) and hand wrapper
		// (handWrp)
		if (depHandID != null && !depHandID.isEmpty()) {
			EventsEntity hndEnt = handWrp.getField().getEntity(depHandID);

			// if hand entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (hndEnt != null) {
				ChainedEventsEntity hndEntity = (ChainedEventsEntity) hndEnt;
				hndEnt.setRefEntityId(gloss.getID());
				gloss.setDependentHandEntity(hndEntity);
			} else {
				// else: gloss has hand entity ID saved, but there is not a corresponding hand
				// entity object in database/track (and thus the wrapper)
				String fieldID = handWrp.getField().getFieldID();
				SS3Track hndTrack = database.getTrack(fieldID);

				ChainedEventsEntity handEntity = null;

				// try to re-create hand entity strictly from db track
				SS3Entity handSS3Entity = hndTrack.getSS3Entity(depHandID);
				if (handSS3Entity != null) {
					handEntity = createTrackEntity(handSS3Entity, handWrp.getField());
				}

				// if db recreation failed, make a new hand entity object and give it ID of
				// depHandID
				if (handEntity == null) {
					ChainedEventsCollection collection = new ChainedEventsCollection();
					GlossChainedEvent glossTextValueEvent = (GlossChainedEvent) gloss.getTextValue();
					collection.setTextValueEvent(glossTextValueEvent.cloneGlossChainedEvent());
					handEntity = new ChainedEventsEntity(collection, segmentPanel, handWrp.getField());

					handEntity.setID(depHandID);
					handEntity.setField(handWrp.getField());
					handEntity.determineParentEntity();

					boolean isLockedGlossPhonAlignments = gloss.isLockedGlossPhonAlignments();
					handEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);

					handEntity.setStartTimeInfo(gloss.getStartTimeInfo());
					handEntity.setEndTimeInfo(gloss.getEndTimeInfo());
				}

				// add hand entity into the already created hand wrapper
				handWrp.addEventsEntity(handEntity);
				// handWrp.getField().addEventsEntity(handEntity); // commenting this - it does
				// the same thing as the above line

				// set reference id's in both directions (gloss entity <--> hand entity)
				handEntity.setRefEntityId(gloss.getID());
				gloss.setDependentHandEntity(handEntity);
			}
		}

		// block concerning dependencies for hs ID (depHandShapeID) and hs wrapper
		// (handShapeWrp)
		if (depHandShapeID != null && !depHandShapeID.isEmpty()) {
			EventsEntity hndShapeEnt = handShapeWrp.getHandShapeField().getEntity(depHandShapeID);

			// if hs entity was properly constructed and is in the wrapper already, set
			// dependencies using that one
			if (hndShapeEnt != null) {
				HandShapeEntity hndShapeEntity = (HandShapeEntity) hndShapeEnt;
				hndShapeEntity.setRefEntityId(
						gloss.getDependentHandEntity() != null ? gloss.getDependentHandEntityID() : null);
				gloss.setDependentHandShapeEntity(hndShapeEntity);
			} else {
				// else: gloss has hs entity ID saved, but there is not a corresponding hs
				// entity object in database/track (and thus the wrapper)
				String fieldID = handShapeWrp.getField().getFieldID();
				SS3Track handShapeTrack = database.getTrack(fieldID);

				HandShapeEntity handShapeEntity = null;

				SS3Entity handShapeSS3Entity = handShapeTrack.getSS3Entity(depHandShapeID);

				// try to re-create hs entity strictly from db track
				if (handShapeSS3Entity != null) {
					handShapeEntity = new HandShapeEntity(fieldID, handShapeSS3Entity, segmentPanel,
														  handShapeWrp.getHandShapeField());
					handShapeEntity.setField(handShapeWrp.getField());

					handShapeEntity.setID(handShapeSS3Entity.getID());
					handShapeEntity.setRefEntityId(handShapeSS3Entity.getRefEntityId());
					handShapeEntity.determineParentEntity();
				}

				// if db recreation failed, make a new hs entity object and give it ID of
				// depHandShapeID
				if (handShapeEntity == null) {
					handShapeEntity = new HandShapeEntity(segmentPanel, handShapeWrp.getHandShapeField());

					handShapeEntity.setID(depHandShapeID);
					handShapeEntity.setField(handShapeWrp.getHandShapeField());

					HandShapeSubEntity subEntity = new HandShapeSubEntity(segmentPanel,
																		  handShapeWrp.getHandShapeField());
					subEntity.setID(handShapeEntity.getID());
					
					SS3Item item = new SS3Item();
					item.setId("94");	// hardcode for now - this is the "other" or "none" or "question mark" handshape symbol, dummy value
					item.setStartTimeInfo(gloss.getStartTimeInfo());
					item.setEndTimeInfo(gloss.getEndTimeInfo());
					
					// also tag the gloss name to something searchable
					ChainedEvent textvalue = gloss.getTextValue();
					textvalue.setText("(recovered) " + textvalue.getText());
					
					HandShapeEvent hsEvent = new HandShapeEvent(item, segmentPanel);
					subEntity.addHandShape(hsEvent);
					
					handShapeEntity.addHandShapeSubEntity(subEntity);
					
					handShapeEntity.determineParentEntity();
					
					this.displayWarningDialog = true;
				}

				handShapeWrp.addEventsEntity(handShapeEntity);

				handShapeEntity.setRefEntityId(
						gloss.getDependentHandEntity() != null ? gloss.getDependentHandEntityID() : null);
				gloss.setDependentHandShapeEntity(handShapeEntity);
			}
		}

		if (depLocationID != null && !depLocationID.isEmpty()) {
			LocationEntity locEntity = (LocationEntity) locationWrp.getField().getEntity(depLocationID);
			if (locEntity != null) {
				HashMap<LocationType, Region> locations = new HashMap<LocationType, Region>();
				for (LocationType locationType : SS3Singleton.getLocationConfig().getLocationTypes()) {
					for (LocationEvent event : locEntity.getEvents()) {
						if (event.getLocationType().equals(locationType)) {
							locations.put(locationType, event.getLocation());
							break;
						}
					}
				}
				LocationInfo parsedInfo = new LocationInfo(locations);
				if (parsedInfo.isEmpty()) {
					locationWrp.getField().deleteEntity(locEntity);
				} else {
					gloss.getGlossEntityProperties().setLocationInfo(parsedInfo);
					
					gloss.setDependentLocationEntity(locEntity);
	
					GlossChainedEventsEntity depGlsEntity = gloss.getDependentGlossEntity();
					if (depGlsEntity != null) {
						depGlsEntity.setDependentLocationEntity(locEntity);
					}
	
					if (glossFieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
						locEntity.setRefGlossEntity(gloss);
					} else {
						if (depGlsEntity != null) {
							locEntity.setRefGlossEntity(depGlsEntity);
						} else {
							locEntity.setRefGlossEntity(gloss);
						}
					}
	
					locEntity.setStartTime(gloss.getStartTimeInfo().getMovieTime());
					locEntity.setEndTime(gloss.getEndTimeInfo().getMovieTime());
				}
			} else {
				gloss.setDependentLocationEntityID(null);
			}
		}

		if (depSignTypeID != null && !depSignTypeID.isEmpty()) {
			SignTypeEntity signTypeEntity = (SignTypeEntity) signTypeWrp.getField().getEntity(depSignTypeID);
			if (signTypeEntity != null) {
				gloss.setDependentSignTypeEntity(signTypeEntity);

				GlossChainedEventsEntity depGlsEntity = gloss.getDependentGlossEntity();
				if (depGlsEntity != null) {
					depGlsEntity.setDependentSignTypeEntity(signTypeEntity);
					;
				}

                if (glossFieldID.equals(Util.DOM_GLOSS_FIELD_ID)) {
					signTypeEntity.setRefGlossEntity(gloss);
				} else {
					if (depGlsEntity != null) {
						signTypeEntity.setRefGlossEntity(depGlsEntity);
					} else {
						signTypeEntity.setRefGlossEntity(gloss);
					}
				}
				signTypeEntity.getEvent().setSignType(SignType.getSignTypeFromGloss(gloss));
				signTypeEntity.getEvent().setNameSign(gloss.getGlossEntityProperties().isNameSign());
			} else {
				gloss.setDependentSignTypeEntity(null);
			}
		}
	}

	private ChainedEventsEntity createTrackEntity(SS3Entity entity, Field field) {

		String fieldID = field.getFieldID();

		ChainedEventsCollection collection = new ChainedEventsCollection();
		ArrayList<SS3Item> items = entity.getItemsAsList();

		for (SS3Item vItem : items) {
			String vID = vItem.getId();
			if (!vID.equalsIgnoreCase("" + Constants.ONSET_CODE) && !vID.equalsIgnoreCase("" + Constants.OFFSET_CODE)
					&& !vID.equalsIgnoreCase("" + Constants.HOLD_CODE)) {
				if (fieldID.equalsIgnoreCase("10501") || fieldID.equalsIgnoreCase("10502")) {
					collection.setTextValueEvent(new GlossChainedEvent(vItem, segmentPanel));
				} else {
					collection.setValueEvent(new ChainedEvent(vItem, segmentPanel));
				}
			}
		}

		for (SS3Item item : items) {
			String id = item.getId();

			if (id.equalsIgnoreCase("" + Constants.ONSET_CODE)) {
				collection.setOnsetEvent(new ChainedEvent(item, segmentPanel));
			} else if (id.equalsIgnoreCase("" + Constants.OFFSET_CODE)) {
				collection.setOffsetEvent(new ChainedEvent(item, segmentPanel));
			} else {
				if (collection.getValueEvent() == null) {
//					System.out.println("no value event!!!" + id);
				}
				ChainedEvent valueEvent = collection.getValueEvent();
				if (valueEvent == null) {
					valueEvent = collection.getTextValueEvent();
				}
				int startTime = valueEvent.getStartTimeInfo().getMovieTime();
				int endTime = valueEvent.getEndTimeInfo().getMovieTime();
				if (startTime != endTime) {
					if (item.getStartTimeInfo().getMovieTime() == endTime) {
						collection.setFinalHoldEvent(new ChainedEvent(item, segmentPanel));
					} else if (item.getEndTimeInfo().getMovieTime() == startTime) {
						collection.setInitialHoldEvent(new ChainedEvent(item, segmentPanel));
					}
				}
			}
		}

		ChainedEventsEntity cEntity = new ChainedEventsEntity(collection, segmentPanel, field);
		cEntity.setID(entity.getID());
		cEntity.setField(field);
		cEntity.setRefEntityId(entity.getRefEntityId());
		cEntity.determineParentEntity();

		boolean isLockedGlossPhonAlignments = entity.isLockedGlossPhoneAlignments();
		cEntity.setLockedGlossPhonAlignments(isLockedGlossPhonAlignments);

		return cEntity;
	}

	private void setDependentGlossEntity(GlossChainedEventsEntity entity, GlossFieldWrapper glsWrp) {
		String id = entity.getDependentGlossEntityID();
		if (id != null && !id.isEmpty()) {
			EventsEntity ndEntity = glsWrp.getGlossField().getEntity(id);
			if (ndEntity != null) {
				GlossChainedEventsEntity dependentGlossEntity = (GlossChainedEventsEntity) ndEntity;
				entity.setDependentGlossEntity(dependentGlossEntity);
			}
		}
	}

	private LabelObject addField(SS3Field field, LabelObject parent, boolean addDependentWrapper,
								 ArrayList<SS3GlossWindowItem> items, PresentationField parentField) {

		if (field != null) {

			LabelObject obj = new LabelObject(field.getId(), field.getLabelOrName());

			SS3Track track = database.getTrack(field.getId(), items);

			FieldWrapper wrapper = factory.createFieldWrapper(track, segmentPanel, parentField);

			boolean isVisible = !track.getEntities().isEmpty();
			obj.setVisible(isVisible);
			wrapper.setVisible(isVisible);

			wrapper.getField().setColor(field.getColor());

			if (addDependentWrapper) {
				parent.getWrapper().addDependentWrapper(wrapper);
			}

			obj.setWrapper(wrapper);
			parent.addChild(obj);
			return obj;
		}

		return null;
	}

	// parse the input graph file to multiple maps of a raw format - i.e.
	// non-normalized
	private Map<String, List<GraphFileRow>> parseGraphFileToRawFormat(String file) {

		Map<String, List<GraphFileRow>> graphMap = new HashMap<String, List<GraphFileRow>>();
		String rowFieldID = null;
		String rowFieldName = null;
		double rowTimeValue = 0;
		double rowFieldValue = 0.0d;

		List<String[]> lines = new ArrayList<String[]>();
		BufferedReader in = null;

		try {
			in = new BufferedReader(new FileReader(file));
			String str = "";

			while ((str = in.readLine()) != null) {
				String[] splitLine = str.split(Util.CSV_REGEX_PATTERN, -1);
				lines.add(splitLine);
			}
		} catch (IOException e) {
			SS3SignStreamApplication.logger.log(Level.INFO, "File Read Error");
		}

		for (String[] line : lines) {

			// parts per line
			if (line.length != Util.GRAPH_FILE_LINE_ITEM_COUNT) {
				continue;
			}

			// time attribute
			String[] timeAttr = line[0].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (timeAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			String timeValue = timeAttr[1] != null ? timeAttr[1].trim() : "";

			// field id attribute
			String[] fieldIDAttr = line[1].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (fieldIDAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			// field name attribute
			String[] fieldNameAttr = line[2].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (fieldNameAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			// field value attribute
			String[] fieldValueAttr = line[3].split(Util.GRAPH_FILE_DELIMITER_ITEM);
			if (fieldValueAttr.length != Util.GRAPH_FILE_ITEM_COUNT) {
				continue;
			}

			try {
				rowTimeValue = Double.parseDouble(timeValue);
				rowFieldID = fieldIDAttr[1].trim();
				rowFieldName = fieldNameAttr[1].trim();
				rowFieldValue = Double.parseDouble(fieldValueAttr[1].trim());

				GraphFileRow row = new GraphFileRow(rowTimeValue, rowFieldID, rowFieldName, rowFieldValue);

				if (graphMap.get(rowFieldID) == null) {
					List<GraphFileRow> rows = new ArrayList<GraphFileRow>();
					rows.add(row);
					graphMap.put(rowFieldID, rows);
				}
				List<GraphFileRow> rows = graphMap.get(rowFieldID);
				rows.add(row);

			} catch (NumberFormatException e) {
				SS3SignStreamApplication.logger.log(Level.INFO, e.getMessage());
				continue;
			}

		}
		try {
			in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return graphMap;
	}

	private void setOneHandDependency(GlossFieldWrapper glsWrp, RightLeftHandFieldWrapper hndWrp,
									  HandShapeFieldWrapper hsWrp) {

		ArrayList hsEntities = hsWrp.getHandShapeField().getEntities();

		ArrayList hnEntities = hndWrp.getField().getEntities();

		ArrayList<HandShapeEntity> validHsEntities = new ArrayList<HandShapeEntity>();
		ArrayList<ChainedEventsEntity> validHndEntities = new ArrayList<ChainedEventsEntity>();

		for (Object hsObject : hsEntities) {
			HandShapeEntity hsEntity = (HandShapeEntity) hsObject;

			ChainedEventsEntity hndEntity = getHandEntity(hsEntity, hndWrp);
			GlossChainedEventsEntity glsEntity = getGlossEntity(hndEntity, glsWrp);

			if (hndEntity != null && glsEntity != null) {
				validHsEntities.add(hsEntity);
				validHndEntities.add(hndEntity);

				setDependencies(glsEntity, hndEntity, hsEntity);
			} else {
				glsEntity = getGlossEntity(hsEntity, glsWrp.getGlossField().getGlossChainedEventsEntityCollection());

				if (glsEntity != null) {
					hndEntity = glsEntity.clone(hndWrp.getField().getFieldID());
					hndWrp.getField().addEventsEntity(hndEntity);

					setDependencies(glsEntity, hndEntity, hsEntity);

					validHsEntities.add(hsEntity);
					validHndEntities.add(hndEntity);
				}
			}
		}

		// clean up hs entities
		if (validHsEntities.size() < hsEntities.size()) {
			ArrayList<HandShapeEntity> delHsEntities = new ArrayList<HandShapeEntity>();

			for (Object hsEntity1 : hsEntities) {
				HandShapeEntity hsEntity = (HandShapeEntity) hsEntity1;
				if (!validHsEntities.contains(hsEntity)) {
					delHsEntities.add(hsEntity);
				}
			}

			for (HandShapeEntity del : delHsEntities) {
				hsEntities.remove(del);
			}
		}

		// clean up hand entities
		if (validHndEntities.size() < hnEntities.size()) {
			ArrayList<ChainedEventsEntity> delHndEntities = new ArrayList<ChainedEventsEntity>();

			for (Object hnEntity : hnEntities) {
				ChainedEventsEntity hEntity = (ChainedEventsEntity) hnEntity;
				if (!validHndEntities.contains(hEntity)) {
					delHndEntities.add(hEntity);
				}
			}

			for (ChainedEventsEntity del : delHndEntities) {
				hnEntities.remove(del);
			}
		}
	}

	private void setDependency(GlossFieldWrapper glsWrp, GlossFieldWrapper ndGlsWrp,
							   RightLeftHandFieldWrapper rightHandWrp, RightLeftHandFieldWrapper leftHandWrp,
							   HandShapeFieldWrapper rightHandShapeWrp, HandShapeFieldWrapper leftHandShapeWrp) {

		setOneHandDependency(glsWrp, rightHandWrp, rightHandShapeWrp);
		setOneHandDependency(ndGlsWrp, leftHandWrp, leftHandShapeWrp);

		GlossChainedEventsEntityCollection glsColl = glsWrp.getGlossField().getGlossChainedEventsEntityCollection();
		GlossChainedEventsEntityCollection ndGlsColl = ndGlsWrp.getGlossField().getGlossChainedEventsEntityCollection();

		ArrayList<GlossChainedEventsEntity> ndGlss = ndGlsColl.getEntities();
		for (GlossChainedEventsEntity gls : ndGlss) {
			String refID = gls.getRefEntityId();
			GlossChainedEventsEntity nGls = glsColl.getEntity(refID);
			if (nGls != null) {
				gls.setDependentGlossEntity(nGls);
				nGls.setDependentGlossEntity(gls);
			} else {
				String txt = gls.getText();
			}
		}
	}

	private void addSignTypeDependencies(GlossFieldWrapper dGlsWrapper, GlossFieldWrapper ndGlsWrapper, SignTypeFieldWrapper dSignTypeFieldWrapper, SignTypeFieldWrapper ndSignTypeFieldWrapper) {
		SignTypeField dSignTypeField = (SignTypeField) dSignTypeFieldWrapper.getField();
		SignTypeField ndSignTypeField = (SignTypeField) ndSignTypeFieldWrapper.getField();

		ArrayList glossEntities = dGlsWrapper.getGlossField().getEntities();
		for (Object obj : glossEntities) {
			GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) obj;
			if (glsEntity.getDependentSignTypeEntity() == null) {
				SignTypeEntity newEntity = getSignTypeEntity(glsEntity, dSignTypeField);
				glsEntity.setDependentSignTypeEntity(newEntity);
				newEntity.setRefGlossEntity(glsEntity);
				GlossChainedEventsEntity depGlsEntity = glsEntity.getDependentGlossEntity();
				if (depGlsEntity != null) {
					depGlsEntity.setDependentSignTypeEntity(newEntity);
				}
			}
		}

		ArrayList ndGlossEntities = ndGlsWrapper.getGlossField().getEntities();
		for (Object obj : ndGlossEntities) {
			GlossChainedEventsEntity ndGlsEntity = (GlossChainedEventsEntity) obj;
			if (ndGlsEntity.getDependentGlossEntity() == null && ndGlsEntity.getDependentSignTypeEntity() == null) {
				SignTypeEntity newEntity = getSignTypeEntity(ndGlsEntity, ndSignTypeField);
				ndGlsEntity.setDependentSignTypeEntity(newEntity);
				newEntity.setRefGlossEntity(ndGlsEntity);
			}
		}
	}

	private SignTypeEntity getSignTypeEntity(GlossChainedEventsEntity glsEntity, SignTypeField signTypeField) {
		SignTypeEvent newEvent = new SignTypeEvent(segmentPanel, glsEntity.getStartTimeInfo(), glsEntity.getEndTimeInfo());
		newEvent.setSignType(SignType.getSignTypeFromGloss(glsEntity));
		newEvent.setNameSign(glsEntity.getGlossEntityProperties().isNameSign());

		SignTypeEntity newEntity = new SignTypeEntity(newEvent, segmentPanel, signTypeField);
		newEntity.setID(Util.getLongUniqueNumber());

		signTypeField.addEventsEntity(newEntity);
		return newEntity;
	}

	private ChainedEventsEntity getHandEntity(HandShapeEntity handShapeEntity, RightLeftHandFieldWrapper handWrp) {

		String refHandID = handShapeEntity.getRefEntityId();

		EventsEntity entity = null;

		// first try to get the hand entity based on the hs entity ref entity id
		if (refHandID != null)
			entity = handWrp.getField().getEntity(refHandID);

		// if the ref entity was blank or the hand entity in question is missing from
		// the wrapper for whatever, try by midpoint coord of handShapeEntity
		if (entity == null) {

			int startCoordinate = handShapeEntity.getStartTimeCoordinate();
			int endCoordinate = handShapeEntity.getEndTimeCoordinate();
			int midCoordinate = startCoordinate + (endCoordinate - startCoordinate) / 2;

			entity = handWrp.getField().getEntityAt(midCoordinate); // this eventually makes use of checking start ==
			// end (one frame)
		}

		// if we still haven't found our hand entity, look in the wrapper for an entity
		// matching the start/end points of this handShapeEntity
		if (entity == null) {
			ArrayList list = handWrp.getField().getEntities();

			for (int i = 0; i < list.size(); i++) {
				EventsEntity ent = (EventsEntity) list.get(i);
				int startTime = ent.getStartTimeInfo().getMovieTime();
				int endTime = ent.getEndTimeInfo().getMovieTime();

				int hsStartTime = handShapeEntity.getStartTime();
				int hsEndTime = handShapeEntity.getEndTime();

				if (startTime == hsStartTime && endTime == hsEndTime) {
					entity = ent;
				}
			}
		}

		if (entity != null && isValid(handShapeEntity, entity))
			return (ChainedEventsEntity) entity;

		return null;
	}

	private boolean isValid(EventsEntity entity1, EventsEntity entity2) {
		if (entity2 == null)
			return false;

		int startTime = entity1.getStartTimeInfo().getMovieTime();
		int endTime = entity1.getEndTimeInfo().getMovieTime();

		Line2D domLine = new Line2D.Double(startTime, 0, endTime, 0);

		int startMovieTime = entity2.getStartTimeInfo().getMovieTime();
		int w = entity2.getEndTimeInfo().getMovieTime() - startMovieTime;

		boolean isValid = domLine.intersects(startMovieTime, 0, w, 1);

		if (startTime == startMovieTime) {
			isValid = true;
		}

		return isValid;
	}

	private GlossChainedEventsEntity getGlossEntity(EventsEntity hsEntity, GlossChainedEventsEntityCollection glsColl) {

		int hsStartTime = hsEntity.getStartTimeInfo().getMovieTime();
		int hsEndTime = hsEntity.getEndTimeInfo().getMovieTime();

		Line2D domLine = new Line2D.Double(hsStartTime, 0, hsEndTime, 0);

		ArrayList<GlossChainedEventsEntity> glosses = glsColl.getEntities();

		GlossChainedEventsEntity entity = null;

		ArrayList<GlossChainedEventsEntity> entitiesToDelete = new ArrayList<GlossChainedEventsEntity>();
		for (GlossChainedEventsEntity gloss : glosses) {
			if (gloss == null || gloss.getStartTimeInfo() == null || gloss.getEndTimeInfo() == null) {
				entitiesToDelete.add(gloss);
			}
		}

		for (GlossChainedEventsEntity toDelete : entitiesToDelete) {
			glsColl.deleteEntity(toDelete);
		}

		for (GlossChainedEventsEntity gloss : glosses) {

			int startMovieTime = gloss.getStartTimeInfo().getMovieTime();
			int endMovieTime = gloss.getEndTimeInfo().getMovieTime();
			int w = endMovieTime - startMovieTime;

			if (domLine.intersects(startMovieTime, 0, w, 1)) {
				entity = gloss;
			}
		}
		return entity;
	}

	private GlossChainedEventsEntity getGlossEntity(ChainedEventsEntity handEntity, GlossFieldWrapper glsFieldWrapper) {

		if (handEntity == null)
			return null;

		GlossChainedEventsEntityCollection coll = glsFieldWrapper.getGlossField()
				.getGlossChainedEventsEntityCollection();

		GlossChainedEventsEntity entity = coll.getEntity(handEntity.getRefEntityId());

		// first try to get the gloss entity based on the hand entity ref entity id
		if (entity == null)
			entity = coll.getEntity(handEntity.getRefEntityId());

		// if it was not found by ref entity id, try next by coordinate
		if (entity == null) {
			EventsEntity ent = glsFieldWrapper.getGlossField()
					.getEntityAt(handEntity.getValueOrTextValueEvent().getStartTimeCoordinate());

			if (ent != null)
				entity = (GlossChainedEventsEntity) ent;
		}

		// if still not found, call getGlossEntity
		if (entity == null) {
			entity = getGlossEntity(handEntity, coll);
		}

		if (isValid(handEntity, entity))
			return entity;

		return null;
	}

	private void setDependencies(GlossChainedEventsEntity glsEntity, ChainedEventsEntity handEntity,
								 HandShapeEntity handShapeEntity) {

		glsEntity.setDependentHandEntity(handEntity);
		glsEntity.setDependentHandShapeEntity(handShapeEntity);

		// check if gloss and hand entities are displayed correctly
		Event value = glsEntity.getValueOrTextValueEvent();
		ChainedEvent initialHold = glsEntity.getInitialHold();
		ChainedEvent finalHold = glsEntity.getFinalHold();

		Event hndValue = handEntity.getValueOrTextValueEvent();

		ChainedEvent hndText = (ChainedEvent) handEntity.getValueOrTextValueEvent();
		hndText.setText(glsEntity.getText());

		int valueStartTime = value.getStartTimeInfo().getMovieTime();
		int valueEndTime = value.getEndTimeInfo().getMovieTime();
		int initialHoldStartTime, finalHoldEndTime;

		// if initial hold exists, then set the entity in the RightLeftHandFieldWrapper
		// to its start time
		if (initialHold != null) {
			initialHoldStartTime = initialHold.getStartTimeInfo().getMovieTime();
			hndValue.getStartTimeInfo().setMovieTime(initialHoldStartTime);
			handEntity.setInitialHoldEvent(initialHold.cloneChainedEvent());
		} else {
			hndValue.getStartTimeInfo().setMovieTime(valueStartTime);
		}

		// if final hold exists, then set the entity in the RightLeftHandFieldWrapper to
		// its end time
		if (finalHold != null) {
			finalHoldEndTime = finalHold.getEndTimeInfo().getMovieTime();
			hndValue.getEndTimeInfo().setMovieTime(finalHoldEndTime);
			handEntity.setFinalHoldEvent(finalHold.cloneChainedEvent());
		} else {
			hndValue.getEndTimeInfo().setMovieTime(valueEndTime);
		}

		glsEntity.setLockedGlossPhonAlignments(true);
		handEntity.setLockedGlossPhonAlignments(true);
		handShapeEntity.setLockedGlossPhonAlignments(true);
	}
}
