package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class DatabaseCollectionRepresentation extends
      SegmentGraphicRepresentation {

   private static final long serialVersionUID = 1L;
   private final SignStreamSegmentPanel segmentPanel;
   public int SIZE = 22;

   public DatabaseCollectionRepresentation(SignStreamSegmentPanel segmentPanel, Segment segment) {
      super(segmentPanel, segment);
      setOpaque(false);
      this.segmentPanel = segmentPanel;
   }

   @Override
public Dimension getPreferredDimension() {
	   ArrayList<FieldWrapper> wrappers = new ArrayList<FieldWrapper>();
	   ArrayList<Segment> segments = segmentPanel.getSegments();
	   
	   for(Segment segment:segments) {
		   wrappers.addAll(segment.getFieldWrappersCollection().getWrappers());
	   }

      double X = 0;
      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrapper = wrappers.get(i);
         int e = wrapper.getField().getEndTimeCoordinate();
         X = X > e ? X : e;
      }

      double Y = wrappers.size() * (SegmentGraphReprUtil.cellHeight + 1);
      double X1 = getPreferredSize().getWidth();
      double Y1 = getPreferredSize().getHeight();

      X = X > X1 ? X : X1;
      Y = Y > Y1 ? Y : Y1;

      return new Dimension((int) X, (int) Y);
   }

   @Override
public void paint(Graphics g) {
      Graphics2D g2d = (Graphics2D) g;

      g2d.setColor(Palette.BACKGROUND_COLOR);
      g2d.fillRect(0, 0, getWidth(), getHeight());

      // draw axis
      g2d.setColor(Palette.AXIS_COLOR);
      g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.axesBottomHeight
            + SegmentGraphReprUtil.linenHeight, getWidth(),
            SegmentGraphReprUtil.axesBottomHeight
                  + SegmentGraphReprUtil.linenHeight));
      
	   ArrayList<FieldWrapper> wrappers = new ArrayList<FieldWrapper>();
	   ArrayList<Segment> segments = segmentPanel.getSegments();
	   
	   for(Segment segment:segments) {
		   wrappers.addAll(segment.getFieldWrappersCollection().getWrappers());
	   }


      int i = 0;
      for (i = 1; i < wrappers.size() + 1; i++) {
         g2d.setColor(Palette.AXIS_COLOR);
         g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.cellHeight * i
               + SegmentGraphReprUtil.axesTopHeight
               + SegmentGraphReprUtil.linenHeight, getWidth(),
               SegmentGraphReprUtil.cellHeight * i
                     + SegmentGraphReprUtil.axesTopHeight
                     + SegmentGraphReprUtil.linenHeight));
         g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.cellHeight * i
               + SegmentGraphReprUtil.axesBottomHeight
               + SegmentGraphReprUtil.linenHeight, getWidth(),
               SegmentGraphReprUtil.cellHeight * i
                     + SegmentGraphReprUtil.axesBottomHeight
                     + SegmentGraphReprUtil.linenHeight));

         double Y = SegmentGraphReprUtil.cellHeight * i
               + SegmentGraphReprUtil.valueHeight
               + SegmentGraphReprUtil.linenHeight;
         FieldWrapper wrapper = wrappers.get(i - 1);
         wrapper.getField().paint(g2d, Y,
               SegmentGraphReprUtil.getGraphReprFont());
      }

      g2d.draw(new Line2D.Double(0, SegmentGraphReprUtil.cellHeight * i
            + SegmentGraphReprUtil.axesTopHeight
            + SegmentGraphReprUtil.linenHeight, getWidth(),
            SegmentGraphReprUtil.cellHeight * i
                  + SegmentGraphReprUtil.axesTopHeight
                  + SegmentGraphReprUtil.linenHeight));

      double Y = SegmentGraphReprUtil.cellHeight * i
            + SegmentGraphReprUtil.valueHeight
            + SegmentGraphReprUtil.linenHeight;
      FieldWrapper wrapper = wrappers.get(i - 1);
      wrapper.getField().paint(g2d, Y, SegmentGraphReprUtil.getGraphReprFont());
   }

   @Override
public String toString() {
      StringBuffer bf = new StringBuffer("SegmentGraphicRepresentation: ");

	   ArrayList<FieldWrapper> wrappers = new ArrayList<FieldWrapper>();
	   ArrayList<Segment> segments = segmentPanel.getSegments();
	   
	   for(Segment segment:segments) {
		   wrappers.addAll(segment.getFieldWrappersCollection().getWrappers());
	   }

      bf.append("size = ");
      bf.append(wrappers.size());
      bf.append("\n");

      for (int i = 0; i < wrappers.size(); i++) {
         FieldWrapper wrp = wrappers.get(i);
         bf.append("\n name = ");
         bf.append(wrp.getField().getName());
         bf.append(", fieldID = ");
         bf.append(wrp.getField().getFieldID());
      }

      return bf.toString();
   }
}
