package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.logging.Level;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.listener.SegmentMouseListener;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.video.MultipleMovieController;

public class DatabaseView extends JPanel {

	private static final long serialVersionUID = 1L;

	protected LinenView linenView;
	protected LabelViewTemporalPartitions labelView;
	private UpperLeftCorner upperLeft = null;
	protected JScrollPane pictureScrollPane;
	protected JSplitPane splitPane;
	
	private SegmentGraphicRepresentation rep = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	private int length = 350;
	public static final int screenBufferAmount = 50;
	
	/** Controls how much the database view scrolls when dragging a new utterance outside the view */
	private int rectScrollGranularity = 50;

	public DatabaseView(Segment segment, SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;

		setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));

		rep = new SegmentGraphicRepresentation(segmentPanel, segment);

		try {
			MultipleMovieController contr = SS3Singleton
					.getVideoControlManager().getMultipleMovieController();
			int movieDuration = contr.getMovieDuration();
			int l = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertTimeToCoordinate(movieDuration);

			if (l > length)
				length = l + 20;

		} catch (NullPointerException ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}

		rep.setPreferredSize(new Dimension(length, (int) rep
				.getPreferredDimension().getHeight()));

		SegmentMouseListener listener = new SegmentMouseListener(segmentPanel);
		rep.addMouseListener(listener);
		rep.addMouseMotionListener(listener);

		linenView = new LinenView(LinenView.HORIZONTAL, true, length,
				segmentPanel);
		labelView = new LabelViewTemporalPartitions(segment, this);

		linenView.setPreferredWidth(length);
		labelView.setPreferredSize(new Dimension(125, rep
				.getPreferredDimension().height));

		pictureScrollPane = new JScrollPane(rep,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		pictureScrollPane.setPreferredSize(new Dimension(300, 250));
		pictureScrollPane.setViewportBorder(BorderFactory
				.createLineBorder(Color.black));

		pictureScrollPane.setColumnHeaderView(linenView);
		pictureScrollPane.setCorner(ScrollPaneConstants.UPPER_RIGHT_CORNER,
				new UpperRightCorner());

		splitPane = new JSplitPane();
		
		JPanel left = new JPanel();
		left.setLayout(new BoxLayout(left, BoxLayout.Y_AXIS));
		upperLeft = new UpperLeftCorner(segmentPanel);
		upperLeft.setMinimumSize(new Dimension(100,linenView.getPreferredSize().height));
		upperLeft.setMaximumSize(new Dimension(Integer.MAX_VALUE,linenView.getPreferredSize().height));
		left.add(upperLeft);
		left.add(labelView);
		
		splitPane.setLeftComponent(left);
		splitPane.setRightComponent(pictureScrollPane);
		splitPane.setPreferredSize(new Dimension(300, 250));
		
		add(splitPane);
	}

	public void scrollRectToVisible(Point point) {
		JViewport vp = pictureScrollPane.getViewport();
		vp.setViewPosition(point);
	}
	
	/**
	 * Scrolls the database view forward an incremental amount. Meant to provide more granularity
	 * when scrolling than the scrollRectToVisible(Point) method.
	 */
	public void scrollRectForward(){
		JViewport vp = pictureScrollPane.getViewport();
		Point temp = new Point(vp.getViewPosition().x + vp.getExtentSize().width/rectScrollGranularity, 0);
		vp.setViewPosition(temp);
	}
	
	/**
	 * Scrolls the database view backward an incremental amount. Meant to provide more granularity
	 * when scrolling than the scrollRectToVisible(Point) method.
	 */
	public void scrollRectBackward(){
		JViewport vp = pictureScrollPane.getViewport();
		Point temp = new Point(vp.getViewPosition().x - vp.getExtentSize().width/rectScrollGranularity, 0);
		if (temp.x < 0)
			temp.x = 0;
		vp.setViewPosition(temp);
	}
	
	@Override
	public int getWidth() {
		return splitPane.getWidth();
	}
	
	public int getLength() {
		return length;
	}

	public int getVisibleWidth2() {
		if (pictureScrollPane == null) {
			return -1;
		} else {
			JViewport vp = pictureScrollPane.getViewport();
			Dimension d = vp.getExtentSize();
			return d.width;
		}
	}

	public LabelViewTemporalPartitions getLabelView() {
		return labelView;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRepresentation() {
		return rep;
	}

	public Point getVisiblePoint() {
		JViewport vp = pictureScrollPane.getViewport();
		return vp.getViewPosition();
	}

	/**
	 * Keeps the new utterance visible on the screen while it is being created.
	 * @param coord - The coordinate of the mouse while it is being dragged. 
	 */
	public void keepUtteranceVisible(int coord) {
		
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int rectLeftCorner = viewPosition.x;
			int rectRightCorner = rectLeftCorner + visibleWidth;

			if (rectLeftCorner < coord && coord < rectRightCorner) {
				return;
			} else {
				if (coord >= rectRightCorner)
					scrollRectForward();
				else
					scrollRectBackward();
			}

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}
	

	public void setVisible(int startCoord, int endCoord) {
		Point scrollTo = null;

		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int rectLeftCorner = viewPosition.x;
			int rectRightCorner = rectLeftCorner + visibleWidth;

			if (rectLeftCorner < startCoord && rectRightCorner > endCoord) {
				return;
			} else {
				int eventWidth = endCoord - startCoord;
				if (eventWidth < visibleWidth) {
					int dWidth = visibleWidth - eventWidth;
					int X3 = startCoord - dWidth / 2;

					if (X3 < 0)
						X3 = 0;

					scrollTo = new Point(X3, 0);
				} else {
					scrollTo = new Point(startCoord, 0);
				}
			}

			scrollRectToVisible(scrollTo);

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}


	/**
	 * updatePanels() function updates the local LabelViewTemporalPartitions
	 * and SegmentGraphicRepresentation panels.
	 * Both require that the Segment object be updated prior to calling this function.
	 */
	public void updatePanels() {
		labelView.updateTree();
		rep.update();
		repaint();
	}

	public synchronized void zoom(double zoomValue) {
		Point scrollTo = null;

		int sliderVisibleCoord = getDistanceToCoord(segmentPanel.getSlider()
				.getSliderPosition());
		Event selectedEvent = segmentPanel.getSelectedEvent();

		if (selectedEvent != null) {
			int eventCoord = selectedEvent.getStartTimeCoordinate();
			int eventVisibleCoord = getDistanceToCoord(eventCoord);
			int movieTime = SS3Singleton.getVideoControlManager()
					.getMultipleMovieController().getCurrentTime();
			segmentPanel.getSlider().setSliderPositionTime(movieTime);

			int X2 = eventCoord - eventVisibleCoord;

			if (X2 < 0)
				X2 = 0;

			scrollTo = new Point(X2, 0);
		} else if (sliderVisibleCoord > -1) {

			segmentPanel.getZoomer().zoom(zoomValue);
			segmentPanel.repaint();

			int movieTime = SS3Singleton.getVideoControlManager()
					.getMultipleMovieController().getCurrentTime();
			segmentPanel.getSlider().setSliderPositionTime(movieTime);
			int newSliderPosition = segmentPanel.getSlider()
					.getSliderPosition();
			int X = newSliderPosition - sliderVisibleCoord;

			if (X < 0)
				X = 0;

			scrollTo = new Point(X, 0);
		} else {
			EventsEntity selectedEntity = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();		
			Event parentEvent = selectedEntity.getParentEntity().getEvent();

			if (parentEvent != null) {
				setVisible(parentEvent.getStartTimeCoordinate(),
						parentEvent.getEndTimeCoordinate());
			} else {
				int midPointTime = getVisibleMiddlePointTime();
				setMidPointTimeVisible(midPointTime);
			}
		}
		
		updateDrawingsAreaSize();

		scrollRectToVisible(scrollTo);

		segmentPanel.repaint();
		segmentPanel.updateUI();
	}

	public void updateDrawingsAreaSize() {
		int duration = segmentPanel.getMultipleMovieController()
				.getMovieDuration();

		int sw = 0;
		if (SegmentGraphReprUtil.utteranceMovieTimeOnset > 0
				&& SegmentGraphReprUtil.utteranceMovieTimeOffset > 0) {
			sw = segmentPanel
					.getZoomer()
					.getCoordinateTimeConvertor()
					.convertTimeToCoordinate(
							SegmentGraphReprUtil.utteranceMovieTimeOffset);
		} else {
			sw = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertTimeToCoordinate(duration);
		}
		int screenWidth = sw + DatabaseView.screenBufferAmount;

		Dimension d = rep.getPreferredSize();
		Dimension dd = new Dimension(screenWidth, d.height);
		rep.setPreferredSize(dd);
		rep.revalidate();

		if (linenView != null) {
			linenView.setPreferredWidth(screenWidth);
			linenView.revalidate();
		}
	}

	public void setMidPointTimeVisible(int time) {

		int midCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(time);
		Rectangle visibleRect = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();

			int halfVisibleWidth = visibleRect.width / 2;
			int X1 = midCoord - halfVisibleWidth;

			if (X1 < 0)
				X1 = 0;

			Point scrollTo = new Point(X1, 0);
			scrollRectToVisible(scrollTo);

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	public int getVisibleMiddlePointTime() {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X = X1 + visibleWidth / 2;

			int time = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertCoordinateToTime(X);

			return time;

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}

		return -1;
	}

	int distance = 0;

	private int getDistanceToCoord(int coord) {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			JViewport viewport = pictureScrollPane.getViewport();
			visibleRect = viewport.getVisibleRect();
			viewPosition = viewport.getViewPosition();

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X2 = X1 + visibleWidth;

			if (X1 < coord && X2 > coord) {
				distance = coord - X1;
			}
		} catch (Exception ex) {
			return distance;
		}
		return distance;
	}

	public void unselectAll() {
		FieldWrappersCollection masterWrappersCollection = rep
				.getFieldWrappersCollection();
		masterWrappersCollection.unselectAll();
	}

	public UpperLeftCorner getUpperLeft() {
		return upperLeft;
	}
}
