package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.BoundedRangeModel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.grepresentation.fields.Segment;

public class GlossFieldsLabelView extends JPanel {

	private static final long serialVersionUID = 1L;

	private Segment segment;
	private LabelObject glossRelatedObj = null;
	private GlossNRelatedView glossFieldView = null;

	private LabelViewGlossNRelatedFields glossRelated = null;

	private UtteranceView view = null;
	
	protected JScrollPane pictureScrollPane;

	public GlossFieldsLabelView(Segment segment, UtteranceView sView,
			GlossNRelatedView view2) {
		this.segment = segment;
		this.view = sView;
		this.glossFieldView = view2;
		
		glossRelatedObj = segment.getRootLabelObject().getGlossNRelatedLabelObject();
		
		ArrayList<LabelObject> objs = glossRelatedObj.getChildrenAsList();
		for (LabelObject obj : objs) {
			obj.setVisible(true);
			for (LabelObject child: obj.getAllChildren()) {
				child.setVisible(true);
			}
		}

		initUI();
	}
	
	public void update(Segment segment) {
		this.segment = segment;
		glossRelatedObj = segment.getRootLabelObject().getGlossNRelatedLabelObject();
		
//		ArrayList<LabelObject> objs = glossRelatedObj.getChildrenAsList();
//		for (LabelObject obj : objs) {
//			obj.setVisible(true);
//		}

		initUI();
	}

	public void showSelectedGlossFields(ArrayList<String> slctFieldIDs) {
		ArrayList<LabelObject> objs = glossRelatedObj.getChildrenAsList();
		for (LabelObject obj : objs) {
			
			if (slctFieldIDs.contains(obj.getFieldID())) {
				obj.setVisible(true);
			} else {
				obj.setVisible(false);
			}
		}
	}

	public void initUI() {
		removeAll();

		JTree oldTree = null;

		if (glossRelated != null)
			oldTree = glossRelated.getTree();

		glossRelatedObj = segment.getRootLabelObject()
				.getGlossNRelatedLabelObject();

		glossRelated = new LabelViewGlossNRelatedFields(glossRelatedObj,
				segment, view, glossFieldView);
		
		pictureScrollPane = new JScrollPane(glossRelated,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		JPanel p = new JPanel();
		p.setBackground(UIPalette.getBackground());
		p.setPreferredSize(new Dimension(getWidth(),
				SegmentGraphReprUtil.axesBottomHeight));

		setLayout(new BorderLayout(0, 1));
		add(pictureScrollPane, BorderLayout.CENTER);

		JTree newTree = glossRelated.getTree();

		if (oldTree != null) {
			for (int i = 0; i < oldTree.getRowCount(); i++) {
				TreePath path = oldTree.getPathForRow(i);

				if (oldTree.isExpanded(path)) {
					Object obj = path.getLastPathComponent();
					String label = obj.toString().trim();
					expandTree(newTree, label);
				}
			}
		} else {
			for (int j = 0; j < newTree.getRowCount(); j++) {
				TreePath newPath = newTree.getPathForRow(j);
				Object newObj = newPath.getLastPathComponent();
				String label = newObj.toString().trim();
				
//				if(label.equalsIgnoreCase("right hand") || label.equalsIgnoreCase("left hand") ) {
				expandTree(newTree, label);
//				}
			}
		}

		ArrayList<LabelObject> visibleObj = glossRelatedObj.getVisibleObjects();

		for (int j = 0; j < newTree.getRowCount(); j++) {
			TreePath newPath = newTree.getPathForRow(j);
			Object newObj = newPath.getLastPathComponent();
			String label = newObj.toString().trim();
			
			if (isVisibleObject(label, visibleObj)) {

				if (!label.equals("rhshape") && !label.equals("lhshape")) {
					;
				}
					//expandTreeLeaf(newTree, label);
			}
		}
	}

	public GlossNRelatedView getGlossFieldView() {
		return glossFieldView;
	}

	public void newUpdate(LabelObject obj) {
		glossRelated.showNode(obj);
	}

	@Override
	public void repaint() {
		if (glossRelated != null)
			glossRelated.update();
	}

	public void setPreferredHeight(int ph) {
		int size = getPreferredSize().width;
		setPreferredSize(new Dimension(size, ph));
	}

	public void setPreferredWidth(int pw) {
		int size = getPreferredSize().height;
		setPreferredSize(new Dimension(pw, size));
	}

	public ArrayList<LabelObject> getLabels() {
		return glossRelatedObj.getChildrenAsList();
	}

	public LabelObject getParentLabelObject() {
		return glossRelatedObj;
	}

	public void update() {
		removeAll();

		initUI();
	}

	private boolean isVisibleObject(String label,
			ArrayList<LabelObject> visibleObj) {
		for (LabelObject obj : visibleObj) {
			if (label.equals(obj.getLabel())) {
				return true;
			}
		}
		return false;
	}

	private void expandTreeLeaf(JTree tree, String label) {
		for (int i = 0; i < tree.getRowCount(); i++) {
			TreePath path = tree.getPathForRow(i);
			Object obj = path.getLastPathComponent();
			String treeLabel = obj.toString().trim();
			if (treeLabel.equals(label)) {
				tree.expandPath(path);
				return;
			}
		}
	}

	private void expandTree(JTree tree, String label) {
		for (int i = 0; i < tree.getRowCount(); i++) {
			TreePath path = tree.getPathForRow(i);
			Object obj = path.getLastPathComponent();
			String treeLabel = obj.toString().trim();
			if (treeLabel.equals(label)) {
				tree.expandRow(i);
				return;
			}
		}
	}
	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}
	
	public void setVerticalScrollModel(BoundedRangeModel model) {
		pictureScrollPane.getVerticalScrollBar().setModel(model);
	}
}
