package edu.bu.signstream.grepresentation.view;

import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.fields.locationField.LocationField;

import javax.swing.tree.MutableTreeNode;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class LabelObject {

    private LabelObject parent;

    private ArrayList<LabelObject> children = new ArrayList<LabelObject>();

    private int rowNumber = 0;

    private String fieldID;
    private String label;
    private boolean updatable = false;
    private boolean visible = true;
    private boolean isDefaultSegmentTier = false;

    private FieldWrapper wrapper;

    private MutableTreeNode mutableTreeNode = null;

    public LabelObject() {
    }

    public LabelObject(String fieldID, String label) {
        this.fieldID = fieldID;
        this.label = label;
    }

    public ArrayList<LabelObject> getVisibleObjects() {
        ArrayList<LabelObject> allChildren = getAllChildren();
        ArrayList<LabelObject> visibleObjects = new ArrayList<LabelObject>();

        for (int i = 0; i < allChildren.size(); i++) {
            LabelObject object = allChildren.get(i);

            if (object.isVisible()) {
                visibleObjects.add(object);
                object.getWrapper().setVisible(true);
            }

        }

        return visibleObjects;
    }

    public ArrayList<LabelObject> getNonEmptyObjectsBetween(int startTime, int endTime) {
        ArrayList<LabelObject> allChildren = getAllChildren();
        ArrayList<LabelObject> nonEmptyObjects = new ArrayList<LabelObject>();

        for (int i = 0; i < allChildren.size(); i++) {
            LabelObject object = allChildren.get(i);

            if (!object.getWrapper().getField().getEntities().isEmpty()) {
            	ArrayList<EventsEntity> entities = new ArrayList<EventsEntity>(object.getWrapper().getField().getEntities());
            	// making a copy in case the removal below takes place

            	if (object.getWrapper().getField() instanceof LocationField) {
            		LocationField locField = (LocationField) object.getWrapper().getField();
            		ArrayList<EventsEntity> noneEntities = locField.getRegionNoneEntities(); 
            		entities.removeAll(noneEntities);
            	}

                for (EventsEntity entity : entities) {
                    int entityStartTime = entity.getStartTimeInfo().getMovieTime();
                    int entityEndTime = entity.getEndTimeInfo().getMovieTime();

                    // if any part of entity is visible, then include its field
                    if ((entityStartTime >= startTime && entityStartTime <= endTime)
                            || (entityEndTime >= startTime && entityEndTime <= endTime)) {
                        nonEmptyObjects.add(object);
                        object.setVisible(true);
                        object.getWrapper().setVisible(true);

                        break;
                    }
                }
            }
        }
        return nonEmptyObjects;
    }

    public ArrayList<LabelObject> getNonEmptyObjectsAcrossTimeTuples(Map<Integer, Integer> timeTuples) {
        Set<LabelObject> nonEmptyObjects = new HashSet<LabelObject>();

        for (Map.Entry<Integer, Integer> entry : timeTuples.entrySet()) {
            Integer startTime = entry.getKey();
            Integer endTime = entry.getValue();
            nonEmptyObjects.addAll(getNonEmptyObjectsBetween(startTime, endTime));
        }

        // Return ArrayList to keep method signature consistent
        return new ArrayList<LabelObject>(nonEmptyObjects);
    }

    public String getFieldID() {
        return fieldID;
    }

    public String getLabel() {
        return label;
    }

    public void setFieldID(String string) {
        fieldID = string;
    }

    public void setLabel(String string) {
        label = string;
    }

    @Override
    public String toString() {
        return label;
    }

    public void setVisible(boolean visible) {
        this.visible = visible;
    }

    public boolean isVisible() {
        return visible;
    }

    public void setUpdatable(boolean updatable) {
        this.updatable = updatable;
    }

    public boolean isUpdatable() {
        return updatable;
    }

    public void setDefaultSegmentTier(boolean isDefault) {
        this.isDefaultSegmentTier = isDefault;
    }

    public boolean isDefaultSegmentTier() {
        return isDefaultSegmentTier;
    }

    public void setRowNumber(int rowNumber) {
        this.rowNumber = rowNumber;
    }

    public int getRowNumber() {
        return rowNumber;
    }

    /**
     * Returns the parent of the receiver.
     */
    public LabelObject getParent() {
        return parent;
    }

    /**
     * Returns true if the receiver represents a leaf, that is it is
     * isn't a directory.
     */
    public boolean isLeaf() {

        for (int i = 0; i < children.size(); i++) {
            LabelObject obj = children.get(i);
            if (obj.isVisible() || !obj.isLeaf())
                return false;
        }

        return true; //children.size() == 0;
    }

    /**
     * Loads the children, caching the results in the children
     * instance variable.
     */
    public LabelObject[] getChildren() {
        LabelObject[] obj = new LabelObject[children.size()];
        for (int i = 0; i < children.size(); i++) {
            obj[i] = children.get(i);
        }
        return obj;
    }

    public ArrayList<LabelObject> getAllChildren() {
        ArrayList<LabelObject> list = new ArrayList<LabelObject>();
        for (int i = 0; i < children.size(); i++) {
            LabelObject obj = children.get(i);
            list.add(obj);

            ArrayList<LabelObject> children = obj.getAllChildren();
            if (children != null && !children.isEmpty()) {
                list.addAll(obj.getAllChildren());
            }
        }
        return list;
    }

    // returns all children that would be visible only for a specified participant
    public ArrayList<LabelObject> getAllChildren(SS3Participant participant) {
        ArrayList<LabelObject> children = getAllChildren();
        for (int i = children.size() - 1; i >= 0; i--) {
            FieldWrapper wrapper = children.get(i).getWrapper();
            if (wrapper instanceof GraphFieldWrapper) {
                GraphFieldWrapper gphWrapper = (GraphFieldWrapper) wrapper;
                if (participant != null && !gphWrapper.getGraphFileID().equals(participant.getGraphFileID())) {
                    children.remove(i);
                }
            }
        }
        return children;
    }

    public LabelObject getChildWithFieldID(String fieldID) {
        ArrayList<LabelObject> allChildren = getAllChildren();
        for (LabelObject child : allChildren) {
            if (child.getFieldID().equalsIgnoreCase(fieldID)) {
                return child;
            }
        }
        return null;
    }

    public ArrayList<LabelObject> getChildrenAsList() {
        return children;
    }

    public void addChild(LabelObject node) {
        children.add(node);
    }

    public void removeChild(LabelObject node) {
        children.remove(node);
    }

    /**
     * Returns true if the children have been loaded.
     */
    protected boolean loadedChildren() {
        return (children != null);
    }

    public FieldWrapper getWrapper() {
        return wrapper;
    }

    public void setWrapper(FieldWrapper wrapper) {
        this.wrapper = wrapper;
    }

    public MutableTreeNode getMutableTreeNode() {
        return mutableTreeNode;
    }

    public void setMutableTreeNode(MutableTreeNode node) {
        mutableTreeNode = node;
    }
}
