package edu.bu.signstream.grepresentation.view;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.NonManualFieldOrdering;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelViewIndependentItems.Listener;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

public class LabelViewNonManualFields extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private LabelViewIndependentItems smoother;
	ArrayList<LabelObject> DATA = new ArrayList<LabelObject>();
	JList list;

	private SegmentGraphicRepresentation rep;
	private SegmentControlPanel controlPanel;

	public LabelViewNonManualFields(final ArrayList<LabelObject> data, final SegmentGraphicRepresentation rep,
									final SegmentControlPanel controlPanel) {

		DATA.addAll(data);
		this.rep = rep;
		this.controlPanel = controlPanel;

		sortData();

		list = new JList(new AbstractListModel() {
			@Override
			public int getSize() {
				return DATA.size();
			}

			@Override
			public Object getElementAt(int index) {
				return DATA.get(index);
			}
		});

		list.setBackground(UIPalette.getBackground());

		smoother = new LabelViewIndependentItems(list) {
			@Override
			protected void move(int fromIndex, int toIndex) {
				LabelObject o = DATA.remove(fromIndex);
				DATA.add(toIndex, o);
				list.invalidate();
				recreateList(); // call methos to recreate jlist
				rep.getFieldWrappersCollection().moveField(fromIndex, toIndex);


				SS3Database currentDb = controlPanel.getSignStreamSegmentPanel().getSS3Database();
				// if the current DB has no custom field ordering, set it to the ordering that is currently shown
				// in the on-screen list
				if (currentDb.getCustomNmFieldOrder() == null) {
					currentDb.setCustomNmFieldOrder(new NonManualFieldOrdering());
					for (LabelObject lo : DATA) {
						currentDb.getCustomNmFieldOrder().add(lo.getFieldID());
					}
				} else { // otherwise
					// if the current on-screen list has more fields than the current DB's custom ordering list
					if (DATA.size() > currentDb.getCustomNmFieldOrder().getRawOrderList().size()) {
						ArrayList<String> oldRawList = currentDb.getCustomNmFieldOrder().getRawOrderList();
						HashSet<String> oldSet = new HashSet<String>(oldRawList);
						HashSet<String> newSet = new HashSet<String>();
						for (LabelObject lo : DATA) {
							newSet.add(lo.getFieldID());
						}
						// find all the fields present in the current DB's ordering list
						// but that are not in the new on-screen list
						oldSet.removeAll(newSet);

						// then create a new custom field ordering for the DB and
						// add all the fields in the order they show up on-screen
						currentDb.setCustomNmFieldOrder(new NonManualFieldOrdering());
						for (LabelObject lo : DATA) {
							currentDb.getCustomNmFieldOrder().add(lo.getFieldID());
						}
						// and finally add back in the old fields not present in the new list
						// at the indices they were at before
						for (String oldFieldID : oldSet) {
							currentDb.getCustomNmFieldOrder().add(oldRawList.indexOf(oldFieldID), oldFieldID);
						}
					} else { // the new list size is less than or equal to the size of the current custom DB ordering
						// add all the fields that are not in the DB"s list but in DATA
						for (LabelObject lo : DATA) {
							if (!currentDb.getCustomNmFieldOrder().hasField(lo.getFieldID())) {
								currentDb.getCustomNmFieldOrder().add(lo.getFieldID());
							}
						}
						// remove the field that was just dragged and moved in the on-screen list from the DB's list
						// and add it back in the appropriate position
						currentDb.getCustomNmFieldOrder().remove(o.getFieldID());
						if (toIndex == DATA.size() - 1) {
							currentDb.getCustomNmFieldOrder().add(o.getFieldID()); // added at the end
						} else if (toIndex == 0) {
							currentDb.getCustomNmFieldOrder().add(0, o.getFieldID()); // added the beginning
						} else {
							// get the fields that the field was just dragged in-between (before and after it in the
							// on-screen list), get their positions in the DB's list, then add the field that was just
							// dragged at the given index (calculated by the average of the indices of the before/after
							// fields in the DB's list).
							int prevIndex = currentDb.getCustomNmFieldOrder().getRawOrderList().indexOf(DATA.get(toIndex - 1).getFieldID());
							int nextIndex = currentDb.getCustomNmFieldOrder().getRawOrderList().indexOf(DATA.get(toIndex + 1).getFieldID());
							currentDb.getCustomNmFieldOrder().add((prevIndex + nextIndex) / 2 + 1, o.getFieldID());
						}
					}
				}
				for (LabelObject obj : DATA) {
					obj.getWrapper().setSelectedNHighlighted(false);
				}
				o.getWrapper().setSelectedNHighlighted(true);
			}
		};

		GraphLabelDoubleClickListener graphListener = new GraphLabelDoubleClickListener(this, rep);
		list.addMouseListener(graphListener);

		Listener listener = new Listener(smoother);
		list.addMouseListener(listener);
		list.addMouseMotionListener(listener);
		list.setSelectionBackground(UIPalette.getBackgroundSelection().darker());
		list.getSelectionModel().addListSelectionListener(new ListSelectionHandler(list, controlPanel, rep));
		list.addKeyListener(SS3Singleton.getVideoControlKeyListener());

		recreateList();
		setLayout(new BorderLayout());
		add(list, BorderLayout.CENTER);

		rep.setPreferredSize(rep.getPreferredDimension());
	}

	public void recreateList() {

		list.setCellRenderer(new DefaultListCellRenderer() {

			@Override
			public Component getListCellRendererComponent(JList list, Object value, int index, boolean sel,
					boolean focus) {
				Component c = super.getListCellRendererComponent(list, value, index, sel, focus);

				JLabel label = (JLabel) c;
				label.setPreferredSize(new Dimension(200, SegmentGraphReprUtil.cellHeight));
				label.setBorder(new LineBorder(Palette.AXIS_COLOR));

				UserInterfaceUtil.setLabelLNF(label);

				LabelObject object = (LabelObject) value;
				if (object.isUpdatable()) {
					label.setForeground(Color.RED);
				} else {
					label.setForeground(Color.black);
				}
				FieldWrapper wrapper = object.getWrapper();
				if (wrapper instanceof GraphFieldWrapper) {
					int height =
							SegmentGraphReprUtil.getCorrectCellHeight(((GraphFieldWrapper) wrapper).getGraphField());
					label.setPreferredSize(new Dimension(200, height));
				}

				if ((index % 2) == 0) {
					((JComponent) c).setOpaque(true);
				} else if (c.getBackground().equals(list.getBackground())) {
					((JComponent) c).setOpaque(false);
				}

				return c;
			}
		});
		list.revalidate();
		list.repaint();
		rep.repaint();
		// setLayout(new BorderLayout());
		// add(list, BorderLayout.CENTER);
	}

	public LabelViewIndependentItems getLabelViewIndependentItems() {
		return smoother;
	}

	public void update() {
		sortData();
		recreateList();
	}

	private void sortData() {
		NonManualFieldOrdering order =
				NonManualFieldOrdering.getCorrectFieldOrder(controlPanel.getSignStreamSegmentPanel().getSS3Database());
		NonManualFieldOrdering.sortLabelObjectList(DATA, order);
	}
}

class ListSelectionHandler implements ListSelectionListener {
	private JList list = null;
	private SegmentControlPanel controlPanel = null;
	private SegmentGraphicRepresentation rep = null;

	public ListSelectionHandler(JList list, SegmentControlPanel controlPanel, SegmentGraphicRepresentation rep) {
		this.list = list;
		this.rep = rep;
		this.controlPanel = controlPanel;
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		Object obj = list.getSelectedValue();

		if (e.getValueIsAdjusting() && obj != null) {
			LabelObject labelObj = (LabelObject) obj;

			for (int i = 0; i < list.getModel().getSize(); i++) {
				Object obj1 = list.getModel().getElementAt(i);
				LabelObject labelObj1 = (LabelObject) obj1;
				labelObj1.getWrapper().setSelectedNHighlighted(false);
			}

			FieldWrapper wrapper = labelObj.getWrapper();
			wrapper.setSelectedNHighlighted(true);
			rep.repaint();
		}

		controlPanel.focusUtteranceList();
	}
}

class GraphLabelDoubleClickListener extends MouseAdapter {
	private LabelViewNonManualFields lblView;
	private SegmentGraphicRepresentation rep;

	public GraphLabelDoubleClickListener(LabelViewNonManualFields lblView, SegmentGraphicRepresentation rep) {
		this.rep = rep;
		this.lblView = lblView;
	}

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getClickCount() == 2) {
			LabelObject obj = (LabelObject) lblView.list.getModel().getElementAt(lblView.list.locationToIndex(e.getPoint()));
			if (obj.getWrapper() instanceof GraphFieldWrapper) {
				((GraphFieldWrapper)obj.getWrapper()).getGraphField().toggleMaximized();
				lblView.recreateList();
			}
		}
		rep.setPreferredSize(rep.getPreferredDimension()); // update scrollpane viewport height
	}
}