package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Toolkit;
import java.awt.geom.Line2D;

import javax.swing.JComponent;

import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class LinenView extends JComponent {
	private SignStreamSegmentPanel segmentPanel;

   private int length = 0;
   public int SIZE = 21; //22; // 33
   private int TALL_LINE_HEIGHT = 15;
   private int SMALL_LINE_HEIGHT = 10;
   private int TINY_LINE_HEIGHT = 8;
   
   private int MINUTE_TEXT_HORIZONTAL_OFFSET = 2;
   private int MINUTE_TEXT_VERTICAL_OFFSET = 10;
   
   private int SECOND_TEXT_VERTICAL_OFFSET = 3;
 
   public static final int INCH =
      Toolkit.getDefaultToolkit().getScreenResolution();
   public static final int HORIZONTAL = 0;
   public static final int VERTICAL = 1;

   public int orientation;
   public boolean isMetric;
   private int increment;
   private int units;
   
   private static final int offset = 2;

   public LinenView(int o, boolean m, int length, SignStreamSegmentPanel segmentPanel) {
      this.length = length;
      this.orientation = o;
      this.isMetric = m;
      this.segmentPanel = segmentPanel;
      setIncrementAndUnits();
   }
   
   public void update(int length) {
	   this.length = length;
	   repaint();
   }

   public void setIsMetric(boolean isMetric) {
      //this.isMetric = isMetric;
      setIncrementAndUnits();
      repaint();
   }

   public void setPreferredHeight(int ph) {
      setPreferredSize(new Dimension(SIZE, ph));
   }

   public void setPreferredWidth(int pw) {
      setPreferredSize(new Dimension(pw, SIZE));
   }

   private void setIncrementAndUnits() {
      if (isMetric) {
         units = (int) (INCH / 2.54);
         // dots per centimeter
         increment = units;
      } else {
         units = INCH;
         increment = units / 2;
      }
   }

   public boolean isMetric() {
      return this.isMetric;
   }

   public int getIncrement() {
      return increment;
   }

   @Override
   protected void paintComponent(Graphics g) {
	   
	   	  //Determine Length of component based on length of video on segment panel.
	   	  int movieDuration = SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovieDuration();
	      length = this.segmentPanel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(movieDuration);
	      
	      Graphics2D g2d = (Graphics2D) g;
	      
	      Font largeFont = new Font(g2d.getFont().getName(), Font.PLAIN, 10);
	      Font smallFont = new Font(g2d.getFont().getName(), Font.PLAIN, 8);
	      
	      g2d.setColor(UIPalette.getBackground());
	      g2d.fillRect(0, 0, getWidth(), SIZE);

	      g2d.setColor(UIPalette.getLinenBackgroundColor());
	      g2d.fillRect(0, 0, getWidth(), SIZE);

	      g2d.setColor(UIPalette.getLinenFontColor());
	      g2d.setFont(new Font(g2d.getFont().getName(), Font.PLAIN, 10));
	      g2d.draw(new Line2D.Double(0, SIZE, getWidth(), SIZE));
	      g2d.draw(new Line2D.Double(0, 0, 0, SIZE));

	      int ticksPerMinute = 30;
	      int secondsPerTick = 60 / ticksPerMinute;
	      long durationInMicroSeconds = 0;
	      if (SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovies().size()>0) {
	      		durationInMicroSeconds = (long)(SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovieDurationMs() * 1000.0);
//	    	  	durationInMicroSeconds = SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovies().get(0).getContainer().getDuration();
	      }
	      //1,000,000 microseconds in a second.
	      double durationInSeconds = durationInMicroSeconds / 1000000.0;
	      
	      //Number of ticks to display (Each tick representing 2 seconds).
	      double numTicks = (durationInSeconds / 2);
	      
	      //Determine number of pixels between each tick.
	      double interval;
	      if (durationInSeconds > 0) {
	    	  	interval = length / numTicks;
	      } else {
	    	  	//If no video loaded, don't display any ticks.
	    	  	interval = 0;
	      }
	      for (int i = 1; i < numTicks + 1; i++) {
	    	  	//The X-Coordinate to draw the tick.
	         int X = (int) Math.floor(interval * i) + offset;
	         String currentMinute = ""+ i/ticksPerMinute;
             String currentSecond = ""+(i%ticksPerMinute) * secondsPerTick;
             
	         if(interval <= 1) {
	        	 	if (i % 30 == 0) {
	        	 		if(i % 300 == 0) {
	        	 			g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	        	 			g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);
	         		} else {
	      				g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	        	 		}
	        	 	} 
	         } else if(interval <= 1.1) { 
	        	 	if (i % 30 == 0) {
	        	 		if(i % 60 == 0) {
	            			 g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	            			 g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);
	        	 		} else {
	            			 g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	        	 		}
	            } else if(i % 10 == 0) {
	            	 	g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	    			}
	         } else if(interval <= 2) {
	        	 	if (i % 30 == 0) {        		 
	        	 		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	        	 		g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);        	
	            } else if(i % 5 == 0) {
	            	 	g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	    			}
	         } else if(interval < 3) {
	        	 	if (i % 30 == 0) {
	        	 		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	        	 		g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);
	            } else if(i % 2 == 0) {
	            	 	g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	    			} 
	         } else if(interval < 20) {
	        	 	if (i % 30 == 0) {
	        	 		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	        	 		g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);
	        	 	}  else if (i % 15 == 0) {
	                g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	                g.setFont(smallFont);
	                int width = g.getFontMetrics().stringWidth(currentSecond);
	                g.drawString(currentSecond, X-width/2, SIZE - SMALL_LINE_HEIGHT - SECOND_TEXT_VERTICAL_OFFSET);
	                g.setFont(largeFont);
	            } else {
	            	 g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TINY_LINE_HEIGHT));
	    			} 
	         } else if (interval < 100){
	        	 	if (i % 30 == 0) {
	        	 		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	        	 		g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);
	        	 	}  else if (i % 15 == 0) {
	                g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TINY_LINE_HEIGHT)); 
	                g.setFont(smallFont);
	                int width = g.getFontMetrics().stringWidth(currentSecond);
	                g.drawString(currentSecond, X-width/2, SIZE - TINY_LINE_HEIGHT - SECOND_TEXT_VERTICAL_OFFSET);
	                g.setFont(largeFont);
	            } else if(i % 5 == 0 ){
	            		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TINY_LINE_HEIGHT)); 
	                g.setFont(smallFont);
	                int width = g.getFontMetrics().stringWidth(currentSecond);
	                g.drawString(currentSecond, X-width/2, SIZE - TINY_LINE_HEIGHT - SECOND_TEXT_VERTICAL_OFFSET);
	                g.setFont(largeFont);
	            } else {
	            	 g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	    			} 
	         } else {
	        	 	if (i % 30 == 0) {
	        	 		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TALL_LINE_HEIGHT));
	        	 		g.drawString(currentMinute, X + MINUTE_TEXT_HORIZONTAL_OFFSET, MINUTE_TEXT_VERTICAL_OFFSET);
	        	 	} else if(i % 5 == 0 ){
	        	 		g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - SMALL_LINE_HEIGHT));
	                g.setFont(smallFont);
	                int width = g.getFontMetrics().stringWidth(currentSecond);
	                g.drawString(currentSecond, X-width/2, SIZE - SMALL_LINE_HEIGHT - SECOND_TEXT_VERTICAL_OFFSET);
	                g.setFont(largeFont);
	            } else {
	            	 	g2d.draw(new Line2D.Double(X, SIZE, X, SIZE - TINY_LINE_HEIGHT));
	            	 	g.setFont(smallFont);
	            	 	int width = g.getFontMetrics().stringWidth(currentSecond);
	            	 	g.drawString(currentSecond, X-width/2, SIZE - TINY_LINE_HEIGHT - SECOND_TEXT_VERTICAL_OFFSET); 
	            	 	g.setFont(largeFont);
	    			} 
	         }
	      }
   	} 
}
