package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import edu.bu.signstream.common.files.SS3Resources;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UIPalette;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldDialog;
import edu.bu.signstream.ui.panels.codingScheme.GraphFieldDialog;

public class NonManualFieldsButtonPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private NonManualFieldsLabelView nonManualFieldsLabelView = null;
	private GlossNRelatedView glossNRelatedView;
	private SignStreamSegmentPanel segmentPanel = null;

	private JButton plus, minus, expand, graph, reset;

	public NonManualFieldsButtonPanel(SignStreamSegmentPanel segmentPanel,
			NonManualFieldsLabelView nonManualFieldsLabelView, GlossNRelatedView glossNRelatedView) {
		this.segmentPanel = segmentPanel;
		this.nonManualFieldsLabelView = nonManualFieldsLabelView;
		this.glossNRelatedView = glossNRelatedView;

		setBackground(UIPalette.getBackground());
		setPreferredSize(new Dimension(getWidth(), SegmentGraphReprUtil.axesBottomHeight));
		setLayout(new FlowLayout(FlowLayout.LEFT, 15, 0));

		addIcons();
	}

	/**
	 * This helper method adds the plus, minus, and expand icons
	 * 
	 */
	// TODO: WTF, refactor this...
	private void addIcons() {
		ImageIcon icon;

		String plusFilePath = "plus.gif";
		icon = SS3Resources.icon(plusFilePath);
		if (icon != null) {
			plus = new JButton(icon);
		} else {
			plus = new JButton("+");
			System.out.println("File '" + plusFilePath + "' was not found");
		}
		AddNewFieldListener addNewFieldListener = new AddNewFieldListener(this);
		plus.addActionListener(addNewFieldListener);
		setFieldsButtonsLNF(plus);

		String minusFilePath = "minus.gif";
		icon = SS3Resources.icon(minusFilePath);
		if (icon != null) {
			minus = new JButton(icon);
		} else {
			minus = new JButton("-");
			System.out.println("File '" + minusFilePath + "' was not found");
		}
		HideFieldListener hideFieldListener = new HideFieldListener(this);
		minus.addActionListener(hideFieldListener);
		setFieldsButtonsLNF(minus);

		String expandFilePath = "group.gif";
        icon = SS3Resources.icon(expandFilePath);
		if (icon != null) {
			expand = new JButton(icon);
		} else {
			expand = new JButton("O");
			System.out.println("File '" + expandFilePath + "' was not found");
		}
		ShowAllFieldListener showAllFieldListener = new ShowAllFieldListener(this);
		expand.addActionListener(showAllFieldListener);
		setFieldsButtonsLNF(expand);

		String gphFilePath = "graph.gif";
        icon = SS3Resources.icon(gphFilePath);
		if (icon != null) {
			graph = new JButton(icon);
		} else {
			graph = new JButton("G");
			System.out.println("File '" + gphFilePath + "' was not found");
		}
		GraphFieldListener graphFieldListener = new GraphFieldListener(this);
		graph.addActionListener(graphFieldListener);
		graph.setToolTipText("Graph Field Selector");
		setFieldsButtonsLNF(graph);

		String resetFilePath = "reset.gif";
		icon = SS3Resources.icon(resetFilePath);
		if (icon != null) {
			reset = new JButton(icon);
		} else {
			reset = new JButton("R");
			System.out.println("File '" + resetFilePath + "' was not found");
		}
		ResetFieldOrderListener resetListener = new ResetFieldOrderListener(nonManualFieldsLabelView, segmentPanel);
		reset.addActionListener(resetListener);
		reset.setToolTipText("Reset Field Order");
		setFieldsButtonsLNF(reset);

		add(plus);
		add(minus);
		add(expand);
		add(graph);
		add(reset);
	}

	private static void setFieldsButtonsLNF(JButton btn) {
		Font btnFont = new Font(btn.getFont().getName(), Font.PLAIN, 8);
		FontMetrics btnFontMetrics = btn.getFontMetrics(btnFont);
		int addBtnHeight = btnFontMetrics.getHeight();
		int addBtnWidth = btnFontMetrics.stringWidth(btn.getText());

		btn.setFont(btnFont);

		Dimension addBtnMinSize = new Dimension(addBtnWidth + 15, addBtnHeight);
		btn.setMaximumSize(addBtnMinSize);
		btn.setPreferredSize(addBtnMinSize);
		btn.setMaximumSize(addBtnMinSize);
	}

	public NonManualFieldsLabelView getNonManualFieldsLabelView() {
		return nonManualFieldsLabelView;
	}

	public GlossNRelatedView getGlossNRelatedView() {
		return glossNRelatedView;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	void clickExpandButton() {
		expand.doClick(); // triggers ShowAllFieldListener.actionPerformed()
	}
}

class AddNewFieldListener implements ActionListener {

	private NonManualFieldsButtonPanel buttonPanel = null;
	private CSFieldDialog csFieldDialog = null;

	public AddNewFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		LabelObject selectedFieldLabel = buttonPanel.getNonManualFieldsLabelView().getSelectedField();
		String fieldID = null;

		if (selectedFieldLabel != null)
			fieldID = selectedFieldLabel.getFieldID();

		if (csFieldDialog == null)
			csFieldDialog = (CSFieldDialog) buttonPanel.getNonManualFieldsLabelView().getCSFieldDialog();

		csFieldDialog.setVisible(true, fieldID);
	}
}

class HideFieldListener implements ActionListener {
	private NonManualFieldsButtonPanel buttonPanel = null;
	private CSFieldDialog csFieldDialog = null;
	private GraphFieldDialog graphFieldDialog = null;

	public HideFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		LabelObject selectedFieldLabel = buttonPanel.getNonManualFieldsLabelView().getSelectedField();

		if (selectedFieldLabel == null) {
			selectedFieldLabel = buttonPanel.getGlossNRelatedView().getSelectedField();
			if (selectedFieldLabel == null) {
				return;
			}
		}

		if (csFieldDialog == null)
			csFieldDialog = buttonPanel.getNonManualFieldsLabelView().getCSFieldDialog();
		
		if(graphFieldDialog == null)
			graphFieldDialog =  buttonPanel.getNonManualFieldsLabelView().getGraphDialog();

		// pass in false and set that field to unchecked, and the enter will hide it
		// from view
		csFieldDialog.setSelectedField(selectedFieldLabel.getFieldID(), false);
		csFieldDialog.apply();
		
		graphFieldDialog.setSelectedField(selectedFieldLabel.getFieldID(), false);
		graphFieldDialog.enter();
	}

}

class ShowAllFieldListener implements ActionListener {
	private NonManualFieldsButtonPanel buttonPanel = null;
	private CSFieldDialog csFieldDialog = null;
	private GraphFieldDialog graphFieldDialog = null;

	public ShowAllFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		if (csFieldDialog == null)
			csFieldDialog = buttonPanel.getNonManualFieldsLabelView().getCSFieldDialog();
		
		if(graphFieldDialog == null)
			graphFieldDialog =  buttonPanel.getNonManualFieldsLabelView().getGraphDialog();

		csFieldDialog.getCSFieldPanel().showAllFields();
		
		LabelObject gphLabelObject = graphFieldDialog.getSegment().getRootLabelObject().getGraphLabelObject();
		List<LabelObject> gphFields = gphLabelObject.getAllChildren();
		
		for (LabelObject fieldLabel : gphFields) {
			fieldLabel.getWrapper().setSelectedNHighlighted(false);
			graphFieldDialog.setSelectedField(fieldLabel.getFieldID(), true);
		}
		
		graphFieldDialog.enter();
	}

}

class GraphFieldListener implements ActionListener {
	private NonManualFieldsButtonPanel buttonPanel = null;
	private GraphFieldDialog graphFieldDialog = null;

	public GraphFieldListener(NonManualFieldsButtonPanel parent) {
		this.buttonPanel = parent;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		LabelObject selectedFieldLabel = buttonPanel.getNonManualFieldsLabelView().getSelectedField();
		String fieldID = null;

		if (selectedFieldLabel != null)
			fieldID = selectedFieldLabel.getFieldID();

		if (graphFieldDialog == null)
			graphFieldDialog = buttonPanel.getNonManualFieldsLabelView().getGraphDialog();

		graphFieldDialog.setVisible(true, fieldID);
	}

}

class ResetFieldOrderListener implements ActionListener {

	private NonManualFieldsLabelView lblView;
	private SignStreamSegmentPanel segmentPanel;

	public ResetFieldOrderListener(NonManualFieldsLabelView lblView, SignStreamSegmentPanel segmentPanel) {
		this.lblView = lblView;
		this.segmentPanel = segmentPanel;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		segmentPanel.getSS3Database().setCustomNmFieldOrder(null);
		lblView.updateLabelOrder();
	}
}