package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.BoundedRangeModel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.graphField.GraphFieldWrapper;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.panels.codingScheme.CSFieldDialog;
import edu.bu.signstream.ui.panels.codingScheme.GraphFieldDialog;

public class NonManualFieldsLabelView extends JPanel {

	private static final long serialVersionUID = 1L;

	private LabelViewNonManualFields labelView = null;
	private SegmentGraphicRepresentation rep = null;
	private NonManualFieldsButtonPanel fieldsButtonPanel = null;
	private SegmentControlPanel controlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	private CSFieldDialog csFieldDialog = null;
	private GraphFieldDialog graphFieldDialog = null;

	private ArrayList<LabelObject> labels = null;

	private GlossNRelatedView glossNRelatedView;

	private Segment segment;

	protected JScrollPane pictureScrollPane;

	public NonManualFieldsLabelView(Segment segment, SignStreamSegmentPanel panel, SegmentControlPanel controlPanel,
			GlossNRelatedView glossNRelatedView, CSFieldDialog csFieldDialog, GraphFieldDialog graphFieldDialog) {

		this.segmentPanel = panel;
		this.segment = segment;
		this.controlPanel = controlPanel;
		this.csFieldDialog = csFieldDialog;
		this.graphFieldDialog = graphFieldDialog;
		this.glossNRelatedView = glossNRelatedView;

		rep = new SegmentGraphicRepresentation(panel, segment);

		FieldWrappersCollection wrappers1 = new FieldWrappersCollection();

		labels = segment.getRootLabelObject().getNonManuelLabelObject().getVisibleObjects();

		labels = filterLabelObjs(labels);

		for (int i = 0; i < labels.size(); i++) {
			LabelObject object = labels.get(i);
			wrappers1.addFieldWrapper(object.getWrapper());
		}

		fieldsButtonPanel = new NonManualFieldsButtonPanel(segmentPanel, this, glossNRelatedView);

		rep.setFieldWrappersCollection(wrappers1);
		labelView = new LabelViewNonManualFields(labels, rep, controlPanel);

		pictureScrollPane = new JScrollPane(labelView, ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		removeAll();
		setLayout(new BorderLayout(0, 1));
		add(fieldsButtonPanel, BorderLayout.NORTH);
		add(pictureScrollPane, BorderLayout.CENTER);
	}

	public void update(Segment segment) {
		this.segment = segment;

		rep.update();

		labels = segment.getRootLabelObject().getNonManuelLabelObject().getVisibleObjects();
		
		ArrayList<LabelObject> labelObjs = segment.getRootLabelObject().getGraphLabelObject().getVisibleObjects();
		labelObjs = filterLabelObjs(labelObjs);
		
		labelObjs.addAll(labels);

		labelView = new LabelViewNonManualFields(labelObjs, rep, controlPanel);

		pictureScrollPane.setViewportView(labelView);

		add(pictureScrollPane, BorderLayout.CENTER);
	}

	public void updateLabelOrder() {
		labelView.update();
	}

	public ArrayList<LabelObject> getLabels() {
		return labels;
	}

	public int getLabelsHeight() {
		int totalHeight = 0;

		for (LabelObject obj : labels) {
			int currentWrapperHeight;
			FieldWrapper wrapper = obj.getWrapper();
			if (wrapper instanceof GraphFieldWrapper) {
				currentWrapperHeight =
						SegmentGraphReprUtil.getCorrectCellHeight(((GraphFieldWrapper) wrapper).getGraphField());
			} else {
				currentWrapperHeight = SegmentGraphReprUtil.cellHeight;
			}
			totalHeight += currentWrapperHeight;
		}

		totalHeight += SegmentGraphReprUtil.axesTopHeight; // just to buffer the space at the top (should be enough)

		return totalHeight;
	}

	public SegmentGraphicRepresentation getSegmentGraphicRepresentation() {
		return rep;
	}

	public CSFieldDialog getCSFieldDialog() {
		return csFieldDialog;
	}

	public GraphFieldDialog getGraphDialog() {
		return graphFieldDialog;
	}

	public void setPreferredHeight(int ph) {
		int size = getPreferredSize().width;
		setPreferredSize(new Dimension(size, ph));
	}

	public void setPreferredWidth(int pw) {
		int size = getPreferredSize().height;
		setPreferredSize(new Dimension(pw, size));
	}

	public void loadWrappers() {
		ArrayList<LabelObject> visibleNonManualObjects = segment.getRootLabelObject().getNonManuelLabelObject()
				.getVisibleObjects();
		ArrayList<LabelObject> visibleObjects = segment.getRootLabelObject().getGraphLabelObject().getVisibleObjects();
		visibleObjects = filterLabelObjs(visibleObjects);
		
		visibleObjects.addAll(visibleNonManualObjects);
		FieldWrappersCollection collection = new FieldWrappersCollection();

		for (LabelObject obj : visibleObjects) {
			FieldWrapper fieldWrapper = obj.getWrapper();
			fieldWrapper.setVisible(true);
			collection.addFieldWrapper(fieldWrapper);
		}

		fieldsButtonPanel = new NonManualFieldsButtonPanel(segmentPanel, this, glossNRelatedView);

		rep.setFieldWrappersCollection(collection);
		labelView = new LabelViewNonManualFields(visibleObjects, rep, controlPanel);

		pictureScrollPane.setViewportView(labelView);

		removeAll();
		setLayout(new BorderLayout(0, 1));
		add(fieldsButtonPanel, BorderLayout.NORTH);
		add(pictureScrollPane, BorderLayout.CENTER);

	}

	public LabelObject getSelectedField() {
		LabelObject selectedFieldLabel = (LabelObject) labelView.getLabelViewIndependentItems().getList()
				.getSelectedValue();
		return selectedFieldLabel;
	}

	void clickExpandButton() {
		fieldsButtonPanel.clickExpandButton();
	}

	public SegmentControlPanel getControlPanel() {
		return controlPanel;
	}

	public JScrollPane getJScrollPane() {
		return pictureScrollPane;
	}

	public void setVerticalScrollModel(BoundedRangeModel model) {
		pictureScrollPane.getVerticalScrollBar().setModel(model);
	}

	private ArrayList<LabelObject> filterLabelObjs(ArrayList<LabelObject> labelObjects) {
		ArrayList<LabelObject> filteredLabels = new ArrayList<LabelObject>();

		for (LabelObject object : labelObjects) {
			FieldWrapper wrapper = object.getWrapper();
			if (wrapper instanceof GraphFieldWrapper) {
				GraphFieldWrapper gphWrapper = ((GraphFieldWrapper) wrapper);
				SS3Participant participant = segment.getParentField().getSS3GlossWindowSegment().getParticipant();
				if (gphWrapper.getGraphFileID() != null && participant != null &&
						gphWrapper.getGraphFileID().equals(participant.getGraphFileID())) {
					filteredLabels.add(object);
				}
			} else {
				filteredLabels.add(object);
			}
		}

		return filteredLabels;
	}
}
