package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;
import java.security.Key;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Entity;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Item;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class SegmentControlPanel extends JPanel implements ActionListener, MouseListener {

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private SegmentTemplate segmentTemplate;
	
	private JComboBox<String> partitionComboBox = new JComboBox<String>();
	private String currentPartitionString = null;
	private SS3TemporalPartition currentPartition = null;
	
	private JComboBox<String> segmentComboBox = new JComboBox<String>();
	private String currentSegmentString= null;
	private SS3GlossWindowSegment currentSegment = null;
	
	private JList<String> utteranceList = null;
	private JScrollPane utteranceListScroller = null;
	private UtteranceListSelectionListener utteranceListSelectionListener = null;
	private int selectedUtteranceIndex = 0;
	
	private JButton prevUtteranceBtn = null;
	private JButton nextUtteranceBtn = null;
	
	private JLabel annotatorLabel = new JLabel("  Participant: N/A  ");
	private JButton notesBtn = new JButton(" Notes ");
	
	private JPanel top;
	private JRadioButton radioButton;
	
	private SS3Participant selectedParticipant = null;
	private SS3Participant selectedAnnotator = null;
	
	//public static final String selectedIndexKey = "selectedIndex";

	public static final int PREV_BTN = 1;
	public static final int NEXT_BTN = 2;
	public final int NOTES_BTN = 4;

	// objects to discard
	private JDialog notesDialog = null;
	private NotesPanel notesPanel = null;

	public SegmentControlPanel(final SignStreamSegmentPanel segmentPanel,
							   Segment segment, SegmentTemplate template) {
		
		this.segmentTemplate = template;
		this.segmentPanel = segmentPanel;
		this.segment = segment;

		PresentationField parentField = segment.getParentField();		
		setCurrentPartition(parentField.getTemporalPartition());
		setCurrentSegment(parentField.getSS3GlossWindowSegment());
		setSelectedParticipant(parentField.getSS3GlossWindowSegment().getParticipant());
//		setSelectedAnnotator(parentField.getSS3GlossWindowSegment().getAnnotator());
		
		ArrayList<SS3TemporalPartition> temporalPartitions = segmentPanel.getSS3Database().getSS3GlossWindow().getTemporalPartitions();		
		for (SS3TemporalPartition temporalPartition: temporalPartitions) {
			String partitionLabel = temporalPartition.getLabel();
			partitionComboBox.addItem(partitionLabel);
			
			if (partitionLabel.equals(getCurrentPartitionString())) {
				partitionComboBox.setSelectedItem(partitionLabel);
			}
		}
		TemporalPartitionActionListener partitionActionListener = new TemporalPartitionActionListener(this);
		partitionComboBox.addActionListener(partitionActionListener);
		
		ArrayList<SS3GlossWindowSegment> temporalPartitionSegments = getCurrentPartition().getTemporalPartitionSegments();		
		for (SS3GlossWindowSegment temporalPartitionSegment: temporalPartitionSegments) {
			String segmentLabel = temporalPartitionSegment.getLabel();
			String participantLabel = temporalPartitionSegment.getParticipantLabel();
			String combinedLabel = participantLabel + ":" + segmentLabel;
			
			if (combinedLabel.indexOf(":") == 0)
				combinedLabel = combinedLabel.substring(1);
			
			segmentComboBox.addItem(combinedLabel);
			
			if (combinedLabel.equals(getCurrentSegmentString())) {
				segmentComboBox.setSelectedItem(combinedLabel);
			}
		}
		SegmentActionListener segmentActionListener = new SegmentActionListener(this);
		segmentComboBox.addActionListener(segmentActionListener);

		
		utteranceList = new JList();
		utteranceList.setCellRenderer(new DefaultListCellRenderer() {
			@Override
		     public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		         Component c = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		         if (isSelected) {
		             c.setBackground(Color.LIGHT_GRAY);
		         }
		         if (index == selectedUtteranceIndex) {
		        	 	c.setBackground(Color.BLUE);
		         }
		         return c;
		     }
		});
		utteranceListScroller = new JScrollPane(utteranceList);
		utteranceListScroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		
		utteranceListScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		utteranceList.setDragEnabled(false);
		utteranceList.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		utteranceListSelectionListener = new UtteranceListSelectionListener(this);
		utteranceList.getSelectionModel().addListSelectionListener(utteranceListSelectionListener);

		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0), "focusZoomAndRight");
		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0), "focusZoomAndLeft");
		utteranceList.getActionMap().put("focusZoomAndRight", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut()
						.requestFocus();
				SS3Singleton.getMediaToolBar(null).currentFrameForward();
			}
		});
		utteranceList.getActionMap().put("focusZoomAndLeft", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				segmentPanel.getUtteranceView().getHeader().getTimelineView().getUpperLeftCorner().getZoomInOut()
						.requestFocus();
				SS3Singleton.getMediaToolBar(null).currentFrameBack();
			}
		});
		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0), "moveListUp");
		utteranceList.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0), "moveListDown");
		utteranceList.getActionMap().put("moveListUp", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				incrementSelectedUtterance(-1);
			}
		});
		utteranceList.getActionMap().put("moveListDown", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				incrementSelectedUtterance(1);
			}
		});
		
		
		updateUtteranceList();
		
		String currentParticipant = currentSegment.getParticipantLabel();
		annotatorLabel.setText(currentParticipant.isEmpty() ? "  Participant: N/A  " : "Participant: " + currentParticipant);
		
		prevUtteranceBtn = new JButton("< Prev Utterance" );
		nextUtteranceBtn = new JButton(" Next Utterance >");
		
		prevUtteranceBtn.setActionCommand("" + PREV_BTN);
		nextUtteranceBtn.setActionCommand("" + NEXT_BTN);
		
		prevUtteranceBtn.addActionListener(new PrevNextUtteranceListener(this));
		nextUtteranceBtn.addActionListener(new PrevNextUtteranceListener(this));
		
		notesBtn.setActionCommand("" + NOTES_BTN);
		notesBtn.addActionListener(this);
		
		radioButton = new JRadioButton();
		radioButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				segmentTemplate.selectTemplate();
			}
			
		});

		UserInterfaceUtil.setComboBoxLNF(partitionComboBox);
		partitionComboBox.setPreferredSize(null);
		partitionComboBox.setMaximumSize(null);
		UserInterfaceUtil.setComboBoxLNF(segmentComboBox);
		segmentComboBox.setPreferredSize(null);
		segmentComboBox.setMaximumSize(null);
		UserInterfaceUtil.setListLNF(utteranceList);
		UserInterfaceUtil.setLabelLNF(annotatorLabel);
		UserInterfaceUtil.setButtonsLNF(notesBtn);
		UserInterfaceUtil.setButtonsLNF(prevUtteranceBtn);
		UserInterfaceUtil.setButtonsLNF(nextUtteranceBtn);

		annotatorLabel.setAlignmentX(Component.LEFT_ALIGNMENT);
		notesBtn.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		top = new JPanel();
		
		addMouseListener(this);
		top.addMouseListener(this);
		
		setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.BOTH;
		c.gridx = 0;
		c.weightx = 1.0;
		c.gridwidth = 2;
		
		top.setLayout(new GridBagLayout());
		c.gridy = 0;
		top.add(partitionComboBox, c);
		
		c.gridy = 1;
		top.add(segmentComboBox, c);
		
		c.gridy = 0;
		c.gridx = 2;
		top.add(radioButton, c);
		
		c.gridy = 0;
		c.gridx = 0;
		add(top,c);
		
		c.gridy = 2;
		c.weighty = 1.0;
		add(utteranceListScroller, c);
		
		c.weighty = 0.1;
		c.gridwidth = 1;
		
		c.gridy = 3;
		add(prevUtteranceBtn, c);
		
		c.gridx = 1;
		add(nextUtteranceBtn, c);
		
		c.gridx = 0;
		c.gridy = 4;
		add(annotatorLabel, c);
		
		c.gridx = 1;
		add(notesBtn, c);

	}

	public void update(Segment segment) {
		this.segment = segment;
	}
	
	public void updateUtteranceList() {
		SS3GlossWindowSegment currentSegment = getCurrentSegment();
		ArrayList<SS3GlossWindowItem> items = currentSegment.getSS3GlossWindowItems();
		ArrayList<String> utterances = new ArrayList<String>();
		
		for (int i = 0; i < items.size(); i++) {
			SS3GlossWindowItem item = items.get(i);
			String combinedTextLabels = "  ";
			
			if (item.getSS3StatementField(Util.DOM_GLOSS_FIELD_ID) != null) {
				// entities are in order by startTime
				ArrayList<SS3Entity> ss3Entities = item.getSS3StatementField(Util.DOM_GLOSS_FIELD_ID).getSS3Entities();
				
				for (SS3Entity ss3Entity: ss3Entities) {
					ArrayList<SS3Item> entItems = ss3Entity.getItems();
					for (SS3Item entItem: entItems) {
						// if entItem.isTextOnly() gave back the correct/up-to-date result, then it would be used here instead
						String text = entItem.getText();
						if (text.equals("Onset") || text.equals("Offset")
								|| text.equals("Initial Hold") || text.equals("Final Hold"))
							continue;
						
						combinedTextLabels += entItem.getText() + " ";
					}
				}
			}
			
			String utteranceLabel = item.getLabel();
			utterances.add(utteranceLabel + combinedTextLabels);
		}
		
		utteranceList.setListData(Arrays.copyOf(utterances.toArray(), utterances.size(), String[].class));
	}
	
	public void updateUtteranceListSelectedItems(ArrayList<PresentationEventsEntity> utterances) {

		ArrayList<String> utteranceLabels = new ArrayList<String>();
		for (PresentationEventsEntity utterance: utterances) {
			utteranceLabels.add(utterance.getEvent().getSS3GlossWindowItem().getLabel());
		}
		
		ArrayList<Integer> indiciesList = new ArrayList<Integer>();
		
		ListModel<String> listModel = utteranceList.getModel();
		int size = listModel.getSize();
		
		for (int i = 0; i < size; i++) {
			String elementLabelWithText = listModel.getElementAt(i);
			
			int indexOfSpace = elementLabelWithText.indexOf(" ");
			String elementLabel = elementLabelWithText.substring(0, indexOfSpace);
			
			for (String utteranceLabel: utteranceLabels) {
				if (elementLabel.equals(utteranceLabel)) {
					indiciesList.add(i);
				}
			}
		}
		
		// code necessary to create an int array from an Integer arraylist
		int indiciesListSize = indiciesList.size();
		// indiciesArray must be final to be used in inner Runnable class below
		final int[] indiciesArray = new int[indiciesListSize];
		for (int i = 0; i < indiciesListSize; i++) {
			indiciesArray[i] = indiciesList.get(i);
		}

		utteranceList.clearSelection();
		
		if (indiciesListSize > 0) {
			SwingUtilities.invokeLater(new Runnable() {
				@Override
				public void run() {
					
					// first remove the listener from the utteranceList to avoid an infinite action firing
					utteranceList.getSelectionModel().removeListSelectionListener(utteranceListSelectionListener);
					
					utteranceList.setSelectedIndices(indiciesArray);

					// then re-add the listener after we have manually set the indicies
					utteranceList.getSelectionModel().addListSelectionListener(utteranceListSelectionListener);
					
					utteranceList.ensureIndexIsVisible(indiciesArray[0]);
				}
			});
		}
	}
	
	public PresentationEventsEntity getSelectedUtterance() {
		if (getSegment().getParentField().getEntities().size() == 0) {
			return null;
		}
		return (PresentationEventsEntity) this.getSegment().getParentField().getEntities().get(selectedUtteranceIndex);
	}
	
	public boolean setSelectedUtterance(EventsEntity utterance) {
		ArrayList<PresentationEventsEntity> utterances = getSegment().getParentField().getEntities();
		for (int i=0 ; i < utterances.size(); i++) {
			if (utterances.get(i) == utterance) {
				selectedUtteranceIndex = i;
				selectAndScrollToUtteranceByIndex(i);
				return true;
			}
		}
		return false;
	}
	
	public boolean setSelectedUtterance(String id) {
		ArrayList<PresentationEventsEntity> utterances = getSegment().getParentField().getEntities();
		for (int i=0 ; i < utterances.size(); i++) {
			if (utterances.get(i).getEvent().getID().equals(id)) {
				selectedUtteranceIndex = i;
				selectAndScrollToUtteranceByIndex(i);
				return true;
			}
		}
		return false;
	}
	
	public boolean incrementSelectedUtterance(int offset) {
		int newIndex = selectedUtteranceIndex + offset;
		int listSize = getUtteranceList().getModel().getSize();
		if (newIndex >= listSize || newIndex < 0) {
			return false;
		}
		selectedUtteranceIndex = newIndex;
		
		selectAndScrollToUtteranceByIndex(selectedUtteranceIndex);
		return true;
	}
	
	public JComboBox<String> getTemporalPartitionComboBox() {
		return partitionComboBox;
	}
	
	public String getCurrentPartitionString() {
		return currentPartitionString;
	}
	
	public void setCurrentPartitionString(String currentPartitionString) {
		this.currentPartitionString = currentPartitionString;
	}
	
	public SS3TemporalPartition getCurrentPartition() {
		return currentPartition;
	}
	
	public void setCurrentPartition(SS3TemporalPartition currentPartition) {
		this.currentPartition = currentPartition;
		
		if (currentPartition != null)
			setCurrentPartitionString(currentPartition.getLabel());
		else
			setCurrentPartitionString(null);
	}
	
	public JComboBox<String> getSegmentComboBox() {
		return segmentComboBox;
	}
	
	public String getCurrentSegmentString() {
		return currentSegmentString;
	}
	
	public void setCurrentSegmentString(String currentSegmentString) {
		this.currentSegmentString = currentSegmentString;
	}
	
	public SS3GlossWindowSegment getCurrentSegment() {
		return this.currentSegment;
	}
	
	public void setCurrentSegment(SS3GlossWindowSegment currentSegment) {
		this.currentSegment = currentSegment;	
		
		if (currentSegment != null) {
			String currentParticipantLabel = currentSegment.getParticipantLabel();
			String currentSegmentLabel = currentSegment.getLabel();
			String combinedLabel = currentParticipantLabel + ":" + currentSegmentLabel;	

			if (combinedLabel.indexOf(":") == 0)
				combinedLabel = combinedLabel.substring(1);
			
			setCurrentSegmentString(combinedLabel);
		}
		else
			setCurrentSegmentString(null);
	}
	
	public JList<String> getUtteranceList() {
		return utteranceList;
	}

	public void moveUtteranceListSelection(boolean up) {
		incrementSelectedUtterance(up ? -1 : 1);
	}
	
	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}
	
	public Segment getSegment() {
		return segment;
	}

	public SS3Participant getSelectedParticipant() {
		return selectedParticipant;
	}

	public void setSelectedParticipant(SS3Participant selectedParticipant) {
		this.selectedParticipant = selectedParticipant;
	}

	public SS3Participant getSelectedAnnotator() {
		return selectedAnnotator;
	}

	public void setSelectedAnnotator(SS3Participant selectedAnnotator) {
		this.selectedAnnotator = selectedAnnotator;
	}
	
	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();
		switch (Integer.parseInt(str)) {
		case NOTES_BTN:
			if (notesDialog == null)
				createNotesDialog();
			notesDialog.setVisible(true);
			break;
		}
	}

	private void createNotesDialog() {
		notesDialog = new JDialog();
		notesDialog.setTitle("Note Editor");
		notesDialog.setModal(true);

		notesPanel = new NotesPanel( getCurrentSegment() != null ? getCurrentSegment().getNote() : null,
				notesDialog);

		JPanel panel1 = new JPanel(new BorderLayout());
		panel1.add(notesPanel, BorderLayout.CENTER);
		panel1.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		Container notesContentPane = notesDialog.getContentPane();
		notesContentPane.setLayout(new BorderLayout());
		notesContentPane.add(notesPanel, BorderLayout.CENTER);

		notesDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				notesPanel.save();
			}
		});

		notesDialog.setSize(400, 350);
		notesDialog.setPreferredSize(new Dimension(400, 350));
		notesDialog.setLocation(150, 150);
		notesDialog.pack();
	}

	public void discard() {
		if (notesDialog != null) {
			notesPanel.discard();

			WindowListener[] listeners = notesDialog.getWindowListeners();
			for (int i = 0; i < listeners.length; i++) {
				WindowListener listener = listeners[i];
				notesDialog.removeWindowListener(listener);
			}

			notesPanel = null;
			notesDialog = null;
		}

	}
	
	public JPanel getTop() {
		return top;
	}
	
	public void setRadioButtonSelected(boolean val) {
		radioButton.setSelected(val);
	}

	public void focusUtteranceList() {
		utteranceList.requestFocus();
	}
	
	/**
	 * Based on an index into a list of utterances, reframe the visible viewbox on-screen to the start of the indexed utterance.
	 * This will: 1) Automatically update the utterance list in this SegmentControlPanel; 2) Set a new movie time; 
	 * 3) Move the slider to this new movie time; 4) refresh the visible nonmanual fields in every segment tier.
	 * @param utteranceIndexToSelect The index of the utterance to select, based on the parent field of this SegmentControlPanel
	 */
	private void selectAndScrollToUtteranceByIndex(int utteranceIndexToSelect) {  
        PresentationField parentField = getSegment().getParentField();
        ArrayList<PresentationEventsEntity> utterances = parentField.getEntities();
        PresentationEventsEntity reframeToUtterance = utterances.get(utteranceIndexToSelect);

        var start = reframeToUtterance.getStartTime();
        var end = reframeToUtterance.getEndTime();

        SegmentGraphReprUtil.utteranceMovieTimeOnset = start;
        SegmentGraphReprUtil.utteranceMovieTimeOffset = end;
        
        SS3Singleton.getVideoControlManager().getMultipleMovieController().setCurrentTime(SegmentGraphReprUtil.utteranceMovieTimeOnset);

		segmentPanel.getUtteranceView().setVisible(reframeToUtterance.getStartTimeCoordinate(), reframeToUtterance.getEndTimeCoordinate(), true);

		segmentPanel.getUtteranceView().expandAllNonmanualFields();
	}
	
	public SegmentTemplate getSegmentTemplate() {
		return segmentTemplate;
	}
	
	@Override
	public void mouseClicked(MouseEvent e) {
		segmentTemplate.selectTemplate();
	}

	@Override
	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub
		
	}
}

class PrevNextUtteranceListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	
	public PrevNextUtteranceListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
        
        // just check the utterance is not out of bounds for either prev/next
		String str = e.getActionCommand();		
		switch (Integer.parseInt(str)) {
		case SegmentControlPanel.PREV_BTN:
			
			if (!this.segmentControlPanel.incrementSelectedUtterance(-1)) {
				String msg = "<HTML>Warning: Previous utterance does not exist <BR>";
				msg = msg + "Please use \"Next Utterance\" to view other utterances</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resource Update", msg);
				return;
			}
			break;
		case SegmentControlPanel.NEXT_BTN:
			if (!this.segmentControlPanel.incrementSelectedUtterance(1)) {
				String msg = "<HTML>Warning: Next utterance does not exist <BR>";
				msg = msg + "Please use \"Prev Utterance\" to view other utterances</HTML>";
				SS3Singleton.getSignStreamApplication()
						.confirmationDialog("Resource Update", msg);
				return;
			}
			break;
		}
		
		// unselect events, and unhighlight everything so that a new look at an utterance plays that utterance
		segmentControlPanel.getSignStreamSegmentPanel().getToolBarPanels().getMediaToolBar().selectEvent(null, null, null);
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getFieldWrappersCollection().unselectAll();
		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getGlossNRelatedView().getFieldWrappersCollection().unselectAll();
		
		// gets the focus off of the utterance list and allows the up/down to move the zoom slider
//		segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getHeader().getTimelineView()
//				.getUpperLeftCorner().getZoomInOut().requestFocus();
	}
}

class UtteranceListSelectionListener implements ListSelectionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public UtteranceListSelectionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();		
	}
	@Override
	public void valueChanged(ListSelectionEvent e) {
		// this will proceed once the value is not longer adjusting, meaning the mouse click is released
		if (e.getValueIsAdjusting() == true)
			return;
		
        ListSelectionModel utteranceListSelectionModel = (ListSelectionModel) e.getSource();
        
        if (utteranceListSelectionModel.isSelectionEmpty()) {
        		return;
        } else {
	    		// go to selected utterance
            int utteranceListIndex = utteranceListSelectionModel.getMaxSelectionIndex();
            EventsEntity utterance = (EventsEntity) segmentControlPanel.getSegment().getParentField().getEntities().get(utteranceListIndex);
            
            // unselect events, and unhighlight everything so that a new look at an utterance plays that utterance
            segmentControlPanel.getSignStreamSegmentPanel().getToolBarPanels().getMediaToolBar().selectEvent(null, null, null);
            segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getNonmanualFieldsView().getFieldWrappersCollection().unselectAll();
            segmentControlPanel.getSignStreamSegmentPanel().getUtteranceView().getGlossNRelatedView().getFieldWrappersCollection().unselectAll();

            segmentControlPanel.setSelectedUtterance(utterance);
            segmentPanel.getUtteranceView().expandAllNonmanualFields();
        }        
	}
	
}

class SegmentActionListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public SegmentActionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
	}
	@Override
	public void actionPerformed(ActionEvent e) {
		// first check that this segment tier is not visible in any other segment in the utterance view
		String selectedTemporalPartitionName = 
				(String) segmentControlPanel.getTemporalPartitionComboBox().getSelectedItem();		

		String selectedSegmentTierNameWithParticipant = 
				(String) segmentControlPanel.getSegmentComboBox().getSelectedItem();
		String selectedSegmentTierName = selectedSegmentTierNameWithParticipant.
				substring(selectedSegmentTierNameWithParticipant.indexOf(":") + 1);
		
		segmentPanel.getUtteranceView().addUtterancePane(segmentControlPanel.getSegment(),
				selectedTemporalPartitionName, selectedSegmentTierName);

	}
	
}

class TemporalPartitionActionListener implements ActionListener {
	private SegmentControlPanel segmentControlPanel = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public TemporalPartitionActionListener(SegmentControlPanel segmentControlPanel) {
		this.segmentControlPanel = segmentControlPanel;
		this.segmentPanel = segmentControlPanel.getSignStreamSegmentPanel();
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		// first check that this segment tier is not visible in any other segment in the utterance view
		String selectedTemporalPartitionName = 
				(String) segmentControlPanel.getTemporalPartitionComboBox().getSelectedItem();		
		
		SS3TemporalPartition currentTemporalPartition = 
				segmentPanel.getSS3Database().getSS3GlossWindow().getTemporalPartitionByLabel(selectedTemporalPartitionName);
		
		SS3GlossWindowSegment defaultSegment = currentTemporalPartition.getDefaultSegment();
		String defaultSegmentName = defaultSegment.getLabel();
		
		// go to selected segment tier, with the same utterance as the current
		segmentPanel.getUtteranceView().addUtterancePane(segmentControlPanel.getSegment(),
				selectedTemporalPartitionName, defaultSegmentName);
	}
	
}