package edu.bu.signstream.grepresentation.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JSplitPane;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class SegmentTemplate extends JPanel {	

	private static final long serialVersionUID = 1L;
	
	private SignStreamSegmentPanel segmentPanel = null;
	private Segment segment = null;
	private SegmentControlPanel controlPanel = null;
	private NonmanualFieldsView topView = null;
	private GlossNRelatedView bottomView = null;
	
	private JSplitPane jsp = null;
	private GridBagConstraints c = new GridBagConstraints();
	private boolean isHidden = false;
	private boolean selected = false;
	private JSplitPane sp;
	
	SegmentTemplate(Segment segment, 
			UtteranceView view, 
			SignStreamSegmentPanel segmentPanel, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;
		bottomView = new GlossNRelatedView(segment, segmentPanel, view, parentField);
		controlPanel = new SegmentControlPanel(segmentPanel, segment, this);
		topView = new NonmanualFieldsView(segment, segmentPanel, controlPanel, bottomView, parentField);
		
		jsp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, topView, bottomView);

		jsp.setOneTouchExpandable(true);
		jsp.setDividerLocation(0.5);
		jsp.setResizeWeight(0.5);
		jsp.setDividerSize(5);
		

		jsp.setPreferredSize(new Dimension(
				(SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH + 
				SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH),  
				this.getHeight()));

		controlPanel.setPreferredSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				this.getHeight()));
		controlPanel.setMinimumSize(new Dimension(
				SegmentGraphReprUtil.UTTERANCE_VIEW_LEFT_PANEL_WIDTH, 
				this.getHeight()));
		
		controlPanel.setBackground(Palette.BACKGROUND_COLOR);
		controlPanel.getTop().setBackground(Palette.BACKGROUND_COLOR);

		setBackground(Palette.BACKGROUND_COLOR);
		setBorder(BorderFactory.createLineBorder(Color.BLACK));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1.0;

		sp = new JSplitPane();
		
		sp.setLeftComponent(controlPanel);
		sp.setRightComponent(jsp);
		
		add(sp);
		
	}
	
	/**
	 * Is this segment template pane currently hidden? Returns boolean
	 * Called by UtteranceView.renderSegmentTemplatePanes()
	 */
	public boolean isHidden() {
		return isHidden;
	}
	
	public void selectTemplate() {
		segmentPanel.getUtteranceView().unselectAllSegmentTemplates();
		this.selected = true;
		segmentPanel.getUtteranceView().renderSegmentTemplatePanes();
	}
	
	/** 
	 * Is this segment the currently mouse-selected item?
	 * Called by UtteranceView.renderSegmentTemplatePanes()
	 * @return selected - boolean value activated by a mouse click on the panel, or default from UtteranceView
	 */
	public boolean isSelected() {
		return selected;
	}
	
	public void unselect() {
		this.selected = false;
	}
	
	public void update(Segment segment) {
		controlPanel.update(segment); 
		topView.update(segment);
		bottomView.update(segment);
	}
	
	public Segment getSegment() {
		return segment;
	}
	
	public SegmentControlPanel getSegmentControlPanel() {
		return controlPanel;
	}

	public NonmanualFieldsView getTopView() {
		return topView;
	}
	
	public GlossNRelatedView getBottomView() {
		return bottomView;
	}
	
	public JSplitPane getSplitPane() {
		return sp;
	}
	
	public EventsEntity getSelectedUtterance() {
		return getSegmentControlPanel().getSelectedUtterance();
	}
	
	
} 