package edu.bu.signstream.grepresentation.view;

import java.awt.Dimension;
import java.awt.Point;

import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.ScrollPaneConstants;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class TimelineView extends JPanel {

	   private static final long serialVersionUID = 1L;
	
	   protected LinenView linenView;
	   protected JScrollPane pictureScrollPane;

	   private SignStreamSegmentPanel panel;
	   private UpperLeftCorner upperLeft = null;
	   
	   private JSplitPane sp;

	   public TimelineView(
	      final SignStreamSegmentPanel panel) {
	      this.panel = panel;

	      createUI();
	   }

	   public void createUI() {
	      upperLeft = new UpperLeftCorner(panel);
	      
		  int mDuration = panel.getMultipleMovieController().getMovieDuration(); 
		  int length = panel.getZoomer().getCoordinateTimeConvertor().getScreenWidth(mDuration);

	      linenView = new LinenView(LinenView.HORIZONTAL, true, length, panel);

	      pictureScrollPane =
	         new JScrollPane(
	        	linenView,
	        	ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER,
	            ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
	      pictureScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

	      upperLeft.setMinimumSize(new Dimension(
	    		  SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH, 
	    		  SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));	 
	         
	      //pictureScrollPane.setRowHeaderView(upperLeft);
	      pictureScrollPane.setBackground(Palette.BACKGROUND_COLOR);

	      sp = new JSplitPane();
	      sp.setLeftComponent(upperLeft);
	      sp.setRightComponent(pictureScrollPane);
	      
	      sp.setMaximumSize(new Dimension(
	    		  (SegmentGraphReprUtil.UTTERANCE_VIEW_LABEL_PANEL_WIDTH +
	    				  SegmentGraphReprUtil.UTTERANCE_VIEW_GRAPH_PANEL_WIDTH), 
	    		  SegmentGraphReprUtil.UTTERANCE_VIEW_HEADER_FOOTER_HEIGHT));  
	      
	      setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
	      
	      add(sp);
	   }

	   public JScrollPane getJScrollPane() {
	      return pictureScrollPane;
	   } 

	   public SignStreamSegmentPanel getSignStreamSegmentPanel() {
	      return panel;
	   }

	   public UpperLeftCorner getUpperLeftCorner() {
	      return upperLeft;
	   }
	   
	   public JSplitPane getSplitPane() {
		   return sp;
	   }
	   
	public void scrollRectToVisible(Point point) {
		if (point == null)
			return;
		JViewport vp = pictureScrollPane.getViewport();
		if (vp != null) {
			//System.out.println("Scroll rect to visible");
			vp.setViewPosition(point);
			pictureScrollPane.invalidate();
			pictureScrollPane.revalidate();
		}
	}
}
