package edu.bu.signstream.grepresentation.view;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.logging.Level;

import javax.swing.BoundedRangeModel;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.*;
import edu.bu.signstream.grepresentation.listener.GlossNRelatedFieldsListener;
import edu.bu.signstream.grepresentation.listener.NonManualFieldsListener;
import edu.bu.signstream.grepresentation.reguestsHandler.Manager;
import edu.bu.signstream.grepresentation.reguestsHandler.RequestHandler;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.panels.search.SearchSavedState;

public class UtteranceView extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private UtteranceViewHeader header = null;
	private UtteranceViewFooter footer = null;

	protected JScrollPane pictureScrollPane;
	private JPanel centerPanel = null;

	private ArrayList<Segment> segments;
	private SignStreamSegmentPanel segmentPanel;
	private ArrayList<SegmentGraphicRepresentation> repList = new ArrayList<SegmentGraphicRepresentation>();
	private ArrayList<SegmentTemplate> templates = new ArrayList<SegmentTemplate>();

	/** 
	 * This template object represents the "prototypical" example of a template currently being rendered on screen.
	 * It is technically always pointing to the last one rendered in a list of templates.
	 */
	private SegmentTemplate template = null;
	private NonmanualFieldsView topView = null;
	private GlossNRelatedView bottomView = null;
	
	private SplitPaneListener controlPanelSplitPaneListener = new SplitPaneListener();
	private SplitPaneListener utteranceSplitPaneListener = new SplitPaneListener();

	private boolean showEndOnZoom = false;
	
	public UtteranceView(ArrayList<Segment> segments,
			SignStreamSegmentPanel segmentPanel) {
		this.segments = segments;
		this.segmentPanel = segmentPanel;

		int movieTime = SegmentGraphReprUtil.utteranceMovieTimeOnset;
		if (movieTime < 0)
			movieTime = 0;

		SS3Singleton.getVideoControlManager().getMultipleMovieController().setCurrentTime(movieTime);
		segmentPanel.getSlider().setSliderPositionTime(movieTime);	

		int newSliderPosition = segmentPanel.getSlider().getSliderPosition();

		if (newSliderPosition < 0)
			newSliderPosition = 0;
		
		Point scrollTo = new Point(newSliderPosition, 0);
		
		header = new UtteranceViewHeader(segmentPanel, this);		
		footer = new UtteranceViewFooter(segmentPanel, this);
		
		BoundedRangeModel tempModel = null;

		repList.clear();

		centerPanel = new JPanel();
		centerPanel.setLayout(new GridLayout(0, 1));
		
		for (Segment segment1 : segments) {
			String segmentID = Util.getUniqueNumber();
			segment1.setID(segmentID);

			template = createNewSegmentTemplate(segment1, segment1.getParentField());
			
			if (tempModel != null) {
				template.getTopView().getJScrollPane().getHorizontalScrollBar().setModel(tempModel);
				template.getBottomView().getJScrollPane().getHorizontalScrollBar().setModel(tempModel);
			}
			
			tempModel = template.getBottomView().getJScrollPane().getHorizontalScrollBar().getModel();
			template.getTopView().getJScrollPane().getHorizontalScrollBar().setModel(tempModel);
			
			
			utteranceSplitPaneListener.addSplitPane(template.getTopView().getSplitPane());
			utteranceSplitPaneListener.addSplitPane(template.getBottomView().getSplitPane());
			
			controlPanelSplitPaneListener.addSplitPane(template.getSplitPane());
			
		}
		
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				template.selectTemplate();	// select the last template by default, just to have something selected initially
			}
		});

		// just set this UtteranceView's pictureScrollPane to the last bottomView's own pictureScrollPane, for sizing purposes
		if (bottomView != null)
			pictureScrollPane = bottomView.getJScrollPane();
		
		scrollRectToVisible(scrollTo);
		
		if (tempModel != null) {
			tempModel.addChangeListener(new ChangeListener() {
				@Override
				public void stateChanged(ChangeEvent e) {
					// as user scrolls, update the utterance lists, same as scrollRectToVisible would do
					updateUtteranceLists();
				}
			});
			
			footer.getScrollBarView().getJScrollBar().setModel(tempModel);
			header.getTimelineView().pictureScrollPane.getHorizontalScrollBar().setModel(
					footer.getScrollBarView().getJScrollBar().getModel());
			
			utteranceSplitPaneListener.addSplitPane(header.getTimelineView().getSplitPane());
			utteranceSplitPaneListener.addSplitPane(footer.getScrollBarView().getSplitPane());
			
			controlPanelSplitPaneListener.addSplitPane(header.getSplitPane());
			controlPanelSplitPaneListener.addSplitPane(footer.getSplitPane());
			
		}
		
		setLayout(new BorderLayout(0, 0));
		add(header, BorderLayout.NORTH);
		add(centerPanel, BorderLayout.CENTER);
		add(footer, BorderLayout.SOUTH);	
		
		expandAllNonmanualFields();

		updateDrawingsAreaSize();
	}

	public NonmanualFieldsView getNonmanualFieldsView() {
		return topView;
	}
	
	public GlossNRelatedView getGlossNRelatedView() {
		return bottomView;
	}
	
	public Segment getSegment() {
		return segments.get(0);
	}

	public void setVisible(PresentationEvent event, boolean start) {
		if (pictureScrollPane != null) {
			JViewport vp = pictureScrollPane.getViewport();
			int visibleWidth = vp.getVisibleRect().width;
			int x1 = vp.getViewPosition().x;
			int x2 = x1 + visibleWidth;
			if (x1 > event.getStartTimeCoordinate() || x2 < event.getEndTimeCoordinate()) {
				int newXCoord;
				if (start) {
					int utteranceStart = event.getStartTimeCoordinate();
					newXCoord = utteranceStart - 1 > 0 ? utteranceStart - 1 : utteranceStart;
				} else {
					newXCoord = event.getEndTimeCoordinate() + 1 - visibleWidth;
				}
				scrollRectToVisible(new Point(newXCoord, 0));
			}
		}
	}

	public void setVisible(int startCoord, int endCoord, boolean forceScroll) {
		
		Point scrollTo = null;
		Point viewPosition = null;
		Rectangle visibleRect = null;

		if (pictureScrollPane != null) {
			try {
				JViewport viewport = pictureScrollPane.getViewport();
				visibleRect = viewport.getVisibleRect();
				viewPosition = viewport.getViewPosition();

				int visibleWidth = visibleRect.width;
				int X1 = viewPosition.x;
				int X2 = X1 + visibleWidth;
				

				if (!forceScroll && X1 < startCoord && X2 > endCoord) {
					return;
				}
				scrollTo = new Point(startCoord - 1 < 0 ? startCoord : startCoord - 1, 0);
				scrollRectToVisible(scrollTo);
			} catch (Exception ex) {
				ex.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
			}
		}
	}

	public void updateDrawingsAreaSize() {
		int duration = segmentPanel.getMultipleMovieController()
				.getMovieDuration();

		int sw = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertTimeToCoordinate(duration);
		int screenWidth = sw + DatabaseView.screenBufferAmount;

		// going through the rep's for nonman and gloss views all together (resetting them to length of movie duration)
		for (SegmentGraphicRepresentation rep : repList) {
			Dimension d = rep.getPreferredSize();
			Dimension dd = new Dimension(screenWidth, d.height);
			rep.setPreferredSize(dd);
			rep.revalidate();
		}
		
		LinenView l = header.getTimelineView().linenView;
		Dimension d = l.getPreferredSize();
		Dimension dd = new Dimension(screenWidth, d.height);
		l.setPreferredSize(dd);
		l.revalidate();
	}

	public int getWidht() {

		try {
			Rectangle visibleRect = null;
			if (template != null && template.getTopView() != null) {
				visibleRect = template.getTopView().getJScrollPane()
						.getViewport().getVisibleRect();
			}
			return visibleRect.width;
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
		return -1;
	}

	public Point getVisiblePoint() {		
		JScrollPane sp = template.getBottomView().getJScrollPane();		
		int value = sp.getHorizontalScrollBar().getValue();
		return new Point(value, 0);
	}

	public int getVisibleMiddlePointTime() {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			if (template != null && template.getTopView() != null) {
				visibleRect = template.getTopView().getJScrollPane()
						.getViewport().getVisibleRect();
				viewPosition = template.getTopView().getJScrollPane()
						.getViewport().getViewPosition();
			}

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X = X1 + visibleWidth / 2;

			int time = segmentPanel.getZoomer().getCoordinateTimeConvertor()
					.convertCoordinateToTime(X);

			return time;

		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}

		return -1;
	}

	public void setMidPointTimeVisible(int time) {
		int midCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor()
				.convertTimeToCoordinate(time);
		Rectangle visibleRect = null;

		try {
			if (template != null && template.getTopView() != null) {
				visibleRect = template.getTopView().getJScrollPane()
						.getViewport().getVisibleRect();
			}

			int halfVisibleWidth = visibleRect.width / 2;
			int X1 = midCoord - halfVisibleWidth;

			if (X1 < 0)
				X1 = 0;

			Point scrollTo = new Point(X1, 0);
			
			scrollRectToVisible(scrollTo);
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	int distance = 0;

	private int getDistanceToCoord(int coord) {
		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			if (template != null && template.getTopView() != null) {
				visibleRect = template.getTopView().getJScrollPane()
						.getViewport().getVisibleRect();
				viewPosition = template.getTopView().getJScrollPane()
						.getViewport().getViewPosition();
			}

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X2 = X1 + visibleWidth;

			if (X1 < coord && X2 > coord) {
				distance = coord - X1;
			}
		} catch (Exception ex) {
			return distance;
		}
		return distance;
	}
	
	public void setZoomInOutValue(double value) {
		header.getTimelineView().getUpperLeftCorner().setValue(value);
	}

	// if center of selected item is on the left - keep start coord fixed
	// if center of selected item is on the right - keep end coord fixed
	// if item is not selected, keep video slider fixed
	public synchronized void zoom(double zoomValue) {
		segmentPanel.getZoomer().zoom(zoomValue);

		Point scrollTo = null;

		int sliderVisibleCoord = getDistanceToCoord(segmentPanel.getSlider()
				.getSliderPosition());
		Event selectedEvent = segmentPanel.getSelectedEvent();

		if (selectedEvent != null) {
			int eventCoord = showEndOnZoom ? selectedEvent.getEndTimeCoordinate() : selectedEvent.getStartTimeCoordinate();
			int eventVisibleCoord = getDistanceToCoord(eventCoord);
			// TODO: Why set the time to the current time?
			int movieTime = SS3Singleton.getVideoControlManager()
					.getMultipleMovieController().getCurrentTime();
			segmentPanel.getSlider().setSliderPositionTime(movieTime);

			int X2 = eventCoord - eventVisibleCoord;

			if (X2 < 0)
				X2 = 0;

			scrollTo = new Point(X2, 0);
		} else if (sliderVisibleCoord > -1) {
			
			segmentPanel.getZoomer().zoom(zoomValue);
			segmentPanel.repaint();

            // TODO: Why set the time to the current time?
			int movieTime = SS3Singleton.getVideoControlManager()
					.getMultipleMovieController().getCurrentTime();
			segmentPanel.getSlider().setSliderPositionTime(movieTime);
			int newSliderPosition = segmentPanel.getSlider()
					.getSliderPosition();
			int X = newSliderPosition - sliderVisibleCoord;

			if (X < 0)
				X = 0;

			scrollTo = new Point(X, 0);
		} else {
			EventsEntity selectedEntity = SS3Singleton.getMediaToolBar(null).getSelectedEventsEntity();
			Event parentEvent = selectedEntity.getParentEntity().getEvent();

			if (parentEvent != null) {
				setVisible(parentEvent.getStartTimeCoordinate(),
						parentEvent.getEndTimeCoordinate(), false);
			} else {
				int midPointTime = getVisibleMiddlePointTime();
				setMidPointTimeVisible(midPointTime);
			}
		}

		updateDrawingsAreaSize();
		
		scrollRectToVisible(scrollTo);

		segmentPanel.repaint();
		segmentPanel.updateUI();
	}

	public void setShowEndOnZoom(boolean showEndOnZoom) {
		this.showEndOnZoom = showEndOnZoom;
	}

	public void setVisibleCoord(int coord) {
		Point scrollTo = null;

		Rectangle visibleRect = null;
		Point viewPosition = null;

		try {
			if (template != null && template.getTopView() != null) {
				visibleRect = template.getTopView().getJScrollPane()
						.getViewport().getVisibleRect();
				viewPosition = template.getTopView().getJScrollPane()
						.getViewport().getViewPosition();
			}

			int visibleWidth = visibleRect.width;
			int X1 = viewPosition.x;
			int X2 = X1 + visibleWidth;

			if (X1 < coord && X2 > coord) {
				return;
			} else {
				int dWidth = visibleWidth - coord;
				int X3 = coord - dWidth / 2;

				if (X3 < 0)
					X3 = 0;

				scrollTo = new Point(X3, 0);
				
				scrollRectToVisible(scrollTo);
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	public ArrayList<Segment> getSegments() {
		return segments;
	}

	// 2020-09-24: turning this from public to private, since its not used
	private JScrollPane getPictureScrollPane() {
		return pictureScrollPane;
	}

	public JScrollBar getHorizontalScrollBar() {
		if (templates.size() > 0) {
			SegmentTemplate template = templates.get(templates.size() - 1);
			return template.getBottomView().getJScrollPane()
					.getHorizontalScrollBar();
		} else if (pictureScrollPane != null) {
			return pictureScrollPane.getHorizontalScrollBar();
		} else {
			return null;
		}
	}

	public int getStartTime() {
		int startTime = -1;
		for (Segment segment : segments) {
			int time = segment.getStartTime();
			if (time == -1) {
				startTime = time;
			} else if (time < startTime) {
				startTime = time;
			}
		}
		return startTime;
	}

	public ArrayList<SegmentTemplate> getSegmentTemplates() {
		return templates;
	}

	public SegmentTemplate getSegmentTemplate(String segmentID) {

		if (templates.size() == 1) {
			return templates.get(0);
		} else if (segmentID == null || segmentID.trim().length() == 0) {
			return null;
		} else {

			for (SegmentTemplate template : templates) {
				if (segmentID.equals(template.getSegment().getID())) {
					return template;
				}
			}
			return null;
		}
	}

	public int getEndTime() {
		int endTime = -1;
		for (Segment segment : segments) {
			int time = segment.getEndTime();
			if (time == -1) {
				endTime = time;
			} else if (time > endTime) {
				endTime = time;
			}
		}
		return endTime;
	}

	public void scrollRectToVisible(Point point) {
		//System.out.println("Scroll Rect to Visible");
		for (SegmentTemplate template: templates) {
			if (template != null) {
				if (template.getTopView() != null) {
					template.getTopView().scrollRectToVisible(point);
				}

				if (template.getBottomView() != null) {
					template.getBottomView().scrollRectToVisible(point);
				}
			}
		}
		header.getTimelineView().scrollRectToVisible(point);
		
		updateUtteranceLists();
	}
	
	/**
	 * In each SegmentTemplate of this UtteranceView, programmatically click the expand button 
	 * on its NonManualFieldsView to trigger a refresh of visible nonmanual fields in the viewbox
	 */
	void expandAllNonmanualFields() {
		for (SegmentTemplate template: templates) {
			if (template != null) {
				if (template.getTopView() != null) {
					template.getTopView().clickExpandButton();
				}
			}
		}
	}
	
	public void updateUtteranceLists() {
		if (pictureScrollPane != null) {
			for (SegmentTemplate template: templates) {
				Rectangle rect = pictureScrollPane.getViewport().getViewRect();
				int minx = (int) rect.getMinX();
				int maxx = (int) rect.getMaxX();
				
				int beginTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(minx);
				int endTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(maxx);
				
				if (beginTime != endTime) {
					SegmentGraphReprUtil.utteranceViewVisibleMovieOnset = beginTime;
					SegmentGraphReprUtil.utteranceViewVisibleMovieOffset = endTime;
				}

				ArrayList entities = 
						template.getSegment().getParentField().getEntitiesBetween(beginTime, endTime, false);
				
				template.getSegmentControlPanel().updateUtteranceListSelectedItems(entities);
				template.getSegmentControlPanel().updateUtteranceList();
			}
		}
	}

	public Dimension getViewportPreferredSize() {
		JViewport vp = pictureScrollPane.getViewport();
		return vp.getPreferredSize();
	}

	public int getVisibleWidth2() {
		if (pictureScrollPane == null) {
			return -1;
		} else {
			JViewport vp = pictureScrollPane.getViewport();
			Dimension d = vp.getExtentSize();
			
			return d.width;
		}
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	public ArrayList<SegmentGraphicRepresentation> getSegmentGraphicRep() {
		return repList;
	}

	public SegmentTemplate getSegmentTemplate() {
		return template;
	}
	
	public void renderSegmentTemplatePanes() {
		centerPanel.removeAll();
		
		for (SegmentTemplate st: templates) {
			if (st.isSelected()) {
				st.getSegmentControlPanel().setBackground(Palette.BACKGROUND_COLOR.darker());
				st.getSegmentControlPanel().getTop().setBackground(Palette.BACKGROUND_COLOR.darker());
				st.getSegmentControlPanel().setRadioButtonSelected(true);
			} else {
				st.getSegmentControlPanel().setBackground(Palette.BACKGROUND_COLOR);
				st.getSegmentControlPanel().getTop().setBackground(Palette.BACKGROUND_COLOR);
				st.getSegmentControlPanel().setRadioButtonSelected(false);
			}
			
			centerPanel.add(st);
		}
		
		revalidate();
		repaint();
		
		segmentPanel.revalidate();
		segmentPanel.repaint();
	}
	
	public void unselectAllSegmentTemplates() {
		for (SegmentTemplate st: templates) {
			st.unselect();
		}
	}
	
	public SegmentTemplate getSelectedSegmentTemplate() {
		for (SegmentTemplate st: templates) {
			if (st.isSelected()) {
				return st;
			}
		}
		
		return null;
	}
	
	public int getSelectedSegmentTemplateIndex() {
		for (int i=0 ; i <templates.size() ; i++) {
			if (templates.get(i).isSelected()) {
				return i;
			}
		}
		
		return -1;
	}
	
	public void removeSelectedPane() {
		SegmentTemplate selectedSegmentTemplate = getSelectedSegmentTemplate();
		if (selectedSegmentTemplate == null) {
			return;		// if nothing is selected, don't do anything
		}
		
		if (templates.size() == 1)
			return; // don't allow there to ever be no templates remaining
		
		// save changes to db
		segmentPanel.updateLoadedDatabase();
		
		// remove the selected template row and segment object itself		
		templates.remove(selectedSegmentTemplate);

		Segment selectedSegment = selectedSegmentTemplate.getSegment();		
		segments.remove(selectedSegment);
		
		// if after deleting a template, none remains selected, force by default the 0th template to selected
		boolean noneSelected = true;		
		for (SegmentTemplate st: templates) {
			if (st.isSelected())
				noneSelected = false;
		}
		if (noneSelected)
			templates.get(0).selectTemplate();
		
		// repaint
		renderSegmentTemplatePanes();
	}

	public void moveSelectedPaneUp() {
		int moveTo = 0, moveFrom = 0;
		
		for (int i = 0; i < templates.size(); i++) {
			if (templates.get(i).isSelected()) {
				moveFrom = i;
				moveTo = i - 1;
				break;
			}
		}
		
		if (moveFrom == 0)
			return;
		
		SegmentTemplate selectedTemplate = templates.get(moveFrom);
		SegmentTemplate previousTemplate = templates.get(moveTo);
		
		templates.set(moveFrom, previousTemplate);
		templates.set(moveTo, selectedTemplate);
		
		renderSegmentTemplatePanes();
	}

	public void moveSelectedPaneDown() {
		int moveTo = 0, moveFrom = 0;
		
		for (int i = 0; i < templates.size(); i++) {
			if (templates.get(i).isSelected()) {
				moveFrom = i;
				moveTo = i + 1;
				break;
			}
		}
		
		if (moveFrom == templates.size() - 1)
			return;
		
		if (templates.size() == 0)
			return;
		
		
		SegmentTemplate selectedTemplate = templates.get(moveFrom);
		SegmentTemplate nextTemplate = templates.get(moveTo);
		
		templates.set(moveFrom, nextTemplate);
		templates.set(moveTo, selectedTemplate);
		
		renderSegmentTemplatePanes();
	}
	
	public UtteranceViewHeader getHeader() {
		return header;
	}
	
	public UtteranceViewFooter getFooter() {
		return footer;
	}
	
	public boolean canSwitchThisSegment(Segment switchingSegment, 
			String temporalPartitionLabel, String segmentTierWithParticipantLabel) {

		String segmentTierLabel = segmentTierWithParticipantLabel;
		int index = segmentTierWithParticipantLabel.indexOf(":");
		if (index != -1)
			segmentTierLabel = segmentTierWithParticipantLabel.substring(index + 1);
				
		for (SegmentTemplate template: templates) {
			SegmentControlPanel scp = template.getSegmentControlPanel();
			
			// only compare the switchingSegment to segments other than itself
			if (scp.getSegment().equals(switchingSegment))
				continue;
			
			String tpLabel = scp.getCurrentPartitionString();
			String stLabel = scp.getCurrentSegmentString();
			index = scp.getCurrentSegmentString().indexOf(":"); 
			if (index != -1)
			 	stLabel = scp.getCurrentSegmentString().substring(index + 1);
			
			if (tpLabel.equals(temporalPartitionLabel) && stLabel.equals(segmentTierLabel)) {
				return false;
			}
		}
		
		return true;
	}

	/**
	 * Add a new segment object (utterance pane).
	 * If segmentToRemove is not null, remove it from the list of segments and replace in its position
	 */
	public void addUtterancePane(Segment segmentToRemove,
			String temporalPartitionName, String segmentTierName) {
		
		// 1) go through the DBview wrappers to find the one matching the segment tier label		
		ArrayList<FieldWrapper> databaseViewWrappers = 	segmentPanel.
				getDatabaseView().getSegmentGraphicRepresentation().getFieldWrappersCollection().getWrappers();		
		
		PresentationFieldWrapper partFldWrapper = null;
		SegmentFieldWrapper segFldWrapper = null;
		PresentationField segFld = null;
		
		for (FieldWrapper wrapper: databaseViewWrappers) {
			if (wrapper.getField().getName().equals(temporalPartitionName)
					&& wrapper instanceof PresentationFieldWrapper) {
				partFldWrapper = (PresentationFieldWrapper) wrapper;
				for (SegmentFieldWrapper wrapper2: partFldWrapper.getSegmentsFieldWrappers()) {
					if (wrapper2.getField().getName().equals(segmentTierName)) {
						segFldWrapper = wrapper2;
						segFld = (PresentationField) wrapper2.getField();
						break;
					}
				}
			}
		}
		
		if (segFldWrapper == null) {
			System.out.println("There was no field found with name " + segmentTierName);
			return;
		}
		
		ArrayList<PresentationEventsEntity> eventsEntitiesToLoad = segFld.getEntities();

		// 2) get the utterances of the segment tier
		ArrayList<SS3GlossWindowItem> surroundingItems = new ArrayList<SS3GlossWindowItem>();
		
		int size = eventsEntitiesToLoad.size();
		// use these indices below to load all items regardless of the clicked one
		int startIndex = 0;
		int endIndex = size;	// top bound exclusive
		
		for (PresentationEventsEntity entity: eventsEntitiesToLoad.subList(startIndex, endIndex)) {
			surroundingItems.add(entity.getSS3GlossWindowItem());
		}
		
		// 3) before proceeding, reset start/end times and save changes to db
		segmentPanel.updateLoadedDatabase();
		
		boolean isSelected = true;
		if (segmentToRemove != null) {
			isSelected = this.getSegmentTemplate(segmentToRemove.getID()).isSelected();
		}
		// 4) remove the Segment obj (if currently in view) and replace it with a new one 	
		int indexOfRemovedSegment = removeSegment(segmentToRemove);
		
		// 5) create the new segment out of the chosen item and reorient start/end times
		SS3Singleton.getMediaToolBar(null).setPlayButtonLabel(MediaToolBar.UTTERANCE);	
		
		Manager mngr = SS3Singleton.getManager();
		RequestHandler handler = mngr.getRequestHandler();

		Segment newSegment = null;
		
		if (indexOfRemovedSegment != -1)
			newSegment = handler.convertMultipleItemsAndAddToSegmentPanelAtIndex(surroundingItems, segFld, indexOfRemovedSegment);
		else
			newSegment = handler.convertMultipleItemsAndAddToSegmentPanel(surroundingItems, segFld);	
		
		String segmentID = Util.getUniqueNumber();
		newSegment.setID(segmentID);
		
		// 6) create the new template which will be displayed on-screen
		template = createNewSegmentTemplate(newSegment, segFld, indexOfRemovedSegment, isSelected);

		// just set this UtteranceView's pictureScrollPane to the last bottomView's own pictureScrollPane, for sizing purposes
		if (bottomView != null)
			pictureScrollPane = bottomView.getJScrollPane();

		template.getTopView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		template.getBottomView().getJScrollPane().getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		header.getTimelineView().pictureScrollPane.getHorizontalScrollBar().setModel(
				footer.getScrollBarView().getJScrollBar().getModel());
		
		utteranceSplitPaneListener.addSplitPane(template.getTopView().getSplitPane());
		utteranceSplitPaneListener.addSplitPane(template.getBottomView().getSplitPane());
		
		controlPanelSplitPaneListener.addSplitPane(template.getSplitPane());
		
		
		footer.getScrollBarView().getJScrollBar().getModel().addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				// as user scrolls, update the utterance lists, same as scrollRectToVisible would do
				updateUtteranceLists();
			}
		});

		int visibleMovieCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(
				SegmentGraphReprUtil.utteranceViewVisibleMovieOnset);
		final Point scrollTo = new Point(visibleMovieCoord, 0);
		
		this.renderSegmentTemplatePanes();

		// does some sort of updating so graph fields are drawn initially
		template.getSegmentControlPanel().incrementSelectedUtterance(0);

		// need to invoke later to ensure that the pictureScrollPane is the correct size before actually scrolling
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				scrollRectToVisible(scrollTo);
			}
		});				
		
		updateDrawingsAreaSize();
		
	}
	
	private SegmentTemplate createNewSegmentTemplate(Segment segment, PresentationField segFld) {
		return createNewSegmentTemplate(segment, segFld, -1, false);
	}
	
	private SegmentTemplate createNewSegmentTemplate(Segment segment, PresentationField segFld, int index, boolean selected) {
		template = new SegmentTemplate(segment, this, segmentPanel, segFld);
		if (index == -1) {
			templates.add(template);
		} else {
			templates.add(index, template);
		}
		
		if (selected) {
			template.selectTemplate();
		}
		
		ArrayList<FieldWrapper> wrappers = segment
				.getFieldWrappersCollection().getWrappers();
		for (FieldWrapper wrapper : wrappers) {
			wrapper.getField().setSegmentID(segment.getID());
		}

		// remove SegmentMouseListener
		MouseListener[] mouseListeners = segmentPanel.getMouseListeners();
		for (int i = 0; i < mouseListeners.length; i++) {
			segmentPanel.removeMouseListener(mouseListeners[i]);
		}

		MouseMotionListener[] mouseMotionListeners = segmentPanel
				.getMouseMotionListeners();
		for (int j = 0; j < mouseMotionListeners.length; j++) {
			segmentPanel.removeMouseMotionListener(mouseMotionListeners[j]);
		}

		topView = template.getTopView();
		bottomView = template.getBottomView();
		
		SegmentGraphicRepresentation topRep = topView
				.getSegmentGraphicRep();
		
		SegmentGraphicRepresentation bottomRep = bottomView
				.getSegmentGraphicRep();
		
		repList.add(topRep);
		repList.add(bottomRep);
		

		NonManualFieldsListener nmListener = new NonManualFieldsListener(
				segmentPanel, topRep, bottomRep, topView.getLabelView(),
				bottomView.getLableView());
		topRep.addMouseListener(nmListener);
		topRep.addMouseMotionListener(nmListener);

		GlossNRelatedFieldsListener glsListener = new GlossNRelatedFieldsListener(
				segmentPanel, topRep, bottomRep, topView.getLabelView(),
				bottomView.getLableView());
		bottomRep.addMouseListener(glsListener);
		bottomRep.addMouseMotionListener(glsListener);
		
		return template;
	}

	// removes the specified segment if it exists and return the index it was removed from or -1 if it does not exist
	public int removeSegment(Segment segmentToRemove) {
		int indexOfSegmentToRemove = -1;
		ArrayList<Segment> segments = segmentPanel.getSegments();
		
		for (int i = 0; i < segments.size(); i++) {
			Segment segment = segments.get(i);
			if (segment.equals(segmentToRemove)) {
				indexOfSegmentToRemove = i;
			}
		}
		
		// editing (meaning, replacing) already existent segment
		if (indexOfSegmentToRemove != -1) {						
			segments.remove(indexOfSegmentToRemove);
			
			SegmentTemplate templateToRemove = getSegmentTemplate(segmentToRemove.getID());
			if (templateToRemove != null) {
				templates.remove(templateToRemove);
				centerPanel.remove(templateToRemove);
				
				//Change scroll bar model to not interfere with existing templates after removal.
				templateToRemove.getTopView().getJScrollPane().getHorizontalScrollBar().setModel(new DefaultBoundedRangeModel());
				templateToRemove.getBottomView().getJScrollPane().getHorizontalScrollBar().setModel(new DefaultBoundedRangeModel());
			}
		}
		
		return indexOfSegmentToRemove;
	}
	
	public SearchSavedState saveCurrentState() {
		SS3Database database = segmentPanel.getSS3Database();
		int visibleOnsetTime = SegmentGraphReprUtil.utteranceViewVisibleMovieOnset;
		int sliderPosition = segmentPanel.getSlider().getSliderPosition();
		double zoomValue = segmentPanel.getZoomer().zoom();
		String selectedUtteranceId = this.getSelectedSegmentTemplate().getSegmentControlPanel().getSelectedUtterance().getEvent().getID();
		
		ArrayList<String> segmentsList = new ArrayList<String>();
		
		for (SegmentTemplate template: templates) {
			SegmentControlPanel scp = template.getSegmentControlPanel();
			
			String tpLabel = scp.getCurrentPartitionString();
			String stLabel = scp.getCurrentSegmentString();
			int index = scp.getCurrentSegmentString().indexOf(":"); 
			if (index != -1)
			 	stLabel = scp.getCurrentSegmentString().substring(index + 1);
			
			String combinedLabel = tpLabel + ";" + stLabel;
			segmentsList.add(combinedLabel);			
		}
		
		return new SearchSavedState(database, segmentsList, visibleOnsetTime, sliderPosition, zoomValue, getSelectedSegmentTemplateIndex(), selectedUtteranceId);
	}
	
	public void revertToSavedState(SearchSavedState searchSavedState) {
		// first get rid of everything
		centerPanel.removeAll();
		templates.clear();
		segments.clear();
		
		int visibleOnsetTime = searchSavedState.getVisibleOnsetTime();
		int sliderPosition = searchSavedState.getSliderPosition();
		double zoomValue = searchSavedState.getZoomerValue();
		String selectedUtteranceId = searchSavedState.getSelectedUtteranceId();
		int selectedSegmentTemplateIndex = searchSavedState.getSelectedSegmentTemplateIndex();
		ArrayList<String> segmentsList = searchSavedState.getSegmentsList();

		for (String segmentString: segmentsList) {
			StringTokenizer tokSemicolon = new StringTokenizer(segmentString, ";");
			
			String partitionLabel = tokSemicolon.nextToken();
			String segmentLabel = tokSemicolon.nextToken();
			
			addUtterancePane(null, partitionLabel, segmentLabel);
		}
		
		
		SegmentTemplate t = this.getSegmentTemplates().get(selectedSegmentTemplateIndex);
		t.selectTemplate();
		t.getSegmentControlPanel().setSelectedUtterance(selectedUtteranceId);
		
		
		int visibleOnsetCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(
				visibleOnsetTime);
		final Point scrollTo = new Point(visibleOnsetCoord, 0);
		
		// need to invoke later to ensure that the pictureScrollPane is the correct size before actually scrolling
		SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				scrollRectToVisible(scrollTo);
			}
		});				

		segmentPanel.getSlider().setSliderPositionCoord(sliderPosition);
		int currentMovieTime = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertCoordinateToTime(sliderPosition);
		SS3Singleton.getVideoControlManager().getMultipleMovieController().setCurrentTime(currentMovieTime);
	}
	
	public void parseSegmentStringAndReloadView(String str) {
		// first get rid of everything
		centerPanel.removeAll();
		templates.clear();
		segments.clear();
		
		StringTokenizer tokSemicolon = new StringTokenizer(str, ";");
		
		String partitionLabel = tokSemicolon.nextToken();
		String segmentLabel = tokSemicolon.nextToken();
		
		addUtterancePane(null, partitionLabel, segmentLabel);
		// template has just been set here

		String utteranceLabelAndGlosses = tokSemicolon.nextToken();
		StringTokenizer tokSpace = new StringTokenizer(utteranceLabelAndGlosses, " ");
		
		String utteranceLabel = tokSpace.nextToken();		
		
		ArrayList<PresentationEventsEntity> utterances = template.getSegment().getParentField().getEntities();
		for (int i=0;i<utterances.size();i++) {
			final PresentationEventsEntity utterance = utterances.get(i);
			SS3GlossWindowItem item = utterance.getEvent().getSS3GlossWindowItem();
			if (item.getLabel().equals(utteranceLabel)) {
                int itemStartTime = item.getStartTimeInfo().getMovieTime();
                SS3Singleton.getVideoControlManager().getMultipleMovieController().setCurrentTime(itemStartTime);

//				int itemStartCoord = segmentPanel.getZoomer().getCoordinateTimeConvertor().convertTimeToCoordinate(itemStartTime);
//				segmentPanel.getSlider().setSliderPositionCoord(itemStartCoord);

				final int index = i;
				// need to invoke later to ensure that the pictureScrollPane is the correct size before actually scrolling
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						segmentPanel.getUtteranceView().getSegmentTemplate().getSegmentControlPanel().setSelectedUtterance(utterance);
					}
				});				

				SS3Singleton.getMediaToolBar(segmentPanel).setCurrentParentField(utterance.getField());

				break;
			}
		}
	}

}

class SplitPaneListener implements MouseListener {
	
	private ArrayList<BasicSplitPaneDivider> components = new ArrayList<BasicSplitPaneDivider>();
	
	public void addSplitPane(JSplitPane sp) {
		BasicSplitPaneUI ui = (BasicSplitPaneUI) sp.getUI();
		BasicSplitPaneDivider divider = ui.getDivider();
		divider.addMouseListener(this);
		components.add(divider);
	}
	
	@Override
	public void mouseClicked(MouseEvent e) {
		//System.out.println("Mouse Click.");
	}

	@Override
	public void mousePressed(MouseEvent e) {
		//System.out.println("Mouse Press.");
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		Component divider = (Component) e.getSource();
		JSplitPane originalSP = (JSplitPane) divider.getParent();
		for (Component c : components) {
			if (c != e.getSource()) {
				JSplitPane newSP = (JSplitPane) c.getParent();
				newSP.setDividerLocation(originalSP.getDividerLocation());
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		//System.out.println("Mouse Enter.");
	}

	@Override
	public void mouseExited(MouseEvent e) {
		//System.out.println("Mouse Exit.");
	}
}

