package edu.bu.signstream.grepresentation.view.mediaControl;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.JTextField;

import edu.bu.signstream.common.util.CoordinateTimeConverter;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.grepresentation.fields.Event;
import edu.bu.signstream.grepresentation.fields.EventsEntity;
import edu.bu.signstream.grepresentation.fields.Field;
import edu.bu.signstream.grepresentation.fields.glossField.EnteredText;
import edu.bu.signstream.grepresentation.fields.glossField.GlossChainedEventsEntity;
import edu.bu.signstream.grepresentation.fields.glossField.GlossField;
import edu.bu.signstream.grepresentation.fields.glossField.Text;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationEventsEntity;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

public class SetStartTimeListener implements ActionListener {

	private MediaToolBar bar;

	public SetStartTimeListener(MediaToolBar bar) {
		this.bar = bar;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		if (bar == null)
			return;

		Event event = bar.getSelectedEvent();
		Field field = bar.getSelectedField();
		SignStreamSegmentPanel panel = bar.getSignStreamsegmentPanel();

		if (panel == null)
			return;

		if (panel.getMultipleMovieController().getMovies().isEmpty()) {
			// no video files are loaded
			SS3Singleton.getErrorMessages().showErrorMessage(
					ErrorMessages.SELECT_VALID_VIDEO_FILE);
			return;
		}

		MediaToolbarVideoController.stopPlayingVideo(panel);

		EventsEntity entity = bar.getSelectedEventsEntity();

		EventsEntity selectedEntity = bar.getSelectedEventsEntity();
		EventsEntity parentEntity = null;

		CoordinateTimeConverter convertor = panel.getZoomer()
				.getCoordinateTimeConvertor();

		int time = panel.getMultipleMovieController().getCurrentTime();
        System.out.println("Set start time: " + time);
		int coord = convertor.convertTimeToCoordinate(time);

		if (selectedEntity != null) {
			parentEntity = selectedEntity.getParentEntity();
		} else {
			PresentationField parentField = field.getParentField();
			parentEntity = (EventsEntity) parentField.getEventAt(coord);
		}
		
		if (field instanceof GlossField) {
			GlossField glsField = (GlossField) field;	
			
			// enforce that a one-frame gloss occupies its frame all by itself with no other gloss entities intersecting
			for (Object ent: glsField.getEntities()) {
				GlossChainedEventsEntity glsFieldEntity = (GlossChainedEventsEntity) ent;
				
				int glsFieldEntityStartTime = glsFieldEntity.getStartTimeInfo().getMovieTime();
				int glsFieldEntityEndTime = glsFieldEntity.getEndTimeInfo().getMovieTime();
				
				if (time == glsFieldEntityStartTime && glsFieldEntityStartTime == glsFieldEntityEndTime) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.CHAINS_CANNOT_OVERLAP);
					return;
				}
			}
			
			EnteredText enteredText = glsField
					.getGlossChainedEventsEntityCollection().getEnteredText();

			ArrayList<Text> selectedText = enteredText.getSelectedText();

			if (selectedText.isEmpty()) {
				if (event == null)
				{
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_NOT_SELECTED);
					return;
				}
				
				MediaToolbarCommons.setSelectedEventStartTime(bar, time);
			} else if (parentEntity != null) {
				int utteranceStartTime = parentEntity.getStartTimeInfo()
						.getMovieTime();
				int utteranceEndTime = parentEntity.getEndTimeInfo()
						.getMovieTime();
				
				if (time < utteranceStartTime && time > utteranceEndTime) {
					SS3Singleton.getErrorMessages().showErrorMessage(
							ErrorMessages.EVENT_EXTEND_BEYOND_UTTERANCE);
					return;
				}

				GlossChainedEventsEntity glsEntity = (GlossChainedEventsEntity) entity;
				MediaToolbarCommons.setTextStartEndTime(glsField, glsEntity,
						parentEntity, time, coord, bar);
			}
		} else if (field instanceof PresentationField
				|| event instanceof PresentationEventsEntity) {
			if (MediaToolbarCommons.isUpdatablePresentationEvent(entity)) {
				JTextField txtField = bar.getStartTimeTextField();
				MediaToolbarCommons.setPresentationEventStartTime(time, event,
						entity, field, panel, txtField);
			}
		} else {
			MediaToolbarCommons.setSelectedEventStartTime(bar, time);
		}
		
		panel.repaint();
	}
}
