package edu.bu.signstream.media;

import java.net.URI;
import java.time.Duration;
import java.util.List;
import java.util.function.Consumer;

public interface MediaPlayer {
    // Media
    List<Media> media();
    List<Video> videos();
    List<Audio> audio();

    Video addVideo(URI uri);
    Audio addAudio(URI uri);
    void remove(Media m);
    void offset(Media m, long offset);

    // State
    void prepare();
    boolean prepared();

    // Controls
    void play();
    void pause();
    boolean playing();
    boolean stopped();

    void rate(double rate);
    double rate();

    void seek(long pos);
    void seek(long pos, long end);

    // Time
    long pos();
    Duration duration();

    // Callbacks
    void addPosListener(Consumer<Long> fn);
    void removePosListener(Consumer<Long> fn);
    void addStateListener(Consumer<Boolean> fn);
    void removeStateListener(Consumer<Boolean> fn);
    void addRateListener(Consumer<Double> fn);
    void removeRateListener(Consumer<Double> fn);

    // Cleanup
    void close();
}
