package edu.bu.signstream.media.gstreamer;


import edu.bu.signstream.media.Audio;
import edu.bu.signstream.media.ui.AudioView;
import org.freedesktop.gstreamer.Element;
import org.freedesktop.gstreamer.ElementFactory;
import org.freedesktop.gstreamer.Pad;
import org.freedesktop.gstreamer.PadLinkException;

import java.net.URI;
import java.time.Duration;

public class GstAudio implements GstMedia, Audio {
    private static final String FORMAT = "audio/x-raw";
    private static final String ID_PREFIX = "A";

    private static int ID = 0;

    private final URI uri;

    private int rate;
    private Duration duration;

    private final GstMediaPlayer player;

    private final Element decode;
    private final Element convert;
    private final Element resample;
    private final Element sink;

    public GstAudio(URI uri, GstMediaPlayer player) {
        this.player = player;
        this.uri = uri;

        var id = ID_PREFIX + ID;

        System.out.println("Probing audio " + uri.toString());
        GstProbe.probe(uri, FORMAT, (caps, duration) -> {
            this.duration = duration;
            rate = caps.getInteger("rate");
        });
        System.out.println("Probing finished!");

        // Set up the elements for the real pipeline
        decode = ElementFactory.make("uridecodebin", "decode" + id);
        decode.set("uri", uri.toString());
        decode.connect(new Element.PAD_ADDED() {
            @Override
            public void padAdded(Element e, Pad pad) {
				System.out.println(uri + ": AUDIO " + pad.getName() + " [" + pad.getTypeName() + "]");
				var caps = pad.getCurrentCaps().getStructure(0);
                System.out.println(caps);

                if (caps.getName().equals(FORMAT)) {
                    try {
                        System.out.println("LINKED AUDIO");
                        pad.link(convert.getStaticPad("sink"));
                    } catch (PadLinkException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        });
        convert = ElementFactory.make("audioconvert", "convert" + id);
        resample = ElementFactory.make("audioresample", "resample" + id);
        sink = ElementFactory.make("autoaudiosink", "sink" + id);

        player.pipeline.addMany(decode, convert, resample, sink);
        convert.link(resample);
        resample.link(sink);

        System.out.println("Done loading audio " + id);

        ID++;
    }

    @Override
    public URI uri() {
        return uri;
    }

    @Override
    public int rate() {
        return rate;
    }

    @Override
    public Duration duration() {
        return duration;
    }

    @Override
    public AudioView view() {
        return new AudioView(player, this);
    }

    @Override
    public Element sink() {
        return sink;
    }

    @Override
    public void close() {
        player.pipeline.remove(decode);
        player.pipeline.remove(convert);
        player.pipeline.remove(resample);
        player.pipeline.remove(sink);
    }


}
