package edu.bu.signstream.media.gstreamer;

import org.freedesktop.gstreamer.*;
import org.freedesktop.gstreamer.query.DurationQuery;

import java.net.URI;
import java.time.Duration;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import java.util.function.BiConsumer;

public class GstProbe {
    private final CountDownLatch latch = new CountDownLatch(2);
    private final Pipeline pipeline = new Pipeline();

    protected Structure structure;
    protected Duration duration;

    public static void probe(URI uri, String format, BiConsumer<Structure, Duration> fn) {
        new GstProbe(uri, format).await(fn);
    }

    private GstProbe(URI uri, String format) {
        var src = ElementFactory.make("uridecodebin", "decode");
        src.set("uri", uri.toString());

        src.connect(new Element.PAD_ADDED() {
            @Override
            public void padAdded(Element element, Pad pad) {
                var s = pad.getCurrentCaps().getStructure(0);
                System.out.println("PAD: " + pad);
                System.out.println("CAPS: " + s);
                if (s.getName().equals(format)) {
                    structure = s;
                    src.disconnect(this);
                    latch.countDown();
                }
            }
        });
        pipeline.getBus().connect(new Bus.ASYNC_DONE() {
            @Override
            public void asyncDone(GstObject source) {
                var query = new DurationQuery(Format.TIME);
                src.query(query);
                duration = Duration.ofNanos(query.getDuration());

                pipeline.getBus().disconnect(this);
                latch.countDown();
            }
        });

        pipeline.getBus().connect((Bus.INFO) (arg0, arg1, arg2) ->
            System.err.println("<probe:info> " + arg0.getName() + " [" + arg1 + "]: " + arg2));
        pipeline.getBus().connect((Bus.ERROR) (arg0, arg1, arg2) ->
            System.err.println("<probe:error> " + arg0.getName() + " [" + arg1 + "]: " + arg2));

        pipeline.add(src);
    }

    private void await(BiConsumer<Structure, Duration> fn) {
        pipeline.play();

        try {
            if (!latch.await(5, TimeUnit.SECONDS)) {
                throw new RuntimeException("Failed to find a compatible media stream!");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
            System.out.println("<probe> Interrupted!");
        }

        pipeline.stop();
        pipeline.close();

        fn.accept(structure, duration);
    }
}
