package edu.bu.signstream.navigation;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.swing.ButtonGroup;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.UIManager;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.Palette;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class DbNavigationPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	private JTree tree = new JTree();
	private DatabaseDelegate dbDelegate = null;
	private SignStreamSegmentPanel segmentPanel = null;
	
	public DbNavigationPanel(SignStreamSegmentPanel segmentPanel,
			DatabaseDelegate dbDelegate) {
		this.segmentPanel = segmentPanel;

		if (dbDelegate == null) {
			this.dbDelegate = new DatabaseDelegate(segmentPanel);
		} else {
			this.dbDelegate = dbDelegate;
		}

		try {
			DefaultMutableTreeNode treeNode = dbDelegate.getMutableTreeNode();
			this.tree = new JTree(treeNode);

			initTreeUI();
			setTree();
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}
	}

	private void initTreeUI() {
		Color treeColor = new Color(255, 247, 200);

		int fontSize = 12, fontStyle = Font.PLAIN;
		String fontFamily = "Times-Roman";
		Font font = new Font(fontFamily, fontStyle, fontSize);

		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setLeafIcon(null);
		renderer.setIcon(null);
		renderer.setClosedIcon(null);
		renderer.setOpenIcon(null);

		renderer.setBackground(treeColor);
		renderer.setBackgroundNonSelectionColor(treeColor);
		renderer.setBackgroundSelectionColor(treeColor);
		renderer.setBorderSelectionColor(treeColor);
		renderer.setTextSelectionColor(Color.RED);
		renderer.setFont(font);
		
		tree.getInputMap().put(KeyStroke.getKeyStroke("LEFT"), "none");
		tree.getInputMap().put(KeyStroke.getKeyStroke("RIGHT"), "none");

		tree.addKeyListener(SS3Singleton.getVideoControlKeyListener());		
		tree.setCellRenderer(renderer);
		tree.setBackground(treeColor);
		tree.setForeground(treeColor);
		tree.putClientProperty("JTree.lineStyle", "Horizontal");

		UIManager.put("Tree.line", Palette.AXIS_COLOR);
	}

	public DbNavigationPanel(TreeModel model) {
		this.tree = new JTree(model);
		this.dbDelegate = new DatabaseDelegate(segmentPanel);
		initTreeUI();

		try {
			setTree();
		} catch (Exception ex) {
			SS3SignStreamApplication.logger.log(Level.INFO, ex.getMessage());
		}
	}

	public void setSignStreamSegmentPanel(SignStreamSegmentPanel segmentPanel) {
		this.segmentPanel = segmentPanel;
	}

	public DatabaseDelegate getDatabaseDelegate() {
		return dbDelegate;
	}

	public JTree getJTree() {
		return tree;
	}

	public SignStreamSegmentPanel getSignStreamSegmentPanel() {
		return segmentPanel;
	}

	public void refresh(TreeNode treeNode) {
		tree = new JTree(treeNode);
		initTreeUI();
		setTree();
		updateUI();
	}

	public void setTree() {
		removeAll();

		JScrollPane hashPane = new JScrollPane(tree);
		hashPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		tree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);

		tree.addMouseListener(new TreeMouseListener(this));

		tree.addTreeSelectionListener(new TreeSelectionListener() {
			@Override
			public void valueChanged(TreeSelectionEvent e) {
			}
		});

		tree.expandPath(new TreePath(tree.getModel().getRoot()));

		setLayout(new BorderLayout());

		add(hashPane, BorderLayout.CENTER);
	}
}

class Perspective {
	// Construct the panel with the toggle buttons.

	public Perspective() {
		JRadioButton showDescendant = new JRadioButton("segmentsnts", true);
		JRadioButton showAncestor = new JRadioButton("utterances");

		ButtonGroup bGroup = new ButtonGroup();
		bGroup.add(showDescendant);
		bGroup.add(showAncestor);

		JPanel buttonPanel = new JPanel();
		buttonPanel.add(showDescendant);
		buttonPanel.add(showAncestor);
	}
}
