package edu.bu.signstream.ui;

import java.io.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.awt.*;
import java.awt.desktop.QuitEvent;
import java.awt.desktop.QuitHandler;
import java.awt.desktop.QuitResponse;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.swing.*;
import javax.swing.filechooser.FileView;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import edu.bu.signstream.common.EnterKeyListener;
import edu.bu.signstream.common.db.parser.SS3Parser;
import edu.bu.signstream.common.files.FilesLocation;
import edu.bu.signstream.common.files.SS3Resources;
import edu.bu.signstream.common.util.ErrorMessages;
import edu.bu.signstream.common.util.ImageMaker;
import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.FieldWrappersCollection;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.MorphologicalPhonologicalInfoDialog;
import edu.bu.signstream.grepresentation.fields.presentation.*;
import edu.bu.signstream.grepresentation.fields.presentation.dialog.RenameSegmentDialog;
import edu.bu.signstream.grepresentation.fields.presentation.dialog.RenameTemporalPartitionDialog;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.grepresentation.view.signbank.SignBankCollection;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.navigation.DatabaseDelegate;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.panels.AnnotatorSelectionDialog;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnit;
import edu.bu.signstream.ui.panels.newDatabase.CreateUpdateMacroUnitDialog;
import edu.bu.signstream.ui.panels.search.SearchDialog;
import edu.bu.signstream.ui.video.XugglerMovie;

/**
 * DO not use JOptionPane as it disables MenuItems on Mac.
 * */
public class SS3SignStreamApplication extends JFrame implements ActionListener,
		KeyListener {

	private static final long serialVersionUID = 1L; 

	public static Logger logger = Logger.getLogger("edu.bu.signstream.ui");

	public final int NEW_COLLECTION = 0;
	public final int CLOSE_DB = 3;
	public final int SAVE_DB = 4;
	private final int SAVE_AS_DB = 5;
	public final int OPEN_AS_SS3 = 6;
	private final int NEW_SEARCH = 50;
	private final int SAVED_SEARCH = 51;

	private final int NEW_MACRO_UNIT = 7;
	private final int DELETE_MACRO_UNIT = 8;
	private final int RENAME_MACRO_UNIT = 9;

	private final int CREATE_SEGMENT = 10;
	private final int RENAME_SEGMENT = 11;
	private final int RENAME_PARTICIPANT = 12;
	private final int DELETE_SEGMENT = 13;
	private final int SET_DEFAULT_SEGMENT = 14;
	
	private final int CREATE_TEMPORAL_PARTITION = 20;
	private final int RENAME_TEMPORAL_PARTITION = 21;
	private final int DELETE_TEMPORAL_PARTITION = 22;
	private final int RENUMBER_UTTERANCES = 23;
	
	private final int HELP = 100;
	private final int UPDATE = 150;
	private final int UPDATE_SIGN_BANK = 151;
	private final int INSTALL_SIGN_BANK = 152;
	private final int UNINSTALL_SIGN_BANK = 153;

	private final int EXIT = 30;

	private SS3Parser parser = null;
	private DatabaseDelegate dbDelegate = null;
	private SignStreamSegmentPanel panel = null;
	private SearchDialog searchDialog = null;
	
	private SignBankCollection signBankCollection = null;

	public static String path = "newXMLFiles";
	public static String globalParticipantsFileName = "globalParticipants.xml";
	public static String localParticipantsFileName = "localParticipants.xml";
	public static String globalAnnotatorsFileName = "globalAnnotators.xml";
	public static String localAnnotatorsFileName = "localAnnotators.xml";
	public static String mediaLibraryFileName = "mediaLibrary.xml";

	private String scPath = null;

	JMenu fileMenu = new JMenu("Collection");
	JMenuItem newItem = new JMenuItem(" New Blank Collection ");
	JMenuItem openAsItem = new JMenuItem(" Open...");
	JMenuItem openLegacyItem = new JMenuItem(" Open Legacy...");
	JMenuItem closeItem = new JMenuItem(" Close ");
	JMenuItem saveItem = new JMenuItem(" Save ");
	JMenuItem saveAsItem = new JMenuItem(" Save As... ");
	JMenuItem newSearch = new JMenuItem("New Search");
	JMenuItem savedSearch = new JMenuItem("Open Saved Search");

	private CreateUpdateMacroUnitDialog macroUnitDialog = null;
	private SaveAndOpenCollectionDialog saveCollectionANdOpenCollectionDialog = null;
	
	public static int SAVE = 4;
	public static int SAVE_AS = 5;
	
	private final String BASH_UPDATE_SCRIPT = SS3Resources.rootPath + "/Update.command";
	private final String BASH_SIGN_BANK_SCRIPT = SS3Resources.rootPath + "/GetSignBank.command";
	private final int BASH_EXIT_CODE_YES = 0;
	private final int BASH_EXIT_CODE_NO = 1;

    public static void main(String[] args) {
        var log = new LogStream(System.out);
        System.setOut(log);
        System.setErr(log);

        SwingUtilities.invokeLater(new Runnable() {
            @Override
            public void run() {
                SS3SignStreamApplication t = new SS3SignStreamApplication();

                t.setSize(1350, 640);
                t.setVisible(true);
                t.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
            }
        });
    }

    private static class LogStream extends PrintStream {
        private final OutputStream log;

        public LogStream(OutputStream out) {
            super(out);

            try {
                log = new FileOutputStream(new File(SS3Resources.rootPath + "/SignStreamOut.txt"));
            } catch (FileNotFoundException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public void write(byte[] buf, int off, int len) {
            super.write(buf, off, len);
            try {
                log.write(buf, off, len);
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }

        @Override
        public void close() {
            super.close();
            try {
                log.close();
            } catch (IOException e) {
                throw new RuntimeException(e);
            }
        }
    }

	public SS3SignStreamApplication() {
        Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
			@Override
			public void uncaughtException(Thread t, Throwable e) {
				logger.log(Level.SEVERE, "Uncaught Exception", e);
			}
		});

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception ex) {
			;
			logger.log(Level.INFO, "Could not set system look and feel", ex);
		}

		System.setProperty("apple.laf.useScreenMenuBar", "true");

		"com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName());

		System.setProperty("com.apple.eawt.CocoaComponent.CompatibilityMode",
				"false");

		System.setProperty("jna.library.path", SS3Resources.libPath);
		System.setProperty("jna.debug_load", "true");

		KeyStroke nextFrameKeyStroke = KeyStroke.getKeyStroke(39, 0);
		KeyStroke prevFrameKeyStroke = KeyStroke.getKeyStroke(37, 0);	
		KeyStroke prevUtteranceKeyStroke = KeyStroke.getKeyStroke(38, 0);	
		KeyStroke nextUtteranceKeyStroke = KeyStroke.getKeyStroke(40, 0);	

		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				nextFrameKeyStroke, "nextFrame");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				prevFrameKeyStroke, "prevFrame");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				nextUtteranceKeyStroke, "nextUtterance");
		
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(
				prevUtteranceKeyStroke, "prevUtterance");
	
		getRootPane().getActionMap().put("nextFrame", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				SS3Singleton.getMediaToolBar(null).currentFrameForward();			}
		});
		
		getRootPane().getActionMap().put("prevFrame", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				SS3Singleton.getMediaToolBar(null).currentFrameBack();
			}
		});
		
		getRootPane().getActionMap().put("nextUtterance", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (getSignStreamSegmentPanel().isUtteranceLoaded()) {
					panel.getUtteranceView().getSelectedSegmentTemplate().getSegmentControlPanel().incrementSelectedUtterance(1);
				}
			}
		});
		
		getRootPane().getActionMap().put("prevUtterance", new AbstractAction() {
			@Override
			public void actionPerformed(java.awt.event.ActionEvent e) {
				if (getSignStreamSegmentPanel().isUtteranceLoaded()) {
					panel.getUtteranceView().getSelectedSegmentTemplate().getSegmentControlPanel().incrementSelectedUtterance(-1);
				}
			}
		});

		deleteTempFiles();

		try {
			FileHandler fh = new FileHandler(SS3Resources.rootPath + "/SignStreamLog.txt");
			logger.addHandler(fh);
		} catch (Exception ex) {
			ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
		}

		Toolkit.getDefaultToolkit().addAWTEventListener( new AWTEventListener() {
		    @Override
			public void eventDispatched(AWTEvent e) {
		        if (e.getID() == KeyEvent.KEY_PRESSED) {
                    if (((KeyEvent) e).getKeyCode() == KeyEvent.VK_ESCAPE) {
                        Component comp = (Component) e.getSource();
                        while (!(comp.getParent() == null || comp instanceof Window)) {
                            comp = comp.getParent();
                        }
                        if (comp instanceof Window) {
                            if (!Constants.importantDialogs.contains(comp.getClass()) &&
                                    (!(comp instanceof JDialog && ((JDialog)comp).isModal()))) {
                                comp.dispatchEvent(new WindowEvent((Window) comp, WindowEvent.WINDOW_CLOSING));
                            }
                        }
                    }
                }
		    }
        }, AWTEvent.KEY_EVENT_MASK);

		KeyboardFocusManager.getCurrentKeyboardFocusManager()
				.addPropertyChangeListener("permanentFocusOwner", new PropertyChangeListener() {
			@Override
			public void propertyChange(final PropertyChangeEvent e){
				if (e.getNewValue() instanceof JButton) {
					JButton btn = (JButton) e.getNewValue();
					for (KeyListener keyListener : btn.getKeyListeners()) {
						if (keyListener instanceof EnterKeyListener) {
							return;
						}
					}
					btn.addKeyListener(new EnterKeyListener(btn));
				}
			}
		});

		logger.log(Level.INFO, "Starting the Sign Stream application... ");

		String dv = System.getProperty("file.separator", "/");
		path = SS3Resources.xmlPath + dv;
		scPath = path + "defCodingScheme.xml";

		this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);

		parser = new SS3Parser();

		SS3Database ss3db = new SS3Database(new TimeInfo(), new TimeInfo());
		ss3db.setSS3GlossWindow(createNewGlossWindow());
		Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
				ss3db.getFileName());
		ss3db.setExcerptObj(excerpt);
		
		// Begin: Startup Sign Bank check
		// On startup, check if we are running the latest sign bank file version
		boolean menuInstallSignBank = false; // set to 'uninstall' by default, true means 'install'
		Process proc = null;
		int ret = BASH_EXIT_CODE_NO;
		
		try {
			proc = Runtime.getRuntime().exec("/bin/bash " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			ret = proc.waitFor();	
		} catch (Exception e) {
			System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Exception: " + e.getMessage());
			
			menuInstallSignBank = true;
		}

		if (ret == BASH_EXIT_CODE_YES) {	// sign bank is from ASLLRP
			ret = BASH_EXIT_CODE_NO;
			try {	
				proc = Runtime.getRuntime().exec("/bin/bash " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				ret = proc.waitFor();
			} catch (Exception e) {
				System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
				
				menuInstallSignBank = true;
			}
		} else if (ret == BASH_EXIT_CODE_NO) {	// sign bank is not from ASLLRP
			menuInstallSignBank = true;
		}

		if (ret == BASH_EXIT_CODE_YES) {	// if we are at a 'yes' here it means to do an update of the sign bank
			try {				
				JFrame dummyFrame = new JFrame();

				dummyFrame.setVisible(false);		// false because we don't want an extra frame in space
				dummyFrame.setLocationRelativeTo(null);		// centers the panel
				dummyFrame.setAlwaysOnTop(true);
				dummyFrame.toFront();

				String warningDialogText = "SignStream® will launch momentarily."
						+ "\n\nThe ASLLRP Sign Bank first needs to be updated (or deleted). "
						+ "\nPlease follow the steps in the Terminal window to do that."
						+ "\n\nThanks very much!";
				String warningTitleText = "Sign Bank Check";
				int optionType = JOptionPane.YES_NO_OPTION;
				int messageType = JOptionPane.WARNING_MESSAGE;
				String options[] = {"OK"};
				try {
					JOptionPane.showOptionDialog(dummyFrame,
							warningDialogText,
							warningTitleText,
							optionType,
							messageType,
							null,
							options,
							options[0]);
				} catch (HeadlessException he) {
					System.out.println("");
				}
				
				String currentDir = System.getProperty("user.dir");
				String command1 = 
						"tell application \"Terminal\" to do script \"" + BASH_SIGN_BANK_SCRIPT + "; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};

				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging	
				Thread.sleep(1000);		// sleep for 1 second, give time for script to detect the signstream is running
				p.start().waitFor();
			} catch (Exception e) {
				System.out.println("'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
				
				menuInstallSignBank = true;
			}
		}
		
		// test again to make sure the process went through and the user did not close the Terminal window
		ret = BASH_EXIT_CODE_NO;
		
		try {
			proc = Runtime.getRuntime().exec("/bin/bash " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			ret = proc.waitFor();	
		} catch (Exception e) {
			System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_asllrp");
			logger.log(Level.INFO, "Exception: " + e.getMessage());
		}

		if (ret == BASH_EXIT_CODE_YES) {	// sign bank is from ASLLRP
			ret = BASH_EXIT_CODE_NO;
			try {	
				proc = Runtime.getRuntime().exec("/bin/bash " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				ret = proc.waitFor();

				// if we are at a 'yes' here on this second pass, it means the user closed the Terminal window in the first pass, so now close SignStream
				if (ret == BASH_EXIT_CODE_YES) {					
					System.exit(0);
				}		
			} catch (Exception e) {
				System.out.println("Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Error running " + BASH_SIGN_BANK_SCRIPT + " --check_outdated");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
		}
		// End: Startup Sign Bank check
		
		// At this point, we have either an empty global sign bank file, or an up-to-date global sign bank file
		
		// parse sign bank collection on startup as well
		parseSignBank();
		
		MenuItemsCollection menuItemsCollection = new MenuItemsCollection();
		JMenuBar mb = new JMenuBar();
		JMenu fileMenu = createFileMenu(menuItemsCollection);
		mb.add(fileMenu);

		JMenu muMenu = new JMenu("Macro Unit");
		JMenuItem newItem = new JMenuItem(" New Macro Unit ");
		JMenuItem renameItem = new JMenuItem(" Edit Macro Unit ");
				
		JMenu helpMenu = new JMenu("Help");
		JMenuItem helpItem = new JMenuItem(" Help... ");
		JMenuItem updateItem = new JMenuItem("Update SignStream...");
		
		// logic for what to display in menu for sign bank
		JMenuItem updateSignBankItem = null;
		if (menuInstallSignBank) {
			updateSignBankItem = new JMenuItem("Install ASLLRP Sign Bank...");
			updateSignBankItem.setActionCommand(""+INSTALL_SIGN_BANK);
		} else {
			updateSignBankItem = new JMenuItem("Uninstall ASLLRP Sign Bank...");
			updateSignBankItem.setActionCommand(""+UNINSTALL_SIGN_BANK);
		}
		
		//helpMenu.add(helpItem);
		helpItem.setActionCommand("" + HELP);
		helpItem.addActionListener(this);
		helpMenu.add(updateItem);
		updateItem.setActionCommand("" + UPDATE);
		updateItem.addActionListener(this);
		helpMenu.add(updateSignBankItem);
		updateSignBankItem.addActionListener(this);
		updateSignBankItem.setEnabled(true);
		
		menuItemsCollection.setMacroUnitMenu(fileMenu);
		menuItemsCollection.setNewMacroUnitItem(newItem);
		menuItemsCollection.setEditMacroUnitItem(renameItem);
		muMenu.add(renameItem);

		newItem.setActionCommand("" + NEW_MACRO_UNIT);
		renameItem.setActionCommand("" + RENAME_MACRO_UNIT);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		
		muMenu.addSeparator();
		
		JMenu sMenu = createSegmentMenu(menuItemsCollection);
		muMenu.add(sMenu);

		muMenu.addSeparator();
		
		JMenu tpMenu = createTemporalPartitionMenu(menuItemsCollection);
		muMenu.add(tpMenu);
			
		mb.add(muMenu);
		
		mb.add(helpMenu);

		setJMenuBar(mb);
		
		//If user is on Mac, add listener to ask user to save before quitting (via Command + Q or SignStream -> Quit)
		if (System.getProperty("os.name").toLowerCase().contains("mac")) {
			addQuitHandler();
		}

		panel = new SignStreamSegmentPanel(null, this); //SignStreamSegmentPanel(excerpt, ss3db, null, this);
		dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();

		loadNewMacroUnitDialog(ss3db);

		logger.log(Level.INFO, "Starting the Sign Stream application... 1");

		SignStreamWindowListener windowListener = new SignStreamWindowListener(this);
		addWindowListener(windowListener);		// handles save/dont-save on clicking 'x' to close window

		logger.log(Level.INFO, "Starting the Sign Stream application... 2");

		logger.log(Level.INFO, "Starting the Sign Stream application... 3");

		SS3Singleton.loadSignStreamApplication(this);

		logger.log(Level.INFO, "Starting the Sign Stream application... 4");

		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(panel, BorderLayout.CENTER);
		
		logger.log(Level.INFO,
				"The Sign Stream application started successfully");
	}

    @Override
    public boolean isFocusable() {
        return true;
    }

    /**
     * Due to a recurrent problem on OSX applications, JPopupMenu objects do not disappear when
     * the user switches to another application, and remains visible until the user switches back to the
     * Signstream application and clicks away from the JPopupMenu, which closes it.
     * This method is meant to be called by any client class in Signstream which uses a JPopupMenu object
     * and is experiencing this issue - here it is forced to invisible when the window focus switches.
     * @param popup The JPopupMenu which requires a forced setVisible(false) call on window focus switch
     */
    public void forcePopupInvisibleOnWindowLostFocus(final JPopupMenu popup) {
        final JFrame frame = this;	// need a final reference to this SS3SignStreamApplication object for the inner class below
        frame.addWindowFocusListener(new WindowFocusListener() {
            @Override public void windowGainedFocus(WindowEvent arg0) {}
            @Override public void windowLostFocus(WindowEvent arg0) {
                popup.setVisible(false);
                frame.removeWindowFocusListener(this);
            }
        });

    }

    public SignStreamSegmentPanel getSignStreamSegmentPanel() {
        return panel;
    }

    public DatabaseDelegate getDatabaseDelegate() {
        return dbDelegate;
    }

    public CreateUpdateMacroUnitDialog getMacroUnitDialog() {
        return macroUnitDialog;
    }

    public SS3CodingScheme getDefaultSS3CodingScheme() {
		return parser.parseSS3CodingScheme(scPath);
	}

	private void loadNewMacroUnitDialog(SS3Database ss3db) {
		Point location = new Point(300, 100);

		if (macroUnitDialog != null) {
			location = macroUnitDialog.getLocation();
			macroUnitDialog.dispose();
		}

		Calendar rightNow = Calendar.getInstance();
		int dayOfMonth = rightNow.get(Calendar.DAY_OF_MONTH);
		int month = rightNow.get(Calendar.MONTH) + 1;
		int year = rightNow.get(Calendar.YEAR);
		int timeHrs = rightNow.get(Calendar.HOUR_OF_DAY);
		int timeMin = rightNow.get(Calendar.MINUTE);
		int timeSec = rightNow.get(Calendar.SECOND);

		String defCollectionCreatedOnLoad = "Collection_" + month + "_"
				+ dayOfMonth + "_" + year + "_" + timeHrs + "_" + timeMin + "_"
				+ timeSec;

		setTitle(defCollectionCreatedOnLoad);
		
		SS3Singleton.getFilesLocation().setLastOpenedDatabaseLocation(
				FilesLocation.defaultDatabaseLocation
				+ File.separator
				+ defCollectionCreatedOnLoad
				+ SS3Database.dbFileExtension);
		
		DbNavigationPanel navigation = panel.getDbNavigationPanel();
		navigation.getDatabaseDelegate().addNewDatabase(
				defCollectionCreatedOnLoad);
		navigation.refresh(navigation.getDatabaseDelegate()
				.getMutableTreeNode());
		navigation.getJTree().repaint();

		String globalParticipantsFileName = path + "globalParticipants.xml";
		String localParticipantsFileName = path + "localParticipants.xml";
		String globalAnnotatorsFileName = path + "globalAnnotators.xml";
		String localAnnotatorsFileName = path + "localAnnotators.xml";
		String mediaLibraryFileName = path + "mediaLibrary.xml";
		
		SS3Singleton.getManager().loadParticipantsLibrary(globalParticipantsFileName, localParticipantsFileName);
		SS3Singleton.getManager().loadAnnotatorsLibrary(globalAnnotatorsFileName, localAnnotatorsFileName);	
		
		ParticipantsLibrary participantsLibrary = SS3Singleton.getManager().getLoadedParticipantsLibrary();
		ParticipantsLibrary annotatorsLibrary = SS3Singleton.getManager().getLoadedAnnotatorsLibrary();

		SS3MediaLibrary mLibrary = parser
				.parseMediaLibrary(mediaLibraryFileName);

		if(macroUnitDialog == null) {
			macroUnitDialog = new CreateUpdateMacroUnitDialog(
					CreateUpdateMacroUnit.FUNCTION_ADD, ss3db,
					ss3db.getExcerptObj(), new ArrayList<LoadedMediaFile>(),
					mLibrary, participantsLibrary, annotatorsLibrary,
					signBankCollection.getSignBankResourceCollection(),
					panel, this);
		} else {
			macroUnitDialog.update(
					CreateUpdateMacroUnit.FUNCTION_ADD, ss3db,
					ss3db.getExcerptObj(), new ArrayList<LoadedMediaFile>(),
					mLibrary, participantsLibrary, annotatorsLibrary,
					signBankCollection.getSignBankResourceCollection());
		}

		macroUnitDialog.setVisible(true);
		macroUnitDialog.setLocation(location);
	}

	@Override
	public void keyTyped(KeyEvent e) {
		displayInfo(e, "KEY TYPED: ");
	}

	/** Handle the key-pressed event from the text field. */
	@Override
	public void keyPressed(KeyEvent e) {
		displayInfo(e, "KEY PRESSED: ");
	}

	/** Handle the key-released event from the text field. */
	@Override
	public void keyReleased(KeyEvent e) {
		displayInfo(e, "KEY RELEASED: ");
	}

	private void displayInfo(KeyEvent e, String keyStatus) {
		int modifiersEx = e.getModifiersEx();
		String modString = "extended modifiers = " + modifiersEx;
		String tmpString = InputEvent.getModifiersExText(modifiersEx);
		if (tmpString.length() > 0) {
			modString += " (" + tmpString + ")";
		} else {
			modString += " (no extended modifiers)";
		}

		String actionString = "action key? ";
		if (e.isActionKey()) {
			actionString += "YES";
		} else {
			actionString += "NO";
		}

		String locationString = "key location: ";
		int location = e.getKeyLocation();
		if (location == KeyEvent.KEY_LOCATION_STANDARD) {
			locationString += "standard";
		} else if (location == KeyEvent.KEY_LOCATION_LEFT) {
			locationString += "left";
		} else if (location == KeyEvent.KEY_LOCATION_RIGHT) {
			locationString += "right";
		} else if (location == KeyEvent.KEY_LOCATION_NUMPAD) {
			locationString += "numpad";
		} else { // (location == KeyEvent.KEY_LOCATION_UNKNOWN)
			locationString += "unknown";
		}
	}

	private JMenu createSegmentMenu(MenuItemsCollection menuItemsCollection) {
		JMenu sMenu = new JMenu("Segment Tier");
		JMenuItem newItem = new JMenuItem(" New Segment Tier");
		JMenuItem renameItem = new JMenuItem(" (Re)name Segment Tier");
		JMenuItem renameParticipantItem = new JMenuItem(" (Re)name Participant");
		JMenuItem deleteItem = new JMenuItem(" Delete Segment Tier");
		JMenuItem setDefaultItem = new JMenuItem(" Set Default Segment Tier");

		menuItemsCollection.setSegmentMenu(sMenu);
		menuItemsCollection.setNewSegmentItem(newItem);
		menuItemsCollection.setRenameSegmentItem(renameItem);
		menuItemsCollection.setRenameParticipantItem(renameParticipantItem);
		menuItemsCollection.setDeleteSegmentItem(deleteItem);
		menuItemsCollection.setSetDefaultSegmentItem(setDefaultItem);

		sMenu.add(newItem);
		sMenu.add(renameItem);
		sMenu.add(renameParticipantItem);
		sMenu.add(deleteItem);
		sMenu.add(setDefaultItem);

		newItem.setActionCommand("" + CREATE_SEGMENT);
		renameItem.setActionCommand("" + RENAME_SEGMENT);
		renameParticipantItem.setActionCommand("" + RENAME_PARTICIPANT);
		deleteItem.setActionCommand("" + DELETE_SEGMENT);
		setDefaultItem.setActionCommand("" + SET_DEFAULT_SEGMENT);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		renameParticipantItem.addActionListener(this);
		deleteItem.addActionListener(this);
		setDefaultItem.addActionListener(this);

		return sMenu;
	}
	
	private JMenu createTemporalPartitionMenu(MenuItemsCollection menuItemsCollection) {
		JMenu tpMenu = new JMenu("Temporal Partition");
		JMenuItem newItem = new JMenuItem(" New Temporal Partition");
		JMenuItem renameItem = new JMenuItem(" Rename Temporal Partition");
		JMenuItem deleteItem = new JMenuItem(" Delete Temporal Partition");
		JMenuItem renumberItem = new JMenuItem(" Renumber Utterances");

		menuItemsCollection.setSegmentMenu(tpMenu);
		menuItemsCollection.setNewSegmentItem(newItem);
		menuItemsCollection.setRenameSegmentItem(renameItem);
		menuItemsCollection.setDeleteSegmentItem(deleteItem);

		tpMenu.add(newItem);
		tpMenu.add(renameItem);
		tpMenu.add(deleteItem);
		tpMenu.add(renumberItem);

		newItem.setActionCommand("" + CREATE_TEMPORAL_PARTITION);
		deleteItem.setActionCommand("" + DELETE_TEMPORAL_PARTITION);
		renameItem.setActionCommand("" + RENAME_TEMPORAL_PARTITION);
		renumberItem.setActionCommand("" + RENUMBER_UTTERANCES);

		newItem.addActionListener(this);
		renameItem.addActionListener(this);
		deleteItem.addActionListener(this);
		renumberItem.addActionListener(this);

		return tpMenu;
	}

	private JMenu createFileMenu(MenuItemsCollection menuItemsCollection) {

		menuItemsCollection.setCollectionMenu(fileMenu);
		menuItemsCollection.setNewCollectionItem(newItem);
		menuItemsCollection.setOpenCoolectionItem(openAsItem);
		menuItemsCollection.setOpenLegacyItem(openLegacyItem);
		menuItemsCollection.setCloseCollectionItem(closeItem);
		menuItemsCollection.setSaveCollectionItem(saveItem);
		menuItemsCollection.setSaveAsCollectionItem(saveAsItem);
		menuItemsCollection.setNewSearchCollectionItem(newSearch);
		menuItemsCollection.setSavedSearchCollectionItem(savedSearch);


		fileMenu.add(newItem);
		fileMenu.add(openAsItem);
		fileMenu.addSeparator();
		fileMenu.add(closeItem);
		fileMenu.add(saveItem);

		saveItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S,
                																			java.awt.Event.META_MASK));
		fileMenu.add(saveAsItem);
		fileMenu.addSeparator();
		fileMenu.add(newSearch);
		fileMenu.add(savedSearch);
		savedSearch.setEnabled(false);

		newItem.setActionCommand("" + NEW_COLLECTION);
		newItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N,
																						java.awt.Event.META_MASK));
		
		openAsItem.setActionCommand("" + OPEN_AS_SS3);

        openAsItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O,
                																				java.awt.Event.META_MASK));
		closeItem.setActionCommand("" + CLOSE_DB);
		closeItem.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W,
																						java.awt.Event.META_MASK));
		saveItem.setActionCommand("" + SAVE_DB);
		saveAsItem.setActionCommand("" + SAVE_AS_DB);
		newSearch.setActionCommand("" + NEW_SEARCH);
		savedSearch.setActionCommand("" + SAVED_SEARCH);

		newItem.addActionListener(this);
		openAsItem.addActionListener(this);
		closeItem.addActionListener(this);
		saveItem.addActionListener(this);
		saveAsItem.addActionListener(this);
		newSearch.addActionListener(this);
		savedSearch.addActionListener(this);

		return fileMenu;
	}

	private SS3GlossWindow createNewGlossWindow() {
		SS3GlossWindow glsWindow = new SS3GlossWindow();

		SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
		displayTimePeriod.setLabel("Display Time Period");
		glsWindow.setDisplayTimePeriod(displayTimePeriod);

		SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
		temporalPartition.setLabel("Temporal Partition 1");

		SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
		tpSegment.setLabel("Segment Tier 1");
		temporalPartition.addTemporalPartitionSegment(tpSegment);
		temporalPartition.setDefaultSegment(tpSegment);

		ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
		partitions.add(temporalPartition);
		glsWindow.setTemporalPartitions(partitions);

		return glsWindow;
	}

	public File openZipDatabase() {

		File selectedFile = null;
		FilesLocation location = SS3Singleton.getFilesLocation();

		if ("com.sun.java.swing.plaf.windows.WindowsLookAndFeel"
				.equalsIgnoreCase(UIManager.getSystemLookAndFeelClassName())) {
			JFileChooser fc = new JFileChooser();
			fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES); // .DIRECTORIES_ONLY);
																			// //
																			// .FILES_AND_DIRECTORIES);
			String dbLocation = location.getLastOpenedDatabaseLocation();

			fc.addChoosableFileFilter(new SignStream3FilesFilter());

			File file = new File(dbLocation);
			String absPath = file.getAbsolutePath();

			fc.setSelectedFile(new File(absPath));
			fc.setFileView(new ImageFileView());

			int returnVal = fc.showOpenDialog(this);

			if (returnVal == JFileChooser.APPROVE_OPTION) {
				selectedFile = fc.getSelectedFile();

				logger.log(Level.INFO, "Opening [" + selectedFile.getName()
						+ "]");
			} else {
				logger.log(Level.INFO, "Open command cancelled by user.");
			}
		} else {

			FileDialog fd = new FileDialog(this, "Open a Collection",
					FileDialog.LOAD);
			String dbLocation = location.getLastOpenedDatabaseLocation();
			File file = new File(dbLocation);
			String absPath = file.getAbsolutePath();
			fd.setFilenameFilter(new ZipFormatFilenameFilter());
			fd.setDirectory(absPath);
			fd.setFile(absPath);

			fd.setVisible(true);

			System.setProperty("apple.awt.fileDialogForDirectories", "false");

			if (fd.getFile() != null) {
				String selectedFileName = fd.getDirectory() + File.separator
						+ fd.getFile();
				selectedFile = new File(selectedFileName);
			}
		}
		return selectedFile;
	}

	private void saveChangesToTheCollection(final boolean loadNewDatabase) {
		final JTree tree = panel.getDbNavigationPanel().getJTree();

		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();

		if (SS3Singleton.getApplicationStateController()
				.isLoadedCollectionUpdated()) {

			final JDialog newCollectionDialog = new JDialog(this,
					"Save Resource", true);
			JButton dontSaveBtn = new JButton("Don't Save");
			JButton cancelBtn = new JButton("Cancel");
			JButton saveBtn = new JButton("Save");

			dontSaveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					SS3DatabaseCollection dbCol = SS3Singleton
							.getSS3DatabaseCollection();
					dbCol.removeAll();
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

					panel.closeAllLoadedVideoFiles();

					rootNode.removeAllChildren();
					((DefaultTreeModel) tree.getModel()).reload(rootNode);

					SS3Singleton.getSS3DatabaseCollection().removeAll();

					if (loadNewDatabase)
						panel.loadDatabase(null);

					newCollectionDialog.dispose();
					System.exit(0);
				}
			});

			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					newCollectionDialog.dispose();
				}

			});

			saveBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					DefaultMutableTreeNode node = null;
					int childCount = rootNode.getChildCount();

					if (childCount > 0) {
						node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

						if (node.getChildCount() > 0) {
							DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
									.getChildAt(0);
							Object uo = dNode.getUserObject();
							if (uo instanceof String) {
								node = dNode;
							}
						}
					}
					
					saveDatabaseToFile(node, SS3SignStreamApplication.SAVE);
					
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

					panel.closeAllLoadedVideoFiles();

					panel.getMultipleMovieController().getMovieDuration();

					rootNode.removeAllChildren();
					((DefaultTreeModel) tree.getModel()).reload(rootNode);

					SS3Singleton.getSS3DatabaseCollection().removeAll();

					if (loadNewDatabase)
						panel.loadDatabase(null);

					newCollectionDialog.dispose();

					System.exit(0);
				}
			});

			ButtonPanel newCollectionButtonPanel = new ButtonPanel();
			newCollectionButtonPanel.add(dontSaveBtn);
			newCollectionButtonPanel.add(cancelBtn);
			newCollectionButtonPanel.add(saveBtn);

			JPanel newCollectionPanel = new JPanel();
			newCollectionPanel.setLayout(new BorderLayout());
			JLabel newCollectionLabel = new JLabel(
					" Do you want to save the changes you made to Collection?");
			newCollectionPanel.add(newCollectionLabel, BorderLayout.CENTER);
			newCollectionPanel
					.add(newCollectionButtonPanel, BorderLayout.SOUTH);

			newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

			Dimension d = newCollectionPanel.getPreferredSize();
			Dimension nD = new Dimension(d.width + 10, d.height + 40);
			newCollectionPanel.setPreferredSize(nD);

			Container contentPane = newCollectionDialog.getContentPane();
			contentPane.setLayout(new BorderLayout(40, 40));
			contentPane.add(newCollectionPanel, BorderLayout.CENTER);

			newCollectionDialog.setLocation(new Point(150, 150));
			newCollectionDialog.pack();
			newCollectionDialog.setVisible(true);
		}
	}

	/**
	 * Deals with event handling when user selects a menu option from the top menu bar.
	 */
	@Override
	public void actionPerformed(ActionEvent ae) {
		panel.updateLoadedDatabase();

		String str = ae.getActionCommand();

		final JTree tree = panel.getDbNavigationPanel().getJTree();
		dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();
		int childCount = rootNode.getChildCount();
		DefaultMutableTreeNode node = null;

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		MorphologicalPhonologicalInfoDialog dialog;

		switch (Integer.parseInt(str)) {

		case OPEN_AS_SS3:

			logger.log(Level.INFO, "Opening a collection...");

			if (macroUnitDialog != null) {
				macroUnitDialog.setVisible(false);
			}

			if (!SS3Singleton.getApplicationStateController().isLoadedCollectionUpdated()) {				
				openDatabaseSS3(tree, rootNode, node);

				for (XugglerMovie movie : SS3Singleton.getVideoControlManager().getMultipleMovieController().getMovies()) {
					movie.requestFocus();
//					movie.getVideoImage().requestFocus();
				}
			} else {
				if (saveCollectionANdOpenCollectionDialog == null) {
					saveCollectionANdOpenCollectionDialog = new SaveAndOpenCollectionDialog(this);
				}
				saveCollectionANdOpenCollectionDialog.setVisible(true);
			}

			break;

		case NEW_COLLECTION:
			if (getSignStreamSegmentPanel().getSS3Database().isEmpty()) {
				break;
			}

			panel.closeAllLoadedVideoFiles();
			rootNode.removeAllChildren();
			((DefaultTreeModel) tree.getModel()).reload(rootNode);
			SS3Singleton.getSS3DatabaseCollection().removeAll();

			panel.loadDatabase(null);

			macroUnitDialog.dispose();
			macroUnitDialog = null;

			if ((dialog = SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog()) != null) {
				dialog.discardChanges();
			}
			
			// below taken from constructor - mimicking functionality
			parser = new SS3Parser();
			
			SS3Database ss3db = new SS3Database(new TimeInfo(), new TimeInfo());
			ss3db.setSS3GlossWindow(createNewGlossWindow());
			Excerpt excerpt = new Excerpt(Util.getUniqueNumber(),
					ss3db.getFileName());
			ss3db.setExcerptObj(excerpt);
			
			panel.loadDatabase(ss3db);
			
			// parse sign bank collection on startup as well
			parseSignBank();

			loadNewMacroUnitDialog(ss3db);

			break;
		case CLOSE_DB:

			if (node == null) {
				confirmationDialog("Save Resource", "Collection not available.");
				return;
			}

			final DefaultMutableTreeNode dNode = node;

			if ((dialog = SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog()) != null) {
				dialog.discardChanges();
			}

			if (SS3Singleton.getApplicationStateController()
					.isLoadedCollectionUpdated()) {
				final JDialog newCollectionDialog = new JDialog(this,
						"Save Resource", true);
				final JButton dontSaveBtn = new JButton("Don't Save");
				final JButton cancelBtn = new JButton("Cancel");
				final JButton saveBtn = new JButton("Save");

				dontSaveBtn.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						panel.closeAllLoadedVideoFiles();
						rootNode.removeAllChildren();
						((DefaultTreeModel) tree.getModel()).reload(rootNode);
						SS3Singleton.getSS3DatabaseCollection().removeAll();
						setTitle("");
						panel.loadDatabase(null);
						newCollectionDialog.dispose();
						dontSaveBtn.removeActionListener(this);
					}
				});

				cancelBtn.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						newCollectionDialog.dispose();
						cancelBtn.removeActionListener(this);
					}
				});

				saveBtn.addActionListener(new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						saveDatabaseToFile(dNode, SS3SignStreamApplication.SAVE);

						panel.closeAllLoadedVideoFiles();
						rootNode.removeAllChildren();
						((DefaultTreeModel) tree.getModel()).reload(rootNode);
						SS3Singleton.getSS3DatabaseCollection().removeAll();

						panel.loadDatabase(null);
						newCollectionDialog.dispose();
						saveBtn.removeActionListener(this);
					}
				});

				ButtonPanel newCollectionButtonPanel = new ButtonPanel();
				newCollectionButtonPanel.add(dontSaveBtn);
				newCollectionButtonPanel.add(cancelBtn);
				newCollectionButtonPanel.add(saveBtn);

				JPanel newCollectionPanel = new JPanel();
				newCollectionPanel.setLayout(new BorderLayout());
				JLabel newCollectionLabel = new JLabel(
						" Do you want to save the changes you made to Collection?");
				newCollectionPanel.add(newCollectionLabel, BorderLayout.CENTER);
				newCollectionPanel.add(newCollectionButtonPanel,
						BorderLayout.SOUTH);

				newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

				Dimension d = newCollectionPanel.getPreferredSize();
				Dimension nD = new Dimension(d.width + 10, d.height + 40);
				newCollectionPanel.setPreferredSize(nD);

				Container contentPane = newCollectionDialog.getContentPane();
				contentPane.setLayout(new BorderLayout(40, 40));
				contentPane.add(newCollectionPanel, BorderLayout.CENTER);

				newCollectionDialog.setLocation(new Point(150, 150));
				newCollectionDialog.pack();
				newCollectionDialog.setVisible(true);
				//
			} else {
				panel.closeAllLoadedVideoFiles();
				rootNode.removeAllChildren();
				((DefaultTreeModel) tree.getModel()).reload(rootNode);
				SS3Singleton.getSS3DatabaseCollection().removeAll();

				panel.loadDatabase(null);
			}
			break;

		case SAVE_DB:
			if (node == null || !node.children().hasMoreElements()) {
				confirmationDialog("Save Resource", "Collection is not available.");
				return;
			}

			saveDatabaseToFile(node, SS3SignStreamApplication.SAVE);
			break;

		case SAVE_AS_DB:
			if (node == null || !node.children().hasMoreElements()) {
				confirmationDialog("Save Resource", "Collection not available.");
				return;
			}

			saveDatabaseToFile(node, SS3SignStreamApplication.SAVE_AS);
			break;
			
		case NEW_SEARCH:
			if (searchDialog == null || searchDialog.getSearchPanel() == null) {
				searchDialog = new SearchDialog(this);
				searchDialog.setVisible(true);
			}
			break;

		case CREATE_SEGMENT:
			logger.log(Level.INFO, "Create Segment");	
			if (node == null) {
				confirmationDialog("Create Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode 
					= panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode
					= labelViewSelectedNode.getPreviousSibling();
				int level = labelViewSelectedNode.getLevel();
				
				if (prevSiblingNode == null && level == 1) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
					return;
				}
				ArrayList<SS3Participant> participants = panel.getSS3Database().getSelectedParticipants();
				if (participants.size() == 1) {
					panel.createSegment(participants.get(0));
				}
				else {					
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							participants, 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog(), macroUnitDialog);
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.createSegment(participant);	
					
					SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				}
			}
			break;

		case RENAME_PARTICIPANT:
			logger.log(Level.INFO, "Rename Participant");	
			if (node == null) {
				confirmationDialog("Rename Participant Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
					return;
				}
				else {		
					// select participant screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedParticipants(), 
							macroUnitDialog.getMacroUnitPanel().getParticipantsDialog(), this);
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
					// get last selected item in list of participants before confirmation, then pass it along to createSegment()
					SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
					
					panel.renameParticipant(participant);
				}
			}
			break;
		
		case RENAME_SEGMENT:
			logger.log(Level.INFO, "Rename Segment");
			
			if (node == null) {
				confirmationDialog("Rename Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					RenameSegmentDialog renameDialog = new RenameSegmentDialog(panel);
					renameDialog.setLocation(150, 150);
					renameDialog.setVisible(true);
					
					if (renameDialog.getText() == null || renameDialog.getText().isEmpty())
						break;
					
					String txt = renameDialog.getText();
					
					panel.renameSegment(txt);	
					SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
				}
			}

			break;

		case DELETE_SEGMENT:
			logger.log(Level.INFO, "Delete Segment");	
			if (node == null) {
				confirmationDialog("Delete Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					DefaultMutableTreeNode parentOfSelectedNode = 
							(DefaultMutableTreeNode) panel.getDatabaseView().getLabelView().getSelectedNode().getParent();
				
					// there is only one segment tier child of this temporal partition - can't delete it
					if(parentOfSelectedNode.getChildCount() == 1)	{
						confirmationDialog("Delete Segment Tier",
								"You cannot delete the last Segment Tier of any given Temporal Partition");
					} 
					else {
						LabelObject selectedLabelObject = (LabelObject) panel.getDatabaseView().getLabelView().getSelectedNode().getUserObject();
						String label = selectedLabelObject.getLabel();
						
						String warningDialogText =
								"All annotated data for segment \""+ label + "\" will be DELETED.\n\n"
								+ "Are you absolutely sure you want to delete segment \"" + label + "\"?";
						String warningTitleText = "WARNING";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"Proceed", "Cancel"};
						
						int result = -1;
						try {
							result = JOptionPane.showOptionDialog(this, 
									warningDialogText, 
									warningTitleText, 
									optionType, 
									messageType, 
									null, 
									options, 
									options[1]);
						} catch (HeadlessException he) {
							logger.log(Level.INFO, he.getMessage());
						}
						
						if (result == JOptionPane.OK_OPTION)
							panel.deleteSegment();
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}					
				}
			}
			break;
			
		case SET_DEFAULT_SEGMENT:
			logger.log(Level.INFO, "Set Default Segment");
			if (node == null) {
				confirmationDialog("Set Default Segment Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_SEGMENT_TIER);
				}
				else {
					panel.setDefaultSegment();
				}
			}
			
			break;

		case CREATE_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Create Temporal Partition");	
			if (node == null) {
				confirmationDialog("Create Temporal Partition Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode 
					= panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode
					= labelViewSelectedNode.getPreviousSibling();
				int level = labelViewSelectedNode.getLevel();
				
				if (prevSiblingNode == null && level == 1) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_SELECT_DISPLAY_TIME_PERIOD);
					return;
				}
				else {
					ArrayList<SS3Participant> participants = panel.getSS3Database().getSelectedParticipants();
					if (participants.size() == 1) {
						panel.createTemporalPartition(participants.get(0));
					}
					else {					
						// select participant screen	
						AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
								participants, 
								macroUnitDialog.getMacroUnitPanel().getParticipantsDialog(), this);
						annotatorSelectionDialog.setLocation(250, 250);
						annotatorSelectionDialog.setVisible(true);
						
						// get last selected item in list of participants before confirmation, then pass it along to createSegment()
						SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
						
						panel.createTemporalPartition(participant);		
						
						SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}			
				}
			}
			break;
		case RENAME_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Rename Temporal Partition");
			
			if (node == null) {
				confirmationDialog("Rename Temporal Partition Warning",
						"Please select a Macro Unit and try again");
			} else {
				
				int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
				if (level != 1 && level !=2) {
					SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
				}
				else {
					DefaultMutableTreeNode labelViewSelectedNode = panel.getDatabaseView().getLabelView().getSelectedNode();
					DefaultMutableTreeNode prevSiblingNode = null;
					if(labelViewSelectedNode.getLevel() == 1)
						prevSiblingNode = labelViewSelectedNode.getPreviousSibling();
					else{
						DefaultMutableTreeNode parent = (DefaultMutableTreeNode)labelViewSelectedNode.getParent();
						prevSiblingNode = parent.getPreviousSibling();
					}
					
					if( prevSiblingNode == null )
					{
						confirmationDialog("Rename Temporal Partiton",
								"You cannot rename the Display Time Period");
					}
					else
					{
						RenameTemporalPartitionDialog renameDialog = new RenameTemporalPartitionDialog(panel);
						renameDialog.setLocation(150, 150);
						renameDialog.setVisible(true);
						
						
						if (renameDialog.getText() == null || renameDialog.getText().isEmpty())
							break;
						
						if(panel.getSS3Database().getSS3GlossWindow().getTemporalPartitionByLabel(renameDialog.getText()) != null)
						{
							confirmationDialog("Rename Temporal Partiton",
									"The name '" + renameDialog.getText() + "' has already been used.  Please choose a different name");
						}
						else{
							panel.renameTemporalPartition(renameDialog.getText());
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
						}
					}
				}
			}

			break;
		case DELETE_TEMPORAL_PARTITION:
			logger.log(Level.INFO, "Delete Temporal Partiton");	
			if (node == null) {
				confirmationDialog("Delete Temporal Partiton",
						"Please select a Macro Unit and try again");
			} else {
				
				DefaultMutableTreeNode labelViewSelectedNode = panel.getDatabaseView().getLabelView().getSelectedNode();
				DefaultMutableTreeNode prevSiblingNode = null;
				if(labelViewSelectedNode.getLevel() == 1)
					prevSiblingNode = labelViewSelectedNode.getPreviousSibling();
				else{
					DefaultMutableTreeNode parent = (DefaultMutableTreeNode)labelViewSelectedNode.getParent();
					prevSiblingNode = parent.getPreviousSibling();
				}

			
				DefaultMutableTreeNode databaseViewRootNode = (DefaultMutableTreeNode)panel.getDatabaseView().getLabelView().getSelectedNode().getRoot();
				if(databaseViewRootNode.getChildCount() <= 2 || prevSiblingNode == null)
				{
					confirmationDialog("Delete Temporal Partiton",
							"You cannot delete the last Temporatoral Partition or the Display Time Period");
				}
				else
				{	
					int level = panel.getDatabaseView().getLabelView().getSelectedNode().getLevel();
					if (level != 1 && level !=2) {
						SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.CANT_EDIT_NON_TEMPORAL_PARTITION);
					}
					else {
						LabelObject selectedLabelObject = null;
						String label = "";
						
						if(level == 2)
						{
							DefaultMutableTreeNode parentNode = (DefaultMutableTreeNode) panel.getDatabaseView().getLabelView().getSelectedNode().getParent();
							selectedLabelObject = (LabelObject)parentNode.getUserObject();
							
							label = selectedLabelObject.getLabel();
						}
						else
						{
							selectedLabelObject = (LabelObject) panel.getDatabaseView().getLabelView().getSelectedNode().getUserObject();
							label = selectedLabelObject.getLabel();
						}
						
						String warningDialogText =
								"All annotated data for Temporal Partition  \""+ label + "\" will be DELETED.\n\n"
								+ "Are you absolutely sure you want to delete Temporal Partition \"" + label + "\"?";
						String warningTitleText = "WARNING";
						int optionType = JOptionPane.YES_NO_OPTION;
						int messageType = JOptionPane.ERROR_MESSAGE;
						String options[] = {"Proceed", "Cancel"};
						
						int result = -1;
						try {
							result = JOptionPane.showOptionDialog(this, 
									warningDialogText, 
									warningTitleText, 
									optionType, 
									messageType, 
									null, 
									options, 
									options[1]);
						} catch (HeadlessException he) {
							logger.log(Level.INFO, he.getMessage());
						}
						
						if (result == JOptionPane.OK_OPTION)
							panel.deleteTemporalPartition();
							SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(true);
					}
				}
			}
			break;

		case RENUMBER_UTTERANCES:
			SignStreamSegmentPanel segmentPanel = getSignStreamSegmentPanel();
			FieldWrappersCollection collection = segmentPanel.getDatabaseView().getSegmentGraphicRepresentation()
					.getFieldWrappersCollection();

			DefaultMutableTreeNode selectedNode = segmentPanel.getDatabaseView().getLabelView().getSelectedNode();
			LabelObject partitionObj;
			if (selectedNode.getLevel() == 2) {
				partitionObj = (LabelObject) ((DefaultMutableTreeNode) selectedNode.getParent()).getUserObject();
			} else {
				partitionObj = (LabelObject) selectedNode.getUserObject();
			}
			SS3TemporalPartition partition = segmentPanel.getSS3Database().getSS3GlossWindow()
					.getTemporalPartitionByLabel(partitionObj.getLabel());
			if (partition == null) {
				break;
			}

			int result = JOptionPane.showOptionDialog(
					this,
					"You are about to renumber all utterances\nin the current temporal partition. This\n" +
							"action is irreversible. Are you sure you\nwant to continue?",
					"Renumber Utterances?", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE, null,
					new Object[] { "Cancel", "OK" }, "Cancel"
			);
			if (result == 0) {
				break;
			}

			ArrayList<SS3GlossWindowSegment> segmentTiers = partition.getTemporalPartitionSegments();

			for (FieldWrapper wrapper : collection.getWrappers()) {
				PresentationField field = (PresentationField) wrapper.getField();
				ArrayList<PresentationEventsEntity> entities = field.getEntities();

				if (field.getName().equals(partitionObj.getLabel()) ||
						segmentTiers.contains(field.getSS3GlossWindowSegment())) {
					for (int i = 0; i < entities.size(); i++) {
						PresentationEvent event = entities.get(i).getEvent();
						event.setText("U" + (i + 1));
					}
				}
			}
			segmentPanel.getDatabaseView().getSegmentGraphicRepresentation().repaint();
			break;
			
		case EXIT:
			exit();
			break;
			
		case HELP:
			confirmationDialog("Help Dialog",
					"<HTML>"  
					+ "     <center><h3>Summary of SignStream Terms </h3></center>    <br>"  
					+ "     <b>Temporal Partition: </b>  A tier for a macro unit containing non-overlapping utterances. <br>"  
					+ "     <b>Segment Tier: </b> Shown below a temporal partition, corresponding to a unique participant. <br>"  
					+ "     <b>Utterance: </b>  A time unit that is represented by the Utterance Bar in the temporal partition. <br>"  
					+ "     <b>Segment: </b>  A time unit under a segment tier with the same boundaries as its corresponding utterance. <br>"  
					+ "     <b>Entity (Chained Event): </b>  An Entity includes the main event (its value is represented by a label)<br> plus any associated elements that are tied to it. <br>"  
					+ "</HTML>");
			break;
			
		case UPDATE:
			try {
				String currentDir = System.getProperty("user.dir");
				
				String command1 = 
						"tell application \"Terminal\" to do script \"" + BASH_UPDATE_SCRIPT + " --patch --install latest; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};
				
				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging
				p.start();
			} catch (IOException e) {
				System.out.println("'" + BASH_UPDATE_SCRIPT + " --patch --install latest' failed");
				logger.log(Level.INFO, "'" + BASH_UPDATE_SCRIPT + " --patch --install latest' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
			break;
		case INSTALL_SIGN_BANK:
			try {
				String currentDir = System.getProperty("user.dir");

				String command1 = 
						"tell application \"Terminal\" to do script \"" + BASH_SIGN_BANK_SCRIPT + "; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};

				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging	
				p.start();
			} catch (IOException e) {
				System.out.println("'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "'" + BASH_SIGN_BANK_SCRIPT + "' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
			break;
		case UNINSTALL_SIGN_BANK:
			try {
				String currentDir = System.getProperty("user.dir");

				String command1 = 
						"tell application \"Terminal\" to do script \"" + BASH_SIGN_BANK_SCRIPT + " --uninstall; exit; \"";
				String command2 = "tell application \"Terminal\" to activate";	// bring the Terminal window to the front
				String[] commands = {"osascript",  "-e",  command1, "-e", command2};

				ProcessBuilder p = new ProcessBuilder(commands);
//				p.inheritIO().start();	// leave commented - for debugging	
				p.start();
			} catch (IOException e) {
				System.out.println("'" + BASH_SIGN_BANK_SCRIPT + " --uninstall' failed");
				logger.log(Level.INFO, "'" + BASH_SIGN_BANK_SCRIPT + " --uninstall' failed");
				logger.log(Level.INFO, "Exception: " + e.getMessage());
			}
			break;		
		case NEW_MACRO_UNIT:
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED);
			break;

		case RENAME_MACRO_UNIT:
			if (macroUnitDialog.isVisible()) {
				break;
			}

			Object sObject1 = tree.getLastSelectedPathComponent();

			if (sObject1 == null) {
				confirmationDialog("Rename Macro Unit Warning",
						"Please select a Macro Unit to rename.");
				return;
			}

			DefaultMutableTreeNode node6 = (DefaultMutableTreeNode) sObject1;

			if (!node6.isLeaf()) {
				confirmationDialog("Rename Macro Unit Warning",
						"Please select a Macro Unit to rename.");
				return;
			}

			Object robj = node6.getUserObject();
			Excerpt excerpt1 = (Excerpt) robj;

			SS3DatabaseCollection dbCol123 = SS3Singleton
					.getSS3DatabaseCollection();
			SS3Database db = dbCol123.getSS3Database(excerpt1);
			db.setExcerpt(excerpt1.getExcerpt());

			ArrayList<LoadedMediaFile> lmFiles1 = new ArrayList<LoadedMediaFile>();
			HashMap<String, SS3MediaFile> mFiles1 = db.getMediaFiles();
			Collection<SS3MediaFile> files = mFiles1.values();

			for (SS3MediaFile mFile : files) {
				LoadedMediaFile mf = new LoadedMediaFile();
				mf.setMediaFile(mFile);
				mf.setDisplay(true);
				lmFiles1.add(mf);
			}

			String globalParticipantsFileName1 = path
					+ "globalParticipants.xml";
			String localParticipantsFileName1 = path + "localParticipants.xml";
			String globalAnnotatorsFileName1 = path + "globalAnnotators.xml";
			String localAnnotatorsFileName1 = path + "localAnnotators.xml";
			String mediaLibraryFileName1 = path + "mediaLibrary.xml";
			
			ParticipantsLibrary participantsLibrary1 = parser
					.parseParticipantsLibrary(globalParticipantsFileName1,
							localParticipantsFileName1);
			ParticipantsLibrary annotatorsLibrary1 = parser
					.parseAnnotatorsLibrary(globalAnnotatorsFileName1,
							localAnnotatorsFileName1);
			SS3MediaLibrary mLibrary1 = parser
					.parseMediaLibrary(mediaLibraryFileName1);

			if (macroUnitDialog != null) {
				macroUnitDialog.dispose();
			}
			macroUnitDialog = new CreateUpdateMacroUnitDialog(
					CreateUpdateMacroUnit.FUNCTION_UPDATE, db, excerpt1,
					lmFiles1, mLibrary1, participantsLibrary1,
					annotatorsLibrary1,
					signBankCollection.getSignBankResourceCollection(),
					panel, this);

			macroUnitDialog.setVisible(true);
			break;

		case DELETE_MACRO_UNIT:
			SS3Singleton.getErrorMessages().showErrorMessage(ErrorMessages.FUNCTIONALITY_NOT_YET_IMPLEMENTED);
			break;
		}

		fileMenu.setEnabled(true);
		newItem.setEnabled(true);
		openAsItem.setEnabled(true);
		openLegacyItem.setEnabled(true);

		closeItem.setEnabled(true);
		saveItem.setEnabled(true);
		saveAsItem.setEnabled(true);

		repaint();

		fileMenu.setEnabled(true);
		newItem.setEnabled(true);
		openAsItem.setEnabled(true);
		openLegacyItem.setEnabled(true);

		closeItem.setEnabled(true);
		saveItem.setEnabled(true);
		saveAsItem.setEnabled(true);
	}
	
	/**
	 * ((This is a package level method for now, with the intention of only being used by SS3SignStreamApplication and contained classes.))
	 * This code was originally called in the OPEN_AS_SS3 block of the local actionPerformed().
	 * Similar code was found in the SAVE and DONT_SAVE blocks of the SaveAndOpenCollectionDialog listener,
	 * so this method is standardized between them all for easy readability. 
	 * @param tree JTree from the db navigational panel
	 * @param rootNode Root node from the db navigational panel
	 * @param node Sub node from the db navigational panel, under the root node
	 */
	void openDatabaseSS3(JTree tree, DefaultMutableTreeNode rootNode, DefaultMutableTreeNode node) {
		File file = openZipDatabase();

		if(file != null) {

			// When a new file is selected/opened, close any existing morph-phon dialog
			MorphologicalPhonologicalInfoDialog dialog;
			if ((dialog = SS3Singleton.getNonManuelFieldDialogs().getMorphologicalPhonologicalInfoDialog()) != null) {
				dialog.discardChanges();
			}

			File unzippedFiles = unzipDirectory(file);
			if (unzippedFiles != null) {
				
				panel.closeAllLoadedVideoFiles();
				rootNode.removeAllChildren();
				
				((DefaultTreeModel) tree.getModel()).reload(rootNode);
				panel.loadDatabase(null);
				SS3Singleton.getSS3DatabaseCollection().removeAll();
				
				try {
					dbDelegate.openDb(unzippedFiles, file);

					// Override MMC timescale with stored values from the DB when loading
					var odb = SS3Singleton.getSS3DatabaseCollection().getDatabases().entrySet().iterator().next().getValue();
					var timestep = odb.getPacketDuration();
					var timebase = odb.getMovieTimescale();
					var duration = odb.getMovieDuration();

                    System.out.println("openDB: GOT DB TIMESCALES:");
                    System.out.println("timestep = " + timestep);
                    System.out.println("timebase = " + timebase);
                    System.out.println("duration = " + duration);

                    if (timestep != 0 && timebase != 0 && duration != 0) {
                        SS3Singleton.getVideoControlManager().getMultipleMovieController().setTimescale(timestep, timebase, duration);
                        SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getZoomer().getCoordinateTimeConvertor().setTimeScale(timestep, duration);
                    } else {
                        System.out.println("Read one or more timescale zeros from DB, no override!");
                    }

					SS3Singleton.getFilesLocation()
							.setLastOpenedDatabaseLocation(file.getPath());
					panel.getDbNavigationPanel().refresh(
							dbDelegate.getMutableTreeNode());

					if (rootNode.getChildCount() > 0) {
						DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) rootNode.getChildAt(0);
                        DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode) node1.getFirstChild();

                        panel.getDbNavigationPanel().getJTree().requestFocus();

                        panel.getDbNavigationPanel().getJTree().expandRow(0);
                        panel.getDbNavigationPanel().getJTree().expandRow(1);
                        panel.getDbNavigationPanel().getJTree().setSelectionRow(2);

                        SS3SignStreamApplication.logger.log(Level.INFO, "Node [" + node
                                + "] had been selected.");

                        SignStreamSegmentPanel segmentPanel = SS3Singleton
                                .getSignStreamApplication().getSignStreamSegmentPanel();

                        if (selectedNode != null) {
                            SegmentGraphReprUtil.utteranceMovieTimeOnset = 0;
                            SegmentGraphReprUtil.utteranceMovieTimeOffset = -1;

                            Object obj = selectedNode.getUserObject();

                            if (obj instanceof Excerpt) {
                                Excerpt excerpt = (Excerpt) obj;
                                SS3DatabaseCollection dbCol = SS3Singleton
                                        .getSS3DatabaseCollection();
                                SS3Database db = dbCol.getSS3Database(excerpt);

                                panel.loadDatabase(db);

                                // // calculate coef that will allow users to see entire
                                // // duration of a video file on a screen
//                                int movieDuration = panel.getMultipleMovieController().getMovieDuration();
//                                int packetDuration = (int) panel
//                                        .getMultipleMovieController().getPacketDuration();

                                try {
                                    MediaToolBar mToolBar = SS3Singleton.getMediaToolBar(null);
                                    mToolBar.selectOverviewPanel();
                                } catch(Exception ex) {
                                    ;
                                }

//                                if (movieDuration == 0) {
//                                    SS3GlossWindow glossWindow = db.getSS3GlossWindow();
//                                    ArrayList<SS3TemporalPartition> temporalPartitions = glossWindow
//                                            .getTemporalPartitions();
//                                    for (SS3TemporalPartition temporalPartition : temporalPartitions) {
//                                        SS3GlossWindowSegment segment = temporalPartition
//                                                .getGlossWindowSegment();
//                                        ArrayList<SS3GlossWindowItem> items = segment
//                                                .getSS3GlossWindowItems();
//                                        for (SS3GlossWindowItem item : items) {
//                                            int time = item.getEndTimeInfo().getMovieTime();
//                                            if (time > movieDuration)
//                                                movieDuration = time;
//                                        }
//                                    }
//                                    packetDuration = 1001;
//                                    movieDuration = movieDuration + 20 * packetDuration;
//                                }

//                                panel.getZoomer().getCoordinateTimeConvertor().setPacketDuration(packetDuration);
//
//                                double pixelsInTimeframe = db
//                                        .getTemporalPartitionPixelInTimeFrame();
//
//                                int width = panel.getDatabaseView().getVisibleWidth2();
//
//                                if (pixelsInTimeframe <= 0) {
//                                    panel.getZoomer()
//                                            .fillEntirePage(movieDuration, packetDuration,
//                                                    width);
//                                } else {
//                                    panel.getZoomer()
//                                            .getCoordinateTimeConvertor()
//                                            .setPixelInTimeFrame(pixelsInTimeframe);
//                                }
//
//                                int movieTime = SS3Singleton.getVideoControlManager()
//                                        .getMultipleMovieController().getCurrentTime();

                                int width = panel.getDatabaseView().getVisibleWidth2();
                                panel.getZoomer().getCoordinateTimeConvertor().resize(width);

                                var contr = SS3Singleton.getVideoControlManager().getMultipleMovieController();
                                panel.getZoomer().getCoordinateTimeConvertor().setTimeScale(contr.getPacketDuration(), contr.getMovieDuration());
//                                panel.getZoomer().fillEntirePage(movieDuration, packetDuration, width);
//                                panel.getSlider().setSliderPositionTime(movieTime);

//                                SS3Singleton.getVideoControlManager().getMultipleMovieController().setTimescale(packetDuration, movieTimescale);
                            }

						Object uo = node1.getUserObject();
						if (uo instanceof String) {
							setTitle((String) uo);
						}
					}
                    }
					SS3Singleton.getApplicationStateController()
							.setLoadedCollectionUpdated(false);

					logger.log(Level.INFO, "Database [" + file.getName()
							+ "] had been loaded.");
					
					System.out.println(unzippedFiles.getPath());
					
					removeDirectory(unzippedFiles.getPath());	// remove the unzipped "tmp_<file>.ss3" directory
					
					// select annotator screen	
					AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
							SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators(), 
							macroUnitDialog.getMacroUnitPanel().getAnnotatorsDialog(), this);
					annotatorSelectionDialog.setLocation(250, 250);
					annotatorSelectionDialog.setVisible(true);
					
				} catch (Exception ex) {
					logger.log(Level.SEVERE, "Could not open collection ["
							+ file.getName() + "].", ex);
				}
			} else {
				logger.log(Level.INFO, "Wrong file format.");
				
				String txt = "<HTML>SignStream is unable to open this file.  Either it was not " + "<BR>" +
						"created by SIgnStream or it has been corrupted.</HTML>";
				
				confirmationDialog("File format", txt);
			}
		}
	}

	public SS3Database openDatabaseSS3() {

		File file = openZipDatabase();
		File unzippedFiles = unzipDirectory(file);

		if (unzippedFiles != null) {
			try {
				dbDelegate.openDb(unzippedFiles, file);

				// set correct path!!!
				SS3Singleton.getFilesLocation().setLastOpenedDatabaseLocation(
						file.getPath());

				logger.log(Level.INFO, "Database [" + file.getName()
						+ "] had been loaded.");
				
				// find db which was created in dbDelegate.openDB
				for (SS3Database db: SS3Singleton.getSS3DatabaseCollection().getDatabases().values()) {
					String str1 = db.getReadFromXMLFile().toString();
					int index = str1.indexOf("/");
					String str2 = str1.substring(0, index);
					
					if (unzippedFiles.getPath().equals(str2)) {
						return db;
					}
				}
				
			} catch (Exception ex) {
				logger.log(Level.SEVERE,
						"Could not open collection [" + file.getName() + "].",
						ex);
			}
		}
		
		return null;
	}
	/**
	 * Saves the collection to a ".ss3" file.
	 * Does this by saving the macro units to a temporary directory and then zipping up that
	 * directory into a ".ss3" archive.
	 * @param node - Node to represent the collection
	 * @param saveType - Used to distinguish between a regular "Save" and a "Save as"
	 */
	public void saveDatabaseToFile(DefaultMutableTreeNode node, int saveType) {
		
		if(panel.isUtteranceLoaded()) {
			panel.updateLoadedDatabase();
		}
		
		String newArchivePath = null;
		String collectionName = (String) node.getUserObject();
		FilesLocation lastLocation = SS3Singleton.getFilesLocation();
		
		// Getting the database object
		Excerpt ex = (Excerpt) node.getFirstLeaf().getUserObject();
		SS3DatabaseCollection dbCol = SS3Singleton.getSS3DatabaseCollection();
		SS3Database db = dbCol.getSS3Database(ex);
		// Collection name does not have the ".ss3" file extension so you must add it here
		db.setNumber(ex.getID());
		db.setExcerpt(ex.getExcerpt());
		db.setFileName(ex.getExcerpt() + SS3Database.macroFileExtension);
		
		if(saveType == SAVE_AS){
			FileDialog fd = new FileDialog(this, "Save Collection As...", FileDialog.SAVE);
			
			// Sets the FileDialog to open the location of the last opened database
			File lastOpened = new File(lastLocation.getLastOpenedDatabaseLocation());
			fd.setDirectory(lastOpened.getParent());
			fd.setFile(lastOpened.getName());
			
			fd.setVisible(true);
			collectionName = fd.getFile();
			String parentDirectoryName = fd.getDirectory();
			
			if (collectionName == null) {
				return;
			}
			
			// Set the parent directory, the name of the temporary directory, and the name of
			// the path of the new archive file
			db.setParentDirectoryPath(parentDirectoryName);
			if (!collectionName.endsWith(SS3Database.dbFileExtension))
				collectionName += SS3Database.dbFileExtension;
			db.setDirectory(SS3Database.tempDirectoryPrefix + collectionName);
			newArchivePath = parentDirectoryName + File.separator + collectionName;
		} else {
			// Collection name does not have the ".ss3" file extension so you must add it here
			db.setDirectory(
					SS3Database.tempDirectoryPrefix
					+ collectionName
					+ SS3Database.dbFileExtension);
		}

		// Saves the macro units to a temporary directory
		boolean saved = saveDatabase(db);
		if(!saved) {
			return;
		}
		
		// Zips the temporary directory into a ".ss3" archive and then deletes it
		String tempDirectoryPath = db.getParentDirectoryPath() + File.separator + db.getDirectory();
		boolean ziped = zipDirectory(tempDirectoryPath);
		// If zipping fails, delete the temporary directory
		if (!ziped) {
			removeDirectory(tempDirectoryPath);
		}

		// If everything was successful, update the title of the application frame if necessary
		// and show a confirmation message.
		if (saved && ziped) {
			if(saveType == SAVE_AS){
				String nodeName = collectionName
						.substring(0, collectionName.lastIndexOf(SS3Database.dbFileExtension));
				node.setUserObject(nodeName);
				panel.getDbNavigationPanel().getJTree().repaint();
				setTitle(nodeName);
				lastLocation.setLastOpenedDatabaseLocation(newArchivePath);
			}
			confirmationDialog("Saved Resource", "Collection " + node.getUserObject()
					+ " has been saved.");
			SS3Singleton.getApplicationStateController().setLoadedCollectionUpdated(false);
		}
	}

	public File unzipDirectory(File zFile) {

		try {
			String name = zFile.getName();

			if (name.endsWith(".zip")) {
				name = name.substring(0, name.length() - 4);
			}

			FileInputStream fin = new FileInputStream(zFile.getPath());
			// create temporary directory
			File tmpDir = new File(SS3Database.tempDirectoryPrefix + name);
			tmpDir.mkdir();

			// create ZipInputStream from FileInputStream object
			ZipInputStream zin = new ZipInputStream(fin);

			int i = 0;
			while ((zin.getNextEntry()) != null) {
				// create OutputStream to extract the entry from zip file
				String fileName = tmpDir.getName() + File.separator + "file"
						+ i + ".xml";
				OutputStream os = new FileOutputStream(fileName);

				byte[] buffer = new byte[1024];
				int length;

				// read the entry from zip file and extract it to disk
				while ((length = zin.read(buffer)) > 0) {
					os.write(buffer, 0, length);
				}

				// close the streams
				os.close();
				i++;
			}
			// close the zip file
			zin.close();

			return tmpDir;
		} catch (IOException e) {
			System.out.println("IOException :" + e);
			return null;
		}
	}

	/**
	 * Zips a directory into a ".ss3" archive file.
	 * Assumes the input has already had its ".ss3" extension added.
	 * @param dirName
	 * @return True if directory was zipped and deleted and false if an exception was thrown.
	 */
	public boolean zipDirectory(String dirName) {
		try {
			File dirFile = new File(dirName);
			String cDirName = dirFile.getName();

			if (cDirName.startsWith(SS3Database.tempDirectoryPrefix)) {
				cDirName = cDirName.substring(4);
			}

			String zipFile = dirFile.getParentFile().getPath()
					+ File.separatorChar + cDirName;

			byte[] buffer = new byte[1024];

			try {
				FileOutputStream fout = new FileOutputStream(zipFile);
				ZipOutputStream zout = new ZipOutputStream(fout);

				File dir = new File(dirName);

				File[] children = dir.listFiles();
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];

						FileInputStream fin = new FileInputStream(file);
						zout.putNextEntry(new ZipEntry(file.getName()));
						int length;

						while ((length = fin.read(buffer)) > 0) {
							zout.write(buffer, 0, length);
						}

						zout.closeEntry();
						fin.close();
					}
				}

				// close the ZipOutputStream
				zout.close();
				fout.close();

				// remove unzipped files
				removeDirectory(dir.getAbsolutePath());

				if (zipFile != null) {
					String fileToDelete = zipFile + ".zip";
					File file = new File(fileToDelete);
					if (file.exists()) {
						file.delete();
					}
				}

			} catch (FileNotFoundException ex) {
				ex.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);

				String[] str = new String[4];
				str[0] = "Collection could not be saved as .ss3 file";
				str[1] = "because access is denied.";
				str[2] = "Please close zip file and try again.";
				str[3] = "";

				SS3Singleton.getErrorMessages()
						.showSaveCollectionErrorMessage(str);
				return false;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ioe);
			
			String[] str = new String[4];
			str[0] = "Collection could not be saved as .ss3 file";
			str[1] = ioe.getMessage();
			str[2] = "";
			str[3] = "";

			SS3Singleton.getErrorMessages()
					.showSaveCollectionErrorMessage(str);
			
			return false;
		} catch (NullPointerException e) {
			String[] str = new String[4];
			str[0] = "Collection could not be saved as it does not exist yet.";
			str[1] = "Please Confirm your settings in the Macro Unit Dialog";
			str[2] = "before saving.";
			str[3] = "";
			SS3Singleton.getErrorMessages()
				.showSaveCollectionErrorMessage(str);
			return false;
		}
		return true;
	}

	private boolean removeDirectory(String dirName) {
		File dir = new File(dirName);
		File[] children = dir.listFiles();

		// remove files
		if (children != null) {
			for (int i = 0; i < children.length; i++) {
				File file = children[i];
				file.delete();
			}
		}
		dir.delete();
		return true;
	}

	public void confirmationDialog(String title, String msg) {
		final JDialog collectionSavedDialog = new JDialog(this, title, true);
		JButton okBtn = new JButton("OK");

		ButtonPanel collectionSavedButtonPanel = new ButtonPanel();
		collectionSavedButtonPanel.add(okBtn);

		JPanel collectionSavedPanel = new JPanel();
		collectionSavedPanel.setLayout(new BorderLayout());
		JLabel collectionSavedLabel = new JLabel(msg);

		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				collectionSavedDialog.dispose();
			}
		});

		collectionSavedPanel.add(collectionSavedLabel, BorderLayout.CENTER);
		collectionSavedPanel
				.add(collectionSavedButtonPanel, BorderLayout.SOUTH);

		collectionSavedPanel.add(new JLabel("  "), BorderLayout.WEST);

		Dimension d = collectionSavedPanel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 20, d.height + 40);
		collectionSavedPanel.setPreferredSize(nD);

		Container contentPane = collectionSavedDialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(collectionSavedPanel, BorderLayout.CENTER);

		collectionSavedDialog.setLocation(new Point(150, 150));
		collectionSavedDialog.pack();
		collectionSavedDialog.setVisible(true);

	}

	public void saveCollection() {
		panel.updateLoadedDatabase();

		Object root = panel.getDbNavigationPanel().getJTree().getModel()
				.getRoot();

		if (root == null)
			return;

		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
		int childCount = rootNode.getChildCount();
		DefaultMutableTreeNode node = null;

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		if (node == null) {
			confirmationDialog("Save Collection Warning",
					"Collection not available");
			return;
		}

		String name = (String) node.getUserObject();
		File dirToDel = new File(name);

		if (dirToDel.exists())
			dirToDel.delete();

		Enumeration enum4 = node.children();

		boolean saved = true;

		while (enum4.hasMoreElements()) {
			DefaultMutableTreeNode node1 = (DefaultMutableTreeNode) enum4
					.nextElement();
			Excerpt ex = (Excerpt) node1.getUserObject();
			SS3DatabaseCollection dbCol = SS3Singleton
					.getSS3DatabaseCollection();
			SS3Database db = dbCol.getSS3Database(ex);

			if (!ex.getExcerpt().equals(db.getExcerpt())) {
				// delete old file
				db.deleteFile();
				db.setExcerpt(ex.getExcerpt());
			}

			db.setNumber(ex.getID());
			db.setExcerpt(ex.getExcerpt());
			db.setDirectory(name);
			db.setFileName(ex.getExcerpt() + ".xml");

			if (!saveDatabase(db)) {
				saved = false;
			}
		}

		if (saved) {
			confirmationDialog("Save Resource", "Collection " + name
					+ " has been saved.");
		}
	}

	private boolean saveDatabase(SS3Database db) {
		try {
			return db.save();
		} catch (Exception e) {
			e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			return false;
		}
	}

	public void exit() {

		if (SS3Singleton.getApplicationStateController()
				.isLoadedCollectionUpdated()) {
			saveChangesToTheCollection(false);
		} else {
			dispose();
			System.exit(0);
		}

		return;
	}

	private void deleteTempFiles() {
		try {
			File currentDir = new File(".");
			File[] tmpDirs = currentDir.listFiles(new FileFilter() {
				@Override
				public boolean accept(File pathname) {
					return pathname.isDirectory() &&
							pathname.getName().startsWith(SS3Database.tempDirectoryPrefix) &&
							pathname.getName().endsWith(SS3Database.dbFileExtension);
				}
			});
			if (tmpDirs != null) {
				for (File tmpDir : tmpDirs) {
					deleteDir(tmpDir);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void deleteDir(File file) {
		File[] contents = file.listFiles();
		if (contents != null) {
			for (File f : contents) {
				deleteDir(f);
			}
		}
		file.delete();
	}

	private void addQuitHandler() {
		QuitHandler quitHandler = new QuitHandler() {
			
			@Override
			public void handleQuitRequestWith(QuitEvent event, final QuitResponse res) {
				deleteTempFiles();
				if (SS3Singleton.getApplicationStateController()
						.isLoadedCollectionUpdated()) {
					final JDialog newCollectionDialog = new JDialog(SS3SignStreamApplication.this,
							"Save Resource", true);
					final JButton dontSaveBtn = new JButton("Don't Save");
					final JButton cancelBtn = new JButton("Cancel");
					final JButton saveBtn = new JButton("Save");

					dontSaveBtn.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent arg0) {
							res.performQuit();
						}
					});

					cancelBtn.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent arg0) {
							newCollectionDialog.dispose();
							cancelBtn.removeActionListener(this);
							res.cancelQuit();
						}
					});

					saveBtn.addActionListener(new ActionListener() {
						@Override
						public void actionPerformed(ActionEvent arg0) {
							dbDelegate = panel.getDbNavigationPanel().getDatabaseDelegate();
							final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();
							int childCount = rootNode.getChildCount();
							DefaultMutableTreeNode node = null;

							if (childCount > 0) {
								node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

								if (node.getChildCount() > 0) {
									DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
											.getChildAt(0);
									Object uo = dNode.getUserObject();
									if (uo instanceof String) {
										node = dNode;
									}
								}
							}
							
							saveDatabaseToFile(node, SS3SignStreamApplication.SAVE);
							res.performQuit();
						}
					});

					ButtonPanel newCollectionButtonPanel = new ButtonPanel();
					newCollectionButtonPanel.add(dontSaveBtn);
					newCollectionButtonPanel.add(cancelBtn);
					newCollectionButtonPanel.add(saveBtn);

					JPanel newCollectionPanel = new JPanel();
					newCollectionPanel.setLayout(new BorderLayout());
					JLabel newCollectionLabel = new JLabel(
							" Do you want to save the changes you made to Collection?");
					newCollectionPanel.add(newCollectionLabel, BorderLayout.CENTER);
					newCollectionPanel.add(newCollectionButtonPanel,
							BorderLayout.SOUTH);

					newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

					Dimension d = newCollectionPanel.getPreferredSize();
					Dimension nD = new Dimension(d.width + 10, d.height + 40);
					newCollectionPanel.setPreferredSize(nD);

					Container contentPane = newCollectionDialog.getContentPane();
					contentPane.setLayout(new BorderLayout(40, 40));
					contentPane.add(newCollectionPanel, BorderLayout.CENTER);

					newCollectionDialog.setLocation(new Point(150, 150));
					newCollectionDialog.pack();
					newCollectionDialog.setVisible(true);
					//
				} else {
					res.performQuit();
				}
			}
			
		};
		Desktop.getDesktop().setQuitHandler(quitHandler);
	}
	
	public SignBankCollection getSignBankCollection() {
		return signBankCollection;
	}
	
	public SignBankCollection parseSignBank() {
		signBankCollection =  SS3Singleton.getSignBankParser().parseSignBank();
		return signBankCollection;
	}
	
	public void writeSignBankCollection() {
		SS3Singleton.getSignBankWriter().writeCollectionToSignBank(signBankCollection);
	}
}

class SignStreamWindowListener implements WindowListener {

	private SS3SignStreamApplication appl = null;

	public SignStreamWindowListener(SS3SignStreamApplication appl) {
		this.appl = appl;
	}

	@Override
	public void windowClosing(WindowEvent e) {
		appl.exit();
	}

	@Override
	public void windowClosed(WindowEvent e) {
	}

	@Override
	public void windowOpened(WindowEvent e) {
	}

	@Override
	public void windowIconified(WindowEvent e) {
	}

	@Override
	public void windowDeiconified(WindowEvent e) {
	}

	@Override
	public void windowActivated(WindowEvent e) {
	}

	@Override
	public void windowDeactivated(WindowEvent e) {
	}
}

class ImageFileView extends FileView {

	@Override
	public String getName(File f) {
		return null; 
	}

	@Override
	public String getDescription(File f) {
		return null; 
	}

	@Override
	public Boolean isTraversable(File f) {
		return null; 
	}

	@Override
	public String getTypeDescription(File f) {
		String type = null;

		if (f.isDirectory()) {
			FileFilter filter = new SignStreamFileFilter();
			File[] filteredFiles = f.listFiles(filter);
			if (filteredFiles != null && filteredFiles.length > 0) {
				type = "SignStream file";
			}
		}

		return type;
	}

	@Override
	public Icon getIcon(File f) {
		Icon icon = null;

		if (f.isDirectory()) {
			FileFilter filter = new SignStreamFileFilter();
			File[] filteredFiles = f.listFiles(filter);

			if (filteredFiles != null && filteredFiles.length > 0) {
				icon = new ImageIcon(ImageMaker.PALETTE_ICON, "SignStream");
			} 
		}

		return icon;
	}
}

class SignStreamFileFilter implements FileFilter {
	@Override
	public boolean accept(File file) {
		if (Util.isXML3File(file.getName())) {
			return true;
		}
		return false;
	}
}

class PopupListener extends MouseAdapter {
	@Override
	public void mousePressed(MouseEvent e) {
		maybeShowPopup(e);
	}

	@Override
	public void mouseReleased(MouseEvent e) {
		maybeShowPopup(e);
	}

	private void maybeShowPopup(MouseEvent e) {
	}
}

class SignStream3FilesFilter extends javax.swing.filechooser.FileFilter {

	@Override
	public boolean accept(File f) {
		if (f.isDirectory()) {
			return true;
		} else {
			return f.getName().endsWith(".ss3.zip");
		}
	}

	@Override
	public String getDescription() {
		return "Accepts SignStream3 files only.";
	}
}

class ZipFormatFilenameFilter implements FilenameFilter {
	String ext1;
	String ext2;

	public ZipFormatFilenameFilter() {
		this.ext1 = ".ss3.zip";
		this.ext2 = SS3Database.dbFileExtension;
	}

	@Override
	public boolean accept(File dir, String name) {
		if (name.endsWith(ext1)) {
			return true;
		} else {
			return name.endsWith(ext2);
		}
	}
}

class SaveAndOpenCollectionDialog extends JDialog implements ActionListener {

	private static final long serialVersionUID = 1L;

	private SS3SignStreamApplication ssApl;

	private final int SAVE = 0;
	private final int DONT_SAVE = 1;
	private final int CANCEL = 2;

	public SaveAndOpenCollectionDialog(SS3SignStreamApplication ssApl) {
		super(ssApl, "Save Resource", true);
		this.ssApl = ssApl;

		JButton dontSaveBtn = new JButton("Don't Save");
		JButton cancelBtn = new JButton("Cancel");
		JButton saveBtn = new JButton("Save");

		dontSaveBtn.setActionCommand("" + DONT_SAVE);
		saveBtn.setActionCommand("" + SAVE);
		cancelBtn.setActionCommand("" + CANCEL);

		dontSaveBtn.addActionListener(this);
		saveBtn.addActionListener(this);
		cancelBtn.addActionListener(this);

		ButtonPanel btnPanel = new ButtonPanel();
		btnPanel.add(dontSaveBtn);
		btnPanel.add(cancelBtn);
		btnPanel.add(saveBtn);

		JLabel saveChangesLbl = new JLabel(
				"Do you want to save the changes you made?");

		JPanel newCollectionPanel = new JPanel();
		newCollectionPanel.setLayout(new BorderLayout());
		newCollectionPanel.add(saveChangesLbl, BorderLayout.CENTER);
		newCollectionPanel.add(btnPanel, BorderLayout.SOUTH);
		newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

		Dimension d = newCollectionPanel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 10, d.height + 40);
		newCollectionPanel.setPreferredSize(nD);

		Container contentPane = this.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(newCollectionPanel, BorderLayout.CENTER);

		this.setLocation(new Point(150, 150));
		this.pack();
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		SignStreamSegmentPanel segmentPanel = ssApl.getSignStreamSegmentPanel();
		DatabaseDelegate dbDelegate = ssApl.getDatabaseDelegate();

		JTree tree = segmentPanel.getDbNavigationPanel().getJTree();
		final DefaultMutableTreeNode rootNode = dbDelegate.getMutableTreeNode();

		DefaultMutableTreeNode node = null;
		int childCount = rootNode.getChildCount();

		if (childCount > 0) {
			node = (DefaultMutableTreeNode) rootNode.getChildAt(0);

			if (node.getChildCount() > 0) {
				DefaultMutableTreeNode dNode = (DefaultMutableTreeNode) node
						.getChildAt(0);
				Object uo = dNode.getUserObject();
				if (uo instanceof String) {
					node = dNode;
				}
			}
		}

		switch (Integer.parseInt(str)) {
		case SAVE:
			if (node == null) {
				SS3Singleton
				.getSignStreamApplication().confirmationDialog("Save Resource", "Collection is not available.");
				return;
			}
			ssApl.saveDatabaseToFile(node, SS3SignStreamApplication.SAVE);
		case DONT_SAVE:
			this.setVisible(false);
			
			ssApl.openDatabaseSS3(tree, rootNode, node);
			
			break;
		case CANCEL:
			this.setVisible(false);
			break;
		}
	}
}
