package edu.bu.signstream.ui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import edu.bu.signstream.common.util.UserInterfaceUtil;

public class SerchableDirectory extends JPanel implements ActionListener, ItemListener {
	
	private static final long serialVersionUID = 1L;
	
	private JButton selectBtn = new JButton("Select");
	private JButton removeBtn = new JButton("Remove");
	
	private JLabel label = new JLabel();
	private JCheckBox displayCb = new JCheckBox();
	
	private MoveToTop moveBtn = new MoveToTop();
	private JLabel priorityLbl = new JLabel("1");
	
	private int numberOfSearches = 0;
	
	private DirectoryLookupPanel lookupPanel; 
	
	private final int SELECT = 0;
	private final int REMOVE = 1;
	private final int MOVE = 2;
	
	public SerchableDirectory(String directory, boolean isSearchable) {
		label.setText(directory);
		displayCb.setSelected(isSearchable);
		
		init();
	}
	
	public void setNumberOfSearches(int numberOfSearches) {
		this.numberOfSearches = numberOfSearches;
	}
	
	public int getNumberOfSearches() {
		return numberOfSearches;
	}
	
	public JLabel getPriorityLbl() {
		return priorityLbl;
	}
	
	public void setPriority(int priority) {
		priorityLbl.setText(""+priority);
	}
	
	public MoveToTop getMoveToTopBtn() {
		return moveBtn;
	}
	
	public SerchableDirectory() {
		init();	
	}
	
	private void init() {
		selectBtn.setActionCommand("" + SELECT);
		selectBtn.addActionListener(this);
		
		removeBtn.setActionCommand("" + REMOVE);
		removeBtn.addActionListener(this);
		
		moveBtn.setActionCommand("" + MOVE);
		moveBtn.addActionListener(this);
		
		UserInterfaceUtil.setLabelLNF(label);
		UserInterfaceUtil.setButtonsLNF(selectBtn);
		UserInterfaceUtil.setButtonsLNF(removeBtn);
		
		UserInterfaceUtil.setCheckBoxLNF(displayCb);
		
		moveBtn.setPreferredSize(moveBtn.getMaximumSize());
		moveBtn.setMaximumSize(moveBtn.getMaximumSize());
		
		priorityLbl.setHorizontalTextPosition(SwingConstants.LEFT);
		priorityLbl.setVerticalAlignment(SwingConstants.CENTER);
		
		label.setHorizontalTextPosition(SwingConstants.LEFT);
		label.setVerticalAlignment(SwingConstants.CENTER);
		
		displayCb.addItemListener(this);
		
		if(displayCb.isSelected()) {
			moveBtn.setEnabled(true);
		} else {
			moveBtn.setEnabled(false);
		}
	}
	
	public void setDirectoryLookupPanel(DirectoryLookupPanel lookupPanel) {
		this.lookupPanel = lookupPanel;
	} 
	
	public JButton getSelectBtn() {
		return selectBtn;
	}

	public void setSelectBtn(JButton selectBtn) {
		this.selectBtn = selectBtn;
	}

	public JButton getRemoveBtn() {
		return removeBtn;
	}

	public void setRemoveBtn(JButton removeBtn) {
		this.removeBtn = removeBtn;
	}

	public JLabel getLabel() {
		return label;
	}

	public void setLabel(JLabel lable) {
		this.label = lable;
	}

	public JCheckBox getDisplayCb() {
		return displayCb;
	}

	public void setDisplayCb(JCheckBox displayCb) {
		this.displayCb = displayCb;
	}

	public String getDirectory() {
		return label.getText().trim();
	}

	public void setDirectory(String directory) {
		label.setText(directory);
	}

	public boolean isSearchable() {
		return displayCb.isSelected(); 
	}

	public void setSearchable(boolean isSearchable) {
		displayCb.setSelected(isSearchable);
	}
	
	private String lastTimeOpenedDirectory = "";
	
	@Override
	public void actionPerformed(ActionEvent ae) {	
		String str = ae.getActionCommand();
		
		switch (Integer.parseInt(str)) {
			case SELECT:
				File file = new File(lastTimeOpenedDirectory);
				File selectedFile = UserInterfaceUtil.selectDirectory(lookupPanel.getParentFrame(), file, this, "SignStream Locate Video File");
				if (selectedFile != null && selectedFile.exists()) {
					label.setText(selectedFile.getPath());
					label.repaint();
				}
				break;
			
			case REMOVE:
				lookupPanel.removeDirectory(this);
				lookupPanel.initUI();
				break;
				
			case MOVE:
				lookupPanel.moveToTop(this);
				break;
		}
	}
	
	@Override
	public void itemStateChanged(ItemEvent event) {
		if(displayCb.isSelected()) {
			moveBtn.setEnabled(true);
		} else {
			moveBtn.setEnabled(false);
		}
		// call re-arrange method
		if(lookupPanel != null)
			lookupPanel.reArrangeDirectories();
	}

	@Override
	public String toString() {
		return getDirectory();
	}
}
