package edu.bu.signstream.ui.panels;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.grepresentation.view.UtteranceView;

public class SegmentSelectionPanel extends JPanel {

	private static final long serialVersionUID = 1L;
	
	private JLabel selectionLabel = null;

	private JComboBox<String> partitionComboBox = new JComboBox<String>();
	private String currentPartitionString = null;
	
	private JComboBox<String> segmentComboBox = new JComboBox<String>();
	private String currentSegmentString= null;
	
	private DefaultComboBoxModel[] models;
	private String[] defaultSegmentTiers;
	
	private JButton enterButton = null;
	
	private JButton cancelButton = null; 
	
	private SegmentSelectionDialog segmentSelectionDialog = null;
	private UtteranceView utteranceView = null;
	
	public SegmentSelectionPanel(SegmentSelectionDialog segmentSelectionDialog, 
			UtteranceView utteranceView) {
		this.segmentSelectionDialog = segmentSelectionDialog;
		this.utteranceView = utteranceView;
		
		selectionLabel = new JLabel(
				"<html><body> <br> Please select the Temporal Partition and Segment Tier you wish to view. <br> </body></html>");
		
		ArrayList<SS3TemporalPartition> temporalPartitions = 
				utteranceView.getSignStreamSegmentPanel().getSS3Database().getSS3GlossWindow().getTemporalPartitions();
		int tpSize = temporalPartitions.size();
		
		// combobox model array
		models = new DefaultComboBoxModel[tpSize];
		defaultSegmentTiers = new String[tpSize];
		
		for (int i = 0; i < temporalPartitions.size(); i++) {
			SS3TemporalPartition temporalPartition = temporalPartitions.get(i);
			String partitionLabel = temporalPartition.getLabel();
			partitionComboBox.addItem(partitionLabel);

			ArrayList<SS3GlossWindowSegment> temporalPartitionSegments = temporalPartition.getTemporalPartitionSegments();
			int stSize = temporalPartitionSegments.size();	
			
			// each TP selection is associated with a different model of ST strings
			String[] segmentTierLabels = new String[stSize];
			
			// find the default segment to set as selected when TP is switched
			SS3GlossWindowSegment defaultSegment = temporalPartition.getDefaultSegment();
			String defaultSegmentString = null;
			
			for (int k = 0; k < stSize; k++) {
				SS3GlossWindowSegment temporalPartitionSegment = temporalPartitionSegments.get(k);
				String segmentLabel = temporalPartitionSegment.getLabel();
				String participantLabel = temporalPartitionSegment.getParticipantLabel();
				String combinedLabel = participantLabel + ":" + segmentLabel;

				if (combinedLabel.indexOf(":") == 0)
					combinedLabel = combinedLabel.substring(1);
				
				segmentTierLabels[k] = combinedLabel;
				
				if (temporalPartitionSegment.equals(defaultSegment)) {
					defaultSegmentString = combinedLabel;
				}
			}
			
			models[i] = new DefaultComboBoxModel<String>(segmentTierLabels);
			defaultSegmentTiers[i] = defaultSegmentString;
		}
		
		// guaranteed to have at least one index (i.e. at least one TP in the collection)
		segmentComboBox.setModel(models[0]);
		segmentComboBox.setSelectedItem(defaultSegmentTiers[0]);
		
		partitionComboBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				int selectedIndex = partitionComboBox.getSelectedIndex();
				
				segmentComboBox.setModel(models[selectedIndex]);
				segmentComboBox.setSelectedItem(defaultSegmentTiers[selectedIndex]);
			}			
		});	
		
		enterButton = new JButton("Confirm Selection");
		enterButton.addActionListener(new ConfirmSelectionListener(this));
		
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				getParentDialog().dispose();
			}
		});
		
		setLayout(new GridLayout(0, 1));
		add(selectionLabel);
		add(partitionComboBox);
		add(segmentComboBox);
		add(enterButton);
		add(cancelButton);
	}
	
	public SegmentSelectionDialog getParentDialog() {
		return segmentSelectionDialog;
	}
	
	public UtteranceView getUtteranceView() {
		return utteranceView;
	}
	
	public String getSelectedPartitionString() {
		return (String) partitionComboBox.getSelectedItem();
	}
	
	public String getSelectedSegmentString() {
		return (String) segmentComboBox.getSelectedItem();
	}
}

class ConfirmSelectionListener implements ActionListener {
	SegmentSelectionPanel segmentSelectionPanel = null;
	
	public ConfirmSelectionListener(SegmentSelectionPanel segmentSelectionPanel) {
		this.segmentSelectionPanel = segmentSelectionPanel;
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		String temporalPartitionLabel = segmentSelectionPanel.getSelectedPartitionString();
		String segmentTierWithParticipantLabel = segmentSelectionPanel.getSelectedSegmentString();
		String segmentTierLabel = segmentTierWithParticipantLabel.
				substring(segmentTierWithParticipantLabel.indexOf(":") + 1);
		
		// pass in null as the Segment here since we are intending to add a new Segment
//		boolean canSwitch = segmentSelectionPanel.getUtteranceView()
//				.canSwitchThisSegment(null, temporalPartitionLabel, segmentTierWithParticipantLabel);
//		
//		if ( !canSwitch) {
//			String msg = "<HTML>Warning: That Segment Tier is already in view.<BR>";
//			msg = msg + "Please select a different Segment Tier.</HTML>";
//			SS3Singleton.getSignStreamApplication()
//					.confirmationDialog("Switch Segment Warning", msg);					
//			return;
//		} else {
			segmentSelectionPanel.getUtteranceView().addUtterancePane(null, temporalPartitionLabel, segmentTierLabel);
			
			segmentSelectionPanel.getParentDialog().dispose();
//		}
	}
}
