package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Constants;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Track;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualField;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFieldWrapper;
import edu.bu.signstream.grepresentation.fields.nonManualField.NonManualFreeTextFieldWrapper;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.LabelObject;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.DocumentModelListener;
import edu.bu.signstream.ui.panels.colorChooser.ColorChooser;
import edu.bu.signstream.ui.panels.table.sortable.TableSorter;

public class AddOrUpdateField extends JPanel implements ActionListener, ItemListener {

	private static final long serialVersionUID = 1L;

	private JDialog parent = null;
	// private SS3CodingScheme scheme = null;
	private SignStreamSegmentPanel segmentPanel;
	private JPanel fieldPanel = null;
	private Segment segment = null;
	private PresentationField parentField = null;

	private JLabel fieldNameLbl = new JLabel("Field Name: ");
	private JLabel fieldLabelLbl = new JLabel("Field Label: ");
	private JLabel fieldPrefixLbl = new JLabel("Value Prefix: ");
	private JLabel fieldColorLbl = new JLabel("Field Color: ");
	private JLabel fieldGroupLbl = new JLabel("Field Group: ");

	private JTextField fieldNameTxt = new JTextField(13);
	private JTextField fieldLabelTxt = new JTextField(7);
	private JTextField fieldPrefixTxt = new JTextField(7);

	private ColorChooser fieldColorChooser = new ColorChooser();

	private String[] fieldGroups = { "Head", "Face", "Body", "Grammatical", "No Group" };
	private JComboBox fieldGroupCb = new JComboBox(fieldGroups);

	private JRadioButton inputText = new JRadioButton(" Text ");
	private JRadioButton inputTextFree = new JRadioButton(" Free ");
	private JRadioButton inputTextMenu = new JRadioButton(" Menu ");
	private JRadioButton inputGraphic = new JRadioButton(" Graphic ");

	private JRadioButton rightAlignment = new JRadioButton("");
	private JRadioButton leftAlignment = new JRadioButton("");
	private JRadioButton centerAlignment = new JRadioButton("");
	private JRadioButton rightNLeftAlignment = new JRadioButton("");

	private JTable valuesTable;
	private DefaultTableModel valuesTableModel = null;

	private JLabel newValueNameLbl = new JLabel("New Value Name:");
	private JLabel newValueLabelLbl = new JLabel("New Value Label:");
	private JTextField newValueNameTxt = new JTextField(9);
	private JTextField newValueLabelTxt = new JTextField(6);

	private JButton doneBtn = new JButton("Done");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton newValueBtn = new JButton("New Value");
	private JButton deleteValueBtn = new JButton("Delete Value");

	// value ids that are already in a permanent coding scheme and can't be removed
	private ArrayList<String> valuesThatCantBeRemoved = new ArrayList<String>();

	private boolean updateFieldInfo = true;

	private final int INPUT_TEXT = 0;
	private final int INPUT_TEXT_FREE = 1;
	private final int INPUT_TEXT_MENU = 2;
	private final int INPUT_GRAPHIC = 3;
	private final int DONE = 4;
	private final int CANCEL = 5;
	private final int ADD_VALUE = 6;
	private final int REMOVE_VALUE = 7;

	private GridBagConstraints gbc = new GridBagConstraints();
	private JPanel fieldTypeInputPanel = null;
	private JPanel valueAlignmentPanel = null;
	private JScrollPane valuesTablePane = null;

	private boolean showNewValueFields = true;

	private SS3Field field = null;

	private DocumentModelListener dmListener = null;

	private UpdateValueListener updateNameValueListener = null;
	private UpdateValueListener updateLabelValueListener = null;
	private TableSelectionListener1 tableSelectionListener = null;

	public AddOrUpdateField(SignStreamSegmentPanel segmentPanel, Segment segment, int group, SS3Field field,
			JPanel fieldPanel, JDialog dialog, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.segment = segment;
		this.field = field;
		this.fieldPanel = fieldPanel;
		this.parent = dialog;
		this.parentField = parentField;

		UserInterfaceUtil.setLabelLNF(fieldNameLbl);
		UserInterfaceUtil.setLabelLNF(fieldLabelLbl);
		UserInterfaceUtil.setLabelLNF(fieldPrefixLbl);
		UserInterfaceUtil.setLabelLNF(fieldColorLbl);
		UserInterfaceUtil.setLabelLNF(fieldGroupLbl);
		UserInterfaceUtil.setLabelLNF(newValueNameLbl);
		UserInterfaceUtil.setLabelLNF(newValueLabelLbl);

		UserInterfaceUtil.setTextFieldLNF(fieldNameTxt);
		UserInterfaceUtil.setTextFieldLNF(fieldLabelTxt);
		UserInterfaceUtil.setTextFieldLNF(fieldPrefixTxt);
		UserInterfaceUtil.setTextFieldLNF(newValueNameTxt);
		UserInterfaceUtil.setTextFieldLNF(newValueLabelTxt);

		UserInterfaceUtil.setRadioBtnLNF(inputText);
		UserInterfaceUtil.setRadioBtnLNF(inputTextFree);
		UserInterfaceUtil.setRadioBtnLNF(inputTextMenu);
		UserInterfaceUtil.setRadioBtnLNF(inputGraphic);
		UserInterfaceUtil.setRadioBtnLNF(rightAlignment);
		UserInterfaceUtil.setRadioBtnLNF(leftAlignment);
		UserInterfaceUtil.setRadioBtnLNF(centerAlignment);
		UserInterfaceUtil.setRadioBtnLNF(rightNLeftAlignment);
		UserInterfaceUtil.setComboBoxLNF(fieldGroupCb);

		Dimension btnSize = deleteValueBtn.getPreferredSize();
		doneBtn.setPreferredSize(btnSize);
		cancelBtn.setPreferredSize(btnSize);
		newValueBtn.setPreferredSize(btnSize);

		doneBtn.setActionCommand("" + DONE);
		doneBtn.addActionListener(this);

		cancelBtn.setActionCommand("" + CANCEL);
		cancelBtn.addActionListener(this);

		newValueBtn.setActionCommand("" + ADD_VALUE);
		newValueBtn.addActionListener(this);

		deleteValueBtn.setActionCommand("" + REMOVE_VALUE);
		deleteValueBtn.addActionListener(this);

		fieldTypeInputPanel = createFieldTypeInputPanel();
		valueAlignmentPanel = createValuesAlignmentPanel();

		valuesTablePane = createValuesTable(field);

		deleteValueBtn.setEnabled(false);

		doneBtn.setEnabled(false);
		dmListener = new DocumentModelListener(fieldNameTxt, doneBtn);
		fieldNameTxt.getDocument().addDocumentListener(dmListener);

		fieldGroupCb.setSelectedIndex(group);

		setEnabled(updateFieldInfo);
		initRadioButtons();
		initUI();

		if (field != null) {
			fieldNameTxt.setText(field.getName());
			fieldLabelTxt.setText(field.getLabel());
			fieldPrefixTxt.setText(field.getPrefix());
			fieldColorChooser.setColor(field.getColor());

			inputText.setSelected(true);
			
			SS3Field.Alignment alignment = field.getAlignment();
			if (alignment == SS3Field.Alignment.RIGHT) {
				rightAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.LEFT) {
				leftAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.CENTER) {
				centerAlignment.setSelected(true);
			} else {
				rightNLeftAlignment.setSelected(true);
			}

			ArrayList<SS3FieldValue> values = field.getValuesAsList();
			if (Constants.TYPE_NON_MANUAL.equals(field.getType()) || values.size() > 0) {
				inputTextMenu.setSelected(true);
			} else {
				inputTextFree.setSelected(true);
			}
		}
	}

	private void initRadioButtons() {
		inputGraphic.setEnabled(false);

		inputText.setSelected(true);
		// inputTextFree.setSelected(true);
		inputTextMenu.setSelected(true);

		if (!inputText.isSelected()) {
			inputTextFree.setEnabled(false);
			inputTextMenu.setEnabled(false);
		}

		ButtonGroup grp1 = new ButtonGroup();
		ButtonGroup grp2 = new ButtonGroup();
		ButtonGroup grp3 = new ButtonGroup();

		grp1.add(inputText);
		grp1.add(inputGraphic);

		grp2.add(inputTextFree);
		grp2.add(inputTextMenu);

		grp3.add(rightAlignment);
		grp3.add(leftAlignment);
		grp3.add(centerAlignment);
		grp3.add(rightNLeftAlignment);

		inputText.addItemListener(this);
		inputTextFree.addItemListener(this);
		inputTextMenu.addItemListener(this);
		inputGraphic.addItemListener(this);

		inputText.setActionCommand("" + INPUT_TEXT);
		inputTextFree.setActionCommand("" + INPUT_TEXT_FREE);
		inputTextMenu.setActionCommand("" + INPUT_TEXT_MENU);
		inputGraphic.setActionCommand("" + INPUT_TEXT);

//		inputText.setEnabled(false);
//		inputTextFree.setEnabled(false);
//		inputTextMenu.setEnabled(false);
//		inputGraphic.setEnabled(false);
//
//		rightAlignment.setEnabled(false);
//		leftAlignment.setEnabled(false);
//		centerAlignment.setEnabled(false);
//		rightNLeftAlignment.setEnabled(false);
		
		if (field != null) {
			SS3Field.Alignment alignment = field.getAlignment();
			if (alignment == SS3Field.Alignment.RIGHT) {
				rightAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.LEFT) {
				leftAlignment.setSelected(true);
			} else if (alignment == SS3Field.Alignment.CENTER) {
				centerAlignment.setSelected(true);
			} else {
				rightNLeftAlignment.setSelected(true);
			}
		}
	}

	private void initUI() {
		setLayout(new GridBagLayout());

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.WEST;

		gbc.gridy = 0;
		gbc.gridx = 0;
		gbc.gridwidth = 1;
		add(fieldGroupLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldGroupCb, gbc);
		gbc.gridwidth = 1;
		gbc.gridx = 5;
		add(doneBtn, gbc);

		gbc.gridy = 1;
		gbc.gridx = 0;
		add(fieldNameLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldNameTxt, gbc);
		gbc.gridwidth = 1;
		gbc.gridx = 5;
		add(cancelBtn, gbc);

		gbc.gridy = 2;
		gbc.gridx = 0;
		add(fieldLabelLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldLabelTxt, gbc);
		gbc.gridwidth = 1;

		gbc.gridy = 3;
		gbc.gridx = 0;
		add(fieldPrefixLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 4;
		add(fieldPrefixTxt, gbc);
		gbc.gridwidth = 1;

		gbc.gridy = 4;
		gbc.gridx = 0;
		add(fieldColorLbl, gbc);
		gbc.gridx = 1;
		gbc.gridwidth = 3;
		add(fieldColorChooser, gbc);
		gbc.gridwidth = 1;

		gbc.gridy = 5;
		gbc.gridx = 0;
		gbc.gridwidth = 2;
		add(fieldTypeInputPanel, gbc);
		gbc.gridx = 2;
		gbc.gridwidth = 3;
		add(valueAlignmentPanel, gbc);

		if (inputTextMenu.isSelected()) {
			gbc.gridy = 10;
			gbc.gridx = 0;
			gbc.gridwidth = 5;
			gbc.gridheight = 15;
			add(valuesTablePane, gbc);
			gbc.gridwidth = 1;
			gbc.gridheight = 1;

			gbc.gridy = 15;
			gbc.gridx = 5;
			add(newValueBtn, gbc);
			gbc.gridy = 17;
			add(deleteValueBtn, gbc);

			if (showNewValueFields) {
				gbc.gridy = 25;
				gbc.gridx = 0;
				add(newValueNameLbl, gbc);
				gbc.gridx = 1;
				gbc.gridwidth = 4;
				add(newValueNameTxt, gbc);
				gbc.gridwidth = 1;

				gbc.gridy = 27;
				gbc.gridx = 0;
				add(newValueLabelLbl, gbc);
				gbc.gridx = 1;
				gbc.gridwidth = 4;
				add(newValueLabelTxt, gbc);
			}
		}
	}

	public String getNameTxt() {
		return fieldNameTxt.getText();
	}

	@Override
	public void itemStateChanged(ItemEvent event) {
		JRadioButton obj = (JRadioButton) event.getSource();
		String str = obj.getActionCommand();

		switch (Integer.parseInt(str)) {
		case INPUT_TEXT:
			if (inputText.isSelected()) {
				inputTextFree.setEnabled(true);
				inputTextMenu.setEnabled(true);
			}
			break;

		case INPUT_TEXT_FREE:
			if (inputTextFree.isSelected()) {
				remove(valuesTablePane);
				remove(newValueBtn);
				remove(deleteValueBtn);
				remove(newValueNameLbl);
				remove(newValueNameTxt);
				remove(newValueLabelLbl);
				remove(newValueLabelTxt);
			}
			break;

		case INPUT_TEXT_MENU:
			if (inputTextMenu.isSelected()) {
				gbc.gridy = 10;
				gbc.gridx = 0;
				gbc.gridwidth = 5;
				gbc.gridheight = 15;
				add(valuesTablePane, gbc);
				gbc.gridwidth = 1;
				gbc.gridheight = 1;

				gbc.gridy = 15;
				gbc.gridx = 5;
				add(newValueBtn, gbc);
				gbc.gridy = 17;
				add(deleteValueBtn, gbc);

				if (showNewValueFields) {
					gbc.gridy = 25;
					gbc.gridx = 0;
					add(newValueNameLbl, gbc);
					gbc.gridx = 1;
					gbc.gridwidth = 4;
					add(newValueNameTxt, gbc);
					gbc.gridwidth = 1;

					gbc.gridy = 27;
					gbc.gridx = 0;
					add(newValueLabelLbl, gbc);
					gbc.gridx = 1;
					gbc.gridwidth = 4;
					add(newValueLabelTxt, gbc);
				}
			}
			break;

		case INPUT_GRAPHIC:
			if (inputGraphic.isSelected()) {
				inputTextFree.setEnabled(false);
				inputTextMenu.setEnabled(false);
			}
			break;
		}
		parent.pack();
	}

	private JPanel createFieldTypeInputPanel() {
		JPanel panel = new JPanel();
		panel.setLayout(new GridBagLayout());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.LINE_START;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.gridwidth = 1;
		gbc.insets = new Insets(1, 1, 1, 1);
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;
		panel.add(inputText, gbc);

		int hStrut = 25;
		panel.add(Box.createHorizontalStrut(hStrut));
		panel.add(inputTextFree, gbc);
		panel.add(Box.createHorizontalStrut(hStrut));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;
		panel.add(inputTextMenu, gbc);

		gbc.gridwidth = 1;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.fill = GridBagConstraints.NONE;
		panel.add(inputGraphic, gbc);

		panel.setBorder(new TitledBorder("Field input type"));
		return panel;
	}

	private JPanel createValuesAlignmentPanel() {
		JPanel panel = new JPanel();
		Dimension d = new Dimension(120, 17);

		ValueAlignmentPanel v1 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_RIGHT);
		ValueAlignmentPanel v2 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_LEFT);
		ValueAlignmentPanel v3 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_CENTER);
		ValueAlignmentPanel v4 = new ValueAlignmentPanel(ValueAlignmentPanel.ALGMNT_RIGHT_N_LEFT);

		v1.setPreferredSize(d);
		v2.setPreferredSize(d);
		v3.setPreferredSize(d);
		v4.setPreferredSize(d);

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		panel.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.gridwidth = 1;
		gbc.insets = new Insets(1, 1, 1, 1);
		panel.add(rightAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v1, gbc);

		gbc.gridwidth = 1;
		panel.add(leftAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v2, gbc);

		gbc.gridwidth = 1;
		panel.add(centerAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v3, gbc);

		gbc.gridwidth = 1;
		panel.add(rightNLeftAlignment, gbc);
		panel.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		panel.add(v4, gbc);

		panel.setBorder(new TitledBorder("Alignment of values"));
		return panel;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case DONE:
			if (fieldNameTxt.getText().length() == 0) {
				String label = "<HTML>The field's name and/or label has no text.<BR>Please provide some text for it or else<BR>use the Cancel button to exit this editor.</HTML>";
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", label);
				newValueNameTxt.requestFocus();

				return;

			} else if (inputTextMenu.isSelected() && valuesTableModel.getRowCount() == 0) {
				String label = "<HTML>The field does not have values. Please provide at least<BR>one value or change fields input type to Free Text.</HTML>";
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", label);
				newValueNameTxt.requestFocus();

				return;

			} else if (fieldGroupCb.getSelectedIndex() == 4) {
				String label = "<HTML>The field must belong to a specific group of existing fields.<BR>Please select a group from the dropdown.</HTML>";
				SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", label);
				newValueNameTxt.requestFocus();

				return;

			} else {
				int groupID = fieldGroupCb.getSelectedIndex();
				String fieldName = fieldNameTxt.getText();
				String fieldLabel = fieldLabelTxt.getText();

				CSFieldPanel csFieldPanel = null;
				GraphFieldPanel graphFieldPanel = null;

				if (fieldPanel instanceof CSFieldPanel) {
					csFieldPanel = (CSFieldPanel) fieldPanel;
				} else if (fieldPanel instanceof GraphFieldPanel) {
					graphFieldPanel = (GraphFieldPanel) fieldPanel;
				}

				if (csFieldPanel != null) {
					// not unique
					if (field == null && !csFieldPanel.isFieldNameAndLabelUnique(fieldName, fieldLabel, groupID)) {
						StringBuffer nameNotUniqueBfr = new StringBuffer();
						nameNotUniqueBfr.append("<HTML>The field name and/or label are already in use by<BR>");
						nameNotUniqueBfr.append("another field. Please change it or use the Cancel <BR>");
						nameNotUniqueBfr.append("button to exit this editor.</HTML>");

						SS3Singleton.getSignStreamApplication().confirmationDialog("Warning",
								nameNotUniqueBfr.toString());

						newValueNameTxt.requestFocus();
					} else {

						if (field == null) {
							csFieldPanel.addNewField(getCreatedOrUpdatedSS3Field(field));
							parent.setVisible(false);
						} else {
							SS3Field updatedField = getCreatedOrUpdatedSS3Field(field);
							csFieldPanel.updateField(updatedField);
							parent.setVisible(false); // .dispose();
						}
					}
				} else {
					if (field == null && !graphFieldPanel.isFieldNameAndLabelUnique(fieldName, fieldLabel, groupID)) {
						StringBuffer nameNotUniqueBfr = new StringBuffer();
						nameNotUniqueBfr.append("<HTML>The field name and/or label are already in use by<BR>");
						nameNotUniqueBfr.append("another field. Please change it or use the Cancel <BR>");
						nameNotUniqueBfr.append("button to exit this editor.</HTML>");

						SS3Singleton.getSignStreamApplication().confirmationDialog("Warning",
								nameNotUniqueBfr.toString());

						newValueNameTxt.requestFocus();
					} else {

						if (field == null) {
							graphFieldPanel.addNewField(getCreatedOrUpdatedSS3Field(field));
							parent.setVisible(false);
						} else {
							SS3Field updatedField = getCreatedOrUpdatedSS3Field(field);
							graphFieldPanel.updateField(updatedField);
							parent.setVisible(false); // .dispose();
						}
					}
				}
			}
			discard();
			break;
		case CANCEL:
			StringBuffer cancelBfr = new StringBuffer();
			cancelBfr.append("<HTML>If proceeded, all entered data will be lost. <BR><BR>");
			cancelBfr.append("Are you sure you want to continue?</HTML>");

			final JDialog dialog = new JDialog(parent, "Save Resource", true);
			final JButton cancelBtn = new JButton("Cancel");
			final JButton continueBtn = new JButton("Continue");

			continueBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					ActionListener[] al = cancelBtn.getActionListeners();
					for (int i = 0; i < al.length; i++) {
						cancelBtn.removeActionListener(al[i]);
					}

					ActionListener[] cl = continueBtn.getActionListeners();
					for (int j = 0; j < cl.length; j++) {
						continueBtn.removeActionListener(cl[j]);
					}

					dialog.dispose();
					parent.setVisible(false);
					parent.dispose();
					discard();
				}

			});

			cancelBtn.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					ActionListener[] al = cancelBtn.getActionListeners();
					for (int i = 0; i < al.length; i++) {
						cancelBtn.removeActionListener(al[i]);
					}

					ActionListener[] cl = continueBtn.getActionListeners();
					for (int j = 0; j < cl.length; j++) {
						continueBtn.removeActionListener(cl[j]);
					}

					dialog.dispose();
				}
			});

			ButtonPanel buttonPanel = new ButtonPanel();
			buttonPanel.add(cancelBtn);
			buttonPanel.add(continueBtn);

			JPanel panel = new JPanel();
			panel.setLayout(new BorderLayout());
			JLabel l1 = new JLabel(cancelBfr.toString());
			panel.add(l1, BorderLayout.CENTER);
			panel.add(buttonPanel, BorderLayout.SOUTH);

			panel.add(new JLabel("  "), BorderLayout.WEST);

			Dimension d = panel.getPreferredSize();
			Dimension nD = new Dimension(d.width + 10, d.height + 40);
			panel.setPreferredSize(nD);

			Container contentPane = dialog.getContentPane();
			contentPane.setLayout(new BorderLayout(40, 40));
			contentPane.add(panel, BorderLayout.CENTER);

			dialog.setLocation(new Point(150, 150));
			dialog.pack();
			dialog.setVisible(true);

			break;
		case ADD_VALUE:
			int rowCount = valuesTableModel.getRowCount();
			int selectedRow = valuesTable.getSelectedRow();

			if (selectedRow > -1 && rowCount > 0) {
				String newValueName = (String) valuesTable.getValueAt(selectedRow, 0);
				String newValueLabel = (String) valuesTable.getValueAt(selectedRow, 1);

				if (newValueName.trim().length() == 0 && newValueLabel.trim().length() == 0) {
					StringBuffer noNameBfr = new StringBuffer();
					noNameBfr.append("<HTML>The selected value name and/or label has no text. <BR>");
					noNameBfr.append("Please provide some text for it or delete the value.</HTML>");

					SS3Singleton.getSignStreamApplication().confirmationDialog("Warning", noNameBfr.toString());

					newValueNameTxt.requestFocus();
					return;
				} else {
					if (newValueName.trim().length() == 0) {
						valuesTableModel.setValueAt(newValueLabel, selectedRow, 0);
						newValueName = newValueLabel;
					}

					if (newValueLabel.trim().length() == 0) {
						valuesTableModel.setValueAt(newValueName, selectedRow, 1);
						newValueLabel = newValueName;
					}

					for (int i = 0; i < rowCount; i++) {
						if (selectedRow != i) {
							String value = (String) valuesTableModel.getValueAt(i, 0);
							String label = (String) valuesTableModel.getValueAt(i, 1);
							if (value.equals(newValueName) || label.equals(newValueLabel)) {
								StringBuffer notUniqueNameOrLabelMsg = new StringBuffer();
								notUniqueNameOrLabelMsg
										.append("<HTML>The selected value name and/or label is already <BR>");
								notUniqueNameOrLabelMsg
										.append("in use by another value within this field. Please <BR>");
								notUniqueNameOrLabelMsg
										.append("change it, delete it or use Cancel button to exit <BR>");
								notUniqueNameOrLabelMsg.append("this editor.</HTML>");

								SS3Singleton.getSignStreamApplication().confirmationDialog("Warning",
										notUniqueNameOrLabelMsg.toString());

								newValueNameTxt.requestFocus();
								return;
							}
						}
					}

					if (valuesTable.getRowCount() > 0) {
						deleteValueBtn.setEnabled(true);
					} else {
						deleteValueBtn.setEnabled(false);
					}
				}
			}

			Object[] obj = { newValueNameTxt.getText().trim(), newValueLabelTxt.getText().trim(),
					"" + Util.getUniqueNumber() };
			valuesTableModel.addRow(obj);
			ListSelectionModel selectionModel = valuesTable.getSelectionModel();
			selectionModel.setSelectionInterval(valuesTableModel.getRowCount() - 1, valuesTableModel.getRowCount() - 1);

			if (!showNewValueFields) {
				showNewValueFields = true;

				gbc.gridy = 25;
				gbc.gridx = 0;
				add(newValueNameLbl, gbc);
				gbc.gridx = 1;
				gbc.gridwidth = 4;
				add(newValueNameTxt, gbc);
				gbc.gridwidth = 1;

				gbc.gridy = 27;
				gbc.gridx = 0;
				add(newValueLabelLbl, gbc);
				gbc.gridx = 1;
				gbc.gridwidth = 4;
				add(newValueLabelTxt, gbc);

				parent.pack();
			}

			newValueNameTxt.setText("");
			newValueLabelTxt.setText("");
			newValueNameTxt.requestFocus();

			valuesTablePane.getVerticalScrollBar().revalidate();
			valuesTablePane.getVerticalScrollBar().setValue(valuesTablePane.getVerticalScrollBar().getMaximum() + 10);

			setRowVisible(valuesTableModel.getRowCount() - 1);
			break;
		case REMOVE_VALUE:
			final int row = valuesTable.getSelectedRow();
			final int selectRow = row > 0 ? row - 1 : row;

			String valueID = (String) valuesTableModel.getValueAt(row, 2);

			if (valuesThatCantBeRemoved.contains(valueID)) {

				StringBuffer removeBfr = new StringBuffer();
				removeBfr.append("<HTML>Before a value specification is deleted, all instances where this <BR>");
				removeBfr.append("value has been coded should be located and deleted or recoded <BR>");
				removeBfr.append("from the fields defined by this field specification in all <BR>");
				removeBfr.append("databases/utterances manually by a user. <BR><BR>");
				removeBfr.append("(Note that after you have exit the main field specifications editor, <BR>");
				removeBfr.append("any changes that you have made are not undoable) <BR><BR>");
				removeBfr.append("Are you sure you want to continue?</HTML>");

				final JDialog dialog1 = new JDialog(parent, "Delete Resource", true);
				final JButton cancelBtn1 = new JButton("Cancel");
				final JButton continueBtn1 = new JButton("Continue");

				continueBtn1.addActionListener(new ActionListener() {

					@Override
					public void actionPerformed(ActionEvent arg0) {
						valuesTableModel.removeRow(row);
						if (valuesTable.getRowCount() > selectRow) {
							valuesTable.setRowSelectionInterval(selectRow, selectRow);
							setRowVisible(selectRow);
						}

						if (valuesTable.getRowCount() < 1) {
							deleteValueBtn.setEnabled(false);
							newValueNameTxt.setText("");
							newValueLabelTxt.setText("");
						}

						ActionListener[] al = cancelBtn1.getActionListeners();
						for (int i = 0; i < al.length; i++) {
							cancelBtn1.removeActionListener(al[i]);
						}

						ActionListener[] cl = continueBtn1.getActionListeners();
						for (int j = 0; j < cl.length; j++) {
							continueBtn1.removeActionListener(cl[j]);
						}

						dialog1.dispose();
					}
				});

				cancelBtn1.addActionListener(new ActionListener() {

					@Override
					public void actionPerformed(ActionEvent arg0) {

						ActionListener[] al = cancelBtn1.getActionListeners();
						for (int i = 0; i < al.length; i++) {
							cancelBtn1.removeActionListener(al[i]);
						}

						ActionListener[] cl = continueBtn1.getActionListeners();
						for (int j = 0; j < cl.length; j++) {
							continueBtn1.removeActionListener(cl[j]);
						}

						dialog1.dispose();
					}

				});

				ButtonPanel buttonPanel1 = new ButtonPanel();
				buttonPanel1.add(cancelBtn1);
				buttonPanel1.add(continueBtn1);

				JPanel panel1 = new JPanel();
				panel1.setLayout(new BorderLayout());
				JLabel label2 = new JLabel(removeBfr.toString());
				panel1.add(label2, BorderLayout.CENTER);
				panel1.add(buttonPanel1, BorderLayout.SOUTH);

				panel1.add(new JLabel("  "), BorderLayout.WEST);

				Dimension d1 = panel1.getPreferredSize();
				Dimension nD1 = new Dimension(d1.width + 10, d1.height + 40);
				panel1.setPreferredSize(nD1);

				Container contentPane1 = dialog1.getContentPane();
				contentPane1.setLayout(new BorderLayout(40, 40));
				contentPane1.add(panel1, BorderLayout.CENTER);

				dialog1.setLocation(new Point(150, 150));
				dialog1.pack();
				dialog1.setVisible(true);
			} else {

				valuesTableModel.removeRow(row);
				if (valuesTable.getRowCount() > selectRow) {
					valuesTable.setRowSelectionInterval(selectRow, selectRow);
					setRowVisible(selectRow);
				}

				if (valuesTable.getRowCount() < 1) {
					deleteValueBtn.setEnabled(false);
					newValueNameTxt.setText("");
					newValueLabelTxt.setText("");
				}
			}

			break;
		}
	}

	private void setRowVisible(int row) {
		Rectangle rect = valuesTable.getCellRect(row, 0, true);
		valuesTable.scrollRectToVisible(rect);
	}

	private SS3Field getCreatedOrUpdatedSS3Field(SS3Field csField) {
		boolean newField = false;

		if (csField == null) {
			newField = true;
			csField = new SS3Field();
			csField.setId(Util.getUniqueNumber());
		}

		boolean textFreeSelected = inputTextFree.isSelected();
		boolean textMenuSelected = inputTextMenu.isSelected();

		boolean rAlgnmnt = rightAlignment.isSelected();
		boolean lAlgnmnt = leftAlignment.isSelected();
		boolean cAlgnmnt = centerAlignment.isSelected();
		boolean rlAlgnmnt = rightNLeftAlignment.isSelected();

		csField.setGroupID(fieldGroupCb.getSelectedIndex());
		csField.setName(fieldNameTxt.getText());
		csField.setLabel(fieldLabelTxt.getText());
		csField.setPrefix(fieldPrefixTxt.getText());
		csField.setColor(fieldColorChooser.getColor());

		int groupIndex = fieldGroupCb.getSelectedIndex();

		switch (groupIndex) {
		case 0:
			csField.setGroup("HEAD");
			break;
		case 1:
			csField.setGroup("FACE");
			break;
		case 2:
			csField.setGroup("BODY");
			break;
		case 3:
			csField.setGroup("GRAMMATICAL");
			break;
		}

		if (textFreeSelected)
			csField.setType(Constants.TYPE_NON_MANUAL_FREE_TEXT);

		if (textMenuSelected) {
			csField.setType(Constants.TYPE_NON_MANUAL);
			Vector data = valuesTableModel.getDataVector();

			csField.removeAllValues(); // need to remove all existing values in case something was deleted by user
			// that user-deletion is so far only in the valuesTableModel object (and now the
			// data vector) and any deletions should
			// be remembered so that the csField does not keep old (deleted) values

			for (int i = 0; i < data.size(); i++) {
				Vector obj = (Vector) data.get(i);
				SS3FieldValue fValue = new SS3FieldValue();
				fValue.setId((String) obj.get(2));
				fValue.setName((String) obj.get(0));
				fValue.setLabel((String) obj.get(1));
				csField.addValue(fValue);
			}
		}

		if (rAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.RIGHT);

		if (lAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.LEFT);

		if (cAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.CENTER);

		if (rlAlgnmnt)
			csField.setAlignment(SS3Field.Alignment.RIGHT_LEFT);

		if (newField) {
			SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
			scheme.addField(csField);

			SS3Track track = new SS3Track();
			track.setId(csField.getId());
			NonManualField nmField = new NonManualField(track, segmentPanel, parentField);
			nmField.setName(csField.getLabelOrName());
			nmField.setColor(csField.getColor());
			NonManualFieldWrapper nmWrapper;
			if (csField.getType().equals(Constants.TYPE_NON_MANUAL_FREE_TEXT)) {
				nmWrapper = new NonManualFreeTextFieldWrapper(nmField, segmentPanel);
			} else {
				nmWrapper = new NonManualFieldWrapper(nmField, segmentPanel);
			}
			LabelObject lObj = new LabelObject(csField.getId(), csField.getLabelOrName());
			lObj.setWrapper(nmWrapper);

			segment.getRootLabelObject().getNonManuelLabelObject().addChild(lObj);
		} else {
			// if field already exists, then edit the name of the label, name of the
			// wrapper, and color of the field
			ArrayList<LabelObject> labelObjs = segment.getRootLabelObject().getNonManuelLabelObject()
					.getChildrenAsList();
			for (LabelObject obj : labelObjs) {
				if (obj.getFieldID().equals(csField.getId())) {
					obj.setLabel(csField.getLabelOrName());
					obj.getWrapper().getField().setName(csField.getLabelOrName());
					obj.getWrapper().getField().setColor(csField.getColor());
				}
			}

			SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
			scheme.addField(csField);
		}

		// SS3GlossWindowItem item = segment.getSS3GlossWindowItem();
		// SS3StatementField statementField =
		// item.getSS3StatementField(csField.getId());
		// if (statementField != null) {
		// statementField.setColor(csField.getColor());
		// }

		return csField;
	}

	private JScrollPane createValuesTable(SS3Field csField) {
		ArrayList<Object[]> valueList = new ArrayList<Object[]>();

		if (csField != null) {
			ArrayList<SS3FieldValue> values = csField.getValuesAsList();
			for (int i = 0; i < values.size(); i++) {
				SS3FieldValue csValue = values.get(i);
				Object[] value = { csValue.getName(), csValue.getLabel(), csValue.getId() };
				valueList.add(value);

				valuesThatCantBeRemoved.add(csValue.getId());
			}
		}

		Object[][] valueData = new Object[valueList.size()][];

		for (int j = 0; j < valueList.size(); j++) {
			valueData[j] = valueList.get(j);
		}

		Object[] valuesColumnNames = { "Value Names", "Value Labels", "" };

		valuesTableModel = new DefaultTableModel(valueData, valuesColumnNames);
		TableSorter sorter = new TableSorter(valuesTableModel);
		valuesTable = new JTable(sorter);

		if (updateNameValueListener == null)
			updateNameValueListener = new UpdateValueListener(newValueNameTxt, valuesTable, valuesTableModel, 0,
					deleteValueBtn);

		newValueNameTxt.getDocument().addDocumentListener(updateNameValueListener);

		if (updateLabelValueListener == null)
			updateLabelValueListener = new UpdateValueListener(newValueLabelTxt, valuesTable, valuesTableModel, 1,
					deleteValueBtn);

		newValueLabelTxt.getDocument().addDocumentListener(updateLabelValueListener);

		sorter.setTableHeader(valuesTable.getTableHeader());
		valuesTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		tableSelectionListener = new TableSelectionListener1(valuesTable, newValueNameTxt, newValueLabelTxt,
				deleteValueBtn);
		valuesTable.getSelectionModel().addListSelectionListener(tableSelectionListener);
		valuesTable.getColumnModel().removeColumn(valuesTable.getColumnModel().getColumn(2));
		valuesTable.setAutoCreateColumnsFromModel(false);

		JScrollPane valuesTablePane = new JScrollPane(valuesTable);
		valuesTablePane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		valuesTablePane.setPreferredSize(new Dimension(valueAlignmentPanel.getPreferredSize().width - 5,
				valueAlignmentPanel.getPreferredSize().height));

		return valuesTablePane;
	}

	public int getPreferredWidthForColumn(TableColumn col) {
		int hw = columnHeaderWidth(col), // hw = header width
				cw = widestCellInColumn(col); // cw = column width
		return hw > cw ? hw : cw;
	}

	private int columnHeaderWidth(TableColumn col) {
		// TableCellRenderer renderer = col.getHeaderRenderer();
		return 75;
	}

	private int widestCellInColumn(TableColumn col) {
		int c = col.getModelIndex(), width = 0, maxw = 0;

		for (int r = 0; r < valuesTable.getRowCount(); ++r) {
			TableCellRenderer renderer = valuesTable.getCellRenderer(r, c);

			Component comp = renderer.getTableCellRendererComponent(valuesTable, valuesTable.getValueAt(r, c), false,
					false, r, c);

			width = comp.getPreferredSize().width;
			maxw = width > maxw ? width : maxw;
		}
		return maxw + 10;
	}

	public void discard() {
		doneBtn.removeActionListener(this);
		cancelBtn.removeActionListener(this);
		newValueBtn.removeActionListener(this);
		deleteValueBtn.removeActionListener(this);

		newValueNameTxt.getDocument().removeDocumentListener(updateNameValueListener);
		newValueLabelTxt.getDocument().removeDocumentListener(updateLabelValueListener);

		valuesTable.getSelectionModel().removeListSelectionListener(tableSelectionListener);

		fieldNameTxt.getDocument().removeDocumentListener(dmListener);

		inputText.removeItemListener(this);
		inputTextFree.removeItemListener(this);
		inputTextMenu.removeItemListener(this);
		inputGraphic.removeItemListener(this);

		removeAll();
	}
}

class TableSelectionListener1 implements ListSelectionListener {
	private JTable table;
	private JTextField newValueNameTxt;
	private JTextField newValueLabelTxt;
	private JButton deleteValueBtn;

	TableSelectionListener1(JTable table, JTextField newValueNameTxt, JTextField newValueLabelTxt,
			JButton deleteValueBtn) {
		this.table = table;
		this.newValueNameTxt = newValueNameTxt;
		this.newValueLabelTxt = newValueLabelTxt;
		this.deleteValueBtn = deleteValueBtn;
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {
		SwingUtilities.invokeLater(new Runnable () {
			public void run() {
				int row = table.getSelectedRow();

				if (row >= 0) {
					String name = (String) table.getValueAt(row, 0);
					newValueNameTxt.setText(name);
					newValueLabelTxt.setText((String) table.getValueAt(row, 1));
					deleteValueBtn.setEnabled(true);
					newValueNameTxt.requestFocus();
				}
			}
		});

	}
}

class UpdateValueListener implements DocumentListener {
	private JTextField txt = null;
	private JTable table = null;
	private DefaultTableModel tableModel = null;
	private int column;
	private JButton deleteValueBtn;

	public UpdateValueListener(JTextField txt, JTable table, DefaultTableModel tableModel, int column,
			JButton deleteValueBtn) {
		super();
		this.txt = txt;
		this.table = table;
		this.tableModel = tableModel;
		this.column = column;
		this.deleteValueBtn = deleteValueBtn;
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		updateSelectedColumn();
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		updateSelectedColumn();
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		updateSelectedColumn();
	}

	private void updateSelectedColumn() {
		String newValue = txt.getText();
		int selectedRow = table.getSelectedRow();

		if (selectedRow < 0) {
			Object[] obj = { "", "", "" + Util.getUniqueNumber() };
			tableModel.addRow(obj);
			ListSelectionModel selectionModel = table.getSelectionModel();
			selectionModel.setSelectionInterval(tableModel.getRowCount() - 1, tableModel.getRowCount() - 1);
			deleteValueBtn.setEnabled(true);
		}

		table.setValueAt(newValue, selectedRow, column);
	}
}