package edu.bu.signstream.ui.panels.codingScheme;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.*;
import java.util.List;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.*;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.grepresentation.view.*;
import org.jdesktop.swingx.JXTreeTable;
import org.jdesktop.swingx.treetable.AbstractMutableTreeTableNode;
import org.jdesktop.swingx.treetable.DefaultTreeTableModel;
import org.jdesktop.swingx.treetable.MutableTreeTableNode;
import org.jdesktop.swingx.treetable.TreeTableNode;

import edu.bu.signstream.common.util.SegmentGraphReprUtil;
import edu.bu.signstream.common.util.Sorter;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.FieldsGroup;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3Field;
import edu.bu.signstream.grepresentation.fields.FieldWrapper;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.grepresentation.fields.presentation.PresentationField;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;

// TODO: Refactor this. The actual logic here should take 10x less code.
public class CSFieldPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	private JDialog parent = null;

	private SignStreamSegmentPanel segmentPanel;
	private Segment segment;
	private PresentationField parentField;
	private NonmanualFieldsView nonManualView;
	private GlossNRelatedView glossNRelatedView;
	private SegmentGraphicRepresentation rep;
	private CodingSchemaFieldNode fieldsRootNode;
	private CodingSchemaFieldNode manualFieldsRootNode;
	private JXTreeTable nmFieldsTable;
	private JXTreeTable manualFieldsTable;

	private JDialog editFieldDialog = null;
	private AddOrUpdateField editFieldPanel = null;

	private SelectionListener selectionListener = null;
	private MutableTreeTableNode selectedNode = null;

	private JButton newFieldBtn = new JButton("New Nonmanual Field");
	private JButton deleteBtn = new JButton("Delete");
	private JButton editBtn = new JButton("Edit Nonmanual Field");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton doneBtn = new JButton("Done");

	private final List<String> glossAndHandFieldIDs = Arrays.asList(
			Util.DOM_GLOSS_FIELD_ID, Util.NDOM_GLOSS_FIELD_ID, Util.DOM_HAND_FIELD_ID, Util.NDOM_HAND_FIELD_ID,
			Util.FREE_TEXT_FIELD_ID
	);

	private final int NEW_FIELD = 8;
	private final int DELETE = 9;
	private final int EDIT = 10;
	private final int CANCEL = 11;
	private final int DONE = 12;

	private GridBagConstraints gbc = new GridBagConstraints();

	// field ids that might have been used to code data
	private ArrayList<String> fieldIDs = new ArrayList<String>();

	// fields recently created and added to the coding schema
	private ArrayList<SS3Field> recentlyAddedFields = new ArrayList<SS3Field>();

	private JDialog newFieldDialog = null;
	private AddOrUpdateField newFieldPanel = null;

	public CSFieldPanel(SignStreamSegmentPanel segmentPanel, Segment segment, NonmanualFieldsView nonManualView,
			JDialog dialog, GlossNRelatedView glossNRelatedView, PresentationField parentField) {
		this.segmentPanel = segmentPanel;
		this.nonManualView = nonManualView;
		this.glossNRelatedView = glossNRelatedView;
		this.segment = segment;
		this.parent = dialog;
		this.parentField = parentField;

		initUI();
	}
	
	private void initUI() {
		deleteBtn.setEnabled(false);
		editBtn.setEnabled(false);

		newFieldBtn.setActionCommand("" + NEW_FIELD);
		newFieldBtn.addActionListener(this);
		deleteBtn.setActionCommand("" + DELETE);
		deleteBtn.addActionListener(this);
		editBtn.setActionCommand("" + EDIT);
		editBtn.addActionListener(this);
		cancelBtn.setActionCommand("" + CANCEL);
		cancelBtn.addActionListener(this);
		doneBtn.setActionCommand("" + DONE);
		doneBtn.addActionListener(this);

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		selectionListener = new SelectionListener(this, scheme);

		JScrollPane scrollPane = getTreeTableScrollPanel();
		Dimension d = new Dimension(scrollPane.getPreferredSize().width + 260, 250);

		scrollPane.setPreferredSize(d);
		scrollPane.setMinimumSize(d);
		scrollPane.setMaximumSize(d);

		setLayout(new GridBagLayout());

		gbc.fill = GridBagConstraints.BOTH;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = new Insets(1, 1, 1, 1);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 9;
		add(scrollPane, gbc);

		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 0;
		gbc.weighty = 0;
		gbc.gridwidth = 1;
		gbc.gridx = 10;
		add(newFieldBtn, gbc);

		gbc.gridx = 0;
		gbc.gridy = 14;
		add(deleteBtn, gbc);

		gbc.gridx = 1;
		add(editBtn, gbc);
		gbc.gridx = 2;
		add(Box.createHorizontalStrut(365), gbc);
		gbc.gridx = 8;
		add(cancelBtn, gbc);
		gbc.gridx = 10;
		add(doneBtn, gbc);

		for (int i = 0; i < 5; i++) {
			manualFieldsTable.getColumnModel().getColumn(i).setPreferredWidth(nmFieldsTable.getColumnModel().getColumn(i).getPreferredWidth());
		}
	}

	public void setSelectedTreeTableNode(MutableTreeTableNode selectedNode) {
		this.selectedNode = selectedNode;

		deleteBtn.setEnabled(false);
		editBtn.setEnabled(false);

		if (selectedNode != null && selectedNode.isLeaf()) {
			if (fieldsRootNode.equals(selectedNode.getParent())) {
				cancelBtn.setEnabled(true);
				doneBtn.setEnabled(true);
			} else {
				boolean doNext = true;
				while (doNext) {
					TreeTableNode pNode = selectedNode.getParent();
					if (fieldsRootNode.equals(pNode.getParent())) {
						doNext = false;
						int selectedGroup = getGroup(pNode);
						if (selectedGroup != 4) {
							editBtn.setEnabled(true);
						}
					}
				}
			}
		}
	}

	public boolean isFieldNameAndLabelUnique(String fieldName, String fieldLabel, int groupID) {
		MutableTreeTableNode node = groupID >= 0 && groupID < 4
				? (MutableTreeTableNode) fieldsRootNode.getChildAt(groupID)
				: fieldsRootNode;

		int numberOfChildren = node.getChildCount();
		for (int i = 0; i < numberOfChildren; i++) {
			MutableTreeTableNode childNode = (MutableTreeTableNode) node.getChildAt(i);
			String existingFieldName = (String) childNode.getValueAt(0);
			String existingFieldLabel = (String) childNode.getValueAt(1);

			if (existingFieldName.equals(fieldName) || existingFieldLabel.equals(fieldLabel)) {
				return false;
			}
		}
		return true;
	}

	public void addNewField(SS3Field field) {
		recentlyAddedFields.add(field);

		segmentPanel.getSS3Database().getLocalSS3CodingScheme().addField(field);

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		scheme.addField(field);

		int groupID = field.getGroupID();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
		MutableTreeTableNode node = groupID >= 0 && groupID < 4
				? (MutableTreeTableNode) fieldsRootNode.getChildAt(groupID)
				: fieldsRootNode;

		Object[] newFieldObject = new Object[] { field.getName(), field.getLabel(), new Boolean(true), " ",
				field.getColor(), "" + field.getId() };
		CodingSchemaFieldNode newFieldNode = new CodingSchemaFieldNode(newFieldObject);
		fieldsModel.insertNodeInto(newFieldNode, node, node.getChildCount());

		scrollToAndSelectNode(newFieldNode);
	}

	public void updateField(SS3Field field) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();

		// check if field is still in the same group
		int legacyGroupID = getGroup(selectedNode.getParent());
		int groupID = field.getGroupID();

		segmentPanel.getSS3Database().getLocalSS3CodingScheme().addField(field);
		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		scheme.addField(field);

		Object[] newFieldObject = new Object[] { field.getName(), field.getLabel(), new Boolean(true), " ",
				field.getColor(), "" + field.getId() };
		fieldsModel.setUserObject(selectedNode, newFieldObject);

		if (legacyGroupID != groupID) {
			fieldsModel.removeNodeFromParent(selectedNode);
			addNewField(field);
		}
	}

	private void createNewFieldDialog() {
		if (newFieldDialog == null) {
			newFieldDialog = new JDialog(parent, "Field Specifications Editor", true);
		}

		newFieldPanel = new AddOrUpdateField(segmentPanel, segment, getGroup(selectedNode), null, this, newFieldDialog,
				parentField);
		newFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

		newFieldDialog.pack();

		Container newFieldContentPane = newFieldDialog.getContentPane();
		newFieldContentPane.setLayout(new BorderLayout());
		newFieldContentPane.add(newFieldPanel, BorderLayout.CENTER);

		newFieldDialog.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent we) {
				newFieldDialog.setVisible(false);
			}
		});

		newFieldDialog.setSize(newFieldDialog.getPreferredSize());
		newFieldDialog.setLocation(150, 150);
		newFieldDialog.setVisible(true);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		SS3CodingScheme defaultCodingScheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		SS3CodingScheme localCodingScheme = segmentPanel.getSS3Database().getLocalSS3CodingScheme();	

		switch (Integer.parseInt(str)) {
		case NEW_FIELD:
			createNewFieldDialog();
			break;

		case EDIT:

			SS3Field fieldToEdit = null;
			if (selectedNode != null) {
				String id = (String) selectedNode.getValueAt(5);
				fieldToEdit = localCodingScheme.getField(id) != null 
						? localCodingScheme.getField(id)
						: defaultCodingScheme.getField(id);
			}

			if (editFieldDialog == null) {
				editFieldDialog = new JDialog(parent, "Field Specifications Editor", true);
			}

			editFieldPanel = new AddOrUpdateField(segmentPanel, segment, getGroup(selectedNode.getParent()),
					fieldToEdit, this, editFieldDialog, parentField);
			editFieldPanel.setBorder(BorderFactory.createEmptyBorder(15, 15, 15, 15));

			editFieldDialog.pack();
			editFieldDialog.setLayout(new BorderLayout());
			editFieldDialog.add(editFieldPanel, BorderLayout.CENTER);

			editFieldDialog.addWindowListener(new WindowAdapter() {
				@Override
				public void windowClosing(WindowEvent we) {
					if (editFieldPanel != null) {
						editFieldPanel.discard();
						editFieldPanel = null;
					}
					editFieldDialog.dispose();
				}
			});

			editFieldDialog.setSize(editFieldDialog.getPreferredSize());
			editFieldDialog.setLocation(150, 150);
			editFieldDialog.setVisible(true);

			break;

		case CANCEL:
		    discard();
			parent.dispose();
			break;
		case DONE:
			apply();
			parent.dispose();
			break;
		}
	}

    @Deprecated
	public void enter() {
		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		SS3CodingScheme localScheme = segmentPanel.getSS3Database().getLocalSS3CodingScheme();

		ArrayList<String> slctFieldIDs = getSelectedFieldIDsNUpdateCodingSchema();
		HashMap<String, Color> selectedColors = getSelectedColors();

		LabelObject nonManuelLabelObject = segment.getRootLabelObject().getNonManuelLabelObject();
		LabelObject manualLabelObject = segment.getRootLabelObject().getGlossNRelatedLabelObject();

		ArrayList<LabelObject> children = nonManuelLabelObject.getAllChildren();
		children.addAll(manualLabelObject.getAllChildren());

		// make all not visible and set their colors
		for (LabelObject child : children) {
			if (!glossAndHandFieldIDs.contains(child.getFieldID())) {
				child.setVisible(false);
			}

			String fieldID = child.getFieldID();
			Color color = selectedColors.get(fieldID);
			child.getWrapper().getField().setColor(color);
			SS3Field currentField = scheme.getField(fieldID);
			SS3Field localField = localScheme.getField(fieldID);
			if (localField != null) {
				localField.setColor(color);
			} else if (!currentField.getColor().equals(color)) {
				localScheme.addField(currentField.cloneField());
			}
			currentField.setColor(color);
		}

		// set to true only those fields that should be visible
		for (LabelObject child : children) {
			String id = child.getFieldID();

			if (slctFieldIDs.remove(id)) {
				child.setVisible(true);
			}
		}

		if (segmentPanel.getUtteranceView() != null) {
			// for each segment currently in view in UtteranceView, delete empty entities
			// from its nonmanual fields
			ArrayList<Segment> segmentsList = segmentPanel.getUtteranceView().getSegments();
			for (Segment uvSegment : segmentsList) {
				LabelObject uvObject = uvSegment.getRootLabelObject().getNonManuelLabelObject();
				ArrayList<LabelObject> uvChildren = uvObject.getAllChildren();

				// delete empty entities from nonmanual fields
				for (LabelObject uvChild : uvChildren) {
					FieldWrapper wrapper = uvChild.getWrapper();
					wrapper.getField().deleteEmptyEntities();
				}
			}
		}

		nonManualView.update(segment);
		nonManualView.getLabelView().loadWrappers();
		glossNRelatedView.update(segment);

		nonManualView.getLabelView().updateUI();
		glossNRelatedView.getLableView().updateUI();

		nonManualView.getLabelView().repaint();
		glossNRelatedView.getLableView().repaint();

		parent.dispose();
	}

	public JScrollPane getTreeTableScrollPanel() {
		populateFieldsTable();
		populateManualFieldsTable();

		String[] cNames = { " Field  Names   ", "Field Labels", "Display", "Data", "Color", " " };

		DefaultTreeTableModel fieldsModel = new DefaultTreeTableModel(fieldsRootNode, Arrays.asList(cNames)) {
			@Override
			public Class<?> getColumnClass(int column) {

				if (column == 2) {
					return Boolean.class;
				} else if (column == 4) {
					return Color.class;
				} else {
					return String.class;
				}
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
				return column == 4 || column == 2;
			}

			@Override
			public int getChildCount(Object parent) {
				// Only override count if at the root of the tree.
				if (parent == fieldsRootNode) {
					CodingSchemaFieldNode root = (CodingSchemaFieldNode) parent;
					int children = 0;
					for (int i = 0; i < root.getChildCount(); i++) {
						TreeTableNode child = root.getChildAt(i);
						// Nonmanual Fields contain an ArrayList of child IDs in their 5th column
						if (child.getValueAt(5) instanceof ArrayList)
							children++;
					}
					return children;
				} else
					return super.getChildCount(parent);
			}
		};

		nmFieldsTable = new JXTreeTable(fieldsModel) {
			private static final long serialVersionUID = 1L;

			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == 4) {
					return new ColorRenderer(true);
				}
				// else...
				return super.getCellRenderer(row, column);
			}
		};

		final DefaultTreeTableModel manualFieldsModel = new DefaultTreeTableModel(manualFieldsRootNode, Arrays.asList(
				"Field Name",
				"Field Label",
				"Display",
				"Data",
				"Color",
				" "
		)) {
			@Override
			public Class<?> getColumnClass(int column) {
				if (column == 2) {
					return Boolean.class;
				} else if (column == 4) {
					return Color.class;
				}
				return String.class;
			}

			@Override
			public boolean isCellEditable(Object node, int column) {
				return column == 2 && !glossAndHandFieldIDs.contains(((CodingSchemaFieldNode) node).getData()[5]);
			}
		};
		manualFieldsTable = new JXTreeTable(manualFieldsModel) {
			@Override
			public TableCellRenderer getCellRenderer(int row, int column) {
				if (column == 4) {
					return new ColorRenderer(true);
				}
				return super.getCellRenderer(row,column);
			}
		};
		manualFieldsTable.expandAll();

		TableColumn customCol = nmFieldsTable.getColumnModel().getColumn(4);
		customCol.setCellEditor(new ColorEditor());

		TableColumn manualColorCol = manualFieldsTable.getColumnModel().getColumn(4);
		manualColorCol.setCellEditor(new ColorEditor());

		nmFieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		nmFieldsTable.setHorizontalScrollEnabled(true);
		nmFieldsTable.setFillsViewportHeight(false);
		nmFieldsTable.setRootVisible(false);

		manualFieldsTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		manualFieldsTable.setHorizontalScrollEnabled(true);
		manualFieldsTable.setFillsViewportHeight(false);
		manualFieldsTable.setRootVisible(false);

		// do not show field id column
		nmFieldsTable.getColumnModel().removeColumn(nmFieldsTable.getColumnModel().getColumn(5));
		nmFieldsTable.setAutoCreateColumnsFromModel(false);

		manualFieldsTable.getColumnModel().removeColumn(manualFieldsTable.getColumnModel().getColumn(5));
		manualFieldsTable.setAutoCreateColumnsFromModel(false);

		UIDefaults uiDefs = UIManager.getDefaults();
		uiDefs.put("Tree.leftChildIndent", new Integer(3));

		nmFieldsTable.setOpenIcon(null);
		nmFieldsTable.setClosedIcon(null);
		nmFieldsTable.setLeafIcon(null);

		manualFieldsTable.setOpenIcon(null);
		manualFieldsTable.setClosedIcon(null);
		manualFieldsTable.setLeafIcon(null);

		nmFieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		nmFieldsTable.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		manualFieldsTable.setRowHeight(SegmentGraphReprUtil.treeTableCellHeight);
		manualFieldsTable.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);

		nmFieldsTable.addTreeSelectionListener(selectionListener);

		nmFieldsTable.packAll();
		manualFieldsTable.packAll();

		JPanel tablesPanel = new JPanel(new GridLayout(0,1, 5, 5));
		tablesPanel.add(nmFieldsTable);
		tablesPanel.add(manualFieldsTable);

		JScrollPane jsp = new JScrollPane(tablesPanel);
		jsp.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		return jsp;
	}

	public int getGroup(TreeTableNode node) {
		if (node == null)
			return 4;

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
		TreeTableNode rootNode = fieldsModel.getRoot();
		TreeTableNode headNode = rootNode.getChildAt(0);
		TreeTableNode faceNode = rootNode.getChildAt(1);
		TreeTableNode bodyNode = rootNode.getChildAt(2);
		TreeTableNode gramNode = rootNode.getChildAt(3);

		if (node.equals(headNode))
			return 0;
		if (node.equals(faceNode))
			return 1;
		if (node.equals(bodyNode))
			return 2;
		if (node.equals(gramNode))
			return 3;
		return 4;
	}

	private class CodingSchemaFieldNode extends AbstractMutableTreeTableNode {
		private boolean manualNode;

		public CodingSchemaFieldNode(Object[] data) {
			this(data, false);
		}

		public CodingSchemaFieldNode(Object[] data, boolean manualNode) {
			super(data);
			if (data == null) {
				throw new IllegalArgumentException("Node data cannot be null");
			}
			this.manualNode = manualNode;
		}

		@Override
		public int getColumnCount() {
			return getData().length;
		}

		@Override
		public Object getValueAt(int column) {
			return getData()[column];
		}

		public Object[] getData() {
			return (Object[]) getUserObject();
		}

		private boolean isGroup(TreeTableNode node) {
			DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
			TreeTableNode rootNode = fieldsModel.getRoot();
			TreeTableNode headNode = rootNode.getChildAt(0);
			TreeTableNode faceNode = rootNode.getChildAt(1);
			TreeTableNode bodyNode = rootNode.getChildAt(2);
			TreeTableNode gramNode = rootNode.getChildAt(3);

			TreeTableNode manualNode = manualFieldsRootNode.getChildAt(0);

			return node.equals(headNode) || node.equals(faceNode) || node.equals(bodyNode) || node.equals(gramNode) || node.equals(manualNode);
		}

		// Set value without invoking any custom functionality
		public void setValueAtSelf(Object value, int column) {
		    super.setValueAt(value, column);
		    getData()[column] = value;
        }

		@Override
		public void setValueAt(Object value, int column) {
			super.setValueAt(value, column);
			getData()[column] = value;

			// Below logic is for checkbox column behavior only
			if (column != 2) return;
			var checked = (boolean)value;

            if (!isLeaf()) {
                // For non leaf nodes, all children should be updated to the same value
                for (int i = 0; i < getChildCount(); i++)
                    getChildAt(i).setValueAt(checked, 2);
            } else {
                // For leaf nodes, the parent should be checked only if all children are
                var parent = (CodingSchemaFieldNode)getParent();
                var any = false;
                for (int i = 0; i < parent.getChildCount(); i++) {
                    var child = parent.getChildAt(i);
                    any = any || (boolean)child.getValueAt(2);
                }
                parent.setValueAtSelf(any, 2);
            }
		}
	}

	private HashMap<String, CodingSchemaFieldNode> getFieldsList(Collection<SS3Field> fieldsToDisplay,
			SegmentGraphicRepresentation rep) {

		HashMap<String, SS3Field> fields = new HashMap<String, SS3Field>();
		Iterator<SS3Field> iter = fieldsToDisplay.iterator();
		while (iter.hasNext()) {
			SS3Field csField = iter.next();
			fields.put(csField.getId(), csField);
		}

		LabelObject object = segment.getRootLabelObject().getNonManuelLabelObject();
		ArrayList<LabelObject> leafsOnly = object.getAllChildren();
		ArrayList<String> available = new ArrayList<String>();
		ArrayList<String> visible = new ArrayList<String>();

		Hashtable<String, Color> colors = new Hashtable<String, Color>();
		HashMap<String, CodingSchemaFieldNode> nodes = new HashMap<String, CodingSchemaFieldNode>();

		for (int i = 0; i < leafsOnly.size(); i++) {
			LabelObject obj = leafsOnly.get(i);
			FieldWrapper wrapper = obj.getWrapper();
			String id = wrapper.getField().getFieldID();

			Color color = wrapper.getField().getColor();
			if (color != null)
				colors.put(id, color);

			if (obj.isVisible())
				visible.add(id);

			if (!available.contains(id) && wrapper.getField().getFieldSize() > 0)
				available.add(id);
		}

		if (fields != null) {
			Set<String> csKeys = fields.keySet();
			Object[] sortedKeys = Sorter.sortIntegerSet(csKeys);

			for (int i = 0; i < sortedKeys.length; i++) {
				SS3Field csField = fields.get(sortedKeys[i]);

				String fieldID = "" + csField.getId();

				Boolean checked = visible.contains(fieldID) ? new Boolean(true) : new Boolean(false);

				String dt = available.contains(fieldID) ? "  *  " : "";

				Color color = csField.getColor();
				Object col = colors.get(fieldID);

				if (col != null)
					color = (Color) col;

				Object[] values = new Object[] { csField.getName(), csField.getLabel(), checked, dt, color, fieldID };

				CodingSchemaFieldNode node = new CodingSchemaFieldNode(values);
				nodes.put(fieldID, node);
			}
		}
		return nodes;
	}

	private void populateFieldsTable() {
		fieldsRootNode = new CodingSchemaFieldNode(
				new Object[] { "Field Name", "Field Label", "true", "Data", "Color.BLUE", "" });

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		HashMap<String, SS3Field> fields = scheme.getDependentRootFields();
		HashMap<String, CodingSchemaFieldNode> nmNodes = getFieldsList(scheme.getFields().values(), rep);

		ArrayList<FieldsGroup> groups = scheme.loadFieldsGroups();

//		fieldsRootNode.add(new CodingSchemaFieldNode(new Object[] { "baabaa", "", true, "*", Color.BLACK, "" }));

		for (int i = 0; i < groups.size(); i++) {
			FieldsGroup group = groups.get(i);
			boolean checked = false;
			String dt = "";
			ArrayList<CodingSchemaFieldNode> nodes = new ArrayList<CodingSchemaFieldNode>();
			ArrayList<String> ids = group.getFieldsID();

			for (String id : ids) {

				fieldIDs.add(id);

				CodingSchemaFieldNode node = nmNodes.get(id);
				if (node != null) {
					Object[] values = (Object[]) node.getUserObject();
					Boolean checked1 = (Boolean) values[2];
					String dt1 = (String) values[3];

					if (checked1.booleanValue()) {
						checked = true;
					}

					if (dt1.trim().length() > 0) {
						dt = dt1;
					}

					nodes.add(node);
				}
			}

			Object[] value = new Object[] { group.getGroupName(), "", checked, dt, Color.BLACK, group.getFieldsID() };
			CodingSchemaFieldNode gNode = new CodingSchemaFieldNode(value);
			fieldsRootNode.add(gNode);

			for (CodingSchemaFieldNode node1 : nodes) {
				gNode.add(node1);
			}
		}

		if (fields != null) {
			Set<String> csKeys = fields.keySet();
			Object[] sortedKeys = Sorter.sortIntegerSet(csKeys);

			for (int i = 0; i < sortedKeys.length; i++) {
				SS3Field childField = fields.get(sortedKeys[i]);
				addChild(fieldsRootNode, childField);
			}
		}

	}

	private void populateManualFieldsTable() {
		manualFieldsRootNode = new CodingSchemaFieldNode(
				new Object[] { "Field Name", "Field Label", "true", "Data", Color.BLACK, ""}, true);

		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();

		FieldsGroup manualGroup = scheme.loadManualFieldGroup();

		ArrayList<CodingSchemaFieldNode> nodes = new ArrayList<CodingSchemaFieldNode>();
		ArrayList<String> ids = manualGroup.getFieldsID();
		ids.removeAll(glossAndHandFieldIDs);		// since we cannot hide these fields (ever) do not display them here

		for (String id : ids) {

			fieldIDs.add(id);

			SS3Field field = scheme.getField(id);
			FieldWrapper wrapper = segment.getFieldWrappersCollection().getFieldWrapperWithFieldID(id);
			String dataStr = wrapper.getField().getFieldSize() > 0 ? "*" : "";

			CodingSchemaFieldNode node = new CodingSchemaFieldNode(new Object[] {
				field.getName(), field.getLabel(), true, dataStr, Color.BLACK, id
			}, true);
			nodes.add(node);
		}

		Object[] value = new Object[] { manualGroup.getGroupName(), "", true, "", Color.BLACK, ""};
		CodingSchemaFieldNode gNode = new CodingSchemaFieldNode(value, true);
		manualFieldsRootNode.add(gNode);

		for (CodingSchemaFieldNode node1 : nodes) {
			gNode.add(node1);
		}
	}

	private void addChild(CodingSchemaFieldNode parent, SS3Field childField) {
		CodingSchemaFieldNode child = createFieldNode(childField);
		parent.add(child);
		SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
		ArrayList<SS3Field> depFields = scheme.getDependentFields(childField.getId());

		for (int i = 0; i < depFields.size(); i++) {
			SS3Field f = depFields.get(i);
			addChild(child, f);
		}
	}

	private CodingSchemaFieldNode createFieldNode(SS3Field field) {
		LabelObject object = segment.getRootLabelObject().getGlossNRelatedLabelObject();
		ArrayList<LabelObject> list = object.getAllChildren();
		ArrayList<String> available = new ArrayList<String>();
		ArrayList<String> visible = new ArrayList<String>();

		for (int i = 0; i < list.size(); i++) {
			LabelObject obj = list.get(i);
			FieldWrapper wrapper = obj.getWrapper();
			String id = wrapper.getField().getFieldID();

			if (obj.isVisible())
				visible.add(id);

			if (!available.contains(id) && wrapper.getField().getFieldSize() > 0)
				available.add(id);
		}

		String fieldID = "" + field.getId();
		boolean checked = visible.contains(fieldID) ? true : false;
		boolean data = available.contains(fieldID);
		String dataStr = data ? "*" : "";

		Object[] fieldNodeObj = new Object[] { field.getName(), field.getLabel(), checked, dataStr, field.getColor(),
				field.getId() };
		return new CodingSchemaFieldNode(fieldNodeObj);
	}

	public void scrollToAndSelectFieldByID(String fieldID) {
		if (fieldID == null || fieldID.isEmpty())
			return;

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				CodingSchemaFieldNode childNode = (CodingSchemaFieldNode) groupNode.getChildAt(i);
				String nodeID = (String) childNode.getValueAt(5);

				if (nodeID.equals(fieldID)) {
					scrollToAndSelectNode(childNode);
				}
			}
		}
	}

	private void scrollToAndSelectNode(MutableTreeTableNode node) {
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
		TreeNode[] nodes = fieldsModel.getPathToRoot(node);
		TreePath path = new TreePath(nodes);

		nmFieldsTable.expandPath(path.getParentPath());
		int row = nmFieldsTable.getRowForPath(path);
		nmFieldsTable.scrollRowToVisible(row);
		nmFieldsTable.scrollPathToVisible(path);
		nmFieldsTable.setRowSelectionInterval(row, row);
	}

	public HashMap<String, Color> getSelectedColors() {
		HashMap<String, Color> fieldColors = new HashMap<String, Color>();
		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < 4; groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				MutableTreeTableNode childNode = (MutableTreeTableNode) groupNode.getChildAt(i);

				String fieldID = (String) childNode.getValueAt(5);
				Color color = (Color) childNode.getValueAt(4);
				fieldColors.put(fieldID, color);
			}
		}

		return fieldColors;
	}

	public ArrayList<String> getSelectedFieldIDsNUpdateCodingSchema() {
		ArrayList<String> visibleFieldIDs = new ArrayList<String>();

		DefaultTreeTableModel fieldsModel = (DefaultTreeTableModel) nmFieldsTable.getTreeTableModel();
		TreeTableNode fieldsRootNode = fieldsModel.getRoot();

		for (int groupNumber = 0; groupNumber < fieldsRootNode.getChildCount(); groupNumber++) {
			MutableTreeTableNode groupNode = (MutableTreeTableNode) fieldsRootNode.getChildAt(groupNumber);
			int childCount = groupNode.getChildCount();
			for (int i = 0; i < childCount; i++) {
				MutableTreeTableNode childNode = (MutableTreeTableNode) groupNode.getChildAt(i);
				Boolean display = (Boolean) childNode.getValueAt(2);

				if (display.booleanValue()) {
					visibleFieldIDs.add((String) childNode.getValueAt(5));
				}
			}
		}

		// always set gloss fields to visible
		getSelectedGlossFldIDs(4, visibleFieldIDs);
		getSelectedGlossFldIDs(5, visibleFieldIDs);
		getSelectedHandFields(6, visibleFieldIDs);
		getSelectedHandFields(7, visibleFieldIDs);

		TreeTableNode manualGroupNode = manualFieldsRootNode.getChildAt(0);
		for (int i = 0; i < manualGroupNode.getChildCount(); i++) {
			TreeTableNode fieldNode = manualGroupNode.getChildAt(i);
			String fieldID = (String) fieldNode.getValueAt(5);
			visibleFieldIDs.remove(fieldID);
			if ((Boolean) fieldNode.getValueAt(2) && !visibleFieldIDs.contains(fieldID)) {
				visibleFieldIDs.add(fieldID);
			}
		}

		return visibleFieldIDs;
	}

	private void getSelectedGlossFldIDs(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode.getChildAt(index);
		Object obj = node.getValueAt(2);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(5));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(2);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(5));
				}
			}
		}
	}

	private void getSelectedHandDependentFieldIDs(MutableTreeTableNode node, ArrayList<String> ids) {
		int childCount = node.getChildCount();
		for (int count = 0; count < childCount; count++) {
			MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
			Boolean cDisplay = (Boolean) cNode.getValueAt(2);

			if (cDisplay.booleanValue()) {
				ids.add((String) cNode.getValueAt(5));
				if (cNode.getChildCount() > 0) {
					getSelectedHandDependentFieldIDs(cNode, ids);
				}
			}
		}
	}

	private void getSelectedHandFields(int index, ArrayList<String> ids) {
		MutableTreeTableNode node = (MutableTreeTableNode) fieldsRootNode.getChildAt(index);
		Object obj = node.getValueAt(2);
		Boolean display = (Boolean) obj;
		if (display.booleanValue()) {
			ids.add((String) node.getValueAt(5));
			int childCount = node.getChildCount();
			for (int count = 0; count < childCount; count++) {
				MutableTreeTableNode cNode = (MutableTreeTableNode) node.getChildAt(count);
				Boolean cDisplay = (Boolean) cNode.getValueAt(2);

				if (cDisplay.booleanValue()) {
					ids.add((String) cNode.getValueAt(5));
					getSelectedHandDependentFieldIDs(cNode, ids);
				}
			}
		}
	}

	// Iterate through all TreeTable child nodes in each group.
	private void iterTable(JXTreeTable table, Consumer<TreeTableNode> fn) {
        var root = (TreeTableNode)table.getTreeTableModel().getRoot();
        for (int i = 0; i < root.getChildCount(); i++) {
            var group = root.getChildAt(i);
            for (int j = 0; j < group.getChildCount(); j++) {
                var node = group.getChildAt(j);
                fn.accept(node);
            }
        }
    }

    // Iterate through

    public void select(String id, boolean selected) {
        if (glossAndHandFieldIDs.contains(id)) {
            // don't change selected state of gloss/hand/translation fields in the table
            return;
        }

        Consumer<TreeTableNode> selectIfMatching = node -> {
            if (node.getValueAt(5).equals(id))
                node.setValueAt(selected, 2);
        };

        iterTable(nmFieldsTable, selectIfMatching);
        iterTable(manualFieldsTable, selectIfMatching);
    }

	// Reset any checkbox changes since opening the dialog
	public void update() {
		//prevents newField/editField windows from displaying multiple times
		if (newFieldBtn.getActionListeners().length == 0) {
			newFieldBtn.addActionListener(this);
		}
		if (editBtn.getActionListeners().length == 0) {
			editBtn.addActionListener(this);
		}
		deleteBtn.addActionListener(this);
		cancelBtn.addActionListener(this);
		doneBtn.addActionListener(this);
		//must be re-added for proper field selection (for editing)
		nmFieldsTable.addTreeSelectionListener(selectionListener);
	}

	// Discard any changes made in the dialog by re-populating the tree with the underlying field values
	public void discard() {
        var start = SegmentGraphReprUtil.utteranceViewVisibleMovieOnset;
        var end = SegmentGraphReprUtil.utteranceViewVisibleMovieOffset;
        if (segmentPanel.getUtteranceView() != null) {
            var template = segmentPanel.getUtteranceView().getNonmanualFieldsView().getLabelView().getControlPanel().getSegmentTemplate();
            var utterance = template.getSegmentControlPanel().getSelectedUtterance();
            start = utterance.getStartTime();
            end = utterance.getEndTime();
        }

        // Reset non manual visibility
        var nonManualVisible = segment.getRootLabelObject().getNonManuelLabelObject().getNonEmptyObjectsBetween(start, end);
        iterTable(nmFieldsTable, node -> {
            node.setValueAt(false, 2);
            for (var child : nonManualVisible) {
                if (child.getFieldID().equals(node.getValueAt(5))) {
                    node.setValueAt(true, 2);
                    break;
                }
            }
        });

        // Reset non manual colors
        // TODO: Does not work. Where is the actual color stored?
        var nonManualLabels = segment.getRootLabelObject().getNonManuelLabelObject();
        iterTable(nmFieldsTable, node -> {
            for (var child : nonManualLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(5))) {
                    node.setValueAt(child.getWrapper().getField().getColor(), 4);
                }
            }
        });

        // Reset manual visibility
        var manualVisible = segment.getRootLabelObject().getGlossNRelatedLabelObject().getNonEmptyObjectsBetween(start, end);
        iterTable(manualFieldsTable, node -> {
            node.setValueAt(false, 2);
            for (var child : manualVisible) {
                if (child.getFieldID().equals(node.getValueAt(5))) {
                    node.setValueAt(true, 2);
                    break;
                }
            }
        });

        // Reset manual colors
        // TODO: Does not work. Where is the actual color stored?
        var manualLabels = segment.getRootLabelObject().getGlossNRelatedLabelObject();
        iterTable(manualFieldsTable, node -> {
            for (var child : manualLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(5))) {
                    node.setValueAt(child.getWrapper().getField().getColor(), 4);
                }
            }
        });
	}

	// Apply any changes made in the dialog to the underlying field values
    public void apply() {
        SS3CodingScheme scheme = SS3Singleton.getSS3CodingSchemeCollection().getDefaultCodingScheme();
        SS3CodingScheme localScheme = segmentPanel.getSS3Database().getLocalSS3CodingScheme();

        BiConsumer<TreeTableNode, LabelObject> updateVisibility = (node, child) -> {
            child.setVisible((boolean)node.getValueAt(2));
        };

        BiConsumer<TreeTableNode, LabelObject> updateColor = (node, child) -> {
            var id = child.getFieldID();
            var color = (Color)node.getValueAt(4);

            child.getWrapper().getField().setColor(color);

            var currentField = scheme.getField(id);
            var localField = localScheme.getField(id);
            if (localField != null) {
                localField.setColor(color);
            } else if (!currentField.getColor().equals(color)) {
                localScheme.addField(currentField.cloneField());
            }
            currentField.setColor(color);
        };

        var nonManualLabels = segment.getRootLabelObject().getNonManuelLabelObject();
        iterTable(nmFieldsTable, node -> {
            for (var child : nonManualLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(5))) {
                    updateVisibility.accept(node, child);
                    updateColor.accept(node, child);
                }
            }
        });

        var manualLabels = segment.getRootLabelObject().getGlossNRelatedLabelObject();
        iterTable(manualFieldsTable, node -> {
            for (var child : manualLabels.getAllChildren()) {
                if (child.getFieldID().equals(node.getValueAt(5))) {
                    updateVisibility.accept(node, child);
                    updateColor.accept(node, child);
                }
            }
        });

        nonManualView.update(segment);
        glossNRelatedView.update(segment);

        nonManualView.getLabelView().loadWrappers();

        nonManualView.getLabelView().updateUI();
        glossNRelatedView.getLableView().updateUI();

        nonManualView.getLabelView().repaint();
        glossNRelatedView.getLableView().repaint();
    }

	public Segment getSegment() {
		return segment;
	}

	public void showAllFields() {
	    discard();
	    apply();
	}
}

class SelectionListener implements TreeSelectionListener {
	private CSFieldPanel panel = null;
	private SS3CodingScheme scheme = null;

	public SelectionListener(CSFieldPanel panel, SS3CodingScheme scheme) {
		this.panel = panel;
		this.scheme = scheme;
	}

	@Override
	public void valueChanged(TreeSelectionEvent se) {
		JTree tree = (JTree) se.getSource();
		MutableTreeTableNode selectedNode = (MutableTreeTableNode) tree.getLastSelectedPathComponent();

		panel.setSelectedTreeTableNode(selectedNode);

		if (selectedNode == null) {
			return;
		} else if (selectedNode.isLeaf()) {
			String fieldID = (String) selectedNode.getValueAt(5);
			SS3Field field = scheme.getField(fieldID);

			Color selectedColor = (Color) selectedNode.getValueAt(4);
			Segment segment = panel.getSegment();
			field.setColor(selectedColor);

			LabelObject object = segment.getRootLabelObject().getNonManuelLabelObject();
			ArrayList<LabelObject> leafsOnly = object.getAllChildren();

			for (int i = 0; i < leafsOnly.size(); i++) {
				LabelObject obj = leafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

			LabelObject gObject = segment.getRootLabelObject().getGlossNRelatedLabelObject();
			ArrayList<LabelObject> gLeafsOnly = gObject.getAllChildren();

			for (int i = 0; i < gLeafsOnly.size(); i++) {
				LabelObject obj = gLeafsOnly.get(i);
				FieldWrapper wrapper = obj.getWrapper();
				String id = wrapper.getField().getFieldID();
				if (id.equalsIgnoreCase(fieldID)) {
					wrapper.getField().setColor(selectedColor);
				}
			}

			if (selectedNode.isLeaf())
				selectedNode = (MutableTreeTableNode) selectedNode.getParent();
		}
	}
}
