package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3DatabaseCollection;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.media.fileNavigation.FileRenderer;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.media.fileNavigation.MediaDelegate;
import edu.bu.signstream.media.fileNavigation.MediaTable;
import edu.bu.signstream.media.fileNavigation.MediaTableModel;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.panels.table.sortable.CSFieldTableSorter;

public class AddNewMediaPanel extends JPanel implements ActionListener {

	private MediaTable table = null;
	private MediaTableModel tableModel = null;
	private SignStreamSegmentPanel segmentPanel = null;

	private JFrame frame = null;

	private JButton addBtn = new JButton("Add Movie");
	private JButton deleteBtn = new JButton("Delete Movie");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton okBtn = new JButton("OK");
	private JDialog parent;

	private JTextField excerptTxt = new JTextField(25);
	private JTextField participantTxt = new JTextField(25);

	private final int ADD = 0;
	private final int DELETE = 1;
	private final int CANCEL = 2;
	private final int OK = 3;

	private Object[] columnNames = { "Display ", "File Name ", "Delete ", "" };

	public AddNewMediaPanel(JFrame frame, SignStreamSegmentPanel segmentPanel,
			JDialog parent) {
		this.frame = frame;
		this.segmentPanel = segmentPanel;
		this.parent = parent;
		initUI();
	}

	private void initUI() {
		addBtn.setActionCommand("" + ADD);
		deleteBtn.setActionCommand("" + DELETE);
		cancelBtn.setActionCommand("" + CANCEL);
		okBtn.setActionCommand("" + OK);

		addBtn.addActionListener(this);
		deleteBtn.addActionListener(this);
		cancelBtn.addActionListener(this);
		okBtn.addActionListener(this);

		JScrollPane pane = createValuesTable();
		JPanel centerPanel = createPanel(pane);

		table.setDefaultRenderer(File.class, new FileRenderer(true));

		ButtonPanel bPanel = new ButtonPanel();
		bPanel.add(addBtn);
		bPanel.add(deleteBtn);
		bPanel.add(cancelBtn);
		bPanel.add(okBtn);

		setLayout(new BorderLayout());
		add(centerPanel, BorderLayout.CENTER);
		add(bPanel, BorderLayout.SOUTH);

		table.getColumnModel()
				.removeColumn(table.getColumnModel().getColumn(3));
		table.setAutoCreateColumnsFromModel(false);
	}

	private JPanel createPanel(Component fPane) {

		JPanel pane = new JPanel();
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		pane.setLayout(gbl);

		gbc.anchor = GridBagConstraints.NORTHWEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;

		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = 1;
		gbc.insets = new Insets(10, 5, 5, 0);
		pane.add(new JLabel("Excerpt"), gbc);

		pane.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		pane.add(excerptTxt, gbc);

		gbc.gridwidth = 1;
		pane.add(new JLabel("Participant"), gbc);

		pane.add(Box.createHorizontalStrut(10));
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		pane.add(participantTxt, gbc);

		gbc.anchor = GridBagConstraints.NORTH;
		gbc.fill = GridBagConstraints.HORIZONTAL;

		gbc.anchor = GridBagConstraints.WEST;
		gbc.gridwidth = GridBagConstraints.REMAINDER;
		gbc.insets = new Insets(15, 3, 3, 3);
		pane.add(fPane, gbc);

		return pane;
	}

	private JScrollPane createValuesTable() {
		tableModel = new MediaTableModel(columnNames, createTableModelData(),
				parent);
		CSFieldTableSorter sorter = new CSFieldTableSorter(tableModel);

		table = new MediaTable(sorter);
		sorter.setTableHeader(table.getTableHeader());
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

		table.getSelectionModel().addListSelectionListener(
				new ListSelectionListener() {
					@Override
					public void valueChanged(ListSelectionEvent e) {
						if (e.getValueIsAdjusting()) {
							CSFieldTableSorter sorter1 = (CSFieldTableSorter) table
									.getModel();
							sorter1.update(table.getSelectedRow(), table
									.getSelectedColumn());

							setColumnsSize();
						}
					}
				});

		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

		Dimension d = new Dimension(400, 200);
		table.setMaximumSize(d);
		scrollPane.setMaximumSize(d);
		scrollPane.setPreferredSize(d);

		setColumnsSize();

		tableModel.fireTableDataChanged();
		parent.setResizable(false);
		return scrollPane;
	}

	private Object[][] createTableModelData() {
		Object[][] data = null;
		ArrayList valueList = new ArrayList();
		MediaDelegate delegate = new MediaDelegate();
		ArrayList mFiles = delegate.getLoadedMediaFiles(segmentPanel);

		for (int i = 0; i < mFiles.size(); i++) {
			LoadedMediaFile lFile = (LoadedMediaFile) mFiles.get(i);

			Object[] values = { new Boolean(lFile.isLoaded()),
					lFile.getMediaFile().getFile(), Boolean.FALSE,
					lFile.getMediaFile().getId() };

			valueList.add(values);
		}

		data = new Object[valueList.size()][];

		for (int j = 0; j < valueList.size(); j++) {
			data[j] = (Object[]) valueList.get(j);
		}
		return data;
	}

	private void setColumnsSize() {

		int displayWidth = 75;
		int fileWidth = 250;
		int deleteWidth = 75;

		TableColumn displayColumn = table.getColumn(columnNames[0]);
		TableColumn fileColumn = table.getColumn(columnNames[1]);
		TableColumn deleteColumn = table.getColumn(columnNames[2]);

		displayColumn.setMinWidth(displayWidth);
		displayColumn.setMaxWidth(displayWidth);
		displayColumn.setPreferredWidth(displayWidth);

		fileColumn.setMinWidth(fileWidth);
		fileColumn.setMaxWidth(fileWidth);
		displayColumn.setPreferredWidth(fileWidth);

		displayColumn.setMinWidth(deleteWidth);
		displayColumn.setMaxWidth(deleteWidth);
		displayColumn.setPreferredWidth(deleteWidth);

		table.revalidate();
	}

	private int setSize(TableColumn column) {
		int columnSize = getPreferredWidthForColumn(column);
		column.setMinWidth(columnSize);
		column.setMaxWidth(columnSize);
		column.setPreferredWidth(columnSize);
		table.revalidate();
		return columnSize;
	}

	public int getPreferredWidthForColumn(TableColumn col) {
		int hw = columnHeaderWidth(col), // hw = header width
		cw = widestCellInColumn(col); // cw = column width

		return hw > cw ? hw : cw;
	}

	private int columnHeaderWidth(TableColumn col) {
		return 75;
	}

	private int widestCellInColumn(TableColumn col) {
		int c = col.getModelIndex(), width = 0, maxw = 0;

		for (int r = 0; r < table.getRowCount(); ++r) {
			TableCellRenderer renderer = table.getCellRenderer(r, c);

			Component comp = renderer.getTableCellRendererComponent(table,
					table.getValueAt(r, c), false, false, r, c);

			width = comp.getPreferredSize().width;

			maxw = width > maxw ? width : maxw;
		}
		return maxw;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case ADD:
			String mLocation = SS3Singleton.getMediaFilesLocator()
					.getLastLoadedDirectory();

			FileDialog fd = new FileDialog(frame, "Select medial file",
					FileDialog.LOAD);

			File mf = new File(mLocation);
			String absPath = mf.getAbsolutePath();
			fd.setDirectory(absPath);
			fd.setFile(absPath);
			fd.show();

			if (fd.getFile() != null) {
				String fileName = fd.getDirectory() + File.separator
						+ fd.getFile();
				File selectedFile = new File(fileName);
				tableModel.addNewValue(selectedFile);
				SS3Singleton.getMediaFilesLocator().setLastLoadedDirectory(
						selectedFile.getParentFile().getPath());
			}

			break;
		case DELETE:
			tableModel.deleteTableRows();
			break;
		case CANCEL:
			parent.dispose();
			break;
		case OK:
			ArrayList files = tableModel.getMediaFiles();
			ArrayList lFiles = new ArrayList();

			Excerpt excerpt = null;

			SS3DatabaseCollection dbCol = SS3Singleton
					.getSS3DatabaseCollection();
			DbNavigationPanel navigation = segmentPanel.getDbNavigationPanel();

			Object root = navigation.getJTree().getModel().getRoot();

			if (root == null)
				return;

			DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
			DefaultMutableTreeNode cNode = null;
			DefaultMutableTreeNode node = null;

			if (rootNode.getChildCount() > 0) {
				Object obj = rootNode.getChildAt(0);

				node = (DefaultMutableTreeNode) obj;

				int counter = 0;
				Enumeration enum2 = node.children();
				while (enum2.hasMoreElements()) {
					DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) enum2
							.nextElement();
					Excerpt excpt = (Excerpt) fNode.getUserObject();
					int fnID = Integer.parseInt(excpt.getID());
					counter = fnID > counter ? fnID : counter;
				}

				String excTxt = excerptTxt.getText().trim();
				if (excTxt.length() == 0)
					excTxt = "macro_unit_" + (counter + 1);

				excerpt = new Excerpt("" + (counter + 1), excTxt);

				cNode = new DefaultMutableTreeNode(excerpt);
				node.add(cNode);
			}

			TimeInfo startTimeInfo = new TimeInfo();
			TimeInfo endTimeInfo = new TimeInfo();
			startTimeInfo.setMovieTime(0);
			endTimeInfo.setMovieTime(0);

			SS3Database db = new SS3Database(startTimeInfo, endTimeInfo);
			db.setParticipantName(participantTxt.getText());
			dbCol.addSS3SDatabase(db);

			for (int i = 0; i < files.size(); i++) {
				LoadedMediaFile file = (LoadedMediaFile) files.get(i);
				SS3MediaFile mFile = file.getMediaFile();
				String id = mFile.getId();

				if (id == null || id.length() == 0)
					mFile.setId(Util.getLongUniqueNumber());

				db.addMediaFile(mFile);
				excerpt.addOrOverrideMediaFileID(mFile.getId());

				if (file.isLoaded())
					lFiles.add(mFile);
			}

			navigation.refresh(rootNode);
			expandAll(navigation.getJTree(), new TreePath(rootNode), true);

			if (cNode != null) {
				int rowToSelect = node.getChildCount() + cNode.getLevel() - 1;

				segmentPanel.updateLoadedDatabase();
				SS3Database nDB = dbCol.getSS3Database(excerpt);
				segmentPanel.loadDatabase(nDB);
				segmentPanel.repaint();

				JFrame frame = segmentPanel.getParentFrame();

				if (frame != null) {
					String fileName = db.getFileName();
					String newTitle = fileName + ": " + excerpt;
					frame.setTitle(newTitle);
					frame.repaint();
				}

				navigation.getJTree().setSelectionRow(rowToSelect);
				navigation.getJTree().repaint();
			}

			DefaultMutableTreeNode bNode = (DefaultMutableTreeNode) node
					.getChildBefore(cNode);

			if (bNode != null) {
				Excerpt ex = (Excerpt) bNode.getUserObject();
				SS3Database ndb = dbCol.getSS3Database(ex);

				startTimeInfo.setMovieTime(ndb.getEndTimeInfo().getMovieTime());
				endTimeInfo.setMovieTime(ndb.getEndTimeInfo().getMovieTime());

				Excerpt cEx = (Excerpt) cNode.getUserObject();
				SS3Database cdb = dbCol.getSS3Database(cEx);
				cdb.getStartTimeInfo().setMovieTime(
						ndb.getEndTimeInfo().getMovieTime());
				cdb.getEndTimeInfo().setMovieTime(
						ndb.getEndTimeInfo().getMovieTime());
			}

			int duration = segmentPanel.getMultipleMovieController()
					.getMovieDuration();
			int startTime = startTimeInfo.getMovieTime() > duration ? duration
					: startTimeInfo.getMovieTime();
			segmentPanel.getMultipleMovieController().setCurrentTime(startTime);

			int startTimeCoord = segmentPanel.getZoomer()
					.getCoordinateTimeConvertor().convertTimeToCoordinate(
							startTimeInfo.getMovieTime());

			segmentPanel.getSlider().setSliderPositionCoord(startTimeCoord);
			Point scrollTo = new Point(startTimeCoord, 0);
			// segmentPanel.getView().scrollRectToVisible(scrollTo);
			
			if(segmentPanel.isUtteranceLoaded()){
				segmentPanel.getUtteranceView().scrollRectToVisible(scrollTo);
			} else {
				segmentPanel.getDatabaseView().scrollRectToVisible(scrollTo);
			}

			//segmentPanel.getUtteranceView().scrollRectToVisible(scrollTo);
			
			parent.dispose();
			break;
		}
	}

	//
	private void expandAll(JTree tree, TreePath parent, boolean expand) {
		// Traverse children
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0) {
			for (Enumeration e = node.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode) e.nextElement();
				TreePath path = parent.pathByAddingChild(n);
				expandAll(tree, path, expand);
			}
		}

		// Expansion or collapse must be done bottom-up
		if (expand) {
			tree.expandPath(parent);
		} else {
			tree.collapsePath(parent);
		}
	}
}
