package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import edu.bu.signstream.common.db.SS3DatabaseWriter;
import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResourceCollection;
import edu.bu.signstream.common.util.vo.Excerpt;
import edu.bu.signstream.common.util.vo.TimeInfo;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindow;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.grepresentation.view.mediaControl.MediaToolBar;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.DirectoryLookupScrolledView;
import edu.bu.signstream.ui.SS3SignStreamApplication;
import edu.bu.signstream.ui.panels.AnnotatorSelectionDialog;
import edu.bu.signstream.ui.panels.AnnotatorsDialog;
import edu.bu.signstream.ui.video.MultipleMovieController;
import edu.bu.signstream.ui.video.XugglerMovie;

public class CreateUpdateMacroUnit extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;

	private ParticipantsLibrary participantsLibrary = new ParticipantsLibrary();
	private ParticipantsLibrary annotatorsLibrary = new ParticipantsLibrary();

	private SignStreamSegmentPanel segmentPanel = null;
	private ArrayList<LoadedMediaFile> mFiles = new ArrayList<LoadedMediaFile>();
	private HashMap<String, SS3MediaFile> graphFiles = new HashMap<String, SS3MediaFile>();
	private ArrayList<ParticipantView> prtView = new ArrayList<ParticipantView>();
	private ArrayList<ParticipantView> antView = new ArrayList<ParticipantView>();
	
	private ParticipantsPaneScrolledView participantsPaneScrolledView = null;
	private ParticipantsPaneScrolledView annotatorsPaneScrolledView = null;
	
	private AnnotatorsDialog participantsDialog = null;
	private AnnotatorsDialog annotatorsDialog = null;
	
	private SignBankResourceCollectionView signBankResourceCollectionView = null;

	private SS3MediaLibrary mLibrary = null;

	private JTextField macroUnitTxt = new JTextField(29);

	private JButton cancelBtn = new JButton("Cancel");
	private JButton submitBtn = new JButton("");
	private JButton lookupBtn = new JButton("Lookup");

	private final int CANCEL = 1;
	private final int SUBMIT = 2;
	private final int LOOKUP = 3;

	public static final int FUNCTION_ADD = 10;
	public static final int FUNCTION_UPDATE = 11;
	public static final int FUNCTION_DELETE = 12;

	private int function = FUNCTION_ADD;

	private VideoPanelScrolledView videoPanel = null;
	private JFrame frame = null;
	private JDialog parentDialog = null;

	private SS3Database db = null;
	
	private boolean didMakeChanges = false;

	public CreateUpdateMacroUnit(int function, SS3Database db,
			Excerpt excerptToUpdate, ArrayList<LoadedMediaFile> mFiles,
			SS3MediaLibrary mLibrary, ParticipantsLibrary participantsLibrary,
			ParticipantsLibrary annotatorsLibrary,
			SignBankResourceCollection resourceCollection,
			SignStreamSegmentPanel segmentPanel, JDialog parentDialog,
			JFrame frame) {

		this.mFiles = mFiles;
		this.mLibrary = mLibrary;
		this.participantsLibrary = participantsLibrary;
		this.annotatorsLibrary = annotatorsLibrary;
		this.segmentPanel = segmentPanel;
		this.parentDialog = parentDialog;
		this.frame = frame;
		this.function = function;
		this.db = db;
		this.function = function;

		if (this.db == null)
			this.db = new SS3Database(new TimeInfo(), new TimeInfo());

		videoPanel = new VideoPanelScrolledView(mFiles, lookupBtn,
				new Dimension(396, 135), frame);
		
		signBankResourceCollectionView = new SignBankResourceCollectionView(resourceCollection);

		switch (function) {
		case FUNCTION_ADD:
			submitBtn.setText("Confirm Selections");
			submitBtn.setActionCommand("" + SUBMIT);
			submitBtn.addActionListener(this);
			cancelBtn.setText("Cancel");
			cancelBtn.setActionCommand("" + CANCEL);
			cancelBtn.addActionListener(this);

			macroUnitTxt.setText(getDefaultMacroUnitLabel());

			break;
		case FUNCTION_UPDATE:
			submitBtn.setText("Confirm Selections");
			submitBtn.setActionCommand("" + SUBMIT);
			submitBtn.addActionListener(this);
			cancelBtn.setText("Cancel");
			cancelBtn.setActionCommand("" + CANCEL);
			cancelBtn.addActionListener(this);

			macroUnitTxt.setText(db.getExcerpt());

			ArrayList<SS3TemporalPartition> tPartitions = db
					.getSS3GlossWindow().getTemporalPartitions();

			ArrayList<SS3Participant> selectedParticipants = db
					.getSelectedParticipants();
			ArrayList<SS3Participant> selectedAnnotators = db
					.getSelectedAnnotators();

			for (SS3Participant participant : selectedParticipants) {
				boolean isAssociatedParticipant = false;

				for (SS3TemporalPartition partition : tPartitions) {
					ArrayList<SS3GlossWindowSegment> segments = partition
							.getTemporalPartitionSegments();
					for (SS3GlossWindowSegment segment : segments) {
						SS3Participant segmentParticipant = segment.getParticipant();
						if (segmentParticipant != null && participant.getId().equals(segmentParticipant.getId())) {
							isAssociatedParticipant = true;
						}
					}
				}

				participant.setSelected(true);
				
				
				ParticipantView pv = new ParticipantView(participant, true,
						isAssociatedParticipant, this.participantsLibrary,
						this.annotatorsLibrary, prtView);

				prtView.add(pv);
				participant.setSelected(true);
			}

			for (SS3Participant annotator : selectedAnnotators) {
				ParticipantView pv = new ParticipantView(annotator, false,
						false, this.participantsLibrary,
						this.annotatorsLibrary, antView);
				antView.add(pv);
			}

			graphFiles.putAll(db.getGraphFiles());

			break;
		case FUNCTION_DELETE:
			submitBtn.setText("Delete");
			break;
		}

			lookupBtn.setActionCommand("" + LOOKUP);
			lookupBtn.addActionListener(this);

			JLabel macroUnitLbl = new JLabel("Macro Unit: Label");
			UserInterfaceUtil.setLabelLNF(macroUnitLbl);
			UserInterfaceUtil.setTextFieldLNF(macroUnitTxt);

			Dimension prtD = new Dimension(400, 91);
			
			participantsPaneScrolledView = new ParticipantsPaneScrolledView(prtView,
					participantsLibrary, db, "Participants", prtD, true);
 
			annotatorsPaneScrolledView = new ParticipantsPaneScrolledView(antView,
					annotatorsLibrary, db, "Annotators", prtD, false);

			participantsDialog = participantsPaneScrolledView.getAnnotatorsDialog();
			annotatorsDialog = annotatorsPaneScrolledView.getAnnotatorsDialog();
			
			JPanel centerPanel = new JPanel();
			centerPanel.setLayout(new BoxLayout(centerPanel, BoxLayout.Y_AXIS));
			
			JPanel labelPanel = new JPanel();
			labelPanel.setLayout(new BoxLayout(labelPanel, BoxLayout.X_AXIS));
			labelPanel.add(macroUnitLbl);
			labelPanel.add(macroUnitTxt);
			
			centerPanel.add(labelPanel);
			
			lookupBtn.setAlignmentX(Component.RIGHT_ALIGNMENT);
			// TODO: Re-add lookup label when it is fixed
//			centerPanel.add(lookupBtn);
			
			centerPanel.add(videoPanel);
			
			centerPanel.add(participantsPaneScrolledView);
			centerPanel.add(annotatorsPaneScrolledView);

			centerPanel.setBorder(BorderFactory.createEmptyBorder(20, 25, 10, 25));

			ButtonPanel bPanel = new ButtonPanel();
			bPanel.add(cancelBtn);
			bPanel.add(submitBtn);

			setLayout(new BorderLayout());
			add(centerPanel, BorderLayout.CENTER);
			add(bPanel, BorderLayout.SOUTH);
	}
	
	public void update(
		int function, SS3Database db,
		ArrayList<LoadedMediaFile> mFiles,
		SS3MediaLibrary mLibrary, ParticipantsLibrary participantsLibrary,
		ParticipantsLibrary annotatorsLibrary,
		SignBankResourceCollection resourceCollection) {

		this.mFiles = mFiles;
		this.mLibrary = mLibrary;
		this.participantsLibrary = participantsLibrary;
		this.annotatorsLibrary = annotatorsLibrary;
		this.function = function;
		this.db = db;
		this.function = function;

		if (this.db == null)
			this.db = new SS3Database(new TimeInfo(), new TimeInfo());
		
		videoPanel.update(mFiles);
		videoPanel.repaint();

		signBankResourceCollectionView.update(resourceCollection);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case LOOKUP:
			JDialog lDialog = new JDialog();

			DirectoryLookupScrolledView directoryPanel = new DirectoryLookupScrolledView(
					videoPanel, mLibrary,
					new Dimension(400, 180), frame, lDialog);

			Container contentPane = lDialog.getContentPane();
			contentPane.setLayout(new BorderLayout(50, 50));
			contentPane.add(directoryPanel, BorderLayout.CENTER);
			lDialog.setTitle("Lookup Directories ");
			lDialog.pack();
			lDialog.setVisible(true);
			break;
		case SUBMIT:
			if (function == CreateUpdateMacroUnit.FUNCTION_ADD) {
				enterEntry();
				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
			} else if (function == CreateUpdateMacroUnit.FUNCTION_UPDATE) {
				editEntry();
				SS3Singleton.getApplicationStateController()
						.setLoadedCollectionUpdated(true);
			} else if (function == CreateUpdateMacroUnit.FUNCTION_DELETE) {
			}

			break;

		case CANCEL:
			cancelEntry();
			break;
		}
	}

	public void cancelEntry() {
		switch (function) {
		case FUNCTION_ADD:
			if (antView.isEmpty() || mFiles.isEmpty()) {
				enterEntry();
				return;
			}
			
		case FUNCTION_UPDATE:
			if (!didMakeChanges) {
				parentDialog.dispose();
				return;
			}
		}

		final JDialog dialog = new JDialog(parentDialog, "Unsaved data", true);
		JButton cancelBtn = new JButton("Discard");
		JButton saveBtn = new JButton("Save");

		saveBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				switch (function) {
				case FUNCTION_ADD:					
					enterEntry();
					break;
				case FUNCTION_UPDATE:
					editEntry();
					break;
				}
			}
		});

		cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				parentDialog.dispose();
			}
		});

		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.add(cancelBtn);
		buttonPanel.add(saveBtn);

		JPanel centerPanel = new JPanel();
		centerPanel.setLayout(new GridLayout(1, 1));
		JLabel l1 = new JLabel("Warning:  Unless you save this information");
		JLabel l2 = new JLabel("before proceeding, it will be discarded.");
		centerPanel.add(l1);
		centerPanel.add(l2);

		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(centerPanel, BorderLayout.CENTER);
		panel.add(buttonPanel, BorderLayout.SOUTH);
		panel.add(new JLabel(" "), BorderLayout.WEST);

		Dimension d = panel.getPreferredSize();
		Dimension nD = new Dimension(d.width + 10, d.height + 40);
		panel.setPreferredSize(nD);

		Container contentPane = dialog.getContentPane();
		contentPane.setLayout(new BorderLayout(40, 40));
		contentPane.add(panel, BorderLayout.CENTER);

		dialog.setLocation(new Point(250, 250));
		dialog.pack();
		dialog.setVisible(true);

		updateMediaToolBarAnnotatorList();
	}

	public void editEntry() {
		// at least one valid video file should be selected
		if (!videoPanel.isAtLeastOneValidVideoFileAvailable()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"Warning:  Please select and display at least one valid available video file.");
			return;
		} else if (participantsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
			.getSignStreamApplication()
			.confirmationDialog("Macro Unit Warning",
					"<html>Warning:  You must add at least one participant to the collection before proceeding."
					+ "<br><br>To do this, click the 'Add' button under the 'Participants' section of this window.</html>");
			return;	
		} else if (annotatorsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"<html>Warning:  You must add at least one annotator to the collection before proceeding."
							+ "<br><br>To do this, click the 'Add' button under the 'Annotators' section of this window.</html>");
			return;			
		} else if (videoPanel.equals("")) {
			return;
		} else {

            // set participants in db and library
            setParticipants();

			// set annotators in db and library
            setAnnotators();

			// save participants, annotators, and search directories to the local and global files
            saveParticipantsAndAnnotatorsLibraries();

            // set macro unit label
            setMacroUnitLabel();

            ArrayList<LoadedMediaFile> selectedFiles = getSelectedFiles();
            ArrayList<SS3MediaFile> filesToLoad = getFilesToLoad(selectedFiles);
            ArrayList<SS3MediaFile> allFiles = getAllFiles(selectedFiles);

            db.setGraphFiles(graphFiles);

            updateDatabase(allFiles);

            updateMediaToolBarAnnotatorList();

            // update the segment tier labels to reflect any edits made in the participants
            segmentPanel.update(db);

            // load video files
            loadMovies(filesToLoad);

            updateZooming();

            parentDialog.dispose();
		}
	}
    
    public AnnotatorsDialog getParticipantsDialog() {
    		return participantsDialog;
    }
    
    public AnnotatorsDialog getAnnotatorsDialog() {
    		return annotatorsDialog;
    }

	private void setMacroUnitLabel() {

        String macroUnitLabel = macroUnitTxt.getText().trim();

        // update Macro Unit Excerpt
        if (!db.getExcerpt().equals(macroUnitLabel)) {

            DbNavigationPanel navigation = segmentPanel
                    .getDbNavigationPanel();
            Object root = navigation.getJTree().getModel().getRoot();

            if (root == null)
                return;

            DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
            DefaultMutableTreeNode node;

            if (rootNode.getChildCount() > 0) {
                Object obj = rootNode.getChildAt(0);
                node = (DefaultMutableTreeNode) obj;

                String str = (String) node.getUserObject();

                SS3Singleton.getSignStreamApplication().setTitle(str + ": " + macroUnitLabel);

                Enumeration children = node.children();
                while (children.hasMoreElements()) {
                    DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) children
                            .nextElement();
                    Excerpt excerpt = (Excerpt) fNode.getUserObject();

                    if (excerpt.getExcerpt().equals(db.getExcerpt())) {
                        excerpt.setExcerpt(macroUnitLabel);

                        navigation.refresh(rootNode);
                        expandAll(navigation.getJTree(), new TreePath(
                                rootNode), true);
                    }
                }
            }
        }
    }

    private void saveParticipantsAndAnnotatorsLibraries() {
        // save participants, annotators, and search directories
        edu.bu.signstream.common.db.UtilityFilesXMLWriter writer = new edu.bu.signstream.common.db.UtilityFilesXMLWriter();
        SS3DatabaseWriter dbWriter = new SS3DatabaseWriter();
        try {
			participantsLibrary = participantsPaneScrolledView.getParticipantLibrary();
			annotatorsLibrary = annotatorsPaneScrolledView.getParticipantLibrary();
			
            writer.saveParticipantsLibrary(participantsLibrary);
            writer.saveAnnotatorsLibrary(annotatorsLibrary);
            dbWriter.saveMediaLibrary(mLibrary);
        } catch (Exception ex) {
            ex.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);
        }
    }

    private void setParticipants() {

        String prtName = "";
        if (prtView.size() > 0) {
            ParticipantView participantView = prtView.get(0);
            prtName = participantView.getParticipant().getFullName();
        }

        db.setParticipantName(prtName);

        for (ParticipantView view : prtView) {
            SS3Participant prt = view.getParticipant();
            if (db.getSelectedParticipant(prt.getId()) == null) {	// if new participant, add it
                db.addSelectedParticipant(prt);
            }
            else {	// if edited participant, reset in db and library
            		participantsPaneScrolledView.getParticipantLibrary().editParticipant(prt);
            }
        }
    }

    private void setAnnotators() {
        // set annotators
        for (ParticipantView view : antView) {
            SS3Participant annotator = view.getParticipant();
            SS3Participant databaseAnnotator 
            		= db.getSelectedAnnotator(annotator.getId());
            		
            if (databaseAnnotator == null) {	// if new annotator, add it
                db.addSelectedAnnotator(annotator);
            }
            else { 	// if edited annotator, reset in db and library
            		databaseAnnotator = annotator;            		
            		annotatorsPaneScrolledView.getParticipantLibrary().editParticipant(annotator);
            }
        }
    }

    private void updateDatabase(ArrayList<SS3MediaFile> filesToLoad) {

        db.getMediaFiles().clear();

        for(SS3MediaFile file : filesToLoad) {
            db.getMediaFiles().put(file.getId(), file);
        }

        try {
            db.save();
            
            String dbDirectory = db.getParentDirectoryPath();
            String dir = db.getDirectory();
            String directory = dbDirectory + File.separator + dir;

            zipDirectory(directory);

        } catch (Exception e) {
            e.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
        }
    }

    private ArrayList<SS3MediaFile> getFilesToLoad(ArrayList<LoadedMediaFile> selectedFiles) {

        ArrayList<SS3MediaFile> filesToLoad = new ArrayList<SS3MediaFile>();

        for (LoadedMediaFile file : selectedFiles) {
            SS3MediaFile mFile = file.getMediaFile();
            String id = mFile.getId();

            if (id == null || id.isEmpty())
                mFile.setId(Util.getLongUniqueNumber());

            db.addMediaFile(mFile);

            if (file.isDisplay())
                filesToLoad.add(mFile);
        }

        return filesToLoad;
    }
    
    private ArrayList<SS3MediaFile> getAllFiles(ArrayList<LoadedMediaFile> selectedFiles) {

        ArrayList<SS3MediaFile> allFiles = new ArrayList<SS3MediaFile>();

        for (LoadedMediaFile file : selectedFiles) {
            SS3MediaFile mFile = file.getMediaFile();
            String id = mFile.getId();

            if (id == null || id.isEmpty())
                mFile.setId(Util.getLongUniqueNumber());

            db.addMediaFile(mFile);

            allFiles.add(mFile);
        }

        return allFiles;
    }

    private ArrayList<LoadedMediaFile> getSelectedFiles() {

        ArrayList<LoadedMediaFile> selectedFiles = new ArrayList<LoadedMediaFile>();

        ArrayList<VideoPanel> videoPanels = videoPanel.getVideoPanels();

        for (VideoPanel vp : videoPanels) {
            LoadedMediaFile lFile = vp.getMediaFile();
            if (lFile != null && lFile.getMediaFile() != null
                    && lFile.getMediaFile().getFile() != null) {
                selectedFiles.add(lFile);
            }
        }

        return selectedFiles;
    }

    private void updateZooming() {
//        MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
//
//        int movieDuration = controller.getMovieDuration();
//        int packetDuration = (int) controller.getPacketDuration();
//
//        segmentPanel.getZoomer().getCoordinateTimeConvertor()
//                .setPacketDuration(packetDuration);

        double width;

        if(segmentPanel.isUtteranceLoaded()){
            width = segmentPanel.getUtteranceView().getVisibleWidth2();
        } else {
            width = segmentPanel.getDatabaseView().getVisibleWidth2();
        }

        if (width > -1) {

            // Only width is used
            segmentPanel.getZoomer().fillEntirePage(0, 0, width);

//            double pixelInTimeFrame = segmentPanel.getZoomer()
//                    .getMinPixelInTimeFramePresentation();
//            segmentPanel.getZoomer().getCoordinateTimeConvertor()
//                    .setPixelInTimeFrame(pixelInTimeFrame);

            segmentPanel.updateJMenuBar();

            segmentPanel.repaint();
        }
    }

    private void disposeMovies(ArrayList<SS3MediaFile> files) {

        ArrayList<XugglerMovie> disposableMovies = getDisposableMovies(files);

        for (XugglerMovie movie : disposableMovies) {
        	// no!
//            movie.getMScreen().dispose();
            segmentPanel.getMultipleMovieController().getMovies().remove(movie);
//            segmentPanel.getMultipleMovieController().videoClosed();
        }
    }

    private ArrayList<XugglerMovie> getDisposableMovies(ArrayList<SS3MediaFile> files) {

        ArrayList<XugglerMovie> disposableMovies = new ArrayList<XugglerMovie>();

        ArrayList<XugglerMovie> movies = segmentPanel
                .getMultipleMovieController().getMovies();

        for (XugglerMovie movie : movies) {
            if (isMovieRemovable(movie, files)) {
                disposableMovies.add(movie);
            }
        }
        return disposableMovies;
    }

    private boolean isMovieRemovable(XugglerMovie movie, ArrayList<SS3MediaFile> files) {
        for (SS3MediaFile file : files) {
            String name = file.getFile().getAbsolutePath();
            if (movie.getFilename().equals(name)) {
               return false;
            }
        }
        return true;
    }

    private void loadMovies(ArrayList<SS3MediaFile> filesToLoad) {

        int currentMovieTime = segmentPanel.getMultipleMovieController()
                .getCurrentTime();

        registerMoviesWithController(filesToLoad);

        disposeMovies(filesToLoad);

        MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();

        controller.load();

        controller.setCurrentTime(currentMovieTime);

        double n = controller.getMovieDuration()
                / MultipleMovieController.NUMBER_OF_SLIDER_TICKS;

        double sliderNewPosition = currentMovieTime / n;

        SS3Singleton.getVideoControlManager().moveSliderTo(
                (int) sliderNewPosition);
    }

    private void registerMoviesWithController(ArrayList<SS3MediaFile> filesToLoad) {
        SS3SignStreamApplication.logger.log(Level.INFO,
                "Creating XugglerMovie object");

        ArrayList<XugglerMovie> movies = segmentPanel
                .getMultipleMovieController().getMovies();

        for (SS3MediaFile file : filesToLoad) {
            try {
                registerMovieWithController(file, movies);
            } catch (Exception ex) {
                String msg = "<HTML>Warning: " + ex.getMessage() + "<BR>";
                msg = msg + "Please select valid video file.</HTML>";
                SS3Singleton.getSignStreamApplication()
                        .confirmationDialog("Resouce Update", msg);
            }
        }
        
        //Simulate drag to get videos to render.
        SS3Singleton.getVideoControlManager().getMultipleMovieController().goToVideoPercent(0);
    }

    private boolean isMovieLoaded(SS3MediaFile file, ArrayList<XugglerMovie> movies) {

        String name = file.getFile().getAbsolutePath();

        for (XugglerMovie movie : movies) {
            if (movie.getFilename().equals(name)) {
                return true;
            }
        }

        return false;
    }

    private void registerMovieWithController(SS3MediaFile file, ArrayList<XugglerMovie> movies) throws Exception {

        if (!isMovieLoaded(file, movies)) {

            MultipleMovieController controller = SS3Singleton
                    .getVideoControlManager().getMultipleMovieController();

            XugglerMovie movie = new XugglerMovie(file, controller);
            controller.registerMovie(movie);
        }
    }
	
    /**
     * Zips a directory into a ".ss3" archive file.
     * Assumes the input has already had its ".ss3" extension added.
     * @param dirName
     * @return True if directory was zipped and deleted and false if an exception was thrown
     */
	public boolean zipDirectory(String dirName) {
		try {
			File dirFile = new File(dirName);
			String cDirName = dirFile.getName();

			if (cDirName.startsWith(SS3Database.tempDirectoryPrefix)) {
				cDirName = cDirName.substring(4);
			}

			String zipFile = dirFile.getParentFile().getPath()
					+ File.separatorChar + cDirName;

			byte[] buffer = new byte[1024];

			try {
				FileOutputStream fout = new FileOutputStream(zipFile);
				ZipOutputStream zout = new ZipOutputStream(fout);

				File dir = new File(dirName);

				File[] children = dir.listFiles();
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];

						FileInputStream fin = new FileInputStream(file);
						zout.putNextEntry(new ZipEntry(file.getName()));
						int length;

						while ((length = fin.read(buffer)) > 0) {
							zout.write(buffer, 0, length);
						}

						zout.closeEntry();
						fin.close();
					}
				}

				// close the ZipOutputStream
				zout.close();
				fout.close();

				// remove unziped files
				if (children != null) {
					for (int i = 0; i < children.length; i++) {
						File file = children[i];
						file.delete();
					}
				}
				dir.delete();

				if (zipFile != null) {
					String fileToDelete = zipFile + ".zip";
					File file = new File(fileToDelete);
					if (file.exists()) {
						file.delete();
					}
				}

			} catch (FileNotFoundException ex) {
				ex.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ex);

				return false;
			}
		} catch (IOException ioe) {
			ioe.printStackTrace();
			SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", ioe);
			return false;
		}
		return true;
	}

	private String getDefaultMacroUnitLabel() {
		DbNavigationPanel navigation = segmentPanel.getDbNavigationPanel();
		Object root = navigation.getJTree().getModel().getRoot();

		if (root == null)
			return "";

		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
		DefaultMutableTreeNode node;

		String txt = macroUnitTxt.getText().trim();

		if (rootNode.getChildCount() > 0) {
			Object obj = rootNode.getChildAt(0);
			node = (DefaultMutableTreeNode) obj;

			int counter = 0;
			Enumeration children = node.children();
			while (children.hasMoreElements()) {
				DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) children
						.nextElement();
				Excerpt excerpt = (Excerpt) fNode.getUserObject();
				int fnID = Integer.parseInt(excerpt.getID());
				counter = fnID > counter ? fnID : counter;
			}

			if (txt.isEmpty())
				txt = "macro_unit_" + (counter + 1);

			return txt;
		}
		return "";
	}

	private void enterEntry() {
		// at least one valid video file should be selected
		if (!videoPanel.isAtLeastOneValidVideoFileAvailable()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"Warning:  Please select at least one valid available video file.");

			return;
		} else if (participantsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
			.getSignStreamApplication()
			.confirmationDialog("Macro Unit Warning",
					"<html>Warning:  You must add at least one participant to the collection before proceeding."
					+ "<br><br>To do this, click the 'Add' button under the 'Participants' section of this window.</html>");
			return;	
		} else if (annotatorsPaneScrolledView.isParticipantViewListEmpty()) {
			SS3Singleton
					.getSignStreamApplication()
					.confirmationDialog("Macro Unit Warning",
							"<html>Warning:  You must add at least one annotator to the collection before proceeding."
							+ "<br><br>To do this, click the 'Add' button under the 'Annotators' section of this window.</html>");
			return;			
		} else {
            // save participants, annotators, and search directories
            saveParticipantsAndAnnotatorsLibraries();

			db = null;
			Excerpt excerpt = null;

			DbNavigationPanel navigation = segmentPanel.getDbNavigationPanel();
			Object root = navigation.getJTree().getModel().getRoot();

			if (root == null)
				return;

			DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) root;
			DefaultMutableTreeNode cNode = null;
			DefaultMutableTreeNode node = null;

			String excTxt = macroUnitTxt.getText().trim();

			if (rootNode.getChildCount() > 0) {
				Object obj = rootNode.getChildAt(0);
				node = (DefaultMutableTreeNode) obj;

				int counter = 0;
				Enumeration children = node.children();
				while (children.hasMoreElements()) {
					DefaultMutableTreeNode fNode = (DefaultMutableTreeNode) children
							.nextElement();
					Excerpt ex = (Excerpt) fNode.getUserObject();
					int fnID = Integer.parseInt(ex.getID());
					counter = fnID > counter ? fnID : counter;
				}

				if (excTxt.length() == 0)
					excTxt = "macro_unit_" + (counter + 1);

				excerpt = new Excerpt("" + (counter + 1), excTxt);

				cNode = new DefaultMutableTreeNode(excerpt);
				node.add(cNode);
			}

			db = new SS3Database(new TimeInfo(), new TimeInfo());

			// copy over graph files
			db.setGraphFiles(graphFiles);

			SS3GlossWindow glsWindow = new SS3GlossWindow();

			SS3GlossWindowSegment displayTimePeriod = new SS3GlossWindowSegment();
			displayTimePeriod.setLabel("Display Time Period");
			glsWindow.setDisplayTimePeriod(displayTimePeriod);

			SS3TemporalPartition temporalPartition = new SS3TemporalPartition();
			temporalPartition.setLabel("Temporal Partition 1");

			SS3GlossWindowSegment tpSegment = new SS3GlossWindowSegment();
			tpSegment.setLabel("Segment Tier 1");
			tpSegment.setID("" + Util.getUniqueNumber());
			if (prtView.size() == 1) {
				tpSegment.setParticipant(prtView.get(0).getParticipant());
			}
			else {
				ArrayList<SS3Participant> participants = new ArrayList<SS3Participant>(prtView.size());
				for (ParticipantView pView : prtView) {
					participants.add(pView.getParticipant());
				}
				AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
						participants, 
						getParticipantsDialog(), parentDialog);
				annotatorSelectionDialog.setLocation(250, 250);
				annotatorSelectionDialog.setVisible(true);
				
				// get last selected item in list of participants before confirmation, then pass it along to createSegment()
				SS3Participant participant = annotatorSelectionDialog.getAnnotatorSelectionPanel().getSelectedAnnotator();
				tpSegment.setParticipant(participant);
			}
			temporalPartition.addTemporalPartitionSegment(tpSegment);
			temporalPartition.setDefaultSegment(tpSegment);

			ArrayList<SS3TemporalPartition> partitions = new ArrayList<SS3TemporalPartition>();
			partitions.add(temporalPartition);
			glsWindow.setTemporalPartitions(partitions);

			db.setSS3GlossWindow(glsWindow);

			db.setFileName(excTxt + ".xml");
			db.setDirectory(node.getUserObject().toString());

			db.setExcerpt(excerpt.getExcerpt());
			db.setExcerptObj(excerpt);
			SS3Singleton.getSS3DatabaseCollection()
					.addSS3SDatabase(db);
			
			segmentPanel.updateLoadedDatabase();

			for (ParticipantView view : prtView) {
				db.addSelectedParticipant(view.getParticipant());
			}

			// set annotators
			for (ParticipantView annot : antView) {
				db.addSelectedAnnotator(annot.getParticipant());
			}

			try {
				db.save();
			} catch (Exception e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}

			// set participant
			String prtName = "";
			if (prtView.size() > 0) {
				ParticipantView participantView = prtView.get(0);
				prtName = participantView.getParticipant().getFullName();
			}

			db.setParticipantName(prtName);

			// load video files
			ArrayList<SS3MediaFile> lFiles = new ArrayList<SS3MediaFile>();
			ArrayList<SS3MediaFile> allFiles = new ArrayList<SS3MediaFile>();
			ArrayList<LoadedMediaFile> loadedFiles = new ArrayList<LoadedMediaFile>();

			ArrayList<VideoPanel> videoPanels = videoPanel.getVideoPanels();

			for (VideoPanel vp : videoPanels) {
				LoadedMediaFile lFile = vp.getMediaFile();
				if (lFile != null && lFile.getMediaFile() != null
						&& lFile.getMediaFile().getFile() != null) {
					loadedFiles.add(lFile);
				}
			}

			for (int i = 0; i < loadedFiles.size(); i++) {
				LoadedMediaFile file = loadedFiles.get(i);
				SS3MediaFile mFile = file.getMediaFile();
				String id = mFile.getId();

				if (id == null || id.length() == 0)
					mFile.setId(Util.getLongUniqueNumber());

				db.addMediaFile(mFile);
				excerpt.addOrOverrideMediaFileID(mFile.getId());

				if (file.isDisplay())
					lFiles.add(mFile);
				allFiles.add(mFile);
			}

			navigation.refresh(rootNode);
			expandAll(navigation.getJTree(), new TreePath(rootNode), true);

			if (cNode != null) {
				int rowToSelect = node.getChildCount() + cNode.getLevel() - 1;

				segmentPanel.updateLoadedDatabase();
				SS3Database nDB = SS3Singleton.getSS3DatabaseCollection()
						.getSS3Database(excerpt);
				segmentPanel.loadDatabase(nDB);
				segmentPanel.repaint();

				JFrame frame = segmentPanel.getParentFrame();
				
				if (frame != null) {
					Object uo = cNode.getUserObject();
					if (uo instanceof String) {
						frame.setTitle((String) uo);
						frame.repaint();
					}
				}

				navigation.getJTree().setSelectionRow(rowToSelect);
				navigation.getJTree().repaint();
			}

			TimeInfo startTimeInfo = new TimeInfo();
			TimeInfo endTimeInfo = new TimeInfo();
			startTimeInfo.setMovieTime(0);
			endTimeInfo.setMovieTime(0);

			DefaultMutableTreeNode bNode = (DefaultMutableTreeNode) node
					.getChildBefore(cNode);
			
			if (bNode != null) {
				Excerpt ex = (Excerpt) bNode.getUserObject();
				SS3Database ndb = SS3Singleton.getSS3DatabaseCollection()
						.getSS3Database(ex);
				
				startTimeInfo.setMovieTime(ndb.getEndTimeInfo().getMovieTime());
				endTimeInfo.setMovieTime(ndb.getEndTimeInfo().getMovieTime());

				Excerpt cEx = (Excerpt) cNode.getUserObject();
				SS3Database cdb = SS3Singleton.getSS3DatabaseCollection()
						.getSS3Database(cEx);
				cdb.getStartTimeInfo().setMovieTime(
						ndb.getEndTimeInfo().getMovieTime());
				cdb.getEndTimeInfo().setMovieTime(
						ndb.getEndTimeInfo().getMovieTime());
			}

			SS3Singleton.getMediaToolBar(null).setPlayButtonLabel(MediaToolBar.VIDEO);

            // set macro unit label
            setMacroUnitLabel();

            updateDatabase(allFiles);
            
            updateMediaToolBarAnnotatorList();

			segmentPanel.loadDatabase(db);

            // update the segment tier labels to reflect any edits made in the participants
    			segmentPanel.update(db);

            // load video files
            loadMovies(lFiles);
            
            updateZooming();

			parentDialog.dispose();
			
			// select annotator screen	
			AnnotatorSelectionDialog annotatorSelectionDialog = new AnnotatorSelectionDialog(
					SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators(), 
					annotatorsDialog, parentDialog);
			annotatorSelectionDialog.setLocation(250, 250);
			annotatorSelectionDialog.setVisible(true);
		}
	}

	private void expandAll(JTree tree, TreePath parent, boolean expand) {

		// Traverse children
		TreeNode node = (TreeNode) parent.getLastPathComponent();
		if (node.getChildCount() >= 0) {
			for (Enumeration e = node.children(); e.hasMoreElements();) {
				TreeNode n = (TreeNode) e.nextElement();
				TreePath path = parent.pathByAddingChild(n);
				expandAll(tree, path, expand);
			}
		}

		// Expansion or collapse must be done bottom-up
		if (expand) {
			tree.expandPath(parent);
		} else {
			tree.collapsePath(parent);
		}
	}

    private void updateMediaToolBarAnnotatorList() {
		ArrayList<SS3Participant> selectedAnnotators = 
				SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getSelectedAnnotators();
		ArrayList<String> annotatorStrings = new ArrayList<String>();
		
		for (SS3Participant annotator: selectedAnnotators) {
			annotatorStrings.add(annotator.getLabel());
		}
		
		// update media tool bar annotator combobox
		SS3Singleton.getMediaToolBar(SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel())
				.updateAnnotatorList(annotatorStrings, SS3Singleton.getCurrentAnnotator());			
	}    	
    
    public void madeChange() {
    		this.didMakeChanges = true;
    }

	public void addGraphFile(SS3MediaFile gphFile) {
		graphFiles.put(gphFile.getId(), gphFile);
	}

	public void removeGraphFile(String id) {
		graphFiles.remove(id);
	}

}
