package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.navigation.DbNavigationPanel;
import edu.bu.signstream.ui.ButtonPanel;

public class NewDatabasePanel extends JPanel implements ActionListener {
   private JDialog parentDialog = null;

   private SignStreamSegmentPanel segmentPanel = null;

   private JTextField dbNameTxt = new JTextField(25);

   private JButton okBtn = new JButton("OK");

   private JButton cancelBtn = new JButton("Cancel");

   private final int OK = 0;

   private final int CANCEL = 1;

   public NewDatabasePanel(
      SignStreamSegmentPanel segmentPanel,
      JDialog parentDialog) {
      this.segmentPanel = segmentPanel;
      this.parentDialog = parentDialog;
      initUI();
   }

   private void initUI() {
      okBtn.setActionCommand("" + OK);
      cancelBtn.setActionCommand("" + CANCEL);
      okBtn.addActionListener(this);
      cancelBtn.addActionListener(this);

      okBtn.setSelected(true);

      setLayout(new BorderLayout());

      JPanel centerPanel = createCenterPanel();
      ButtonPanel bPanel = new ButtonPanel();
      bPanel.add(okBtn);
      bPanel.add(cancelBtn);

      add(centerPanel, BorderLayout.CENTER);
      add(bPanel, BorderLayout.SOUTH);
   }

   private JPanel createCenterPanel() {
      JPanel panel = new JPanel();
      JPanel dbPanel = createDBPanel();

      GridBagLayout gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      panel.setLayout(gbl);

      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      gbc.insets = new Insets(0, 0, 5, 0);
      panel.add(dbPanel, gbc);

      gbc.anchor = GridBagConstraints.NORTH;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.insets = new Insets(0, 0, 5, 0);

      return panel;
   }

   private JPanel createDBPanel() {
      JPanel panel = new JPanel();

      GridBagLayout gbl = new GridBagLayout();
      GridBagConstraints gbc = new GridBagConstraints();

      panel.setLayout(gbl);

      gbc.anchor = GridBagConstraints.NORTHWEST;
      gbc.gridwidth = GridBagConstraints.REMAINDER;

      gbc.anchor = GridBagConstraints.NORTH;
      gbc.fill = GridBagConstraints.HORIZONTAL;
      gbc.insets = new Insets(10, 0, 13, 0);

      gbc.anchor = GridBagConstraints.WEST;
      gbc.gridwidth = 1;
      gbc.insets = new Insets(13, 13, 13, 13);
      panel.add(new JLabel("Database Name"), gbc);

      add(Box.createHorizontalStrut(10));
      gbc.gridwidth = GridBagConstraints.REMAINDER;
      panel.add(dbNameTxt, gbc);

      return panel;
   }

   @Override
public void actionPerformed(ActionEvent ae) {
      String str = ae.getActionCommand();

      switch (Integer.parseInt(str)) {
         case OK :
            String dbName = dbNameTxt.getText();
            
            if(dbName == null | dbName.length() == 0) {
               
               SS3Singleton.getSignStreamApplication()
				.confirmationDialog("Create Resource Warning",
						"Warning:  Please enter the database name.");


               return;
            }
  
            DbNavigationPanel navigation = segmentPanel.getDbNavigationPanel();
            navigation.getDatabaseDelegate().addNewDatabase(dbName);
            navigation.refresh(navigation.getDatabaseDelegate().getMutableTreeNode());            
            navigation.getJTree().repaint();
             

            parentDialog.dispose();
            break;
         case CANCEL :
            segmentPanel.repaint();
            parentDialog.dispose();
            break;
      }
   }
}
