package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.Dimension;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.media.SS3MediaFile;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;

public class ParticipantView {

	private SS3Participant participant;

	private ParticipantButton infoBtn = null;  
	private ParticipantButton editBtn = null;  
	private ParticipantButton addBtn = null;   
	private ParticipantButton removeBtn = null; 
	private ParticipantButton graphBtn = null;
	private JLabel prtLbl = new JLabel();
	private JLabel graphFileLbl;

	private boolean isParticipant = true;

	private ParticipantsLibrary participantsLibrary = null;
	private ParticipantsLibrary annotatorsLibrary = null;

	private ArrayList<ParticipantView> enteredViews = null;

	public ParticipantView(SS3Participant participant, boolean isParticipant,
			boolean isAssociated, ParticipantsLibrary participantsLibrary,
			ParticipantsLibrary annotatorsLibrary,
			ArrayList<ParticipantView> enteredViews) {

		this.participant = participant;
		this.isParticipant = isParticipant;
		this.participantsLibrary = participantsLibrary;
		this.annotatorsLibrary = annotatorsLibrary;
		this.enteredViews = enteredViews;
		
		infoBtn = new ParticipantButton("Info",participant, isParticipant, this, null);
		editBtn = new ParticipantButton("Edit", participant, isParticipant, this, null);
		addBtn = new ParticipantButton("Add", participant, isParticipant, this, null);
		removeBtn = new ParticipantButton("Remove", participant, isParticipant, this, null);
		graphBtn = new ParticipantButton("Graph", participant, isParticipant, this, null);
		String id = participant.getGraphFileID();
		String name = "None";
		if (id != null) {
			SS3MediaFile graphFile = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getSS3Database().getGraphFile(id);
			if (graphFile != null) 
				name = graphFile.getName();
			else
				participant.setGraphFileID(null);		// if graph file is not located right now, remove reference to it, so the collection does not get saved with it
		}
		graphFileLbl = new JLabel(name);

		if (participant != null && participant.getLabel() != null) {
			prtLbl.setText(participant.getLabel());
		}

		prtLbl.setMaximumSize(new Dimension(200, 10));
		graphFileLbl.setMaximumSize(new Dimension(300, 10));

		infoBtn.setActionCommand("" + ParticipantListener.INFO);
		editBtn.setActionCommand("" + ParticipantListener.EDIT);
		addBtn.setActionCommand("" + ParticipantListener.ADD);
		removeBtn.setActionCommand("" + ParticipantListener.REMOVE);
		
		if (isParticipant) {
			graphBtn.setActionCommand("" + ParticipantListener.SELECT_GPH);
		}

		boolean isUpdatable = true;
		if (isParticipant) {
			isUpdatable = participantsLibrary.isLocalParticipant(participant
					.getId());
		} else {
			isUpdatable = annotatorsLibrary.isLocalParticipant(participant
					.getId());
		}

		if (isAssociated) {
			removeBtn.setEnabled(false);
		}

		editBtn.setEnabled(isUpdatable);

		UserInterfaceUtil.setButtonsLNF(infoBtn);
		UserInterfaceUtil.setButtonsLNF(editBtn);
		UserInterfaceUtil.setButtonsLNF(addBtn);
		UserInterfaceUtil.setButtonsLNF(removeBtn);
		
		if (isParticipant) {
			UserInterfaceUtil.setButtonsLNF(graphBtn);
		}

		removeBtn.setEnabled(!isAssociated);
	}

	public void addActionListener(ParticipantListener participantListener) {
		infoBtn.addActionListener(participantListener);
		editBtn.addActionListener(participantListener);
		addBtn.addActionListener(participantListener);
		removeBtn.addActionListener(participantListener);
		if (isParticipant) {
			graphBtn.addActionListener(participantListener);
		}
	}
	
	public ArrayList<ParticipantView> getEnteredViews() {
		return enteredViews;
	}

	public ParticipantsLibrary getParticipantsLibrary() {
		return participantsLibrary;
	}

	public ParticipantsLibrary getAnnotatorsLibrary() {
		return annotatorsLibrary;
	}

	public SS3Participant getParticipant() {
		return participant;
	}

	public boolean isParticipant() {
		return isParticipant;
	}

	public void setParticipant(SS3Participant participant) {
		this.participant = participant;

		boolean isUpdatable = true;
		if (isParticipant) {
			isUpdatable = participantsLibrary.isLocalParticipant(participant
					.getId());
		} else {
			isUpdatable = annotatorsLibrary.isLocalParticipant(participant
					.getId());
		}
		editBtn.setEnabled(isUpdatable);

		prtLbl.setText(participant.getLabel());
		prtLbl.repaint();
	}

	public JButton getInfoBtn() {
		return infoBtn;
	}

	public JButton getEditBtn() {
		return editBtn;
	}

	public JButton getAddBtn() {
		return addBtn;
	}

	public JButton getRemoveBtn() {
		return removeBtn;
	}
	
	public JButton getGraphBtn() {
		return graphBtn;
	}

	public JLabel getParticipantLbl() {
		return prtLbl;
	}

	public JLabel getGraphFileLbl() {
		return graphFileLbl;
	}

	public void setGraphFileLblText(String text) {
		graphFileLbl.setText(text);
	}
}
