package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowItem;
import edu.bu.signstream.common.util.vo.ss3.db.SS3GlossWindowSegment;
import edu.bu.signstream.common.util.vo.ss3.db.SS3TemporalPartition;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.ui.panels.AnnotatorsDialog;

public class ParticipantsPaneScrolledView extends JPanel {

	private static final long serialVersionUID = 1L;

	private ParticipantsLibrary annotatorsLibrary;
	private ArrayList<ParticipantView> prtView = new ArrayList<ParticipantView>();

	private ParticipantButton addParticipantBtn = null;
	private JButton editParticipantBtn = new JButton("Edit");
	private JButton infoParticipantBtn = new JButton("Info");
	private JButton removeParticipantBtn = new JButton("Remove");
	private JButton selectParticipantBtn = new JButton("Select");
	private JScrollPane scroller = null;

	private ParticipantPane participantPane;

	private Dimension d = new Dimension(200, 200);

	private boolean isParticipant = true;

	private SS3Database db = null;
	
	private ParticipantListener participantListener = null;
	
	private AnnotatorsDialog annotatorsDialog = null;

	public ParticipantsPaneScrolledView(ArrayList<ParticipantView> prtViews,
			ParticipantsLibrary annotatorsLibrary, SS3Database db,
			String title, Dimension d, boolean isParticipant) {
		super(new BorderLayout());
		this.prtView = prtViews;
		this.d = d;
		this.isParticipant = isParticipant;
		this.annotatorsLibrary = annotatorsLibrary;
		this.db = db;
		this.annotatorsDialog = new AnnotatorsDialog(null, this, null);
		
		participantListener = new ParticipantListener(this, annotatorsDialog);

		for (ParticipantView prtView : prtViews) {
			prtView.addActionListener(participantListener);
		}

		JLabel titleLbl = new JLabel(title);
		
		addParticipantBtn = new ParticipantButton("Add", null, isParticipant, null, null);

		UserInterfaceUtil.setLabelLNF(titleLbl);
		UserInterfaceUtil.setButtonsLNF(addParticipantBtn);
		UserInterfaceUtil.setButtonsLNF(editParticipantBtn);
		UserInterfaceUtil.setButtonsLNF(infoParticipantBtn);
		UserInterfaceUtil.setButtonsLNF(removeParticipantBtn);
		if (isParticipant) {
			UserInterfaceUtil.setButtonsLNF(selectParticipantBtn);
			selectParticipantBtn.setEnabled(false);
		}

		editParticipantBtn.setEnabled(false);
		infoParticipantBtn.setEnabled(false);
		removeParticipantBtn.setEnabled(false);		

		JPanel instructionPanel = new JPanel(new GridLayout(0, 1));
		instructionPanel.setFocusable(true);
		instructionPanel.add(new JLabel(title));
		
		addParticipantBtn.setActionCommand("" + ParticipantListener.ADD);

		participantPane = new ParticipantPane();
		participantPane.setBackground(Color.white);
		addParticipantBtn.addActionListener(participantListener);

		scroller = new JScrollPane(participantPane);
		scroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		scroller.setPreferredSize(d);

		add(instructionPanel, BorderLayout.PAGE_START);
		add(scroller, BorderLayout.CENTER);
	}

	public boolean isParticipant() {
		return isParticipant;
	}

	public ParticipantsLibrary getParticipantLibrary() {
		return annotatorsLibrary;
	}

	public ArrayList<ParticipantView> getParticipantViews() {
		return prtView;
	}
	
	public AnnotatorsDialog getAnnotatorsDialog() {
		return annotatorsDialog;
	}
	
	public boolean isParticipantViewListEmpty() {
		return prtView.isEmpty();
	}

	public boolean addParticipant(SS3Participant participant) {

	    for (ParticipantView participantView : getParticipantViews()) {
	        if (participantView.getParticipant().equals(participant)) {
	            return false;
            }
        }

		String id = participant.getId();
		boolean isAssociated = false;

		ArrayList<SS3TemporalPartition> partitions = db.getSS3GlossWindow().getTemporalPartitions();
		for (SS3TemporalPartition partition : partitions) {
			for (SS3GlossWindowSegment segment : partition.getTemporalPartitionSegments()) {
				if (isParticipant) {
					SS3Participant segmentParticipant = segment.getParticipant();
					if (segmentParticipant != null && id.equals(segmentParticipant.getId())) {
						isAssociated = true;
					}
				}
			}
		}

		ParticipantView view = new ParticipantView(participant, isParticipant, isAssociated, annotatorsLibrary, annotatorsLibrary, prtView);
		view.addActionListener(participantListener);
		prtView.add(view);

		participantPane.revalidate();
		participantPane.initUI();

		int h = participantPane.getPreferredSize().height;

		Rectangle rect = new Rectangle(0, h - d.height, participantPane.getPreferredSize().width, h);
		participantPane.scrollRectToVisible(rect);

		return true;
	}

	public void removeParticipantView(ParticipantView view) {
		prtView.remove(view);

		if (isParticipant) {
			SS3Participant participant 
				= db.getSelectedParticipant(view.getParticipant().getId());
			db.getSelectedParticipants().remove(participant);
		}
		else {
			SS3Participant annotator 
				= db.getSelectedAnnotator(view.getParticipant().getId());
			db.getSelectedAnnotators().remove(annotator);
		}

		participantPane.revalidate();
		participantPane.initUI();

		int h = participantPane.getPreferredSize().height;

		Rectangle rect = new Rectangle(0, h - d.height,
				participantPane.getPreferredSize().width, h);
		participantPane.scrollRectToVisible(rect);
	}

	/** The component inside the scroll pane. */
	public class ParticipantPane extends JPanel {

		private static final long serialVersionUID = 1L;

		public ParticipantPane() {
			initUI();
		}

		public void initUI() {
			removeAll();

			repaint();

			JPanel gbPanel = new JPanel(new GridBagLayout());
			gbPanel.setBackground(Color.WHITE);
			gbPanel.setForeground(Color.white);
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.HORIZONTAL;
			c.insets = new Insets(1, 1, 1, 1);
			c.weightx = 0;

			for (int i = 0; i < prtView.size() ; i++) {
				ParticipantView view = prtView.get(i);
				c.anchor = GridBagConstraints.NORTHWEST;
				c.gridy = i * 2;
				c.gridx = 0;
				gbPanel.add(view.getInfoBtn(), c);
				c.gridx = 1;
				gbPanel.add(view.getEditBtn(), c);
				c.gridx = 2;
				gbPanel.add(view.getAddBtn(), c);
				c.gridx = 3;
				gbPanel.add(view.getRemoveBtn(), c);
				c.gridx = 4;
				gbPanel.add(view.getParticipantLbl(), c);
				if (isParticipant) {
					c.gridy = i * 2 + 1;
					c.gridx = 0;
					gbPanel.add(view.getGraphBtn(), c);
					c.gridx = 1;
					gbPanel.add(view.getGraphFileLbl(), c);
				}
			}

			c.anchor = GridBagConstraints.NORTHWEST;
			c.gridy = prtView.size() * 2;
			c.gridx = 0;
			gbPanel.add(infoParticipantBtn, c);
			c.gridx = 1;
			gbPanel.add(editParticipantBtn, c);
			c.gridx = 2;
			gbPanel.add(addParticipantBtn, c);
			c.gridx = 3;
			gbPanel.add(removeParticipantBtn, c);
			if (isParticipant) {
				c.gridx = 4;
				gbPanel.add(selectParticipantBtn, c);
				c.gridx = 5;
			}else {
				c.gridx = 4;
			}
			gbPanel.add(new JLabel(""), c);

			setLayout(new FlowLayout(FlowLayout.LEFT));
			add(gbPanel);
		}
	}
}
