package edu.bu.signstream.ui.panels.newDatabase;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.bu.signstream.common.util.UserInterfaceUtil;
import edu.bu.signstream.media.fileNavigation.LoadedMediaFile;

public class VideoPanelScrolledView extends JPanel implements ItemListener {

	private static final long serialVersionUID = 1L;
	
	private ArrayList<VideoPanel> videoPanels = new ArrayList<VideoPanel>();
	
	private VideoPanel videoPanel1 = null;
	private VideoPanel videoPanel2 = null;
	private VideoPanel videoPanel3 = null;
	private VideoPanel videoPanel4 = null;
	
	private JScrollPane scroller = null;
	private VideoFilesPane videoFilesPane;

	private JLabel videoLabel = new JLabel(" Video ");
	private JLabel displayLabel = new JLabel(" Display ");
	private JLabel fileNameLabel = new JLabel(" File Name ");
	private JLabel lookupLabel = new JLabel(" Lookup ");	
	private JButton lookupBtn = null;
	
	public VideoPanelScrolledView(ArrayList<LoadedMediaFile> mFiles, JButton lookupBtn, Dimension d, JFrame parentFrame) {
		super(new BorderLayout());
		this.lookupBtn = lookupBtn;
		
		UserInterfaceUtil.setLabelLNF(videoLabel);
		UserInterfaceUtil.setLabelLNF(displayLabel);
		UserInterfaceUtil.setLabelLNF(fileNameLabel);
		UserInterfaceUtil.setLabelLNF(lookupLabel);

		if(mFiles.size() >= 1) {
			videoPanel1 = new VideoPanel(mFiles.get(0), parentFrame);
		} else {
			videoPanel1 = new VideoPanel(null, parentFrame);
		}
		videoPanel1.getLookupCb().addItemListener(this);
		videoPanels.add(videoPanel1);
		
		if(mFiles.size() >= 2) {
			videoPanel2 = new VideoPanel(mFiles.get(1), parentFrame);
		} else {
			videoPanel2 = new VideoPanel(null, parentFrame);
		}
		videoPanel2.getLookupCb().addItemListener(this);
		videoPanels.add(videoPanel2);
		
		if(mFiles.size() >= 3) {
			videoPanel3 = new VideoPanel(mFiles.get(2), parentFrame);
		} else {
			videoPanel3 = new VideoPanel(null, parentFrame);
		}
		videoPanel3.getLookupCb().addItemListener(this);
		videoPanels.add(videoPanel3);
		
		if(mFiles.size() >= 4) {
			videoPanel4 = new VideoPanel(mFiles.get(3), parentFrame);
		} else {
			videoPanel4 = new VideoPanel(null, parentFrame);
		}
		videoPanel4.getLookupCb().addItemListener(this);
		videoPanels.add(videoPanel4);

		lookupBtn.setEnabled(isLookupEnabled());

		videoFilesPane = new VideoFilesPane();
		videoFilesPane.setBackground(Color.white);

		scroller = new JScrollPane(videoFilesPane);
		scroller.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		scroller.setPreferredSize(d);

		add(scroller, BorderLayout.CENTER);
	}
	
	public void update(ArrayList<LoadedMediaFile> mFiles) {
		if(mFiles.size() >= 1) {
			videoPanel1.setMediaFile(mFiles.get(0));
		} else {
			videoPanel1.setMediaFile(null);
		}
		videoPanel1.repaint();
		
		if(mFiles.size() >= 2) {
			videoPanel2.setMediaFile(mFiles.get(1));
		} else {
			videoPanel2.setMediaFile(null);
		}
		videoPanel2.repaint();
		
		if(mFiles.size() >= 3) {
			videoPanel3.setMediaFile(mFiles.get(2));
		} else {
			videoPanel3.setMediaFile(null);
		}
		videoPanel3.repaint();
		
		if(mFiles.size() >= 4) {
			videoPanel4.setMediaFile(mFiles.get(3));
		} else {
			videoPanel4.setMediaFile(null);
		}
		videoPanel4.repaint();
	}
	
	public boolean isAtLeastOneValidVideoFileAvailable() {
		for(VideoPanel vp : videoPanels) {
			LoadedMediaFile lFile = vp.getMediaFile();
			if(lFile != null && lFile.getMediaFile() != null && lFile.getMediaFile().getFile() != null && lFile.getMediaFile().getFile().exists()
					&& vp.getDisplayCb().isSelected()) {
				return true;
			}
		}
		return false;
	}
	
	public ArrayList<VideoPanel> getVideoPanels() {
		return videoPanels;
	}
	
	private boolean isLookupEnabled() {
		for(VideoPanel  vp : videoPanels) {
			if(vp.getLookupCb().isSelected())
				return true;
		}
		return false;
	}
	
	@Override
	public void itemStateChanged(ItemEvent event) {
			lookupBtn.setEnabled(isLookupEnabled());
	}

	/** The component inside the scroll pane. */
	public class VideoFilesPane extends JPanel {

		private static final long serialVersionUID = 1L;

		public VideoFilesPane() {
			JPanel videoPanel = new JPanel();
			GridBagConstraints c = new GridBagConstraints();
			videoPanel.setLayout(new GridBagLayout());
			videoPanel.setBackground(Color.WHITE);
			videoPanel.setForeground(Color.white);

			c.anchor = GridBagConstraints.WEST;
			c.insets = new Insets(0, 1, 0, 1);
			
			c.gridx = 1;
			c.gridwidth = 3;
			videoPanel.add(videoLabel, c);
			c.gridx = 4;
			c.gridwidth = 1;
			videoPanel.add(displayLabel, c);
			c.gridx = 5;
//			videoPanel.add(lookupLabel, c);
			c.gridx = 6;
			videoPanel.add(fileNameLabel, c);
			
			addComponent( videoPanel1, c, 1, videoPanel);
			addComponent( videoPanel2, c, 2, videoPanel);
			addComponent( videoPanel3, c, 3, videoPanel);
			addComponent( videoPanel4, c, 4, videoPanel);

			setLayout(new FlowLayout(FlowLayout.LEFT));
			add(videoPanel);
		}
	}
	
	private void addComponent(VideoPanel vp, GridBagConstraints c, int gridy, JPanel videoPanel) {
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 1;
		c.gridy = gridy;
		videoPanel.add(vp.getInfoBtn(), c);
		c.gridx = 2;
		videoPanel.add(vp.getSelectBtn(), c);
		c.gridx = 3;
		videoPanel.add(vp.getRemoveBtn(), c);
		c.gridx = 4;
		c.anchor = GridBagConstraints.CENTER;
		videoPanel.add(vp.getDisplayCb(), c);
		c.gridx = 5;
		// TODO: Re-add lookup checkbox when it is fixed
//		videoPanel.add(vp.getLookupCb(), c);
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 6;
		videoPanel.add(vp.getFileNameLbl(), c);
	}
}
