package edu.bu.signstream.ui.panels.notes;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import edu.bu.signstream.common.db.parser.SS3Parser;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.vo.ss3.db.SS3Database;
import edu.bu.signstream.common.util.vo.ss3.note.SS3Note;
import edu.bu.signstream.common.util.vo.ss3.participants.ParticipantsLibrary;
import edu.bu.signstream.common.util.vo.ss3.participants.SS3Participant;
import edu.bu.signstream.grepresentation.fields.Segment;
import edu.bu.signstream.ui.ButtonPanel;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SimpleNotesPanel extends JPanel implements ActionListener {

	 private JDialog parent = null;
	private SS3Participant annotator = null;
	private SS3Database db = null;
	private Segment segment = null;

	private ParticipantsLibrary annotatorsLibrary = null;

	private JButton createBtn = new JButton("New Note");
	private JButton deleteBtn = new JButton("Delete");
	private JButton cancelBtn = new JButton("Cancel");
	private JButton doneBtn = new JButton("Done");

	private JPanel notesPanel = null;
	private JScrollPane scrollPane = null;

	private final int CREATE_NOTE = 1;
	private final int DELETE_NOTE = 2;
	private final int CANCEL_NOTE = 3;
	private final int DONE_NOTE = 4;

	private ArrayList<SS3Note> loadedNotes = new ArrayList<SS3Note>();
	private GridBagConstraints gbc = new GridBagConstraints();

	public SimpleNotesPanel(SS3Database db, Segment segment,
			SS3Participant annotator, JDialog parent) {
		this.annotator = annotator;
		this.parent = parent;
		this.db = db;
		this.segment = segment;

		ArrayList<String> noteIDs = segment.getNoteIDs();

		for (String noteID : noteIDs) {
			loadedNotes.add(db.getNote(noteID));
		}

		SS3Parser parser = new SS3Parser();
		String localAnnotatorsFileName = SS3SignStreamApplication.path
				+ SS3SignStreamApplication.localAnnotatorsFileName;
		String globalAnnotatorsFileName = SS3SignStreamApplication.path
				+ SS3SignStreamApplication.globalAnnotatorsFileName;
		this.annotatorsLibrary = parser.parseAnnotatorsLibrary(
				globalAnnotatorsFileName, localAnnotatorsFileName);

		createBtn.setActionCommand("" + CREATE_NOTE);
		createBtn.addActionListener(this);
		deleteBtn.setActionCommand("" + DELETE_NOTE);
		deleteBtn.addActionListener(this);
		cancelBtn.setActionCommand("" + CANCEL_NOTE);
		cancelBtn.addActionListener(this);
		doneBtn.setActionCommand("" + DONE_NOTE);
		doneBtn.addActionListener(this);

		Dimension preferredSize = createBtn.getPreferredSize();
		deleteBtn.setPreferredSize(preferredSize);
		deleteBtn.setMaximumSize(preferredSize);
		deleteBtn.setMinimumSize(preferredSize);
		cancelBtn.setPreferredSize(preferredSize);
		cancelBtn.setMaximumSize(preferredSize);
		cancelBtn.setMinimumSize(preferredSize);
		doneBtn.setPreferredSize(preferredSize);
		doneBtn.setMaximumSize(preferredSize);
		doneBtn.setMinimumSize(preferredSize);

		deleteBtn.setEnabled(false);

		for (SS3Note note : loadedNotes) {
			note.getCheckBox().addItemListener(new ItemListener() {
				@Override
				public void itemStateChanged(ItemEvent e) {
					boolean check = false;
					for (SS3Note note1 : loadedNotes) {
						if (note1.getCheckBox().isSelected())
							check = true;
					}
					deleteBtn.setEnabled(check);
				}
			});
		}

		initUI();
	}

	private void initUI() {
		setLayout(new GridBagLayout());

		notesPanel = new JPanel();
		notesPanel.setLayout(new GridLayout(loadedNotes.size(), 1));

		loadPanel();

		gbc.gridx = 10;
		add(createBtn, gbc);
		gbc.gridx = 0;
		gbc.gridy = 14;
		add(deleteBtn, gbc);
		gbc.gridx = 2;
		add(Box.createHorizontalStrut(255), gbc);
		gbc.gridx = 8;
		add(cancelBtn, gbc);
		gbc.gridx = 10;
		add(doneBtn, gbc);
	}

	private void loadPanel() {
		notesPanel = new JPanel();
		notesPanel.setLayout(new GridLayout(loadedNotes.size(), 1));

		for (SS3Note note : loadedNotes) {
			JTextArea noteTxt = note.getTextArea();

			String date = note.getDayNTimeCreated();
			String coderID = note.getCoderID();
			
			SS3Participant participant = annotatorsLibrary.getParticipant(coderID);
			String prtName = participant.getFullName();
			if (prtName == null || prtName.length() == 0) {
				prtName = "Anonymous";
			}

			JPanel notePanel = new JPanel();
			GridBagConstraints gbc = new GridBagConstraints();
			notePanel.setLayout(new GridBagLayout());

			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.WEST;
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.insets = new Insets(2, 2, 2, 2);
			notePanel.add(note.getCheckBox(), gbc);
			gbc.gridx = 1;
			notePanel.add(new JLabel("Annotator: "), gbc);
			gbc.gridx = 2;
			notePanel.add(new JLabel(participant.getFullName()), gbc);
			gbc.anchor = GridBagConstraints.EAST;
			gbc.gridx = 6;
			notePanel.add(new JLabel("Date: "), gbc);
			gbc.gridx = 7;
			notePanel.add(new JLabel(date));

			noteTxt.setLineWrap(true);
			noteTxt.setPreferredSize(new Dimension(400, 179));
			JScrollPane scroll = new JScrollPane(noteTxt);
			scroll.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);

			gbc.gridx = 0;
			gbc.gridy = 2;
			gbc.gridwidth = 15;
			notePanel.add(scroll, gbc);

			gbc.gridy = 17;
			gbc.gridwidth = 1;
			notePanel.add(new JLabel(), gbc);

			notesPanel.add(notePanel);
		}

		scrollPane = new JScrollPane(notesPanel);
		scrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		Dimension d = new Dimension(450, 220);

		scrollPane.setPreferredSize(d);
		scrollPane.setMinimumSize(d);
		scrollPane.setMaximumSize(d);

		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.gridwidth = 9;
		add(scrollPane, gbc);
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		String str = ae.getActionCommand();

		switch (Integer.parseInt(str)) {
		case CREATE_NOTE:
			remove(scrollPane);

			SS3Note newNote = new SS3Note();
			newNote.setID(Util.getUniqueNumber());
			newNote.setCurrentTime();

			newNote.getCheckBox().addItemListener(new ItemListener() {
				@Override
				public void itemStateChanged(ItemEvent e) {
					boolean check = false;
					for (SS3Note note1 : loadedNotes) {
						if (note1.getCheckBox().isSelected())
							check = true;
					}
					deleteBtn.setEnabled(check);
				}
			});

			if (annotator != null)
				newNote.setCoderID(annotator.getId());

			JTextArea newArea = newNote.getTextArea();
			newArea.setFocusable(true);

			loadedNotes.add(0, newNote);

			loadPanel();
			revalidate();
			repaint();
			break;

		case DELETE_NOTE:

			final JDialog newCollectionDialog = new JDialog(parent,
					"Delete Resource", true);
			JButton cancelBtn = new JButton("Cancel");
			final JButton deleteBtn = new JButton("Delete");

			deleteBtn.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent arg0) {

					ArrayList<SS3Note> toDelete = new ArrayList<SS3Note>();

					for (SS3Note note : loadedNotes) {
						JCheckBox cb = note.getCheckBox();
						if (cb.isSelected())
							toDelete.add(note);
					}

					for (SS3Note delNote : toDelete) {
						loadedNotes.remove(delNote);
					}

					deleteBtn.setEnabled(false);

					remove(scrollPane);
					loadPanel();
					revalidate();
					repaint();
				}
			});

			ButtonPanel newCollectionButtonPanel = new ButtonPanel();
			newCollectionButtonPanel.add(cancelBtn);
			newCollectionButtonPanel.add(deleteBtn);

			JPanel newCollectionPanel = new JPanel();
			newCollectionPanel.setLayout(new BorderLayout());
			JLabel newCollectionLabel = new JLabel(
					" Are you sure you want to delete selected notes?");
			newCollectionPanel.add(newCollectionLabel, BorderLayout.CENTER);
			newCollectionPanel
					.add(newCollectionButtonPanel, BorderLayout.SOUTH);

			newCollectionPanel.add(new JLabel("  "), BorderLayout.WEST);

			Dimension d = newCollectionPanel.getPreferredSize();
			Dimension nD = new Dimension(d.width + 10, d.height + 40);
			newCollectionPanel.setPreferredSize(nD);

			Container contentPane = newCollectionDialog.getContentPane();
			contentPane.setLayout(new BorderLayout(40, 40));
			contentPane.add(newCollectionPanel, BorderLayout.CENTER);

			newCollectionDialog.setLocation(new Point(150, 150));
			newCollectionDialog.pack();
			newCollectionDialog.setVisible(true);
			break;

		case CANCEL_NOTE:
			parent.dispose();
			break;

		case DONE_NOTE:
			ArrayList<String> newNoteIDs = new ArrayList<String>();
			for (SS3Note note : loadedNotes) {
				newNoteIDs.add(note.getID());
				db.addNote(note);
			}

			segment.setNotesIDs(newNoteIDs);
			parent.dispose();

			break;
		}
	}
}
