package edu.bu.signstream.ui.panels.signbank;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.logging.Level;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;

import edu.bu.signstream.common.files.SS3Resources;
import edu.bu.signstream.common.util.Util;
import edu.bu.signstream.common.util.signbank.SignBankResource;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3CodingScheme;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldTypeImg;
import edu.bu.signstream.common.util.vo.ss3.codingScheme.SS3FieldValue;
import edu.bu.signstream.grepresentation.fields.glossField.dialog.associatedFields.AssociatedFieldsPanel;
import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.signbank.SignBankOccurrence;
import edu.bu.signstream.grepresentation.view.signbank.SignBankVariant;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class SignBankVariantPanel extends JPanel implements ActionListener {

	private static final long serialVersionUID = 1L;
	
	protected SignBankPanel parent = null;
	
	private SignBankOccurrence currentlyDisplayingOccurrence = null;
	
	private AssociatedFieldsPanel assocFieldsPanel = null;
	
	private JLabel startHSLabel = null;
	private JLabel endHSLabel = null;
	private JLabel domStartHSLabel = null;
	private JLabel nondomStartHSLabel = null;
	private JLabel domEndHSLabel = null;
	private JLabel nondomEndHSLabel = null;
	
	private JPanel domStartHSPanel = null;
	private JPanel nondomStartHSPanel = null;
	private JPanel domEndHSPanel = null;
	private JPanel nondomEndHSPanel = null;
	private JPanel framesPanel = null;
	
	private JPanel variantDataPanel = null;
	private JPanel videoPanel = null;
	
	private BufferedImage domStartHSImage = null;
	private BufferedImage nondomStartHSImage = null;
	private BufferedImage domEndHSImage = null;
	private BufferedImage nondomEndHSImage = null;
	
	private JLabel mainGlossLabel = null;
	private JLabel variantLabel = null;
    private JLabel frameImagesLabel = null;
	
	private JButton insertOccurrence = null;
	private JButton viewOccurrences = null;
	private JButton playVideo = null;
	private JButton playCompositeVideo = null;

	private JTextArea extraDataText = null;
	private JScrollPane extraDataTextScrollPane = null;
	private JPanel extraDataContainer = null;
	
	private JPanel mainGlossLabelPanel = null;
	private JPanel variantLabelPanel = null;
	private JPanel hsPanelRow = null;
	
	private String participant = null;
	private String collection = null;
	private String signType = null;
	private String twoHanded = null;
	private String passiveBaseArm = null;
	private String markedHands = null;
	private String leftRightHS = null;
	private String startEndHS = null;
	private String startEndLeftHS = null;
	private String startEndRightHS = null;	

	private final int INSERT = 10;
	
	private final String insertEntryVariant = "Insert All Data";
	
	private static final int maxFrameImageHeight = 240;
	private static final int maxFrameImageWidth = 480;
	private static final int maxHandshapeHeight = 80;
	private static final int maxHandshapeWidth = 80;
	
	private static final String DAI_URL = "http://dai.cs.rutgers.edu/dai/s/occurrence?sign_text=";
	
	private static final String DAI_PROTOCOL = "http";
	private static final String DAI_SITE = "dai.cs.rutgers.edu";
	private static final String DAI_PATH = "/dai/s/occurrence";
	private static final String DAI_QUERY = "sign_text=";
	private static final String DAI_FRAGMENT = null;
	
	public SignBankVariantPanel(SignBankPanel parent, AssociatedFieldsPanel assocFieldsPanel) {
		this.parent = parent;
		this.assocFieldsPanel = assocFieldsPanel;
		
		createUI();
	}
	
	private void createUI() {

		startHSLabel = new JLabel("  Start HS  ");
		domStartHSLabel = new JLabel("  Dom  ");
		nondomStartHSLabel = new JLabel(  "Non-Dom  ");
		
		endHSLabel = new JLabel("  End HS  ");
		domEndHSLabel = new JLabel("  Dom  ");
		nondomEndHSLabel = new JLabel("  Non-Dom  ");
		
		frameImagesLabel = new JLabel("Frame Images");
		frameImagesLabel.setAlignmentX(CENTER_ALIGNMENT);
		
		domStartHSPanel = new JPanel();
		configureHSPanel(domStartHSPanel);
		
		nondomStartHSPanel = new JPanel();
		configureHSPanel(nondomStartHSPanel);
		
		domEndHSPanel = new JPanel();
		configureHSPanel(domEndHSPanel);
		
		nondomEndHSPanel = new JPanel();
		configureHSPanel(nondomEndHSPanel);
		
		JPanel framesContainerPanel = new JPanel();
		framesContainerPanel.setLayout(new BoxLayout(framesContainerPanel, BoxLayout.Y_AXIS));
		framesPanel = new JPanel();
		framesPanel.setLayout(new BoxLayout(framesPanel, BoxLayout.X_AXIS));
		framesPanel.setAlignmentX(CENTER_ALIGNMENT);
		framesContainerPanel.add(framesPanel);
		
		mainGlossLabel = new JLabel("Primary Entry: n/a");
		variantLabel = new JLabel("Entry/Variant: n/a");
		
		insertOccurrence = new JButton(insertEntryVariant);
		insertOccurrence.addActionListener(this);
		insertOccurrence.setActionCommand("" + INSERT);
		insertOccurrence.setVisible(false);
		
		extraDataText =  new JTextArea();
		extraDataText.setEditable(false);
		extraDataText.setMargin(new Insets(10,10,10,10));
		extraDataText.setLineWrap(true);
		extraDataTextScrollPane = new JScrollPane(extraDataText);
		extraDataTextScrollPane.getViewport().setScrollMode(javax.swing.JViewport.SIMPLE_SCROLL_MODE);
		extraDataContainer = new JPanel();
		extraDataContainer.setLayout(new BorderLayout());
		extraDataContainer.add(extraDataTextScrollPane, BorderLayout.CENTER);
		extraDataContainer.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
		
		
		// VARIANT DATA PANEL
		
		mainGlossLabelPanel = new JPanel();
		mainGlossLabelPanel.add(mainGlossLabel);
		mainGlossLabelPanel.setAlignmentX(CENTER_ALIGNMENT);
		
		variantLabelPanel = new JPanel();
		variantLabelPanel.add(variantLabel);
		variantLabelPanel.setAlignmentX(CENTER_ALIGNMENT);
		variantLabelPanel.setBorder(BorderFactory.createEmptyBorder(0,0,10,0));
		
		extraDataContainer.setAlignmentX(CENTER_ALIGNMENT);
		insertOccurrence.setAlignmentX(CENTER_ALIGNMENT);

		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		add(mainGlossLabelPanel);
		add(variantLabelPanel);
		
		viewOccurrences = new JButton("View Occurrences");
		viewOccurrences.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				try {
					String stringToEncode = currentlyDisplayingOccurrence
							.getParentVariant()
							.getVariantLabel();
					
					String encodedLabel = URLEncoder.encode(
							stringToEncode, "UTF-8")
							.toString()
							.replace("+","%20");
					
					String manualURI = DAI_PROTOCOL + "://" + DAI_SITE + DAI_PATH + "?" + DAI_QUERY + encodedLabel;
					
					URI uri = new URI(manualURI);
					
					// tried to make the uri via the class below, but it escapes the query parameters AGAIN, which is not what we need
//					URI uri = new URI(
//							DAI_PROTOCOL,
//							DAI_SITE,
//							DAI_PATH,
//							DAI_QUERY
////								+ "'" 
////								+ URLEncoder.encode(currentlyDisplayingOccurrence.getParentVariant().getVariantLabel(), "UTF-8") 
//								+ encodedLabel,
////								+ "'",
//							DAI_FRAGMENT);
					
					Desktop.getDesktop().browse(uri);
				} catch(MalformedURLException exception) {
					System.out.println("Invalid URL Format. " + exception);
					viewOccurrences.setEnabled(false);
				} catch (IOException exception) {
					System.out.println("Error opening URL using browser: " + exception);
					viewOccurrences.setEnabled(false);
				} catch (URISyntaxException exception) {
					System.out.println("Error converting URL to URI: " + exception);
					viewOccurrences.setEnabled(false);
				}
			}
			
		});
		playVideo = new JButton("Play Sign Video");
		
		playVideo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				new SignBankVariantVideoDialog(SignBankVariantPanel.this, currentlyDisplayingOccurrence.getSignVideoURLs().get(0));
			}
		});
		
		playCompositeVideo = new JButton("Play Composite/Utterance Video");
		
		playCompositeVideo.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				new SignBankVariantVideoDialog(SignBankVariantPanel.this, currentlyDisplayingOccurrence.getSignVideoURLs().get(1));
			}
		});
		
		add(insertOccurrence);

		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(viewOccurrences);
		buttonPanel.add(playVideo);
		buttonPanel.add(playCompositeVideo);
		add(buttonPanel);
		//buttons
		
		JPanel imagesPanel = new JPanel();
		imagesPanel.setLayout(new BoxLayout(imagesPanel, BoxLayout.X_AXIS));
		imagesPanel.add(domStartHSPanel);
		imagesPanel.add(nondomStartHSPanel);
		imagesPanel.add(framesContainerPanel);
		imagesPanel.add(domEndHSPanel);
		imagesPanel.add(nondomEndHSPanel);
		add(imagesPanel);
		//frame images
		
		
		add(extraDataContainer);
		
		setVisible(false);
		
	}
	
	private void configureHSPanel(JPanel panel) {		
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		int padding = 2;
		int margin = 5;
		Border insideBorder = BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(Color.BLACK), BorderFactory.createEmptyBorder(padding, padding, padding, padding));
		panel.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(margin, margin, margin, margin), insideBorder));
	}
	
	public SignBankOccurrence getCurrentlyDisplayingOccurrence() {
		return currentlyDisplayingOccurrence;
	}
	
	public void populate(SignBankOccurrence occurrence) {
		
		ArrayList<String> frameImageURLs = occurrence.getFrameImageURLs();
		
		//Show the Sign Bank Variant Panel if hidden.
		SwingUtilities.getWindowAncestor(this).pack();
		
		this.currentlyDisplayingOccurrence = occurrence;
		
		String domStartHS = occurrence.getDomStartHS();
		String domEndHS = occurrence.getDomEndHS();
		String nondomStartHS = occurrence.getNondomStartHS();
		String nondomEndHS = occurrence.getNondomEndHS();

        SS3CodingScheme cs = SS3Singleton
                .getSS3CodingSchemeCollection().getDefaultCodingScheme();
		
		String signLabelText = "", mainGlossLabelText = "";

		String hsDir = SS3Resources.resourcesPath + "/images/handShapes/";
		String domStartHSFilePath = "", domEndHSFilePath = "", nondomStartHSFilePath = "", nondomEndHSFilePath = "";

		signLabelText = occurrence.getParentVariant().getVariantLabel();
		mainGlossLabelText = occurrence.getParentVariant().getParentMainGloss().getMainGlossLabel();
		variantLabel.setText("Entry/Variant: " + signLabelText);
		mainGlossLabel.setText("Primary Entry: " + mainGlossLabelText);
		
		
		insertOccurrence.setText(insertEntryVariant + ": " + signLabelText);
		if(SignBankVariant.isCompound(signLabelText))
			insertOccurrence.setEnabled(false);
		else
			insertOccurrence.setEnabled(true);
		insertOccurrence.setMaximumSize(insertOccurrence.getMinimumSize());

		String occurrenceCount = occurrence.getParentVariant().getOccurrenceCount();
		if (!occurrenceCount.isEmpty()) {
			viewOccurrences.setText("View Occurrences" + " (" + occurrenceCount + ")" );
		} else {
			viewOccurrences.setText("View Occurrences");
		}
		String resourceFileName = occurrence.getSIgnBankResourceFileName();
		SignBankResource resource = parent.getSignBankCollection().getSignBankResourceCollection().getResourceByFileName(resourceFileName);
		if (resource.isNonEditableResource()) {
			//From Global Sign Bank: Enable DAI Link and Video Playback.
			this.viewOccurrences.setEnabled(true);
			this.playVideo.setEnabled(true);
			this.playCompositeVideo.setEnabled(true);
		} else {
			//From Local Sign Bank: Disable DAI Link and Video Playback.
			this.viewOccurrences.setEnabled(false);
			this.playVideo.setEnabled(false);
			this.playCompositeVideo.setEnabled(false);
		}
		
		String rightHandFieldID = Util.DOM_HANDSHAPE_FIELD_ID;
		String leftHandFieldID = Util.NDOM_HANDSHAPE_FIELD_ID;
		
		ArrayList<SS3FieldValue> rightFieldValues = cs.getFieldImg(rightHandFieldID);
		ArrayList<SS3FieldValue> leftFieldValues = cs.getFieldImg(leftHandFieldID);

		SS3FieldValue rightHandStartValue = null, rightHandEndValue = null, leftHandStartValue = null, leftHandEndValue = null;

		// right start hs		
		rightHandStartValue = Util.matchSignBank(rightFieldValues, domStartHS);
		
		// right end hs
		rightHandEndValue = Util.matchSignBank(rightFieldValues, domEndHS);

		// left start hs
		leftHandStartValue = Util.matchSignBank(leftFieldValues, nondomStartHS);

		// left end hs
		leftHandEndValue = Util.matchSignBank(leftFieldValues, nondomEndHS);
		
		SS3FieldTypeImg domFirstImage = null, domLastImage = null, nondomFirstImage = null, nondomLastImage = null;

		if (rightHandStartValue != null) {
			domFirstImage = cs.getImage(rightHandStartValue.getIid());
            domStartHSFilePath = hsDir + domFirstImage.getPath1();
		}

		if (rightHandEndValue != null) {
			domLastImage = cs.getImage(rightHandEndValue.getIid());
            domEndHSFilePath = hsDir + domLastImage.getPath1();
		}

		if (leftHandStartValue != null) {
			nondomFirstImage = cs.getImage(leftHandStartValue.getIid());
            nondomStartHSFilePath = hsDir + nondomFirstImage.getPath1();
		}

		if (leftHandEndValue != null) {
			nondomLastImage = cs.getImage(leftHandEndValue.getIid());
            nondomEndHSFilePath = hsDir + nondomLastImage.getPath1();
		}
		
		domStartHSImage = null;
		nondomStartHSImage = null;
		domEndHSImage = null;
		nondomEndHSImage = null;

		File domStartHSFile = new File(domStartHSFilePath);
		if (domStartHSFile.exists() && !domStartHSFile.isDirectory()) {
			try {
				domStartHSImage = ImageIO.read(domStartHSFile);
			} catch (IOException e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}
		}

		File nondomStartHSFile = new File(nondomStartHSFilePath);
		if (nondomStartHSFile.exists() && !nondomStartHSFile.isDirectory()) {
			try {
				nondomStartHSImage = ImageIO.read(nondomStartHSFile);
			} catch (IOException e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}
		}

		File domEndHSFile = new File(domEndHSFilePath);
		if (domEndHSFile.exists() && !domEndHSFile.isDirectory()) {
			try {	
				domEndHSImage = ImageIO.read(domEndHSFile);
			} catch (IOException e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}
		}

		File nondomEndHSFile = new File(nondomEndHSFilePath);
		if (nondomEndHSFile.exists() && !nondomEndHSFile.isDirectory()) {
			try {	
				nondomEndHSImage = ImageIO.read(nondomEndHSFile);
			} catch (IOException e) {
				e.printStackTrace();
				SS3SignStreamApplication.logger.log(Level.SEVERE, "Exception", e);
			}
		}

		domStartHSPanel.removeAll();
		nondomStartHSPanel.removeAll();
		domEndHSPanel.removeAll();
		nondomEndHSPanel.removeAll();

		framesPanel.removeAll();

		if (domStartHSImage != null) 
			domStartHSPanel.add(new JLabel(new ImageIcon(domStartHSImage.getScaledInstance(maxHandshapeWidth, maxHandshapeHeight, 0))));
		if (nondomStartHSImage != null)
			nondomStartHSPanel.add(new JLabel(new ImageIcon(nondomStartHSImage.getScaledInstance(maxHandshapeWidth, maxHandshapeHeight, 0))));

		if (domEndHSImage != null)
			domEndHSPanel.add(new JLabel(new ImageIcon(domEndHSImage.getScaledInstance(maxHandshapeWidth, maxHandshapeHeight, 0))));

		if (nondomEndHSImage != null)
			nondomEndHSPanel.add(new JLabel(new ImageIcon(nondomEndHSImage.getScaledInstance(maxHandshapeWidth, maxHandshapeHeight, 0))));
		
		domStartHSPanel.setVisible(domStartHSImage != null);
		nondomStartHSPanel.setVisible(nondomStartHSImage != null);
		domEndHSPanel.setVisible(domEndHSImage != null);
		nondomEndHSPanel.setVisible(nondomEndHSImage != null);
		
		
		// lexical text box
		participant = occurrence.getParticipant();
		if (participant == null || participant.isEmpty())
			participant = "n/a";
		signType = occurrence.getSignType();
		if (signType == null || signType.isEmpty()) 
			signType = "n/a";
		if (occurrence.isTwoHanded())
			twoHanded = "Two Handed";
		else
			twoHanded = "One Handed";
		if (occurrence.isPassiveBaseArm())
			passiveBaseArm = "Yes";
		else
			passiveBaseArm = "No";
		if (occurrence.isMarkedHands())
			markedHands = "Yes";
		else
			markedHands = "No";
		leftRightHS = occurrence.getLeftRightHS();
		startEndHS = occurrence.getStartEndHS();
		startEndLeftHS  = occurrence.getStartEndLeftHS();
		startEndRightHS = occurrence.getStartEndRightHS();
		
		String text = "";
		text += "SignType: " + signType;
		text += ", " + twoHanded;
		text += ", PassiveBaseArm: " + passiveBaseArm;
		text += ", MarkedHands: " + markedHands;
		if (!leftRightHS.isEmpty())
			text += ", LeftRightHandShapes: " + leftRightHS;
		if (!startEndHS.isEmpty())
			text += ", StartEndHandShapes: " + startEndHS;
		if (!startEndLeftHS.isEmpty())
			text += ", StartEndLeftHandShapes: " + startEndLeftHS;
		if (!startEndRightHS.isEmpty())
			text += ", StartEndRightHandShapes: " + startEndRightHS;
		
		extraDataText.setText(text);
		extraDataContainer.setPreferredSize(new Dimension(getWidth(), 100));
		extraDataContainer.setMaximumSize(new Dimension(getWidth(), 200));
		mainGlossLabelPanel.setMaximumSize(mainGlossLabelPanel.getPreferredSize());
		variantLabelPanel.setMaximumSize(variantLabelPanel.getPreferredSize());
		
		insertOccurrence.setVisible(true);

		revalidate();
		repaint();
		SwingUtilities.getWindowAncestor(this).pack();
		
		//Keep track of total image width to enforce panel's width.

		//Keep track of previous width/height for any error panels
		int previousHeight = maxFrameImageHeight;
		int previousWidth = this.getWidth() / 2;
		// start end frame images
		if (frameImageURLs.size() == 0) {
				JPanel errorPanel = new JPanel();
				JLabel errorMessage = new JLabel("No images.");
				errorPanel.setPreferredSize(new Dimension(previousWidth, previousHeight));
				errorPanel.setMaximumSize(new Dimension(previousWidth, previousHeight));
				errorPanel.setLayout(new GridBagLayout());
				errorPanel.add(errorMessage);
				errorPanel.setBackground(Color.GRAY);
				errorPanel.setAlignmentX(CENTER_ALIGNMENT);
				framesPanel.add(errorPanel);
		} else try {
			//If occurrence start/end image URLs are set, use those instead of the arraylist of images.
			if (occurrence.getStartImageURL() != null || occurrence.getEndImageURL() != null) {
				frameImageURLs = new ArrayList<String>();
				if (occurrence.getStartImageURL() != null) {
					frameImageURLs.add(occurrence.getStartImageURL());
				} if (occurrence.getEndImageURL() != null) {
					frameImageURLs.add(occurrence.getEndImageURL());
				}
			}

			ArrayList<BufferedImage> frameImages = new ArrayList<BufferedImage>();
			int numImages = frameImageURLs.size();
			for (String frameImageURL: frameImageURLs) {
				URL url = new URL(frameImageURL);
				try {
					BufferedImage image = ImageIO.read(url);
					frameImages.add(image);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			int avgResizedWidth = maxFrameImageWidth / numImages;

			BufferedImage stitchedImage = new BufferedImage(maxFrameImageWidth, maxFrameImageHeight,
															BufferedImage.TYPE_3BYTE_BGR);
			Graphics2D imgGraphics = stitchedImage.createGraphics();
			for (int i = 0; i < frameImages.size(); i++) {
				BufferedImage image = frameImages.get(i);
				int x = i * avgResizedWidth;
				imgGraphics.drawImage(image, x + 3, 0, x + avgResizedWidth - 3, maxFrameImageHeight,
									  0, 0, image.getWidth(), image.getHeight(), null);
			}
			imgGraphics.dispose();
			framesPanel.add(new JLabel(new ImageIcon(stitchedImage)));
		} catch (MalformedURLException e) {
			System.out.println("MalformedURLException caught");
		} catch (IllegalArgumentException e) {
			System.out.println("IllegalArgumentException caught");
		}
		
		SwingUtilities.getWindowAncestor(this).pack();
	}
	
	private void insertOccurrence() {
		if (currentlyDisplayingOccurrence == null)
			return;
		
		assocFieldsPanel.populateFromSignBankOccurrence(currentlyDisplayingOccurrence);	

		SwingUtilities.getWindowAncestor(this).dispose();	
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String str = e.getActionCommand();
		switch (Integer.parseInt(str)) {
		
		case INSERT:
			insertOccurrence();
			break;
		
		}
	}

}
