package edu.bu.signstream.ui.panels.signbank;

import edu.bu.signstream.common.files.SS3Resources;
import edu.bu.signstream.media.MediaPlayer;
import edu.bu.signstream.media.gstreamer.GstMediaPlayer;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.concurrent.CountDownLatch;
import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class SignBankVariantVideoDialog extends JDialog {
    private static final int BUFFER_SIZE = 1024;
    private static final String TEMP_FILE = SS3Resources.rootPath + "/.signbank.mp4";
    private static final int PROGRESS_BAR_MAX = 1000;

    public SignBankVariantVideoDialog(SignBankVariantPanel parent, String url) {
        super(SwingUtilities.getWindowAncestor(parent));

        MediaPlayer player = new GstMediaPlayer();
        var file = new File(TEMP_FILE);

        // Create the loading dialog
        var bar = new JProgressBar(0, PROGRESS_BAR_MAX);
        var loading = new JFrame();
        loading.setTitle("Loading Video...");
        loading.setContentPane(bar);
        loading.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
        loading.setSize(400, 50);
        loading.setLocationRelativeTo(null);
        loading.setAlwaysOnTop(true);
        loading.setVisible(true);

        // Create the play button
        var play = new JButton("Play");
        play.setAlignmentX(Component.CENTER_ALIGNMENT);
        play.addActionListener(e -> {
            player.seek(0);
            player.play();
        });
        add(play);

        // Set up the main window layout
        setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
        setLocationRelativeTo(null);
        setAlwaysOnTop(true);

        // Clean up on close
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                player.close();
                file.delete();
            }
        });

        // Spawn the downloader thread, update the progress bar, and signal the waiting thread
        var latch = new CountDownLatch(1);
        new Thread(() -> {
            try {
                var con = (HttpURLConnection)new URL(url).openConnection();
                var len = con.getContentLength();

                var in  = new BufferedInputStream(con.getInputStream());
                var out = new BufferedOutputStream(new FileOutputStream(file));

                byte[] buf = new byte[BUFFER_SIZE];
                int n;
                int count = 0;
                while ((n = in.read(buf, 0, BUFFER_SIZE)) >= 0) {
                    count += n;
                    var progress = (int)(((double)count / len) * PROGRESS_BAR_MAX);
                    SwingUtilities.invokeLater(() -> bar.setValue(progress));
                    out.write(buf, 0, n);
                }

                out.close();
                in.close();
            } catch (Exception e) {
                e.printStackTrace();
            }

            latch.countDown();
        }).start();

        // Spawn the waiting thead, wait for the download, and show the video
        new Thread(() -> {
            try {
                latch.await();
            } catch (InterruptedException e) {
                e.printStackTrace();
                return;
            }

            var video = player.addVideo(file.toURI());
            player.prepare();

            SwingUtilities.invokeLater(() -> {
                var view = video.image();
                view.setPreferredSize(new Dimension(video.width(), video.height()));
                add(view);

                loading.dispose();

                pack();
                setVisible(true);
            });
        }).start();
    }
}