package edu.bu.signstream.ui.video;

import java.nio.file.Paths;
import java.time.Duration;
import java.util.ArrayList;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.grepresentation.view.main.SignStreamSegmentPanel;
import edu.bu.signstream.media.MediaPlayer;
import edu.bu.signstream.media.Video;
import edu.bu.signstream.media.gstreamer.GstMediaPlayer;

import javax.swing.*;

public class MultipleMovieController {
	private final MediaPlayer player;
	// TODO: Remove this. Only used to access temp internal methods of underlying MediaPlayer implementation
    @Deprecated
    private final GstMediaPlayer _player;

	private final ArrayList<XugglerMovie> movies = new ArrayList<XugglerMovie>();
//	private int numVideosOpen = 0;
//	private final VideoSegment selectedVideoSegment = new VideoSegment(0, 0);
//	private long currentPacketTimeStamp = 0;
//	private int packetDuration = 2000; // 4000;

	private int speedPointer = 2;

//	private int dragToTime = -1;
//	private int skippedFrames = 0;
//	private boolean play = false;
//	private boolean doneSkipping = true;

	public static int NUMBER_OF_SLIDER_TICKS = 10000;

//	private MovieActionDelegate actionDelegate = null;

    private long start = 0;
    private long end = -1;

    private boolean timescaleOverride = false;

	public MultipleMovieController() {
	    player = _player = new GstMediaPlayer();

	    player.addPosListener(this::updateSliders);
	    player.addStateListener(this::updatePlaying);
//		actionDelegate = new MovieActionDelegate(this);
	}

	public void setTimescale(int timestep, int timebase, int duration) {
	    System.out.println("Timescale manual override to " + timebase + " / " + timestep + "(" + duration + ")");
	    timescaleOverride = true;
        _player.timestep = timestep;
        _player.timebase = timebase;
        _player.framerate = (double)timebase / (double)timestep;

        int frames = duration / timestep;
        var fps = (double)timebase / timestep;
        _player.duration = Duration.ofNanos(Math.round((frames / fps) * 1000000000));
    }

//	MoviePlayer player = null;

    private void updateSliders(long pos) {
	    if (!player.prepared()) return;

	    var ts = _player.tsFromPos(pos);
	    var fr = _player.frFromPos(pos);

        SwingUtilities.invokeLater(() -> {
            var tick = (int)(fr * NUMBER_OF_SLIDER_TICKS);
            for (var v : SS3Singleton.getVideoControlManager().getVideoControlPanels()) {
                v.moveSliderTo(tick);
            }

            SS3Singleton.getVideoControlManager().getSlider().setSliderPosition(ts);
            SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().repaint();
        });
    }

    private void updatePlaying(boolean playing) {
	    SwingUtilities.invokeLater(() -> {
            for (VideoControlPanel videoPanel : SS3Singleton.getVideoControlManager().getVideoControlPanels())
                videoPanel.enableFunctions(!playing);
        });
    }

    public void pause() {
        if (!player.prepared()) return;

		if (player.playing()) {
			player.pause();
			SwingUtilities.invokeLater(() -> {
                for (VideoControlPanel videoPanel : SS3Singleton.getVideoControlManager().getVideoControlPanels()) {
                    videoPanel.setStopVideoControls();
                }
            });
		}
//		if (isPlaying()) {
//			play = false;
//			if (player != null) {
//				player.cancel(true);
//			}
//			ArrayList<VideoControlPanel> videoPanels = SS3Singleton.getVideoControlManager().getVideoControlPanels();
//			for (VideoControlPanel videoPanel : videoPanels) {
//				videoPanel.setStopVideoControls();
//			}
//		}
	}

//	public MovieActionDelegate getMovieActionDelegate() {
//		return actionDelegate;
//	}

	public int getNumVideosOpen() {
		return (int)player.videos().spliterator().getExactSizeIfKnown();
//		return numVideosOpen;
	}

	public void load() {
		System.out.println("MultipleMovieControler loading...");

		for (XugglerMovie movie : movies) {
			String filename = movie.getFilename();
			System.out.println("FILENAME = " + filename);
		}

        var v = player.videos().iterator().next();
		if (!timescaleOverride) {
		    _player.timestep = v.timestep();
		    _player.timebase = v.timebase();
		    _player.framerate = v.framerate();
        }

        System.out.println("Timescale overriden: " + timescaleOverride);
        System.out.println("Timestep is: " + _player.timestep);
		System.out.println("Timebase is: " + _player.timebase);

        if (player.videos().spliterator().getExactSizeIfKnown() == 0)
            return;

        player.prepare();

//		if (!play) {
//			play = true;
//			// calculate packet duration
//			// long timeStamp = actionDelegate.playNextFrame();
//			// currentPacketTimeStamp = actionDelegate.playNextFrame();
//			// packetDuration = (int) (currentPacketTimeStamp - timeStamp);
//
//			// new method: oftentimes the playNextFrame will return 0 as the timestamp, when
//			// IPacket.getTimeStamp() returns
//			// a negative number (previously this was using packetDuration as a means of
//			// advancing the timestamp automatically,
//			// but since packetDuration is being calculated on the fly here, we cannot do it
//			// that way)
//			// so: playNextFrame() until we have positive timestamps (twice) and then
//			// calculate the time between them
//			long timeStamp, nextTimeStamp;
//
//			do {
//				timeStamp = actionDelegate.playNextFrame();
//			} while (timeStamp <= 0);
//
//			// theoretically if the above do-while already passed the 0/negative timestamps,
//			// then the next call to
//			// playNextFrame() should be fine, but just wrapping nextTimeStamp calulation in
//			// a do-while as well out of an abundance of caution
//			do {
//				nextTimeStamp = actionDelegate.playNextFrame();
//			} while (nextTimeStamp <= 0);
//
//			packetDuration = (int) (nextTimeStamp - timeStamp);
//
//			play = false;
//		}
	}

	public double stepRate(boolean increase) {
	    if (!player.prepared()) return 1;

	    speedPointer = increase ? Math.min(5, speedPointer + 1) : Math.max(0, speedPointer - 1);
        var rate = Math.pow(2, speedPointer - 2);

        player.rate(rate);

        return rate;
	}

	public void playSegment(int startMovieTime, int endMovieTime, SignStreamSegmentPanel segmentPanel) {
        if (!player.prepared()) return;

        // Restart on reaching EOS
        if (player.stopped()) {
            start = 0;
        }

        start = startMovieTime;
        end = endMovieTime;

		player.seek(_player.posFromTs(start), _player.posFromTs(end));
		player.play();
//		if (!play) {
//			play = true;
//			ArrayList<VideoControlPanel> videoPanels = SS3Singleton.getVideoControlManager().getVideoControlPanels();
//			player = new MoviePlayer(this, videoPanels, startMovieTime, endMovieTime, segmentPanel);
//			player.execute();
//		}
	}

	public long getPacketDuration() {
	    return _player.timestep;
//		return packetDuration;
	}

	public void skipNumberOfFrames(int moveNumberOfFrames) {
        if (!player.prepared()) return;

        player.pause();

        long frame = _player.frame() + moveNumberOfFrames;
        player.seek(_player.posFromFrame(frame));
//		if (!play) {
//			play = true;
//			if (this.doneSkipping) {
//				skippedFrames = skippedFrames + moveNumberOfFrames;
//				FrameSkipper fs = new FrameSkipper(this, skippedFrames);
//				this.doneSkipping = false;
//				this.skippedFrames = 0;
//				fs.execute();
//			} else {
//				skippedFrames = skippedFrames + moveNumberOfFrames;
//			}
//			play = false;
//		}
	}

	public long frameBackward(long packetTimeStamp) {
        if (!player.prepared()) return 0;

		long ts = packetTimeStamp - getPacketDuration();

		player.pause();
		player.seek(_player.posFromTs(ts));

		return ts;
//		long goToPacketTimeStamp = packetTimeStamp - packetDuration;
//		actionDelegate.goToPacketTimeStamp(goToPacketTimeStamp);
//
//		return currentPacketTimeStamp;
	}

	public long frameForward(long packetTimeStamp) {
	    if (!player.prepared()) return 0;

		long ts = packetTimeStamp + getPacketDuration();

		player.pause();
		player.seek(_player.posFromTs(ts));

		return ts;
//		actionDelegate.goToPacketTimeStamp(packetTimeStamp);
//
//		return currentPacketTimeStamp;
	}

	// This is only ever called to fix some Xuggler brokenness,
	// so we should be safe to ignore.
	public long goToVideoPercent(int value) {
		return 0;
//		if (movies.size() < 1)
//			return -1;
//
//		stopPlayingVideoUpdateVideoControlPanel();
//
//		play = true;
//
//		double numberOfSticks = MultipleMovieController.NUMBER_OF_SLIDER_TICKS;
//		int movieDuration = getMovieDuration();
//
//		double numberOfFrames = movieDuration / packetDuration;
//		double framesInOneStick = numberOfFrames / numberOfSticks;
//		double goToFrameNumber = value * framesInOneStick;
//
//		int frameToGo = (int) goToFrameNumber;
//		int movieTimeToSeek = frameToGo * packetDuration;
//
//		long frame = 0;
//
//		if (value <= 2) {
//			// go to first frame
//			goToFirstFrame();
//		} else if (value + 2 >= numberOfSticks) {
//			// go to last frame
//			goToLastFrame();
//		} else {
//			for (XugglerMovie movie : movies) {
//				frame = actionDelegate.goToPreviousKeyFrame(movieTimeToSeek, movie);
//			}
//		}
//
//		play = false;
//		return frame;
	}

//	public XugglerMovie getFirstMovie() {
//
//		if (movies.size() == 0) {
//			throw new RuntimeException("No Movie Attached");
//		}
//
//		return movies.get(0);
//	}

	public void setCurrentTime(int currentTime) {
	    if (!player.prepared()) return;

		goToPacketTimeStamp(currentTime);
//		actionDelegate.goToPacketTimeStamp(currentTime);
	}

//	public int getDragToTime() {
//		return dragToTime;
//	}

//	public void setDragToTime(int dragToTime) {
//		this.dragToTime = dragToTime;
//	}

	public void dragToCurrentTime(int currentTime) {
	    if (!player.prepared()) return;

		player.pause();
		player.seek(_player.posFromTs(currentTime));
//		updateSliders();
//		if (!play) {
//			play = true;
//			if (currentTime <= getMovieDuration()) {
//				if (dragToTime == -1) {
//					this.dragToTime = currentTime;
//					actionDelegate.executeDragging();
//				} else {
//					this.dragToTime = currentTime;
//				}
//			}
//		}
//		play = false;
	}

	public void goToFirstFrame() {
	    if (!player.prepared()) return;

		player.pause();
		player.seek(0);
//		goToFrame(1);
//		goToFrame(1);
		// SS3Singleton.getVideoControlManager().synchronizeSlider();
	}

	public void goToLastFrame() {
        if (!player.prepared()) return;

		if (movies.size() < 1)
			return;

		player.pause();
		player.seek(_player.posFromFrame(_player.frameFromPos(player.duration().toNanos()) - 1));
//		if (movies.size() < 1)
//			return;
//
//		int targetMovieTime = getMovieDuration() - packetDuration;
//		actionDelegate.goToPacketTimeStamp(targetMovieTime);
	}

	public void removeAllMovies() {
		movies.clear();
	}

	public int getMovieDuration() {
	    return (int) _player.tsFromPos(player.duration().toNanos());
//	    if (!mgr.prepared() && timescaleOverride) return (int)mgr.tsFromPos(mgr.duration.toNanos());
//        if (!mgr.prepared()) return 0;
//
//		if (movies.size() < 1)
//			return 0;
//
//		long pos = mgr.videos().get(0).duration().toNanos();
//		return (int)mgr.tsFromPos(pos);

//		if (movies.size() < 1)
//			return 0;
//
//		return (int) getTotalNoOfFrames() * packetDuration;
	}

	public double getMovieDurationMs() {
        return player.duration().toNanos() / 1000000.0;
	}

	public long getTotalNoOfFrames() {
	    return getMovieDuration() / getPacketDuration();
//		XugglerMovie movie = getFirstMovie();
//		return movie.getContainer().getStream(movie.getVideoStreamId()).getNumFrames();
	}

	public double getFrameRate() {
	    return _player.framerate;
//		XugglerMovie movie = getFirstMovie();
//		return movie.getFrameRate();
	}
	
	/**
	 * Gets the timescale (derived from the timebase) that Xuggler is using for the current video(s)
	 * Timebase is a rational number = typically the inverse of timescale, i.e. 1/timescale
	 * @return An int which is the timescale
	 */
	public int getMovieTimescale() {
	    return _player.timebase;
//		if (movies.size() < 1)
//			return 0;
//
//		XugglerMovie movie = movies.get(0);
//		IRational movieTimeBase = movie.getContainer()
//													.getStream(movie.getVideoStreamId())
//													.getTimeBase();
//		int movieTimescale = movieTimeBase.getDenominator();
//
//		return movieTimescale;
	}

	public int getPrecedingFrameTime(int time) {
		return time - (int)getPacketDuration();
//		return time - packetDuration;
	}

	public int getFollowingFrameTime(int time) {
		return time + (int)getPacketDuration();
//		return time + packetDuration;
	}

	public int getCurrentTime() {
	    if (!player.prepared()) return 0;
		return (int) _player.ts();
//		return (int) currentPacketTimeStamp;
	}

	public void registerMovie(XugglerMovie movie) {
		System.out.println("Registered movie" + movies.size());
		movies.add(movie);
	}

	public void removeMovie(XugglerMovie movie) {
		movies.remove(movie);
		movie.close();
		// TODO: Remove from mgr?
	}

	public void goToPacketTimeStamp(long goToPacketTime) {
        if (!player.prepared()) return;

		player.pause();
		if (player.videos().isEmpty())
		    player.seek(0);
		else
		    player.seek(_player.posFromTs(goToPacketTime));
//		actionDelegate.goToPacketTimeStamp(goToPacketTime);
	}

	public void unloadVideoFiles(ArrayList<XugglerMovie> movies) {
		// for(XugglerMovie movie:movies) {
		// JDialog dialog = movie.getVideoImage();
		// dialog.removeAll();
		// dialog.removeNotify();
		// dialog.setVisible(false);
		// movies.remove(movie);
		// }
	}

	public ArrayList<XugglerMovie> getMovies() {
		return movies;
	}

	public boolean isPlaying() {
        if (!player.prepared()) return false;

		return player.playing();
//		return play;
	}

	// TODO: Remove these methods and XugglerMovie entirely
	public Video _addVideo(XugglerMovie movie) {
        var video = player.addVideo(Paths.get(movie.getFilename()).toUri());

        // JACK: Temporary hack until VideoImage is rewritten:
        // Update the "rate" display proactively when adding a new video.
        var rate = player.rate();
        for (var img : SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel().getOpenedMediaDialogs().keySet())
            ((VideoImage)img).setSpeedLabel(rate == 1 ? "" : "X" + rate);

        return video;
    }
    public void _removeVideo(Video video) {
	    player.remove(video);
    }

//	private class FrameSkipper extends SwingWorker<Integer, ArrayList<XugglerMovie>> {
//
//		private MultipleMovieController controller = null;
//		private int numberOfFramesToSkip = 0;
//
//		FrameSkipper(final MultipleMovieController controller, int numberOfFramesToSkip) {
//			this.controller = controller;
//			this.numberOfFramesToSkip = numberOfFramesToSkip;
//		}
//
//		private void goToFrame() {
//			long move = numberOfFramesToSkip * packetDuration;
//			long goToPacketTime = currentPacketTimeStamp + move;
//
//			controller.getMovieActionDelegate().goToPacketTimeStamp(goToPacketTime);
//
//			SignStreamSegmentPanel segmentPanel = SS3Singleton.getSignStreamApplication().getSignStreamSegmentPanel();
//
//			Slider slider = segmentPanel.getSlider();
//			int time = getCurrentTime();
//			slider.setSliderPositionTime(time);
//			segmentPanel.repaint();
//
//			int X = segmentPanel.getSlider().getSliderPosition();
//
//			if (segmentPanel.isUtteranceLoaded()) {
//				segmentPanel.getUtteranceView().setVisible(X, X, false);
//			} else {
//				segmentPanel.getDatabaseView().setVisible(X, X);
//			}
//		}
//
//		@Override
//		protected Integer doInBackground() throws Exception {
//			goToFrame();
//			return new Integer(1);
//		}
//
//		@Override
//		protected void done() {
//			controller.setDoneSkipping(true);
//		}
//	}
}
