package edu.bu.signstream.ui.video;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.KeyStroke;

import edu.bu.signstream.grepresentation.reguestsHandler.SS3Singleton;
import edu.bu.signstream.media.Video;
import edu.bu.signstream.ui.SS3SignStreamApplication;

public class VideoImage extends JDialog {
	private static final long serialVersionUID = -4752966848100689153L;

    private final JComponent view;
    private final JLabel speedLabel = new JLabel();

    private final Dimension originalSize;
	private final Dimension smallerSize;

	public VideoImage(MultipleMovieController controller, final XugglerMovie movie, Video v) {
		super();

		originalSize = new Dimension(v.width(), v.height());
		smallerSize = new Dimension(v.width() / 2, v.height() / 2);

        view = v.image();
        view.setPreferredSize(originalSize);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(view, BorderLayout.CENTER);

        VideoControlPanel controlPanel = SS3Singleton.getVideoControlManager().createVideoControlPanel();
        getContentPane().add(controlPanel, BorderLayout.SOUTH);

        final JPanel glass = (JPanel) this.rootPane.getGlassPane();
        glass.setVisible(true);
        glass.setLayout(new FlowLayout(FlowLayout.LEFT));

        speedLabel.setFont(new Font(speedLabel.getFont().getName(), Font.PLAIN, 22));
        speedLabel.setForeground(Color.white);
        glass.add(speedLabel);

        this.setVisible(true);
        this.pack();

        KeyStroke prevFrameKeyStroke = KeyStroke.getKeyStroke(37, 0);
        //getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("R"), "videoControl");
        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(prevFrameKeyStroke, "prevFrame");
        getRootPane().getActionMap().put("prevFrame", new AbstractAction() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent e) {
                MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
                controller.skipNumberOfFrames(-1);

            }
        });

        KeyStroke nextFrameKeyStroke = KeyStroke.getKeyStroke(39, 0);
        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(nextFrameKeyStroke, "nextFrame");
        getRootPane().getActionMap().put("nextFrame", new AbstractAction() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent e) {
                MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
                controller.skipNumberOfFrames(1);
            }
        });

        KeyStroke playKeyStroke = KeyStroke.getKeyStroke(32, 0);
        getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(playKeyStroke, "play");
        getRootPane().getActionMap().put("play", new AbstractAction() {
            @Override
            public void actionPerformed(java.awt.event.ActionEvent e) {
                //MultipleMovieController controller = SS3Singleton.getVideoControlManager().getMultipleMovieController();
                //if(controller.isPlaying()) {
                //	controller.stopPalayingVideoUpdateVideoControlPanel(); //stopPlayingMovie();
                //} else {
                ArrayList<VideoControlPanel> videoPanels = SS3Singleton.getVideoControlManager().getVideoControlPanels();
                if(!videoPanels.isEmpty()) {
                    VideoControlPanel videoControlPanel = videoPanels.get(0);
                    videoControlPanel.playMovie();
                }
                //}
            }
        });

        addMenuKeyboardShortcuts();
		
		this.addComponentListener(new XugglerListener(movie));
		
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
            if (SS3Singleton.getVideoControlManager().getMultipleMovieController().getNumVideosOpen() <= 1) {
                setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
            } else {
                setDefaultCloseOperation(DISPOSE_ON_CLOSE);
                MultipleMovieController contr = SS3Singleton.getVideoControlManager().getMultipleMovieController();
                contr.removeMovie(movie);
            }
			}
		});
	}

	/**
	 * Adds the main SignStream application window's menu shortcuts to this window.
	 */
	private void addMenuKeyboardShortcuts() {
		final SS3SignStreamApplication app = SS3Singleton.getSignStreamApplication();
		InputMap iM = getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap aM = getRootPane().getActionMap();
		
		iM.put(KeyStroke.getKeyStroke(KeyEvent.VK_S, KeyEvent.META_DOWN_MASK), "saveCollection");
		aM.put("saveCollection", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				app.actionPerformed(new ActionEvent(e.getSource(), e.getID(), "" + app.SAVE_DB));
			}
		});
		iM.put(KeyStroke.getKeyStroke(KeyEvent.VK_W, KeyEvent.META_DOWN_MASK), "closeCollection");
		aM.put("closeCollection", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				app.actionPerformed(new ActionEvent(e.getSource(), e.getID(), "" + app.CLOSE_DB));
			}
		});
		iM.put(KeyStroke.getKeyStroke(KeyEvent.VK_O, KeyEvent.META_DOWN_MASK), "openCollection");
		aM.put("openCollection", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				app.actionPerformed(new ActionEvent(e.getSource(), e.getID(), "" + app.OPEN_AS_SS3));
			}
		});
		iM.put(KeyStroke.getKeyStroke(KeyEvent.VK_N, KeyEvent.META_DOWN_MASK), "newCollection");
		aM.put("newCollection", new AbstractAction() {
			@Override
			public void actionPerformed(ActionEvent e) {
				app.actionPerformed(new ActionEvent(e.getSource(), e.getID(), "" + app.NEW_COLLECTION));
			}
		});
	}

	public void setSpeedLabel(String text) {
		speedLabel.setText(text);
	}

	public void goToMinSize() {
        view.setPreferredSize(smallerSize);
        this.pack();
	}
	
	public void goToMaxSize() {			
		view.setPreferredSize(originalSize);
        this.pack();
	}
}

class XugglerListener implements ComponentListener {	
	private XugglerMovie movie = null;
	
	public XugglerListener(XugglerMovie movie) {
		this.movie = movie;
	}
	
	@Override
	public void componentHidden(ComponentEvent arg0) {
		MultipleMovieController contr = SS3Singleton.getVideoControlManager().getMultipleMovieController();
		contr.removeMovie(movie);
	}

	@Override
	public void componentMoved(ComponentEvent arg0) {}
	@Override
	public void componentResized(ComponentEvent arg0) {}
	@Override
	public void componentShown(ComponentEvent arg0) {}
}
